package ch.unizh.ifi.adora.constraints.compiler;

import java.io.*;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.editor.CompileManager;
import ch.unizh.ifi.adora.constraints.ui.preferences.PreferencesConstants;
import ch.unizh.ifi.adora.constraints.ui.table.ConstraintConverter;

public final class DynamicCodeLoader {

	/**
	 * @author Christina Cramer
	 */
	private static class LoadedClass {
		File binFile;

		String className;

		Class clazz;

		long lastModified;

		SourceDir srcDir;


		
		File srcFile;

		LoadedClass(String className, SourceDir src) {
			this.className = className;
			this.srcDir = src;

			String path = className.replace('.', '/');
			this.srcFile = new File(src.srcDir, path + ".java");
			this.binFile = new File(src.binDir, path + ".class");

			compileAndLoadClass();

		}

		void compileAndLoadClass() {
			if (clazz != null) {
				return; // class already loaded
			}

			// compile, if required
			String error = null;

			// ****** [JMT] start ******
			// Added 'true OR', for that it is compiled everytime.
			// Without it, it was not always compiled when it had to be...
			// ****** [JMT] end ******

			if (binFile.lastModified() < srcFile.lastModified()) {

				error = srcDir.javac.compile(new File[] { srcFile });

			}

			// ****** [JMT] start ******
			// Changed error message.
			// ****** [JMT] end ******

			if (false && error != null) {
				throw new RuntimeException("Failed to compile " + srcFile.getAbsolutePath() + ". ***Error: " + error);
			} else if (error != null) {
				// throw new TransformationFailedException(
				// "Failed to compile funtions in MTL:" + error);
			}

			try {
				// load class
				clazz = srcDir.classLoader.loadClass(className);

				// load class success, remember timestamp
				lastModified = srcFile.lastModified();

			} catch (ClassNotFoundException e) {
				throw new RuntimeException("Failed to load DynaCode class " + srcFile.getAbsolutePath());
			}

			info("Init " + clazz);
		}

		boolean isChanged() {
			return srcFile.lastModified() != lastModified;
		}
	}

	private class MyInvocationHandler implements InvocationHandler {

		Object backend;

		String backendClassName;

		MyInvocationHandler(String className) {
			backendClassName = className;

			try {
				Class clz = loadClass(backendClassName);
				backend = newDynaCodeInstance(clz);

			} catch (ClassNotFoundException e) {
				throw new RuntimeException(e);
			}
		}

		public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {

			// check if class has been updated
			Class clz = loadClass(backendClassName);
			if (backend.getClass() != clz) {
				backend = newDynaCodeInstance(clz);
			}

			try {
				// invoke on backend
				return method.invoke(backend, args);

			} catch (InvocationTargetException e) {
				throw e.getTargetException();
			}
		}

		private Object newDynaCodeInstance(Class clz) {
			try {
				return clz.newInstance();
			} catch (Exception e) {
				throw new RuntimeException("Failed to new instance of DynaCode class " + clz.getName(), e);
			}
		}

	}

	/**
	 * @author Christina Cramer
	 */
	private class SourceDir {
		File binDir;

		URLClassLoader classLoader;

		Javac javac;

		File srcDir;

		SourceDir(File srcDir) {
			this.srcDir = srcDir;

			String subdir = srcDir.getAbsolutePath().replace(':', '_').replace('/', '_').replace('\\', '_');
			// this.binDir = new File(System.getProperty("java.io.tmpdir"),
			// "bin/" + subdir);
			this.binDir = new File(ConstraintsPlugin.getDefault().getPreferenceStore().getString(
					PreferencesConstants.SRC_FILES));
			// System.out.println(binDir);
			this.binDir.mkdirs();

			// prepare compiler
			this.javac = new Javac(compileClasspath, binDir.getAbsolutePath());

			// class loader
			recreateClassLoader();
		}

		void recreateClassLoader() {
			try {
				classLoader = new URLClassLoader(new URL[] { binDir.toURL() }, parentClassLoader);
			} catch (MalformedURLException e) {
				// should not happen
			}
		}
	}

	/**
	 * Extracts a classpath string from a given class loader. Recognizes only
	 * URLClassLoader.
	 */
	public static String extractClasspath(ClassLoader cl) {
		StringBuffer buf = new StringBuffer();

		while (cl != null) {
			if (cl instanceof URLClassLoader) {
				URL urls[] = ((URLClassLoader) cl).getURLs();
				for (int i = 0; i < urls.length; i++) {

					if (urls[i].getFile().toString().endsWith(".jar")) {
						if (buf.length() > 0) {
							buf.append(File.pathSeparatorChar);
						}
						buf.append(urls[i].getFile().toString());
					}

				}
			}
			cl = cl.getParent();
		}

		return buf.toString();
	}

	/**
	 * Log a message.
	 */
	private static void info(String msg) {
	}

	private String compileClasspath;

	// class name => LoadedClass
	private HashMap<String, LoadedClass> loadedClasses = new HashMap<String, LoadedClass>();

	private ClassLoader parentClassLoader;

	private ArrayList<SourceDir> sourceDirs = new ArrayList<SourceDir>();

	public DynamicCodeLoader() {
		this(Thread.currentThread().getContextClassLoader());
	}

	public DynamicCodeLoader(ClassLoader parentClassLoader) {
		this(extractClasspath(parentClassLoader), parentClassLoader);

	}

	/**
	 * @param compileClasspath
	 *            used to compile dynamic classes
	 * @param parentClassLoader
	 *            the parent of the class loader that loads all the dynamic
	 *            classes
	 */
	public DynamicCodeLoader(String compileClasspath, ClassLoader parentClassLoader) {
		this.compileClasspath = compileClasspath;
		this.parentClassLoader = parentClassLoader;

	}

	/**
	 * Add a directory that contains the source of dynamic java code.
	 * 
	 * @param srcDir
	 * @return true if the add is successful
	 */
	public boolean addSourceDir(File srcDir) {

		try {
			srcDir = srcDir.getCanonicalFile();
		} catch (IOException e) {
			e.printStackTrace();
			// ignore
		}

		synchronized (sourceDirs) {

			// check existence
			for (int i = 0; i < sourceDirs.size(); i++) {
				SourceDir src = sourceDirs.get(i);
				if (src.srcDir.equals(srcDir)) {
					return false;
				}
			}

			// add new
			SourceDir src = new SourceDir(srcDir);
			sourceDirs.add(src);

			info("Add source dir " + srcDir);
		}

		return true;
	}

	/**
	 * Get a resource from added source directories.
	 * 
	 * @param resource
	 * @return the resource URL, or null if resource not found
	 */
	public URL getResource(String resource) {
		try {

			SourceDir src = locateResource(resource);
			return src == null ? null : new File(src.srcDir, resource).toURL();

		} catch (MalformedURLException e) {
			// should not happen
			return null;
		}
	}

	/**
	 * Get a resource stream from added source directories.
	 * 
	 * @param resource
	 * @return the resource stream, or null if resource not found
	 */
	public InputStream getResourceAsStream(String resource) {
		try {

			SourceDir src = locateResource(resource);
			return src == null ? null : new FileInputStream(new File(src.srcDir, resource));

		} catch (FileNotFoundException e) {
			// should not happen
			return null;
		}
	}

	/**
	 * Returns the up-to-date dynamic class by name.
	 * 
	 * @param className
	 * @return
	 * @throws ClassNotFoundException
	 *             if source file not found or compilation error
	 */
	public Class loadClass(String className) throws ClassNotFoundException {
		

		LoadedClass loadedClass = null;
		synchronized (loadedClasses) {
			loadedClass = loadedClasses.get(className);
		}

		// first access of a class
		if (loadedClass == null) {
			String resource = className.replace(".", System.getProperty("file.separator")) + ".java";
			SourceDir src = locateResource(resource);
			if (src == null) {
				String shortclassName=className.substring(className.lastIndexOf(".")+1)+".txt";
				File constraintfile=this.getConstraintFile(shortclassName);
				ConstraintConverter converter = new ConstraintConverter();
				String constrainttext = converter.getConstraintText(constraintfile);
				CompileManager compMan=new CompileManager();
				compMan.compile(constrainttext);
				resource = className.replace(".", System.getProperty("file.separator")) + ".java";
				src = locateResource(resource);
				//try compiling the corresponding constraint
				
				//throw new ClassNotFoundException("DynaCode class not found " + className);
			}
			synchronized (this) {

				// compile and load class

				loadedClass = new LoadedClass(className, src);
				synchronized (loadedClasses) {
					loadedClasses.put(className, loadedClass);
				}
			}

			return loadedClass.clazz;
		}

		// subsequent access

		if (loadedClass.isChanged()) {

			// unload and load again
			unload(loadedClass.srcDir);
			return loadClass(className);
		}

		return loadedClass.clazz;
	}
	
	/**
	 * 
	 * @param filename
	 *            name of the file (only local)
	 * @return File at location constraintlocation with the given filename or
	 *         null if file does not exist
	 */
	public File getConstraintFile(String filename) {
		String constraintlocation = (ConstraintsPlugin.getPath() + "doc");
		try {
			return new File(constraintlocation.concat(System.getProperty("file.separator") + filename));
		} catch (NullPointerException e) {
		}
		try {
			return new File(constraintlocation.concat(System.getProperty("file.separator") + filename + ".txt"));
		} catch (NullPointerException e) {
			return null;
		}
	}

	private SourceDir locateResource(String resource) {
		for (int i = 0; i < sourceDirs.size(); i++) {
			SourceDir src = sourceDirs.get(i);
			if (new File(src.srcDir, resource).exists()) {
				return src;
			}
		}
		return null;
	}

	/**
	 * Create a proxy instance that implements the specified access interface
	 * and delegates incoming invocations to the specified dynamic
	 * implementation. The dynamic implementation may change at run-time, and
	 * the proxy will always delegates to the up-to-date implementation.
	 * 
	 * @param interfaceClass
	 *            the access interface
	 * @param implClassName
	 *            the backend dynamic implementation
	 * @return
	 * @throws RuntimeException
	 *             if an instance cannot be created, because of class not found
	 *             for example
	 */
	public Object newProxyInstance(Class interfaceClass, String implClassName) throws RuntimeException {
		MyInvocationHandler handler = new MyInvocationHandler(implClassName);
		return Proxy.newProxyInstance(interfaceClass.getClassLoader(), new Class[] { interfaceClass }, handler);
	}

	private void unload(SourceDir src) {
		// clear loaded classes
		synchronized (loadedClasses) {
			for (Iterator iter = loadedClasses.values().iterator(); iter.hasNext();) {
				LoadedClass loadedClass = (LoadedClass) iter.next();
				if (loadedClass.srcDir == src) {
					iter.remove();
				}
			}
		}

		// create new class loader
		src.recreateClassLoader();
	}
}