package ch.unizh.ifi.adora.constraints.constraints.util;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Collection;

import ch.unizh.ifi.adora.model.representation.ConstraintCheckable;

/**
 * The ConstraintResult class stores for a certain incident a number of
 * objects that are involved and a boolean success value.
 *
 * @author seybold
 * @history
 * 2004-02-09  CS  Initial Version
 * 2004-05-28  SM  Additional methods and extensions complex logic functions.
 *
 * @invariant true
 * @version $Date: 2007-10-30 16:30:03 +0100 (Tue, 30 Oct 2007) $, $Author: cramer $, $Revision: 3214 $
 */
public class ConstraintResult {

	/**
	 * Message which describes the problem.
	 */
    private final static String DEFAULT_MESSAGE = "[message not set]";

    /** 
     * Contains the message what has gone wrong 
     */
    private String message;

    /** 
     * true, if constraint was successful 
     */
    private boolean successful;
    
    /**
    *
    */
    private Vector<String> category=new Vector<String>();
    
    /**
    * the description of the constraint
    */
    private String description="";

    /** 
     * pairs of affected object - description object 
     */
    private Vector successfulIncidentObjects;

	/** 
	 * pairs of affected object - description object 
	 */
	private Vector failedIncidentObjects;

    /**
     * Construct an empty result object.
     * Message is set to a default value, success is set to true.
     * @pre true
     * // @post incidentObjects != null && message != null && successful
     */
    public ConstraintResult() {
        successfulIncidentObjects = new Vector();
        failedIncidentObjects = new Vector();
        message = DEFAULT_MESSAGE;
        successful = true;
    }


    /**
     * Construct an empty result object with the default successful
     * set to the given parameter.
     * Message is set to a default value.
     * @param successValue default success value
     * @pre true
     * // @post this.incidentObjects != null && this.message != null
     */
    public ConstraintResult(boolean successValue) {
        successfulIncidentObjects = new Vector();
        failedIncidentObjects = new Vector();
        message = DEFAULT_MESSAGE;
        successful = successValue;
    }


    /**
     * Copies the attributes of the given ConstraintResult to this 
     * ConstraintResult.
     * @pre cr != null
     * @post this.message == cr.message && this.successful == cr.successful
     * @post this.successfulIncidentObjects == cr.successfulIncidentObjects 
     * @post this.failedIncidentObjects == cr.failedIncidentObjects
     * @param cr the ConstraintResult to copy from.
     */
    public void copy(ConstraintResult cr) {
		this.message = cr.message;
		this.successful = cr.successful;
		this.successfulIncidentObjects = cr.successfulIncidentObjects;
		this.failedIncidentObjects = cr.failedIncidentObjects;
    }


    /**
     * Boolean value to String.
     */
    public String toString() {
         return String.valueOf(successful);
    }


    /**
     * Construct an empty result object with the default successful
     * set to the given parameter and the given list of incident objetcs.
     * @param successValue default success value
     * @pre incidentObjects != null
     * // @post this.incidentObjects != null && this.message != null
     */
    public ConstraintResult(boolean successValue, Vector incidentObjects) {
        successfulIncidentObjects = new Vector();
        failedIncidentObjects = new Vector();
        this.message = DEFAULT_MESSAGE;
		this.successful = successValue;
    }



    /**
     * Add an incident object to the list of incident objects.
     * @param o an object that is involved in this incident.
     * @pre o != null
     * // @post incidentObjects.size() == incidentObjects.size()@pre + 1
     * // @post incidentObjects.contains(o)
     */
    public void addIncidentObject(Object o, boolean success) {
		if (success) {		    
			if (o instanceof Collection) {
				this.successfulIncidentObjects.addAll((Collection)o);
			} else {
				this.successfulIncidentObjects.add(o);
			}
		} else {
			if (o instanceof Collection) {
				this.failedIncidentObjects.addAll((Collection)o);
			} else {
				this.failedIncidentObjects.add(o);
			}		    
			
		}
        
    }

    /**
     * Add an incident object to the list of incident objects.
     * @param o an integer that is involved in this incident.
     * @param success Indicates if the object o caused was
     *        in a failing expression or in a successful expression
     * @pre true
     * // @post incidentObjects.size() == incidentObjects.size()@pre + 1
     * // @post incidentObjects.contains(new Integer(o))
     */
    public void addIncidentObject(int o, boolean success) {
		if (success) {
			successfulIncidentObjects.add(new Integer(o));
		} else {
			failedIncidentObjects.add(new Integer(o));
		}
        
    }

    /**
     * Add an incident object to the list of incident objects.
     * @param o a boolean that is involved in this incident.
     * @param success Indicates if the object o caused was
     *        in a failing expression or in a successful expression
     * @pre true
     * // @post incidentObjects.size() == incidentObjects.size()@pre + 1
     * // @post incidentObjects.contains(new Boolean(o))
     */
    public void addIncidentObject(boolean o, boolean success) {
		if (success) {
			successfulIncidentObjects.add(new Boolean(o));
		} else {
			failedIncidentObjects.add(new Boolean(o));
		}
        
    }

    /**
     * Add an incident object to the list of incident objects.
     * @param o a boolean that is involved in this incident.
     * @pre true
     * @param success Indicates if the object o caused was
     *        in a failing expression or in a successful expression     
     * // @post incidentObjects.size() == incidentObjects.size()@pre + 1
     * // @post incidentObjects.contains(new Character(o))
     */
    public void addIncidentObject(char o, boolean success) {
		if (success) {
			successfulIncidentObjects.add(new Character(o));
		} else {
			failedIncidentObjects.add(new Character(o));
		}
        
    }

    /**
     * Add an incident object to the list of incident objects.
     * @param o a boolean that is involved in this incident.
     * @pre true
     * @param success Indicates if the object o caused was
     *        in a failing expression or in a successful expression     
     * // @post incidentObjects.size() == incidentObjects.size()@pre + 1
     * // @post incidentObjects.contains(new Float(o))
     */
    public void addIncidentObject(float o, boolean success) {
		if (success) {
			successfulIncidentObjects.add(new Float(o));
		} else {
			failedIncidentObjects.add(new Float(o));
		}
        
    }

    /**
     * Returns all involved objects.
     * @return an enumeration of objects.
     * @pre true
     * @post return != null
     */
    public Enumeration<ConstraintCheckable> getIncidentObjects(boolean success) {
        Enumeration<ConstraintCheckable> res = null;
		if (success) {
			res = successfulIncidentObjects.elements();
		} else {
			res = failedIncidentObjects.elements();
		}
        return res; 
    }

	/**
	 * Clears the indcident objects and the constraint object. 
	 * @param successfulInit Declares with what value this successful
	 *        flag should be initialized.
	 */
	public void clear(boolean successfulInit) {
		successful = successfulInit;
		
		failedIncidentObjects.clear();
		successfulIncidentObjects.clear();
	}

    /**
     * The cause or name of this incident.
     * @return a message
     * @pre true
     * @post return != null
     */
    public String getMessage() {
        return message;
    }

    /**
     * Sets an incident message.
     * @param message describes this incident
     * @pre message != null
     * @post true
     */
    public void setMessage(String message) {
        this.message = message;
    }

    /**
     * Returns whether this result was successful or not.
     * @return a boolean value
     * @pre true
     * @post true
     */
    public boolean isSuccessful() {
        return successful;
    }

    /**
     * Sets the category of the constraint.
     * @param category
     * @pre true
     * @post true
     */
    public void setCategory(Vector<String> category) {
        this.category = category;
    }
    
    /**
     * Returns the category of the constraint.
     * @pre true
     * @post true
     */
    public Vector<String> getCategory() {
        return category;
    }
    
    
    
     /**
     * Sets the description of the constraint.
     * @param description
     * @pre true
     * @post true
     */
    public void setDescription(String description) {
        this.description = description;
    }
    
    /**
     * Returns the description of the constraint.
     * @pre true
     * @post true
     */
    public String getDescription() {
        return description;
    }
    

    /**
     * Sets the result value.
     * @param successful true means successful
     * @pre true
     * @post true
     */
    public void setSuccessful(boolean successful) {
        this.successful = successful;
    }


    /**
     * Integrates the given result into this with and. 
     * Message is taken of the given one unless it is unset.
     * Object sets are joined.
     */
    public ConstraintResult and(ConstraintResult res) {
		if (!res.getMessage().equals(DEFAULT_MESSAGE)) {
	    	this.setMessage(res.getMessage());
        }
        
        boolean a = this.isSuccessful();
        boolean b = res.isSuccessful();
        
        this.setSuccessful(a && b);

		// copy successful and failed result set to this constraint result set
   		this.addIncidentObject((res.failedIncidentObjects), false);
   		this.addIncidentObject((res.successfulIncidentObjects), true);

        
/*        if (a == false) {
        	this.setSuccessful(false);
       		
        	if (b == false) {
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);
        	} else {
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);
        	}
        } else {
        	if (b == false) {
        		this.setSuccessful(false);
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);
        	} else {
        		this.setSuccessful(true);
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);
        	}
        }
*/        
        return this;
    }


    /**
     * Integrates the given result into this with and. 
     * Message is taken of the given one unless it is unset.
     * Object sets are joined.
     */
    public ConstraintResult or(ConstraintResult res) {
		if (!res.getMessage().equals(DEFAULT_MESSAGE)) {
	    	this.setMessage(res.getMessage());
        }
        
        boolean a = this.isSuccessful();
        boolean b = res.isSuccessful();
		this.setSuccessful(a || b);

   		this.addIncidentObject((res.failedIncidentObjects), false);
   		this.addIncidentObject((res.successfulIncidentObjects), true);

/*		        
        if (a == false) {
        	       		
        	if (b == false) {
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);
        	} else {
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);
        	}
        } else {
        	if (b == false) {
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);
        	} else {
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);
        	}
        }
 */       
        return this;       
    }

	public ConstraintResult not() {		
		this.setSuccessful(!(this.isSuccessful()));
		Vector v;

		// change set of successful and failed incident objects
		v=successfulIncidentObjects;
		successfulIncidentObjects = failedIncidentObjects;
		failedIncidentObjects = v;		
		
		return this;
	}


    /**
     * Integrates the given result into this with and. 
     * Message is taken of the given one unless it is unset.
     * Object sets are joined.
     */
    public ConstraintResult equals(ConstraintResult res) {
		boolean result = (this.isSuccessful() == res.isSuccessful());
		Vector v;
		if (result) {
			if(this.isSuccessful()) {
        		this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);				
			} else {
				// swap result sets
				v = this.failedIncidentObjects;
				this.failedIncidentObjects = this.successfulIncidentObjects;
				this.successfulIncidentObjects = v;
				// copy crossover
				this.addIncidentObject((res.failedIncidentObjects), true);
        		this.addIncidentObject((res.successfulIncidentObjects), false);			
			}	
		} else {
			if(this.isSuccessful()) {
        		this.addIncidentObject((res.failedIncidentObjects), true);
        		this.addIncidentObject((res.successfulIncidentObjects), false);				
			} else {
				// swap result sets
				// copy crossover
				this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);			
			}						
		}
		this.setSuccessful(result);
		return this;
    }

    /**
     * Integrates the given result into this with and. 
     * Message is taken of the given one unless it is unset.
     * Object sets are joined.
     */
    public ConstraintResult unequals(ConstraintResult res) {
		boolean result = (this.isSuccessful() != res.isSuccessful());

		Vector v;
		if (result) {
			if(this.isSuccessful()) {
        		this.addIncidentObject((res.failedIncidentObjects), true);
        		this.addIncidentObject((res.successfulIncidentObjects), false);				
			} else {
				// swap result sets
				// copy crossover
				this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);			
			}						
		} else {
			if(this.isSuccessful()) {
				// failed because res and this were true
				
				// in this case change the local set
				v = this.failedIncidentObjects;
				this.failedIncidentObjects = this.successfulIncidentObjects;
				this.successfulIncidentObjects = v;
				
				// add crossover the failed set and the successfulset
        		this.addIncidentObject((res.failedIncidentObjects), true);
        		this.addIncidentObject((res.successfulIncidentObjects), false);				
			} else {
				// failed because res and this were false
				// swap result sets
				// leave the result sets as they are and copy the 
				// failed set of res to the this failed set. Do this
				// also with the successful set
				this.addIncidentObject((res.failedIncidentObjects), false);
        		this.addIncidentObject((res.successfulIncidentObjects), true);			
			}	
		}		
		this.setSuccessful(result);
		return this;
    }

    /**
     * Integrates the given result into this with and. 
     * Message is taken of the given one unless it is unset.
     * Object sets are joined.
     */
    public ConstraintResult equivalence(ConstraintResult res) {        
		// same as equals operation for logic expression
		return this.equals(res);
    }

    /**
     * Integrates the given result into this with and. 
     * Message is taken of the given one unless it is unset.
     * Object sets are joined.
     */
    public ConstraintResult implication(ConstraintResult res) {
    	boolean result = (( this.isSuccessful() ^ true ) ||  res.isSuccessful());
		Vector v;
		
		
		// if the overall result was false, ie. the implication
		// evaluated to false.
		if (this.isSuccessful() && (!res.isSuccessful())) {
			// change result sets for this constraint result
			v = failedIncidentObjects;
			failedIncidentObjects = successfulIncidentObjects;
			successfulIncidentObjects = v;
		} else { // implication was successful
			// if true --> true
			// the failed object set and the successful object set can
			// remain as they are for this constraint result

			// *****
			// false --> true or false --> false, i.e. the failed 
			// object set caused a successful result, i.e. result set change
			if (! this.isSuccessful()) {
				// change result sets for this constraint result
				v = failedIncidentObjects;
				failedIncidentObjects = successfulIncidentObjects;
				successfulIncidentObjects = v;				
				
				this.addIncidentObject((res.failedIncidentObjects), false);
				this.addIncidentObject((res.successfulIncidentObjects), true);																		

/*				if (res.isSuccessful()) {
					this.addIncidentObject((res.failedIncidentObjects), false);
   					this.addIncidentObject((res.successfulIncidentObjects), true);																		
				} else {
					this.addIncidentObject((res.failedIncidentObjects), false);
   					this.addIncidentObject((res.successfulIncidentObjects), true);																		
				}*/
			} else  {
				// true --> true, just copy the failed object set and the
				// successful object set to the corresponding sets of this
				// constraint result object
				this.addIncidentObject((res.failedIncidentObjects), false);
   				this.addIncidentObject((res.successfulIncidentObjects), true);													
			}						
		}
		this.setSuccessful(result);
		return this;
    }


}
