package ch.unizh.ifi.adora.constraints.editor;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.*;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.compiler.DynamicCodeLoader;
import ch.unizh.ifi.adora.constraints.ui.preferences.PreferencesConstants;
import ch.unizh.ifi.adora.constraints.ui.table.ConstraintConverter;

/**
 * class used to manage creating and modifing constraints in the editor
 * communicates with the ConstraintTable (ConstraintManageView) over a
 * TableEditorMediator
 * 
 * @author Christina Cramer
 * 
 */
public class EditorManager implements IPartListener {
	
	 private static Color COMMENT_COLOR =
	        new Color(Display.getCurrent(), new RGB(0, 200, 0));

	
	String[] classes=new String[]{"Component","State","Actor","Scenario"};
	
	public static final String CONSTRAINT_LIB = "ch.unizh.ifi.adora.constraints.constraints.util";

	public static final String CONSTRAINT_PKG = "ch.unizh.ifi.adora.constraints.constraints";
	
	public Vector keywords;

	/**
	 * folder where icl-compiled constraints will be saves
	 */
	private static final String constraintlocation = (ConstraintsPlugin.getPath() + "doc");

	/**
	 * String that is to be shown in an empty ConstraintTextEditor
	 */
	private static String standartconstraint = ConstraintsPlugin.getDefault().getPreferenceStore().getString(
			PreferencesConstants.modelconstraint);

	/**
	 * used to compile constraints (ICL)
	 */
	private CompileManager compiler;

	public String CONSTRAINT_SRC = ConstraintsPlugin.getDefault().getPreferenceStore().getString(
			PreferencesConstants.SRC_FILES);

	/**
	 * conversion between file and filename
	 */
	ConstraintConverter converter = new ConstraintConverter();

	/**
	 * mediator that organizes the communication between editor and table
	 */
	public TableEditorMediator mediator;

	public int newconstraints;

	IWorkbenchPage page;

	/**
	 * the workbenchwindow
	 */
	IWorkbenchWindow window;

	public EditorManager() {
	}

	public EditorManager(TableEditorMediator mediator) {
		this.mediator = mediator;
		window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		page = window.getActivePage();
		if (page != null) {
			page.addPartListener(this);
		}

		newconstraints = 0;
		keywords=new Vector();
		keywords.add("constraint");
		keywords.add("on");
		keywords.add("is");
	}

	/**
	 * inform the mediator that an editor was closed saving is not handled here,
	 * only closing
	 * 
	 * @param editor
	 *            editor that was closed
	 */
	public void disposeEditor(ConstraintTextEditor editor) {
		mediator.removeConstraint(editor.getPartName());
	}

	/**
	 * sets the ConstraintTextEditor that shows the constraintfile to the
	 * foreground of showing editors
	 * 
	 * @param constraintfile
	 * @pre ConstraintTextEditor that shows the constraintfile must be a showing
	 *      ConstraintTextEditor
	 */
	public void focus(String constraintname) {
		File constraintfile = getConstraintFile(constraintname + ".txt");
		ConstraintEditorInput input = new ConstraintEditorInput(new EditorMetaData(constraintname));
		IWorkbenchPage page = window.getActivePage();
		IEditorReference[] editors = page.getEditorReferences();
		for (int i = 0; i < editors.length; i++) {
			IEditorPart editor = editors[i].getEditor(false);
			if (editor instanceof ConstraintTextEditor) {
				if (((ConstraintEditorInput) editor.getEditorInput()).sameConstraint(input)) {
					IEditorPart part = page.findEditor(editor.getEditorInput());
					page.activate(part);
				}
			}
		}

	}

	/**
	 * 
	 * @param filename
	 *            name of the file (only local)
	 * @return File at location constraintlocation with the given filename or
	 *         null if file does not exist
	 */
	public File getConstraintFile(String filename) {

		try {
			return new File(constraintlocation.concat(System.getProperty("file.separator") + filename));
		} catch (NullPointerException e) {
		}
		try {
			return new File(constraintlocation.concat(System.getProperty("file.separator") + filename + ".txt"));
		} catch (NullPointerException e) {
			return null;
		}
	}

	/**
	 * get the (formatted and cleaned) text of the error file
	 * 
	 * @return
	 */
	public String getErrorString() {
		String newline = System.getProperty("line.separator");
		File constraintfile = new File(CONSTRAINT_SRC + "/errors.txt");
		BufferedReader br;
		String next = "";
		try {
			FileReader fr = new FileReader(constraintfile);
			br = new BufferedReader(fr);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return next;
		}
		String constraintstring = "";
		try {
			while (next != null) {
				next = br.readLine();
				if (next == null) {
					break;
				}
				if ((!next.startsWith("#")) && (!next.contains("ERROR in")) && (!next.contains("error"))
						&& (!next.contains("problem")) && (!next.trim().equals(""))) {
					constraintstring = constraintstring.concat(next + "" + newline);
				}
			}
			return constraintstring;
		} catch (IOException e) {
		}
		return "";
	}

	/**
	 * returns the active page of the platform
	 * 
	 * @return
	 */
	private IWorkbenchPage getPage() {
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		return window.getActivePage();
	}

	/**
	 * checks if a given string is a valid constraint (can be compiled in icl)
	 * if it is not valid, mediator will be informed
	 * 
	 * @param txt
	 *            constraint
	 * @return null if can be compiled in icl, Enumeration with errors if not
	 *         compile
	 */
	public Enumeration isValidConstraint(String txt) {
		if (!txt.trim().startsWith("constraint")) {
			Vector notvalidconstraint = new Vector();
			notvalidconstraint.add("not valid");
			return notvalidconstraint.elements();
		}
		if (compiler == null) {
			compiler = new CompileManager();
		}
		Enumeration compileerrors = compiler.compile(txt);
		return compileerrors;
	}

	/**
	 * checks if the java constraint is valid with the AdoraModel classes
	 * 
	 * @return "" if it is valid, String containing ErrorMessage otherwhize
	 */
	public String isValidJavaConstraint(String constraintname) {
		DynamicCodeLoader loader = new DynamicCodeLoader();
		loader.addSourceDir(new File(CONSTRAINT_SRC));
		try {
			Object o = loader.loadClass(CONSTRAINT_PKG + "." + constraintname).newInstance();
			String errors = this.getErrorString();
			return errors;
		} catch (Exception e) {
			return "";
		}
	}

	public ConstraintTextEditor openEditor(String flagtxt) {
		ConstraintEditorInput input = new ConstraintEditorInput(new EditorMetaData(flagtxt));
		if (page != null) {
			try {
				IEditorPart part = page.openEditor(input, ConstraintTextEditor.ID);
				ConstraintTextEditor editor = (ConstraintTextEditor) part;
				editor.setInput(input);
				editor.addEditorManager(this);
				return editor;
			} catch (PartInitException e) {
				return null;
			}
		} else {
			return null;
		}
	}

	public void partActivated(IWorkbenchPart part) {
		/*
		 * if (part instanceof ConstraintTextEditor) { Vector e =
		 * ((ConstraintTextEditor) part).getInput().getErrors();
		 * mediator.updateErrorMsg(e.elements()); }
		 */
	}

	public void partBroughtToTop(IWorkbenchPart part) {

	}

	public void partClosed(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}

	public void partDeactivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}

	public void partOpened(IWorkbenchPart part) {
		// TODO Auto-generated method stub

	}

	public void propertyChange(PropertyChangeEvent event) {
		// TODO Auto-generated method stub

	}

	public void propertyChanged(Object source, int propId) {
	}

	/**
	 * checks if the constraint that is in the current editor can be saved if it
	 * does not icl-compile, it cannot be saved, saving will be cancelled and
	 * the mediator will be informed about the failure.
	 * 
	 * @return true if successfully saved
	 */
	public boolean saveConstraint() {
		ConstraintTextEditor editor = (ConstraintTextEditor) getPage().getActiveEditor();
		ConstraintEditorInput input = (ConstraintEditorInput) editor.getEditorInput();
		IDocument document = editor.getDocumentProvider().getDocument(input);
		String constrainttxt = document.get().trim();
		Enumeration actualerrors = isValidConstraint(constrainttxt);
		if (actualerrors == null) {
			String constraintname = converter.getConstraintName(constrainttxt);
			String errors = this.isValidJavaConstraint(constraintname);
			if (!errors.equals("")) { // there were compiling problems that
				// need to be shown
				Vector v = new Vector();
				v.add(errors);
				mediator.updateErrorMsg(v.elements());
				input.setErrors(v.elements());
				return false;
			}
			input.setErrors(null);
			if (constraintname.equals(editor.getPartName())) {
				// name remained same,
				if (saveToFile(constrainttxt, constraintname)) {
					mediator.setChanged(constraintname);
					mediator.updateErrorMsg(null);
					input.setErrors(null);
					return true;
				}
			} else if (mediator.constraintEntryExists(constraintname)) {
				// save
				return false;
			} else if (saveToFile(constrainttxt, constraintname)) {
				mediator.addConstraint(constraintname);
				mediator.changeConstraintFile(editor.getPartName(), constraintname);
				editor.setPartName(constraintname);
				mediator.updateErrorMsg(null);
				input.setErrors(null);
				return true;
			}
		}
		Vector v = new Vector();
		while (actualerrors.hasMoreElements()) {
			v.add(actualerrors.nextElement());
		}
		mediator.updateErrorMsg(v.elements());
		input.setErrors(v.elements());
		return false;
	}

	public boolean saveToFile(String constrainttxt, String filename) {
		ConstraintConverter c = new ConstraintConverter();
		try {
			PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(constraintlocation.concat("/"
					+ filename + ".txt"))));
			out.print(constrainttxt);
			out.close();
			return true;
		} catch (Exception ioe) {
			ioe.printStackTrace();
			return false;
		}

	}

	/**
	 * shows the standartconstraint in the editor
	 * 
	 * @return true if constraint is opened in editor, false otherwize
	 */
	public boolean showConstraint() {
		try {
			IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
			ConstraintEditorInput input = new ConstraintEditorInput(new EditorMetaData("new constraint"
					+ newconstraints));
			IWorkbenchPage page = window.getActivePage();
			if (page != null) {
				IEditorPart part = page.openEditor(input, ConstraintTextEditor.ID);
				ConstraintTextEditor editor = (ConstraintTextEditor) part;
				editor.setInput(input);
				editor.addEditorManager(this);
				IDocument document = editor.getDocumentProvider().getDocument(input);
				document.set(standartconstraint);
			}
		} catch (PartInitException e) {
			return false;
		}
		newconstraints++;
		return true;
	}

	/**
	 * shows the constraint that is the constraintfile in the editor
	 * 
	 * @param constraintfile
	 * @return true if constraint is opened in editor, false otherwize
	 */
	public boolean showConstraint(String constraintname) {
		try {
			File constraintfile = getConstraintFile(constraintname + ".txt");
			ConstraintEditorInput input = new ConstraintEditorInput(new EditorMetaData(constraintname));
			IWorkbenchPage page = window.getActivePage();
			if (page != null) {
				page.addPartListener(this);
				IEditorPart part = page.openEditor(input, ConstraintTextEditor.ID);
				final ConstraintTextEditor editor = (ConstraintTextEditor) part;
				editor.setInput(input);
				editor.addEditorManager(this);
				IDocument document = editor.getDocumentProvider().getDocument(input);
				
				ConstraintConverter converter = new ConstraintConverter();
				String constrainttext = converter.getConstraintText(constraintfile);
				document.set(constrainttext);
				final IDocument doc=editor.getDocumentProvider().getDocument(input);
				doc.addDocumentListener(new IDocumentListener(){

					public void documentAboutToBeChanged(DocumentEvent event) {
						// TODO Auto-generated method stub
						
					}

					public void documentChanged(DocumentEvent event) {
						//System.out.println(event.getText());
						//if(knowClass(event.getText())){
							try {
								int offset=event.getOffset();
								char actualchar=doc.getChar(offset);
								String param="";
								String lastword="";
								if(java.lang.Character.isWhitespace(actualchar)){
									
									//go return to get the last word
									while(java.lang.Character.isWhitespace(actualchar)){
										offset--;
										actualchar=doc.getChar(offset);
									}
									while(java.lang.Character.isLetter(actualchar)&&offset>=0){
										lastword=actualchar+lastword;
										offset--;
										actualchar=doc.getChar(offset);
									}
									if(keywords.contains(lastword)){
										//System.out.println(lastword);
										Token t=new Token(lastword);

										
									}
								}
//								offset=event.getOffset()-1;
//								//while(!java.lang.Character.isSpace(actualchar)&&actualchar!='('){
//								while(!java.lang.Character.isJavaIdentifierStart(actualchar)){
//									offset--;
//									actualchar=doc.getChar(offset);
//								}
//								while(java.lang.Character.isJavaIdentifierStart(actualchar)){
//								param=actualchar+param;
//
//									offset--;
//									actualchar=doc.getChar(offset);
//								}
//								
//							editor.addParams(param, event.getText());
//								
								//doc.replace(event.getOffset(), 0, "*******");
							} catch (BadLocationException e) {
								// TODO Auto-generated catch block
								//e.printStackTrace();
							}	
						}			
						
					//}

					
					

					});
			}
		}
		// not able to open the editor
		catch (PartInitException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	private boolean knowClass(String text) {
		for(String s:classes){
			if(s.equals(text)) return true;
		}
		return false;
	}
	
}
