package ch.unizh.ifi.adora.constraints.editor;

import java.util.Enumeration;
import java.util.Vector;

import ch.unizh.ifi.adora.constraints.ui.ConstraintManageView;

/**
 * class handling communication between ConstraintTable and Editor also has the
 * information, which constraints are currently showing
 * 
 * @author Christina Cramer
 * 
 */
public class TableEditorMediator {

	private static TableEditorMediator instance;

	/**
	 * returns the (only) instance of the TableEditorMediator for the given View
	 * 
	 * @return
	 */
	public static TableEditorMediator getInstance(ConstraintManageView newview) {
		if (instance == null) {
			instance = new TableEditorMediator(newview);
		}
		return instance;
	}

	/**
	 * editormanager
	 */
	private EditorManager editormanager;

	/**
	 * names of the constraints that are currently showing
	 */
	private Vector shownConstraints = new Vector();

	/**
	 * view that is connected by this TableEditorMediator
	 */
	private ConstraintManageView view;

	public TableEditorMediator() {
	}

	/**
	 * constructor, sets the given view as the view connected by
	 * TableEditorMediator and also opens a EditorManager
	 * 
	 * @param view
	 */
	public TableEditorMediator(ConstraintManageView view) {
		this.view = view;
		this.editormanager = new EditorManager(this);

	}

	/**
	 * informs view about a new constraint
	 * 
	 * @param constraintName
	 */
	public void addConstraint(String constraintName) {
		view.addConstraint(constraintName);
	}

	public void changeConstraintFile(String oldfile, String newfile) {
		shownConstraints.remove(oldfile);
		shownConstraints.add(newfile);
	}

	/**
	 * checks if a constraintEntry with that name exists in the
	 * ConstraintTableEntriesList
	 * 
	 * @param name
	 * @return true if exists, false if not
	 */
	public boolean constraintEntryExists(String name) {
		return (view.getConstraintList().containsConstraintEntry(name));
	}

	/**
	 * informs the view that a constraint was successfully opened in the editor
	 * 
	 * @param constraintfile
	 */
	public void constraintIsShowing(String constraintfile) {
		shownConstraints.add(constraintfile);
	}

	/**
	 * @param constraintfile
	 * @return true if that file is showing in a TextEditor, false otherwize
	 */
	public boolean isShowing(String constraintfile) {
		for (int i = 0; i < shownConstraints.size(); i++) {
			if (shownConstraints.elementAt(i).equals(constraintfile)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * informs the editormanager that a new editor shall be created
	 * 
	 */
	public void makeNewAction() {
		editormanager.showConstraint();
	}

	/**
	 * removes the File from the list of showing constraints
	 * 
	 * @param constraintfile
	 */
	public void removeConstraint(String constraintfile) {
		shownConstraints.remove(constraintfile);
		updateErrorMessage("");
	}

	/**
	 * informs view about change of a constraint
	 * 
	 * @param constraintName
	 */
	public void setChanged(String constraintName) {
		view.setChanged(constraintName);
	}

	/**
	 * calls all needed methods to show a constraintfile in the editor if the
	 * constraint is already in an open editor, focus shall be set to this
	 * constraint if not, open the constraint in a new editor informs the view
	 * if constraint is sucessfully shown in the editor
	 * 
	 * @param constraintfile
	 */
	public void showConstraint(String constraintfile) {
		if (isShowing(constraintfile)) {
			editormanager.focus(constraintfile);
		} else {
			boolean success = editormanager.showConstraint(constraintfile);
			if (success) {
				constraintIsShowing(constraintfile);
			}
		}
	}

	/**
	 * changes the text in the errortextfield of the view
	 * 
	 * @param msg
	 *            new text
	 */
	public void updateErrorMessage(String msg) {
		try {
			view.getErrorTextField().setText(msg);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * changes the errormessage to the text of all entries of msg
	 * 
	 * @param msg
	 *            Enumeration (containing Strings)
	 */
	public void updateErrorMsg(Enumeration msg) {
		String s = "";
		if (msg != null) {
			while (msg.hasMoreElements()) {
				s = s.concat(msg.nextElement().toString());
			}
		}
		updateErrorMessage(s);
	}


}
