package ch.unizh.ifi.adora.constraints.editor.preferences;

import java.util.StringTokenizer;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;

/**
 * preference page for automatic entries of ConstraintTextEditor
 */
public class ErrorsPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	private static final String PREFERENCE_DELIMITER = "$";

	/*
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	private Button removeTag;

	private List tags;

	private Text txt;

	public ErrorsPreferencePage() {
		super();
		IPreferenceStore store = ConstraintsPlugin.getDefault().getPreferenceStore();
		setPreferenceStore(store);
	}

	/**
	 * add a new tag
	 * 
	 */
	private void addTag() {
		String tag = txt.getText();
		if (tag != null && tag.length() > 0) {
			tags.add(tag);
		}
		txt.setText("");
	}

	/**
	 * coverts the String that contains the values of the tags as a String []
	 * 
	 * @param preferenceValue
	 *            String containing all the tags
	 * @return
	 */
	private String[] convert(String preferenceValue) {
		StringTokenizer tokenizer = new StringTokenizer(preferenceValue, PREFERENCE_DELIMITER);
		int tokenCount = tokenizer.countTokens();
		String[] elements = new String[tokenCount];

		for (int i = 0; i < tokenCount; i++) {
			elements[i] = tokenizer.nextToken();
		}

		return elements;
	}

	/**
	 * @see org.eclipse.jface.preference.
	 *      PreferencePage#createContents(Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
		Composite top = new Composite(parent, SWT.LEFT);

		// Sets the layout data for the top composite's
		// place in its parent's layout.
		top.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// Sets the layout for the top composite's
		// children to populate.
		top.setLayout(new GridLayout());

		Label listLabel = new Label(top, SWT.NONE);
		listLabel.setText("tags that can be used for auto completion in ConstraintTextEditor");
		tags = new List(top, SWT.BORDER | SWT.V_SCROLL);
		tags.setItems(getTagPreferences());
		GridData data = new GridData(GridData.FILL_BOTH);
		data.horizontalIndent = 10;
		data.verticalSpan = 3;
		// Create a data that takes up the extra space
		// in the dialog and spans both columns.
		GridData listData = new GridData(GridData.FILL_HORIZONTAL);
		listData.heightHint = convertVerticalDLUsToPixels(160);
		tags.setLayoutData(listData);

		tags.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectionChanged();
			}
		});

		Composite addRemoveGroup = new Composite(top, SWT.NONE);
		addRemoveGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout addRemoveLayout = new GridLayout();
		addRemoveLayout.numColumns = 2;
		addRemoveLayout.marginHeight = 0;
		addRemoveLayout.marginWidth = 0;
		addRemoveGroup.setLayout(addRemoveLayout);

		// Create a composite for the add and remove buttons.
		Composite buttonGroup = new Composite(addRemoveGroup, SWT.NONE);
		buttonGroup.setLayoutData(new GridData());
		GridLayout buttonLayout = new GridLayout();
		buttonLayout.marginHeight = 0;
		buttonLayout.marginWidth = 0;
		buttonGroup.setLayout(buttonLayout);

		Button addTag = new Button(buttonGroup, SWT.NONE);
		addTag.setText("Add Ta&g");
		addTag.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				addTag();
			}
		});
		GridData addTagData = new GridData(GridData.FILL_HORIZONTAL);
		addTagData.heightHint = convertVerticalDLUsToPixels(IDialogConstants.BUTTON_HEIGHT);
		addTagData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
		addTag.setLayoutData(addTagData);

		removeTag = new Button(buttonGroup, SWT.NONE);
		removeTag.setText("&Remove Tag");
		removeTag.setEnabled(false);
		removeTag.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				tags.remove(tags.getSelectionIndex());
				selectionChanged();
			}
		});
		GridData removeTagData = new GridData(GridData.FILL_HORIZONTAL);
		removeTagData.heightHint = convertVerticalDLUsToPixels(IDialogConstants.BUTTON_HEIGHT);
		removeTagData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
		removeTag.setLayoutData(removeTagData);

		txt = new Text(addRemoveGroup, SWT.BORDER);

		GridData textData = new GridData(GridData.FILL_HORIZONTAL);
		textData.verticalAlignment = GridData.BEGINNING;
		txt.setLayoutData(textData);

		return top;
	}

	/**
	 * get the default TagPreferences
	 * 
	 * @return
	 */
	public String[] getDefaultTagPreferences() {
		return convert(EditorPreferenceConstants.TAGS_DEFAULT);
	}

	/**
	 * get actual TagPreferences
	 * 
	 * @return
	 */
	public String[] getTagPreferences() {
		return convert(ConstraintsPlugin.getDefault().getPreferenceStore().getString(
				EditorPreferenceConstants.TAGS_PREFERENCE));
	}

	/**
	 * @see IWorkbenchPreferencePage#init
	 */
	public void init(IWorkbench wb) {
	}

	/**
	 * restore default preferences
	 */
	@Override
	protected void performDefaults() {
		tags.setItems(getDefaultTagPreferences());
		// getDefaultExemptTagsPreference() is a convenience
		// method which retrieves the default preference from
		// the preference store.
		super.performDefaults();
	}

	/**
	 * save changes
	 */
	@Override
	public boolean performOk() {
		setTagPreferences(tags.getItems());
		return super.performOk();
	}

	private void selectionChanged() {
		int index = tags.getSelectionIndex();
		removeTag.setEnabled(index >= 0);
	}

	/**
	 * sets the actual TagPreferences
	 * 
	 * @param elements
	 */
	public void setTagPreferences(String[] elements) {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < elements.length; i++) {
			buffer.append(elements[i]);
			buffer.append(PREFERENCE_DELIMITER);
		}
		ConstraintsPlugin.getDefault().getPreferenceStore().setValue(EditorPreferenceConstants.TAGS_PREFERENCE,
				buffer.toString());
	}

}
