package ch.unizh.ifi.adora.constraints.engine;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import org.eclipse.ui.IEditorPart;
import ch.unizh.ifi.adora.constraints.constraints.util.ConstraintResult;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConstraintCheckable;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * manages the constraintrules, makes sure they get loaded at the right moment
 * 
 * @author cramer
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Oct 10, 2007 cramer refactoring
 * @version Oct 10, 2007 cramer
 */
public class RuleManager implements PropertyChangeListener {

	private boolean success=true;
	
	
	private ConstraintCheckManager constraintcheckmanager;
	/**
	 * contains the events that will be ignored
	 */
	private String[] ignoreEvent = {"VisualModElem.Location", "VisualModElem.Size", "SetSelected","ZoomableNodeContainer.ZoomedIn", "VisualModElem.Visibility","Connection.Bendpoints","AbstractableConnection.Calc.Abstracted","changejava.beans.PropertyChangeEvent[source=WaitForUserName]"};
	private Vector ignoreEvents;
	/**
	 * contains all events that (possibly) were caused because of a new element
	 * in the model
	 */
	private String[] newEvent = { "AdoraModel.ChildAdded", "ConnectableElement.SourceConn" };

	private Vector newEvents;
	private ScopeManager scopemanager;

	private ConstraintTypeLoader typeloader;
	private ConstraintsViewManager viewmanager;

	/**
	 * Constructor, also initializes the vectors of the actions
	 */
	public RuleManager() {
		IEditorPart editor = ch.unizh.ifi.adora.constraints.ConstraintsPlugin.getDefault().getWorkbench()
		.getActiveWorkbenchWindow().getActivePage().getActiveEditor();

if (editor instanceof AdoraModelEditor) {
	AdoraModel am = ((AdoraModelEditor) editor).getModel();
	am.addPropertyChangeListener(this);
}
		//listen to model
		if (ignoreEvents == null) {
			ignoreEvents = new Vector();
			for (int i = 0; i < ignoreEvent.length; i++) {
				ignoreEvents.add(ignoreEvent[i]);
			}
		}
		if (newEvents == null) {
			newEvents = new Vector();
			for (int i = 0; i < newEvent.length; i++) {
				newEvents.add(newEvent[i]);
			}
		}

	}

	public void addConstraintCheckManager(ConstraintCheckManager checkmanager) {
		this.constraintcheckmanager = checkmanager;
	}

	public void addLoader(ConstraintTypeLoader loader) {
		this.typeloader = loader;
	}

	public void addScopeManager(ScopeManager smanager) {
		this.scopemanager = smanager;
	}

	public void addViewManager(ConstraintsViewManager viewmanager) {
		this.viewmanager = viewmanager;
	}

	/**
	 * performing actions when a new modelelement was added
	 * 
	 * @param element
	 * @param cause 
	 */
	public void newModelelement(ConstraintCheckable element, String cause) {
		element.addPropertyChangeListener(this);
		Vector ruleinstances = typeloader.getConstraintInstance(element.getClass());
		for (int i = 0; i < ruleinstances.size(); i++) {
			ConstraintResult result = constraintcheckmanager.check(ruleinstances.elementAt(i), element);
			scopemanager.addNew(element, ruleinstances.elementAt(i), result);
			if(!result.isSuccessful()){success=false;}

		}
		Vector<ConstraintInstance> instances = scopemanager.getRuleInstances(element);
		for (int i = 0; i < instances.size(); i++) {
			ConstraintInstance inst = instances.elementAt(i);
			if (inst.getRoolElement().equals(element)) {
				viewmanager.informchanged(inst, element,cause);
			}
		}
	}

	/**
	 * performing actions when notified about a property change
	 */
	public void propertyChange(PropertyChangeEvent evt) {
	if(evt.getPropertyName().equals(VisualizableModelElement.NAME_PROP)){
		viewmanager.informNameChanged((ConstraintCheckable)evt.getSource(),(String)evt.getNewValue());
	}
		
		if (!ignoreEvents.contains(evt.getPropertyName())) {
			String cause="(caused by: "+evt.getSource().getClass().getSimpleName()+" "+evt.getSource()+", "+evt.getPropertyName().substring(evt.getPropertyName().indexOf('.')+1)+")";
			if (newEvents.contains(evt.getPropertyName()) && scopemanager.checkIfNew(evt.getNewValue())) {
				newModelelement((ConstraintCheckable) evt.getNewValue(),cause);
			}
			ConstraintCheckable element = (ConstraintCheckable) evt.getSource();
			if (evt.getPropertyName().equals("VisualModElem.wasDeleted")) {
				scopemanager.remove(element);
				viewmanager.remove(evt.getSource());
			}
			Vector<ConstraintInstance> rules = scopemanager.getRuleInstances(element);
			for (int i = 0; i < rules.size(); i++) {
				ConstraintInstance instance = rules.elementAt(i);
				ConstraintCheckable rootelement = instance.getRoolElement();
				Object ruleinstance = instance.getRuleinstance();
				ConstraintResult result = constraintcheckmanager.check(ruleinstance, rootelement);
				if(!result.isSuccessful()){
					success=false;
				}
				ConstraintResult oldresult = instance.getResult();
				if (!sameElements(result, oldresult)) {
					scopemanager.informchanged(instance, result);
					if (!(result.getIncidentObjects(false).equals(oldresult.getIncidentObjects(false)))) {
						viewmanager.informchanged(instance, rootelement,cause);
					}
				}
			}
		}
	}

	/**
	 * checks if to ConstraintResult objects have the same incidentobjects (for
	 * true and false)
	 * 
	 * @param result
	 *            ConstraintResult
	 * @param oldresult
	 *            ConstraintResult
	 * @return true if both ConstraintResults have exactly the same
	 *         incidentobjects and the true/false value of each incidentobject
	 *         is equal
	 */
	private boolean sameElements(ConstraintResult result, ConstraintResult oldresult) {
		if (!(result.getIncidentObjects(true).equals(oldresult.getIncidentObjects(true)))) {
			return false;
		}
		if (!(result.getIncidentObjects(false).equals(oldresult.getIncidentObjects(false)))) {
			return false;
		}
		return true;
	}

	public void reshow() {
		viewmanager.reshow();
		
	}

	public boolean getSuccess() {
		return success;
	}
}
