package ch.unizh.ifi.adora.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.Clipboard;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;

import ch.unizh.ifi.adora.actions.resources.MenuNames;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.factories.ModelObjectMapper;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.structure.Model;

/**
 * The GEF copy action for the adora tool gets the selected
 * elements from the current GEF editor and puts a reference into 
 * the clipboard. Therefore several steps are needed:
 * <ol>
 *   <li>The currently selected elements are read
 *       from the current GEF editor.</li>
 *   <li>Elements are which fulfill the following
 *       cumulatively are ready to be 
 *       copied into the clipboard (or more precisely
 *       said, the references of these elements are copied):
 *       <ol>
 *       	<li>
 *              The elements which are ready to be referenced
 *              by the clipboard have to be either
 *              of the class {@link ch.unizh.ifi.adora.model.representation.Node} 
 *              or the class {@link ch.unizh.ifi.adora.model.ZoomableNode}.
 *          </li>
 *          <li>    
 *              Connections cannot be copied explicitly (i.e.
 *              by selecting them directly). Rather, there will
 *              be an implicitly copying of connections, i.e.
 *              if connections are between sub nodes of a copied
 *              node which implies that connections may not 
 *              cross the border of a node to be copied implicitly. 
 *          </li>
 *          <li>
 *              The paste mechanism executes a deep copy
 *              of the elements in the clipboard, i.e. 
 *              the elements to copy must not be childs of each other,
 *              otherwise the paste operation behavior is 
 *              unpredictable. This implies that child nodes 
 *              which are selected for copying have to be filtered
 *              out.               
 *          </li>

 *       </ol>
 *       A filter takes care that only the references of the selected 
 *       elements which fulfill the above conditions are copied into 
 *       the clipboard.
 *   </li>
 *   <li>
 *       The references of the elements to copy are copied into 
 *       the clipboard and stay there for being read by the 
 *       paste mechanism.
 *   </li>
 * </ol>
 *      
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			04.02.2006 SM First Version
 *						05.02.2006 IV Changes
 *						28.03.2006 SM Comments and fixed bug #527
 *                      15.07.2006 SM Bug 560 fixed.
 *                      18.07.2006 SM Comments and removal of commented code.
 *                      08.04.2008 CJ Fix bug 853 (copy/cut/paste do not work in the outline view)
 * @version			$Date: 2010-01-28 04:43:33 +0100 (Thu, 28 Jan 2010) $, $Author: vigan $, $Revision: 5098 $	
 * @responsibilities	Prepares selected elements from the 
 *						graphical view to be copied as references
 *						into the clipboard of the GEF.
 */
public  class AdoraCopyAction extends SelectionAction {

	/**
	 * Creates an instance of the copy action.
	 * @pre       part != null
	 * @post      true
	 * @param part The workbench part
	 */
	public AdoraCopyAction(IWorkbenchPart part) {
		super(part);		
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
        // TODO Workaround for GEF-Bug 82622/39369.  Should be removed when 39369 is fixed.
		// should always return true as workaround.
		return true;
	}

	/**
	 * Filters out the elements that cannot be copied explicitly
	 * by the Adora copy/paste mechanism (see above 
	 * in the class comment). Worst case runtime o(n^2).
	 * 
	 * @pre       selection != null
	 * @post      return != null
	 * @param     selection The list of all currently selected
	 * @return    Returns a list object containing the references
	 *            of all elements that can be copied into 
	 *            the clipboard.
	 */
	protected List<Object> getSelectedTopElements(List selection) {
		List<Object> retVal = new ArrayList<Object>();
		boolean copiable = true;
		// Iterate with all elements over the 
		int index=0;
		int currElementIndex=0;
		Object element, currElement;
		ZoomableNodeContainer   n;
		while (currElementIndex < selection.size()) {
			copiable = true;
			currElement = null;
			index = 0;
			if (selection.get(currElementIndex) instanceof EditPart) {
				currElement = ((EditPart)selection.get(currElementIndex)).getModel();
			}
			// if checked element (currElement) is a node
			if ((currElement != null) && (currElement instanceof Node)) {
				// then compare the currently checked element with the other 
				// selected elements the currently checked node 
				// (currElement) must not be a child of a container 
				// in the set of selected elements
				while (index < selection.size()) { // iterate over the elements
					element = null;
					if (selection.get(index) instanceof EditPart) {
						element = ((EditPart)
								selection.get(index)).getModel();  // get current element
					}			
					// if the currently iterated element is a
					// container, the container can be the direct or
					// indirect parent of the curently checked node					
					if (element instanceof ZoomableNodeContainer) {
						n = (ZoomableNodeContainer)element;
						// if so, the element is not added to the list of
						// elements to put into the clipboard.
						if (n.hasChildRecursively((Node)currElement)) {
							copiable = false;
							// because it's proven, that the 
							// currently selected element is
							// a child of another element in
							// the selection list,
							break; // abort inner while!
						}
					}
					index++; // index of next element to compare with.
				}
			} else {
				// any other element is discarded for copy
				copiable = false;
			}
			
            // The next element for which the conditions for 
			// being copyable must be fulfilled.
			currElementIndex++; 
			if (copiable) {
				retVal.add(currElement);
			}
		}
		
		return retVal;
	}
	
	public void weaveContent(AspectContainer ac) {
		
		List<Node> selection = new ArrayList<Node>();
		
		for (ModelElement me : ac.getChildren()) {
			selection.add((Node)me);
		}
		
		
		List<Node> clones = new ArrayList<Node>();
		Node elementToCopy;
		// get all selected top elements
		        
        Iterator it = selection.iterator();
		ModelObjectMapper mop = ModelObjectMapper.getInstance();
		mop.reinitialize();
		mop.setSelectedObjectsList(selection);
		// put the elements to copy into the clipboard.
		while(it.hasNext()){
			elementToCopy = (Node)it.next();
			try {                 
				clones.add((Node)mop.getClone((Node)elementToCopy));
			} catch (CloneNotSupportedException e) {
				e.printStackTrace();
			}
		}
		executeCommandsOnSelection(selection);
		Clipboard.getDefault().setContents(clones);		
		
	}
	
	/**
	 * Creates and returns a command (which may be <code>null</code>) to create a new EditPart
	 * based on the template on the clipboard.
	 * @return the paste command
	 */
	protected void copyContent() {		
		
		List selection = getSelectedObjects();
        
        List<Node> clones = new ArrayList<Node>();
		Node elementToCopy;
		// get all selected top elements
		selection = getSelectedTopElements(selection);
                
        Iterator it = selection.iterator();
		ModelObjectMapper mop = ModelObjectMapper.getInstance();
		mop.reinitialize();
		mop.setSelectedObjectsList(selection);
		// put the elements to copy into the clipboard.
		while(it.hasNext()){
			elementToCopy = (Node)it.next();
			try {                 
				clones.add((Node)mop.getClone((Node)elementToCopy));
			} catch (CloneNotSupportedException e) {
				e.printStackTrace();
			}
		}
		executeCommandsOnSelection(selection);
		Clipboard.getDefault().setContents(clones);		
	}

    /**
	 * Executes one or several commands on the selected nodes.
	 * Used for cutting.
	 * 
	 * @pre       true
	 * @post      true
	 * @param selection The selection of nodes for which 
	 *         commands should be executed.
	 */
    protected void executeCommandsOnSelection(List selection)  {
	}
	    
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		copyContent();
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		setId(ActionFactory.COPY.getId());
		setText(MenuNames.COPY);
		
		ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
		setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
		setDisabledImageDescriptor(sharedImages.getImageDescriptor(
				ISharedImages.IMG_TOOL_COPY_DISABLED));
	}
	
	@Override
	public void dispose() {
		setWorkbenchPart(null);
		super.dispose();
	}
}