package ch.unizh.ifi.adora.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.TreeEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.ui.actions.Clipboard;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.actions.resources.MenuNames;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.factories.AdoraCloneElementFactory;
import ch.unizh.ifi.adora.model.factories.ModelObjectMapper;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNode;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.parts.geometryfactory.GeometryFactory;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;


/**
 * If the current object on the clipboard contains valid elements to paste, 
 * this action will paste the elements to the viewer.
 *      
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			04.02.2006 SM First Version
 *						09.02.2006 SM Removed Bug 523, position and size was wrongly set.
 *						28.03.2006 SM Comments and fixed bug #527.
 *						30.03.2006 SM Secondary notation after paste is now preserved.
 *						15.07.2006 SM Problem Bug 560 solved.
 * 						18.07.2006 SM Comments completed, formatted, removed old code.
 * 						08.04.2008 CJ Bug 853 solved
 * 						28.04.2008 CJ Addressing Bug 783 (paste location)
 * @version			$Date: 2010-02-15 21:00:06 +0100 (Mon, 15 Feb 2010) $, $Author: vigan $, $Revision: 5141 $	   
 * @see org.eclipse.gef.ui.actions.CopyTemplateAction
 *
 */
public class AdoraPasteAction extends SelectionAction {

    /**
     * Represents the minimal distance from the left border of the element where to insert 
     * the clipboard content.
     */
    public static final int RELATIV_MINIMAL_X_COORD = 100;
    
    /**
     * Represents the minimal distance from the top border of the element where to insert 
     * the clipboard content.
     */
    public static final int RELATIV_MINIMAL_Y_COORD = 100;

	/**
	 * The location of the cursor or the context menu, when this action got triggered.
	 */
	private Point cursorLocation;

	 /**
	 * Constructor for PasteTemplateAction.
	 * @param editor
	 */
	public AdoraPasteAction(IWorkbenchPart editor) {
		super(editor);
	}


	/**
	 * @return <code>true</code> if {@link #createPasteCommand()} returns an executable command
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		// TODO Workaround for Bug 82622/39369.  Should be removed when 39369 is fixed.
		return getSelectedGraphicalEditPart() != null && getClipboardContents() != null;
	}
	
	
    
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#runWithEvent(org.eclipse.swt.widgets.Event)
	 */
	/**
	 * This method overrides the one defined in {@link Action} to obtain the location
	 * of the click. The pasted elements will be pasted at
	 * this given location.
	 * See bug report #783
	 */
	@Override
	public void runWithEvent(Event event) {
		org.eclipse.swt.graphics.Point absoluteCursorLocation = null;
		
		if(event.type == SWT.Selection)
		{
			if(event.widget instanceof MenuItem)
			{
				MenuItem menuItem = (MenuItem) event.widget;
				if((menuItem.getParent().getStyle() & SWT.POP_UP) != 0)
				{
					// The action has been triggered within a contextmenu
					if(getWorkbenchPart().getSite().getWorkbenchWindow().getActivePage().getActivePart() == getWorkbenchPart())
					{
						// It comes from the main editor view (and not the outline).
						absoluteCursorLocation = getWorkbenchPart().getLastContextMenuLocation();
					}
				}
			}
		}
		else if(event.type == SWT.KeyDown)
		{
			// The action has been triggered by a keystroke
			absoluteCursorLocation = event.display.getCursorLocation();
		}
		
		if(absoluteCursorLocation != null)
		{
			org.eclipse.swt.graphics.Point relativeCursorLocation = getWorkbenchPart().getViewer().getControl().toControl(absoluteCursorLocation);
			cursorLocation = new Point(relativeCursorLocation.x,relativeCursorLocation.y);
		}
		else
		{
			cursorLocation = null;
		}
		super.runWithEvent(event);
	}
	
	/**
	 * This method extends the weaving semantics for the case where a {@link Decision}
	 * directly connects an {@link AspectContainer} to some other {@link AspectContainer} 
	 * or to the commonatlity. The weaving is realized as a simple copy and past
	 * of all the elements inside the source {@link AspectContainer}.  
	 * 
	 * @param pasteContainer  the source {@link AspectContainer} to be weaved.
	 */
	public List<Node> weaveContent(ZoomableNodeContainer pasteContainer) {
	
		
		List<Node> wovenElements = new ArrayList<Node>();
		
		Command result;
		List<Command> createCommands = new ArrayList<Command>(); 
		
		AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
		
		Map registry = ame.getViewer().getEditPartRegistry();
		
		GraphicalEditPart gep = (GraphicalEditPart)registry.get(pasteContainer);
		
		if(gep != null)
		{
			Object content = getClipboardContents();
			Object elementToCopy;
			if (content instanceof List) {
				
				ModelObjectMapper mop = ModelObjectMapper.getInstance();
				mop.reinitialize();
				mop.setSelectedObjectsList((List)content);
				Iterator i = ((List)content).iterator();
                Point mostLeftUpperLocation = getMostUpperLeftLocation((List)content);
                 
				while (i.hasNext()) {			
					elementToCopy = i.next();
                    if (content != null) {
						CreationFactory factory = getFactory((ModelElement)elementToCopy);
						if (factory != null) {
							CreateRequest request = new CreateRequest();
							request.setFactory(factory);							
							//request.setLocation(new Point(0,0));
							//request.setSize(new Dimension(300, 200));
							
							if (elementToCopy instanceof VisualizableModelElement) {
								
								Point p = new Point(((VisualizableModelElement)elementToCopy).getLocation().x,
													((VisualizableModelElement)elementToCopy).getLocation().y);
								
								Dimension d = new Dimension(((VisualizableModelElement)elementToCopy).getSize().width,
															((VisualizableModelElement)elementToCopy).getSize().height);
								
								request.setLocation(p);
								request.setSize(d);
									
							}
							
							if (elementToCopy instanceof Node)
								wovenElements.add((Node)elementToCopy);
							
							result = gep.getCommand(request);
							if (result == null) {
								return wovenElements;
							}
							createCommands.add(result);
						}
					}
				}
				for (Command cmd : createCommands){
					execute(cmd);
				}
			}
		}
		return wovenElements;
	}
	/**
	 * Creates and executes a command to create a new EditPart
	 * based on the template on the clipboard.
	 * @pre true
     * 
     * @post true
	 */
	protected void createPasteCommand()
	{
		Command result;
		List<Command> createCommands = new ArrayList<Command>(); 
		
		GraphicalEditPart gep = getSelectedGraphicalEditPart();
		if(gep != null)
		{
			Object content = getClipboardContents();
			Object elementToCopy;
			if (content instanceof List) {
				
				ModelObjectMapper mop = ModelObjectMapper.getInstance();
				mop.reinitialize();
				mop.setSelectedObjectsList((List)content);
				Iterator i = ((List)content).iterator();
                Point mostLeftUpperLocation = getMostUpperLeftLocation((List)content);
                 
				while (i.hasNext()) {			
					elementToCopy = i.next();
                    if (content != null) {
						CreationFactory factory = getFactory((ModelElement)elementToCopy);
						if (factory != null) {
							CreateRequest request = new CreateRequest();
							request.setFactory(factory);							
							request.setLocation(getPasteLocation(gep, mostLeftUpperLocation, 
                                    (Point)GeometryFactory.getDraw2dElement(
                                    ((Node)elementToCopy).getLocation())));
							request.setSize(
                                    (org.eclipse.draw2d.geometry.Dimension)GeometryFactory.getDraw2dElement(
                                    ((Node)elementToCopy).getSize()));
							result = gep.getCommand(request);
							if (result == null) {
								return;
							}
							createCommands.add(result);
						}
					}
				}
				for (Command cmd : createCommands){
					execute(cmd);
				}
			}
		}
	}
	
	/**
	 * Obtain and cast the currently selected {@link GraphicalEditPart}.
	 * If the selected object is a {@link TreeEditPart},
	 * the corresponding {@link GraphicalEditPart} is looked up and returned.
	 * @return a {@link GraphicalEditPart} if there is one (and only one) selected, null otherwise.
	 */
	private GraphicalEditPart getSelectedGraphicalEditPart()
	{
		GraphicalEditPart gep = null;
    	List selection = getSelectedObjects();
    	if (selection != null && selection.size() == 1)
    	{
    		Object obj = selection.get(0);

    		if (obj instanceof GraphicalEditPart)
    		{
    			gep = (GraphicalEditPart)obj;
    		}
    		// If the current selected element comes
    		// from the outline view, we find the graphical
    		// edit part from the main view of the editor
    		// to which this action has been instantiated.
    		if(obj instanceof TreeEditPart)
    		{
    			AdoraModelEditor ed = (AdoraModelEditor) getWorkbenchPart();
    			TreeEditPart tep = (TreeEditPart) obj;
    			Object part = ed.getViewer().getEditPartRegistry().get(tep.getModel());
    			if(part instanceof GraphicalEditPart)
    			{
    				gep = (GraphicalEditPart) part;
    			}
    		}
    	}
		return gep;
	}
	
    /**
     * This method returns the location of the unified area of all nodes in the 
     * clipboard. This location is used as relative location for all nodes that 
     * are inserted by the paste mechanism.
     * 
     * @param  pasteSelection
     * @return Returns the location of the most upper left node of the clipboard content.
     */
    protected Point getMostUpperLeftLocation(List clipboardContent) {
        Point p;
        if (clipboardContent.size() > 0) {
            int count = 1;
            Rectangle r = (Rectangle)GeometryFactory.getDraw2dElement(
                                ((Node)clipboardContent.get(0)).getBounds());      
            
            while (count < clipboardContent.size()) {
                r.union((Rectangle)GeometryFactory.getDraw2dElement(
                        ((Node)clipboardContent.get(count)).getBounds()));
                count++;
            }        
            p = r.getLocation();
        } else {
            p = new Point(0,0);
        }
        return p;
    }
    
	/**
	 * Returns the content of the clipboard, if there is one. 
     * 
	 * @return the clipboard's contents
	 */
	protected Object getClipboardContents() {
		return Clipboard.getDefault().getContents();
	}
	
	/**
	 * Returns the appropriate Factory object to be used for the specified nodes to insert. This
	 * factory is used on the CreateRequest that is sent to the target EditPart. Note that
	 * the given template might be from a palette for a different GEF-based editor. In that
	 * case, this method can return <code>null</code>.
	 * @param elementToClone The model element that will be cloned;
	 * @return A factory that will clone the given element.
	 */
	protected CreationFactory getFactory(ModelElement elementToClone) {  
		return new AdoraCloneElementFactory(elementToClone);
	  
	}
	
	/**
     * Computes the location for the request where to paste the elements in. 
     * The location is in the l 
     * 
	 * @param container the parent of the new part that is being pasted.
     * 
     * @param relativeOrigin The location of the bounding box of all elements in the 
     *        clipboard.
     * @param originalLocation The current, relative location of of the element to paste.
	 * @return Returns the location where the clipboard content is pasted.  
	 */
	protected Point getPasteLocation(GraphicalEditPart container, Point relativeOrigin, Point originalLocation) {
		Point result = null;
		
		IFigure fig = container.getContentPane();
		Rectangle clientArea = fig.getClientArea();
		
		if(cursorLocation != null)
		{
    		if(clientArea.contains(cursorLocation))
    		{
    			// We have a cursor point and this point is in the selected element
    			// The new elements will be pasted there.
    			result = new Point(cursorLocation);
    		}
    		//TODO: Should we select the object below the cursor location ?
		}
		if(result != null)
		{
			//Otherwise, they will be placed at the default placement (top, left)
			result = new Point(clientArea.getLocation());
			result.translate(RELATIV_MINIMAL_X_COORD, RELATIV_MINIMAL_Y_COORD);
		}
		// take care of translating nested object
		Point relativeTranslation = new Point(originalLocation.x - relativeOrigin.x, originalLocation.y - relativeOrigin.y);
		return result.translate(relativeTranslation);
	}
	
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.EditorPartAction#init()
	 */	
	protected void init() {
	    setId(ActionFactory.PASTE.getId());
	    setText(MenuNames.PASTE);
	    
		ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
		setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));
		setDisabledImageDescriptor(sharedImages.getImageDescriptor(
				ISharedImages.IMG_TOOL_PASTE_DISABLED));
	}
	
	
	/**
	 * Executes the command returned by {@link #createPasteCommand()}.
	 */
	public void run() {
		createPasteCommand();
	}



	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#getWorkbenchPart()
	 */
	/**
	 * return the {@link AdoraModelEditor} this action is related to
	 */
	@Override
	protected AdoraModelEditor getWorkbenchPart()
	{
		return (AdoraModelEditor) super.getWorkbenchPart();
	}


	@Override
	public void dispose() {
		setWorkbenchPart(null);
		super.dispose();
	}
	
}
