package ch.unizh.ifi.adora.actions.representation;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.core.internal.content.Activator;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IActionDelegate2;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPropertyListener;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationStateChange;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.AdoraModelEditorPaletteFactory;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;

/**
 * This class is the the base class of all Actions associated with the view
 * management.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 16.03.2006 IV First Version 17.07.2006 SM Comments checking,
 *          reformatting, checking code.
 * @version $Date: 2009-12-23 15:35:25 +0100 (Wed, 23 Dec 2009) $, $Author: treinhard $, $Revision: 5006 $
 */
public abstract class AbstractViewAction implements IActionDelegate2, IEditorActionDelegate,
		PropertyChangeListener, IPropertyListener, IPropertyChangeListener {

	/**
     * This field is used to set the state of the correspondent toggle button
     * as well as to determine which node elements will be filtered out.
     */
	protected Class type;

	/**
     * The {@link IEditorPart} to which this Action is associated.
     */
	private AdoraModelEditor targetEditor;

	/**
     * The current model.
     */
	private AdoraModel model;

	/**
     * The action button which is linked with this action.
     */
	private IAction action;

	public AbstractViewAction() {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.addPropertyChangeListener(this);
		
		
	}
	
	/**
     * Updates the button. Gets the current filter state and sets it to the
     * button. This method is invoked by several methods which will be notified
     * if anything changes.
     */
	@SuppressWarnings("restriction")
	public void update() {
		boolean filterActive = getAdoraModel().getViewFilterState(type);

		action.setChecked(filterActive);

		ImageDescriptor id;

		if (!filterActive) {
			id = ImageDescriptor.createFromURL(FileLocator.find(AdoraPlugin.getDefault().getBundle(), new Path(getIconName()),null));
		} else {
			id = ImageDescriptor.createFromURL(FileLocator.find(AdoraPlugin.getDefault().getBundle(), new Path(getIconDeactivatedName()),null));
		}

		action.setImageDescriptor(id);

		updatePalette(type, filterActive);
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		action.setEnabled(!store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_VIEWS_DISABLED));
	}

	/**
     * Returns the icon filename of this action. This method has to be
     * implemented by subclasses.
     * 
     * @pre true
     * @post return != null
     * @return Returns the filename of the icon which represents this action.
     */
	public abstract String getIconName();

	/**
     * Returns the icon filename of this action. This method has to be
     * implemented by subclasses.
     * 
     * @pre true
     * @post != null
     * @return Returns the filename of the icon which represents this action.
     */
	public abstract String getIconDeactivatedName();

	/**
     * Return the AdoraModelEditorPaletteFactory on which icons can be set
     * hidden.
     * 
     * @return Returns the factory for the adora editor palette.
     */
	private AdoraModelEditorPaletteFactory getPaletteFactory() {
		if (targetEditor == null) {
			return null;
		}

		return targetEditor.getPaletteFactory();
	}

	/**
     * Return the AdoraModelEditorPaletteFactory on which icons can be set
     * hidden.
     * 
     * @pre true
     * @post check if returned value == null
     * @return AdoraModelEditorPaletteFactory
     */
	private AdoraModel getAdoraModel() {
		if (targetEditor == null) {
			return null;
		}

		return targetEditor.getModel();
	}

	/**
     * Executes a given {@link org.eclipse.gef.commands.Command} on the
     * {@link org.eclipse.gef.commands.CommandStack} of the editor.
     * 
     * @param cmd
     *            The {@link org.eclipse.gef.commands.Command} which gets
     *            executed on the {@link org.eclipse.gef.commands.CommandStack}.
     */
	protected void executeCommandOnStack(Command cmd) {
		targetEditor.getCommandStack().execute(cmd);
	}

	/**
     * Changes the amount of visible Palette entries. This averts, that a
     * component can be created, whose fellows are filtered at the moment.
     * 
     * @param type
     *            Whose ToolEntry's visibility should be updated
     * @param filterApplied
     *            Indicates that the filtered elements should be shown/hidden.
     */
	protected void updatePalette(Class type, boolean filterApplied) {
		AdoraModelEditorPaletteFactory palette = getPaletteFactory();
		if (palette != null) {
			ToolEntry entry = palette.getToolEntry(type);
			if (entry != null) {
				entry.setVisible(!filterApplied);
			}
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IEditorActionDelegate#setActiveEditor(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IEditorPart)
     */
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		this.action = action;

		if (this.targetEditor != null) {
			this.targetEditor.removePropertyListener(this);
		}

		this.targetEditor = (AdoraModelEditor) targetEditor;

		if (this.targetEditor != null) {
			this.targetEditor.addPropertyListener(this);
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
	public void run(IAction action) {
		RepresentationStateChange cmd = new RepresentationStateChange(type,
				getAdoraModel());
		executeCommandOnStack(cmd);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
	public void selectionChanged(IAction action, ISelection selection) {
		attachModel();
	}

	private void attachModel() {
		if (model != getAdoraModel()) {
			if (model != null) {
				model.removePropertyChangeListener(this);
			}

			model = getAdoraModel();

			if (getAdoraModel() != null) {
				model.addPropertyChangeListener(this);

				update();
			}
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
	public void propertyChange(PropertyChangeEvent evt) {
		if (AdoraModel.VIEW_FILTER_STATE_CHANGE.equals(evt
				.getPropertyName())) {
			update();
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPropertyListener#propertyChanged(java.lang.Object,
     *      int)
     */
	public void propertyChanged(Object source, int propId) {
		if (AdoraModelEditor.PROP_MODEL_CHANGE == propId) {
			attachModel();
		}
	}
	
	
	/**
	 * Disables the buttons if the view mechanism is disabled in the preferences.
	 */
	public void propertyChange(org.eclipse.jface.util.PropertyChangeEvent event) {
		
		if(event.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_VIEWS_DISABLED)) {
			
			IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
			action.setEnabled(!store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_VIEWS_DISABLED));
			
		}
		
	}
	
	/*
	* IActionDelegate2 methods (add lifecycle events)
	*/
	
	/**
	 * This method unregisters all listeners and resets references
	 */
	public void dispose()
	{
		if(model != null)
		{
			model.removePropertyChangeListener(this);
			model = null;
		}
		
		if(targetEditor != null)
		{
			targetEditor.removePropertyListener(this);
			targetEditor = null;
		}
		
		AdoraPlugin.getDefault().getPreferenceStore().removePropertyChangeListener(this);
	}
	
	public void init(IAction action) {
			
	}

	public void runWithEvent(IAction action, Event event) {
		run(action);
	}
}
