package ch.unizh.ifi.adora.actions.representation;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Device;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;

/**
 * This action triggers the unweaving function.
 * 
 * @author Michael Jehle
 */
public class UnweaveAction extends SelectionAction
{
    /**
     * The action's ID.
     */
    public static final String UNWEAVE_ACTION = "Unweave";

    /**
     * The blackboard where the unweaving jobs are pushed.
     * The blackboard is loaded via dynamic class loader to consider
     * the case, that not every user may have checked out the unweaving
     * mechanism.
     */
    private Object blackboard;
    
    /**
     * This mouse listener is used to find out where the unweaved parts should
     * be places. The user can define the location with a mouse click.
     */
    private MouseListener editorMouseListener;
    
    /**
     * To place the extracted model elements the cursor is changed
     * to the estimated size of the new aspect. This is a reference to 
     * that cursor (needed to remove it after the aspect placement).
     */
    private Cursor placeAspectCursor;
    
    /**
     * This variable is set to false by the selection analyzer of the unweaver
     * if the selection was not valid.
     */
    private boolean selectionWasOk;
    
    /**
     * The selected model elements converted to VisualizableModelElements.
     */
    private List<VisualizableModelElement> selection;

    /**
     * The action's constructor, defining the part the action is relevant for.
     * 
     * @param part IWorkbenchPart
     */
    public UnweaveAction(IWorkbenchPart part)
    {
        super(part);
        
        try
        {
        	this.blackboard = Class.forName("ch.unizh.ifi.adora.unweaver.blackboard.board.Blackboard").newInstance();
		}
        catch (ClassNotFoundException e)
        {
        	System.out.println("blackboard not found");
			// do nothing
        	// class is not found if the user has not checked out
        	// the respective project which only has the consequence
        	// that the unweaving will not be available.
		}
        catch (InstantiationException e)
		{
			e.printStackTrace();
		}
        catch (IllegalAccessException e)
		{
			e.printStackTrace();
		}
        
        this.initMouseListener();
    }
    
    private void initMouseListener()
    {
    	this.editorMouseListener = new MouseListener()
    	{
			public void mouseDoubleClick(MouseEvent e)
			{
				
			}

			public void mouseDown(MouseEvent e)
			{
				
			}

			public void mouseUp(MouseEvent e)
			{
				try
				{
					blackboard.getClass().getMethod("addExtractionCoordinates", new Class[] {Integer.class, Integer.class}).invoke(blackboard, new Object[] {new Integer(e.x), new Integer(e.y)});
				}
				catch (IllegalArgumentException e1)
				{
					e1.printStackTrace();
				}
				catch (SecurityException e1)
				{
					e1.printStackTrace();
				}
				catch (IllegalAccessException e1)
				{
					e1.printStackTrace();
				}
				catch (InvocationTargetException e1)
				{
					e1.printStackTrace();
				} 
				catch (NoSuchMethodException e1)
				{
					e1.printStackTrace();
				}
				
				// remove cursor after aspect placement
    			placeAspectCursor.dispose();
				
				AdoraPlugin.getActiveEditor().getViewer().getControl().removeMouseListener(editorMouseListener);
			}
		};
    }
    
    public Object getBlackboard()
    {
    	return this.blackboard;
    }

    @Override
    protected boolean calculateEnabled()
    {
        if (getSelectedObjects().isEmpty() || getSelectedObjects().size() < 1)
        {
            return false;
        }
        
        if (this.blackboard == null)
        {
        	return false;
        }

        return true;
    }
    
    protected void init()
    {
        setId(UnweaveAction.UNWEAVE_ACTION);
        setText("Extract Selection into Variant");
    }

    /**
     * Triggers the action.
     */
    public void run()
    {
    	//note that this boolean is set to false by the selection analyzer
    	//in case of an invalid selection.
    	this.selectionWasOk = true;
    	
        //convert selection into model element references
        List<AbstractGraphicalEditPart> jobVMEEP = convertSelectionToEditParts(getSelectedObjects());
        this.selection = convertSelectionIntoModelElements(jobVMEEP);
    	
    	try
    	{
    		this.blackboard.getClass().getMethod("addJob", List.class).invoke(this.blackboard, this.selection);
		}
    	catch (IllegalArgumentException e)
    	{
			e.printStackTrace();
		}
    	catch (SecurityException e)
    	{
			e.printStackTrace();
		}
    	catch (IllegalAccessException e)
    	{
			e.printStackTrace();
		}
    	catch (InvocationTargetException e)
    	{
			e.printStackTrace();
		}
    	catch (NoSuchMethodException e)
    	{
			e.printStackTrace();
		}
    	
    	
    	if (this.selectionWasOk)
    	{
    		//add the mouse listener to the editor to find out where the extracted
    		// objects should be placed
    		AdoraPlugin.getActiveEditor().getViewer().getControl().addMouseListener(editorMouseListener);
 
    		int[] estimatedSize = this.estimateExtractionSize(this.selection);
    		this.setExtractionCurser(estimatedSize[0], estimatedSize[1]);
    	}
    }
    
    private void setExtractionCurser(int xSize, int ySize)
    {
    	Device device = AdoraPlugin.getActiveEditor().getViewer().getControl().getDisplay();
/*    	
    	Color white = device.getSystemColor(SWT.COLOR_WHITE);
    	Color black = device.getSystemColor(SWT.COLOR_BLACK);
    	
    	PaletteData paletteData = new PaletteData(new RGB[] {white.getRGB(), black.getRGB(), });

    	ImageData imageData = new ImageData(xSize, ySize, 1, paletteData);
    	
    	for (int x = 0; x < xSize; x++)
    	{
    		for (int y = 0; y < ySize; y++)
    		{
    			if (y == 0 || x == 0 || y == ySize - 1 || x == xSize - 1)
    			{
    				imageData.setPixel(x, y, 1);
    			}
    		}
    	}
    	
    	this.placeAspectCursor = new Cursor(device, imageData, 0, 0);
*/
    	this.placeAspectCursor = new Cursor(device, SWT.CURSOR_CROSS);
    	AdoraPlugin.getActiveEditor().getViewer().getControl().setCursor(this.placeAspectCursor);
    }
    
    /**
     * This method estimates the size of the aspect, needed to
     * containt the extracted elements.
     * 
     * @return int[2] {xSize, ySize}
     */
    private int[] estimateExtractionSize(List<VisualizableModelElement> selection)
    {
    	//init values with first element.
//    	int leftMax = selection.get(0).getLocation().x;
//    	int rightMax = selection.get(0).getLocation().x;
//    	int topMax = selection.get(0).getLocation().y;
//    	int bottomMax = selection.get(0).getLocation().y;
//    	
//    	for (VisualizableModelElement vme : selection)
//    	{
//    		if (vme.getLocation().x < leftMax)
//    		{
//    			leftMax = vme.getLocation().x;
//    		}
//    		
//    		if ((vme.getLocation().x + vme.getSize().width) > rightMax)
//    		{
//    			rightMax = vme.getLocation().x + vme.getSize().width;
//    		}
//    		
//    		if (vme.getLocation().y < topMax)
//    		{
//    			topMax = vme.getLocation().y;
//    		}
//    		
//    		if ((vme.getLocation().y + vme.getSize().height) > bottomMax)
//    		{
//    			bottomMax = vme.getLocation().y + vme.getSize().height;
//    		}
//    	}
//    	
//    	return new int[] {rightMax - leftMax, bottomMax - topMax};
    	return new int[] {100, 70};
    }
    
    public List<VisualizableModelElement> convertSelectionIntoModelElements(List<AbstractGraphicalEditPart> job)
    {
        List<VisualizableModelElement> jobConverted = new ArrayList<VisualizableModelElement>();
        
        for (AbstractGraphicalEditPart agep : job)
        {
            if (agep.getModel() instanceof VisualizableModelElement)
            {
                jobConverted.add((VisualizableModelElement) agep.getModel());    
            }
            else
            {
                throw new IllegalArgumentException("Could not convert selection into visualizable model elements.");
            }
        }
        
        return jobConverted;
    }
    
    /**
     * Converts the selection into VisualizableModelElementEditParts.
     * 
     * @param job List<Object>
     * @return List<VisualizableModelElementEditPart>
     */
    private List<AbstractGraphicalEditPart> convertSelectionToEditParts(List<Object> job)
    {
        List<AbstractGraphicalEditPart> jobConverted = new ArrayList<AbstractGraphicalEditPart>();
        
        for (Object o : job)
        {
            if (o instanceof AbstractGraphicalEditPart)
            {
                jobConverted.add((AbstractGraphicalEditPart) o);
            }
            else
            {
                throw new IllegalArgumentException("Unewaving command contained objects not of type AbstractGraphicalEditPart");
            }
        }
        
        return jobConverted;
    }
    
    /**
     * Resets the action in case of an invalid user selection.
     */
    public void resetAction()
    {
    	this.selectionWasOk = false;
    }
    
	@Override
	public void dispose() {
		setWorkbenchPart(null);
		super.dispose();
	}
}