package ch.unizh.ifi.adora.model;

import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * This class offers several methods, which are used to determine if connected
 * {@link ch.unizh.ifi.adora.model.representation.Scenario}s
 * are placed validly and if 
 * {@link ch.unizh.ifi.adora.model.representation.ScenarioConnection} 
 * between {@link ch.unizh.ifi.adora.model.representation.Scenario}s are valid.  
 * 
 * Since these constraints have to be queried in various objects, they have been 
 * situated in this independent class.  
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			19.07.2006 SM First Version
 * @version			$Date: 2006-07-19 16:09:42 +0200 (Wed, 19 Jul 2006) $, $Author: smeier $, $Revision: 1858 $
 */
public class ScenarioHierarchyConstraints {

	/**
	 * Used to check if a {@link ch.unizh.ifi.adora.model.representation.ScenarioConnection}
	 * can be drawn or not.
	 * 
	 * @param source {@link ch.unizh.ifi.adora.model.representation.Scenario}
	 					where the {@link ch.unizh.ifi.adora.model.representation.ScenarioConnection}starts.
	 * @param target {@link ch.unizh.ifi.adora.model.representation.Scenario} 
	 * 					where the {@link ch.unizh.ifi.adora.model.representation.ScenarioConnection}  ends.
	 * @return true, if the target is situated in a {@link ch.unizh.ifi.adora.model.representation.Component}
	 * 					which is a child of the 
	 * 						{@link ch.unizh.ifi.adora.model.representation.Component}
	 * 					of the source.
	 */
	public static boolean isTargetInContainerChild(Node source, Node target) {
		//check if component parent hierarchy is allowed
		return (source.getParent().hasChildRecursively((Node) target
				.getParent()))
				|| source.getParent() == target.getParent();
	}

	/**
	 * Used to determine if a connected scenario is placed validly.
	 * 
	 * @param parent scenario-parent
	 * @param container Container in which a scenario is being moved
	 * 
	 * @return ture, if the scenario's parent is situated in 
	 * a component which is parent of the container.
	 */
	public static boolean isChildContainerChildOfContainer(Node parent,
			INodeContainer container) {
		//check if component parent hierarchy is allowed
		return (parent.getParent().hasChildRecursively((Node) container))
				|| parent.getParent() == container;
	}

	/**
	 * Used to determine if a connected scenario is placed validly.
	 * 
	 * @param child scenario-parent
	 * @param container Container in which a scenario is being moved
	 * 
	 * @return  true, if the container contains the partent of the child.
	 */
	public static boolean isContainerParentOfChildContainer(Node child,
			INodeContainer container) {
		//check if component parent hierarchy is allowed
		return (container.hasChildRecursively((Node) child.getParent()))
				|| child.getParent() == container;
	}
}