package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;

/**
 * Adds a node to a parent node. This is e.g. done if a node is orphaned and
 * dragged into another parent node afterwards. 
 * 
 * 
 * @author 	 	smeier
 * @copyright 		Requirements Engineering Research Group<br>
 *            		Department of Informatics<br>
 *            		University of Zurich<br>
 * @history   		20.09.2005 SM First Version
 *                  31.10.2005 SM Comments
 *                  26.07.2006 JS Comments, required parameters must be
 *                                passed to the constructor 
 * @version   		$Date: 2009-02-09 09:29:16 +0100 (Mon, 09 Feb 2009) $, $Author: vigan $, $Revision: 3997 $
 */
public class Add extends Command {

    /**
     * Stores the the child node
     */
	private Node child;

	/**
     * Stores the parent node.
	 */
	private INodeContainer parent;

	/**
	 * Creates the add command with a label for undo history
	 * 
	 * @param child The child to add to the parent.
	 * @param newParent The parent to which child is added.
	 * 
	 * @pre child != null && newParent != null
	 */
	public Add(Node child, INodeContainer newParent) {
		super("Add Node");
		setChild(child);
		setParent(newParent);
	}

	/**
	 * Executes this command by adding the given child node to the new parent
	 * node.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		parent.addChild(child);
		//check if interrelationships are still valid 
		//only needful, if child instanceof ZoomableNodeContainer. 
		if(child instanceof ZoomableNodeContainer){
			ConnectionAbstractionAlgorithm.getInstance().createInterRelations((ZoomableNodeContainer)child);
		}
	}

	/**
	 * Returns the (new) parent node of the child node.
	 * 
	 * @pre true
	 * @post return != null
	 * @return Returns the parent node.
	 */
	public INodeContainer getParent() {
		return parent;
	}

	/**
	 * Redoes the add command by readding the child node to the parent node
	 * 
	 * @pre true
	 * @post true
	 */
	public void redo() {
		parent.addChild(child);
		ConnectionAbstractionAlgorithm.getInstance().createInterRelations(child);
	}

	/**
	 * Sets the child for this add command.
	 * 
	 * @pre subpart != null
	 * @post true
	 * @param subpart
	 *            The child to add to the parent.
	 */
	public void setChild(Node subpart) {
		child = subpart;
	}

	/**
	 * Sets the parent of this add command.
	 * 
	 * @pre newParent != null
	 * @post true
	 * @param newParent
	 *            The parent of this add command.
	 */
	public void setParent(INodeContainer newParent) {
		parent = newParent;
	}

	/**
	 * Executes the undo of this command by removing the given child from the
	 * parent.
	 * 
	 * @pre true
	 * @post true
	 */
	public void undo() {
		parent.removeChild(child);
	}
	
	@Override
	public String toString() {
		return "Adding " + child + " (" + child.getClass().getSimpleName() + ")";
	}
 }