package ch.unizh.ifi.adora.model.commands;

import org.eclipse.draw2d.geometry.Dimension;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;

/**
 * This command allows a
 * {@link ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer} to get
 * resized in an intuitive way.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 17.08.2006 IV First Version
 * @version $Date: 2007-09-21 17:19:36 +0200 (Fri, 21 Sep 2007) $, $Author: treinhard $, $Revision: 3001 $
 */
public class ChangeContainerSize extends VisualizableModelElementSetConstraint {

	/**
	 * ZoomableNodeContainer to manipulate.
	 */
	transient private final ZoomableNodeContainer container;

	/**
	 * Stores the move delta for undoing reason.
	 */
	private Dimension moveDelta = new Dimension();

	/**
	 * Stores the space all children need.
	 */
	private Rectangle childrenBounds;

	/**
	 * Stores the new Bounds without the border of the parent.
	 */
	private Rectangle cropedNewBounds;

	/**
	 * Stores the new Bounds without the border of the parent.
	 */
	private Rectangle cropedOldBounds;

	/**
	 * Create a command that can resize and/or move a container intuitively.
	 * 
	 * @param container The container to manipulate.
	 * @param newBounds The new Bounds.
	 * @param childrenBounds The space all children need.
	 * @param cropedNewBounds The new Bounds without it's border.
	 */
	public ChangeContainerSize(ZoomableNodeContainer container,
			Rectangle newBounds, Rectangle childrenBounds,
			Rectangle cropedNewBounds, Rectangle cropedOldBounds) {
		super(container, newBounds);
		this.container = container;
		this.cropedNewBounds = cropedNewBounds;
		this.childrenBounds = childrenBounds;
		this.cropedOldBounds = cropedOldBounds;
		setLabel("Change container Bounds");
	}

	/**
	 * This method is responsible for an intuitive resizing of {@link ZoomableNodeContainer} containing children.
	 * The resizing works as follows:
	 * 
	 * The basic resizing strategy is to crop space at the edge where the resizing took place. 
	 * 
	 * Since children coordinates are relative to the parent children have to be moved in the following situations: <br>
	 * 		- If the container gets resized from the left or from the top, all children need to be moved in by thy resizing amount 
	 * 			but in the opposite direction. <br>
	 * 		- If the container gets resized from the right or bottom and the new bounds don't contain the children bounds
	 * 			all children need to be moved in by thy resizing amount.<br>
	 */
	public void redo() {
	
		super.redo();

		moveDelta = new org.eclipse.draw2d.geometry.Dimension(newBounds.width
				- oldBounds.width, newBounds.height - oldBounds.height);

		Rectangle absoluteChildrenBounds = childrenBounds.getCopy();

		absoluteChildrenBounds.x += newBounds.x + newBounds.x
				- oldBounds.x;
		absoluteChildrenBounds.y += newBounds.y + newBounds.y
				- oldBounds.y;

		boolean enoughHorizontalSpace = cropedNewBounds.x <= absoluteChildrenBounds.x
				&& cropedNewBounds.right() >= absoluteChildrenBounds.right();

		boolean enoughVerticalSpace = cropedNewBounds.y <= absoluteChildrenBounds.y
				&& cropedNewBounds.bottom() >= absoluteChildrenBounds.bottom();

		if (newBounds.x != oldBounds.x && !moveDelta.equals(0, 0)) { //resize from left	
			if (newBounds.x - oldBounds.x > childrenBounds.x)
				moveDelta.width = -childrenBounds.x;
		} else if (!enoughHorizontalSpace) { //resize from right where not enough space exists.
			//use the space between the right parent border and the 
			//rightmost child too.
			moveDelta.width += cropedOldBounds.width
						- childrenBounds.getRight().x;
		} else {
			moveDelta.width = 0;
		}

		if (newBounds.y != oldBounds.y && !moveDelta.equals(0, 0)) { //resize from top	
			if (newBounds.y - oldBounds.y > childrenBounds.y)
				moveDelta.height = -childrenBounds.y;
		} else if (!enoughVerticalSpace) { //resize from right where not enough space exists.
			//use the space between the lower parent border and the 
			//lowest child too.
			moveDelta.height += cropedOldBounds.height
					- childrenBounds.getBottom().y;
		} else {
			moveDelta.height = 0;
		}

		if (!moveDelta.equals(0, 0)) {
			moveChildren(container, moveDelta);
		}
	}

	/**
	 * Moves all direct children of the <code> contaienr </code> by the amount
	 * of.
	 * 
	 * @param container {@link ZoomableNodeContainer} in which the children are situated.
	 * @param moveDelta The amount by which all Chidren are moved towards the origin.
	 */
	private void moveChildren(ZoomableNodeContainer container,
			Dimension moveDelta) {

		Node n;
		VisualizableModelElementSetConstraint moveCommand;

		Rectangle constraints;

		for (ModelElement me : container.getChildren()) {
			n = (Node) me;

			constraints = new Rectangle(
					new ch.unizh.ifi.adora.model.geometry.Point(
							n.getLocation().x + moveDelta.width, n
									.getLocation().y
									+ moveDelta.height), n.getSize());

			moveCommand = new VisualizableModelElementSetConstraint(n,
					constraints);

			moveCommand.execute();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		moveChildren(container, new Dimension(-moveDelta.width,
				-moveDelta.height));
		super.undo();
	}
}