package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;


/**
 * A command to create a connection between two elements. The command can be
 * undone or redone.
 * <p>
 * This command is designed to be used together with a GraphicalNodeEditPolicy.
 * To use this command properly, following steps are necessary:
 * </p>
 * <ol>
 *		<li>Create a subclass of GraphicalNodeEditPolicy.</li>
 *		<li>Override the <tt>getConnectionCreateCommand(...)</tt> method to
 *			create a new instance of this class and put it into the
 *			CreateConnectionRequest.</li>
 *		<li>Override the <tt>getConnectionCompleteCommand(...)</tt> method to
 *			obtain the Command from the ConnectionRequest, call setTarget(...) to set the
 *			target endpoint of the connection and return this command instance.</li>
 * </ol>
 * 
 * See {@link ch.unizh.ifi.adora.parts.NodeEditPart#createEditPolicies()} for an
 *      example of the above procedure.
 *
 * @author					Silvio Meier
 * @author					Adrian Gygax
 * @copyright				Requirements Engineering Research Group<br>
 *							Department of Informatics<br>
 *							University of Zurich<br>
 * @history					Dec 5, 2005 SM First Version
 * 							Jul 17, 2006 SM Comments completed, reformatting.
 * 						 	Jul 26, 2006 JS Comments
 * @version				$Date: 2007-06-28 15:55:00 +0200 (Thu, 28 Jun 2007) $, $Author: schoeni $, $Revision: 2807 $
 * @responsibilities		Represents the command for creating a connection.
 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy
 */
public abstract class ConnectionCreate extends Command {
	
	/** 
	 * The connection instance. 
	 */
	private Connection connection;

	/** 
	 * Start endpoint for the connection. 
	 */
	private final ConnectableElement source;

	/**
	 * Stores the relative coordinates of the clickpoint inside the source node. 
	 */
	private Point sourceClickPoint;
	
	/** 
	 * Target endpoint for the connection. 
	 */
	private ConnectableElement target;
	
	/**
	 * Stores the relative coordinates of the clickpoint inside the target node.
	 */
	private Point targetClickPoint;

	/**
	 * Instantiate a command that can create a connection between two shapes.
	 * 
	 * @param source
	 *            the source endpoint (a non-null Shape instance)
	 * @throws IllegalArgumentException
	 *             if source is null
	 */
	public ConnectionCreate(ConnectableElement source) {
		if (source == null) {
			throw new IllegalArgumentException();
		}
		setLabel("Connection Creation");
		this.source = source;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
    @Override
	public abstract void execute(); 

	/**
     * Returns the newly created connection, i.e.
     * the connection that was added to the
     * model for the first time.
     * 
     * @return Returns the connection which was newly 
     *			created.
     */
    public Connection getConnection(){
        return connection;
    }

	/**
     * Returns the source for the connection to create.
     * 
     * @return != null
     */
    public ConnectableElement getSource()
    {
        return source;
    }

    /**
	 * Returns a copy of the first clickpoint of this connection.
	 * 
	 * @pre true 
	 * @post sourceClickPoint.equals(return)
	 * @return Returns a copy of the first clickpoint.
	 */
	public Point getSourceClickPoint() {
		if (sourceClickPoint == null) {
			return null;
		}
		
		return sourceClickPoint.getCopy();
	}
	
	/**
     * Returns the target for the connection to create.
     * 
     * @return != null
     */
    public ConnectableElement getTarget() {
        return target;
    }
	
	/**
	 * Returns a copy of the second clickpoint of this connection.
	 * 
	 * @pre true 
	 * @post targetClickPoint.equals(return)
	 * @return Returns a copy of the second clickpoint.
	 */
	public Point getTargetClickPoint() {
		if (targetClickPoint == null) {
			return null;
		}
		
		return targetClickPoint.getCopy();
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	@Override
	public void redo() {
		connection.reconnect();
	}
	
	/**
	 * Sets the first clickpoint of this connection, i.e. the point
	 * where the user clicked to begin drawing the connection. The 
	 * coordinates have to be relative to the left upper corner of the
	 * figure where the drawing of the connection started.
	 * @pre p != null
	 * @post this.sourceClickPoint == p
	 * @param p The first clickpoint of the connection in relative coordinates. 
	 */
	public void setSourceClickPoint(Point p) {
		sourceClickPoint = p;
	}
    
    /**
	 * Set the target endpoint for the connection.
	 * 
	 * @param target
	 *            that target endpoint (a non-null Shape instance)
	 * @throws IllegalArgumentException
	 *             if target is null
	 */
	public void setTarget(ConnectableElement target) {
		if (target == null) {
			throw new IllegalArgumentException();
		}
		this.target = target;
	}
    
    /**
	 * Sets the second clickpoint of this connection, i.e. the point
	 * where the user clicked to end the drawing of the connection. The 
	 * coordinates have to be relative to the left upper corner of the
	 * figure where the drawing of the connection ended.
	 * @pre p != null
	 * @post this.targetClickPoint == p
	 * @param p The second clickpoint of the connection in relative coordinates. 
	 */
	public void setTargetClickPoint(Point p) {
		targetClickPoint = p;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
    @Override
	public void undo() {
		connection.disconnect();
	}
    
    /**
     * Sets the connection instance created by this command.
     * 
     * @param newConnection the connection that has been created
     */
    protected void setConnection(Connection newConnection) {
        this.connection = newConnection;
    }
}
