package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * A command which disconnects (remove) a connection 
 * from its endpoints. 
 *
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				31.10.2005 SM First Version
 *						19.07.2006 SM Comments added, Reformatting
 *						08.08.2006 JS Added getter for the connection
 * @version			$Date: 2009-07-06 14:50:26 +0200 (Mon, 06 Jul 2009) $, $Author: jehle $, $Revision: 4443 $
 */
public class ConnectionDelete extends Command {
	/**
	 * Avoids deletion of Decision in product line table. 
	 */
	private boolean decisionVisibility=false;
	/** 
	 * Connection instance to disconnect. 
	 */
	private final Connection connection;

	/**
	 * Stores all child connections for undoing.
	 */
	private List<Connection> childConnections;
	
	/**
	 * Stores a reference to the corresponding {@link AdoraModel}.
	 */
	private AdoraModel adoraModel;

	/**
	 * Create a command that will disconnect a connection from its endpoints.
	 * 
	 * @param conn
	 *            the connection instance to disconnect (non-null)
	 * @throws IllegalArgumentException
	 *             if conn is null
	 */
	public ConnectionDelete(Connection conn) {
		if (conn == null) {
			throw new IllegalArgumentException();
		}
		setLabel("Connection Deletion");
		this.connection = conn;
		childConnections = new ArrayList<Connection>();
	}
	
	public ConnectionDelete(Connection conn, boolean decisionInTable){
		this(conn);
		this.decisionVisibility=decisionInTable;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		if (connection instanceof JoinRelation){
			Decision decision = getAdoraModel().getDecision((JoinRelation)connection);
			if (decision != null) {
				String confConstr = "";
				for (VariationPoint vp : adoraModel.getVariationPoints()) {
					if (vp.getDecisions().contains(decision)) {
						confConstr += "VP"+vp.getNr()+" ";
					}
				}
				for (Constraint c : adoraModel.getConstraints()) {
					if (c.containsDecision(decision)) {
						confConstr += "C"+c.getNr()+" ";
					}
				}
				
				if (!confConstr.equals("")) {
				    //showWarning(decision.getNr(), confConstr);
					return;
				}
			}
		}
			
		
		
		for (Connection c : connection.getConnections()){
			childConnections.add(c);
			c.disconnect();
			c.informRemoved();
		}
		if (connection instanceof Transition){
			((Transition)connection).getLabel().informRemoved();
		} 
		
		else if (connection instanceof JoinRelation){
			Decision dec = getAdoraModel().getDecision((JoinRelation)connection);
			if (dec != null && dec.getJoinRelations().size() == 1) {
				removeDecisionFromProductLineDecisionTable(dec);
			}
		}
		
		connection.disconnect();
		connection.informRemoved();
	}
	/**
	 * This method is needed for dynamic weaving. We do not want to delete any decisions there.
	 * @param dec
	 */
	public void removeDecisionFromProductLineDecisionTable(Decision dec){
		//Remove Decision from Product Lines Decision Table if this is no dynamic weaving operation. 
		//Otherwise do not remove them, since they are needed for modeling purposes.
		if(!decisionVisibility){
			getAdoraModel().removeDecision(dec);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		connection.reconnect();
		for (Connection c : childConnections){
			c.reconnect();
		}
	}

	/**
	 * Gets the connection to be deleted.
	 *
	 * @return the connection to be deleted
	 */
	protected Connection getConnection() {
		return connection;
	}
	
	/**
	 * Returns the {@link AdoraModel}.
	 * 
	 * @return The corresponding {@link AdoraModel}.
	 */
	private AdoraModel getAdoraModel() {
		if (adoraModel == null) {

			IContainer parent = connection.getSource().getParent();
			
			while (parent.getParent() != null) {
				parent = parent.getParent();
			}
			adoraModel = (AdoraModel) parent;
		}
		return adoraModel;
	}
	
	private void showWarning(int decNr, String conf) {
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench().
				getActiveWorkbenchWindow().getShell(), 
						SWT.ICON_INFORMATION | SWT.OK);
		mb.setText("Decision Conflict");
		mb.setMessage("The Decision D"+ decNr+ "appears in: "+conf);
	    mb.open();
	}
}