package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;

/**
 * A command to reconnect a connection to a different start point or end point.
 * The command can be undone or redone.
 * <p>
 * This command is designed to be used together with a GraphicalNodeEditPolicy.
 * To use this command propertly, following steps are necessary:
 * </p>
 * <ol>
 * <li>Create a subclass of GraphicalNodeEditPolicy.</li>
 * <li>Override the <tt>getReconnectSourceCommand(...)</tt> method. Here you
 * need to obtain the Connection model element from the ReconnectRequest, create
 * a new ConnectionReconnectCommand, set the new connection <i>source</i> by
 * calling the <tt>setNewSource(Shape)</tt> method and return the command
 * instance.
 * <li>Override the <tt>getReconnectTargetCommand(...)</tt> method.</li>
 * Here again you need to obtain the Connection model element from the
 * ReconnectRequest, create a new ConnectionReconnectCommand, set the new
 * connection <i>target</i> by calling the <tt>setNewTarget(Shape)</tt>
 * method and return the command instance.</li>
 * </ol>
 * 
 * @history 30.10.2006 SM First Version 17.07.2006 SM Comments, Reformatting.
 * @version $Date: 2007-10-10 14:04:59 +0200 (Wed, 10 Oct 2007) $, $Author: schoeni $, $Revision: 3113 $
 *          see ch.unizh.ifi.adora.parts.NodeEditPart#createEditPolicies() for
 *          an example of the above procedure.
 * 
 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy
 * @see #setNewSource(Node)
 * @see #setNewTarget(Node)
 * @author smeier
 */
public abstract class ConnectionReconnect extends Command {

	/**
     * The connection instance to reconnect.
     */
	protected Connection connection;

	/**
     * The new source endpoint.
     */
	protected ConnectableElement newSource;

	/**
     * The new target endpoint.
     */
	protected ConnectableElement newTarget;

	/**
     * The original source endpoint.
     */
	protected final ConnectableElement oldSource;

	/**
     * The original target endpoint.
     */
	protected final ConnectableElement oldTarget;

	/**
     * Stores the relative coordinates of the clickpoint inside the source node.
     */
	private Point sourceClickPoint;

	/**
     * Stores the relative coordinates of the clickpoint inside the target node.
     */
	private Point targetClickPoint;

	/**
     * This attribute is used to store the old source click point for undoing
     * the changes of the command.
     */
	private Point oldSourceClickPoint;

	/**
     * This attribute is used to store the old target click point for undoing
     * the changes of the command.
     */
	private Point oldTargetClickPoint;

	/**
     * Instantiate a command that can reconnect a Connection instance to a
     * different source or target endpoint.
     * 
     * @param conn
     *            the connection instance to reconnect (non-null)
     * @throws IllegalArgumentException
     *             if conn is null
     */
	public ConnectionReconnect(Connection conn) {
		if (conn == null) {
			throw new IllegalArgumentException();
		}
		this.connection = conn;
		this.oldSource = conn.getSource();
		this.oldTarget = conn.getTarget();
		oldSourceClickPoint = conn.getSourceClickPoint();
		oldTargetClickPoint = conn.getTargetClickPoint();
		setLabel("Reconnects Connection");
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
	public boolean canExecute() {
		return true;
	}

	/**
     * Reconnect the connection to newSource (if setNewSource(...) was invoked
     * before) or newTarget (if setNewTarget(...) was invoked before).
     */
	public void execute() {
		if (newSource != null) {
			connection.reconnect(newSource, oldTarget);
			connection.setSourceClickPoint(sourceClickPoint);
		} else if (newTarget != null) {
			connection.reconnect(oldSource, newTarget);
			connection.setTargetClickPoint(targetClickPoint);
		} else {
			throw new IllegalStateException("Should not happen");
		}
		if (connection instanceof Association) {
			ConnectionAbstractionAlgorithm.getInstance()
					.recalculateInterRelations((Association) connection);
		}
	}

	/**
     * Set a new source endpoint for this connection. When execute() is invoked,
     * the source endpoint of the connection will be attached to the supplied
     * Shape instance.
     * <p>
     * Note: Calling this method, deactivates reconnection of the <i>target</i>
     * endpoint. A single instance of this command can only reconnect either the
     * source or the target endpoint.
     * </p>
     * 
     * @param connectionSource
     *            a non-null Shape instance, to be used as a new source endpoint
     * @throws IllegalArgumentException
     *             if connectionSource is null
     */
	public void setNewSource(ConnectableElement connectionSource) {
		if (connectionSource == null) {
			throw new IllegalArgumentException();
		}
		setLabel("move connection startpoint");
		newSource = connectionSource;
		newTarget = null;
	}

	/**
     * Set a new target endpoint for this connection When execute() is invoked,
     * the target endpoint of the connection will be attached to the supplied
     * Shape instance.
     * <p>
     * Note: Calling this method, deactivates reconnection of the <i>source</i>
     * endpoint. A single instance of this command can only reconnect either the
     * source or the target endpoint.
     * </p>
     * 
     * @param connectionTarget
     *            a non-null Shape instance, to be used as a new target endpoint
     * @throws IllegalArgumentException
     *             if connectionTarget is null
     */
	public void setNewTarget(ConnectableElement connectionTarget) {
		if (connectionTarget == null) {
			throw new IllegalArgumentException();
		}
		setLabel("move connection endpoint");
		newSource = null;
		newTarget = connectionTarget;
	}

	/**
     * Reconnect the connection to its original source and target endpoints.
     */
	public void undo() {
		connection.reconnect(oldSource, oldTarget);
		connection.setSourceClickPoint(oldSourceClickPoint);
		connection.setTargetClickPoint(oldTargetClickPoint);
	}

	/**
     * Sets the first clickpoint of this connection, i.e. the point where the
     * user clicked to begin drawing the connection. The coordinates have to be
     * relative to the left upper corner of the figure where the drawing of the
     * connection started.
     * 
     * @pre p != null
     * @post this.sourceClickPoint == p
     * @param p
     *            The first clickpoint of the connection in relative
     *            coordinates.
     */
	public void setSourceClickPoint(Point p) {
		sourceClickPoint = p;
	}

	/**
     * Sets the second clickpoint of this connection, i.e. the point where the
     * user clicked to end the drawing of the connection. The coordinates have
     * to be relative to the left upper corner of the figure where the drawing
     * of the connection ended.
     * 
     * @pre p != null
     * @post this.targetClickPoint == p
     * @param p
     *            The second clickpoint of the connection in relative
     *            coordinates.
     */
	public void setTargetClickPoint(Point p) {
		targetClickPoint = p;
	}

	/**
     * Returns a copy of the first clickpoint of this connection.
     * 
     * @pre true
     * @post sourceClickPoint.equals(return)
     * @return Returns a copy of the first clickpoint.
     */
	public Point getSourceClickPoint() {
		return sourceClickPoint.getCopy();
	}

	/**
     * Returns a copy of the second clickpoint of this connection.
     * 
     * @pre true
     * @post targetClickPoint.equals(return)
     * @return Returns a copy of the second clickpoint.
     */
	public Point getTargetClickPoint() {
		return targetClickPoint.getCopy();
	}

	public ConnectableElement getNewTarget() {
		return newTarget;
	}

	/**
     * @return the newSource
     */
	public ConnectableElement getNewSource() {
		return newSource;
	}

}
