package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;
import ch.unizh.ifi.adora.utils.ZoomAlgorithmsManager;

/**
 * Performs a fisheye zoom on a
 * {@link ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer} and a
 * AssociationAbstraction.
 *
 * @author			smeier
 * @copyright		Requirements Engineering Research Group<br>
 *					Department of Informatics<br>
 *					University of Zurich<br>
 * @history			31.10.2005 SM First Version
 * 					18.07.2006 SM Comments, Reformatting
 * @version			$Date: 2010-02-11 20:38:24 +0100 (Thu, 11 Feb 2010) $, $Author: vigan $, $Revision: 5115 $
 */
public class FisheyeZoom extends SavableCommand {

    /**
     * Target on which zooming is performed.
     */
	transient private ZoomableNodeContainer zoomTarget;
	
	/**
	 * The direction of the zoom: True if all the node should be zoomed-in,
	 * false otherwise
	 */
	private boolean zoomIn;
	
	/**
	 * Stores a reference to the corresponding {@link AdoraModel}.
	 */
	private AdoraModel adoraModel;

	/**
	 * Creates the zoom command for a given node container.
	 */
	public FisheyeZoom() {
		super("Fisheye Zoom");
	}

	/**
	 * Sets the taget container for the zoom operation.
	 * 
	 * @param zoomTarget != null.
	 */
	public void setTarget(ZoomableNodeContainer zoomTarget) {
		this.zoomTarget = zoomTarget;
		setZoomIn(!zoomTarget.isZoomedIn());
	}
	
	/**
	 * Set the direction of the zoom: True if all the node should be zoomed-in,
	 * false otherwise
	 * 
	 * @param zoomIn the direction of the zoom
	 */
	public void setZoomIn(boolean zoomIn) {
		this.zoomIn = zoomIn;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		saveModel();
		ZoomAlgorithmsManager.getInstance().getZoomAlgorithm().zoom(zoomTarget, zoomIn);
		
		calcAbstractedAssociation();
		
		refreshVariationPointConnections();
		refreshConstraintConnections();
		
		zoomTarget.zoomNotice();
	}

	/**
	 * If a {@link ZoomableNodeContainer} gets zoomed we have to recalculate
	 * the {@link ConstraintConnection}s new.
	 */
	private void refreshConstraintConnections() {
		List<Constraint> visibleConst = new ArrayList<Constraint>();
		for (Constraint con : getAdoraModel().getConstraints()) {
			if (con.isVisible()) {
				visibleConst.add(con);
			}
		}
		// This is now realized in the observer of the product-line decision
		// UpdateConstraintConnections constCreate = new UpdateConstraintConnections(getAdoraModel(), visibleConst);
		// constCreate.execute();
	}

	/**
	 * If a {@link ZoomableNodeContainer} gets zoomed we have to recalculate
	 * the {@link VariationPoint}s new.
	 */
	private void refreshVariationPointConnections() {
		List<VariationPoint> visibleVps = new ArrayList<VariationPoint>();
		for (VariationPoint vp : getAdoraModel().getVariationPoints()) {
			if (vp.isVisible()) {
				visibleVps.add(vp);
			}
		}
		VariationPointConnectionCreate showCmd = new VariationPointConnectionCreate(getAdoraModel(), visibleVps);
		showCmd.execute();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		if(zoomTarget == null) {
			return false;
		}
		if(!zoomIn && !zoomTarget.hasChildren()) {
			return false;
		}
		return true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		ZoomAlgorithmsManager.getInstance().getZoomAlgorithm().zoom(zoomTarget, !zoomIn);
		calcAbstractedAssociation();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		execute();
	}
	
    /**
     * Calculates and inserts the abstracted associations or the concrete associations
     * that are again visible.
     */
	public void calcAbstractedAssociation(){
		ConnectionAbstractionAlgorithm.getInstance().run(zoomTarget);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Fisheye Zoom: "+ zoomTarget.toString();
	}
	
	/**
	 * Returns the {@link AdoraModel}.
	 * 
	 * @return The corresponding {@link AdoraModel}.
	 */
	private AdoraModel getAdoraModel() {
		if (adoraModel == null) {

			IContainer parent = zoomTarget.getParent();
			
			while (parent.getParent() != null) {
				parent = parent.getParent();
			}
			adoraModel = (AdoraModel) parent;
		}
		return adoraModel;
	}
}