package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;

/**
 * This class represents the command which executes the visual insertion of 
 * a given node in the parent node. There are two kinds of modes:
 * <ul>
 * 	<li>The insertion is done in a centered way. This is done by 
 *     shifting the node width / 2 to left and height / 2 to top.
 *     This mode is used if a node is newly created, and causes the 
 *     node to appear so that its center is at the position of the
 *     mouse cursor. </li>
 *  <li>The insertion is done with the location given by the node.
 *     This mode is used if an existing object is for example moved
 *     by a drag 'n' drop mechanism.</li>
 * </ul>
 * 
 * The visual insertion is done by shifting surrounding siblings away. Parent
 * and other ancestors in the decomposition hierarchy are justified in size.
 * This is done by the help of the layout algorithm (see 
 * {@link ch.unizh.ifi.adora.layout.algorithm.LayoutAlgorithm}).
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Dec 2, 2005 SM First Version
 *					Apr 26,2006 JS fixed undo() so redoing won't move the Node (#566)
 * 					Aug 2, 2006 JS refactoring: this is now a subclass of 
 * 								   MoveNodeVisually instead of Command
 * 					Aug 3, 2006 JS Comments
 * @version			$Date: 2007-10-31 10:46:16 +0100 (Wed, 31 Oct 2007) $, $Author: schoeni $, $Revision: 3218 $
 */
public class InsertNodeVisually extends MoveNodeVisually {
	/**
	 * Holds the node which will be inserted.
	 */
	private Node nodeToInsert;
	
	/**
	 * An intermediate store for the given size of the node
	 * which is used for the undo/redo mechanism.
	 */
	private Dimension size;

	/**
	 * An intermediate store for the given location of the node
	 * which is used for the undo/redo mechanism.
	 */
	private Point location;
	
	/**
	 * Indicates if the command is calculating visual insertion by doing first
	 * a center shift of the object or not.
	 */
	private boolean calcCentered;
	
	/**
	 * Intializes the visual insertion for the given 
	 * node <i>nc<i>. The node will be centered by default
	 * (see class comment).
	 * @param nc The node which will be visually inserted.
	 */
	public InsertNodeVisually(Node nc) {
		super("Visually Insert Node");
		this.nodeToInsert = nc;
		
		calcCentered = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_INSERT_CENTERED);
		
	}
	
	/**
	 * Intializes the visual insertion for the given 
	 * node <i>nc<i>. See the class comment for an
	 * explanation of the two centered modes.
	 * 
	 * @param nc The node to be visually inserted.
	 * @param centered Indicates whether the node
	 *         should be inserted centeredly. Should be
	 *         true for new elements, and false when the
	 *         node has been moved by drag'n'drop.
	 */	
	public InsertNodeVisually(Node nc, boolean centered) {
		this(nc);
		this.calcCentered = centered; 
	}

	/**
	 * Executes this command by adding the given child node to the new parent
	 * node.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
	    saveModel();
		
	    // if we are in centered mode (i.e. a node which was newly created is 
		// added to the model) then center the given node first
	    if (calcCentered && nodeToInsert.getSize().equals(nodeToInsert.getStoredSize())) {
			nodeToInsert.setLocation(nodeToInsert.getLocation().x - nodeToInsert.getBounds().width / 2,
					nodeToInsert.getLocation().y - nodeToInsert.getBounds().height / 2);
		}
		
	    insert(nodeToInsert);
	}

	/**
	 * Redoes the add command by inserting visually the given node.
	 * 
	 * @pre true
	 * @post true
	 */
	public void redo() {
		nodeToInsert.setSize(size);
		nodeToInsert.setLocation(location);
		insert(nodeToInsert);		
	}

	/**
	 * Executes the undo of this command by visually relayouting the 
	 * parent.
	 * 
	 * @pre true
	 * @post true
	 */
	public void undo() {
		size = nodeToInsert.getSize();
		location = nodeToInsert.getLocation();
		
		remove(nodeToInsert);
	}
	
	@Override
	public String toString() {
		return "Insert node " + nodeToInsert + " visually " + nodeToInsert.getLocation();
	}
}