package ch.unizh.ifi.adora.model.commands;

import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;

/**
 * Command for inserting a group of nodes into a parent. the nodes will be
 * positioned (see the subclasses for the location).
 * 
 * @author Pascal Schoeni
 */
abstract public class InsertNodesVisually extends Command {

	/**
     * The parent node
     */
	private ZoomableNodeContainer parent;

	/**
     * the nodes to insert
     */
	private List<Node> nodes;

	/**
     * the minimum x coord of the nodes in the nodelist
     */
	private int minX = Integer.MAX_VALUE;

	/**
     * the minimum y coord of the nodes in the nodelist
     */
	private int minY = Integer.MAX_VALUE;

	/**
     * the maximum x coord of the nodes in the nodelist
     */
	private int maxX = 0;

	/**
     * the maximum y coord of the nodes in the nodelist
     */
	private int maxY = 0;

	public InsertNodesVisually(ZoomableNodeContainer parent, List<Node> nodes) {
		this.parent = parent;
		this.nodes = nodes;

		calculateSpaceUsed();
	}

	/**
     * Method, which calculates the position where the area of the inserted
     * nodes begins.
     * 
     * @return
     */
	abstract protected Point getLocation();

	@Override
	public void execute() {
		Point location = getLocation();
		
		Rectangle requestedSpace = new Rectangle(location, getDimension());
		AllocateWhiteSpace allocateWhiteSpace = new AllocateWhiteSpace(parent,
				requestedSpace);
	 	allocateWhiteSpace.execute();
		
		for (Node node : nodes) {
			
			Rectangle rectangle = new Rectangle(adaptLocation(
					allocateWhiteSpace.getLocation(), node.getLocation()), node.getSize());

			if (node instanceof Component) {
				rectangle = new Rectangle(0,0,node.getDefaultSize().width,node.getDefaultSize().height);
			}
		
			NodeCreate nodeCreateCommand = new NodeCreate(node, parent,
					rectangle);
			nodeCreateCommand.execute();

			InsertNodeVisually visuallyCommand = new InsertNodeVisually(node,
					false);
			visuallyCommand.execute();
			
		}
	}

	/**
     * Returns the dimension
     * 
     * @return
     */
	protected Dimension getDimension() {
		return new Dimension(maxX - minX, maxY - minY);
	}

	/**
     * Adapts the location from the point "point" to the new position, starting
     * with "sourcePoint"
     * 
     * @param sourcePoint
     * @param point
     * @return
     */
	public Point adaptLocation(Point sourcePoint, Point point) {
		return new Point(2 + (int) sourcePoint.getX() + (int) point.getX()
				- minX, 2 + (int) sourcePoint.getY() + (int) point.getY()
				- minY);
	}

	/**
     * calculates min and max x/y coords for a group of nodes
     * 
     * @param nodes
     */
	private void calculateSpaceUsed() {
		for (Node node : nodes) {
			if (minX > node.getBounds().x) {
				minX = node.getBounds().x;
			}
			if (maxX < node.getBounds().x + node.getBounds().width) {
				maxX = node.getBounds().x + node.getBounds().width;
			}
			if (minY > node.getBounds().y) {
				minY = node.getBounds().y;
			}
			if (maxY < node.getBounds().y + node.getBounds().height) {
				maxY = node.getBounds().y + node.getBounds().height;
			}
		}
	}

	/**
     * @return the parent
     */
	public ZoomableNodeContainer getParent() {
		return parent;
	}
	
	/**
	 * @return The {@link Node}s which get inserted
	 */
	public List<Node> getNodesToInsert() {
		return nodes;
	}
}