package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * A command to create a new Node. The command can be undone or
 * redone.
 *
 * @author				smeier
 * @author				gygax
 * @copyright			Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history			30.08.2006 AG First Version
 * 						18.07.2006 SM Comments, Reformatting.
 * 						08.08.2006 JS Comments
 * 						11.08.2006 JS Replaced local constants with node.getDefaultSize()
 * @version			$Date: 2010-02-13 21:20:24 +0100 (Sat, 13 Feb 2010) $, $Author: vigan $, $Revision: 5134 $
 */
public class NodeCreate extends Command {

	/** 
	 * The new shape. 
	 */
	private final Node newNode;

	/** 
	 * node to add to. 
	 */
	private final INodeContainer parent;

	/** 
	 * The bounds of the new Shape. 
	 */
	private final Rectangle bounds;

	/**
	 * Initializes a new node create command.
	 * The shape which was created (<i>newNode</i>)
	 * will be inserted into an existing parent 
	 * with the given bounds (location and dimension).
	 * 
	 * @pre       	newNode != null &&
	 *				parent   != null &&
	 *				bounds   != null
	 * @post      true
	 * @param newNode 	The newly created Node
	 * @param parent  	The parent where to insert the 
	 *					newly created node.
	 * @param bounds	The bounds, i.e. the location
	 *					and the dimension of the 
	 *					newNode.
	 */
    public NodeCreate(Node newNode, INodeContainer parent, Rectangle bounds) {
		super("Create Node");
		this.newNode = newNode;
		this.parent = parent;
		this.bounds = bounds;
	}

	/**
	 * Can execute if all the necessary information has been provided.
	 * 
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return newNode != null && parent != null && bounds != null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		newNode.setLocation(bounds.getLocation());
		Dimension size = bounds.getSize();
		if (size.width > 0 && size.height > 0) {
			newNode.setSize(size);
		} else {
			//Default size
			newNode.setSize(newNode.getDefaultSize());
		}
		redo();
		
		if (newNode instanceof AspectContainer) {
			setACvariability((AspectContainer)newNode);
		}
	}
	
	/**
	 * Method used to set the variability of a newly created {@link AspectContainer}.
	 * 
	 * @param ac The newly created {@link AspectContainer} for which the variability gets set.
	 */
	private void setACvariability(AspectContainer ac) {
		INodeContainer parent = ac.getParent();
		while  (!(parent instanceof AdoraModel)) {
			if (parent instanceof AspectContainer) {
				ac.setVariability(((AspectContainer)parent).isVariability());
				break; //only update the first parent
			}			
			parent = ((Node)parent).getParent();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		parent.addChild(newNode);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {

		parent.removeChild(newNode);		
	}

}