package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.variability.Decision;

/**
 * Creates a NodeDelete command.
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			30.08.2005 SM First Version
 * 						18.07.2006 SM Comments, Reformatting
 * 						08.08.2006 JS Comments
 * 						08.09.2006 JS 760: Connections attached to connections of 
 * 									  the deleted node are disconnected as well
 * @version			$Date: 2010-01-23 04:28:07 +0100 (Sat, 23 Jan 2010) $, $Author: vigan $, $Revision: 5093 $
 */
public class NodeDelete extends Command {
	
	/** 
	 * Node to remove. 
	 */
	private final Node child;

	/** 
	 * Node to remove from. 
	 */
	private final INodeContainer parent;
	
	/**
	 * Connections to be removed.
	 */
	private List<Connection> connections;

	/** 
     * True, if child was removed from its parent. 
     * Due to safety reasons. 
     */
	private boolean wasRemoved;

	/**
	 * Create a command that will remove the shape from its parent.
	 * 
	 * @param parent
	 *            the Container containing the child
	 * @param child
	 *            the Node to remove
	 * @throws IllegalArgumentException
	 *             if any parameter is null
	 */
	public NodeDelete(INodeContainer parent, Node child) {
		super("Delete Node");
		if (parent == null || child == null) {
			throw new IllegalArgumentException();
		}
		this.parent = parent;
		this.child = child;
	}

	/**
	 * Reconnects a List of Connections with their previous endpoints.
	 * 
	 * @param connections
	 *            a non-null List of connections
	 */
	private void addConnections(List connections) {
		for (Iterator iter = connections.iterator(); iter.hasNext();) {
			Connection conn = (Connection) iter.next();
			conn.reconnect();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#canUndo()
	 */
	public boolean canUndo() {
        return wasRemoved;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		
		
		// store a copy of incoming & outgoing connections before proceeding
		connections = new ArrayList<Connection>();
		getAllConnections(connections, child);
		redo();
	}

	/**
     * Fills list with all connections of a given node.
     * 
     * @param list List for receiving the connections.
     * @param node The node for which all the connections should be searched.
     */
	private void getAllConnections(List<Connection> list, Node node) {
		list.addAll(node.getSourceConnections());
		list.addAll(node.getTargetConnections());
		// add connections attached to attached connections
		Iterator<Connection> connections = node.getConnections().iterator();
		while(connections.hasNext()) {
			Connection currentConnection = connections.next();
			list.addAll(currentConnection.getConnections());
		}
		// add connections attached to child nodes
		if(node instanceof INodeContainer) {
			Iterator children = ((INodeContainer)node).getChildrenIterator();
			while(children.hasNext()) {
				Node nc = (Node)children.next();
				getAllConnections(list, nc);
			}
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		// removes the child and disconnects its connections
		wasRemoved = parent.removeChild(child);
		if (wasRemoved) {
			child.informRemoved();
			if(child instanceof INodeContainer){
				List children=((INodeContainer)child).getChildren();
				for(int i=0;i<children.size();i++){
					((Node)children.get(i)).informRemoved();
				}
			}
			removeConnections(connections);
		}
	}

	/**
	 * Disconnects a List of Connections from their endpoints.
	 * 
	 * @param connections
	 *            a non-null List of connections
	 */
	private void removeConnections(List connections) {
		for (Iterator iter = connections.iterator(); iter.hasNext();) {
			Connection conn = (Connection) iter.next();
			conn.disconnect();
			
			//Delete the Decision associated with a JoinRelation if the Decision only belongs to this JR
 			if (conn instanceof JoinRelation && ((JoinRelation)conn).getDecision() != null) {
 				
 				if (((JoinRelation)conn).getDecision().getJoinRelations().size() <= 1) {
 					
 					//keep decision at its place for dynamic weaving
 					Decision dec = ((JoinRelation)conn).getDecision(); 
 					if (!dec.isWeaved())
 						getAdoraModel().removeDecision(dec);
 				} else {
 					((JoinRelation)conn).getDecision().removeJoinRelation((JoinRelation)conn);
 				}
 			} 
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		// add the child and reconnect its connections
		if (parent.addChild(child)) {
			addConnections(connections);
		}
	}
	
	private AdoraModel adoraModel;

	private AdoraModel getAdoraModel() {
		if (adoraModel == null) {
			//IContainer parent = ((JoinRelation)(getJoinRelations().toArray()[0])).getSource().getParent();
			
			//---debug
			IContainer tmpParent = parent;
			//---

			while (tmpParent.getParent() != null) {
				tmpParent = tmpParent.getParent();
			}
			adoraModel = (AdoraModel) tmpParent;
		}

		return adoraModel;
	}
	
	
	@Override
	public String toString() {
		return "Delete " + child + " (" + child.getClass().getSimpleName() + " / Parent: " + child.getParent() + ")";
	}
}