package ch.unizh.ifi.adora.model.commands;

import java.util.Iterator;

import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;

/**
 * Command to perform a fisheye zoom on a node and all its direct and indirect
 * children.
 * 
 * @author Tobias Reinhard
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2007-10-30 TR First Version
 * @version $Date: 2008-03-18 15:34:31 +0100 (Tue, 18 Mar 2008) $, $Author: treinhard $, $Revision: 3575 $
 * 
 */
public class RecursiveFisheyeZoom extends CompoundCommand {

	/**
     * The direction of the zoom: True if all the nodes should be zoomed-in,
     * false otherwise
     */
	private boolean zoomIn;

	
	/**
	 * Constructor
	 * 
	 * @param zoomIn zooms recursively in if this is set to true, recursively
	 * 		  out otherwise.
	 */
	public RecursiveFisheyeZoom(boolean zoomIn) {
		super("Recursive Fisheye Zoom");

		this.zoomIn = zoomIn;
	}

	/**
     * Sets the taget container for the zoom operation.
     * 
     * @param zoomTarget the node that should be zoomed recursively
     */
	public void setTarget(ZoomableNodeContainer zoomTarget) {
		dispose();

		addFisheyeZoomCommand(zoomTarget);

		addChildren(zoomTarget);
	}

	
	/**
     * Add the zoom command.
     * 
     * @param zoomTarget The {@link ModelElement} which will be zoomed.
     */
	private void addFisheyeZoomCommand(ZoomableNodeContainer zoomTarget) {
		FisheyeZoom cmd = new FisheyeZoom();
		cmd.setTarget(zoomTarget);
		cmd.setZoomIn(zoomIn);
		add(cmd);
	}

	
	/**
     * Adds recursively a FisheyeZoom command for all the direct and indirect
     * children of the node <i>node</i>.
     * 
     * @param node the node for whose children the commands should be added
     */
	private void addChildren(ZoomableNodeContainer node) {
		Iterator<ModelElement> iterator = node.getRecursiveChildrenIterator();

		while (iterator.hasNext()) {
			ModelElement me = iterator.next();

			if (me instanceof ZoomableNodeContainer) {
				ZoomableNodeContainer zoomTarget = (ZoomableNodeContainer) me;

				if (zoomTarget.getChildCount() > 0 && zoomTarget.isZoomedIn() != zoomIn) {
					addFisheyeZoomCommand(zoomTarget);
				}
			}
		}
	}
	
	
	public void execute() {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		boolean animation = store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM);
		
		// Disable the animation of the zoom
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM, false);
		
		super.execute();
		
		// Reset to the previous value
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM, animation);
		
	}
	
	
	public void undo() {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		boolean animation = store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM);
		
		// Disable the animation of the zoom
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM, false);
		
		super.undo();
		
		// Reset to the previous value
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM, animation);
		
	}

}
