package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.Animation;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNode;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.utils.ZoomAlgorithmsManager;

/**
 * This command is used by the model search.
 * It sets the visibility of all the model elements 
 * not contained in the search result to false.
 * 
 * @author Michael Jehle
 * @date 1.2.2009
 */
public class ReduceModelVisibility extends SavableCommand
{
	/**
	 * The elements which match the search.
	 */
	private List<VisualizableModelElement> searchResult;
	
	/**
	 * The last search result. Used for the result table when
	 * undoing a command. 
	 */
	private List<VisualizableModelElement> searchResultBeforeCommand;
	
	/**
	 * The elements which must remain visible to show the search result.
	 * Includes parents of elements which match the search.
	 */
	private List<VisualizableModelElement> remainVisible;
	
	/**
	 * The AdoraModel the command is executed on.
	 */
	private AdoraModel model;
	
	/**
	 * These elements' visibility has been switched to visible by the execution of the command.
	 */
	private List<ModelElement> switchedElementsToVisible = new ArrayList<ModelElement>();

	/**
	 * These elements' visibility has been switched to invisible by the execution of this command. 
	 */
	private List<ModelElement> switchedElementsToInvisible = new ArrayList<ModelElement>();
	
	/**
	 * These connections have been removed by the execution of the command.
	 */
	private List<ConnectionDelete> removedConnections = new ArrayList<ConnectionDelete>();
	
	/**
	 * These elements have been zoomed in by the execution of the command.
	 */
	private List<ZoomableNode> zoomedInElements = new ArrayList<ZoomableNode>();
	
	/**
	 * This is the name of the tab, where the model
	 * of the active search is displayed.
	 * This information is needed to undo the search
	 * when the tab is closed.
	 */
	private String partTitle;
	
	/**
	 * Default constructor.
	 */
	public ReduceModelVisibility()
	{
		super("Reduce Model Visibility");
	}
	
	/**
	 * Setter for the search result. Needed to execute the command.
	 * @param result List<VisualizableModelElement>
	 */
	public void setSearchResult(List<VisualizableModelElement> result)
	{
		this.searchResult = result;
		this.calculateRemainingModelParts();
	}
	
	/**
	 * Returns the search result of this command.
	 * @return List<VisualizableModelElement>
	 */
	public List<VisualizableModelElement> getSearchResult()
	{
		return this.searchResult;
	}
	
	/**
	 * Setter for the second last search result.
	 * @param elements List<VisualizableModelElement>
	 */
	public void setSearchResultBeforeCommand(List<VisualizableModelElement> elements)
	{
		this.searchResultBeforeCommand = elements;
	}
	
	/**
	 * Getter for the second last search result.
	 * @return List<VisualizableModelElment>
	 */
	public List<VisualizableModelElement> getSearchResultBeforeCommand()
	{
		return this.searchResultBeforeCommand;
	}
	
	/**
	 * Setter for the AdoraModel the command is executed on.
	 * @param model AdoraModel
	 */
	public void setModel(AdoraModel model)
	{
		this.model = model;
	}
	
	/**
	 * Setter for the part title (title of the tab), where the model
	 * this command is executed on is shown. 
	 * @param partTitle String
	 */
	public void setPartTitle(String partTitle)
	{
		this.partTitle = partTitle;
	}
	
	/**
	 * Getter for the part title (title of the tab), where the model
	 * the command is executed on is shown.
	 * @return String
	 */
	public String getPartTitle()
	{
		return this.partTitle;
	}
	
	/**
	 * Calculates, which parts of the model should remain visible.
	 */
	private void calculateRemainingModelParts()
	{
		this.remainVisible = new ArrayList<VisualizableModelElement>();
		
		for (int r = 0; r < this.searchResult.size(); r++)
		{
			this.remainVisible.add(this.searchResult.get(r));
			
			IContainer parent = this.searchResult.get(r).getParent();
			
			while (!(parent instanceof AdoraModel) &&
					(parent instanceof VisualizableModelElement))
			{
				if (!this.remainVisible.contains(parent))
				{
					this.remainVisible.add((VisualizableModelElement) parent);
				}
				
				parent = parent.getParent();
			}
		}
	}
	
	/**
	 * Removes the connections, which should no longer be visible after the comands'
	 * execution by using the ConnectionDelete command.
	 */
	private void removeConnecitons()
	{
		Iterator<ModelElement> iterator = this.model.getRecursiveChildrenIterator();
		
		while (iterator.hasNext())
		{
			ModelElement temp = iterator.next();
			
			if (temp instanceof VisualizableModelElement)
			{
    			VisualizableModelElement vme = (VisualizableModelElement) temp;
    			
    			if (vme instanceof ConnectableElement)
    			{
    				ConnectableElement ce = (ConnectableElement) vme;
    				
    				if (ce.getConnections().size() > 0)
    				{
    					List<Connection> connections = ce.getConnections();
    					
    					for (int c = 0; c < connections.size(); c++)
    					{
    						Connection con = connections.get(c);
    						
    						if (con.getTarget() instanceof Connection)
    						{
    							Connection target = (Connection) con.getTarget();
    							
    							if (!this.remainVisible.contains(target.getSource()) ||
    								!this.remainVisible.contains(target.getTarget()))
    							{
    								if (con.isVisible())
    								{
    									ConnectionDelete cd = new ConnectionDelete(con);
    									cd.execute();
    									this.removedConnections.add(cd);
    								}
    							}
    						}
    						else
    						{
    							if (!this.remainVisible.contains(con.getSource()) ||
    								!this.remainVisible.contains(con.getTarget()))
    							{
    								if (con.isVisible())
    								{
    									ConnectionDelete cd = new ConnectionDelete(con);
    									cd.execute();
    									this.removedConnections.add(cd);
    								}
    							}
    						}
    					}
    				}
    			}
			}
		}
	}
	
	/**
	 * Executes the command. Reduces the visibility of the model components
	 * according to the search result.
	 */
	public void execute()
	{
		Animation.markBegin();
		
		this.removeConnecitons();
		
		Iterator<ModelElement> iterator = this.model.getRecursiveChildrenIterator();
		
		while (iterator.hasNext())
		{
			ModelElement temp = iterator.next();
		
			if (remainVisible.contains(temp))
			{
				this.filterNode(temp, true);
			}
			else
			{
				this.filterNode(temp, false);
			}
		}
		
		Animation.run();
	}
	
	public void undo()
	{
		Animation.markBegin();

		// undo the visibility changes on the model elements
		for (ModelElement modelElement : this.switchedElementsToVisible)
		{
			if (modelElement instanceof IZoomNode)
			{
				filterNode(modelElement, false);
			}
		}
		
		for (ModelElement modelElement : this.switchedElementsToInvisible)
		{
			if (modelElement instanceof IZoomNode)
			{
				filterNode(modelElement, true);
			}
		}
		
		// zoom out the nodes which have been zoomed in before
		for (ZoomableNode zn : this.zoomedInElements)
		{
			zn.setZoomedIn(false);
		}

		// reinsert connections
		for (ConnectionDelete cd : this.removedConnections)
		{
			cd.undo();
			
			if (!cd.getConnection().isVisible())
			{
				cd.getConnection().setVisible(true);
			}
		}

		Animation.run();
	}

	private void filterNode(ModelElement element, boolean visibility)
	{
		if (element instanceof VisualizableModelElement)
		{
			if (visibility != this.isElementVisible((VisualizableModelElement) element))
			{
				// Zoom only when an element needs to be made visible.
				if (visibility)
				{
    				// first check if the parent of the element is zoomed in
    				IContainer parent = element.getParent();
    				
    				while (!(parent instanceof AdoraModel))
    				{
    					if (!((ZoomableNode) parent).isZoomedIn())
    					{
    						((ZoomableNode) parent).setZoomedIn(visibility);
    						this.zoomedInElements.add((ZoomableNode) parent);
    						
    						//zooming into a node makes all its children visible
    						//therefore, its checked, if some of them have to be filtered out again.
    						Iterator<ModelElement> chIt = parent.getChildrenIterator();
    						
    						while (chIt.hasNext())
    						{
    							ModelElement temp = chIt.next();
    							
    							if (!this.remainVisible.contains(temp))
    							{
    								this.filterNode(temp, false);
    							}
    						}
    					}
    					
    					parent = parent.getParent();
    				}
				}
				
				// only if the elements visibility is not right yet
				if (visibility != isElementVisible((VisualizableModelElement) element))
				{
					ZoomAlgorithmsManager.getInstance().getFilterAlgorithm().filterNode((IZoomNode) element);
					
    				if (element instanceof AspectContainer)
    				{
    					Node node = (Node)element;
    					node.setManuallyHidden(node.isVisible());
    				}
    				
					if (visibility)
					{
						this.switchedElementsToVisible.add(element);
					}
					else
					{
						this.switchedElementsToInvisible.add(element);
					}
				}
			}
		}		
	}
	
	public boolean isElementVisible(VisualizableModelElement element)
	{
		if (element.getSize().equals(new Dimension(1, 1)))
		{
			return false;
		}
		else
		{
			return element.isVisible();
		}
	}
	
	public String toString()
	{
		StringBuffer result = new StringBuffer();
		
		result.append("Search result: ");
		
		for (int i = 0; i < this.searchResult.size(); i++)
		{
			result.append("\n");
			result.append(this.searchResult.get(i).getName());
		}
		
		return result.toString();
	}
}