package ch.unizh.ifi.adora.model.commands;

import org.eclipse.draw2d.Animation;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;
import ch.unizh.ifi.adora.zoom.berner.algorithm.LayoutAlgorithm;

/**
 * This {@link org.eclipse.gef.commands.Command}, adjusts the size of a
 * {@link ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer} if its
 * {@link org.eclipse.draw2d.Border} changed.
 * 
 * This means that it has to update the size store in the model and to relayout
 * all the children of the
 * {@link ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer}.
 * 
 * Basically, this {@link org.eclipse.gef.commands.Command} encapsulates the the
 * call of the {@link LayoutAlgorithm#adjustNode(ZoomableNodeContainer)}.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 11.05.2006 IV First Version 08.08.2006 JS Comments
 * @version $Date: 2008-12-02 18:06:49 +0100 (Tue, 02 Dec 2008) $, $Author: treinhard $, $Revision: 3932 $
 */
public class SetBorderDimension extends SavableCommand {

	/**
     * Stores the
     * {@link ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer} on
     * which this {@link Command} is applied.
     */
	private ZoomableNodeContainer node;

	/**
     * Stores the new border dimension
     */
	private Dimension newBorderDimension;

	/**
     * Stores the new insets
     */
	private java.awt.Insets newInsets;

	/**
     * Constructs a new AdjustNode command.
     * 
     * @param node
     *            {@link ZoomableNodeContainer} on which this {@link Command} is
     *            applied.
     * @param border
     *            The border that has changed.
     */
	public SetBorderDimension(ZoomableNodeContainer node, Insets border) {
		super("Adjust Node");
		this.node = node;

		this.newBorderDimension = new Dimension(border.right + border.left,
				border.bottom + border.top);
		this.newInsets = new java.awt.Insets(border.top, border.left,
				border.bottom, border.right);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
	public void execute() {
		if (this.node.getBorderDimension().equals(this.newBorderDimension)
				&& this.node.getBorderInsets().equals(this.newInsets)) {
			return;
		}

		saveModel();

		this.node.setBorderDimension(this.newBorderDimension);
		this.node.setBorderInsets(this.newInsets);

		AdoraZoomAdapter.getInstance().adjustBorder(node);

	}

	/**
     * This command can not be executed, if it does not change anything.
     * Otherwise it would set the editor "dirty" even nothing has really
     * changed.
     */
	@Override
	public boolean canExecute() {
		
		// Don't adjust the border during intermediate animation steps
		if(Animation.isAnimating()) {
			return false;
		}
		
		if (this.node.getBorderDimension().equals(this.newBorderDimension)
				&& this.node.getBorderInsets().equals(this.newInsets)) {
			return false;
		}

		return super.canExecute();
	}
}