package ch.unizh.ifi.adora.model.commands;

import java.lang.reflect.Method;

import org.eclipse.gef.commands.Command;


/**
 * This command is used to set an attribute of a model object.
 * The setting has to be realised in a command because only this way undo is 
 * available.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 20.09.2006 IV First Version
 *          16.04.2008 CJ Added Label
 * @version $Date: 2009-03-19 09:21:29 +0100 (Thu, 19 Mar 2009) $, $Author: anil $, $Revision: 4075 $
 */
public class SetProperty extends Command {

	/**
     * Stores the Model Element for which a new attribute value is set.
     */
	private Object modelElement;

	public Object getModelElement() {
		return modelElement;
	}


	public Method getSetMethod() {
		return setMethod;
	}


	public Object getNewValue() {
		return newValue;
	}


	public Object getOldValue() {
		return oldValue;
	}

	/**
     * Stores the {@link Method} which is used to set the new value of a model
     * element.
     */
	private Method setMethod;

	/**
     * Stores the new value which is set.
     */
	private Object newValue;

	/**
     * Stores the old value - the value how it was before applying the changes..
     */
	private Object oldValue;

	/**
     * Instantiates a new Command.
     * 
     * @param modelElement
     *            The Model Element for which a new attribute value
     *            is set.
     * @param newValue
     *            The new value which is set.
     * @param setMethod
     *            The {@link Method} which is used to set a new value of a model
     *            element attribute.
     * @param getMethod
     *            The {@link Method} which is used to obtain the old value of a
     *            model element attribute.
     */
	public SetProperty(Object modelElement, Object newValue,
			Method setMethod, Method getMethod) {
		super ("Set Property");
		this.modelElement = modelElement;
		this.setMethod = setMethod;
		this.newValue = newValue;
		try {
			oldValue = getMethod.invoke(modelElement);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	/**
     * Instantiates a new Command.
     * 
     * @param modelElement
     *            The Model Element for which a new attribute value
     *            is set.
     * @param newValue
     *            The new value which is set.
     * @param setMethod
     *            The {@link Method} which is used to set a new value of a model
     *            element attribute.
     * @param getMethod
     *            The {@link Method} which is used to obtain the old value of a
     *            model element attribute.
     * @param propertyName
     * 			  The name of the property being modified (for the label)
     */
	public SetProperty(Object modelElement, Object newValue,
			Method setMethod, Method getMethod, String propertyName) {
		this(modelElement,newValue,setMethod,getMethod);
		setLabel("Modify " + propertyName);
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		try {
			setMethod.invoke(modelElement, newValue);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		try {
			setMethod.invoke(modelElement, oldValue);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		execute();
	}
}