package ch.unizh.ifi.adora.model.commands.representation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;

/**
 * This {@link org.eclipse.gef.commands.Command} is used to show/ hide 
 * single {@link ch.unizh.ifi.adora.model.representation.Node}s
 * by right-clicking on the {@link Node} in the 
 * {@link ch.unizh.ifi.adora.parts.tree.NodeTreeEditPart} and choosing the 
 * "invert view" menu item.
 * 
 * If the hidden {@link Node} is connected by {@link ch.unizh.ifi.adora.model.representation.Association}s, 
 * they will be abstracted and appended at the hidden {@link Node}'s parent. 
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			31.03.2006 IV First Version
 *						06.06.2006 JS RepresentationNodeShrink and 
 *									  RepresentationNodeBloat is used 
 *									  in execute now
 *						19.7.2006 SM Comments reformatted, checked.
 * @version			$Date: 2006-08-29 15:18:39 +0200 (Tue, 29 Aug 2006) $, $Author: vigan $, $Revision: 2120 $
 */
public class RepresentationStateChangeTree extends Command {

	/**
	 * {@link Node} which will be filtered out visually from the {@link AdoraModel}. 
	 */
	private Node node;

	/**
	 * Creates a new ViewCommand for a given type.
	 * @param type Node type whose visibility will be altered. 
	 */
	public RepresentationStateChangeTree(Node node) {
		this.node = node;
		setLabel("Changes the Representation State via tree");
	}

	/**
	 * Executes the filter operation on the model.
	 */
	public void execute() {
		
		List<ZoomableNodeContainer> parents = new ArrayList<ZoomableNodeContainer>();
		
		Command cmd;
		if (!node.isVisible()) {
			node.setManuallyHidden(node.isVisible());
			cmd = new RepresentationNodeBloat(node);
			cmd.execute();
		} else {
			cmd = new RepresentationNodeShrink(node);
			cmd.execute();
			node.setManuallyHidden(node.isVisible());
		}

		//calculates the abstracted associations
		if (node.getParent() instanceof ZoomableNodeContainer){
			ConnectionAbstractionAlgorithm.getInstance().
				run((ZoomableNodeContainer) node.getParent());
		} else if (node instanceof ZoomableNodeContainer){
			ConnectionAbstractionAlgorithm.getInstance().
			run((ZoomableNodeContainer) node);
		} else if(node instanceof Actor){
			Iterator iter = ((Actor)node).getAssociationsIterator();
			Association asso;
			Node n;
			while (iter.hasNext()) {
				asso = (Association)iter.next();
				n = asso.getSource() == node ? asso.getTarget() : asso.getSource();
				if (n instanceof ZoomableNodeContainer){
					parents.add((ZoomableNodeContainer) n);
				} else if (!(n.getFirstVisibleParentContainer() instanceof AdoraModel)) {
					parents.add((ZoomableNodeContainer)n.getFirstVisibleParentContainer());
				}
			}
		}
		
		for (ZoomableNodeContainer container : parents) {
			ConnectionAbstractionAlgorithm.getInstance().run(container);
		}
	}

	/**
	 * Since a view filter can easily be undone by a single buttons click, 
	 * undoing this command isn't allowed.
	 * @return false
	 */
	public boolean canUndo() {
		return false;
	}

	
}