package ch.unizh.ifi.adora.model.syntaxerrors;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Observable;
import java.util.Set;

import ch.unizh.ifi.adora.model.ModelElement;


/** 
 * This class collects and sorts {@link SyntaxProblem}.
 * Problems are sorted given their rule and their model element.
 * 
 * TODO: For now, it is limited to a single problem per
 * syntax rule and model element.
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ First Version
 * @version   $Date: 2008-04-24 18:22:06 +0200 (Thu, 24 Apr 2008) $, $Author: jeanneret $, $Revision: 3619 $
 *
 */
public class SyntaxProblemsSorter extends Observable {

	private Map<ModelElement, Map<String,SyntaxProblem>> elements2entries;
	private Map<String, Map<ModelElement,SyntaxProblem>> rules2entries;
	
	/**
	 * Constructor for this class
	 */
	SyntaxProblemsSorter() {
		elements2entries = new HashMap<ModelElement, Map<String,SyntaxProblem>>();
		rules2entries = new HashMap<String, Map<ModelElement,SyntaxProblem>>();
	}
	
	/**
	 * @return the set of invalid model elements
	 */
	public Set<ModelElement> getInvalidElements()
	{
		return elements2entries.keySet();
	}
	
	/**
	 * @return a set of violated syntax rules
	 */
	public Set<String> getViolatedRules()
	{
		return rules2entries.keySet();
	}
	
	/**
	 * Get the set of model elements invalid according
	 * to a given rule.
	 * @param ruleID
	 * @return
	 */
	public Set<ModelElement> getInvalidElements(String ruleID)
	{
		Map<ModelElement,SyntaxProblem> internalMap = rules2entries.get(ruleID);
		if(internalMap == null || internalMap.isEmpty())
		{
			return Collections.emptySet();
		}
		else return internalMap.keySet();
	}
	
	/**
	 * Get the set of violated rules by a model element.
	 * @param element
	 * @return
	 */
	public Set<String> getViolatedRules(ModelElement element)
	{
		Map<String,SyntaxProblem> internalMap = elements2entries.get(element);
		if(internalMap == null || internalMap.isEmpty())
		{
			return Collections.emptySet();
		}
		else return internalMap.keySet();
	}
	
	/**
	 * Get the problem of a model element (according to the given rule)
	 * @param ruleID
	 * @param element
	 * @return
	 */
	public SyntaxProblem getEntry(String ruleID, ModelElement element)
	{
		Map<ModelElement,SyntaxProblem> internalMap = rules2entries.get(ruleID);
		if(internalMap == null || internalMap.isEmpty())
		{
			return null;
		}
		else return internalMap.get(element);
	}
	
	/**
	 * Get the problems related to a syntax rule
	 * @param ruleID
	 * @return
	 */
	public Collection<SyntaxProblem> getEntries(String ruleID)
	{
		Map<ModelElement,SyntaxProblem> internalMap = rules2entries.get(ruleID);
		if(internalMap == null || internalMap.isEmpty())
		{
			return Collections.emptyList();
		}
		return internalMap.values();
	}
	
	/**
	 * Get the problems of a model element
	 * @param element
	 * @return
	 */
	public Collection<SyntaxProblem> getEntries(ModelElement element)
	{
		Map<String,SyntaxProblem> internalMap = elements2entries.get(element);
		if(internalMap == null || internalMap.isEmpty())
		{
			return Collections.emptyList();
		}
		return internalMap.values();
	}
	
	/**
	 * Get all the problems stored in this problems sorter
	 * @return
	 */
	public Collection<SyntaxProblem> getEntries()
	{
		Collection<SyntaxProblem> entries = new LinkedList<SyntaxProblem>();
		for (String ruleID : getViolatedRules())
		{
			Map<ModelElement,SyntaxProblem> internalMap = rules2entries.get(ruleID);
			if(internalMap != null)
			{
				entries.addAll(internalMap.values());
			}
		}
		return entries;
	}
	
	/**
	 * Add an entry to this sorter
	 * @param entry a non null entry, with all fields non null
	 */
	public void addEntry(SyntaxProblem entry)
	{
		Map<ModelElement,SyntaxProblem> internalElement2Entry = rules2entries.get(entry.getSyntaxRuleName());
		if(internalElement2Entry == null)
		{
			internalElement2Entry = new HashMap<ModelElement, SyntaxProblem>();
			rules2entries.put(entry.getSyntaxRuleName(), internalElement2Entry);
		}
		internalElement2Entry.put(entry.getModelElement(), entry);
		
		
		Map<String,SyntaxProblem> internalRule2Entry  = elements2entries.get(entry.getModelElement());
		if(internalRule2Entry == null)
		{
			internalRule2Entry = new HashMap<String, SyntaxProblem>();
			elements2entries.put(entry.getModelElement(), internalRule2Entry);
		}
		internalRule2Entry.put(entry.getSyntaxRuleName(), entry);
		
		setChanged();
	}
	
	/**
	 * Remove all the entries related to a model element
	 * @param element
	 */
	public void deleteEntries(ModelElement element)
	{
		if(!getViolatedRules(element).isEmpty())
		{
    		for (String ruleID : getViolatedRules(element))
    		{
    			rules2entries.get(ruleID).remove(element);
    			if(rules2entries.get(ruleID).isEmpty())
    			{
    				rules2entries.remove(ruleID);
    			}
    		}
    		elements2entries.remove(element);
    		setChanged();
		}
	}
	
	/**
	 * Remove all the entries related to a rule
	 * @param ruleID
	 */
	public void deleteEntries(String ruleID)
	{
		if(!getInvalidElements(ruleID).isEmpty())
		{
    		for (ModelElement element : getInvalidElements(ruleID))
    		{
    			elements2entries.get(element).remove(ruleID);
    			if(elements2entries.get(element).isEmpty())
    			{
    				elements2entries.remove(element);
    			}
    		}
    		rules2entries.remove(ruleID);
    		setChanged();
		}
	}
	
	/**
	 * Remove a specific problem
	 * @param entry
	 */
	public void deleteEntry(SyntaxProblem entry)
	{
		if(getEntry(entry.getSyntaxRuleName(), entry.getModelElement())!=null)
		{
    		Map<ModelElement,SyntaxProblem> internalElement2Entry = rules2entries.get(entry.getSyntaxRuleName());
    		internalElement2Entry.remove(entry.getModelElement());
    		if(internalElement2Entry.isEmpty())
    		{
    			rules2entries.remove(entry.getSyntaxRuleName());
    		}
    		
    		Map<String,SyntaxProblem> internalRule2Entry  = elements2entries.get(entry.getModelElement());
    		internalRule2Entry.remove(entry.getSyntaxRuleName());
    		if(internalRule2Entry.isEmpty())
    		{
    			elements2entries.remove(entry.getModelElement());
    		}
    		setChanged();
		}
	}

	/**
	 * Remove the entry related to a rule and an element
	 * @param ruleID
	 * @param el
	 */
	public void clearEntry(String ruleID, ModelElement el)
	{
		SyntaxProblem oldEntry = getEntry(ruleID,el);
		if(oldEntry != null)
		{
			deleteEntry(oldEntry);
			setChanged();
		}
	}
	
	/**
	 * Clear this sorter (remove all the entries)
	 */
	public void clear()
	{
		for(Map<ModelElement,SyntaxProblem> internalMap : rules2entries.values())
		{
			internalMap.clear();
		}
		rules2entries.clear();
		
		for(Map<String,SyntaxProblem> internalMap : elements2entries.values())
		{
			internalMap.clear();
		}
		elements2entries.clear();	
	}
	
}
