package ch.unizh.ifi.adora.model.syntaxerrors;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.properties.ConnectionPropNames;
import ch.unizh.ifi.adora.model.properties.VisualModElemPropNames;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.syntaxerrors.rules.JoinRelationContextMapSyntaxRule;
import ch.unizh.ifi.adora.model.syntaxerrors.rules.ScenarioGuardSyntaxRule;
import ch.unizh.ifi.adora.model.syntaxerrors.rules.ScenarioTransformSyntaxRule;
import ch.unizh.ifi.adora.model.syntaxerrors.rules.SpecificationSyntaxRule;
import ch.unizh.ifi.adora.model.syntaxerrors.rules.TransitionLabelSyntaxRule;

/** 
 * This class can be used to verify the syntax of an {@link AdoraModel}
 * 
 * The verifier can hook itself as a {@link PropertyChangeListener} of
 * {@link ObservableModelElement}.
 * 
 * TODO: Improve listening to model (could be a performance issue)
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ First Version
 * @version   $Date: 2010-02-11 20:38:24 +0100 (Thu, 11 Feb 2010) $, $Author: vigan $, $Revision: 5115 $
 *
 */
public class SyntaxVerifier implements PropertyChangeListener, ISyntaxVerificationContext{
	
	/**
	 * An array of all existing {@link ISyntaxVerificationRule}
	 */
	private final static ISyntaxVerificationRule[] RULES = {
		new SpecificationSyntaxRule(),
		new ScenarioGuardSyntaxRule(),
		new ScenarioTransformSyntaxRule(),
		new TransitionLabelSyntaxRule(),
		new JoinRelationContextMapSyntaxRule()};
	
	
	/**
	 * The {@link ISyntaxVerificationRule} this
	 * verifier should validate on the model
	 * (can be different than RULES)
	 */
	private ISyntaxVerificationRule[] rules;
	
	/**
	 * Keeps track of elements this class has to validate.
	 */
	private Collection<ModelElement> elements;
	
	/**
	 * The {@link SyntaxProblemsSorter} use to gather syntax errors.
	 */
	private SyntaxProblemsSorter errors;
	
	/*
	 * Fields for {@link ISyntaxVerificationContext} implementation
	 * TODO: Create a separate object? One per rule?
	 */
	
	/**
	 * The current target
	 */
	private ModelElement currentTarget;
	
	/**
	 * The change event that triggered the validation
	 */
	private PropertyChangeEvent currentEvent;
	
	/**
	 * The current active rule.
	 */
	private ISyntaxVerificationRule currentRule;
	
	/**
	 * Verify the given model (once)
	 * @param model
	 * @return
	 */
	public static SyntaxProblemsSorter verify(AdoraModel model)
	{
		SyntaxVerifier verifier = new SyntaxVerifier(model);
		verifier.validateElements(null);
		return verifier.errors;
	}

	/**
	 * Create a {@link SyntaxVerifier} to validate
	 * the given model. The verifier register itself
	 * as a property listener to the model
	 * @param model
	 * @return
	 */
	public static SyntaxVerifier createSyntaxVerifier(AdoraModel model)
	{
		SyntaxVerifier verifier = new SyntaxVerifier(model);
		verifier.registerPropertyListener();
		verifier.validateElements(null);
		return verifier;
	}

	/**
	 * Get the syntax errors found by this {@link SyntaxVerifier}
	 * @return a {@link SyntaxProblemsSorter} containing all the errors
	 */
	public SyntaxProblemsSorter getSyntaxErrors()
	{
		return errors;
	}

	/**
	 * Unregister the property listeners and
	 * release resources (of the {@link SyntaxProblemsSorter} too!)
	 */
	public void dispose()
	{
		unregisterPropertyListener();
		elements.clear();
		elements = null;
		errors.clear();
		errors = null;
	}

	/**
	 * Constructor for this class
	 * @param model
	 */
	private SyntaxVerifier(AdoraModel model)
	{
		this.errors = new SyntaxProblemsSorter();
		this.elements = new LinkedList<ModelElement>();
		this.rules = RULES;	
		gatherElementsFromModel(model);
	}

	/**
	 * Auxiliary methods that gathers all the elements in a model
	 * into the elements list.
	 * @param model
	 */
	private void gatherElementsFromModel(AdoraModel model)
	{
		elements.add(model);
		for(Iterator<ModelElement> it = model.getRecursiveChildrenIterator(); it.hasNext();)
		{
			ModelElement me = it.next();
			elements.add(me);
			
			if (me instanceof ConnectableElement)
			{
				ConnectableElement ce = (ConnectableElement) me;
				for (Connection c : ce.getSourceConnections())
				{
					elements.add(c);
					if (c instanceof Transition)
					{
						Transition t = (Transition) c;
						elements.add(t.getLabel());
					}
				}
			}	
		}
	}

	/**
	 * Register this object as a property listener
	 * to all the element in the elements list
	 */
	private void registerPropertyListener()
	{
		for (ModelElement subject : elements)
		{
			registerPropertyListener(subject);
		}
	}
	
	/**
	 * Unregister this object as a property listener
	 * from all the element in the elements list
	 */
	private void unregisterPropertyListener()
	{
		for (ModelElement subject : elements)
		{
			unregisterPropertyListener(subject);
		}
	}
	
	/**
	 * Register this object as a property listener
	 * to the given model element.
	 * @param modelElement
	 */
	private void unregisterPropertyListener(ModelElement modelElement)
	{
		if (modelElement instanceof ObservableModelElement) {
			ObservableModelElement subject = (ObservableModelElement) modelElement;
			subject.removePropertyChangeListener(this);
		}
	}

	/**
	 * Unregister this object as a property listener
	 * from the given model element.
	 * @param modelElement
	 */
	private void registerPropertyListener(ModelElement modelElement)
	{
		if(modelElement instanceof ObservableModelElement)
		{
			ObservableModelElement subject = (ObservableModelElement) modelElement;
			subject.addPropertyChangeListener(this);
		}
	}


	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt)
	{ //TODO: IV commented out  TOO SLOW: COORDINATED WITH REINHARD
		/*if(isVisualEvent(evt))
		{
			return;
		}
		
		if(isAddEvent(evt) && evt.getNewValue() instanceof ModelElement)
		{
			ModelElement element = (ModelElement) evt.getNewValue();
			registerPropertyListener(element);
			elements.add(element);
			
			//TODO: Really necessary ?
			if (element instanceof Transition) {
				Transition transition = (Transition) element;
				registerPropertyListener(transition.getLabel());
				elements.add(transition.getLabel());
			}
		}
		else if(isDeleteEvent(evt))
		{
			ModelElement element = (ModelElement) evt.getSource();
			elements.remove(element);
			unregisterPropertyListener(element);
			errors.deleteEntries(element);
		}

		//TODO: Should we consider the validation of concerned elements only ?
		validateElements(evt);
		
		if(errors.hasChanged())
		{
			errors.notifyObservers();
		}*/
	}
	
	/**
	 * An auxiliary operation that classify the nature of an {@link PropertyChangeEvent}
	 * @param evt the event to be classified
	 * @return true if the event represents the deletion of a model element
	 */
	private static boolean isDeleteEvent(PropertyChangeEvent evt) {
		String propertyName = evt.getPropertyName();
		return propertyName.equals(VisualModElemPropNames.WAS_DELETED_PROP);
	}
	
	/**
	 * An auxiliary operation that classify the nature of an {@link PropertyChangeEvent}
	 * @param evt the event to be classified
	 * @return true if the event represents the creation of a model element
	 */
	private static boolean isAddEvent(PropertyChangeEvent evt) {
		String propertyName = evt.getPropertyName();
		return propertyName.equals(ConnectableElementPropNames.TARGETCONNECTION_NEW_PROP)
				|| propertyName.equals(IContainer.CHILD_ADDED_PROP);
	}
	
	/**
	 * An auxiliary operation that classify the nature of an {@link PropertyChangeEvent}
	 * @param evt the event to be classified
	 * @return true if the event represents the a visual modification of a model element
	 */
	private static boolean isVisualEvent(PropertyChangeEvent evt)
	{
		String propertyName = evt.getPropertyName();
		return propertyName.equals(VisualModElemPropNames.SIZE_PROP)
				|| propertyName.equals(VisualModElemPropNames.LOCATION_PROP)
				|| propertyName.equals(VisualModElemPropNames.VISIBILITY_PROP)
				|| propertyName.equals(ConnectionPropNames.BENDPOINTS_PROP);
	}


	/**
	 * Validate all elements from the elements list
	 * @param evt the event that triggered the validation
	 */
	private void validateElements(PropertyChangeEvent evt)
	{
		for(ModelElement element : elements)
		{
			validateElement(element, evt);
		}
		if(errors.hasChanged())
		{
			errors.notifyObservers();
		}
	}
	
	/**
	 * Validate a specific element
	 * @param el the element to be validated
	 * @param evt the event that triggered the validation
	 */
	private void validateElement(ModelElement el, PropertyChangeEvent evt)
	{
		this.currentTarget = el;
		this.currentEvent = evt;
		for (ISyntaxVerificationRule rule : rules)
		{
			this.currentRule = rule;
			rule.verify(this);
		}
		this.currentRule = null;
		this.currentTarget = null;
		this.currentEvent = null;
	}


	
	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext#getEvent()
	 */
	public PropertyChangeEvent getEvent()
	{
		return currentEvent;
	}


	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext#getTarget()
	 */
	public ModelElement getTarget()
	{
		return currentTarget;
	}


	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext#addErrorToTarget(java.lang.String)
	 */
	public void addErrorToTarget(String errorMessage)
	{
		errors.addEntry(new SyntaxProblem(currentTarget,errorMessage,currentRule.getRuleName(),SyntaxProblem.Severity.Error));
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext#clearAnnotationsFromTarget()
	 */
	public void clearAnnotationsFromTarget()
	{
		errors.clearEntry(currentRule.getRuleName(), currentTarget);
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext#addWarningToTarget(java.lang.String)
	 */
	public void addWarningToTarget(String warningMessage)
	{
		errors.addEntry(new SyntaxProblem(currentTarget,warningMessage,currentRule.getRuleName(),SyntaxProblem.Severity.Warning));
	}
	
}