package ch.unizh.ifi.adora.model.util;

import java.util.HashMap;

import ch.unizh.ifi.adora.model.representation.AdoraModel;

/**
 * Stores additional file property information which is 
 * used when storing changes in these file.
 * 
 * @author		        smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Jun 18, 2006 SM First Version
 * @version            $Date: 2006-06-18 18:27:41 +0200 (Sun, 18 Jun 2006) $, $Author: smeier $, $Revision: 1679 $
 */
public class ModelFileProperties {
	/**
	 * Singleton instance variable.
	 */
	private static ModelFileProperties instance = null;
	
	/**
	 * 
	 * @return
	 */
	public static ModelFileProperties getInstance() {
		if (instance == null) {
			instance = new ModelFileProperties();
		}
		return instance;
	}
	
	/**
	 * Maps the properties on the file names.
	 */
	private HashMap<String,ModelFileProperty> map = null; 
	
	/**
	 * Constructor protects the singleton instantiation.
	 */
	private ModelFileProperties() {		
		map = new HashMap<String, ModelFileProperty>();
	}

	/**
	 * Returns the property for the given filename. If the 
	 * property doesn't exist, it is created and the given 
	 * model is set. 
	 * 
	 * @param filename The unique filename containing the file path.
	 * @param am The ADORA model for the initialization of a new
	 *            property.
	 * @return Returns the property which is associated with
	 *          the given filename.
	 */
	public ModelFileProperty getProperty(String filename, AdoraModel am) {
		ModelFileProperty mp = map.get(filename);
		
		if (mp == null) {
			mp = new ModelFileProperty();
			mp.setAbsolutePathAndFilename(filename);
			mp.setAdoraModel(am);
		}
		return mp;
	}
	
	/**
	 * Returns a new model file property for the given 
	 * filename. If no property exists, a new model file 
	 * property is created with a given filename and not
	 * initially set model object and fileformat.
	 * 
	 * @param filename The filename for which the
	 *         property is returned.
	 * @return Returns the model property for a 
	 *          given filename. 
	 *          
	 */
	public ModelFileProperty getProperty(String filename) {
		return getProperty(filename, null);
	}
	
	/**
	 * Sets the model property for a given filename.
	 * The filename is taken from the property itself.
	 * @param mfp The model file property object to set.
	 */
	public void setProperty(ModelFileProperty mfp) {
		map.put(mfp.getAbsolutePathAndFilename(), mfp);
	}
	
	/**
	 * A property for a given filename and model is set.
	 * The fileformat stays undefined and therefore 
	 * the default file format is chosen for storage.
	 * @param filename The filename which is associated with 
	 *         the property.
	 * @param am The model which is associated with the 
	 *         property object.
	 */
	public void setProperty(String filename, AdoraModel am) {
		ModelFileProperty mp = new ModelFileProperty();
		mp.setAbsolutePathAndFilename(filename);
		mp.setAdoraModel(am);
		map.put(filename, mp);
	}

	/**
	 * A property for a given filename, model and fileformat 
	 * is set.
	 * 
	 * @param filename The filename which is associated with 
	 *         the property.
	 * @param am The model which is associated with the 
	 *         property object.
	 * @param fileFormat The file  format of the file.
	 */	
	public void setProperty(String filename, AdoraModel am, int fileFormat) {
		ModelFileProperty mp = new ModelFileProperty();
		mp.setAbsolutePathAndFilename(filename);
		mp.setAdoraModel(am);
		mp.setFileFormat(fileFormat);
		map.put(filename, mp);
	}
}
