package ch.unizh.ifi.adora.model.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import ch.unizh.ifi.adora.model.representation.AdoraModel;

/**
 * Describes the properties of a particular ADORA model file.
 * The model file is identified by its uniquely absolute
 * path and file name.
 * @author		        smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Jun 18, 2006 SM First Version
 * @version            $Date: 2006-06-18 18:27:41 +0200 (Sun, 18 Jun 2006) $, $Author: smeier $, $Revision: 1679 $
 */
public class ModelFileProperty {

	/**
	 * Representation for the xml gzipped format.
	 */
	public static final int FILE_FORMAT_NO_TYPE_SET=-1;

	/**
	 * Representation for the xml gzipped format.
	 */
	public static final int FILE_FORMAT_XML_GZIPPED=0;
	
	/**
	 * Representation for the plain xml format.
	 */
	public static final int FILE_FORMAT_XML=1;
	
	/**
	 * Representation for the Java serialization format.
	 */
	public static final int FILE_FORMAT_SERIALIZED=2;
	
	/**
	 * Describes the file format of the model file.
	 */
	private int fileFormat = FILE_FORMAT_NO_TYPE_SET;
	
	/**
	 * Describes the path and name of the ADORA model file
	 * which is described by this property. 
	 */
	private String absoluteFilePathName = "";
	
	/**
	 * Describes the ADORA model which is represented by this model.
	 */
	private AdoraModel model = null;
	
	/**
	 * Default constructor of the model file property. 
	 */
	public ModelFileProperty() {
		
	}
	
	/**
	 * Constructor initializes the file format of the model file.
	 * 
	 * @param fileFormat A constant that describes 
	 *                    the type of the file format
	 *                    for the ADORA model.
	 */
	public ModelFileProperty(int fileFormat) {
		this.fileFormat = fileFormat;
	}
	
	/**
	 * Constructor initializes the file format of the model file
	 * and the absoulute path and the name of the ADORA model 
	 * file
	 * 
	 * @param fileFormat A constant that describes 
	 *                    the type of the file format
	 *                    for the ADORA model.
	 * @param filename   The file name for the ADORA model
	 *                    file.
	 */
	public ModelFileProperty(int fileFormat, String filename) {
		this.fileFormat = fileFormat;
	}

	/**
	 * Constructor initializes the file format of the model file
	 * and the absoulute path and the name of the ADORA model 
	 * file and the model which is represented by this properties.
	 * 
	 * @param fileFormat A constant that describes 
	 *                    the type of the file format
	 *                    for the ADORA model.
	 * @param filename   The file name for the ADORA model
	 *                    file.
	 * @param model      The ADORA model which is represented 
	 *                    by this model file property.
	 */
	public ModelFileProperty(int fileFormat, String filename, AdoraModel model) {
		this.fileFormat = fileFormat;
	}
	
	/**
	 * Returns the type of the file format.
	 * @return Returns the type of the file format.
	 */
	public int getFileFormat() {
		return fileFormat;
	}
	
	/**
	 * Returns the type of the file format.
	 * @param fileFormat
	 */
	public void setFileFormat(int fileFormat) {
		this.fileFormat = fileFormat;
	}
	
	/**
	 * Returns the filename with the absolute path
	 * for the described ADORA model file.
	 * @return
	 */
	public String getAbsolutePathAndFilename() {
		return absoluteFilePathName;
	}

	/**
	 * Sets the file name with its absolute path for
	 * the described ADORA model.
	 * 
	 * @param fname The filename and the absolute path 
	 *         to the file containing the described ADORA 
	 *         model.
	 */
	public void setAbsolutePathAndFilename(String fname) {
		this.absoluteFilePathName = fname;
	}
	
	/**
	 * Sets the ADORA model which is represented by this properties.
	 * @param model The model which is represented by this properties.
	 */
	public void setAdoraModel(AdoraModel model) {
		this.model  = model;
	}
	
	/**
	 * Returns the ADORA model represented by this properties.
	 * @return Returns the ADORA model represented by this properties.
	 */
	public AdoraModel getAdoraModel() {
		return this.model;
	}
		
	/**
	 * Returns the correct output stream for the given
	 * model file which writes the given model format.
	 *  
	 * @return Returns the correct output stream 
	 *          for the given model file which writes the 
	 *          model in the chosen format.
	 */
	public ByteArrayOutputStream getModelOutputStream() throws IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
        switch (this.fileFormat) {
        	case FILE_FORMAT_XML_GZIPPED:
        		ModelSerializer.serializeModelToXMLOutputStream(model, out, true);
        		break;
        	case FILE_FORMAT_XML:
        		ModelSerializer.serializeModelToXMLOutputStream(model, out, false);
        		break;
        	case FILE_FORMAT_SERIALIZED:
        		ModelSerializer.serializeModelToStream(out, model);
        		break;        	
        }
		return out;
	}
}
