package ch.unizh.ifi.adora.model.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import ch.unizh.ifi.adora.model.representation.AdoraModel;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * Provides methods for the serialization and deserialization
 * of AdoraModels. This storage functionality is currently
 * thought as intermediate solution for storing the models in 
 * a non binary file which can be edited easier, in the change
 * of the API of the model.
 *      
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Jun 15, 2006 SM First Version
 *                      Jun 18, 2006 SM Some extensions 
 * @version			$Date: 2006-08-22 12:51:28 +0200 (Tue, 22 Aug 2006) $, $Author: vigan $, $Revision: 2089 $
 */
public class ModelSerializer {

	
	/**
	 * Serializes a given AdoraModel object and all its depenent 
	 * classes into an XML string.
	 * @param am The AdoraModel which should be serialized into 
	 *        an XML string.
	 * @return Returns the given AdoraModel as xml string.
	 */
	public static String serializeToXML(AdoraModel am) {
		XStream xstream = new XStream(new DomDriver()); 
		String xml = xstream.toXML(am);
		return xml;
	}
	
	/**
	 * Deserializes an AdoraModel object from a given String which
	 * contains xml.
	 * 
	 * @param xml The AdoraModel given as xml string.
	 * @return The AdoraModel which was deserialized.
	 * @throws Throws a java.langIllegalArgumentException is
	 *         thrown in the case that the given xml
	 *         string does not contain proper xml, or
	 *         it contains invalid serialized information or
	 *         it deserialized an Object which is not a
	 *         {@link ch.unizh.ifi.adora.model.representation.AdoraModel}.
	 *         
	 */
	public static AdoraModel deserializeFromXML(String xml) {
		XStream xstream = new XStream(new DomDriver());
		Object o;
		try {
			o = xstream.fromXML(xml);
		} catch (Exception ex) {
			IllegalArgumentException ie = new IllegalArgumentException("Not a valid XML file or not deserializable content!");
			ie.initCause(ex);
			throw ie;
		}
		AdoraModel am = null;
		if (o instanceof AdoraModel) {
			am = (AdoraModel)o;
		} else {
			IllegalArgumentException ex = new IllegalArgumentException("Deserialized objects are not of the type ch.unizh.ifi.adora.model.representation.AdoraModel.");
			throw ex;
		}
		return am;
	}
	
	/**
	 * Serializes a given AdoraModel as XML output into a given output stream.
	 * The input stream <i>is</i> will not
	 * be closed after using the stream. This has to be done by the caller
	 * of this method.
	 * 
	 * @param am AdoraModel object which should be serialized into a given outputstream
	 * @param os The output sream into which the XML output should be written.
	 * @param zipIt Indicates whether the output should be gzip compressed or not.
	 * @throws java.lang.IllegalArgumentException In the case that the input stream 
	 *         contains an invalid AdoraModel or invalid xml this exception is thrown.
	 * @throws java.io.IOException In the case that the input stream produces an exception.
	 */
	public static void serializeModelToXMLOutputStream(AdoraModel am, OutputStream os, boolean zipIt) throws IOException {
		OutputStream outStream = null;
		if (zipIt) {
			outStream = new GZIPOutputStream(os);			
		} else {
			outStream = os;
		}
		OutputStreamWriter osw = new OutputStreamWriter(outStream, "UTF-8");
		XStream xstream = new XStream(new DomDriver()); 
		ObjectOutputStream oos = xstream.createObjectOutputStream(osw);
		oos.writeObject(am);
		oos.close();
		osw.close();
		if (zipIt) { // only close the outstream if we have an
			         // intermediate zip stream.
			outStream.close();
		}
	}
	
	/**
	 * Deserializes a model from a given input stream which holds an xml input and
	 * converts it to an Adora model. The input stream <i>is</i> will not
	 * be closed after using the stream. This has to be done by the caller
	 * of this method.
	 * 
	 * @param is The input stream which will contains the element. 
	 * @param zipIt Indicates whether the input is gzip compressed or not.
	 * @return Returns an AdoraModel object which is extracted from the input stream.
	 * @throws java.lang.IllegalArgumentException In the case that the input stream 
	 *         contains an invalid AdoraModel or invalid xml this exception is thrown.
	 * @throws java.io.IOException In the case that the input stream produces an exception.
	 */
	public static AdoraModel deserializeModelFromXMLInputStream(InputStream is, boolean zipIt) throws IOException {
		InputStream inputStream = null;
		if (zipIt) {
			inputStream = new GZIPInputStream(is);
		} else {
			inputStream = is;
		}
			
		InputStreamReader isr = new InputStreamReader(inputStream, "UTF-8");
		XStream xstream = new XStream(new DomDriver());
		Object o;
		ObjectInputStream ois = xstream.createObjectInputStream(isr);
		
		try {
			o = ois.readObject();
		} catch (Exception ex) {
			IllegalArgumentException ie = new IllegalArgumentException("Not a valid XML file or not deserializable content!");
			ie.initCause(ex);
			throw ie;
		}
		ois.close();
		isr.close();
		if (zipIt) {
			inputStream.close();
		}
		AdoraModel am = null;
		if (o instanceof AdoraModel) {
			am = (AdoraModel)o;
		} else {
			IllegalArgumentException ex = new IllegalArgumentException("Deserialized objects are not of the type ch.unizh.ifi.adora.model.representation.AdoraModel.");
			throw ex;
		}

		return am;
	}
	
	/**
	 * Checks if the given input stream contains a valid gzip format.
	 * It returns true, if the given input stream contains a gzip format,
	 * otherwise it returns false.
	 * 
	 * @param is The input stream containing the input to check.
	 * @return Returns true if the input stream contains a valid
	 *         gzip format.
	 */
	public static boolean checkIfValidZipStream(InputStream is) {
		GZIPInputStream inputStream = null;
		boolean retVal = true; 
		try {
			inputStream = new GZIPInputStream(is);
		} catch (IOException ex) {
			retVal = false;
		}
		try {
			inputStream.close();
		} catch (Exception ex) {			
		}
		return retVal;
	}
	
	/**
	 * Checks for a given filename whether the file contains 
	 * a valid gzip file or not.
	 *  
	 * @param filename The filename including the path to access the 
	 *        file. 
	 * @return Returns true whether the file with the given filename
	 *         contains a valid gzip archive, otherwise false. 
	 */
	public static boolean checkIfValidZipFile(String filename) {
		boolean retVal = checkIfValidZipFile(new File(filename));
		return retVal;
	}
	
	/**
	 * Checks for a given filename whether the file contains 
	 * a valid gzip file or not.
	 *  
	 * @param file The file descriptor describing the the file to access. 
	 * @return Returns true whether the file with the given file descriptor
	 *         contains a valid gzip archive, otherwise false. 
	 */
	public static boolean checkIfValidZipFile(File file) {
		FileInputStream fis = null;
		boolean retVal = false;
		try {
			fis = new FileInputStream(file);
			retVal = checkIfValidZipStream(fis);
			fis.close();
		} catch (Exception ex) {
			
		}
		return retVal;
	}
	
    /**
     * Serializes the given model to an OutputStream
     * with the Java serialization mechanism.
     * 
     * @param os
     *            The stream to serialize to. != null
     * @throws IOException
     *             if serializing was not successful.
     */
    public static void serializeModelToStream(OutputStream os, AdoraModel model) throws IOException {
        ObjectOutputStream oos = new ObjectOutputStream(os);
        oos.writeObject(model);
        oos.close();
    }

    /**
     * Deserializes an ADORA model from a given input 
     * stream with the Java serialization mechanism.
     * 
     * @param is The input stream which contains the 
     *         model to deserialize.
     */
    public static AdoraModel deserializeModelFromStream(InputStream is) throws IOException, ClassNotFoundException {
        ObjectInputStream in = new ObjectInputStream(is);
        AdoraModel am = (AdoraModel) in.readObject();
        in.close();
        return am;
    }

	
}
