package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.properties.CardinalityPropNames;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.parts.directedit.NodeCellEditorLocator;
import ch.unizh.ifi.adora.parts.directedit.NodeEditManager;
import ch.unizh.ifi.adora.parts.policies.LabelDirectEditPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.ActorNodePolicy;
import ch.unizh.ifi.adora.properties.ActorProperties;
import ch.unizh.ifi.adora.view.anchors.SextangleClickPointAnchor;
import ch.unizh.ifi.adora.view.figures.ActorFigure;

/**
 * This is the EditPart for an Actor model element.
 * 
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			27.09.2005 SM First Version
 * 						20.07.2006 SM Comments completed and checked.
 *									  Small refactoring.
 *						15.08.2006 JS getCastedModel() overridden, unnecessary 
 * 									  casts removed
 * @version			$Date: 2006-10-27 15:02:09 +0200 (Fri, 27 Oct 2006) $, $Author: smeier $, $Revision: 2338 $
 */
public class ActorEditPart extends NodeEditPart implements IAdaptable{

	/**
	 * A reference to the direct editor for the actor's label.
	 */
	private DirectEditManager manager;
	
	/**
	 * IPropertySource for this ModelElement which handles all requests about
	 * setting ang getting properties.
	 */
	private ActorProperties actorProps;

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createFigureForModel()
	 */
	protected IFigure createFigureForModel() {
		ActorFigure p = new ActorFigure(getCastedModel().getMinCardinality(),
						getCastedModel().getMaxCardinality());
		p.setText(getCastedModel().getName());
		p.setComment(getCastedModel().getComment());
		return p;
	}

	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		// allow the creation of connections and
		// and the reconnection of connections between Shape instances
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, 
				new ActorNodePolicy());

		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, 
				new LabelDirectEditPolicy());
	}

	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.NodeEditPart#createConnectionAnchor()
	 */
	protected ConnectionAnchor createConnectionAnchor() {
		// gets this.getFigure() and sets it as 
		// figure for which the anchor has to be computed
		return new SextangleClickPointAnchor(getFigure());
	}

	/**
	 * Returns the casted model element which is associated with
	 * this edit part.
	 * @return Returns the casted model element.
	 */
	protected Actor getCastedModel() {
		return (Actor) getModel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		ActorFigure f = (ActorFigure)this.getFigure();
		// changes in name
		if(State.NAME_PROP.equals(prop)) {
			f.setText(getCastedModel().getName());
			f.layout();
            f.repaint();
        // changes in minimum cardinality
		} else if(CardinalityPropNames.MINCARD_PROP.equals(prop)){
			f.setMinCardinality(getCastedModel().getMinCardinality());
			f.relayout();
            f.repaint();
        // changes in maximum cardinality
		} else if(CardinalityPropNames.MAXCARD_PROP.equals(prop)){
			f.setMaxCardinality(getCastedModel().getMaxCardinality());
			f.relayout();
            f.repaint();
		}
		super.propertyChange(evt);
	}
	
	/**
	 * Executes the direct editing by creating a new node
	 * edit manager which handles the editing.
	 */
	private void performDirectEdit(){
		if(manager == null)
			manager = new NodeEditManager(
					this, new NodeCellEditorLocator(((ActorFigure)getFigure()).getNameLabel()));
		manager.show();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request request){
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
			performDirectEdit();
	}


	/**
	 * Returns the IPropertySource for this Modelelement.
	 * @param adapter The class for which the corresponding object will be returned.
	 * @return Returns an object for a given adapter class.
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		 if (adapter == IPropertySource.class) {
			if (actorProps == null) {
				actorProps = new ActorProperties(getCastedModel());
			}
			return actorProps;
		 }
		 return null;
	}
}