package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.Animation;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayeredPane;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.LayoutAnimator;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.Viewport;
import org.eclipse.gef.CompoundSnapToHelper;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.SnapToGeometry;
import org.eclipse.gef.SnapToGrid;
import org.eclipse.gef.SnapToGuides;
import org.eclipse.gef.SnapToHelper;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.gef.editpolicies.SnapFeedbackPolicy;
import org.eclipse.gef.rulers.RulerProvider;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.parts.policies.AdoraContainerEditPolicy;
import ch.unizh.ifi.adora.parts.policies.containers.AdoraModelContainerPolicy;
import ch.unizh.ifi.adora.representation.management.ImplPropComposer;
import ch.unizh.ifi.adora.representation.management.ViewManagementConstants;
import ch.unizh.ifi.adora.representation.management.ViewManager;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.utils.ModelElementVisibilityFilter;
import ch.unizh.ifi.adora.utils.filteredlist.FilteredArrayList;
import ch.unizh.ifi.adora.view.layers.AdoraFreeformLayer;
import ch.unizh.ifi.adora.zoom.ZoomEventConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;

/**
 * EditPart for the root of an Adora model.
 * <p>
 * This edit part serves as the main diagram container, the white area where
 * everything else is in. 
 * </p>
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				Sep 29, 2005 SM First Version
 *						Dec  5, 2005 SM 
 *						Jul 20, 2006 SM Comments completed, checked and corrected.
 *						Jul 25. 2006 SM Comments checked, small corrections.
 * @version				$Date: 2008-12-23 10:18:36 +0100 (Tue, 23 Dec 2008) $, $Author: treinhard $, $Revision: 3969 $
 * @responsibilities		Represents the root edit part of an Adora model.
 *
 */
public class AdoraModelEditPart extends AbstractGraphicalEditPart implements
		PropertyChangeListener, IPreferenceNameConstants {

	/**
	 * Upon activation, attach to the model element as a property change
	 * listener.
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */	
	public void activate() {
		if (!isActive()) {
			super.activate();
            AdoraModel model = (AdoraModel) getModel(); 
			model.addPropertyChangeListener(this);
			
			AdoraZoomAdapter.getInstance().addPropertyChangeListener(this);
			ViewManager.getInstance().addPropertyChangeListener(this);
			ImplPropComposer.getInstance().addPropertyChangeListener(this);
			
			// Enable the "snap to geometry" feature by default
			getViewer().setProperty(SnapToGeometry.PROPERTY_SNAP_ENABLED, true);
  		}
	}


	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// disallows the removal of this edit part from its parent
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new RootComponentEditPolicy());
		// handles constraint changes (e.g. moving and/or resizing) of model
		// elements
		// and creation of new model elements
		installEditPolicy(EditPolicy.LAYOUT_ROLE,
				new AdoraModelContainerPolicy());
		installEditPolicy(EditPolicy.CONTAINER_ROLE,
				new AdoraContainerEditPolicy());
		
		installEditPolicy("Snap Feedback", new SnapFeedbackPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		FreeformLayer f = new AdoraFreeformLayer() {
		
			public void paint(Graphics graphics) {
				graphics.setAntialias(SWT.ON);
				super.paint(graphics);
			}
		
		};
		f.setBorder(new MarginBorder(3));
		f.setLayoutManager(new FreeformLayout());
        
        // This is just the default line router. The actual router is selected
        // accordingely to the type of the connection that has to be routed.
        ConnectionLayer connLayer = (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
        connLayer.setConnectionRouter(new BendpointConnectionRouter());
        
        f.addLayoutListener(LayoutAnimator.getDefault());
		
		return f;
	}


	/**
	 * Upon deactivation, detach from the model element as a property change
	 * listener.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			((ObservableModelElement) getModel()).removePropertyChangeListener(this);
			
			AdoraZoomAdapter.getInstance().removePropertyChangeListener(this);
			ViewManager.getInstance().removePropertyChangeListener(this);
			ImplPropComposer.getInstance().removePropertyChangeListener(this);
		}
	}

	/**
	 * Returns the corresponding modelelement for this edit part.
	 * @return Returns the root model element of the model.
	 */
	private AdoraModel getCastedModel() {
		return (AdoraModel) getModel();
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		final AdoraModel model = getCastedModel();
		List<ModelElement> children = model.getChildren();
        FilteredArrayList list = new FilteredArrayList(children);
		list.setFilter(new ModelElementVisibilityFilter());
		return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();

		if (AdoraModel.CHILD_ADDED_PROP.equals(prop) || 
        		AdoraModel.CHILD_REMOVED_PROP.equals(prop)){
            refreshChildren();
		} else if (VisualizableModelElement.VISIBILITY_PROP.equals(prop)) {
        	refresh(); // shows hidden pasted nodes, after filter is disabled.
		} else if(ZoomEventConstants.START_ZOOM.equals(prop)
				|| ViewManagementConstants.NODE_FILTER_START.equals(prop)) {
			
			boolean animate = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM);
			
			if(animate) {
				FreeformLayeredPane layeredPane = (FreeformLayeredPane)getFigure().getParent().getParent();
				Layer connectionLayer = layeredPane.getLayer(LayerConstants.CONNECTION_LAYER);
				connectionLayer.setVisible(false);
			
				Animation.markBegin();
			}
			
		} else if(ZoomEventConstants.END_ZOOM.equals(prop)
				|| ViewManagementConstants.NODE_FILTER_END.equals(prop)) {
			
			IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
			boolean animate = store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM);
			
			
			if(animate) {
				int delay = store.getInt(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_ANIMATION_DELAY);
				FreeformLayeredPane layeredPane = (FreeformLayeredPane)getFigure().getParent().getParent();
				Layer connectionLayer = layeredPane.getLayer(LayerConstants.CONNECTION_LAYER);
				connectionLayer.setVisible(false);
				
				Animation.run(delay);
				
				connectionLayer.setVisible(true);
				
			}
			
			// The node that has been zoomed
			IZoomNode target = (IZoomNode)evt.getNewValue();
			
			if(target != null) {
				// Translate the location of the zoomed node to absolute coordinates
				java.awt.Point location = target.getZoomBounds().getLocation();
				IZoomNode parent = target.getZoomParent();
				while(parent != null) {
					java.awt.Point parentLoc = parent.getZoomBounds().getLocation();
					location.translate(parentLoc.x, parentLoc.y);
					parent = parent.getZoomParent();
				}
	    	
				// Take the center of the node
				location.translate(target.getZoomBounds().width / 2, target.getZoomBounds().height / 2);
	    	
				// Center the viewport on the center of the zoomed node or its
				// parent respectively.
				FigureCanvas canvas = (FigureCanvas)getRoot().getViewer().getControl();
				Viewport viewPort = canvas.getViewport();		
				location.translate(-(viewPort.getBounds().width / 2), -(viewPort.getBounds().height / 2));
				if(animate) {
					canvas.scrollSmoothTo(location.x, location.y);
				} else {
					canvas.scrollTo(location.x, location.y);
				}
			}
		}
     }
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		ConnectionLayer cLayer = (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
		cLayer.setAntialias(SWT.ON);
	}
	
	
	/**
	 * Overriden to add the Snap (snap to grid, snap to geometry) helpers.
	 * 
	 */
	public Object getAdapter(Class adapter) {
		
		// Snap to grid or layout
		if (adapter == SnapToHelper.class) {
			List<SnapToHelper> snapStrategies = new ArrayList<SnapToHelper>();
			Boolean val = (Boolean)getViewer().getProperty(RulerProvider.PROPERTY_RULER_VISIBILITY);
			if (val != null && val.booleanValue())
				snapStrategies.add(new SnapToGuides(this));
			val = (Boolean)getViewer().getProperty(SnapToGeometry.PROPERTY_SNAP_ENABLED);
			if (val != null && val.booleanValue())
				snapStrategies.add(new SnapToGeometry(this));
			val = (Boolean)getViewer().getProperty(SnapToGrid.PROPERTY_GRID_ENABLED);
			if (val != null && val.booleanValue())
				snapStrategies.add(new SnapToGrid(this));
			
			if (snapStrategies.size() == 0)
				return null;
			if (snapStrategies.size() == 1)
				return snapStrategies.get(0);

			SnapToHelper ss[] = new SnapToHelper[snapStrategies.size()];
			for (int i = 0; i < snapStrategies.size(); i++)
				ss[i] = (SnapToHelper)snapStrategies.get(i);
			return new CompoundSnapToHelper(ss);
		}
		
		return super.getAdapter(adapter);
	}

}