package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.commands.CalculateInterRelations;
import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Role;
import ch.unizh.ifi.adora.parts.policies.connections.AssociationEditPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.ConstrainableConnectionPolicy;
import ch.unizh.ifi.adora.properties.AssociationProperties;
import ch.unizh.ifi.adora.utils.ConnectionLabelFilter;
import ch.unizh.ifi.adora.utils.filteredlist.FilteredArrayList;
import ch.unizh.ifi.adora.view.figures.AssociationFigure;

/**
 * This is the EditPart for an Association. 
 * This EdiPart is also responsible for creating the RoleEditParts.
 * 
 * @author				smeier
 *						ivo vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			27.09.2005 SM First Version
 * 						14.08.2006 JS Removed unnecessary cast
 * @version			$Date: 2008-11-27 14:11:37 +0100 (Thu, 27 Nov 2008) $, $Author: treinhard $, $Revision: 3905 $
 */
public class AssociationEditPart extends AbstractableConnectionEditPart implements
		IAdaptable {

	/**
	 * IPropertySource for this ModelElement.
	 */
	private AssociationProperties associationProps;

	/**
	 * Returns the underlying model. 
	 * @return Returns the model which belongs to this edit part.
	 */
	public Association getCastedModel() {
		return (Association) getModel();
	}
	
	/**
	 * Return the figure, associated with this edit part.
	 * @return {@link AssociationFigure}
	 */
	public AssociationFigure getCastedFigure() {
		return (AssociationFigure) getFigure();
	}

	/**
	 * Adds this editpart to the propertyListener of the association.
	 * Used to show and hide labels if content == default.
	 * 
	 * @see ch.unizh.ifi.adora.parts.ConnectionEditPart#activate()
	 */
	public void activate() {
		if (!isActive()) {
			List<Role> children = ((Association) getCastedModel()).getRoles();
			Iterator iter = children.iterator();
			while (iter.hasNext()) {
				Role next=(Role) iter.next();
				next.addPropertyChangeListener(this);
				next.addPropertyChangeListener((Association) getCastedModel());
			}
			super.activate();
		}
	}

	/**
	 * Removes this editpart from the propertyListener of the association.
	 * Used to show and hide labels if content = default.
	 * 
	 * @see ch.unizh.ifi.adora.parts.ConnectionEditPart#deactivate()
	 */
	public void deactivate() {
		if (isActive()) {
			List<Role> children = ((Association) getCastedModel()).getRoles();
			Iterator iter = children.iterator();
			while (iter.hasNext()) {
				Role next=(Role) iter.next();
				next.removePropertyChangeListener(this);
				next.removePropertyChangeListener((Association) getCastedModel());
			}
			super.deactivate();
		}
	}

	/**
	 * This method is called to create the Children-EditParts.
	 */
	public EditPart createChild(Object model) {
		RoleEditPart part = null;
		part = new RoleEditPart();
		part.setModel(model);
		return part;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if (Association.MAN_ABSTRACTED_PROP.equals(property)) {
				CalculateInterRelations maa = new CalculateInterRelations(
						getCastedModel());
				executeCommandOnStack(maa);
		} else if (Role.MINCARD_PROP.equals(property)
				|| Role.MAXCARD_PROP.equals(property)
				|| Role.NAME_PROP.equals(property)) {
			refresh();
		} else if(Association.ROLE_CHANGED_PROP.equals(property)) {
			refresh();
		} else if (ConnectableElementPropNames.SOURCE_CONNECTIONS_PROP
				.equals(property)) {
			refreshSourceConnections();
		} else if (ConnectableElementPropNames.TARGET_CONNECTIONS_PROP
				.equals(property)) {
			refreshTargetConnections();
			
		} else if (Connection.PROPERTY_POINTS.equals(property)) {
			getCastedModel().updateParentsClickPoints();
			
		} 
		super.propertyChange(event);
	}



	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		// Selection handle edit policy.
		// Makes the connection show a feedback, when selected by the user.
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ConnectionEndpointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE,
				new AssociationEditPolicy(this));
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new ConstrainableConnectionPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */

	protected IFigure createFigure() {
		AssociationFigure	f = new AssociationFigure();
		f.setComment(getCastedModel().getComment());
		return f;
	}

	/**
	 * Returns all labels which don't contain default text
	 */
	public List<Role> getModelChildren() {
		List<Role> children = getCastedModel().getRoles();
		FilteredArrayList list = new FilteredArrayList(children);
		list.setFilter(new ConnectionLabelFilter());
		return list;
	}

	/**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (associationProps == null) {
				associationProps = new AssociationProperties(
						(Association) getCastedModel());
			}
			return associationProps;
		}
		return null;
	}
}