package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.graphics.Color;

import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.representation.CommentableElement;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.utils.ModelElementVisibilityFilter;
import ch.unizh.ifi.adora.utils.filteredlist.FilteredArrayList;
import ch.unizh.ifi.adora.view.figures.ConnectionFigure;
import ch.unizh.ifi.adora.view.figures.ICommentableFigure;

/**
 * 
 * Edit part for Connection model elements.
 * <p>
 * This edit part must implement the PropertyChangeListener interface, so it can
 * be notified of property changes in the corresponding model element.
 * </p>
 * This edit part is notified by the connection figure if the corresponding line
 * in the view has been rerouted, so that the coordinates of the bendpoints can be
 * stored in the model. This is necessary because the line routing works on the view
 * in the GEF and there's no other posibility to store the bend points in the model.
 * </p>
 * 
 *
 * @author				gygax
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				31.08.2005 AG First Version
 * 						20.07.2006 SM Comments checked, corrected and completed.
 * 						17.09.2007 SM Bug 801: After their creation, connections are selected.
 * @version				$Date: 2008-11-27 14:11:37 +0100 (Thu, 27 Nov 2008) $, $Author: treinhard $, $Revision: 3905 $
 */
public abstract class ConnectionEditPart extends AbstractConnectionEditPart implements
		PropertyChangeListener, IAdaptable{
    
    /**
     * True, if this connection is currently selected. 
	 */
    protected boolean isSelected = false;
		
    /**
	 * This method is responsible for drawing a visual representation
	 * of the selection state. In other words:
	 * If a label or a connection is selected, all involved components
	 * get drawn bordered.
	 */
	public void changeSelectionRepresentation(){
		
		isSelected = !isSelected;
		
		ConnectionFigure figure = ((ConnectionFigure)getFigure());
		
		if (isSelected)
			figure.setForegroundColor(new Color(null, 0, 125, 125));
		else
			figure.setForegroundColor(new Color(null, 125, 125, 125));

	}
	
	
	/**
	 * Used to determine when the {@link LabeledConnectionEditPart} has been selected.
	 * Then the {@link ConnectionLabelEditPart} must also be selected.
	 */
	public void fireSelectionChanged() {
  		super.fireSelectionChanged();
	    
		changeSelectionRepresentation();
	}
	
	/**
	 * Upon activation, attach to the model element as a property change
	 * listener.
	 */
	public void activate() {
		if (!isActive()) {
			super.activate();
			((ObservableModelElement) getModel()).addPropertyChangeListener(this);
         

            //Since the adora-model handles the whole view management, it has to know when 
            // a connection's visibility changed. 
            Iterator iter = this.getRoot().getChildren().iterator();
            while(iter.hasNext()) {
                AdoraModelEditPart ep = (AdoraModelEditPart)iter.next();
                ((ObservableModelElement) getModel()).addPropertyChangeListener(ep);
            }
            // We have to become informed if the line is routed by the routing
			// algorithm so that we can store the position of the bend points in
			// the model.
			getFigure().addPropertyChangeListener(this);
		}
	}
	
	/**
	 * Upon deactivation, detach from the model element as a property change
	 * listener.
	 */
	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			((ObservableModelElement) getModel()).removePropertyChangeListener(this);
			getFigure().removePropertyChangeListener(this);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// Selection handle edit policy.
		// Makes the connection show a feedback, when selected by the user.
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ConnectionEndpointEditPolicy());
		
        // Policy that shows the selection handles at the midpoints of the orthogonal
        // line segments.
		//if (getConnectionFigure().getConnectionRouter() instanceof OrthogonalTileMazeRouter) {
		//	installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new ConnectionBendpointEditPolicy());
        //}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if(ConnectableElement.SET_ELEMENT_SELECTED_PROP.equals(property)){
			if(event.getNewValue().equals(false)){ //no clearing, add this 
				List selection=this.getViewer().getSelectedEditParts();
				Vector selvec=new Vector(selection);
				selvec.add(this);
				this.getViewer().setSelection(new StructuredSelection(selvec.toArray()));
			}
			else {//clear all selections, att this
				this.getViewer().deselectAll();
				this.getViewer().select(this);
			}
			this.getViewer().reveal(this);
		}
		else if (CommentableElement.COMMENT_PROP.equals(property)) {
			((ICommentableFigure)getFigure()).setComment(((CommentableElement)getModel()).getComment());
		} else if(Connection.SOURCECLICKPOINT_PROP.equals(property)) {
			Connection con = (Connection)getModel();
			((ConnectionFigure) getFigure()).setSourceClickPoint(con.getSourceXRatio(), con.getSourceYRatio());
		} else if(Connection.TARGETCLICKPOINT_PROP.equals(property)) {
			Connection con = (Connection)getModel();
			((ConnectionFigure) getFigure()).setTargetClickPoint(con.getTargetXRatio(), con.getTargetYRatio());
		} else if (ConnectableElementPropNames.SOURCE_CONNECTIONS_PROP.equals(property)) {
            refreshSourceConnections();
        } else if (ConnectableElementPropNames.TARGET_CONNECTIONS_PROP.equals(property)) {
            refreshTargetConnections();
        } else if(VisualizableModelElement.VISIBILITY_PROP.equals(property)){
        	refresh();
        }
	}
    
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
     */
    protected List getModelSourceConnections() {
        FilteredArrayList list = new FilteredArrayList((ArrayList)((ConnectableElement)getModel()).getSourceConnections());
        list.setFilter(new ModelElementVisibilityFilter());
        return list;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
     */
    protected List getModelTargetConnections() {
        FilteredArrayList list = new FilteredArrayList((ArrayList)((ConnectableElement)getModel()).getTargetConnections());
        list.setFilter(new ModelElementVisibilityFilter());
        return list;
    }
}