package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Locator;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;

import ch.unizh.ifi.adora.model.properties.VisualModElemPropNames;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ConnectionLabel;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.parts.geometryfactory.GeometryFactory;
import ch.unizh.ifi.adora.tracker.AdoraDragTracker;
import ch.unizh.ifi.adora.view.figures.LabeledConnection;
import ch.unizh.ifi.adora.view.figures.label.algorithm.placing.MovableLocator;

/** 
 * Abstract editpart used to direct edit, 
 * resize and relocate connection labels.
 * 
 * @author				vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Feb 15, 2006 IV First Version
 * 						Jul 20, 2006 SM Comments checked and corrected.
 * 										Some refactoring.
 * 						Aug 15, 2006 JS getCastedModel() overridden and used
 * 										where applicable
 *  				21.09.2007 CR changes to allow setting size according to length of text
 * @version			$Date: 2007-09-25 16:03:24 +0200 (Tue, 25 Sep 2007) $, $Author: cramer $, $Revision: 3009 $
 *
 */
public abstract class ConnectionLabelEditPart extends VisualizableModelElementEditPart {

	/**
	 * {@link LabelPlacementAlgorithm} used for relocating the label.
	 */
	private MovableLocator locator;

	/**
	 * Creates the Figure, associated with this edit part. 
	 */
	protected abstract IFigure createFigureForModel();

	/** 
	 * No anchor needed, therefore null is returned.
	 * @return null
	 */
	protected ConnectionAnchor createConnectionAnchor() {
		return null;
	}
	
	/**
	 * Sets the size of the label according to the model size.
	 * 
	 * @param label The {@link IFigure} for which the size is set.
	 */
	protected void setSize(IFigure label){
		ConnectionLabel model = getCastedModel();
		
		label.setSize((Dimension)
				GeometryFactory.getDraw2dElement(
						model.getSize()));
	}
	
	/**
	 * Returns the casted model element which is associated with
	 * this edit part.
	 * @return Returns the casted model element.
	 */
	protected ConnectionLabel getCastedModel() {
		return (ConnectionLabel) getModel();
	}

	/**
	 * This method gets called each time the 
	 * {@link ch.unizh.ifi.adora.view.figures.LabeledConnection} 
	 * changes his visibility state, or if it gets attached 
	 * to an other node.
	 */
	private void recalculateLabelPosition(){
		//only setLocationOffset 
		//if it's not equal the stored offset.
		if (!(getLocator().getLocationDelta().equals(
					(org.eclipse.draw2d.geometry.Point)
					GeometryFactory.getDraw2dElement(
							getCastedModel().
								getConnectionOffset())))) {
		
			getLocator().setLocationDelta(
					(org.eclipse.draw2d.geometry.Point)
					GeometryFactory.getDraw2dElement(getCastedModel().
									getConnectionOffset()));
			refreshVisuals();
			
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#propertyChange(java.beans.PropertyChangeEvent)
	 */	
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		if (Connection.SOURCECLICKPOINT_PROP.equals(prop) || 
				Connection.TARGETCLICKPOINT_PROP.equals(prop)) {
			recalculateLabelPosition();
			
		} else if (VisualModElemPropNames.NAME_PROP.equals(prop)) {
		
			INodeProperties f = (INodeProperties) getFigure();
			
			f.setText(getCastedModel().getName());

			refresh();
		} else if (Node.SIZE_PROP.equals(prop)) {
				
			getFigure().setSize(
					(org.eclipse.draw2d.geometry.Dimension)
					GeometryFactory.getDraw2dElement(getCastedModel().getSize()));
			
			refreshVisuals();
		}
			 
		else if (VisualizableModelElement.VISIBILITY_PROP.equals(prop)) {
			recalculateLabelPosition();
		}
		
		else if (Node.LOCATION_PROP.equals(prop)) {
			getLocator().setLocationDelta(
					(org.eclipse.draw2d.geometry.Point)
					GeometryFactory.getDraw2dElement(getCastedModel().
									getConnectionOffset()));
		} 
	}

	/**
	 * This method is used to obtain the locator for updating the 
	 * offset made by the user.
	 * 
	 * @return The {@link LabelPlacementAlgorithm}used for this label.
	 */
	protected MovableLocator getLocator() {
		return locator;
	}

	/**
	 * Sets the delegating locator for the connection label.
	 *
	 * @param The {@link Locator} which gets decorated by the {@link MovableLocator}.
	 */
	protected void decorateLocator(Locator decoratedLocator) {
		MovableLocator locator = new MovableLocator(
				(LabeledConnection)((LabeledConnectionEditPart)getParent()).getFigure(),
				decoratedLocator);
		locator.setLocationDelta((Point)GeometryFactory.getDraw2dElement(
				getCastedModel().getConnectionOffset()));
			
		this.locator = locator;
	}

	/**
	 * Updates the visuals of the RoleEditPart and sets
	 * the constraints of the DelegatingLocator.  
	 */
	protected void refreshVisuals() {
		if (getParent() != null) {
			((GraphicalEditPart) getParent()).setLayoutConstraint(this,
					getFigure(),
					getLocator());
		}
	}
	
	/**
	 * Used to know if the ConnectionLabel has been selected.
	 * Then the corresponding connection has to be selected too.
	 */
	protected void fireSelectionChanged() {
		super.fireSelectionChanged();
        ((LabeledConnectionEditPart)getParent()).
        changeSelectionRepresentation();
	}
	
	/**
	 * Overridden to return a {@link ch.unizh.ifi.adora.tracker.AdoraDragTracker} for GraphicalEditParts.
	 * @see org.eclipse.gef.EditPart#getDragTracker(Request)
	 */
	public DragTracker getDragTracker(Request request) {
		return new AdoraDragTracker(this);
	}
}