package ch.unizh.ifi.adora.parts;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.parts.directedit.NodeCellEditorLocator;
import ch.unizh.ifi.adora.parts.directedit.NodeEditManager;
import ch.unizh.ifi.adora.parts.policies.LabelDirectEditPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.ExitPointPolicy;
import ch.unizh.ifi.adora.properties.ExitPointProperties;
import ch.unizh.ifi.adora.view.anchors.RectangleClickPointAnchor;
import ch.unizh.ifi.adora.view.figures.ExitPointFigure;

/**
 * This is an Editpart of an ExitPoint.
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			02.05.2006 IV First Version
 * 						20.07.2006 SM Comments checked and corrected.
 *						15.08.2006 JS getCastedModel() overridden, unnecessary 
 * 									  casts removed
 * @version			$Date: 2006-09-27 17:02:22 +0200 (Wed, 27 Sep 2006) $, $Author: vigan $, $Revision: 2287 $
 */
public class ExitPointEditPart extends NodeEditPart implements IAdaptable, ITransitionGraph{

    /**
     * IPropertySource for this ModelElement. This property
     * source is responsible for setting and getting values into/from
     * the model. 
     */
    private ExitPointProperties exitProps;
 
    /**
     * Handles the direct editing capabilites for a node edit part.
     */
    protected NodeEditManager manager;
    
    /**
     * Used for iteration of the IGraph.
     */
    private boolean visited = false;
    
    /*
     * (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.NodeEditPart#createConnectionAnchor()
     */
    @Override
    protected ConnectionAnchor createConnectionAnchor() {
        return new RectangleClickPointAnchor(getFigure());
    }

    /*
     * (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createFigureForModel()
     */
    protected IFigure createFigureForModel() {
    	ExitPointFigure f = new ExitPointFigure(getCastedModel().getName());
        f.setComment(getCastedModel().getComment());
        return f;
    }

	/**
	 * Returns the casted model element which is associated with
	 * this edit part.
	 * @return Returns the casted model element.
	 */
	protected ExitPoint getCastedModel() {
		return (ExitPoint) getModel();
	}

    /**
     * Performs the direct editing capabilities.
     */
    private void performDirectEdit() {
        if (manager == null)
            manager = new NodeEditManager(this, new NodeCellEditorLocator(
                    ((ExitPointFigure) getFigure()).getNameLabel()));
        manager.setLabel(((ExitPointFigure)getFigure()).getNameLabel());
        manager.show();
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
     */
    public void performRequest(Request request) {
        if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
            performDirectEdit();
    }
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies() {
        super.createEditPolicies();
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE,
                new LabelDirectEditPolicy());
        
        installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
                new ExitPointPolicy());
    }
    
    /**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
    public Object getAdapter(Class adapter) {
         if (adapter == IPropertySource.class) {
            if (exitProps == null) {
                exitProps = new ExitPointProperties(getCastedModel());
            }
            return exitProps;
         }  
         return null;
    }
    
    
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.IGraph#resetVisited()
     */
    public void resetVisited(){
    	if (!visited)
    		return;
    	visited = false;
    	ArrayList<ConnectionEditPart> list = new ArrayList(getSourceConnections());
    	Iterator conIter = getTargetConnections().iterator();
    	while(conIter.hasNext()){
    		list.add((ConnectionEditPart)conIter.next());
    	}
		Iterator iter = list.iterator();
		ConnectionEditPart connectionEP;
		NodeEditPart otherNodePart;
		while (iter.hasNext()){
			connectionEP = (ConnectionEditPart)iter.next();
			if (!(connectionEP instanceof TransitionEditPart))
				continue;
			otherNodePart = connectionEP.getSource() == this ? 
						(NodeEditPart)connectionEP.getTarget() :
						(NodeEditPart)connectionEP.getSource();
						
			((ITransitionGraph)otherNodePart).resetVisited();
		}
    }
	
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.IGraph#depthFirstSearch()
     */
    public boolean depthFirstSearch() {
		
		Node otherNode;

		boolean movable = true;
		
		if (visited)
			return getSelected() > 0;
	
		visited = true;
		
		if (getSelected() == 0)
			return false;
		
		//check for hidden nodes --> no moving allowed
		for (Connection c : getCastedModel().getConnections()) {
			if (!(c instanceof Transition)) 
				continue;
		
			otherNode = c.getSource() == getModel() ? (Node)c.getTarget() :
				(Node)c.getSource();
			
			if(!otherNode.isVisible()){
				return false;
			}
		}
		
		ArrayList<ConnectionEditPart> list = new ArrayList(getSourceConnections());
    	Iterator conIter = getTargetConnections().iterator();
    	while(conIter.hasNext()){
    		list.add((ConnectionEditPart) conIter.next());
    	}
		Iterator iter = list.iterator();
		ConnectionEditPart connectionEP;
		NodeEditPart otherNodePart;
		while (iter.hasNext()){
			connectionEP = (ConnectionEditPart)iter.next();
			if (!(connectionEP instanceof TransitionEditPart))
				continue;
			otherNodePart = connectionEP.getSource() == this ? 
						(NodeEditPart)connectionEP.getTarget() :
						(NodeEditPart)connectionEP.getSource();
						
			movable = ((ITransitionGraph)otherNodePart).depthFirstSearch();
			if (!movable)
				return false;
		}
		return movable;
	}

    /*
     * (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.IGraph#isVisited()
     */
	public boolean wasGraphNodeChecked() {
		return visited;
	}
}