package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.JoinRelationLabel;
import ch.unizh.ifi.adora.parts.policies.connections.JoinRelationEditPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.ConstrainableConnectionPolicy;
import ch.unizh.ifi.adora.properties.JoinRelationProperties;
import ch.unizh.ifi.adora.view.figures.JoinRelationFigure;

/**
 * This is the EditPart for a {@link JoinRelation}. 
 * This EdiPart is also responsible for creating the  
 * {@link ch.unizh.ifi.adora.model.representation.JoinRelationLabel}.   
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			19.07.2006 IV First Version
 * 						20.07.2006 SM Comments reformatting, checking and correcting.
 * 						14.08.2006 JS Removed unnecessary cast
 * @version			$Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class JoinRelationEditPart extends AbstractableConnectionEditPart  implements
		IAdaptable {

	/**
	 * IPropertySource for this ModelElement.
	 */
	private JoinRelationProperties joinRelationProps;

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate() {
		if (!isActive()) {
			JoinRelationLabel child = ((JoinRelation) getCastedModel())
					.getLabel();
			child.addPropertyChangeListener(this);
			super.activate();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		if (isActive()) {
			JoinRelationLabel child = ((JoinRelation) getCastedModel())
					.getLabel();
			child.removePropertyChangeListener(this);
			super.deactivate();
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if (JoinRelation.WEAVING_PRIORITY_PROP.equals(property) ||
			JoinRelation.DECISION_SET_PROP.equals(property) ||
			JoinRelation.WEAVING_TYPE_PROP.equals(property)){
			
			refresh();
		} else if (JoinRelation.DECISION_ABSTRACTED.equals(property)) {
			refresh();
		} else if (ConnectableElementPropNames.SOURCE_CONNECTIONS_PROP
				.equals(property)) {
			refreshSourceConnections();
		} else if (ConnectableElementPropNames.TARGET_CONNECTIONS_PROP
				.equals(property)) {
			refreshTargetConnections();
		} else {
			super.propertyChange(event);
		}
	}

	/**
	 * Returns the casted model of the Join Relationship.
	 * @return the underlying Join Relationship model. 
	 */
	public JoinRelation getCastedModel() {
		return (JoinRelation) getModel();
	}

	/**
	 * Returns the casted Join Relationship figure.
	 * @return the figure, associated with this edit part.
	 */
	public JoinRelationFigure getCastedFigure() {
		return (JoinRelationFigure) getFigure();
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createChild(java.lang.Object)
	 */
	public EditPart createChild(Object model) {
		JoinRelationLabelEditPart part = null;
		part = new JoinRelationLabelEditPart();
		part.setModel(model);
		return part;
	}

	/**
	 * Returns the JoinRelationLabel if its content is unequal the default content.
	 * @return Returns a {@link java.util.List} containing the model children of this
	 *          Join Relationship.
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	public List getModelChildren() {
		List<JoinRelationLabel> children = new ArrayList<JoinRelationLabel>();
		JoinRelationLabel lbl = ((JoinRelation) getCastedModel()).getLabel();
		if (getModel().getPriority() != JoinRelation.noPriority ||
				getModel().getWeavingType() != JoinRelation.noType ||
				getModel().getDecision() != null ||
				getModel().getAbstractedDecNrs() != null ) {
			
			children.add(lbl);
		}
		return children;
	}
	
	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.LabeledConnectionEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();
		// Selection handle edit policy.
		// Makes the connection show a feedback, when selected by the user.
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ConnectionEndpointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE,
				new JoinRelationEditPolicy(this));
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new ConstrainableConnectionPolicy());
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		JoinRelationFigure f =  new JoinRelationFigure();
		
		f.setComment(getCastedModel().getComment());
        		
		return f;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModel()
	 */
	@Override
	public JoinRelation getModel() {
		return (JoinRelation) super.getModel();
	}

	/**
	 * Returns the IPropertySource for this ModelElement.
	 * @pre       true
	 * @post      return != null
	 * @param adapter The adapter class object.
	 * @return Returns the IPropertySource for this ModelElement
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (joinRelationProps == null) {
				joinRelationProps = new JoinRelationProperties(getCastedModel());
			}
			return joinRelationProps;
		}
		return null;
	}
}