package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.DropRequest;
import org.eclipse.jface.viewers.StructuredSelection;

import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties;
import ch.unizh.ifi.adora.utils.ModelElementVisibilityFilter;
import ch.unizh.ifi.adora.utils.filteredlist.FilteredArrayList;
import ch.unizh.ifi.adora.view.anchors.ClickPointAnchor;

/**
 * EditPart used for Node instances.
 * <p>
 * This edit part must implement the PropertyChangeListener interface, so it can
 * be notified of property changes in the corresponding model element.
 * </p>
 * 
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			31.08.2006 SM First Version
 * 						20.07.2006 SM Comments checked and corrected.
 * 									  Some refactoring.
 * 						11.08.2006 JS Some refactoring.
 * 						14.08.2006 JS IZoomableFigures are relayouted after
 * 									  zoom events
 * 						15.08.2006 JS getCastedModel() overriden, unnecessary 
 * 									  casts removed
 * @version			$Date: 2007-10-24 10:19:05 +0200 (Wed, 24 Oct 2007) $, $Author: treinhard $, $Revision: 3164 $
 *
 */
public abstract class NodeEditPart extends CommentableEditPart implements org.eclipse.gef.NodeEditPart {
	
	/**
	 * Creates a new connection anchor.
	 * 
	 * @return Returns the newly created connection anchor.
	 */
	protected abstract ConnectionAnchor createConnectionAnchor();

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		FilteredArrayList list = new FilteredArrayList((ArrayList)getCastedModel().getSourceConnections());
        list.setFilter(new ModelElementVisibilityFilter());
     	return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		FilteredArrayList list = new FilteredArrayList((ArrayList)getCastedModel().getTargetConnections());
		list.setFilter(new ModelElementVisibilityFilter());
        return list;
    }

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(
			ConnectionEditPart connection) {
		
		ConnectionAnchor anchor = createConnectionAnchor();
		if(anchor instanceof ClickPointAnchor) {
			float xRatio = ((Connection)connection.getModel()).getSourceXRatio();
			float yRatio = ((Connection)connection.getModel()).getSourceYRatio();
			((ClickPointAnchor)anchor).setClickPoint(xRatio, yRatio);
		}
		
		return anchor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		ConnectionAnchor anchor = createConnectionAnchor();
		
		if (anchor instanceof ClickPointAnchor) {
            Point clickPoint = ((DropRequest) request).getLocation().getCopy();
            IFigure f = getFigure();
            f.translateToRelative(clickPoint);
            f.translateFromParent(clickPoint);
            clickPoint.translate(getFigure().getClientArea().getLocation()
                    .getNegated());
            // Correct the coordinates if the figure has a border (e.g. for the
            // label)
            if (getFigure().getBorder() != null) {
                Insets insets = getFigure().getBorder().getInsets(getFigure());
                clickPoint.translate(insets.left, insets.top);
            }
            ((ClickPointAnchor) anchor).setClickPoint(clickPoint);
        }
		
		return anchor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(
			ConnectionEditPart connection) {
		
		ConnectionAnchor anchor = createConnectionAnchor();
		if(anchor instanceof ClickPointAnchor) {
			float xRatio = ((Connection)connection.getModel()).getTargetXRatio();
			float yRatio = ((Connection)connection.getModel()).getTargetYRatio();		
			((ClickPointAnchor)anchor).setClickPoint(xRatio, yRatio);
		}
		
		return anchor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		ConnectionAnchor anchor = createConnectionAnchor();

        if (anchor instanceof ClickPointAnchor) {
            Point clickPoint = ((DropRequest) request).getLocation().getCopy();
            IFigure f = getFigure();
            f.translateToRelative(clickPoint);
            f.translateFromParent(clickPoint);
            clickPoint.translate(getFigure().getClientArea().getLocation()
                    .getNegated());
            // Correct the coordinates if the figure has a border (e.g. for the
            // label)
            if (getFigure().getBorder() != null) {
                Insets insets = getFigure().getBorder().getInsets(getFigure());
                clickPoint.translate(insets.left, insets.top);
            }
            ((ClickPointAnchor) anchor).setClickPoint(clickPoint);
        }

        return anchor;
	}
	
	/**
	 * This method refreshes all connections of this node.
	 * @see {@link #refreshAllConnections(NodeEditPart)}
	 */
    public void refreshAllConnections(){
        refreshAllConnections(this);
    }
    
    /**
     * This method refreshes all connections. This method 
     * has to be called after turning one node with ingoing
     * or outgoing connections invisible or zooming a node out.
     * In this case dangling connections from the edit part
     * may result and therefore, the connections of the editparts 
     * have to be refreshed, so that connections from invisble nodes
     * are discarded.
     * 
     * @pre       	np != null
     * @post      	true 
     * @param np	The NodeEditPart which has to refresh the 
     *              connections of itself and of all visible children.
     */
    protected void refreshAllConnections(NodeEditPart np) {
		np.refreshSourceConnections();
		np.refreshTargetConnections();
		Iterator iter = np.getChildren().iterator();
		while(iter.hasNext()) {
			NodeEditPart child = (NodeEditPart)iter.next();
			refreshAllConnections(child);
		}
	}

	/**
	 * Returns the casted model element which is associated with
	 * this edit part.
	 * @return Returns the casted model element.
	 */
	protected Node getCastedModel() {
		return (Node) getModel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		if(ConnectableElement.SET_ELEMENT_SELECTED_PROP.equals(prop)){
			if(evt.getNewValue().equals(false)){ //no clearing, add this node
			List selection=this.getViewer().getSelectedEditParts();
			Vector selvec=new Vector(selection);
			selvec.add(this);
			this.getViewer().setSelection(new StructuredSelection(selvec.toArray()));
			}
			else{//clear selected node, show this node
			this.getViewer().deselectAll();
			this.getViewer().select(this);
			}
			this.getViewer().reveal(this);
		}
		else if (ZoomableNodeContainer.ZOOMED_PROP.equals(prop)) {			
			refreshChildren();
			Iterator iter = this.getRoot().getChildren().iterator();
			while(iter.hasNext()) {
				AdoraModelEditPart ep = (AdoraModelEditPart)iter.next();
				Iterator iter2 = ep.getChildren().iterator();
				while(iter2.hasNext()) {
					NodeEditPart np = (NodeEditPart)iter2.next();
					refreshAllConnections(np);
				}				
			}
			((INodeContainerProperties)getFigure()).setZoomed(((Boolean)evt.getNewValue()).booleanValue());
		}else if (Node.SOURCE_CONNECTIONS_PROP.equals(prop)) {
            refreshSourceConnections();
		} else if (Node.TARGET_CONNECTIONS_PROP.equals(prop)) {
			refreshTargetConnections();
        }  else if (VisualizableModelElement.VISIBILITY_PROP.equals(prop)) {
        	refreshAllConnections(); //updates connection visibility of con
        				//associated view hidden/shown nodes.
        	refresh(); // shows hidden pasted nodes, after filter is disabled.
        } else{
            super.propertyChange(evt);
        }
	}
}