package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.directedit.NodeCellEditorLocator;
import ch.unizh.ifi.adora.parts.directedit.NodeEditManager;
import ch.unizh.ifi.adora.parts.policies.FisheyeZoomEditPolicy;
import ch.unizh.ifi.adora.parts.policies.LabelDirectEditPolicy;
import ch.unizh.ifi.adora.parts.policies.containers.ScenarioContainerPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.ScenarioNodeEditPolicy;
import ch.unizh.ifi.adora.properties.ScenarioProperties;
import ch.unizh.ifi.adora.view.anchors.EllipseClickPointAnchor;
import ch.unizh.ifi.adora.view.figures.ScenarioFigure;

/**
 * 
 * The EditPart of a scenario.
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			27.09.2005 SM First Version
 * 						20.07.2006 SM Comments added, small refactoring.
 *						15.08.2006 JS getCastedModel() overridden, unnecessary 
 * 									  casts removed
 * @version			$Date: 2008-04-29 11:51:04 +0200 (Tue, 29 Apr 2008) $, $Author: treinhard $, $Revision: 3627 $
 *
 */
public class ScenarioEditPart extends NodeEditPart implements IAdaptable {

	  
	/**
	 * IPropertySource for this ModelElement. The IPropertySource is
	 * responsible for getting and setting property values of the 
	 * corresponding model element.
	 */
	ScenarioProperties scenarioProps;

	/**
	 * The direct manager which enables the direct editing
	 * capabilities for the label of this scenario node.
	 */
	private DirectEditManager manager;

	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createFigureForModel()
	 */
	protected IFigure createFigureForModel() {
		Scenario model = getCastedModel();
		ScenarioFigure p = new ScenarioFigure(model.getScenarioType(), 
				model.getSeqNo(), model.isIterator());
		p.setLayoutManager(new XYLayout());
		p.setText(model.getName());
        p.setZoomed(model.isZoomedIn());
        p.setComment(getCastedModel().getComment());
        p.setCalculatedPartial(getCastedModel().isCalculatedPartial());
        
        return p;
	}

	/**
	 * Returns the casted model element which is associated with
	 * this edit part.
	 * @return Returns the casted model element.
	 */
	protected Scenario getCastedModel() {
		return (Scenario) getModel();
	}

	/**
	 * Used in the {@link ch.unizh.ifi.adora.parts.policies.containers.ModulePolicy} 
	 * to determine if the a complete {@link Scenario}-Tree has been selected for moving.
	 * If so, the normal {@link Scenario}-constraints get invalid.
	 * 
	 * @return <li>true</li>, if all scenario children are selected.
	 */
	public boolean areAllScenarioChildrenSelected(){
		Iterator iter = getSourceConnections().iterator();
		ConnectionEditPart connection;
		while (iter.hasNext()) {
			connection = (ConnectionEditPart)iter.next();
			if (connection instanceof ScenarioConnectionEditPart){
				if (connection.getTarget().getSelected() > 0){
					return ((ScenarioEditPart)
						((ScenarioConnectionEditPart)connection).getTarget()).
						areAllScenarioChildrenSelected();
				} else {
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	 * Used to check policy if one is moving a {@link Scenario} sub tree down the hierarchy: 
	 * All {@link Scenario} children of the selected {@link Scenario}s have to be selected too, 
	 * until {@link Scenario}s are found which are situated in the target {@link ZoomableNodeContainer}  
	 * or in a child of it. (These don't have to be selected)
	 * 
	 * @param container The {@link INodeContainer} which is used to determine the 
	 * hierarchical limit down to whom all {@link Scenario}s have to be checked. 
	 * 
	 * @return <li>true</li> if all {@link Scenario} children of this {@link Scenario}
	 * down to the {@link Scenario}s which are situated in the <li>container</li> - 
	 * or in a child of it - are selected. 
	 */
	public boolean areAllScenarioChildrenSelectedDownToContainer(INodeContainer container){
		Iterator iter = getSourceConnections().iterator();
		ConnectionEditPart connection;
		
		if (container == this.getParent().getModel() ||
				((ZoomableNodeContainer)container).
				hasChildRecursively((ZoomableNodeContainer)getParent().getModel())) {
			return true;
		} 
		
		while (iter.hasNext()) {
			connection = (ConnectionEditPart)iter.next();
			if (connection instanceof ScenarioConnectionEditPart) {
				
				if (((ZoomableNodeContainer)container).
						hasChildRecursively((ZoomableNodeContainer)connection.getTarget().
										getParent().getModel()) ||
					connection.getTarget().getParent().getModel() == container) {
			
					return true;
				} else if (connection.getTarget().getSelected() > 0) {
					return ((ScenarioEditPart)
						((ScenarioConnectionEditPart)connection).getTarget()).
						areAllScenarioChildrenSelectedDownToContainer(container);
				} else 
					return false;
			}
		}
		return true;
	}
	
	/**
	 * Used to check policy if one is moving a {@link Scenario} sub tree up the hierarchy: 
	 * All {@link Scenario} parents of the selected {@link Scenario}s have to be selected too, 
	 * until {@link Scenario}s are found which are situated in the target {@link ZoomableNodeContainer}  
	 * or in a parent of it. (These don't have to be selected)
	 * 
	 * @param container The {@link INodeContainer} which is used to determine the 
	 * hierarchical limit up to whom all {@link Scenario}s have to be checked. 
	 * 
	 * @return <li>true</li> if all {@link Scenario} children of this {@link Scenario}
	 * down to the {@link Scenario}s which are situated in the <li>container</li> - 
	 * or in a child of it - are selected. 
	 */
	public boolean areAllScenarioParentsSelectedUpToContainer(INodeContainer container){
		
		if (getParent().getModel() == container ||
				((ZoomableNodeContainer)getParent().getModel()).hasChildRecursively((ModelElement)container)){
			return true;
		}
		
		Iterator iter = getTargetConnections().iterator();
		ConnectionEditPart connection;
		while (iter.hasNext()) {
			connection = (ConnectionEditPart)iter.next();
			if (connection instanceof ScenarioConnectionEditPart){
				if(connection.getSource().getParent().getModel() == container ||
						((ZoomableNodeContainer)connection.getSource().getParent().getModel()).
								hasChildRecursively((ModelElement)container)) {
					return true;
				}
				else if (connection.getSource().getSelected() > 0){
					return ((ScenarioEditPart)
						((ScenarioConnectionEditPart)connection).getSource()).
						areAllScenarioParentsSelectedUpToContainer(container);
				} else {
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	 * Used to determine the root of a {@link Scenario} tree.
	 * 
	 * @return The root {@link Scenario}. 
	 */
	public ScenarioEditPart getRootScenario(){
		Iterator iter = getTargetConnections().iterator();
		ConnectionEditPart connection;
		while (iter.hasNext()) {
			connection = (ConnectionEditPart)iter.next();
			if (connection instanceof ScenarioConnectionEditPart){
				return ((ScenarioEditPart)connection.getSource()).getRootScenario();
			}
		}
		return this;
	}
	
	/**
	 * Used in the {@link ch.unizh.ifi.adora.parts.policies.containers.ModulePolicy} 
	 * to determine which {@link ScenarioEditPart}s are selected.
	 * 
	 * @return List, containing all selected {@link ScenarioEditPart}s which are children
	 * of this {@link ScenarioEditPart}.
	 */
	public List<ScenarioEditPart> getAllSelectedSenarioChildren(){
		List<ScenarioEditPart> selectedParts = new ArrayList<ScenarioEditPart>();
		
		if (getSelected() > 0){
			selectedParts.add(this);
		}
		
		Iterator iter = getSourceConnections().iterator();
		ConnectionEditPart connection;
		
		while (iter.hasNext()) {
			connection = (ConnectionEditPart)iter.next();
			if (connection instanceof ScenarioConnectionEditPart){
					 selectedParts.addAll(((ScenarioEditPart)connection.getTarget()).getAllSelectedSenarioChildren());
			}
		}
		return selectedParts;
	}
	
	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.NodeEditPart#createConnectionAnchor()
	 */
	protected ConnectionAnchor createConnectionAnchor() {
		return new EllipseClickPointAnchor(getFigure());
	}

	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
        
    
		super.createEditPolicies();
		// allow the creation of connections and
		// and the reconnection of connections between Shape instances
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
		        new ScenarioNodeEditPolicy());
		
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, 
				new LabelDirectEditPolicy());
		installEditPolicy(FisheyeZoomEditPolicy.FISHEYE_ZOOM_ROLE,
                new FisheyeZoomEditPolicy());
       
        installEditPolicy(EditPolicy.CONTAINER_ROLE,
                new ScenarioContainerPolicy());
	}
	
	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.NodeEditPart#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
        ScenarioFigure f = (ScenarioFigure)getFigure();
        if (Scenario.NAME_PROP.equals(prop)) {
			f.setText(getCastedModel().getName());
			f.layout();
		}else if (Scenario.TYPE_PROP.equals(prop)) {
			f.setType(getCastedModel().getScenarioType());
			f.repaint();
		}else if (Scenario.SEQNR_PROP.equals(prop)) {
			f.setSeqNr(getCastedModel().getSeqNo());
			f.repaint();
		}else if (Scenario.ITERATOR_PROP.equals(prop)) {
			f.setIteraton(getCastedModel().isIterator());
			f.repaint();
		}else if (Scenario.ZOOMED_SCENARIO_PROP.equals(prop)) {
            f.setZoomed(getCastedModel().isZoomedIn());
            f.layout();
        }else if(ZoomableNodeContainer.ISPARTIAL_PROP.equals(prop)){
            f.setPartial(getCastedModel().isPartial());
            f.layout();
            f.repaint();
        } else if (VisualizableModelElement.VISIBILITY_PROP.equals(prop)) {

			f.setCalculatedPartial(getCastedModel().isCalculatedPartial());
			f.layout();
            f.repaint();
			super.propertyChange(evt);
         
        }else{
            super.propertyChange(evt);
        }
	}
	

	/**
	 * Performs the direct editing by instantiating a 
	 * corresponding node edit manager.
	 */
	private void performDirectEdit(){
		if(manager == null)
			manager = new NodeEditManager(
					this, new NodeCellEditorLocator(((ScenarioFigure)getFigure()).getNameLabel()));
		manager.show();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request request){
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
			performDirectEdit();
	}
	
	/**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
	public Object getAdapter(Class adapter) {
		 if (adapter == IPropertySource.class) {
			if (scenarioProps == null) {
				scenarioProps = new ScenarioProperties(getCastedModel());
			}
			return scenarioProps;
		 }
		 return null;
	}
}