package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.TransitionDescription;
import ch.unizh.ifi.adora.parts.policies.connections.TransitionEditPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.TransitionNodeEditPolicy;
import ch.unizh.ifi.adora.properties.TransitionProperties;
import ch.unizh.ifi.adora.utils.ModelElementVisibilityFilter;
import ch.unizh.ifi.adora.utils.filteredlist.FilteredArrayList;
import ch.unizh.ifi.adora.view.figures.TransitionFigure;

/**
 * EditPart for a Transition. 
 * This EdiPart is also responsible for creating the TransitionDescription.      
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			27.09.2005 SM First Version
 * 						20.07.2006 SM Comments added, checked and completed.
 * 						14.08.2006 JS Minor refactoring
 * @version			$Date: 2008-11-27 14:11:37 +0100 (Thu, 27 Nov 2008) $, $Author: treinhard $, $Revision: 3905 $
 */
public class TransitionEditPart extends LabeledConnectionEditPart implements
		IAdaptable {

	/**
	 * IPropertySource for this ModelElement. This 
	 * IPropertySource is responsible for the setting and 
	 * getting of property values of the corresponding model
	 * element. 
	 */
	private TransitionProperties transitionProps;

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		figure.repaint();
		super.refreshVisuals();
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate() {
		if (!isActive()) {
			TransitionDescription child = ((Transition) getCastedModel())
					.getLabel();
			child.addPropertyChangeListener(this);
			super.activate();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		if (isActive()) {
			TransitionDescription child = ((Transition) getCastedModel())
					.getLabel();
			child.removePropertyChangeListener(this);
			super.deactivate();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if (TransitionDescription.NAME_PROP.equals(property)) {
			refresh();
		} else {
			super.propertyChange(event);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		// Allows the removal of the connection model element
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new TransitionEditPolicy(
				this));

		//Added to allow connection of connections
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new TransitionNodeEditPolicy());
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createChild(java.lang.Object)
	 */
	public EditPart createChild(Object model) {
		TransitionLabelEditPart part = null;
		part = new TransitionLabelEditPart();
		part.setModel(model);
		return part;
	}

	/**
	 * Returns the TransitionLabel if its content 
	 * is unequal the default content.
	 * @param returns the model children in a {@link java.util.List}
	 * object. 
	 */
	public List getModelChildren() {
		List<TransitionDescription> children = new ArrayList<TransitionDescription>();
		TransitionDescription lbl = getCastedModel().getLabel();
		if (!lbl.getName().equalsIgnoreCase("")) {
			children.add(lbl);
		}
		return children;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		TransitionFigure connection = new TransitionFigure();
		connection.setComment(getCastedModel().getComment());
		return connection;
	}

	/**
	 * Returns the casted transition model element.
	 * @return The casted Transition model.
	 */
	public Transition getCastedModel() {
		return (Transition) getModel();
	}

	/**
	 * Returnst the casted transition figure.
	 * @return The casted Transition figure.
	 */
	public TransitionFigure getCastedFigure() {
		return (TransitionFigure) getFigure();
	}

	/**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (transitionProps == null) {
				transitionProps = new TransitionProperties(
						(Transition) getCastedModel());
			}
			return transitionProps;
		}
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		FilteredArrayList list = new FilteredArrayList((ArrayList)getCastedModel().getSourceConnections());
        list.setFilter(new ModelElementVisibilityFilter());
     	return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		FilteredArrayList list = new FilteredArrayList((ArrayList)getCastedModel().getTargetConnections());
		list.setFilter(new ModelElementVisibilityFilter());
        return list;
    }
}