package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.parts.geometryfactory.GeometryFactory;
import ch.unizh.ifi.adora.parts.policies.AdoraComponentEditPolicy;

/**
 * This is a abstract superclass for all elements, which can be displayed.
 * 
 * @author				Silvio Meier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				27.09.2005 SM First Version
 * 						25.07.2006 SM Comments checked, corrected.
 * @version				$Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 */

public abstract class VisualizableModelElementEditPart extends AbstractGraphicalEditPart implements PropertyChangeListener{

    /**
	 * Upon activation, attach to the model element as a property change
	 * listener.
	 */
	public void activate() {
		if (!isActive()) {
			super.activate();
			((ObservableModelElement) getModel()).addPropertyChangeListener(this);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// allow removal of the associated model element
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new AdoraComponentEditPolicy());
    }

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		IFigure f = createFigureForModel();
		f.setOpaque(true); // non-transparent figure
		f.setBackgroundColor(ColorConstants.white);
		return f;
	}

	/**
	 * Return a IFigure depending on the instance of the current model element.
	 * This allows this EditPart to be used for both sublasses of Shape.
	 * @return Returns the figure which was created for this
	 *          edit part.
	 */
	protected abstract IFigure createFigureForModel();


	/**
	 * Upon deactivation, detach from the model element as a property change
	 * listener.
	 */
	public void deactivate() {
		if (isActive()) {
			((ObservableModelElement) getModel()).removePropertyChangeListener(this);
			super.deactivate();
		}
	}

	/**
	 * Returns the casted model element which is associated with
	 * this edit part.
	 * @return Returns the casted model element.
	 */
	protected VisualizableModelElement getCastedModel() {
		return (VisualizableModelElement) getModel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		
		if (Node.SIZE_PROP.equals(prop) || Node.LOCATION_PROP.equals(prop)) {
			refreshVisuals();
		}
        else if (Node.NAME_PROP.equals(prop)) {
			((INodeProperties)getFigure()).setText((String)evt.getNewValue());
			refreshVisuals();
		} 
	}
	
	/**
	 *  Notifies the parent container of changed position & dimension.
	 *  
     *  If this method hadn't been overwritten, 
     *  the {@link org.eclipse.draw2d.XYLayout}-Manager used by the parent's 
     *  container wouldn't know the bounds of this {@link IFigure} 
     *  and therefore wouldn't draw it correctly.
     */    
	protected void refreshVisuals() {
        Rectangle bounds = new Rectangle(getCastedModel().getLocation(),
				getCastedModel().getSize());
        ((GraphicalEditPart) getParent()).setLayoutConstraint(this,
				getFigure(), GeometryFactory.getDraw2dElement(bounds));
	}
    
    /**
     * Only elements which are visible can be selected by the user.
     */
    protected void fireSelectionChanged() {
        super.fireSelectionChanged();
        if(!getCastedModel().isVisible()){
            setSelected(0);
        }
    }
}