package ch.unizh.ifi.adora.parts.directedit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.CellEditorActionHandler;

import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.view.figures.label.WordWrappingLabel;

/**
 * Manages the hiding of the text displayed on the node during direct editing.
 *      
 * @author				Tobias Reinhard
 * 						Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				16.03.2006 SM First Version
 *						22.07.2006 SM Comments reformatted.
 *						25.07.2006 SM Comments checked.
 * @version				$Date: 2006-08-17 16:36:14 +0200 (Thu, 17 Aug 2006) $, $Author: vigan $, $Revision: 2073 $
 */
public class NodeEditManager extends DirectEditManager {

	/**
	 * Used for hooking the cell editor's copy/paste actions to the actionBars so that
	 * they can be invoked via keyboard shortcuts.
	 */
	private IActionBars actionBars;

	/**
	 * Used for handling the redirection of the global actions Cut, Copy, Paste.
	 */
	private CellEditorActionHandler actionHandler;

	/**
	 * Actions to be set as the GlobalActionHandler.
	 */
	private IAction copy, cut, paste, undo, redo, find, selectAll, delete;

	/**
	 * Font used while editing.
	 */
	private Font scaledFont;
	
    /**
     * Node on which the Direct Editing Component appears.
     */
	private IFigure label;
	
 	/**
	 * Locator used to determine the location of the direct editor.
	 */
    private NodeCellEditorLocator locator;
    
    /**
     * Constructs a new DirectEditManager for the given source edit part. 
     * The cell editor will be placed using the given CellEditorLocator.
     * 
     * @param source EditPart, on which this editor will be placed. 
     * @param locator The locator which plaes the direct editing field correctly.
     */
	public NodeEditManager(GraphicalEditPart source,
			NodeCellEditorLocator locator) {
		super(source, null, locator);
        this.locator = locator;
		label = ((NodeCellEditorLocator)locator).getLabel();
	}
	
	/**
	 * Don't show a border around the label, therefore override method
	 * with a void implementation.
	 */
	public void showFeedback() {
	}

	/**
     * Displayes the hidden text after the editing occured and
     * if nothing changed.
     * 
	 * @see org.eclipse.gef.tools.DirectEditManager#bringDown()
	 */
    protected void bringDown() {
		if (actionHandler != null) {
			actionHandler.dispose();
			actionHandler = null;
		}
		if (actionBars != null) {
			restoreSavedActions(actionBars);
			actionBars.updateActionBars();
			actionBars = null;
		}
		
		Font disposeFont = scaledFont;
		scaledFont = null;
		super.bringDown();
		if (disposeFont != null)
			disposeFont.dispose();
		
		((WordWrappingLabel)label).show();
		
	}
    
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.tools.DirectEditManager#createCellEditorOn(org.eclipse.swt.widgets.Composite)
	 */
	protected CellEditor createCellEditorOn(Composite composite) {
		TextCellEditor editor = new TextCellEditor(composite, SWT.SINGLE | SWT.WRAP);
		return editor;
	}

    /**
     * 
     * Hides the text displayed in the node during editing.
     * @see org.eclipse.gef.tools.DirectEditManager#initCellEditor()
     */
    protected void initCellEditor() {

		Text text = (Text) getCellEditor().getControl();
		INodeProperties node = (INodeProperties) getEditPart().getFigure();
		String initialLabelText = node.getText();
		getCellEditor().setValue(initialLabelText);
		scaledFont = label.getFont();
		FontData data = scaledFont.getFontData()[0];
		Dimension fontSize = new Dimension(0, data.getHeight());
		((IFigure) node).translateToAbsolute(fontSize);
		data.setHeight(fontSize.height);
		scaledFont = new Font(null, data);
		text.setFont(scaledFont);
		text.selectAll();

		// Hook the cell editor's copy/paste actions to the actionBars so that
		// they can
		// be invoked via keyboard shortcuts.
		actionBars = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getActivePage().getActiveEditor().getEditorSite()
				.getActionBars();
		saveCurrentActions(actionBars);
		actionHandler = new CellEditorActionHandler(actionBars);
		actionHandler.addCellEditor(getCellEditor());
		actionBars.updateActionBars();

		((WordWrappingLabel)label).hide();
		
  	}

    /**
     * This method is used for restoring the global actions 
     * which were overwritten while direct editing. 
     * Therefore the global actions have to be read from this class' action 
     * variables and set again in the <li>actionBars</li>.
     * 
     * @param actionBars ActionBar which sets the glbobal actions.
     */
 	 private void restoreSavedActions(IActionBars actionBars) {
		actionBars.setGlobalActionHandler(ActionFactory.COPY.getId(), copy);
		actionBars.setGlobalActionHandler(ActionFactory.PASTE.getId(), paste);
		actionBars.setGlobalActionHandler(ActionFactory.DELETE.getId(), delete);
		actionBars.setGlobalActionHandler(ActionFactory.SELECT_ALL.getId(),
				selectAll);
		actionBars.setGlobalActionHandler(ActionFactory.CUT.getId(), cut);
		actionBars.setGlobalActionHandler(ActionFactory.FIND.getId(), find);
		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undo);
		actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redo);
	}

   /**
    * This method is used for hooking up the Actions used while direct editing
    * such copy/paste/cut. Therefore the global actions have to be saved into 
    * variables.
    * 
    * @param actionBars ActionBar which sets the local actions.
    */
	private void saveCurrentActions(IActionBars actionBars) {
		copy = actionBars.getGlobalActionHandler(ActionFactory.COPY.getId());
		paste = actionBars.getGlobalActionHandler(ActionFactory.PASTE.getId());
		delete = actionBars
				.getGlobalActionHandler(ActionFactory.DELETE.getId());
		selectAll = actionBars.getGlobalActionHandler(ActionFactory.SELECT_ALL
				.getId());
		cut = actionBars.getGlobalActionHandler(ActionFactory.CUT.getId());
		find = actionBars.getGlobalActionHandler(ActionFactory.FIND.getId());
		undo = actionBars.getGlobalActionHandler(ActionFactory.UNDO.getId());
		redo = actionBars.getGlobalActionHandler(ActionFactory.REDO.getId());
	}

	/**
	 * Used to set the the label in the locator. 
	 * 
	 * @param label The label which gets located.
	 */
    public void setLabel(IFigure label) {
    	this.label = label;
        locator.setLabel(label);
    }
}