package ch.unizh.ifi.adora.parts.geometryfactory;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;


/**
 * This class operates as a factory and is responisble for converting the 
 * geometry types used in the model,to the geometry types used by draw2d
 * and vice versa.  
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			19.04.2006 SM First Version
 * 						22.07.2006 SM Comments reformatted, checked, completed and corrected.
 * @version			$Date: 2006-07-23 17:40:21 +0200 (Sun, 23 Jul 2006) $, $Author: smeier $, $Revision: 1887 $
 */
public class GeometryFactory {
    
    /**
     * Converts an Adora list of points into an draw2d list of points.
     * 
     * @pre		adoraPoints != null
     * @post	return != null
     * @param	adoraPoints List containing 
     *          {@link ch.unizh.ifi.adora.model.geometry.Point} 
     *          to convert.
     * @return List containing {@link org.eclipse.draw2d.geometry.Point} 
     *          objects.
     */
    public static List<org.eclipse.draw2d.geometry.Point> getGefPoints(List<Point> adoraPoints){
        List<org.eclipse.draw2d.geometry.Point> gefPoints = new ArrayList<org.eclipse.draw2d.geometry.Point>();
        for(Point adoraPoint : adoraPoints){
            gefPoints.add((org.eclipse.draw2d.geometry.Point)getDraw2dElement(adoraPoint));
        }
        return gefPoints;
    }
    
    /**
     * Converts an ADORA geometry class into an draw2d geometry class.
     * 
     * @pre			true
     * @post		true
     * @param		modelGeometry An Adora geometry object which will 
     * 				be converted. If modelGeometry is null, null is returned.
     *  
     * @returns 	Returns the corresponding draw2d geometry object.
     *				If no valid adora geometry object is given, an
     *				exception is thrown. Returns null if the argument is
     *				null. 
     *
     * @throws 	java.lang.IllegalArgumentException is thrown if no
     * 				valid adora geometry object is given.
     */
    public static Object getDraw2dElement(Object modelGeometry) {     
    	if (modelGeometry == null) {
    		return null;
    	}
    	if (modelGeometry instanceof Dimension) {
            Dimension d = (Dimension)modelGeometry;
            return new org.eclipse.draw2d.geometry.Dimension(d.width,d.height);
        }
        
        if (modelGeometry instanceof Point) {
            Point p = (Point)modelGeometry;
            return new org.eclipse.draw2d.geometry.Point(p.x,p.y);
        }
        
        if (modelGeometry instanceof Rectangle) {
            Rectangle r = (Rectangle)modelGeometry;
            return new org.eclipse.draw2d.geometry.Rectangle(r.x,r.y,r.width,r.height);
        }
        
        throw new IllegalArgumentException("Can't convert geometry objects: "
                + ((modelGeometry != null) ? modelGeometry.getClass().getName()
                        : "null"));
    }

    /**
     * Converts an draw2d geometry object into an Adora geometry object.
     * 
     * @pre   true
     * @post  true 
     * @param gefGeometry  A draw2d geometry object which will 
     * 						be converted. Returns null if gefGeometry is
     * 						null. 
     * @return Returns the corresponding Adora geometry object.
     *			If no valid adora geometry object is given, an
     *			exception is thrown. Returns null if the argument is
     *			null.
     * @throws	java.lang.IllegalArgumentException is thrown if no
     * 			valid draw2d geometry object is given.     
     */
    public static Object getAdoraElement(Object gefGeometry) {
    	if (gefGeometry == null) {
    		return null;
    	}
        if (gefGeometry instanceof org.eclipse.draw2d.geometry.Dimension) {
            org.eclipse.draw2d.geometry.Dimension d = (org.eclipse.draw2d.geometry.Dimension)gefGeometry;
            return new Dimension(d.width,d.height);
        }
        if (gefGeometry instanceof org.eclipse.draw2d.geometry.Point) {
            org.eclipse.draw2d.geometry.Point p = (org.eclipse.draw2d.geometry.Point)gefGeometry;
            return new Point(p.x,p.y);
        }
        if (gefGeometry instanceof org.eclipse.draw2d.geometry.Rectangle) {
            org.eclipse.draw2d.geometry.Rectangle r = (org.eclipse.draw2d.geometry.Rectangle)gefGeometry;
            return new Rectangle(r.x,r.y,r.width,r.height);
        }
        throw new IllegalArgumentException("Can't convert geometry objects: "
                + ((gefGeometry != null) ? gefGeometry.getClass().getName()
                        : "null"));
    }
}
