package ch.unizh.ifi.adora.parts.policies;

import org.eclipse.draw2d.Shape;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.AbstractEditPolicy;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.ExplosiveZoom;
import ch.unizh.ifi.adora.model.commands.FisheyeZoom;
import ch.unizh.ifi.adora.model.commands.ScenarioZoom;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.NodeEditPart;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.utils.RequestConstants;

/**
 * EditPolicy for fisheye zooming. Must only be installed on
 * {@link ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer}s. Only understands
 * {@link org.eclipse.gef.Request}s of type {
 * 
 * @author 		Adrian Gygax
 * @copyright 		Requirements Engineering Research Group<br>
 *					Department of Informatics<br>
 *					University of Zurich<br>
 * @history 		07.12.2005 AG First Version
 * 					23.07.2006 SM Comments reformatting.
 * @version 		$Date: 2008-11-07 11:03:19 +0100 (Fri, 07 Nov 2008) $, $Author: treinhard $, $Revision: 3869 $
 */
public class FisheyeZoomEditPolicy extends AbstractEditPolicy {

	/**
	 * Role for fisheye zooming capable containers.
	 */
	public static final String FISHEYE_ZOOM_ROLE = "FisheyeZoomRole";

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPolicy#getCommand(org.eclipse.gef.Request)
	 */
	public Command getCommand(Request request) {
		if (RequestConstants.REQ_FISHEYEZOOM.equals(request.getType())) {
			return getFisheyeZoomCommand();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPolicy#getTargetEditPart(org.eclipse.gef.Request)
	 */
	public EditPart getTargetEditPart(Request request) {
		return getHost();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPolicy#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		return RequestConstants.REQ_FISHEYEZOOM.equals(req.getType());
	}

	/**
	 * Returns a proper command object if zooming is allowed. Zooming is only
	 * allowed on containers with children.
	 * 
	 * @return The command or null if zooming is not allowed.
	 */
	private Command getFisheyeZoomCommand() {
		Command command = null;
		if (getHost().getModel() instanceof ZoomableNodeContainer) {
			final ZoomableNodeContainer container = (ZoomableNodeContainer) getHost()
					.getModel();
            
            
            if (container.hasChildren()) {
                
            	// Imitate an explosive zoom (for the zoom validation)
            	if(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_EXPLOSIVE_ZOOM)) {
            		
            		ExplosiveZoom explZoom = new ExplosiveZoom();
            		explZoom.setTarget(container);
            		return explZoom;
            		
            	}
            	
            	// The node can't be zoomed if all its children are filtered out by
        		// the view mechanism.
        		boolean childrenFiltered = true;
        		for(ModelElement me: container.getChildren()) {
        			
        			boolean childFiltered = ((AdoraModel)container.getRootContainer()).
        											getViewFilterState(me.getClass());
        			
        			childrenFiltered = childrenFiltered && childFiltered;
        			
        		}
        		if(childrenFiltered) {
        			return null;
        		}
            	              
                FisheyeZoom fc = new FisheyeZoom();
				fc.setTarget(container);
				command = fc;
			}
		}
        else if(getHost().getModel() instanceof Scenario){
            //Zooming only possible if children exists in scenario hierarchy.
            if(!((Scenario)getHost().getModel()).getScenarioChildren().isEmpty()){
                ScenarioZoom sc = new ScenarioZoom((Scenario)getHost().getModel());
                command = sc;
            }
        }
		return command;           
	}
	
	
	/**
	 * Show the feedback for the fisheye zooming tool.
	 * 
	 * @param request the request to show the feedback for
	 */
    public void showTargetFeedback(Request request) {
    	
    	if(request.getType().equals(RequestConstants.REQ_FISHEYEZOOM)
    			&& getCommand(request) != null) {
    		
    		((Shape)((NodeEditPart)getHost()).getFigure()).setLineWidth(2);
    		
    	}
    	
    	super.showTargetFeedback(request);
    	
    }
    
    
	/**
	 * Hide the feedback of the fisheye zooming tool.
	 * 
	 * @param request the request to show the feedback for
	 */
    public void eraseTargetFeedback(Request request) {
    	
    	if(request.getType().equals(RequestConstants.REQ_FISHEYEZOOM)
    			&& getCommand(request) != null) {
    		
    		((Shape)((NodeEditPart)getHost()).getFigure()).setLineWidth(1);
    		
    	}
    	
    	super.eraseTargetFeedback(request);
    	
    }
 }