package ch.unizh.ifi.adora.parts.policies;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.NodeGroup;
import ch.unizh.ifi.adora.model.util.NodeGroups;

/**
 * This is a helper class for the policies.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 25.01.2008 SM First Version
 * @version $Date: 2009-03-25 12:35:26 +0100 (Wed, 25 Mar 2009) $, $Author: jehle $, $Revision: 4092 $
 */
public class JoinRelationPolicyHelper {

	/**
     * Private constructor, because this class only has static helper methods.
     */
	private JoinRelationPolicyHelper() {
	}
	
	/**
     * Checks, if a node is part of a behavior or scenario chunk.
     * 
     * @param source
     *            Node which have to be checked.
     * @return true if the node is part of a chunk.
     */
	private static boolean isInChunk(ConnectableElement source) {
		boolean chunk = false;

		NodeGroups nodeGroups = ((ZoomableNodeContainer) source.getParent())
				.getNodeGroups();

		for (NodeGroup nodeGroup : nodeGroups.getNodeGroups()) {
			if (nodeGroup.contains((Node) source)) {
				if (nodeGroup.hasOutgoingJoinRelation()) {
					/*
                     * this group is for sure a chunk group, because of its
                     * outgoing joinrelation.
                     */
					chunk = true;
				} else {
					/*
                     * this could be either a crosscutting group, or a chunk
                     * group which is not yet finished.
                     */
					if (nodeGroup.isScenarioGroup()) {
						/*
                         * a crosscutting scenariogroup has a rootnode with an
                         * actor connected. so search for this actor.
                         */
						boolean hasConnectionToActor = false;

						for (Node node : nodeGroup.getNodes()) {
							for (Connection connection : node.getConnections()) {
								if (connection instanceof Association) {
									Association association = (Association) connection;

									if (association.getSource() instanceof Actor
											|| association.getTarget() instanceof Actor) {
										hasConnectionToActor = true;
										break;
									}
								}
							}

							if (hasConnectionToActor) {
								break;
							}
						}

						if (!hasConnectionToActor) {
							chunk = true;
							break;
						}
					} else {
						/*
                         * a crosscutting stategroup is a group with a start
                         * state. so search for the start state.
                         */
						boolean hasStartState = false;

						for (Node node : nodeGroup.getNodes()) {
							if (node instanceof State) {
								State state = (State) node;
								if (state.isStartState()) {
									hasStartState = true;
									break;
								}
							}
						}

						if (!hasStartState) {
							chunk = true;
						}
					}
				}

				if (chunk) {
					break;
				}
			}
		}

		return chunk;
	}

	/**
     * Method, which checks if an element (state / scenario chunk) can have a
     * certain connection.
     * 
     * @param source
     *            The source node from which the joinrelation starts.
     * @param target
     *            The target node where the joinrelation goes to.
     * @return true, if this connection can be done.
     */
	public static boolean checkChunkCompletely(ConnectableElement source,
			ConnectableElement target) {
		/*
         * Check, if source and target are the same - no connection allowed.
         */
		if (source.equals(target)) {
			return false;
		}

		/*
         * Check, if the elements are on the same level (then no connection is
         * allowed, except the elements are aspects).
         */
		if (source.getParent().equals(target.getParent())
				&& !(source instanceof AspectContainer && target instanceof AspectContainer)) {
			return false;
		}

		/*
         * a join relationship may not cross the borders..
         */
		IContainer sourceParent = getSourceParent(source);

		ModelElement targetParent;

		if (target instanceof Connection) {
			targetParent = (ModelElement) ((Connection) target).getSource();
		} else {
			targetParent = target;
		}

		if (sourceParent.hasChildRecursively(targetParent)) {
			return isInChunk(source);
		} else {
			return false;
		}
	}

	public static IContainer getSourceParent(ConnectableElement source) {
		/*
         * a join relationship may not cross the borders..
         */
		IContainer sourceParent = source.getParent().getParent();

		/*
         * If the source element is in an aspect, we have to "go up" because
         * this is not a real border. we have to take the next component or the
         * adoramodel as a border.
         */
		while (sourceParent instanceof AspectContainer) {
			sourceParent = sourceParent.getParent();
		}

		return sourceParent;
	}
}
