package ch.unizh.ifi.adora.parts.policies.containers;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.EditPart;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.ScenarioEditPart;
import ch.unizh.ifi.adora.parts.StateEditPart;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * Determines which {@link ch.unizh.ifi.adora.model.representation.Node}s can
 * contain be contained by a
 * {@link ch.unizh.ifi.adora.model.representation.AspectContainer} .
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 02.05.2006 SM First Version 22.07.2006 IV Comments reformatted.
 * @version $Date: 2008-12-10 17:26:19 +0100 (Wed, 10 Dec 2008) $, $Author: vigan $, $Revision: 3959 $
 */
public class AspectContainerPolicy extends ModulePolicy {

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policy.ContainerXYLayoutEditPolicy#createCommandAllowed(ch.unizh.ifi.adora.model.representation.INodeContainer,
     *      ch.unizh.ifi.adora.model.representation.Node)
     */
	protected boolean createCommandAllowed(INodeContainer host, Node part) {
		/*
         * If an aspect contains aspect, the aspect may not contain other
         * elements than aspects as children. For instance, take the model of
         * the second attachment. The aspect A should not contain any other
         * elements than aspects. For example, A must not contain states, exit
         * points or scenario nodes. However, child aspects of A can have these
         * elements (or aspects).
         */
		boolean noAspects = false;
		boolean noOthers = false;

		for (ModelElement element : ((AspectContainer) getHost().getModel())
				.getChildren()) {
			if (element instanceof AspectContainer) {
				noOthers = true;
			} else {
				noAspects = true;
			}
		}

		if (noAspects && part instanceof AspectContainer) {
			return false;
		}

		if (noOthers && !(part instanceof AspectContainer)) {
			return false;
		}

		if (part instanceof ExitPoint) {
			return true;
		}

		return super.createCommandAllowed(host, part);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policy.ContainerXYLayoutEditPolicy#addCommandAllowed(ch.unizh.ifi.adora.model.representation.INodeContainer,
     *      ch.unizh.ifi.adora.model.representation.Node)
     */
	protected boolean addCommandAllowed(INodeContainer host, EditPart part) {
		Node model = (Node) part.getModel();

		if (model instanceof ZoomableNodeContainer
				&& hasJoinRelationToPart(host, model)) {
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.JOIN_RELATION_EXISTS);
			return false;
		}
		// only outermost container can have associations
		if (model instanceof AspectContainer
				&& ((AspectContainer) model).getAssociationsIterator()
						.hasNext()) {
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.ASSOCIATION_TO_INNER_ASPECT);
			return false;
		}
		return super.addCommandAllowed(host, part);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.containers.ZoomableNodeContainerPolicy#hasJoinRelationConflict(ch.unizh.ifi.adora.model.representation.INodeContainer,
     *      org.eclipse.gef.EditPart)
     */
	protected boolean hasJoinRelationConflict(INodeContainer host, EditPart part) {

		if (part instanceof StateEditPart) {
			for (Connection c : ((Node) part.getModel()).getSourceConnections()) {
				if (c instanceof JoinRelation) {
					if (host.hasChildRecursively(((Transition) c.getTarget())
							.getSource())) {
						return true;
					}
				}
			}
		} else if (part instanceof ScenarioEditPart) {
			for (Connection c : ((Node) part.getModel()).getSourceConnections()) {
				if (c instanceof JoinRelation) {
					if (host.hasChildRecursively(((Scenario) c.getTarget()))) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
     * Checks if the
     * <li>part</li>
     * which is added to the
     * <li>host</li>
     * contains an object with a {@link Transition}, which is crosscuted by a
     * {@link JoinRelation} orgiginating from a {@link State} of this
     * <li>host</li>.
     * 
     * @param host
     *            The {@link AspectContainer}.
     * @param part
     *            The {@link Node}.
     * 
     * @return
     * <li>true</li>, if above specified case applies.
     */
	private boolean hasJoinRelationToPart(INodeContainer host, Node part) {
		List<ModelElement> nonAspectChildren = getRecursiveNonAspectChildren(host);
		for (ModelElement n : nonAspectChildren) {
			if (n instanceof State) {
				for (Connection con : ((State) n).getSourceConnections()) {
					if (con instanceof JoinRelation) {
						if (con instanceof JoinRelation)
							return false;
						return ((ZoomableNodeContainer) part)
								.hasChild(((Transition) con.getTarget())
										.getSource())
								|| ((ZoomableNodeContainer) part)
										.hasChild(((Transition) con.getTarget())
												.getTarget())
								|| ((Transition) con.getTarget()).getSource() == part
								|| ((Transition) con.getTarget()).getTarget() == part;

					}
				}
			}
		}
		return false;
	}

	/**
     * Returns all non-aspectual elements comprised in this
     * <li>cont</li>.
     * 
     * @param cont
     *            {@link AspectContainer} which gets checked.
     * 
     * @return All NonAspect objects comprised in this
     *         <li>cont</li>.
     */
	private List<ModelElement> getRecursiveNonAspectChildren(INodeContainer cont) {
		List<ModelElement> list = new ArrayList<ModelElement>();
		for (ModelElement ele : cont.getChildren()) {
			if (ele instanceof INodeContainer
					&& !(ele instanceof AspectContainer)) {
				list.add(ele);
				list
						.addAll(getRecursiveNonAspectChildren((INodeContainer) ele));
			}
		}
		return list;
	}
}