package ch.unizh.ifi.adora.parts.policies.containers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.collections.iterators.IteratorChain;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.Add;
import ch.unizh.ifi.adora.model.commands.ChangeContainerSize;
import ch.unizh.ifi.adora.model.commands.InsertNodeVisually;
import ch.unizh.ifi.adora.model.commands.NodeCreate;
import ch.unizh.ifi.adora.model.commands.VisualizableModelElementSetConstraint;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.ITransitionGraph;
import ch.unizh.ifi.adora.parts.NodeEditPart;
import ch.unizh.ifi.adora.parts.ZoomableNodeContainerEditPart;
import ch.unizh.ifi.adora.parts.geometryfactory.GeometryFactory;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;
import ch.unizh.ifi.adora.view.figures.NodeFigure;

/**
 * EditPolicy for figures with an xy layout. 
 * This policy handles the following cases:
 * 
 * <ul>
 * 	<i>The adding of  nodes in host nodes after orphaning them.
 *  <i>Changing the constraint of a node.
 *  <i>Creating commands for the creation of sub nodes.
 *  <i>Deleting depandant elements in this node.
 * </ul>
 *  
 * This class has to be subclassed by xy layout policies for specific node 
 * types. The sub classes have to provide a predicate defining if a given 
 * operation is allowed on the container, e.g. creating or inserting.
 * 
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Oct 10, 2005 SM First Version
 *						Dec 2,  2005 SM Comments and simplification.
 *										Visual handling of node insertions.
 *						Dec 5,  2005 SM Refactoring of class.
 *						Jul 23, 2006 SM Reformatting.
 *						Jul 26, 2006 JS Adapted to refactoring of Add command
 * @version			$Date: 2008-04-11 10:56:28 +0200 (Fri, 11 Apr 2008) $, $Author: treinhard $, $Revision: 3596 $
 * @responsibilities	Contains the policy for handling nodes in this container node.
 */
public abstract class ContainerPolicy extends XYLayoutEditPolicy {

	/**
	 * Returns true, if a specific <i>part</i> can be created within a given 
	 * <i>host</i> container.
	 * 
	 * @pre     host != null && part != null
	 * @post    true
	 * @param   host The host container to which the given part should be added after creation.
	 * @param   part The part which should be added after its creation to the given host container.
	 * @return  Returns true, if the given <i>part</i> can be added to the given 
	 *          <i>host</i> container. 
	 */
	protected boolean createCommandAllowed(INodeContainer host, Node part){
		return true;
	}
	
	/**
	 * Returns true, if a specific container is allowed to add
	 * the given <i>part</i> to the given <i>host</i>.
	 * 
	 * @pre     host != null && part != null
	 * @post    true
	 * @param   host The host container to which the given part should be added.
	 * @param   part The part which should be added to the given host container.
	 * @return  Returns true, if the given <i>part</i> can be added to the given 
	 *          <i>host</i> container. 
	 */
	protected boolean addCommandAllowed(INodeContainer host, EditPart part){
		if (!nonFunctionalJoinRelationsOk(host, part)) {
			ConstraintHint.displayConstraintHint(IConstraintMessages.NON_FUNCTIONAL_CONNECTION);
			return false;
		}
		if (part instanceof ITransitionGraph){  
			if (!transitionConstraintsOk((ITransitionGraph)part)){
				ConstraintHint.displayConstraintHint(IConstraintMessages.TRANSITION_GRAPH);
				return false;
			}
		}
		return true;
	}

	/**
	 * Checks if the add command would violate the nonFunctional JoinRelation constraints.
	 * 
	 * @param   host The host container to which the given part should be added.
	 * @param   part The part which should be added to the given host container.
	 * @return  Returns true, if the given <i>part</i> can be added to the given 
	 *          <i>host</i> container. 
	 */
	private boolean nonFunctionalJoinRelationsOk(INodeContainer host, EditPart part){
		
		IteratorChain conIter = new IteratorChain();
		
		/*
		 * Contains all JoinRelations which end at the part or a child of it. 
		 */
		List <JoinRelation> targetRelations = new ArrayList<JoinRelation>();
		
		/*
		 * Contains all JoinRelations which start at the part or a child of it. 
		 */
		List <JoinRelation> sourceRelations = new ArrayList<JoinRelation>();
		
		List <JoinRelation> connectionRelations = new ArrayList<JoinRelation>();
		
		if (part.getModel() instanceof INodeContainer) {
			Iterator childIter = ((ZoomableNodeContainer)part.getModel()).getRecursiveChildrenIterator();
			while (childIter.hasNext()){
				Node n = (Node) childIter.next();
				conIter.addIterator(n.getConnectionsIterator());
			}
		}
		conIter.addIterator(((Node)part.getModel()).getConnectionsIterator());
		
		Connection c;
		// Adds all JoinRelations - created for non funcional requirements - 
		// which are leading to or originating from the added part or a child of an added part.
		// If the source and the target node are inside the added node no checking has to be done.
		while (conIter.hasNext()) {
			c = (Connection) conIter.next();
			if (c instanceof JoinRelation && c.getSource() instanceof AspectContainer) {
				if ((c.getTarget() == part.getModel() || 
						(part.getModel() instanceof INodeContainer) && 
						((INodeContainer)part.getModel()).hasChildRecursively(c.getTarget())) && 
					   !((INodeContainer)part.getModel()).hasChildRecursively(c.getSource()))
					targetRelations.add((JoinRelation)c);
				else if ((c.getSource() == part.getModel() || 
						(part.getModel() instanceof INodeContainer) && 
						((INodeContainer)part.getModel()).hasChildRecursively(c.getSource())) && 
					   !((INodeContainer)part.getModel()).hasChildRecursively(c.getTarget())) {
					sourceRelations.add((JoinRelation)c);
					if (c.getTarget() instanceof Connection) {
						connectionRelations.add((JoinRelation)c);
					}
				}
			}
			for (Connection joinRel : c.getTargetConnections()) {
				if (joinRel instanceof JoinRelation) {
					connectionRelations.add((JoinRelation)joinRel);
				}
			}
		}
		
		//Check if a violation concerning the crosscutted connections occurs
		for (JoinRelation joinRelation : connectionRelations) {
					 
			if ((joinRelation.getSource() == host || 
				joinRelation.getSource() instanceof INodeContainer &&
				host instanceof ZoomableNodeContainer &&
				((INodeContainer)joinRelation.getSource()).hasChildRecursively((ZoomableNodeContainer)host))
				
			||
			
			(((Connection)joinRelation.getTarget()).getSource() == host || 
					((Connection)joinRelation.getTarget()).getSource() instanceof INodeContainer &&
					host instanceof ZoomableNodeContainer &&
					((INodeContainer)((Connection)joinRelation.getTarget()).getSource()).hasChildRecursively((ZoomableNodeContainer)host))
					
			
			)
				return false;
		}
			
		//Checks contraints if a crosscutted element or a parent of it is been moved. 
		for (JoinRelation joinRelation : targetRelations) {
	
			//The source has also be situated on the root if the target is situated on the root.
			if (host instanceof AdoraModel) {
				if (joinRelation.getSource().getParent() != host){
					return false;
				}
				return true;
			}
			
			//Ensures that the moved node is not placed inside it'	s crosscutting aspect container
			if (joinRelation.getSource() == host || 
					((INodeContainer)joinRelation.getSource()).
							hasChildRecursively((ZoomableNodeContainer)host)) {
				return false;
			}
			
			//Ensures that the JoinRelation does not cross the bounds of the AspectContainer parent.
			if (!(joinRelation.getSource().getParent().
						hasChildRecursively((ZoomableNodeContainer)host) || 
					joinRelation.getSource().getParent() == host)) {
				return false;
			}
		}
		
		//Checks contraints if an Aspect container element or a parent of it is been moved. 
		for (JoinRelation joinRelation : sourceRelations) {
			//connection don't have to be examined.
			if (joinRelation.getTarget().getParent() == null) {
				continue;
			}
			if (!host.hasChildRecursively(joinRelation.getTarget())) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Checks if all selected IGraphs are only connected among each other by transitions.
	 */
	@SuppressWarnings("unchecked")
	private boolean transitionConstraintsOk(ITransitionGraph graphPart){
		
		boolean allowed = true;
		List<EditPart> selectedParts= ((GraphicalEditPart)graphPart).
											getRoot().getViewer().getSelectedEditParts(); 
			
		for (EditPart currentPart : selectedParts){
			if(currentPart instanceof ITransitionGraph && !((ITransitionGraph)currentPart).wasGraphNodeChecked()){
				if (!((ITransitionGraph)currentPart).depthFirstSearch()){
					allowed = false;
					break;
				}
			}
		}
		
		for (EditPart currentPart : selectedParts){
			if(currentPart instanceof ITransitionGraph){
				((ITransitionGraph)currentPart).resetVisited();
			}
		}
		return allowed;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	protected Command createAddCommand(EditPart childEditPart, Object constraint) { 
	
		Command cmd;

		List<EditPart> selectedParts= ((GraphicalEditPart)childEditPart).
		getRoot().getViewer().getSelectedEditParts(); 

	    if (!(childEditPart.getModel() instanceof Node)){
            return null;
        }
        
		Node part = (Node)childEditPart.getModel();
		INodeContainer host = (INodeContainer)getHost().getModel();
		Rectangle rect = (Rectangle) 
							GeometryFactory.getAdoraElement(
							(org.eclipse.draw2d.geometry.Rectangle)constraint);

		
		//Check if all selected nodes are movable.
		for (EditPart currentPart : selectedParts){
			if (!addCommandAllowed(host, currentPart)) {
				return null;
			}
		}		

		ConstraintHint.resetConstraintHint();
		
		Add add = new Add(part, host);

		add.setLabel("Add node");

		VisualizableModelElementSetConstraint setConstraint = 
				new VisualizableModelElementSetConstraint(part, rect);
		
		// set the new constraints of the added node
		cmd = add.chain(setConstraint);
			
		return cmd;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.requests.ChangeBoundsRequest,
	 *      org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(
			ChangeBoundsRequest request, EditPart child, Object constraint) {
		if (child instanceof NodeEditPart && constraint instanceof org.eclipse.draw2d.geometry.Rectangle) {
		       
			ConstraintHint.resetConstraintHint();
			
			boolean overlaps = false;
         
			if (child instanceof ZoomableNodeContainerEditPart && 
					((ZoomableNodeContainer)child.getModel()).isZoomedIn() &&
            		((ZoomableNodeContainer)child.getModel()).getChildCount() > 0 && (request.getSizeDelta().width < 0 ||
            				request.getSizeDelta().height < 0 )) { 
      
				 Rectangle childrenBounds = calcChildrenBounds((ZoomableNodeContainer)child.getModel());
		         
				 java.awt.Insets insets = ((ZoomableNodeContainer)child.getModel()).getZoomInsets();
				 Insets borderInsets = new Insets(insets.top, insets.left, insets.bottom, insets.right);
				 
				 org.eclipse.draw2d.geometry.Rectangle cropedConstraints = 
            									((org.eclipse.draw2d.geometry.Rectangle)constraint).getCropped(borderInsets);
				 
				 org.eclipse.draw2d.geometry.Rectangle cropedOldBounds = 
						(org.eclipse.draw2d.geometry.Rectangle)
							((ZoomableNodeContainerEditPart)child).getFigure().getBounds().getCropped(borderInsets);
				 
				 org.eclipse.draw2d.geometry.Rectangle relBounds = cropedConstraints.getCopy();
				 relBounds.translate(-((ZoomableNodeContainer)child.getModel()).getLocation().x, -((ZoomableNodeContainer)child.getModel()).getLocation().y);
				 
				 overlaps = !(relBounds.x <= childrenBounds.x && relBounds.right() >= childrenBounds.right()
						 && relBounds.y <= childrenBounds.y && relBounds.bottom() >= childrenBounds.bottom());
            	
				 if (!overlaps){
            		return new ChangeContainerSize((ZoomableNodeContainer) child
    						.getModel(), (Rectangle) GeometryFactory
    						.getAdoraElement(constraint), childrenBounds, (Rectangle) GeometryFactory
    						.getAdoraElement( cropedConstraints),(Rectangle) GeometryFactory
    						.getAdoraElement( cropedOldBounds) );
            	}
            } else if (!overlaps){
            	return new VisualizableModelElementSetConstraint((Node) child
            			.getModel(), (Rectangle) GeometryFactory
    					.getAdoraElement(constraint));
            }
		}
		return super.createChangeConstraintCommand(request, child,
				constraint);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart,
     *      java.lang.Object)
     */
	protected Command createChangeConstraintCommand(EditPart child,
			Object constraint) {
		// not used in this example
		return null;
	}
	
	/**
	 * Calculates the bounds of all direct children of the <code> container </code>.
	 * 
	 * @param container {@link ZoomableNodeContainer} in which the children are situated.
	 * @return The smallest Rectangle in which all children have place.
	 */
	private Rectangle calcChildrenBounds(ZoomableNodeContainer container){
		
		Rectangle bounds = null;
		
		for (ModelElement me : container.getChildren()) {

			Node n = (Node)me;
			
			if(bounds == null) {
				bounds = new Rectangle(n.getBounds());
			} else {
				bounds.add(n.getBounds());
			}
	
		}
		
		return bounds;
	}
	
	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
     */
	protected Command getCreateCommand(CreateRequest request) {
		Command command = null;
		NodeCreate createCommand = null;
		//Object childClass = request.getNewObjectType();
		Node nc = (Node)request.getNewObject();
		INodeContainer host = (INodeContainer) getHost().getModel();		
        org.eclipse.draw2d.geometry.Rectangle constraint = (org.eclipse.draw2d.geometry.Rectangle)getConstraintFor(request);
		//Class hostClass = getHost().getModel().getClass();
				
		if (createCommandAllowed(host,nc)) {
				// return a command that can add a Shape to a ShapesDiagram
				createCommand = new NodeCreate(nc, host, (Rectangle)GeometryFactory.getAdoraElement(constraint));
		}
		
		// if a node creation command was created,
		// we need to create command for the visual insertion in the model.
		if (createCommand != null) {
			
			InsertNodeVisually radialMove = new InsertNodeVisually(nc);
			command = createCommand.chain(radialMove);
			
			
			ConstraintHint.resetConstraintHint();
			
		}
				
		return command;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	protected Command getDeleteDependantCommand(Request request) {
		// not used in this example
		return null;
	}
	
	
	/**
	 * Highlight the container that is the target for a moved node.
	 */
	protected void showLayoutTargetFeedback(Request request) { 
		
		if(REQ_ADD.equals(request.getType()) && getHostFigure() instanceof NodeFigure) {
			((NodeFigure)getHostFigure()).setLineWidth(2);
		}
		
	}
	
	
	protected void eraseLayoutTargetFeedback(Request request) {
		
		if(REQ_ADD.equals(request.getType()) && getHostFigure() instanceof NodeFigure) {
			((NodeFigure)getHostFigure()).setLineWidth(1);
		}
		
	}
}