package ch.unizh.ifi.adora.parts.policies.containers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.collections.iterators.IteratorChain;
import org.eclipse.gef.EditPart;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.ScenarioHierarchyConstraints;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.ScenarioEditPart;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * Superclass for Aspect Containers and Component Policies.
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				28.04.2006 IV First Version
 * 						23.07.2006 SM Some reformatting.
 * 						25.07.2006 SM Comments checked and corrected.
 * @version				$Date: 2007-01-25 18:06:03 +0100 (Thu, 25 Jan 2007) $, $Author: smeier $, $Revision: 2403 $
 */
public abstract class ModulePolicy extends ZoomableNodeContainerPolicy {

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policy.ContainerXYLayoutEditPolicy#createCommandAllowed(ch.unizh.ifi.adora.model.INodeContainer, ch.unizh.ifi.adora.model.NodeContainer)
	 */
	protected boolean createCommandAllowed(INodeContainer host, Node part) {
		// Constraint: if the host is a component, then the child must be
		// either a scenario or a component or a state.
		if (!(part instanceof Scenario || part instanceof ZoomableNodeContainer)) {
			ConstraintHint.displayConstraintHint(IConstraintMessages.ZOOMABLE_CHILDREN);
			return false;
		}
		return super.createCommandAllowed(host, part);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policy.ContainerXYLayoutEditPolicy#addCommandAllowed(ch.unizh.ifi.adora.model.INodeContainer, ch.unizh.ifi.adora.model.NodeContainer)
	 */
	protected boolean addCommandAllowed(INodeContainer host, EditPart part) {
		
		Object model = part.getModel();
		
		if (model instanceof Scenario) {
			if (!scenarioAddCommandAllowed(host, (ScenarioEditPart)part)) {
				ConstraintHint.displayConstraintHint(IConstraintMessages.SCENARIO_COMPONENT_HIERARCHY);
				return false;
			}
		} else if(model instanceof ZoomableNodeContainer){
			if (!scenarioCheck(host, (ZoomableNodeContainer) model)) {
				ConstraintHint.displayConstraintHint(IConstraintMessages.SCENARIO_COMPONENT_HIERARCHY);
				return false;
			}
			if (!associationCheck(host, (ZoomableNodeContainer) model)) {
				ConstraintHint.displayConstraintHint(IConstraintMessages.CONTAINER_ASSOCIATIONS);
				return false;
			} 
		}

		if (model instanceof Actor) {
			ConstraintHint.displayConstraintHint(IConstraintMessages.MODULE_CHILDREN);
			return false;
		}
		return super.addCommandAllowed(host, part);
	}

	/**
	 * Checks if a ZoomableNodeContainer can be added to the given module
	 * ({@link INodeContainer}).
	 *  
	 * Components with scenarios or states as children 
	 * have to be checked seperatly since constraints mustn't be violated.
	 *
	 * @param host Potential parent of <li>part</li>.
	 * @param part Potential child of <li>host</li>.
	 * 
	 * @return <li>true</li>, if the <li>part</li> can be added to the <li>host</li>.
	 */
	private boolean scenarioCheck(INodeContainer host,
			ZoomableNodeContainer part) {
		//All children of this part have to be checked, since each could have
		//a connection, which could get illegitimate after moving the part
		Iterator iter = ((ZoomableNodeContainer) part)
				.getRecursiveChildrenIterator();
		while (iter.hasNext()) {
			Node n = (Node) iter.next();
			//Scenario-Constraints
			if (n instanceof Scenario) {
				if (((Scenario) n).getScenarioParent() != null) {
					if (!ScenarioHierarchyConstraints
							.isContainerParentOfChildContainer(((Scenario) n)
									.getScenarioParent(),
									(ZoomableNodeContainer) part)) {
						// scenarios of this part have scenario connection
						// to scenario-parents situated in a parent component
						// of this component (= part)
						if (!ScenarioHierarchyConstraints
								.isChildContainerChildOfContainer(
										((Scenario) n).getScenarioParent(),
										host)) {
							// target component (=host) is above the (component) hierarchy level
							// of the scenario parent's ( = n.getScenarioParent()) component.
							// This would result in a scenario connection leading from a
							// component to its parent component
							return false;
						}
					}
				}
			}
		}
		return true;
	}
	
	/**
	 * Check if the component to add is connected by associations to a Component contained in the  
	 * host Container and not situated on the same level.
	 *
	 * @param host Potential parent of <li>part</li>.
	 * @param part Potential child of <li>host</li>.
	 * 
	 * @return <li>true</li>, if the <li>part</li> can be added to the <li>host</li>.
	 */
	private boolean associationCheck(INodeContainer host,
			ZoomableNodeContainer part) {

		Iterator<ModelElement> childIter = ((ZoomableNodeContainer) part)
				.getRecursiveChildrenIterator();
		IteratorChain assoIter = new IteratorChain();

		assoIter.addIterator(part.getAssociationsIterator());

		while (childIter.hasNext()) {
			assoIter.addIterator(((Node) childIter.next())
					.getAssociationsIterator());
		}

		Association asso;
		while (assoIter.hasNext()) {
			asso = (Association) assoIter.next();
		
			Node otherEnd = asso.getSource() == part || 
					part.hasChildRecursively(asso.getSource()) 
						? asso.getTarget() : asso.getSource(); 
			
			//Has Connection to new host
			if (otherEnd == host || 
					(otherEnd instanceof ZoomableNodeContainer) && 
					((ZoomableNodeContainer)otherEnd).hasChildRecursively((ZoomableNodeContainer)host)){
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Check if a {@link Scenario} can be added to the given module implementing the interface 
	 * {@link INodeContainer}.
	 * 
	 * This is determined by checking if the {@link Scenario}'s children 
	 * are situated in a {@link INodeContainer} which is child of the part {@link INodeContainer}.
	 * 
	 * If several {@link Scenario}s are selected the following constraints apply:
	 * 
	 * If one is moving a {@link Scenario} sub tree up the hierarchy: 
	 * All {@link Scenario} parents of the selected {@link Scenario}s have to be selected too, 
	 * until {@link Scenario}s are found which are situated in the target {@link ZoomableNodeContainer}  
	 * or in a parent of it. (These don't have to be selected) 
	 * 
	 * If one is moving a {@link Scenario} sub tree down the hierarchy: 
	 * All {@link Scenario} children of the selected {@link Scenario}s have to be selected too, 
	 * until {@link Scenario}s are found which are situated in the target {@link ZoomableNodeContainer}  
	 * or in a child of it. (These don't have to be selected)
	 * 
	 * 
	 * @param host Potential parent of <li>part</li>.
	 * @param model Potential child of <li>host</li>.
	 * 
	 * @return <li>true</li>, if the <li>part</li> can be added to the <li>host</li>.
	 */
	@SuppressWarnings("unchecked")
	private boolean scenarioAddCommandAllowed(INodeContainer host, ScenarioEditPart part) {
		
		Scenario model = (Scenario)part.getModel();
		
		List<ScenarioEditPart> selectedScenarios = new ArrayList<ScenarioEditPart>();
		
		List<EditPart> selectedParts= part.getRoot().getViewer().getSelectedEditParts(); 
		for (EditPart currentPart : selectedParts){
			if(currentPart instanceof ScenarioEditPart){
				selectedScenarios.add((ScenarioEditPart)currentPart);
			}
		}
	
		boolean checked = false;
		
		//Several scenarios are selected
		if(selectedScenarios.size() > 1){
			
			/*
			 host is situated in parent of parent of part.
				--> all SP of selected nodes up to the host (but not the host scnearios) have to be selected
			*/
			for (ScenarioEditPart currentPartToCheck : selectedScenarios){
				if (host.hasChildRecursively((ModelElement)
						currentPartToCheck.getParent().getModel())) {
					for (ScenarioEditPart currentPart : selectedScenarios){
						if (!currentPart.areAllScenarioParentsSelectedUpToContainer(host)) {
							return false;
						}
					}
					checked = true;
				}
			}
			
			/*
			host is situated in child of component of part.
				--> all SC of selected nodes have to be selected
			*/ 
			for (ScenarioEditPart currentPartToCheck : selectedScenarios){
				if (((INodeContainer)currentPartToCheck.getParent().getModel()).
						hasChildRecursively((ZoomableNodeContainer)host)) {
					for (ScenarioEditPart currentPart : selectedScenarios){
						if (!currentPart.areAllScenarioChildrenSelectedDownToContainer(host)) {
							return false;
						}
					}
					checked = true;
				}
			}
			
			//whole trees are selected
			if (!checked) {
				for (ScenarioEditPart currentPartToCheck : selectedScenarios){
					if (!currentPartToCheck.getRootScenario().areAllScenarioChildrenSelected()){
						return false;
					}
					checked = true;
				}
			}
			return checked;
		}
		
		for (Scenario s : model.getScenarioChildren()) {
			//if scenario is a parent
			if (!ScenarioHierarchyConstraints
					.isContainerParentOfChildContainer(s, host)) {
				return false;
			}
		}

		//Check if a scenario can be added to this component.
		//This is determined by checking if the scenario's parent
		//is situated in a component which is parent of the part component.
		if (model.getScenarioParent() != null) {
			//if scenario is a child
			if (!ScenarioHierarchyConstraints.isChildContainerChildOfContainer(
					model.getScenarioParent(), host)) {
				return false;
			}
		}
		return true;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	/*protected Command createAddCommand(EditPart childEditPart, Object constraint) { 
		Command cmd;
		
		INodeContainer host = (INodeContainer)getHost().getModel();
		
		if (!addCommandAllowed(host, childEditPart)) {
			return null;
		}
		
		Rectangle rect = (Rectangle) 
		GeometryFactory.getAdoraElement(
		(org.eclipse.draw2d.geometry.Rectangle)constraint);

		Node part = (Node)childEditPart.getModel();
		
		if (childEditPart.getModel() instanceof Scenario){
			for (Scenario child : ((Scenario)childEditPart.getModel()).getScenarioChildren()){
				if (!child.isVisible()) {
					if (addCommandAllowed(host, childEditPart)) {
						cmd = new ScenarioAdd((Scenario) part, host, rect);
					} else {
						cmd = null;
					}
					return cmd;
				}
			}
        }
		return super.createAddCommand(childEditPart, constraint);
	}*/
}