package ch.unizh.ifi.adora.parts.policies.nodes;

import java.util.ArrayList;
import java.util.Stack;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * This policy determines which type of connections can be connected to an
 * actor.
 * 
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 27.09.2005 SM First Version 22.07.2006 SM Comments reformatted.
 *          25.07.2006 SM Comments checked, corrected.
 * @version $Date: 2008-01-22 14:04:49 +0100 (Tue, 22 Jan 2008) $, $Author: vontobel $, $Revision: 3470 $
 */
public class ActorNodePolicy extends ConstrainablePolicy {

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionAllowed(ch.unizh.ifi.adora.model.representation.Node,
     *      ch.unizh.ifi.adora.model.representation.Node,
     *      org.eclipse.gef.commands.Command)
     */
	@SuppressWarnings("deprecation")
	protected boolean connectingTargetAllowed(Node sourceNode,
			ConnectableElement targetComponent, Command command) {
		if (sourceNode == getHost().getModel()) {
			return false;
		}

		if (command instanceof AssociationCreate) {
			if (sourceNode instanceof Scenario || sourceNode instanceof Actor
					|| sourceNode instanceof Component
					&& ((Component) sourceNode).isExternal()) {
				return true;
			}
		}

		if (command instanceof JoinRelationCreate
				&& sourceNode instanceof Actor) {
			if (wouldEstablishCircle((Actor) sourceNode, (Actor) getHost()
					.getModel())) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.ACTOR_CIRCLE);
				return false;
			}

			/*
             * there can be a joinrelation between two actors, but only if there
             * is not any
             */
			Actor actor = (Actor) targetComponent;
			for (Connection connection : actor.getConnections()) {
				if (!(connection instanceof JoinRelation)) {
					// handle only JoinRelations, so skip to the next connection
					continue;
				}

				if (connection.getSource().equals(sourceNode)
						&& connection.getTarget().equals(actor)) {
					// This connection already exists
					return false;
				}
			}

			return true;
		}

		ConstraintHint
				.displayConstraintHint(IConstraintMessages.ACTOR_ASSOCIATION);
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectionAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.Connection)
     */
	@SuppressWarnings("deprecation")
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target,
			ch.unizh.ifi.adora.model.representation.Connection con) {

		if (con instanceof JoinRelation) {
			if (!(source instanceof Actor) && !(target instanceof Actor)) {
				return false;
			}

			if (wouldEstablishCircle((Actor) source, (Actor) target)) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.ACTOR_CIRCLE);
				return false;
			}

			/*
             * there can be a joinrelation between two actors, but only if there
             * is not any
             */
			Actor actor = (Actor) target;
			for (Connection connection : actor.getConnections()) {
				// handle only JoinRelations, so skip to the next connection
				if (connection instanceof JoinRelation 
						&& con != connection
						&& connection.getSource().equals(source)
						&& connection.getTarget().equals(actor)) {
					// This connection already exists
					return false;
				}
			}

			return true;
		}

		if (!(con instanceof Association)) {
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.ACTOR_ASSOCIATION);
			return false;
		}

		if (((Association) con).isCalculatedAbstracted()) {
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.CALC_CONNECTION);
			return false;
		}

		if (!(getOtherEnd(source, target) instanceof Scenario
				|| getOtherEnd(source, target) instanceof Actor || getOtherEnd(
				source, target) instanceof Component
				&& ((Component) getOtherEnd(source, target)).isExternal())) {
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.ACTOR_ASSOCIATION);
			return false;
		}

		return true;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionCreateAllowedd(java.lang.Class)
     */
	protected boolean creatingConnectionAllowed(Class connectionType) {
		if (connectionType == Association.class) {
			return true;
		}

		if (connectionType == JoinRelation.class) {
			return true;
		}

		ConstraintHint
				.displayConstraintHint(IConstraintMessages.ACTOR_ASSOCIATION);
		return false;
	}

	/**
     * Check, if there would be a circle if the actor actor would be connected
     * to this one
     * 
     * @param from
     * @return
     */
	private boolean wouldEstablishCircle(Actor from, Actor to) {
		Stack<Actor> actors = new Stack<Actor>();
		actors.add(to);

		ArrayList<Actor> visited = new ArrayList<Actor>();

		while (actors.size() > 0) {
			Actor actualActor = actors.pop();

			if (visited.contains(actualActor)) {
				continue;
			}

			visited.add(actualActor);

			if (actualActor.equals(from)) {
				return true;
			} else {
				actors.addAll(getNextActors(actualActor));
			}
		}

		return false;
	}

	/**
     * fetchs the actors, which are the target of a joinrelation from this node
     * 
     * @param actor
     * @return
     */
	private ArrayList<Actor> getNextActors(Actor actor) {
		ArrayList<Actor> actors = new ArrayList<Actor>();

		for (Connection connection : actor.getSourceConnections()) {
			if (connection instanceof JoinRelation) {
				actors.add((Actor) connection.getTarget());
			}
		}

		return actors;
	}
}