package ch.unizh.ifi.adora.parts.policies.nodes;

import java.util.logging.Logger;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.policies.JoinRelationPolicyHelper;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * This policy determines which
 * {@link ch.unizh.ifi.adora.model.representation.Node}s, in what way, can be
 * connected to a {@link ch.unizh.ifi.adora.model.representation.Component}.
 * Since {@link ch.unizh.ifi.adora.model.representation.Component}s have a
 * prescribed hierarchial order, additional connection restraints are needed.
 * 
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 27.09.2005 SM First Version<br>
 *          23.07.2006 SM Reformatting.<br>
 *          25.07.2006 SM Comments reformatted, checked, corrected.<br>
 *          25.01.2007 PS Constraints updated (Bug #806).
 * @version $Date: 2009-02-10 11:28:42 +0100 (Tue, 10 Feb 2009) $, $Author: vigan $, $Revision: 4001 $
 */
public class ComponentPolicy extends ConstrainablePolicy {
	private static final Logger logger = Logger.getLogger(ComponentPolicy.class
			.getName());

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionAllowed(ch.unizh.ifi.adora.model.representation.Node,
     *      ch.unizh.ifi.adora.model.representation.Node,
     *      org.eclipse.gef.commands.Command)
     */
	protected boolean connectingTargetAllowed(Node sourceNode,
			ConnectableElement targetComponent, Command command) {

		// checks if a connection from a chlid to a parent is made.
		if (((ZoomableNodeContainer) getHost().getModel())
				.hasChildRecursively(sourceNode)) {
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.PARENT_CHILD_ASSOCIATION);
			return false;
		}

		if (sourceNode instanceof ZoomableNodeContainer) {
			if (((ZoomableNodeContainer) sourceNode)
					.hasChildRecursively((ModelElement) getHost().getModel())) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.PARENT_CHILD_ASSOCIATION);
				return false;
			}
		}

		if (command instanceof TransitionCreate) {
			TransitionCreate tc = (TransitionCreate) command;
			if (tc.getSource().getParent() != tc.getTarget().getParent()) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.TRANSITION_GRAPH);
				return false;
			}

			if (sourceNode instanceof Component || sourceNode instanceof State) {
				return true;
			}
		}

		if (command instanceof AssociationCreate) {
			if (sourceNode instanceof Component
					|| sourceNode instanceof Scenario
					&& ((Component) targetComponent).isExternal()
					|| sourceNode instanceof AspectContainer
					|| sourceNode instanceof Actor
					&& ((Component) getHost().getModel()).isExternal()) {

				return true;
			}
		}

		if (command instanceof JoinRelationCreate) {
			if (sourceNode instanceof State || sourceNode instanceof Component || sourceNode instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(sourceNode, targetComponent);
			}

			return super.connectingTargetAllowed(sourceNode, targetComponent,
					command);
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectionAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.Connection)
     */
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target, Connection con) {
		if (con instanceof Association) {
			return !((Association) con).isCalculatedAbstracted()
					&& (getOtherEnd(source, target) instanceof Component || getOtherEnd(
							source, target) instanceof AspectContainer);
		}

		if (con instanceof Transition) {
			if (source.getParent() == target.getParent()) {
				ConnectableElement otherEnd = getOtherEnd(source, target);

				return otherEnd instanceof Component
						|| otherEnd instanceof State
						|| otherEnd instanceof ExitPoint;
			}
		}

		if (con instanceof JoinRelation) {
			/*
             * a join relationship may not cross the border of the aspect's
             * parent.
             */
			if (source.getParent().getParent() != null && !source.getParent().getParent().equals(target.getParent())) {
				return false;
			}

			if (source instanceof Component || source instanceof State|| source instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(source, target);
			}

			if (isCrosscuttingStategroup((Node) source)) {
				logger
						.info("can not draw a joinrelation from a state which is part of a crosscutting state group");
				return false;
			}

			return super.reconnectingAllowed(source, target, con);
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionCreateAllowedd(java.lang.Class)
     */
	protected boolean creatingConnectionAllowed(Class connectionType) {
		if (connectionType == Association.class
				|| connectionType == Transition.class) {
			return true;
		}

		if (connectionType == JoinRelation.class) {
			Component component = (Component) getHost().getModel();

			if (!component.isInsideAspectContainer()) {
				return false;
			}

			if (isCrosscuttingStategroup(component)) {
				logger
						.info("can not draw a joinrelation from a state which is part of a crosscutting state group");
				return false;
			}

			return true;
		}

		return false;
	}
}