package ch.unizh.ifi.adora.parts.policies.nodes;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.FeedbackHelper;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.commands.AssociationReconnect;
import ch.unizh.ifi.adora.model.commands.ConnectionCreate;
import ch.unizh.ifi.adora.model.commands.ConnectionReconnect;
import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.commands.JoinRelationReconnect;
import ch.unizh.ifi.adora.model.commands.NodeConnectionCreate;
import ch.unizh.ifi.adora.model.commands.ScenarioConnectionCreate;
import ch.unizh.ifi.adora.model.commands.ScenarioConnectionReconnect;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.commands.TransitionReconnect;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.parts.geometryfactory.GeometryFactory;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.view.figures.AssociationFigure;
import ch.unizh.ifi.adora.view.figures.ConnectionFigure;
import ch.unizh.ifi.adora.view.figures.JoinRelationFigure;
import ch.unizh.ifi.adora.view.figures.ScenarioConnectionFigure;
import ch.unizh.ifi.adora.view.figures.TransitionFigure;
import ch.unizh.ifi.adora.view.layers.MultipleRoutersConnectionLayer;

/**
 * Superclass for all
 * {@link ch.unizh.ifi.adora.model.representation.ConnectableElement} policies.
 * This class provides two abstract methods:
 * {@link ConnectableElementPolicy#connectingTargetAllowed(Node, ConnectableElement, Command)}
 * to ascertain if a connection can be created and
 * {@link ConnectableElementPolicy#reconnectingAllowed(ConnectableElement, ConnectableElement, Connection)}
 * to ascertain if a connection can be reconnected.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 24.08.2006 IV First Version
 * @version $Date: 2008-01-25 14:17:46 +0100 (Fri, 25 Jan 2008) $, $Author: schoeni $, $Revision: 3503 $
 */
public abstract class ConnectableElementPolicy extends GraphicalNodeEditPolicy {

	/**
     * Determines if a {@link Connection} can be attached to the selected target
     * {@link ConnectableElement}.
     * 
     * @param source
     *            The source {@link Node} of the
     *            {@link ch.unizh.ifi.adora.model.representation.Connection}.
     * @param target
     *            The target {@link ConnectableElement} of the
     *            {@link ch.unizh.ifi.adora.model.representation.Connection}.
     * @param command
     *            The {@link Command} which creates the given {@link Connection}.
     * 
     * @return
     * <li>true</li>, if a {@link Connection} can be created.
     */
	protected abstract boolean connectingTargetAllowed(Node source,
			ConnectableElement target, Command command);

	/**
     * Determines if a {@link Connection} can be created from the selected
     * source {@link Node}.
     * 
     * @param connectionType
     *            The type of the {@link Connection} which gets created.
     * @return
     *            <li>true</li>, if a {@link Connection} can be created.
     */
	protected abstract boolean creatingConnectionAllowed(Class connectionType);

	/**
     * Determines if a {@link Connection} can be reconnected.
     * 
     * @param source
     *            The source {@link ConnectableElement} of the
     *            {@link ch.unizh.ifi.adora.model.representation.Connection}.
     * @param target
     *            The target {@link ConnectableElement} of the
     *            {@link ch.unizh.ifi.adora.model.representation.Connection}.
     * @param con
     *            The {@link Connection} to be reconnected.
     * 
     * @return
     * <li>true</li>, if a {@link Connection} can be created.
     */
	protected abstract boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target, Connection con);

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
	protected Command getConnectionCompleteCommand(
			CreateConnectionRequest request) {

		ConnectionCreate cmd = (ConnectionCreate) request.getStartCommand();

		if ((ConnectableElement) getHost().getModel() instanceof Connection
				&& cmd instanceof NodeConnectionCreate) {
			return null;
		}
		cmd.setTarget((ConnectableElement) getHost().getModel());

		if (!connectingTargetAllowed((Node) request.getSourceEditPart()
				.getModel(), (ConnectableElement) request.getTargetEditPart()
				.getModel(), request.getStartCommand())) {
			return null;
		}

		ConstraintHint.resetConstraintHint();

		Point clickPoint = request.getLocation().getCopy();
		IFigure figure = getHostFigure();
		figure.translateToRelative(clickPoint);
		figure.translateFromParent(clickPoint);
		clickPoint.translate(getHostFigure().getClientArea().getLocation()
				.getNegated());
		// Correct the coordinates if the figure has a border (e.g. for the
		// label)
		if (getHostFigure().getBorder() != null) {
			Insets insets = getHostFigure().getBorder().getInsets(
					getHostFigure());
			clickPoint.translate(insets.left, insets.top);
		}
		cmd
				.setTargetClickPoint((ch.unizh.ifi.adora.model.geometry.Point) GeometryFactory
						.getAdoraElement(clickPoint));
		return cmd;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {

		ConnectableElement source = (ConnectableElement) getHost().getModel();
		Class oc = (Class) request.getNewObjectType();

		if (!creatingConnectionAllowed(oc)) {
			return null;
		}

		ConstraintHint.resetConstraintHint();

		Point clickPoint = request.getLocation().getCopy();
		IFigure figure = getHostFigure();
		figure.translateToRelative(clickPoint);
		figure.translateFromParent(clickPoint);
		clickPoint.translate(getHostFigure().getClientArea().getLocation()
				.getNegated());

		// Correct the coordinates if the figure has a border (e.g. for the
		// label)
		if (getHostFigure().getBorder() != null) {
			Insets insets = getHostFigure().getBorder().getInsets(
					getHostFigure());
			clickPoint.translate(insets.left, insets.top);
		}

		ConnectionCreate cmd = null;
		if (oc.equals(Transition.class)) {
			cmd = new TransitionCreate((Node) source);
		} else if (oc.equals(Association.class)) {
			cmd = new AssociationCreate((Node) source);
		} else if (oc.equals(ScenarioConnection.class)) {
			cmd = new ScenarioConnectionCreate((Scenario) source);
		} else if (oc.equals(JoinRelation.class)) {
			cmd = new JoinRelationCreate((Node) source);
		}

		if (cmd != null) {
			request.setStartCommand(cmd);
			cmd
					.setSourceClickPoint((ch.unizh.ifi.adora.model.geometry.Point) GeometryFactory
							.getAdoraElement(clickPoint));

		}
		return cmd;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
     */
	protected Command getReconnectTargetCommand(ReconnectRequest request) {

		ConnectionReconnect cmd;

		Connection conn = (Connection) request.getConnectionEditPart()
				.getModel();

		if (conn instanceof Transition) {
			cmd = new TransitionReconnect((Transition) conn);
		} else if (conn instanceof JoinRelation) {
			cmd = new JoinRelationReconnect((JoinRelation) conn);
		} else if (conn instanceof Association) {
			cmd = new AssociationReconnect((Association) conn);
		} else if (conn instanceof ScenarioConnection) {
			cmd = new ScenarioConnectionReconnect((ScenarioConnection) conn);
		} else {
			return null;
		}

		ConnectableElement newTarget = (ConnectableElement) getHost()
				.getModel();
		cmd.setNewTarget(newTarget);

		if (!reconnectingAllowed(conn.getSource(), newTarget, conn)) {
			return null;
		}

		ConstraintHint.resetConstraintHint();

		Point clickPoint = request.getLocation().getCopy();
		IFigure figure = getHostFigure();
		figure.translateToRelative(clickPoint);
		figure.translateFromParent(clickPoint);
		if (getHostFigure() instanceof ConnectionFigure) {

			clickPoint.translate(((ConnectionFigure) getHostFigure())
					.getRealBounds().getLocation().getNegated());

		} else {

			clickPoint.translate(getHostFigure().getClientArea().getLocation()
					.getNegated());
		}
		// Correct the coordinates if the figure has a border (e.g. for the
		// label)
		if (getHostFigure().getBorder() != null) {
			Insets insets = getHostFigure().getBorder().getInsets(
					getHostFigure());
			clickPoint.translate(insets.left, insets.top);
		}

		cmd
				.setTargetClickPoint((ch.unizh.ifi.adora.model.geometry.Point) GeometryFactory
						.getAdoraElement(clickPoint));
		return cmd;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
     */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {

		ConnectionReconnect cmd;

		Connection conn = (Connection) request.getConnectionEditPart()
				.getModel();

		if (conn instanceof Transition) {
			cmd = new TransitionReconnect((Transition) conn);
		} else if (conn instanceof JoinRelation) {
			cmd = new JoinRelationReconnect((JoinRelation) conn);
		} else if (conn instanceof Association) {
			cmd = new AssociationReconnect((Association) conn);
		} else if (conn instanceof ScenarioConnection) {
			cmd = new ScenarioConnectionReconnect((ScenarioConnection) conn);
		} else {
			return null;
		}

		ConnectableElement newSource = (ConnectableElement) getHost()
				.getModel();
		cmd.setNewSource(newSource);

		if (!reconnectingAllowed(newSource, conn.getTarget(), conn)) {
			return null;
		}

		ConstraintHint.resetConstraintHint();

		Point clickPoint = request.getLocation().getCopy();
		IFigure figure = getHostFigure();
		figure.translateToRelative(clickPoint);
		figure.translateFromParent(clickPoint);
		clickPoint.translate(getHostFigure().getClientArea().getLocation()
				.getNegated());
		// Correct the coordinates if the figure has a border (e.g. for the
		// label)
		if (getHostFigure().getBorder() != null) {
			Insets insets = getHostFigure().getBorder().getInsets(
					getHostFigure());
			clickPoint.translate(insets.left, insets.top);
		}

		cmd
				.setSourceClickPoint((ch.unizh.ifi.adora.model.geometry.Point) GeometryFactory
						.getAdoraElement(clickPoint));

		return cmd;
	}

	/**
     * Returns a connection to be used as feeback during creates.
     * 
     * @param req
     *            the operation being performed
     * @return a connection to use as feedback
     */
	protected org.eclipse.draw2d.Connection createDummyConnection(Request req) {

		CreateConnectionRequest createReq = (CreateConnectionRequest) req;

		if (createReq.getNewObjectType() == Association.class) {
			return new AssociationFigure();
		} else if (createReq.getNewObjectType() == ScenarioConnection.class) {
			return new ScenarioConnectionFigure();
		} else if (createReq.getNewObjectType() == Transition.class) {
			return new TransitionFigure();
		} else if (createReq.getNewObjectType() == JoinRelation.class) {
			return new JoinRelationFigure();
		}
		return null;
	}

	/**
     * Returns the FeedbackHelper that is ready to use. The feedback helper must
     * be configured with the connection that will be used to display feedback,
     * and that connection must be added to the appropriate layer in the
     * diagram.<br>
     * <br>
     * This method is overriden to use the multiple connections router for the
     * feedback during the creation of a connection.
     * 
     * @param request
     *            the CreateConnectionRequest
     * @return Returns a FeedbackHelper object.
     */
	protected FeedbackHelper getFeedbackHelper(CreateConnectionRequest request) {

		if (feedbackHelper == null) {
			feedbackHelper = new FeedbackHelper();
			Point p = request.getLocation();
			connectionFeedback = createDummyConnection(request);
			MultipleRoutersConnectionLayer layer = (MultipleRoutersConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
			connectionFeedback.setConnectionRouter(layer
					.getRouter(connectionFeedback.getClass()));
			connectionFeedback
					.setSourceAnchor(getSourceConnectionAnchor(request));
			feedbackHelper.setConnection(connectionFeedback);
			addFeedback(connectionFeedback);
			feedbackHelper.update(null, p);
		}
		return feedbackHelper;
	}

	/**
     * Used for determine which elements can be connected to element for which
     * this policy is used.
     * 
     * @param c
     *            The connection for which the other end gets evaluated
     * @return The other end of the connection.
     */
	protected ConnectableElement getOtherEnd(ConnectableElement source,
			ConnectableElement target) {
		return source == getHost().getModel() ? target : source;
	}

	/**
     * Returns the layer used for displaying the feedback while the user draws
     * the line. Method has to be overriden to return the correct scaled
     * coordinates when a connection is added while the scale factor is not 1.0.
     * 
     * @return the scaled feedback layer
     */
	protected IFigure getFeedbackLayer() {
		return getLayer(LayerConstants.SCALED_FEEDBACK_LAYER);
	}
}