package ch.unizh.ifi.adora.parts.policies.nodes;

import java.util.ArrayList;
import java.util.LinkedList;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.parts.policies.JoinRelationPolicyHelper;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * This policy is used to determine on which elements a JoinRelation can be
 * attached.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 26.09.2006 IV First Version<br>
 *          25.01.2007 PS Constraints updated (Bug #806).
 * @version $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public abstract class ConstrainablePolicy extends ConnectableElementPolicy {
	/**
     * Checks, if this element is part of a crosscutting stategroup.
     * 
     * @param element
     * @return
     */
	protected boolean isCrosscuttingStategroup(Node node) {
		if (!(node instanceof State || node instanceof Component || node instanceof ExitPoint)) {
			return false;
		}

		ArrayList<ConnectableElement> visitedNodes = new ArrayList<ConnectableElement>();
		LinkedList<ConnectableElement> queue = new LinkedList<ConnectableElement>();
		queue.add(node);

		int startStates = 0;

		while (queue.size() > 0) {
			ConnectableElement element = queue.poll();

			if (visitedNodes.contains(element)) {
				continue;
			}

			visitedNodes.add(element);

			// check for a start state
			if (element instanceof State) {
				if (((State) element).isStartState()) {
					startStates++;
				}
			}

			if (!(element instanceof State || element instanceof Component)) {
				return false;
			}

			for (Connection connection : element.getSourceConnections()) {
				if (!(connection instanceof Transition)) {
					return false;
				}

				queue.add(connection.getTarget());
			}

			for (Connection connection : element.getTargetConnections()) {
				if (!(connection instanceof Transition)) {
					return false;
				}

				queue.add(connection.getSource());
			}
		}

		return startStates > 0;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectingTargetAllowed(ch.unizh.ifi.adora.model.representation.Node,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      org.eclipse.gef.commands.Command)
     */
	protected boolean connectingTargetAllowed(Node sourceNode,
			ConnectableElement targetElement, Command command) {
		if (command instanceof JoinRelationCreate) {
			/*
             * a join relationship may not cross the border of the aspect's
             * parent.
             */
			if (!(sourceNode.getParent() instanceof AdoraModel)
					&& !sourceNode.getParent().getParent().hasChildRecursively(
							targetElement)) {
				return false;
			}

			if (sourceNode instanceof AspectContainer) {
				if (targetElement instanceof Connection) {
					return true;
				}
				IContainer sp = JoinRelationPolicyHelper.getSourceParent(sourceNode);
				if (sp == null) {
					//i.e. Parent is AdoraModel
					sp = sourceNode.getParent();
				}
				for (ModelElement n : sp.getChildren()) {
					if (n == targetElement && n != sourceNode) {
						return true;
					} else if (n instanceof INodeContainer && n != sourceNode) {
						if (((INodeContainer) n)
								.hasChildRecursively(targetElement)) {
							return true;
						}
					}
				}
			}

			ConstraintHint
					.displayConstraintHint(IConstraintMessages.NON_FUNCTIONAL_CONNECTION);
			return false;
		}
		return true;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectingAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.Connection)
     */
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target, Connection con) {
		if (con instanceof JoinRelation && source instanceof AspectContainer) {
			if (con.getSource() == source) {
				if (target instanceof Connection)
					return true;

				for (ModelElement n : source.getParent().getChildren()) {
					if (n instanceof INodeContainer && n != source) {
						if (n == target
								|| ((INodeContainer) n)
										.hasChildRecursively(target)) {
							return true;
						}
					}
				}
			} else if (target instanceof Connection
					|| ((AspectContainer) source).hasChildRecursively(target)) {
				return true;
			}
		}
		ConstraintHint
				.displayConstraintHint(IConstraintMessages.NON_FUNCTIONAL_CONNECTION);
		return false;
	}
}