package ch.unizh.ifi.adora.parts.policies.nodes;

import java.util.logging.Logger;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.ScenarioHierarchyConstraints;
import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.commands.ScenarioConnectionCreate;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.parts.policies.JoinRelationPolicyHelper;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * This policy determines, which elements, in what way, can be connected to a
 * {@link ch.unizh.ifi.adora.model.representation.Scenario}. Since
 * {@link ch.unizh.ifi.adora.model.representation.Scenario}s always represent a
 * tree, {@link ch.unizh.ifi.adora.model.representation.Connection} restraints
 * are needed.
 * 
 * @author smeier Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 25.04.2006 SM First Version<br>
 *          23.07.2006 SM Some Reformatting.<br>
 *          25.01.2007 PS Constraints updated (Bug #806).
 * @version $Date: 2008-01-25 15:51:31 +0100 (Fri, 25 Jan 2008) $, $Author: schoeni $, $Revision: 3505 $
 */
public class ScenarioNodeEditPolicy extends ConstrainablePolicy {
	private static final Logger logger = Logger
			.getLogger(ScenarioNodeEditPolicy.class.getName());

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionAllowed(ch.unizh.ifi.adora.model.representation.Node,
     *      ch.unizh.ifi.adora.model.representation.Node,
     *      org.eclipse.gef.commands.Command)
     */
	protected boolean connectingTargetAllowed(Node sourceNode,
			ConnectableElement targetComponent, Command command) {
		if (command instanceof AssociationCreate) {
			return isAssociationTargetPossible(sourceNode, null);
		}

		if (command instanceof ScenarioConnectionCreate) {
			if (!((Scenario) sourceNode).isZoomedIn()) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.ZOOMED_SCENARIO_CONNECTION);
				return false;
			}
			if (!scenarioConnectionAllowed(targetComponent, sourceNode, false)) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.SCENARIO_TREE_HIERARCHY);
				return false;
			}
			return true;
		}

		if (command instanceof JoinRelationCreate) {
			if (sourceNode instanceof State || sourceNode instanceof Component || sourceNode instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(sourceNode,
						targetComponent);
			}

			return super.connectingTargetAllowed(sourceNode, targetComponent, command);
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionCreateAllowedd(java.lang.Class)
     */
	protected boolean creatingConnectionAllowed(Class connectionType) {
		if (connectionType == JoinRelation.class) {
			return isJoinRelationSourcePossible();
		}

		if (connectionType == Association.class) {
			return isAssociationSourcePossible(null);
		}

		// return true, if the connectionType is a scenario connection
		return connectionType == ScenarioConnection.class;
	}

	/**
     * Returns the Root {@link Scenario} for a given {@link Scenario}.
     * 
     * @param s
     *            {@link Scenario} for which the root {@link Scenario} will be
     *            returned.
     * @return Root {@link Scenario} for a given {@link Scenario}.
     */
	private Scenario getRootScenario(Scenario s) {
		if (s.getScenarioParent() == null) {
			return s;
		}
		return getRootScenario(s.getScenarioParent());
	}

	/**
     * Checks, if this node is a possible candidate for being the source of an
     * join relation.
     * 
     * @return
     */
	private boolean isJoinRelationSourcePossible() {
		Scenario scenario = (Scenario) getHost().getModel();

		/*
         * outgoing join relations must have a root scenario, placed in a aspect
         * container, as root
         */
		if (!scenario.isInsideAspectContainer()
				|| scenario.getScenarioParent() != null) {
			logger.info(IConstraintMessages.SCENARIO_JOINRELATION);
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.SCENARIO_JOINRELATION);

			return false;
		}

		/*
         * Check, if there is already a association. if true, this scenario
         * group is a is a crosscutting scenario, which can not be a scenario
         * chunk too.
         */
		if (hasAssociation(scenario)) {
			logger.info(IConstraintMessages.SCENARIO_JOINRELATION_CROSSCUTTING);
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.SCENARIO_JOINRELATION_CROSSCUTTING);

			return false;
		}

		// check also the other scenario children if there are associations
		for (Scenario scenarioChildren : scenario.getAllScenarioChildren()) {
			if (hasAssociation(scenarioChildren)) {
				logger
						.info(IConstraintMessages.SCENARIO_JOINRELATION_CROSSCUTTING);
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.SCENARIO_JOINRELATION_CROSSCUTTING);

				return false;
			}
		}

		return true;
	}

	/**
     * Checks, if a scenario has an association
     * 
     * @param scenario
     * @return
     */
	private boolean hasAssociation(Scenario scenario) {
		return getAssociationsCount(scenario) > 0;
	}

	/**
     * Counts the associations
     * 
     * @param scenario
     * @return
     */
	private int getAssociationsCount(Scenario scenario) {
		int count = 0;

		for (Connection connection : scenario.getConnections()) {
			if (connection instanceof Association) {
				count++;
			}
		}

		return count;
	}

	/**
     * Checks, if this node is a possible candidate for being the source of an
     * association.
     * 
     * @return
     */
	private boolean isAssociationSourcePossible(Node previousSource) {
		Scenario scenario = (Scenario) getHost().getModel();

		Scenario root = getRootScenario(scenario);

		/*
         * The root node of the scenariochart must be connected at least to one
         * environment object.
         */
		if (root != scenario
				&& (!hasAssociation(root) || (root == previousSource && getAssociationsCount(root) == 1))) {

			logger.info(IConstraintMessages.SCENARIO_NO_ROOT_ASSOCIATION);
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.SCENARIO_NO_ROOT_ASSOCIATION);

			return false;
		}

		/*
         * Check, if there is already a joinrelation from the scenario root.
         * then this is a scenario chunk which can not have an association.
         */
		for (Connection connection : root.getConnections()) {
			if (connection instanceof JoinRelation) {
				// this scenario group is a scenario chunk,
				// which can not be a crosscutting scenario too.
				logger.info(IConstraintMessages.SCENARIO_ASSOCIATION_CHUNK);

				ConstraintHint
						.displayConstraintHint(IConstraintMessages.SCENARIO_ASSOCIATION_CHUNK);

				return false;
			}
		}

		return true;
	}

	/**
     * Checks, if this scenario is a possible target for an association.
     * 
     * @param source
     *            the source node of the association
     * @return
     */
	@SuppressWarnings("deprecation")
	private boolean isAssociationTargetPossible(Node source, Node previousSource) {
		Scenario scenario = (Scenario) getHost().getModel();
		Scenario root = getRootScenario(scenario);

		// check, if this scenario is a scenario chunk - then forbid
		if (source instanceof Actor && scenario.isInsideAspectContainer()) {
			for (Connection connection : root.getSourceConnections()) {
				if (connection instanceof JoinRelation) {
					// this scenario group is a scenario chunk,
					// which can not be a crosscutting scenario too.
					logger.info(IConstraintMessages.SCENARIO_ASSOCIATION_CHUNK);
					ConstraintHint
							.displayConstraintHint(IConstraintMessages.SCENARIO_ASSOCIATION_CHUNK);

					return false;
				}
			}

			/*
             * The root node of the scenariochart must be connected at least to
             * one environment object.
             */
			if (root != scenario
					&& (!hasAssociation(root) || (root == previousSource && getAssociationsCount(root) == 1))) {
				logger.info(IConstraintMessages.SCENARIO_NO_ROOT_ASSOCIATION);

				ConstraintHint
						.displayConstraintHint(IConstraintMessages.SCENARIO_NO_ROOT_ASSOCIATION);

				return false;
			}

			return true;
		}

		/*
         * Allow actors only to be connected by an association to root scenario
         * nodes (no parent scenario).
         */
		if (source instanceof Actor) {
			if (scenario.getScenarioParent() != null) {
				logger.info(IConstraintMessages.ACTOR_ASSOCIATION);
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.ACTOR_ASSOCIATION);

				return false;
			}
		}

		// incoming Association from a crosscutting scenario
		if (source instanceof Actor || source instanceof Component
				&& ((Component) source).isExternal()
				&& (scenario.getParent() != source)) {
			return true;
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectionAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.Connection)
     */
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target,
			ch.unizh.ifi.adora.model.representation.Connection con) {

		// replace anchor on same node
		if (con.getSource() == source && con.getTarget() == target) {
			return true;
		}

		Scenario scenario = (Scenario) getHost().getModel();
		if (!scenario.isZoomedIn()) {
			ConstraintHint
					.displayConstraintHint(IConstraintMessages.ZOOMED_SCENARIO_CONNECTION);
			return false;
		}

		if (con instanceof Association) {
			Association association = (Association) con;

			// Check, if the association is calculated abstracted.
			if (association.isCalculatedAbstracted()) {
				logger.info(IConstraintMessages.CALC_CONNECTION);
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.CALC_CONNECTION);
				return false;
			}

			// Check, which side will be reconnected
			if (con.getSource() != source) {
				// reconnect the source of the connection to this scenario
				if (!(target instanceof Actor)) {
					logger.info("Association target is no Actor");
					return false;
				}

				return isAssociationSourcePossible(association.getSource());
			} else {
				// reconnect the target of the connection to this scenario
				if (!(source instanceof Actor)) {
					logger.info("Association source is no Actor");
					return false;
				}

				return isAssociationTargetPossible((Node) source, association
						.getTarget());
			}
		} else if (con instanceof JoinRelation) {
			if (source instanceof State || source instanceof Component || source instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(source, target);
			}
			
			JoinRelation joinRelation = (JoinRelation) con;

			/*
             * a join relationship may not cross the border of the aspect's
             * parent.
             */
			if (!(source instanceof AspectContainer)
					&& !source.getParent().getParent().equals(
							target.getParent().getParent())) {
				return false;
			}

			// Check, if the joinrelation is calculated abstracted.
			if (joinRelation.isCalculatedAbstracted()) {
				logger.info(IConstraintMessages.CALC_CONNECTION);
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.CALC_CONNECTION);
				return false;
			}

			if (con.getSource() != source) {
				// reconnect the source of the connection
				if (!isJoinRelationSourcePossible()) {
					return false;
				}
			}

			if (source instanceof Scenario && target instanceof Scenario) {
				if (!source.getParent().hasChildRecursively(target)) {
					return !((JoinRelation) con).isCalculatedAbstracted()
							&& ((Scenario) source).isInsideAspectContainer();
				}
			} else {
				return super.reconnectingAllowed(source, target, con);
			}

		} else if (con instanceof ScenarioConnection) {

			if (source instanceof Scenario && source != target
					&& target instanceof Scenario) {

				if (con.getSource() != source) {
					// only to chlidren no connection is allowed
					return !((Scenario) target).getAllScenarioChildren()
							.contains(source)
							&& (ScenarioHierarchyConstraints
									.isTargetInContainerChild(
											(Scenario) source,
											(Scenario) target));
				} else {
					return ((Scenario) target).getScenarioParent() == null
							&& !((Scenario) target).getAllScenarioChildren()
									.contains(source)
							&& (ScenarioHierarchyConstraints
									.isTargetInContainerChild(
											(Scenario) source,
											(Scenario) target));
				}
			}
		}
		return false;
	}

	/**
     * Determines, if a {@link ScenarioConnection} is allowed or not.
     * 
     * @param target
     *            Target {@link Node} of {@link ScenarioConnection}.
     * @param source
     *            Source {@link Node} of {@link ScenarioConnection}.
     * 
     * @return
     * <li>true</li>, if the {@link ScenarioConnection} is allowed
     */
	private boolean scenarioConnectionAllowed(ConnectableElement target,
			ConnectableElement source, boolean scenarioSourceReconnect) {
		Scenario scenario = (Scenario) getHost().getModel();

		if (!scenario.isZoomedIn()) {
			return false;
		}

		// If source and target have the same scenario root, a new connection
		// can't be established, since then,
		// it wouldn't be a tree anymore.
		if (source instanceof Scenario && source != target
				&& target instanceof Scenario) {
			if (getRootScenario((Scenario) source) == getRootScenario((Scenario) target)) {
				return false;
			}

			if (!scenarioSourceReconnect) {
				// does a scenario connection allready exist?
				// if so, no new connection can be appended.
				for (ch.unizh.ifi.adora.model.representation.Connection c : target
						.getTargetConnections()) {
					if (c instanceof ScenarioConnection) {
						return false;
					}
				}
			}
			// check if component parent hierarchy is allowed
			return (ScenarioHierarchyConstraints.isTargetInContainerChild(
					(Scenario) source, (Scenario) target));
		}

		return false;
	}
}