package ch.unizh.ifi.adora.parts.policies.nodes;

import java.util.logging.Logger;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.parts.policies.JoinRelationPolicyHelper;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * This policy determines, which elements, in what way, can be connected to a
 * state. Since states have a prescribed hierarchial order, additional
 * connection restraints are needed.
 * 
 * @author smeier, vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 27.09.2005 SM First Version<br>
 *          23.07.2006 SM Comments checked, reformatted and corrected.<br>
 *          31.08.2006 JS Bug 751: fixed class cast exception in
 *          creatingConnectionAllowed(Class)<br>
 *          25.01.2007 PS Constraints updated (Bug #806).
 * @version $Date: 2008-01-25 14:17:46 +0100 (Fri, 25 Jan 2008) $, $Author: schoeni $, $Revision: 3503 $
 */
public class StateNodeEditPolicy extends ConstrainablePolicy {
	private static final Logger logger = Logger
			.getLogger(ScenarioNodeEditPolicy.class.getName());

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionAllowed(ch.unizh.ifi.adora.model.representation.Node,
     *      ch.unizh.ifi.adora.model.representation.Node,
     *      org.eclipse.gef.commands.Command)
     */
	protected boolean connectingTargetAllowed(Node sourceNode,
			ConnectableElement targetComponent, Command command) {
		if (command instanceof TransitionCreate) {
			if (sourceNode.getParent() != targetComponent.getParent()) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.TRANSITION_GRAPH);
				return false;
			}

			if (sourceNode instanceof Component || sourceNode instanceof State) {
				return true;
			}
		}

		if (command instanceof JoinRelationCreate) {
			if (sourceNode instanceof State || sourceNode instanceof Component || sourceNode instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(sourceNode, targetComponent);
			}

			return super.connectingTargetAllowed(sourceNode, targetComponent,
					command);
		}

		ConstraintHint
				.displayConstraintHint(IConstraintMessages.STATE_CONNECTIONS);
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectionAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.Connection)
     */
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target, Connection con) {

		if (con instanceof JoinRelation) {
			JoinRelation joinRelation = (JoinRelation) con;

			if (source instanceof State || source instanceof Component || source instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(source, target);
			}
			
			/*
             * a join relationship may not cross the border of the aspect's
             * parent.
             */
			if (!source.getParent().getParent().equals(
					target.getParent().getParent())) {
				return false;
			}

			if (!(target instanceof Transition)) {
				return false;
			}

			if (!((State) source).isInsideAspectContainer()) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.SOURCE_STATE_CROSSCUT);
				return false;
			}

			if (source.getParent() == ((Connection) target).getSource()
					.getParent()) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.JOIN_RELATION_EXISTS);
				return false;
			}

			if (isCrosscuttingStategroup((Node) source)) {
				return false;
			}

			if (joinRelation.isCalculatedAbstracted()) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.CALC_CONNECTION);
				return false;
			}

			return true;
		}

		if (con instanceof Transition) {
			ConnectableElement otherEnd = getOtherEnd(source, target);

			if (source.getParent() != target.getParent()) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.TRANSITION_GRAPH);
				return false;
			}

			if (otherEnd instanceof State || otherEnd instanceof Component
					|| otherEnd instanceof ExitPoint) {
				return true;
			}
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionCreateAllowedd(java.lang.Class)
     */
	protected boolean creatingConnectionAllowed(Class connectionType) {
		if (connectionType == Transition.class) {
			return true;
		}

		if (connectionType == JoinRelation.class) {
			State state = (State) getHost().getModel();

			if (!state.isInsideAspectContainer()) {
				logger
						.info("can not draw a joinrelation from a state which is not inside an aspect container");
				return false;
			}

			if (isCrosscuttingStategroup(state)) {
				logger
						.info("can not draw a joinrelation from a state which is part of a crosscutting state group");
				return false;
			}

			return true;
		}

		return false;
	}
}