package ch.unizh.ifi.adora.parts.policies.nodes;

import java.util.ArrayList;
import java.util.LinkedList;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;

import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.parts.policies.JoinRelationPolicyHelper;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * This policy determines which elements can be connected to a
 * {@link ch.unizh.ifi.adora.model.representation.Transition} in what way.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * 
 * @history 24.07.2006 IV First Version<br>
 *          25.07.2006 SM Comments checked and corrected.<br>
 *          25.01.2007 PS Constraints updated (Bug #806).
 * 
 * @version $Date: 2009-03-25 12:35:26 +0100 (Wed, 25 Mar 2009) $, $Author: jehle $, $Revision: 4092 $
 */
public class TransitionNodeEditPolicy extends ConnectableElementPolicy {
	/**
     * Checks, if there is already a connection from the one to the other model
     * element.
     * 
     * @param from
     * @param to
     * @return true if there is a connection
     */
	private boolean hasConnection(ConnectableElement from, ConnectableElement to) {
		ArrayList<ConnectableElement> visitedNodes = new ArrayList<ConnectableElement>();
		LinkedList<ConnectableElement> queue = new LinkedList<ConnectableElement>();
		queue.add(from);

		while (queue.size() > 0) {
			ConnectableElement element = queue.poll();

			if (visitedNodes.contains(element)) {
				continue;
			}

			visitedNodes.add(element);

			if (element instanceof Connection) {
				queue.add(((Connection) element).getTarget());
			} else {
				Node node = (Node) element;

				if (to.equals(node)) {
					return true;
				}

				for (Connection connection : node.getConnections()) {
					queue.add(connection);
				}
			}
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionAllowed(ch.unizh.ifi.adora.model.representation.Node,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      org.eclipse.gef.commands.Command)
     */
	protected boolean connectingTargetAllowed(Node sourceNode,
			ConnectableElement targetComponent, Command command) {
		if (command instanceof JoinRelationCreate) {
			if (sourceNode instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(sourceNode, targetComponent);
			}
			
			/*
             * a join relationship may not cross the border of the aspect's
             * parent.
             */
			if (!(sourceNode instanceof AspectContainer) &&
				!sourceNode.getParent().getParent().hasChildRecursively(((Transition) targetComponent).getSource()))
			{
				return false;
			}
			
			if (sourceNode instanceof State || sourceNode instanceof Component) {
				/*
                 * a state will be connected to this transition by a
                 * joinrelation. check if there is a circular connection.
                 */
				if (hasConnection(targetComponent, sourceNode)) {
					return false;
				}

				if (((Transition) targetComponent).getSource().getParent() instanceof AdoraModel
						|| !sourceNode.getParent().hasChildRecursively(
								(Node) ((Transition) targetComponent)
										.getSource().getParent())
						&& sourceNode.getParent() != ((Transition) targetComponent)
								.getSource().getParent()) {
					return true;
				}
			} else if (sourceNode instanceof AspectContainer) {
				return true;
			}
		}

		ConstraintHint
				.displayConstraintHint(IConstraintMessages.JOIN_RELATION_EXISTS);
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectionAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.Connection)
     */
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target, Connection con) {
		if (con instanceof JoinRelation) {
			if (source instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(source, target);
			}

			/*
             * a join relationship may not cross the border of the aspect's
             * parent.
             */
			if (!(source instanceof AspectContainer)
					&& !source.getParent().getParent().hasChildRecursively(
							((Transition) target).getSource())) {
				return false;
			}

			if (source instanceof State || source instanceof Component) {
				/*
                 * a state will be connected to this transition by a
                 * joinrelation. check if there is a circular connection.
                 */
				if (hasConnection(target, source)) {
					return false;
				}

				if (!((JoinRelation) con).isCalculatedAbstracted()
						&& source.getParent() != target.getParent()
						&& (((Transition) target).getSource().getParent() instanceof AdoraModel || !source
								.getParent().hasChildRecursively(
										(Node) ((Transition) target)
												.getSource().getParent()))
						&& source.getParent() != ((Transition) target)
								.getSource().getParent()) {
					return true;
				}
			} else if (source instanceof AspectContainer) {
				return true;
			}
		}

		ConstraintHint
				.displayConstraintHint(IConstraintMessages.JOIN_RELATION_EXISTS);
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		return null;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionCreateAllowedd(java.lang.Class)
     */
	protected boolean creatingConnectionAllowed(Class connectionType) {
		return false;
	}
}