package ch.unizh.ifi.adora.parts.tree;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractTreeEditPart;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.AdoraEditPartFactory;
import ch.unizh.ifi.adora.parts.policies.AdoraComponentEditPolicy;
import ch.unizh.ifi.adora.utils.ModelElementIconFactory;

/**
 * TreeEditPart used for ADORA nodes. 
 * This is used in the Outline View of the
 * ShapesEditor.
 * <p>
 * This edit part must implement the PropertyChangeListener interface, so it can
 * be notified of property changes in the corresponding model element.
 * </p>
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			23.08.2005 SM First Version
 * 						23.07.2006 SM Comments reformatting. 
 * @version			$Date: 2006-08-24 17:29:27 +0200 (Thu, 24 Aug 2006) $, $Author: vigan $, $Revision: 2104 $
 *
 */
public class NodeTreeEditPart extends AbstractTreeEditPart implements
		PropertyChangeListener, IAdaptable {

	/**
	 * Create a new instance of this edit part using the given model element.
	 * 
	 * @param model
	 *            a non-null Shapes instance
	 */
	NodeTreeEditPart(Node model) {
		super(model);		
	}

	/**
	 * Upon activation, attach to the model element as a property change
	 * listener.
	 */
	public void activate() {
		if (!isActive()) {
			super.activate();
			((ObservableModelElement) getModel()).addPropertyChangeListener(this);
        }
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// allow removal of the associated model element
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new AdoraComponentEditPolicy());
	}

	/**
	 * Upon deactivation, detach from the model element as a property change
	 * listener.
	 */
	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			((ObservableModelElement) getModel()).removePropertyChangeListener(this);
		}
	}

	/**
	 * Returns the casted model for this edit part.
	 * @pre       true
	 * @post      return != null
	 */
	private Node getCastedModel() {
		return (Node) getModel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getImage()
	 */
	protected Image getImage() {
		//return ModelElementIconFactory.INSTANCE.getIcon(getCastedModel().getClass(),getCastedModel().isVisible());
        return ModelElementIconFactory.INSTANCE.getIcon(getCastedModel(),getCastedModel().isVisible());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getText()
	 */
	protected String getText() {
		return getCastedModel().toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		if (ZoomableNodeContainer.CHILD_ADDED_PROP.equals(prop)) {
			// add a child to this edit part
			// causes an additional entry to appear in the tree of the outline
			// view
			addChild(createChild(evt.getNewValue()), -1);
		} else if (ZoomableNodeContainer.CHILD_REMOVED_PROP.equals(prop)) {
			// remove a child from this edit part
			// causes the corresponding edit part to disappear from the tree in
			// the outline view
			removeChild(getEditPartForChild(evt.getNewValue()));
		} else {
			refreshVisuals();
		}
	}
	
	/**
	 * Returns the children of this from the model,
	 * as this is capable enough of holding EditParts.
	 *
	 * @return  List of children.
	 */
	protected List getModelChildren() {
		if (getModel() instanceof INodeContainer) {
			return ((INodeContainer)getModel()).getChildren();
		} else {
			return Collections.EMPTY_LIST;
		}
	}
	
	/**
	 * Convenience method that returns the EditPart corresponding to a given
	 * child.
	 * 
	 * @param child
	 *            a model element instance
	 * @return the corresponding EditPart or null
	 */
	private EditPart getEditPartForChild(Object child) {
		return (EditPart) getViewer().getEditPartRegistry().get(child);
	}
	
	/**
	 * Returns the IPropertySource for the selected modelelement.
	 * 
	 * @pre       true
	 * @post      return != null
	 * @param adapter The adapter class object.
	 * @return Returns the IPropertySource for this ModelElement
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			return ((IAdaptable) new AdoraEditPartFactory().createEditPart(null, getModel())).getAdapter(adapter);
		}
		return null;
	}
}
