package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.CardinalityPropNames;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.properties.validators.cardinality.MaxCardinalityValidator;
import ch.unizh.ifi.adora.properties.validators.cardinality.MinCardinalityValidator;

/**
 * This PropertySource object handels all property request (get & set) for an
 * Actor object. Also all necessary PropertyDescriptors are instantiated here.
 * 
 * @author vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Feb 28, 2006 IV First Version Aug 8, 2006 SM Refactoring.
 * @version $Date: 2006-09-28 16:05:58 +0200 (Thu, 28 Sep 2006) $, $Author: vigan $, $Revision: 2297 $
 */
public class ActorProperties extends NodeProperties implements
		CardinalityPropNames {

	/**
     * Constructs a new ActorProperties object for a given actor object.
     * 
     * @param actor
     *            Actor, for which this IPropertySource object is been
     *            instantiated.
     */
	public ActorProperties(Actor actor) {
		super(actor);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.properties.NodeProperties#getLocalDescriptors()
     */
	protected IPropertyDescriptor[] getLocalDescriptors() {

		return null;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
	protected IPropertyDescriptor[] createProperties() {
		IPropertyDescriptor[] desc = new IPropertyDescriptor[] {
				new TextPropertyDescriptor(MINCARD_PROP, "Min Cardinality"),
				new TextPropertyDescriptor(MAXCARD_PROP, "Max Cardinality") };
			((PropertyDescriptor) desc[0])
					.setValidator(new MinCardinalityValidator(getModel().getCardinality()));
			((PropertyDescriptor) desc[0]).setCategory("Cardinality");
			((PropertyDescriptor) desc[1])
				.setValidator(new MaxCardinalityValidator(getModel().getCardinality()));
			((PropertyDescriptor) desc[1]).setCategory("Cardinality");

			setHelpId(desc, PROPERTIES_HELP_CONTEXT);

		return this
				.getJoinedPropertyDescriptors(super.createProperties(), desc);
	}

	/**
     * Returns the property value for the given propertyId, or null.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to
     * obtain the value of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public Object getPropertyValue(Object propertyId) {
		if (MAXCARD_PROP.equals(propertyId)) {
			return getModel().getMaxCardinality();
		}
		if (MINCARD_PROP.equals(propertyId)) {
			return getModel().getMinCardinality();
		}
		return super.getPropertyValue(propertyId);
	}

	/**
     * Sets the property value for the given property id. If no matching id is
     * found, the call is forwarded to the superclass.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to set
     * the values of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public void setPropertyValue(Object propertyId, Object value) {
		if (MINCARD_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class, 
					"setMinCardinality", "getMinCardinality");
			
		} else if (MAXCARD_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class, 
					"setMaxCardinality", "getMaxCardinality");
		}
		super.setPropertyValue(propertyId, value);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
	 */
	protected Actor getModel(){
		return (Actor)super.getModel();
	}
}
