package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.AbstractableConnectionPropNames;
import ch.unizh.ifi.adora.model.properties.AssociationPropNames;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.properties.validators.cardinality.MaxCardinalityValidator;
import ch.unizh.ifi.adora.properties.validators.cardinality.MinCardinalityValidator;

/**
 * This PropertySource object handels all property request (get & set)
 * for an association object. Also all necessary PropertyDescriptors are instantiated
 * here. 

 * @author		       Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             06.03.2006 IV First Version
 *                      12.05.2006 SM Help context ids
 *                      08.08.2006 SM Refactoring.
 *                      18.08.2006 SM Solved Bug with association properties.
 *                      25.09.2007 CR changed order of categories
 * @version             $Date: 2007-09-25 17:37:46 +0200 (Tue, 25 Sep 2007) $, $Author: cramer $, $Revision: 3011 $
 */
public class AssociationProperties extends ConnectionProperties  
    implements AssociationPropNames, AbstractableConnectionPropNames{

    /**
     * Stores the logical pair of variates; 
     */
    protected final static String[] boolValues = {"false","true"};
   
	  /**
     * Creates an association properties object which is the source for 
     * the property sheet. 
     * @param association The association model element which delivers the 
     *        content for the properties.
     */
	public AssociationProperties(Association association) {
		super(association);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	protected IPropertyDescriptor[] createProperties() {
		IPropertyDescriptor[] returnDesc = new IPropertyDescriptor[7];

		returnDesc[0] = new TextPropertyDescriptor(NAME_TO_PROP, "Label");
		((PropertyDescriptor) returnDesc[0])
				.setCategory("Destination to source");

		returnDesc[1] = new TextPropertyDescriptor(MINCARD_TO_PROP,
			"Min Cardinality");
		((PropertyDescriptor) returnDesc[1])
				.setCategory("Destination to source");

		returnDesc[2] = new TextPropertyDescriptor(MAXCARD_TO_PROP,
			"Max Cardinality");
		((PropertyDescriptor) returnDesc[2])
		.setCategory("Destination to source");

		returnDesc[3] = new TextPropertyDescriptor(NAME_FROM_PROP, "Label");
		((PropertyDescriptor) returnDesc[3])
					.setCategory("Source to destination");

		returnDesc[4] = new TextPropertyDescriptor(MINCARD_FROM_PROP,
				"Min Cardinality");
		((PropertyDescriptor) returnDesc[4])
				.setCategory("Source to destination");
		
		returnDesc[5] = new TextPropertyDescriptor(MAXCARD_FROM_PROP,
				"Max Cardinality");
		((PropertyDescriptor) returnDesc[5])
				.setCategory("Source to destination");
		
		
		returnDesc[6] = new ComboBoxPropertyDescriptor(MAN_ABSTRACTED_PROP,
					"Manually Abstracted", boolValues);

		
		// Set NumericValidator for the TextPropertyDescriptor
		((TextPropertyDescriptor) returnDesc[4]).setValidator(
				new MinCardinalityValidator(getModel().getRoleFrom().getCardinality()));
		((TextPropertyDescriptor) returnDesc[5]).setValidator(
				new MaxCardinalityValidator(getModel().getRoleFrom().getCardinality()));
		((TextPropertyDescriptor) returnDesc[1]).setValidator(
				new MinCardinalityValidator(getModel().getRoleTo().getCardinality()));
		((TextPropertyDescriptor) returnDesc[2]).setValidator(
				new MaxCardinalityValidator(getModel().getRoleTo().getCardinality()));
		
		
		setHelpId(returnDesc, PROPERTIES_HELP_CONTEXT);
		return this.getJoinedPropertyDescriptors(super.createProperties(), returnDesc);
	}
	
	/**
     * Return the property value for the given propertyId, or null.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to
     * obtain the value of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public Object getPropertyValue(Object propertyId) {
		if (MAXCARD_FROM_PROP.equals(propertyId)) {
				return getModel().getRoleFrom().getMaxCard();
		}
		if (MINCARD_FROM_PROP.equals(propertyId)) {
			return getModel().getRoleFrom().getMinCard();
		}
		if (MAXCARD_TO_PROP.equals(propertyId)) {
			return getModel().getRoleTo().getMaxCard();
		}
		if (MINCARD_TO_PROP.equals(propertyId)) {
			return getModel().getRoleTo().getMinCard();
		}
		if (MAN_ABSTRACTED_PROP.equals(propertyId)) {
            return new Integer(getModel().isManuallyAbstracted() ? 1:0);
        }
        if (NAME_FROM_PROP.equals(propertyId)) {
			return getModel().getRoleFrom().getName();
		}
		if (NAME_TO_PROP.equals(propertyId)) {
			return getModel().getRoleTo().getName();
		}
	 	return super.getPropertyValue(propertyId);
	}

	/**
	 * Set the property value for the given property id. If no matching id is
	 * found, the call is forwarded to the superclass.
	 * <p>
	 * The property view uses the IDs from the IPropertyDescriptors array to set
	 * the values of the corresponding properties.
	 * </p>
	 * 
	 * @see #descriptors
	 * @see #getPropertyDescriptors()
	 */
	public void setPropertyValue(Object propertyId, Object value) {
		if (MINCARD_FROM_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel().getRoleFrom(), value, 
    				String.class, "setMinCard", "getMinCard");
		} 
		else if (MAXCARD_FROM_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel().getRoleFrom(), value, 
    				String.class, "setMaxCard", "getMaxCard");
		}
		else if (MINCARD_TO_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel().getRoleTo(), value, 
    				String.class, "setMinCard", "getMinCard");
		} 
		else if (MAXCARD_TO_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel().getRoleTo(), value, 
    				String.class, "setMaxCard", "getMaxCard");
		}
		else if (MAN_ABSTRACTED_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer)value).intValue()>0, 
    				boolean.class, "setManuallyAbstracted", "isManuallyAbstracted");
		}
		else if (NAME_TO_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel().getRoleTo(), value, 
    				String.class, "setName", "getName");

		}
		else if (NAME_FROM_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel().getRoleFrom(), value, 
    				String.class, "setName", "getName");
		}
		super.setPropertyValue(propertyId, value);
	}	
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
	 */
	protected Association getModel(){
		return (Association)super.getModel();
	}
}