package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.CardinalityPropNames;
import ch.unizh.ifi.adora.model.properties.ComponentPropNames;
import ch.unizh.ifi.adora.model.representation.AdoraType;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.RootType;
import ch.unizh.ifi.adora.properties.longtextproperty.LongTextPropertyDescriptor;
import ch.unizh.ifi.adora.properties.specification.ComponentSpecificationPropertyAccess;
import ch.unizh.ifi.adora.properties.specification.ExtendedSpecificationPropNames;
import ch.unizh.ifi.adora.properties.validators.cardinality.MaxCardinalityValidator;
import ch.unizh.ifi.adora.properties.validators.cardinality.MinCardinalityValidator;
import ch.unizh.ifi.adora.properties.validators.component.ExternalValidator;
import ch.unizh.ifi.adora.properties.validators.component.TypeValidator;

/**
 * This PropertySource object handels all property request (get & set) for a
 * Component object. Also all necessary PropertyDescriptors are instantiated
 * here.
 * 
 * @author 		vigan
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history   	Feb 28, 2006 First Version 
 *            	Aug 8, 2006 SM Refactoring.
 *            	Sept 20, 2007 SM Call to the syntax view refactord
 *              Apr 09, 2008 CJ Address Bug 855
 *            
 * @version $Date: 2008-04-16 10:45:42 +0200 (Wed, 16 Apr 2008) $, $Author: jeanneret $, $Revision: 3598 $
 */
public class ComponentProperties extends StartStateContainerProperties
		implements 
		ComponentPropNames, CardinalityPropNames, ExtendedSpecificationPropNames {

	/**
     * The hard coded types. Will be extended in future by a name list coming
     * from the type hierarchy.;
     */
	protected static final String[] adoraTypes = { "none", "root" };

	/**
     * Constructs a new ComponentProperties object for a given component object.
     * 
     * @param component
     *            Object on which propertie changes are applied.
     */
	public ComponentProperties(Component component) {
		super(component);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
	protected IPropertyDescriptor[] createProperties() {
		LongTextPropertyDescriptor spec = new LongTextPropertyDescriptor(
				SPECIFICATION_ACCESS_PROP, SPECIFICATION_PROP_NAME);
		IPropertyDescriptor[] desc = new IPropertyDescriptor[] {
				new TextPropertyDescriptor(MINCARD_PROP, "Min Cardinality"),
				new TextPropertyDescriptor(MAXCARD_PROP, "Max Cardinality"),
				new ComboBoxPropertyDescriptor(ISEXTERNAL_PROP, "External",
						boolValues),
				new ComboBoxPropertyDescriptor(ADORA_TYPE_PROP, "Type",
						adoraTypes), spec };

		((PropertyDescriptor) desc[0])
				.setValidator(new MinCardinalityValidator(getModel()
						.getCardinality()));
		((PropertyDescriptor) desc[1])
				.setValidator(new MaxCardinalityValidator(getModel()
						.getCardinality()));

		((PropertyDescriptor) desc[2]).setValidator(new ExternalValidator(
				getModel()));
		((PropertyDescriptor) desc[3]).setValidator(new TypeValidator(
				getModel()));

		for (int i = 0; i < 2; i++) {
			((PropertyDescriptor) desc[i]).setCategory("Cardinality");
		}
		setHelpId(desc, PROPERTIES_HELP_CONTEXT);
		return this
				.getJoinedPropertyDescriptors(super.createProperties(), desc);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
     */
	public Object getPropertyValue(Object propertyId) {
		if (ISEXTERNAL_PROP.equals(propertyId)) {
			return new Integer(getModel().isExternal() ? 1 : 0);
		}
		if (ADORA_TYPE_PROP.equals(propertyId)) {
			return getModel().getType() == null ? 0 : 1;
		}
		if (SPECIFICATION_PROP.equals(propertyId)) {
			return getModel().getSpecification();
		}
		//Bug Fix 855: This property's value is "wrapped" in a ILongTextPropertyAccess
		if (SPECIFICATION_ACCESS_PROP.equals(propertyId)) {
			return new ComponentSpecificationPropertyAccess(getModel(),this);
		}
		if (MAXCARD_PROP.equals(propertyId)) {
			return getModel().getMaxCardinality();
		}
		if (MINCARD_PROP.equals(propertyId)) {
			return getModel().getMinCardinality();
		}
		return super.getPropertyValue(propertyId);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
     *      java.lang.Object)
     */
	public void setPropertyValue(Object propertyId, Object value) {
		if (ISEXTERNAL_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer) value).intValue() > 0,
					boolean.class, "setExternal", "isExternal");
		} else if (ADORA_TYPE_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(),
					((Integer) value).intValue() > 0 ? new RootType() : null,
					AdoraType.class, "setType", "getType");
		} else if (SPECIFICATION_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class,
					"setSpecification", "getSpecification", SPECIFICATION_PROP_NAME);

			//SyntaxProblemsView.getInstance().checkForSyntaxErrors(getModel());
		} else if (MINCARD_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class,
					"setMinCardinality", "getMinCardinality");
		} else if (MAXCARD_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class,
					"setMaxCardinality", "getMaxCardinality");
		}
		super.setPropertyValue(propertyId, value);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
     */
	protected Component getModel() {
		return (Component) super.getModel();
	}
}