
package ch.unizh.ifi.adora.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.properties.AbstractableConnectionPropNames;
import ch.unizh.ifi.adora.model.properties.JoinRelationPropNames;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.properties.contextmap.ExtendedContextMapPropNames;
import ch.unizh.ifi.adora.properties.contextmap.JoinRelationContextMapPropertyAccess;
import ch.unizh.ifi.adora.properties.longtextproperty.LongTextPropertyDescriptor;
import ch.unizh.ifi.adora.properties.validators.joinrelation.WeavingTypeValidator;

/**
 * This PropertySource object handels all property request (get & set) for an
 * JoinRelation object. Also all necessary PropertyDescriptors are instantiated
 * here.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 06.03.2006 IV First Version 12.05.2006 SM Help context ids
 *          08.08.2006 SM Refactoring.
 *          15.04.2008 CJ Fix Bug 855 (and refactoring of long text property editors)
 * @version $Date: 2009-02-09 17:47:30 +0100 (Mon, 09 Feb 2009) $, $Author: vigan $, $Revision: 4000 $
 */
public class JoinRelationProperties extends ConnectionProperties implements
		JoinRelationPropNames, AbstractableConnectionPropNames,
		IPropertySource,
		ExtendedContextMapPropNames {

	/**
     * Stores the logical pair of variates;
     */
	protected final static String[] boolValues = { "false", "true" };

	/**
     * Stores the priority values shown in the combo box.
     */
	protected static final String[] priorityValues = { "none", "1", "2", "3",
			"4", "5", "6", "7", "8", "9", "10" };

	/**
     * Stores the weaving type values shown in the combo box.
     */
	protected static final String[] weavingTypeValues = { "none", "before",
			"instead", "after" };

	/**
     * Creates an association properties object which is the source for the
     * property sheet.
     * 
     * @param association
     *            The association model element which delivers the content for
     *            the properties.
     */
	public JoinRelationProperties(JoinRelation joinRelation) {
		super(joinRelation);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
	protected IPropertyDescriptor[] createProperties() {
		LongTextPropertyDescriptor contextMap = new LongTextPropertyDescriptor(
				CONTEXT_MAP_ACCESS_PROP, CONTEXTMAP_PROP_NAME);

		IPropertyDescriptor[] returnDesc = new IPropertyDescriptor[5];

		returnDesc[0] = new ComboBoxPropertyDescriptor(WEAVING_PRIORITY_PROP,
				"Weaving Priority", priorityValues);
		((PropertyDescriptor) returnDesc[0]).setCategory("General");

		returnDesc[1] = new ComboBoxPropertyDescriptor(WEAVING_TYPE_PROP,
				"Weaving Type", weavingTypeValues);
		((PropertyDescriptor) returnDesc[1]).setCategory("General");
		((PropertyDescriptor) returnDesc[1])
				.setValidator(new WeavingTypeValidator(getModel()));

		returnDesc[2] = new TextPropertyDescriptor(DECISION_SET_PROP, "Decision");
		((PropertyDescriptor) returnDesc[2]).setCategory("General");
		
		
		returnDesc[3] = new ComboBoxPropertyDescriptor(MAN_ABSTRACTED_PROP,
				"Manually Abstracted", boolValues);

		returnDesc[4] = contextMap;

		setHelpId(returnDesc, PROPERTIES_HELP_CONTEXT);
		return this.getJoinedPropertyDescriptors(super.createProperties(),
				returnDesc);
	}

	/**
     * Return the property value for the given propertyId, or null.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to
     * obtain the value of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public Object getPropertyValue(Object propertyId) {
		if (WEAVING_PRIORITY_PROP.equals(propertyId)) {
			return new Integer(getModel().getPriority());
		}
		if (WEAVING_TYPE_PROP.equals(propertyId)) {
			return new Integer(getModel().getWeavingType());
		}
		if (MAN_ABSTRACTED_PROP.equals(propertyId)) {
			return new Integer(getModel().isManuallyAbstracted() ? 1 : 0);
		}
		if (CONTEXT_MAP_PROP.equals(propertyId)) {
			return new String(getModel().getContextMap());
		}
		if (CONTEXT_MAP_ACCESS_PROP.equals(propertyId)) {
			return new JoinRelationContextMapPropertyAccess(getModel(),this);
		}
		if (DECISION_SET_PROP.equals(propertyId)) {
			if (getModel().getAbstractedDecNrs() != null)
				return getModel().getAbstractedDecNrs();
			return getModel().getDecisionNr();
		}
		return super.getPropertyValue(propertyId);
	}

	/**
     * Set the property value for the given property id. If no matching id is
     * found, the call is forwarded to the superclass.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to set
     * the values of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public void setPropertyValue(Object propertyId, Object value) {
		if (WEAVING_PRIORITY_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, int.class, "setPriority",
					"getPriority");
		} else if (WEAVING_TYPE_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, int.class,
					"setWeavingType", "getWeavingType");
		} else if (MAN_ABSTRACTED_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer) value).intValue() > 0,
					boolean.class, "setManuallyAbstracted",
					"isManuallyAbstracted");
		} else if (CONTEXT_MAP_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class,
					"setContextMap", "getContextMap", CONTEXTMAP_PROP_NAME);
		} else if (DECISION_SET_PROP.equals(propertyId)) {
				Decision decision = getModel().getDecision();
				if (getModel().getDecision() != null &&
						((String)value).toLowerCase().matches("d\\d+") &&
						!((String)value).toLowerCase().matches("d0+")) {
					
					String confConstr = "";
					for (VariationPoint vp : AdoraPlugin.getActiveModel().getVariationPoints()) {
						if (vp.getDecisions().contains(decision)) {
							confConstr += "VP"+vp.getNr()+" ";
						}
					}
					for (Constraint c : AdoraPlugin.getActiveModel().getConstraints()) {
						if (c.containsDecision(decision)) {
							confConstr += "C"+c.getNr()+" ";
						}
					}
					
					if (!confConstr.equals("")) {
						showWarning(decision.getNr(), confConstr);
						return;
					}
					
					putNewCommandOnStack(getModel(), value, 
					String.class, "setDecisionNr", "getDecisionNr");
				} 
		}
		super.setPropertyValue(propertyId, value);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
     */
	public Object getEditableValue() {
		return null;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
     */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
     */
	public void resetPropertyValue(Object id) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
     */
	protected JoinRelation getModel() {
		return (JoinRelation) super.getModel();
	}
	
	/**
	 * 
	 * 
	 * @param decNr The {@link Decision} which is conflicting
	 * @param conf The {@link Constraint}s and {@link VariationPoint}s in 
	 * which the {@link Decision} appears.
	 */
	private void showWarning(int decNr, String conf) {
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench().
				getActiveWorkbenchWindow().getShell(), 
						SWT.ICON_INFORMATION | SWT.OK);
		mb.setText("Decision Conflict");
		mb.setMessage("The Decision D"+ decNr+ " appears in: "+conf);
	    mb.open();
	}
}