package ch.unizh.ifi.adora.properties;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.CardinalityPropNames;
import ch.unizh.ifi.adora.model.properties.ScenarioPropNames;
import ch.unizh.ifi.adora.model.properties.ZoomablePropNames;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.properties.validators.cardinality.MaxCardinalityValidator;
import ch.unizh.ifi.adora.properties.validators.cardinality.MinCardinalityValidator;
import ch.unizh.ifi.adora.properties.validators.scenario.SeqNoValidator;

/**
 * This PropertySource object handels all property request (get & set) for a
 * scenario object. Also all necessary PropertyDescriptors are instantiated
 * here.
 * 
 * @author 		Ivo Vigan
 * @copyright	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	06.03.2006 SM First Version 
 * 				08.08.2006 SM Refactoring. 
 * 				22.09.2006 SM Bug 772 fixed.
 *              20.09.2007 SM Refactoring call to the syntax problem view.
 * @version $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class ScenarioProperties extends NodeProperties implements
		ScenarioPropNames, CardinalityPropNames, ZoomablePropNames {

	/**
     * Displayed values in the "Type" combobox.
     */
	protected final static String[] typeValues = { "Sequence", "Alternativ",
			"Concurrency", "Root" };

	/**
     * Constructs a new ScenarioProperties object for a given scenario object.
     * 
     * @param scenario
     *            Scenario, for which this IPropertySource object is been
     *            instantiated.
     */
	public ScenarioProperties(Scenario scenario) {
		super(scenario);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
	protected IPropertyDescriptor[] createProperties() {
		Collection<PropertyDescriptor> list = new ArrayList<PropertyDescriptor>();
		list.add(new ComboBoxPropertyDescriptor(ISPARTIAL_PROP, "Partial",
				boolValues));
		list.add(new PropertyDescriptor(ZOOMED_SCENARIO_PROP, "Zoomed"));
		list.add(new ComboBoxPropertyDescriptor(ITERATOR_PROP, "Iterator",
				boolValues));
		list.add(new TextPropertyDescriptor(GUARD_PROP, "Guard"));
		list.add(new ComboBoxPropertyDescriptor(TYPE_PROP, "Type", typeValues));
		list.add(new TextPropertyDescriptor(TRANSFORMATION_PROP, "Transform"));

		TextPropertyDescriptor minCard = new TextPropertyDescriptor(
				MINCARD_PROP, "Min Cardinality");
		minCard.setValidator(new MinCardinalityValidator(getModel()
				.getCardinality()));

		list.add(minCard);

		TextPropertyDescriptor maxCard = new TextPropertyDescriptor(
				MAXCARD_PROP, "Max Cardinality");
		maxCard.setValidator(new MaxCardinalityValidator(getModel()
				.getCardinality()));
		list.add(maxCard);

		TextPropertyDescriptor seqNrDesc = new TextPropertyDescriptor(
				SEQNR_PROP, "Seq Nr");
		seqNrDesc.setValidator(new SeqNoValidator(getModel()));
		list.add(seqNrDesc);

		Iterator<PropertyDescriptor> iter = list.iterator();
		IPropertyDescriptor[] desc = new IPropertyDescriptor[list.size()];
		for (int i = 0; iter.hasNext(); i++) {
			desc[i] = iter.next();
		}
		setHelpId(desc, PROPERTIES_HELP_CONTEXT);

		return this
				.getJoinedPropertyDescriptors(super.createProperties(), desc);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
     */
	public Object getPropertyValue(Object propertyId) {
		if (ITERATOR_PROP.equals(propertyId)) {
			return new Integer(getModel().isIterator() ? 1 : 0);
		}
		if (TYPE_PROP.equals(propertyId)) {
			return new Integer(getModel().getScenarioType());
		}
		if (GUARD_PROP.equals(propertyId)) {
			return getModel().getGuard();
		}
		if (ISPARTIAL_PROP.equals(propertyId)) {
			return new Integer(getModel().isPartial() ? 1 : 0);
		}
		if (SEQNR_PROP.equals(propertyId)) {
			return Integer.toString(getModel().getSeqNo());
		}
		if (TRANSFORMATION_PROP.equals(propertyId)) {
			return getModel().getTransformation();
		}
		if (ZOOMED_SCENARIO_PROP.equals(propertyId)) {
			return Boolean.toString(getModel().isZoomedIn());
		}
		if (MAXCARD_PROP.equals(propertyId)) {
			return getModel().getMaxCardinality();
		}
		if (MINCARD_PROP.equals(propertyId)) {
			return getModel().getMinCardinality();
		}
		return super.getPropertyValue(propertyId);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
     *      java.lang.Object)
     */
	public void setPropertyValue(Object propertyId, Object value) {
		if (ITERATOR_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer) value).intValue() > 0,
					boolean.class, "setIterator", "isIterator");
		} else if (TYPE_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer) value).intValue(),
					int.class, "setScenarioType", "getScenarioType");
		} else if (ISPARTIAL_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer) value).intValue() > 0,
					boolean.class, "setPartial", "isPartial");
		} else if (GUARD_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class, "setGuard",
					"getGuard");

			//SyntaxProblemsView.getInstance().checkForSyntaxErrors(getModel());
		} else if (SEQNR_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), Integer.parseInt((String) value),
					int.class, "setSeqNr", "getSeqNo");
		} else if (TRANSFORMATION_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class,
					"setTransformation", "getTransformation");

			//SyntaxProblemsView.getInstance().checkForSyntaxErrors(getModel());
		} else if (MINCARD_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class,
					"setMinCardinality", "getMinCardinality");
		} else if (MAXCARD_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class,
					"setMaxCardinality", "getMaxCardinality");
		}
		
		super.setPropertyValue(propertyId, value);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
     */
	protected Scenario getModel() {
		return (Scenario) super.getModel();
	}
}