package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * Provides the base implementation for property sources.
 * 
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history May 12, 2006 SM First Version (help context ids). Aug 08, 2006 SM
 *          Refactoring.
 * 
 * @version $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 1.18
 *          $
 * 
 */
public abstract class VisualizableModelElementProperties implements
		IPropertySource {

	/**
	 * The general context id of the properties.
	 */
	public static final String PROPERTIES_HELP_CONTEXT = "ch.unizh.ifi.adora.editor.Properties";

	/**
	 * Holds a reference to the created property descriptors, as the implemented
	 * interface put the requirement to cache them.
	 */
	private IPropertyDescriptor[] propertyDescriptors;

	/**
	 * Stores the modelelement for which the porperties are set.
	 */
	private VisualizableModelElement element;

	// /
	private SetProperty oldSetPropertyCommand = null;

	// /

	/**
	 * Constructor
	 * 
	 * @param element
	 *            The modelelement for which the porperties are set.
	 */
	public VisualizableModelElementProperties(VisualizableModelElement element) {
		oldSetPropertyCommand=null;
		this.element = element;
		propertyDescriptors = createProperties();
	}

	/**
	 * Returns newly instantiated property descriptors.
	 * 
	 * @return Returns newly instantiated property descriptors.
	 */
	protected IPropertyDescriptor[] createProperties() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public abstract Object getEditableValue();

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return propertyDescriptors;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java
	 * .lang.Object)
	 */
	public abstract Object getPropertyValue(Object id);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang
	 * .Object)
	 */
	public abstract boolean isPropertySet(Object id);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java
	 * .lang.Object)
	 */
	public abstract void resetPropertyValue(Object id);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java
	 * .lang.Object, java.lang.Object)
	 */
	public abstract void setPropertyValue(Object id, Object value);

	/**
	 * Sets the help id of the given properties with a given help context id.
	 * 
	 * @param propertyDescriptors
	 *            The array of proprty descriptors
	 * @param helpid
	 *            The unique help id for the property.
	 */
	protected void setHelpId(IPropertyDescriptor[] propertyDescriptors,
			String helpId) {
		for (int i = 0; i < propertyDescriptors.length - 1; i++) {
			PropertyDescriptor pd = ((PropertyDescriptor) propertyDescriptors[i]);
			pd.setHelpContextIds(helpId);
		}
	}

	/**
	 * Joins two given arrays of property descriptors together. If both
	 * parameters are <i>null</i>, the method returns <i>null</i>.
	 * 
	 * @param array1
	 *            The first array that will be joined with the second array. The
	 *            array can also be null.
	 * @param array2
	 *            The second array that will be joined with the first array. The
	 *            array can also be null.
	 * @return Returns array1 and array2 joined in one array of property
	 *         descriptors.
	 */
	protected IPropertyDescriptor[] getJoinedPropertyDescriptors(
			IPropertyDescriptor[] array1, IPropertyDescriptor[] array2) {

		IPropertyDescriptor[] propertyDescriptors = null;

		// The case that both are null
		if (((array1 != null) || (array2 != null))) {
			// get the length of the first block to copy
			int propertyDescLength = array1.length;

			// if block 1 is null --> make it to block two
			if (array1 == null) {
				array1 = array2;
				array2 = null;
			}

			// Do super Descriptors exist?
			if (array2 != null) {
				propertyDescLength += array2.length;
			}
			// Instantiate new Array for local and super Desc.
			propertyDescriptors = new IPropertyDescriptor[propertyDescLength];

			// Copies all Descriptors of this class into the propertyDescriptors
			// array.
			System.arraycopy(array1, 0, propertyDescriptors, 0, array1.length);

			if (array2 != null) {
				// Copies all Descriptors of the super class into the
				// propertyDescriptors array.
				System.arraycopy(array2, 0, propertyDescriptors, array1.length,
						array2.length);
			}
		}
		return propertyDescriptors;
	}

	/**
	 * Instantiates and Executes a {@link SetProperty} Command on the
	 * {@link org.eclipse.gef.commands.CommandStack}.
	 * 
	 * @param modelElement
	 *            The VisualizableModelElement of whom an attribute changed.
	 * @param newValue
	 *            The new value of the attribute.
	 * @param setType
	 *            The type of the parameter used in the setMethod of the
	 *            <code>modelElement</code>.
	 * @param setMethod
	 *            The setMethod of the <code>modelElement</code>.
	 * @param getMethod
	 *            The getMethod of the <code>modelElement</code>. Used for
	 *            undoing the changes.
	 */
	protected void putNewCommandOnStack(VisualizableModelElement modelElement,
			Object newValue, Class setType, String setMethod, String getMethod) {

		SetProperty setPropertyCmd = null;

		try {
			setPropertyCmd = new SetProperty(modelElement, newValue,
					modelElement.getClass().getMethod(setMethod, setType),
					modelElement.getClass().getMethod(getMethod));
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}
		AdoraModelEditor editor = getEditor();
		if (editor != null && oldSetPropertyCommand == null) {
			oldSetPropertyCommand = setPropertyCmd;
			editor.getCommandStack().execute(setPropertyCmd);
		} else {
			if (editor != null
					&& !(setPropertyCmd.getModelElement()).equals(oldSetPropertyCommand.getModelElement())) {
				oldSetPropertyCommand = setPropertyCmd;
				editor.getCommandStack().execute(setPropertyCmd);
			}
			else{
				oldSetPropertyCommand=null;
			}
		}
	}

	/**
	 * Instantiates and Executes a {@link SetProperty} Command on the
	 * {@link org.eclipse.gef.commands.CommandStack}.
	 * 
	 * @param modelElement
	 *            The VisualizableModelElement of whom an attribute changed.
	 * @param newValue
	 *            The new value of the attribute.
	 * @param setType
	 *            The type of the parameter used in the setMethod of the
	 *            <code>modelElement</code>.
	 * @param setMethod
	 *            The setMethod of the <code>modelElement</code>.
	 * @param getMethod
	 *            The getMethod of the <code>modelElement</code>. Used for
	 *            undoing the changes.
	 * @param propertyName
	 *            The name of the property being set (for the command label)
	 */
	protected void putNewCommandOnStack(VisualizableModelElement modelElement,
			Object newValue, Class setType, String setMethod, String getMethod,
			String propertyName) {

		SetProperty setPropertyCmd = null;

		try {
			setPropertyCmd = new SetProperty(modelElement, newValue,
					modelElement.getClass().getMethod(setMethod, setType),
					modelElement.getClass().getMethod(getMethod), propertyName);
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}
		AdoraModelEditor editor = getEditor();
		if (editor != null) {
			editor.getCommandStack().execute(setPropertyCmd);
		}
	}

	/**
	 * Used to obtain the AdoraModelEditor.
	 * 
	 * @return The AdoraModelEditor.
	 */
	private AdoraModelEditor getEditor() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			IEditorReference editors[] = page.getEditorReferences();
			for (int i = 0; i < editors.length; i++) {
				if (editors[i].getEditor(false) instanceof AdoraModelEditor) {
					return (AdoraModelEditor) editors[i].getEditor(false);
				}
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}

	/**
	 * Returns the casted model, correspondent to the hierachy level.
	 * 
	 * @return The correpsondent model.
	 */
	protected VisualizableModelElement getModel() {
		return element;
	}
}