package ch.unizh.ifi.adora.properties.longtextproperty;

import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess;

/**
 * These data objects conform to the {@link ILongTextPropertyAccess} interface.
 * 
 * To implement this interface, a reference to the model element is kept by objects.
 *  They also rely on a {@link IPropertySource} object, to make sure that modifications 
 *  to the long text property go through a command stack.
 *  
 *  Make sure that instances of this class can be adapted to {@link AbstractLongTextEditorInput}.
 * 
 * @author				Cedric Jeanneret
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				15.04.2008 CJ First Version (Fix Bug 855 and refactoring of long text property editors)
 * @version				$Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 *
 */
public abstract class AbstractLongTextPropertyAccess implements ILongTextPropertyAccess {
	
	/**
	 * A printed name of a long text property
	 */
	private String propertyName;
	
	/**
	 * The property ID of the considered property.
	 * This ID must be defined in regards to
	 * the {@link IPropertySource}
	 */
	private Object propertyID;
	
	/**
	 * The model element this object refers to
	 */
	private Object modelElement;
	
	/**
	 * The {@link IPropertySource} used for getting or
	 * setting a values
	 */
	private IPropertySource propertySource = null;

	/**
	 * Constructor for this class
	 * 
	 * @param propertyID the ID of the long text property
	 * @param propertyName a printed name for the long text property
	 * @param modelElement the model element this object refers to.
	 * @param propertySource a {@link IPropertySource} object (must refers to previous parameter)
	 */
	public AbstractLongTextPropertyAccess(Object propertyID, String propertyName, Object modelElement, IPropertySource propertySource)
	{
		this.modelElement = modelElement;
		this.propertyName = propertyName;
		this.propertyID = propertyID;
		this.propertySource = propertySource;
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess#getLongText()
	 */
	public String getLongText() {
		return (String) getPropertySource().getPropertyValue(getLongTextPropertyID());
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess#refersToSameValue(ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess)
	 */
	public boolean refersToSameValue(ILongTextPropertyAccess other) {
		if(other instanceof AbstractLongTextPropertyAccess)
		{
			AbstractLongTextPropertyAccess otherAccess = ((AbstractLongTextPropertyAccess)other);
			return this.modelElement.equals(otherAccess.modelElement)	&& this.getLongTextPropertyID().equals(otherAccess.getLongTextPropertyID());
		}
		else return false;
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess#setLongtext(java.lang.String)
	 */
	public void setLongtext(String newLongText) {
		getPropertySource().setPropertyValue(getLongTextPropertyID(), newLongText);
	}

	/**
	 * @return the {@link IPropertySource} of this accessor
	 */
	public IPropertySource getPropertySource()
	{
		return propertySource;
	}
	
	public Object getLongTextPropertyID()
	{
		return propertyID;
	}

	/**
	 * @return the modelelement
	 */
	protected Object getModel() {
		return modelElement;
	}
	
	/** 
	 * This method is used by the {@link LongTextPropertyCellEditor} for the label
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getLongText().equalsIgnoreCase("") ? "" : "...";
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess#getArtifactName()
	 */
	public String getArtifactName() {
		if (modelElement instanceof VisualizableModelElement) {
			VisualizableModelElement visualizableModelElement = (VisualizableModelElement) modelElement;
			return visualizableModelElement.getName();
		}
		return "no name";
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess#getPropertyName()
	 */
	public String getPropertyName() {
		return propertyName;
	}
	
	
}
