package ch.unizh.ifi.adora.properties.longtextproperty;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.ui.texteditor.AbstractDocumentProvider;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess;

/**
 * This class provides an {@link IDocument} from an {@link AbstractLongTextEditorInput}. 
 * This object has to responsibilities:
 * a) create a document with the value of a long text property from a model element
 * b) on save event, store this new value in the model element (via a property source, in order to
 *    have undo/redo capabilities through the command stack
 *      
 * @author		        Cedric Jeanneret
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history				15.04.2008 CJ First Version (Fix Bug 855 and refactoring of long text property editors)
 * @version             $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */

public class LongTextPropertyDocumentProvider extends AbstractDocumentProvider {
	
	/**
	 * Instantiates a {@link LongTextPropertyDocumentProvider}.
	 */
	public LongTextPropertyDocumentProvider(){
	}
	
	/**
	 * Creates a new document containing the long text property value stored in the model element.
	 */
	protected IDocument createDocument(Object element) throws CoreException {
		if(element instanceof ILongTextEditorInput)
		{
			ILongTextPropertyAccess propertyAccess = (ILongTextPropertyAccess) ((AbstractLongTextEditorInput)element).getPropertyAccess();
			return new Document(propertyAccess.getLongText());
		}
		throw new CoreException(new Status(IStatus.ERROR,AdoraPlugin.PLUGIN_ID,IStatus.OK,"Cannot create long text property editor: input is not a long property access input.",null));
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.texteditor.AbstractDocumentProvider#getOperationRunner(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IAnnotationModel createAnnotationModel(Object element)
			throws CoreException {
		return new AnnotationModel();
	}

	/**
	 * On save, simply set the long text property to its new value
	 */
	protected void doSaveDocument(IProgressMonitor monitor, Object element,
		IDocument document, boolean overwrite) throws CoreException {
		if(element instanceof ILongTextEditorInput)
		{
			ILongTextPropertyAccess propertyAccess = (ILongTextPropertyAccess) ((AbstractLongTextEditorInput)element).getPropertyAccess();
			propertyAccess.setLongtext(document.get());
		}
		else throw new CoreException(new Status(IStatus.ERROR,AdoraPlugin.PLUGIN_ID,IStatus.OK,"Cannot create long text property editor: input is not a long property access input.",null));
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.texteditor.AbstractDocumentProvider#getOperationRunner(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IRunnableContext getOperationRunner(IProgressMonitor monitor) {
		return null;
	}

	/**
	 * Document is editable.
	 * @return <li>false</li>;
	 */
	public boolean isReadOnly(Object element) {
		return false;
	}

	/**
	 * Document is editable.
	 * @return <li>true</li>;
	 */
	public boolean isModifiable(Object element) {
		return true;
	}
}