package ch.unizh.ifi.adora.properties.specification.scanner;

import java.util.HashMap;

import org.eclipse.jface.text.Assert;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.ITokenScanner;

import ch.unizh.ifi.adora.model.parser.AdoraParserTokenManager;
import ch.unizh.ifi.adora.model.parser.SimpleCharStream;
import ch.unizh.ifi.adora.model.parser.Token;

public class JavaCCSpecificationScanner implements ICharacterScanner, ITokenScanner {
    public final static int STATE_READ_NEXT = 0;
    
    public final static int STATE_CONSUMED_SPECIAL_TOKEN = 1;
    
    public final static int STATE_CONSUME_NORMAL_TOKEN = 2;
    
    private int tokenstate = STATE_READ_NEXT;
    
    private AdoraParserTokenManager lexer;
    
    private LexerInputStream is;
    
    private SimpleCharStream stream;
 
    private Token currToken;
    
    private HashMap<Integer, IToken> dataMappings;
 
    
    /** The list of rules of this scanner */
//    protected IRule[] fRules;
    /** The token to be returned by default if no rule fires */
    protected IToken fDefaultReturnToken;
    /** The document to be scanned */
    protected IDocument fDocument;
    /** The cached legal line delimiters of the document */
    protected char[][] fDelimiters;
    /** The offset of the last read token */
    protected int fTokenOffset;
 
    /** Internal setting for the un-initialized column cache. */
    protected static final int UNDEFINED= -1;

    public JavaCCSpecificationScanner() {
        is = new LexerInputStream();
        stream = new SimpleCharStream(is);
        lexer = new AdoraParserTokenManager(stream);
        dataMappings = new HashMap<Integer, IToken>();
    }
    

    /**
     * Configures the scanner with the given sequence of rules.
     *
     * @param rules the sequence of rules controlling this scanner
     */
    /*public void setRules(IRule[] rules) {
        if (rules != null) {
            fRules= new IRule[rules.length];
            System.arraycopy(rules, 0, fRules, 0, rules.length);
        } else
            fRules= null;
    }*/

    /**
     * Configures the scanner's default return token. This is the token
     * which is returned when non of the rules fired and EOF has not been
     * reached.
     *
     * @param defaultReturnToken the default return token
     * @since 2.0
     */
    public void setDefaultReturnToken(IToken defaultReturnToken) {
        Assert.isNotNull(defaultReturnToken.getData());
        fDefaultReturnToken= defaultReturnToken;
    }

    /*
     * @see ITokenScanner#setRange(IDocument, int, int)
     */
    public void setRange(IDocument document, int offset, int length) {

        fDocument= document;
        is = new LexerInputStream(fDocument);

        
        
        is.setFOffset(offset);
        is.setFColumn(UNDEFINED);
        is.setFRangeEnd(Math.min(fDocument.getLength(), offset + length));

        String[] delimiters= fDocument.getLegalLineDelimiters();
        fDelimiters= new char[delimiters.length][];
        for (int i= 0; i < delimiters.length; i++)
            fDelimiters[i]= delimiters[i].toCharArray();

        if (fDefaultReturnToken == null)
            fDefaultReturnToken= new SpecificationToken(null);
        
        stream.ReInit(is);
        lexer.ReInit(stream);
        tokenstate = STATE_READ_NEXT;
    }

    /*
     * @see ITokenScanner#getTokenOffset()
     */
    public int getTokenOffset() {
        return fTokenOffset;
    }

    /*
     * @see ITokenScanner#getTokenLength()
     */
    public int getTokenLength() {
        if (is.getFOffset() < is.getFRangeEnd())
            return is.getFOffset() - getTokenOffset();
        return is.getFRangeEnd() - getTokenOffset();
    }

    public void setTokenMappingTupel(int tokenType, IToken t) {
        dataMappings.put(tokenType, t);
    }

    public void setTokenMappingTupel(int startTokenType, int endTokenType, IToken t) {
        for (int index = startTokenType; index <= endTokenType; index++) {
            dataMappings.put(index, t);
        }
        
    }
    
    public IToken getMappedToken(int tokenType) {
        IToken t = dataMappings.get(tokenType);
        if (t == null) {
            t = fDefaultReturnToken; 
        } 
        return t;
    }
  
    protected IToken getSpecificationToken(Token t) {
        return getMappedToken(t.kind);
    }
    
    
    /*
     * @see ICharacterScanner#getColumn()
     */
    public int getColumn() {
        if (is.getFColumn() == UNDEFINED) {
            try {
                int line= fDocument.getLineOfOffset(is.getFOffset());
                int start= fDocument.getLineOffset(line);

                is.setFColumn(is.getFColumn() - start);

            } catch (BadLocationException ex) {
            }
        }
        return is.getFColumn();
    }

    /*
     * @see ICharacterScanner#getLegalLineDelimiters()
     */
    public char[][] getLegalLineDelimiters() {
        return fDelimiters;
    }

    /*
     * @see ITokenScanner#nextToken()
     */
    public IToken nextToken() {

        if (is.isEof())
            return SpecificationToken.EOF;

        fTokenOffset= is.getFOffset();
        is.setFColumn(UNDEFINED);

        Token t = getNextToken();
        IToken tokenType = getMappedToken(t.kind);
        return tokenType;
        //
        //    
        //return fDefaultReturnToken;
    }

    
 
  
    
    protected Token getNextToken() {
        Token temp = null; 
        
        if (tokenstate == STATE_READ_NEXT) {
            if (currToken == null) {
                currToken = lexer.getNextToken();
            } else {
                if (currToken.next != null) currToken = currToken.next;
                else currToken = currToken.next = lexer.getNextToken();
            }
            if (currToken.specialToken != null) {
                temp = currToken.specialToken;
                tokenstate = STATE_CONSUMED_SPECIAL_TOKEN;
            } else {
                temp = currToken;
                tokenstate = STATE_READ_NEXT;
            }
        } else {
            if (tokenstate == STATE_CONSUMED_SPECIAL_TOKEN) {
        
                temp = currToken;
                tokenstate = STATE_READ_NEXT;
            }
        }
        return temp;
      }
    
    /*
     * @see ICharacterScanner#read()
     */
    public int read() {
        
        int ret = UNDEFINED;
        try {
            ret = is.read();
        } catch (Exception ex) {
            
        }
        return ret;
        
    }

    /*
     * @see ICharacterScanner#unread()
     */
    public void unread() {
            is.unread();
    }
    
    
}
