package ch.unizh.ifi.adora.representation.abstractions.algorithm;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.collections.iterators.IteratorChain;
import org.eclipse.gef.commands.Command;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Interrelationship;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * Algorithm to calculate the abstract connection if a node is zoomed.
 * Furthermore it generates {@link ch.unizh.ifi.adora.model.representation.Interrelationship}s 
 * if needed.
 *      
 * @author              Tobias Reinhard, 
 * 						Ivo Vigan
 * 
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 *                      
 * @history             2006-04-19 TR First Version <br>
 * 						2006-07-28 IV Interrelations refactored <br>

 * @version             $Date: 2010-02-11 20:38:24 +0100 (Thu, 11 Feb 2010) $, $Author: vigan $, $Revision: 5115 $
 */
public class ConnectionAbstractionAlgorithm {

	/**
	 * Stores the default instance of this abstraction algorithm.
	 */
	private static ConnectionAbstractionAlgorithm INSTANCE = new ConnectionAbstractionAlgorithm();

	/**
	 * Stores the (@link {@link Node}, {@link JoinRelation}) tuple which is used
	 * to generate the {@link Decision}numbers for the abstract {@link JoinRelation}s.
	 */
	private List<Object[]> decisionList;
	/**
	 * Realizes the singleton pattern.
	 *
	 * @return singleton instance.
	 */
	public static ConnectionAbstractionAlgorithm getInstance() {
		return INSTANCE;
	}

	/**
	 * Private Constructor, used for singleton pattern.
	 */
	private ConnectionAbstractionAlgorithm() {
	}

	/**
	 * The node <i>zoomedInNode</i> has changed its zoom state 
	 * (i.e. the children are now visible) 
	 * and the original connections and problably some intermediate
	 * abstract connections have to be made visible.
	 * 
	 * @param zoomedNode the node that has been zoomed in
	 */
	public void run(ZoomableNodeContainer zoomedNode) {
		
		calculateAssociations(zoomedNode);
		
		calculateJoinRelations(zoomedNode);
			
		createInterRelations(zoomedNode);
	}

	/**
	 * Main method for instantiating the {@link Interrelationship}s.
	 * 
	 * Functionality (pseudocode) :
	 * 
	 * for all abstract associations of zoomedOutNode
	 *     currentAssociation.getOtherEndsNode() 
	 * 	   if(child of otherEndsNode || otherEndsNode has connection to zoomedOutNode.child) 
	 *         create Interrelationship
	 * 
	 * @param zoomedOutNode
	 *            {@link ZoomableNodeContainer on which gets zoomed out.
	 */
	private void calculateInterrelationsShips(ZoomableNodeContainer zoomedOutNode) {

		/*
		 *Current Association which is abstractA.
		 */
		Association abstractAssociation;

		// Get all Associations of the node, which is zoomed out
		Iterator<Association> assoIter = zoomedOutNode
				.getAssociationsIterator();
		while (assoIter.hasNext()) {

			abstractAssociation = assoIter.next();

			//Only abstracted associations of the zoomedOutNode should be considered;
			if (!abstractAssociation.isAbstracted()) {
				continue;
			}

			/* If othernode isn't a ZoomableNodeContainer, no
			 interrelationships have to be built. */
			if (!(abstractAssociation.getSource() instanceof ZoomableNodeContainer &&
					abstractAssociation.getTarget() instanceof ZoomableNodeContainer )) {
				continue;
			}
			
			//Recursive abstractAssociations musten't be considered
			if (abstractAssociation.getSource() == abstractAssociation.getTarget()){
				continue;
			}

			instantiateInterrelationShips(zoomedOutNode, 
					getRecursiveAssociationIterator(zoomedOutNode),
					abstractAssociation);
			
			assureHierarchy(abstractAssociation);
		}
	}
	
	/**
	 * Checks the if constraint violations concerning the {@link Interrelationship} hierchy exist.
	 * If so, these {@link Interrelationship}s get deleted. 
	 * 
	 * @param abstractAssociation Abstract Association which gets checked.
	 */
	private void assureHierarchy(Association abstractAssociation) {
		for (Connection con : abstractAssociation.getConnections()) {
			if (con instanceof Interrelationship) {
				Association concreteAsso = ((Interrelationship) con)
						.getTarget();
				for (Connection ir : concreteAsso.getConnections()) {
					// only IR where the abstractAssociation is source
					if (ir instanceof Interrelationship
							&& concreteAsso != abstractAssociation) {
						Association currentAbstractAsso = ((Interrelationship) ir)
								.getSource();

						ZoomableNodeContainer currentSource = ((ZoomableNodeContainer) currentAbstractAsso
								.getSource());
						ZoomableNodeContainer currentTarget = ((ZoomableNodeContainer) currentAbstractAsso
								.getTarget());
						
						// checks if abstract association on which this ir is attached 
						// is associated with at least one node which is an anchestor of one
						// node of the abstractAssociation. 
						if (currentSource.hasChildRecursively(abstractAssociation.getSource()) ||
							currentSource.hasChildRecursively(abstractAssociation.getTarget()) ||										
							currentTarget.hasChildRecursively(abstractAssociation.getSource()) ||										
							currentTarget.hasChildRecursively(abstractAssociation.getTarget())) {

							ConnectionDelete cd = new ConnectionDelete(ir);
							executeCommandOnStack(cd);
						}
					}
				}
			}
		}
	}

	/**
	 * Returns an Iterator, containing associations of the <li>zoomableNode</li> 
	 * 	and of all its children.
	 * 
	 * @param zoomableNode ZoomableNodeContainer, 
	 * for which all associations of it and of all its children are been returned. 
	 * 		
	 * @return Iterator, containing associations of the <li>zoomableNode</li> 
	 * 	and of all its children.
	 */
	@SuppressWarnings("unchecked")
	private Iterator<Association> getRecursiveAssociationIterator(ZoomableNodeContainer zoomableNode){
		
		/*
		 * Contains the node and its children at the other end of the 
		 * current abstractAssociation. 
		 */
		Iterator<ModelElement> childIter = zoomableNode.getRecursiveChildrenIterator();
		
		/*
		 * Contains all associations of the node and its children at the other end of the 
		 * current abstractAssociation.
		 */
		IteratorChain assoIter = new IteratorChain(zoomableNode.getAssociationsIterator());

		while (childIter.hasNext()) {
			assoIter.addIterator(((Node) childIter.next())
					.getAssociationsIterator());
		}
		return assoIter;
	}

	/**
	 * Instantiates {@link Interrelationship}s for {@link Association}s stored in <i>associationIter</i>., 
	 * it meets the {@link Interrelationship} criteria.  
	 * 
	 * @param zoomedOutNode {@link ZoomableNodeContainer}, from which at least one direct child
	 * 						has been hidden.
	 * @param associationIter {@link Iterator} stores all {@link Association}s of the <li>zoomedOutNode</li> 
	 * 						and its children at the other end of the <i>abstractAssociation</i>.
	 * @param abstractAssociation Abstract {@link Association} for which the {@link Interrelationship}s 
	 * 						are instatiated.
	 */
	private void instantiateInterrelationShips(ZoomableNodeContainer zoomedOutNode,
			Iterator associationIter, Association abstractAssociation) {

		/*
		 *Current association in the loop.. 
		 */
		Association currentAssociation;

		/*
		 * Stores the node at that end of the currentAssociation,
		 * which is situated in the zoomedOutNode.
		 * 
		 */
		Node currentNode;

		/*
		 * Stores the node at that end of the currentAssociation,
		 * which isn't situated in the zoomedOutNode.
		 * 
		 * So the oppositeNode is never situated in the zoomedOutNode.
		 */
		Node currentOppositeNode;
		
		/*
		 *Counterpart of the zoomed out node. 
		 */
		ZoomableNodeContainer oppositeZoomableNode = abstractAssociation.getSource() == zoomedOutNode ?
												(ZoomableNodeContainer) abstractAssociation.getTarget() : 
												(ZoomableNodeContainer) abstractAssociation.getSource();  
		
		while (associationIter.hasNext()) {

			currentAssociation = (Association) associationIter.next();
			
			//Model element also exists of hidden associations.
			if (!currentAssociation.isVisible()) {
				continue;
			}	

			if (zoomedOutNode.hasChildRecursively(
					currentAssociation.getSource())
					|| zoomedOutNode == currentAssociation.getSource()) {
				
				currentNode = currentAssociation.getSource();
				currentOppositeNode = currentAssociation.getTarget();
				
			} else{
				currentNode= currentAssociation.getTarget();
				currentOppositeNode = currentAssociation.getSource();
			}
											
			//Criteria for instantiating an InterRelationship.
			if (((zoomedOutNode.hasChildRecursively(currentNode) && 
					(oppositeZoomableNode.hasChildRecursively(currentOppositeNode) ||
							oppositeZoomableNode == currentOppositeNode)) 
							
				||
				
				(oppositeZoomableNode.hasChildRecursively(currentOppositeNode) && 
						(zoomedOutNode.hasChildRecursively(currentNode) ||
								zoomedOutNode == currentNode)))) {
					
				Interrelationship ir = new Interrelationship(
						abstractAssociation, currentAssociation);
				
				ir.setVisible(true);
			}
		}
	}

	/**
	 * Overloaded for better encapsulation.
	 * 
	 * See {@link ConnectionAbstractionAlgorithm#createInterRelations(Node)}
	 *
	 * @param source node for which the {@link Interrelationship}s of all associations of the 
	 * 					top-zoomable-Parent and it's children are been refresed.
	 * @param target node for which the {@link Interrelationship}s of all associations of the 
	 *				 top-zoomable-Parent and it's children are been refresed.
	 */
	public void recalculateInterRelations(Node source, Node target) {
		createInterRelations(source);
		createInterRelations(target);
	}

	/**
	 * Overloaded for better encapsulation.
	 * 
	 * See {@link ConnectionAbstractionAlgorithm#createInterRelations(Node)}
	 *
	 * @param association The top-zoomable-Parent of the source- and target node will be refreshed.
	 */
	public void recalculateInterRelations(Association association) {
		createInterRelations(association.getSource());
		createInterRelations(association.getTarget());
	}

	/**
	 * Gets the top-zoomable-Parent of the passed {@link Node}, 
	 * and refreshes the {@link Interrelationship}s of the {@link Association}s
	 * of all children of this {@link Node}.
	 * Refresh means to delete and recalculate the {@link Interrelationship}s.
	 *
	 * @param node for which the {@link Interrelationship}s of all associations of the 
	 * top-zoomable-Parent and it's children are been refresed.
	 */
	public void createInterRelations(Node node) {
	Iterator<ModelElement> childIter;
		ZoomableNodeContainer topParent;
		Node n;
		IteratorChain assoIter = new IteratorChain();

		//Only Associations which connect ZoomableNodeContainer can have
		//InterRelationships
		if (!(node instanceof ZoomableNodeContainer)) {
			return;
		}

		//gets the top Parent, which is a direct child of AdoraModel
		topParent = getTopZoomableParent((ZoomableNodeContainer)node);

		//assoIter contains all associations of all children of the topParent.
		assoIter.addIterator(topParent.getAssociationsIterator());
		childIter = topParent.getRecursiveChildrenIterator();
		while (childIter.hasNext()) {
			n = (Node) childIter.next();
			if (n instanceof ZoomableNodeContainer)
				assoIter.addIterator(((ZoomableNodeContainer) n)
						.getAssociationsIterator());
		}

		//Deletes the InterRelationships of all associations
		//contained in assoIter.
		while (assoIter.hasNext()) {
			for (Connection c : ((Association) assoIter.next())
					.getConnections()) {
				if (c instanceof Interrelationship) {
					ConnectionDelete del = new ConnectionDelete(c);
					executeCommandOnStack(del);
				}
			}
		}
		//Creates the InterRelationships of all associations
		//contained in assoIter.
		calculateInterrelationsShips(topParent);
		childIter = topParent.getRecursiveChildrenIterator();
		while (childIter.hasNext()) {

			n = (Node) childIter.next();
			if (n instanceof ZoomableNodeContainer) {

				calculateInterrelationsShips((ZoomableNodeContainer) n);
			}
		}
	}
	
	/**
	 * @param zoomableNode Node, for which the top zoomable parent is returned.
	 * 
	 * @return @return The top zoomable parent for the <li>node</li>.
	 */
	private ZoomableNodeContainer getTopZoomableParent(ZoomableNodeContainer zoomableNode){
		ZoomableNodeContainer topParent;
		//gets the top Parent, which is a direct child of AdoraModel
		topParent = zoomableNode;
		while (!(topParent.getParent() instanceof AdoraModel)
				&& topParent.getParent() != null) {
			topParent = (ZoomableNodeContainer)topParent.getParent();
		}
		return topParent;
	}
	
	
	/**
	 * Calcultes the abstract associations that are necessary after the node container
	 * given as argument has been zoomed out or in. The associations are calcualted from
	 * scratch (i.e. existing abstract associations are first removed and if necessary
	 * added again).
	 * 
	 * @param zoomed
	 */
	private void calculateAssociations(ZoomableNodeContainer zoomed) {
		
		// Contains all the newly constructed abstract associations (to check whether the
		// abstract association already exists)
		HashMap<Node, Association> assos = new HashMap<Node, Association>();
		constructAbstractAssociations(assos, zoomed);	
		calculateRecursiveAbstractAssociations(zoomed, assos);
		
	}
	
	
	private void calculateJoinRelations(ZoomableNodeContainer zoomed) {
		
		HashMap<ConnectableElement, JoinRelation> aspCons = new HashMap<ConnectableElement, JoinRelation>();
		decisionList = new ArrayList<Object[]>();
		
		constructAbstractJoinRelations(aspCons, zoomed);
		calculateRecursiveAbstractJoinRelations(zoomed, aspCons);
		
		calculateDecision(aspCons, zoomed);
		//calculateDecision(zoomed);
		
		
	}
	
	
	private void calculateRecursiveAbstractJoinRelations(
			ZoomableNodeContainer nc, HashMap<ConnectableElement, JoinRelation> aspCons) {

		Iterator children = nc.getChildren().iterator();
		while (children.hasNext()) {
			ModelElement me = (ModelElement) children.next();
			if (me instanceof Node) {
				Node node = (Node) me;
				constructAbstractJoinRelations(aspCons, node);

				// Go to the next (deeper) hierarcy level
				if (node instanceof ZoomableNodeContainer) {
					calculateRecursiveAbstractJoinRelations(
							(ZoomableNodeContainer) node, aspCons);
				}
			}
		}

	}
	
	
	/**
	 * Calculates recursively descending the hierarcy all the abstract asscociations
	 * coming from or leading to the node <i>nc</i> and stores the calculated
	 * abstract associations in the hash map <i>assos</i>.
	 * 
	 * @param nc
	 * @param assos
	 */
	private void calculateRecursiveAbstractAssociations(
			ZoomableNodeContainer nc, HashMap<Node, Association> assos) {

		Iterator children = nc.getChildren().iterator();
		while (children.hasNext()) {
			ModelElement me = (ModelElement) children.next();
			if (me instanceof Node) {
				Node node = (Node) me;
				constructAbstractAssociations(assos, node);

				// Go to the next (deeper) hierarcy level
				if (node instanceof ZoomableNodeContainer) {
					calculateRecursiveAbstractAssociations(
							(ZoomableNodeContainer) node, assos);
				}
			}
		}
	}

	
	/**
	 * Checks the source and target connections of the node <i>node</i> and adds 
	 * abstract associations if necessary.
	 * 
	 * @param assos
	 * @param node
	 */
	private void constructAbstractAssociations(HashMap<Node, Association> assos, Node node) {
		removeAllAbstractAssociations(node);	
		
		AdoraModel root = (AdoraModel)(node.getParent()).getRootContainer();
		
		// Check the source connections
		Iterator iter = node.getSourceConnections().iterator();
		while (iter.hasNext()) {
			Connection con = (Connection) iter.next();
			if (con instanceof Association && !(con.getSource().isVisible() && con.getTarget().isVisible())) {
				Node target = (Node) con.getTarget()
						.getFirstVisibleUpTheHierarchy();
				if ((!assos.containsKey(target) || (assos.get(target).getSource() != node.getFirstVisibleUpTheHierarchy() 
						&& assos.get(target).getTarget() != node.getFirstVisibleUpTheHierarchy())) && abstractConnectionAllowed(node.getFirstVisibleUpTheHierarchy(), target)) {
					Node source = node.getFirstVisibleUpTheHierarchy();
					// Don't add reflexive associations
					if(source != target && source != null && target != null) {
						Association asso = new Association(source, target);
						asso.setCalculatedAbstracted(true);
						asso.setSourceClickPoint(((Association)con).getSourceClickPoint(source));
						asso.setTargetClickPoint(((Association)con).getTargetClickPoint(target));
						assos.put(target, asso);
					
						// Set the visibility according to the filters of the view management
						asso.setVisible(!root.getViewFilterState(Association.class));
						asso.updateVisibility();
					}
					
				}
			}
		}

		// Check the target connections
		iter = node.getTargetConnections().iterator();
		while (iter.hasNext()) {
			Connection con = (Connection) iter.next();
			if (con instanceof Association && !(con.getSource().isVisible() && con.getTarget().isVisible())) {
				Node source = (Node) con.getSource()
						.getFirstVisibleUpTheHierarchy();
				if ((!assos.containsKey(source) || (assos.get(source).getSource() != node.getFirstVisibleUpTheHierarchy() 
						&& assos.get(source).getTarget() != node.getFirstVisibleUpTheHierarchy())) && abstractConnectionAllowed(source, node.getFirstVisibleUpTheHierarchy())) {
					Node target = node.getFirstVisibleUpTheHierarchy();
					// Don't add reflexive associations
					if(source != target && source != null && target != null) {
						Association asso = new Association(source, target);
						asso.setCalculatedAbstracted(true);
						asso.setSourceClickPoint(((Association)con).getSourceClickPoint(source));
						asso.setTargetClickPoint(((Association)con).getTargetClickPoint(target));
						assos.put(source, asso);
					
						// Set the visibility according to the filters of the view management
						asso.setVisible(!root.getViewFilterState(Association.class));
						asso.updateVisibility();
					}
				}
			}
		}
	}
	
	
	/**
	 * Checks the source and target connections of the node given as argument and adds 
	 * abstract Join Relationship if necessary. The method additonally checks the
	 * Join Relationshipfor the transitions that lead to or come from the node and 
	 * adds abstract Join Relationship if necessary. The newly created abstract associations
	 * are stored in the given as argument.
	 * 
	 * TODO TR: Merge this method with constructAbstractAssociations(...)
	 * 
	 * @param joinRelations hash map containging the already constructed abstract Join Relationship
	 * @param node the node that has been zoomed in or out
	 */
	@SuppressWarnings("unchecked")
	private void constructAbstractJoinRelations(HashMap<ConnectableElement, JoinRelation> joinRelations, Node node) {
		
		removeAllAbstractJoinRelations(node);
		
		AdoraModel root = (AdoraModel)(node.getParent()).getRootContainer();
		
		// Check the source connections
		Iterator<Connection> iter = node.getSourceConnections().iterator();
		while (iter.hasNext()) {
			Connection con = (Connection) iter.next();
			if (con instanceof JoinRelation && !(con.getSource().isVisible() && con.getTarget().isVisible())) {					
				ConnectableElement target = (ConnectableElement) con.getTarget()
						.getFirstVisibleUpTheHierarchy();
				if ((!joinRelations.containsKey(target) || (joinRelations.get(target).getSource() != node.getFirstVisibleUpTheHierarchy() 
						&& joinRelations.get(target).getTarget() != node.getFirstVisibleUpTheHierarchy())) && abstractConnectionAllowed(node.getFirstVisibleUpTheHierarchy(), target)) {
					Node source = node.getFirstVisibleUpTheHierarchy();
					// Don't add reflexive Join Relationships
					if(source != target && source != null && target != null) {
						JoinRelation aspCon = new JoinRelation(source, target);
						for(Decision dec:AdoraPlugin.getActiveModel().getDecisions()){
							if(dec!=null && ((JoinRelation)con).getDecision()!=null){
							if(dec.getNr()==((JoinRelation)con).getDecision().getNr()){
							aspCon.setDecision(dec);
							}
							}
						}
						aspCon.setCalculatedAbstracted(true);
						joinRelations.put(target, aspCon);
						//---decision part
						boolean found = false;
						for (int i = 0; i < decisionList.size(); i++) {
							if (decisionList.get(i)[0] == target) {
								
								((List<JoinRelation>)decisionList.get(i)[1]).add(aspCon);
								found = true;
								break;
							} 
						}
						if (!found) { 
							List<JoinRelation> tmpL = new ArrayList<JoinRelation>();
							tmpL.add(aspCon);
							decisionList.add(new Object[]{target, tmpL});
						}
						//----
					
						// Set the visibility according to the filters of the view management
						aspCon.setVisible(!root.getViewFilterState(JoinRelation.class));
						aspCon.updateVisibility();
					}
				}
			}
			else if(con instanceof Transition) {
				constructAbstractJoinRelations(joinRelations, (Transition)con);
			}
		}

		// Check the target connections
		iter = node.getTargetConnections().iterator();
		while (iter.hasNext()) {
			Connection con = (Connection) iter.next();
			if (con instanceof JoinRelation && !(con.getSource().isVisible() && con.getTarget().isVisible())) {
				Node source = (Node) con.getSource()
						.getFirstVisibleUpTheHierarchy();
				if ((!joinRelations.containsKey(source) || (joinRelations.get(source).getSource() != node.getFirstVisibleUpTheHierarchy() 
						&& joinRelations.get(source).getTarget() != node.getFirstVisibleUpTheHierarchy())) && abstractConnectionAllowed(source, node.getFirstVisibleUpTheHierarchy())) {
					Node target = node.getFirstVisibleUpTheHierarchy();
					// Don't add reflexive Join Relationship
					if(source != target && source != null && target != null) {
						JoinRelation aspCon = new JoinRelation(source, target);
						for(Decision dec:AdoraPlugin.getActiveModel().getDecisions()){
							if(dec!=null && ((JoinRelation)con).getDecision()!=null){
							if(dec.getNr()==((JoinRelation)con).getDecision().getNr()){
							aspCon.setDecision(dec);
							}}
						}
						aspCon.setCalculatedAbstracted(true);
						joinRelations.put(source, aspCon);
						
						//---decision part
						boolean found = false;
						for (int i = 0; i < decisionList.size(); i++) {
							if (decisionList.get(i)[0] == source) {
								((List<JoinRelation>)decisionList.get(i)[1]).add(aspCon);
								found = true;
								break;
							} 
						}
						if (!found) { 
							List<JoinRelation> tmpL = new ArrayList<JoinRelation>();
							tmpL.add(aspCon);
							decisionList.add(new Object[]{source, tmpL});
						}
						//----
						
						// Set the visibility according to the filters of the view management
						aspCon.setVisible(!root.getViewFilterState(JoinRelation.class));
						aspCon.updateVisibility();
					}
				}
			}
			else if(con instanceof Transition) {
				constructAbstractJoinRelations(joinRelations, (Transition)con);
			}
		}		
	}
	
	/**{
	 * 
	 * Checks the target Join Relationships of the transition given as
	 * argument and adds abstract Join Relationships if necessary. The newly
	 * created abstract Join Relationships are stored in the hash map given as
	 * argument.
	 * 
	 * @param aspCons hash map containing the already existing abstract Join Relationships
	 * @param trans the transition to check
	 */
	private void constructAbstractJoinRelations(HashMap<ConnectableElement, JoinRelation> aspCons, Transition trans) {
		
		//removeAllAbstractJoinRelations(trans);
		
		AdoraModel root = (AdoraModel)(trans.getSource().getParent()).getRootContainer();
		
		// Check the target connections
		Iterator iter = trans.getTargetConnections().iterator();
		while (iter.hasNext()) {
			Connection con = (Connection) iter.next();
			if (con instanceof JoinRelation && !(con.getSource().isVisible() && con.getTarget().isVisible())) {
				Node source = (Node) con.getSource()
						.getFirstVisibleUpTheHierarchy();
				if ((!aspCons.containsKey(source) || (aspCons.get(source).getSource() != trans.getFirstVisibleUpTheHierarchy() 
						&& aspCons.get(source).getTarget() != trans.getFirstVisibleUpTheHierarchy())) && abstractConnectionAllowed(source, trans.getFirstVisibleUpTheHierarchy())) {
					ConnectableElement target = trans.getFirstVisibleUpTheHierarchy();
					// Don't add reflexive Join Relationships
					if(source != target && source != null && target != null) {
						JoinRelation aspCon = new JoinRelation(source, target);
						aspCon.setCalculatedAbstracted(true);
						aspCons.put(source, aspCon);
					
						// Set the visibility according to the filters of the view management
						aspCon.setVisible(!root.getViewFilterState(JoinRelation.class));
						aspCon.updateVisibility();
					}
				}
			}
		}
	}
	
	/**
	 * Removes (i.e. disconnects) all the abstract associations that are coming from 
	 * or leading to the node given as argument.
	 * 
	 * @param node
	 */
	private void removeAllAbstractAssociations(Node node) {
		
		Iterator iter = node.getConnections().iterator();
		while (iter.hasNext()) {
			Connection con = (Connection)iter.next();
			if (con instanceof Association && ((Association)con).isCalculatedAbstracted()) {
				if (((Association)con).isManuallyAbstracted()){
					((Association)con).setCalculatedAbstracted(false);
					continue;
				}
				con.setVisible(false);
				con.disconnect();
			}
		}	
	}
	
	
	/**
	 * Removes (i.e. disconnects) all the abstract Join Relationships that are coming
	 * from or leading to the node given as argument.
	 * 
	 * @param node
	 */
	private void removeAllAbstractJoinRelations(Node node) {
		
		Iterator iter = node.getConnections().iterator();
		while(iter.hasNext()) {
			Connection con = (Connection)iter.next();
			if(con instanceof JoinRelation && ((JoinRelation)con).isCalculatedAbstracted()) {
				con.setVisible(false);
				con.disconnect();
			}
		}
	}
	
	/**
	 * Removes (i.e. disconnects) all the abstract Join Relationships that are coming
	 * from or leading to the transition given as argument.
	 * 
	 * @param transition
	 */
	private void removeAllAbstractJoinRelations(Transition transition) {
		
		Iterator iter = transition.getConnections().iterator();
		while(iter.hasNext()) {
			Connection con = (Connection)iter.next();
			if(con instanceof JoinRelation && ((JoinRelation)con).isCalculatedAbstracted()) {
				con.setVisible(false);
				con.disconnect();
			}
		}
	}
	
	/**
	 * Checks if an abstract connection allready exists and if the elements are parent/child.
	 * 
	 * @param source The source Element.
	 * @param target The target Element.
	 * @return <code>true,</code> if an abstract connection allready exists.
	 */
	private boolean abstractConnectionAllowed(ConnectableElement source, ConnectableElement target){
	
		if(source == null || target == null){
			return false;
		}
		for (Connection c : source.getConnections()){
			if (c instanceof Association && 
					c.getTarget() == target && ((Association)c).isCalculatedAbstracted()) {
				return false;
			}
			if (c instanceof Association && c.getTarget() == target &&
					((Association)c).isManuallyAbstracted()) {
				((Association)c).setCalculatedAbstracted(true);
				return false;
			}
			if(c instanceof JoinRelation && 
					((JoinRelation)c).isCalculatedAbstracted() &&
					c.getTarget() == target) {
				return false;
			}
		}
		boolean unrelated = true;
		if (target instanceof ZoomableNodeContainer) {
			unrelated = !((ZoomableNodeContainer)target).hasChildRecursively(source);
		}
		if (source instanceof ZoomableNodeContainer) {
			unrelated &= !((ZoomableNodeContainer)source).hasChildRecursively(target);
		}
		return unrelated;
	}
	
	 /**
     * Executes a given {@link org.eclipse.gef.commands.Command} on the {@link org.eclipse.gef.commands.CommandStack} 
     * of the editor.
     *
     * @param cmd The {@link org.eclipse.gef.commands.Command} which gets executed on the 
     * {@link org.eclipse.gef.commands.CommandStack}.
     */
    protected void executeCommandOnStack(Command cmd){
    	IWorkbenchPage page = PlatformUI.getWorkbench()
		.getActiveWorkbenchWindow().getActivePage();
		
    	try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				((AdoraModelEditor)editor).getCommandStack().execute(cmd);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
    }

    
    /**
     * Add all Decisions of {@link JoinRelation}s of children of "zoomed" which end/start in the ConnectableElement, to the label of the newly created JR. 
     * 
     * @param map HashMap<ConnectableElement, JoinRelation> stores the newly created {@link JoinRelation}s and their {@link ConnectableElement}
     * on the end outside of "zoomed".
     * @param zoomed the {@link ZoomableNodeContainer} which gets zoomed
     */private void calculateDecision( HashMap<ConnectableElement, JoinRelation> map, ZoomableNodeContainer zoomed) {
     	
    	List<Decision> dec;
     	//ele: newJR connects zoomed <-> ele
     	for (int q = 0; q < decisionList.size(); q++) {
     		
     		ConnectableElement ele = (ConnectableElement)decisionList.get(q)[0];
     		
     		if (!map.containsKey(ele))
     			continue;
     		
     		for (JoinRelation jrToAdd : (List<JoinRelation>)decisionList.get(q)[1]) {
         		String s = "";
             	dec = new ArrayList<Decision>();
    
    			//--TARGET-CONNECTIONS----------------------
              	for (Connection c : ele.getTargetConnections()) {
         			if (!(c  instanceof JoinRelation))
         				continue;
         			
         			//here all decision of JRs from children of zoomed to ele are added to the map(ele) connection
     				if (zoomed.hasChildRecursively(c.getSource())) {
     					((JoinRelation)c).setAbstrectedIn(jrToAdd);
     					if (((JoinRelation)c).getDecision() != null) {
     						dec.add(((JoinRelation)c).getDecision());
     						if (!s.contains(((JoinRelation)c).getDecisionNr())) {
     	     					if (s == "")
     	     						s = ((JoinRelation)c).getDecisionNr();
     	     					else
     	     						s += ", "+((JoinRelation)c).getDecisionNr();
     						}
    					}
     				}	
         		}
              	
            	//in the case where ele is zoomed out, we have to add all decisions from JRs of its children (incl. Transitions)
             	//to children of zoomed
    			if (ele instanceof ZoomableNodeContainer && !((ZoomableNodeContainer)ele).isZoomedIn()) {
    				Iterator<ModelElement> itr = ((ZoomableNodeContainer)ele).getRecursiveChildrenIterator();
        			while (itr.hasNext()) {
        				ModelElement me = itr.next();
        				if (me instanceof Node)  {
        					for (Connection con : ((Node)me).getTargetConnections()) {
        						if (con instanceof JoinRelation) {
        							//if (zoomed.hasChildRecursively(con.getSource())) {
        							
        							ModelElement tmpE = null;
        							if (jrToAdd.getTarget() != ele)
        								tmpE = jrToAdd.getTarget();
        							else 
        								tmpE = jrToAdd.getSource();
        							
        							if (tmpE == con.getSource() || tmpE instanceof ZoomableNodeContainer && 
        									((ZoomableNodeContainer)tmpE).hasChildRecursively(con.getSource())) {
        								((JoinRelation)con).setAbstrectedIn(jrToAdd);
        			 					if (((JoinRelation)con).getDecision() != null) {
        			 						dec.add(((JoinRelation)con).getDecision());
        			 						if (!s.contains(((JoinRelation)con).getDecisionNr())) {
        			 	     					if (s == "")
        			 	     						s = ((JoinRelation)con).getDecisionNr();
        			 	     					else
        			 	     						s += ", "+((JoinRelation)con).getDecisionNr();
        			 						}
        			 					}
        							} 
        						} 
        						else if (con instanceof Transition) {
        							for (Connection tmpC : con.getTargetConnections()) {
        								if (tmpC instanceof JoinRelation) {
        									if (zoomed.hasChildRecursively(tmpC.getSource())) {
        										((JoinRelation)tmpC).setAbstrectedIn(jrToAdd);
        					 					if (((JoinRelation)tmpC).getDecision() != null) {
        	    			 						dec.add(((JoinRelation)tmpC).getDecision());
        	    			 						if (!s.contains(((JoinRelation)tmpC).getDecisionNr())) {
        	    			 	     					if (s == "")
        	    			 	     						s = ((JoinRelation)tmpC).getDecisionNr();
        	    			 	     					else
        	    			 	     						s += ", "+((JoinRelation)tmpC).getDecisionNr();
        	    			 						}
        	    			 					}
        	    							}
        								}
        							}
        						}
        					}
        				}
        			}
    			}
    					
    		  	
    			//--SOURCE-CONNECTIONS----------------------
    			for (Connection c : ele.getSourceConnections()) {
         			if (!(c  instanceof JoinRelation))
         				continue;
         			
         			//here all decision of JRs from children of zoomed to ele are added to the map(ele) connection
     				if (zoomed.hasChildRecursively(c.getTarget())) {
     					((JoinRelation)c).setAbstrectedIn(jrToAdd);
     					if (((JoinRelation)c).getDecision() != null) {
     						dec.add(((JoinRelation)c).getDecision());
     						if (!s.contains(((JoinRelation)c).getDecisionNr())) {
         						if (s == "")
         							s = ((JoinRelation)c).getDecisionNr();
         						else
         							s += ", "+((JoinRelation)c).getDecisionNr();
     						}
    					}
     				}	
     				
     				//Transition Case
     				if (c.getTarget() instanceof Transition) {
     					if (zoomed.hasChildRecursively(((Transition)c.getTarget()).getSource())) {
     						((JoinRelation)c).setAbstrectedIn(jrToAdd);
     	 					if (((JoinRelation)c).getDecision() != null) {
    		 						dec.add(((JoinRelation)c).getDecision());
    		 					if (!s.contains(((JoinRelation)c).getDecisionNr())) {
    		 	     				if (s == "")
    		 	     					s = ((JoinRelation)c).getDecisionNr();
    		 	     				else
    		 	     					s += ", "+((JoinRelation)c).getDecisionNr();
    		 					}
    		 				}
     					}
     				}
         		}
              	
            	//in the case where ele is zoomed out, we have to add all decisions from JRs of its children
             	//to children of zoomed
    			if (ele instanceof ZoomableNodeContainer && !((ZoomableNodeContainer)ele).isZoomedIn()) {
    				Iterator<ModelElement> itr = ((ZoomableNodeContainer)ele).getRecursiveChildrenIterator();
        			while (itr.hasNext()) {
        				ModelElement me = itr.next();
        				if (me instanceof Node)  {
        					for (Connection con : ((Node)me).getSourceConnections()) {
        						if (con instanceof JoinRelation) {
        							//if (zoomed.hasChildRecursively(con.getTarget())) {
        							
        							ModelElement tmpE = null;
        							if (jrToAdd.getTarget() != ele)
        								tmpE = jrToAdd.getTarget();
        							else 
        								tmpE = jrToAdd.getSource();
        							
        							if (con.getTarget() == tmpE  || tmpE instanceof ZoomableNodeContainer 
        									&& ((ZoomableNodeContainer)tmpE).hasChildRecursively(con.getTarget())) {
        							
        								((JoinRelation)con).setAbstrectedIn(jrToAdd);
        			 					if (((JoinRelation)con).getDecision() != null) {
        			 						dec.add(((JoinRelation)con).getDecision());
        			 						
        			 						if (!s.contains(((JoinRelation)con).getDecisionNr())) {
        			 							if (s == "")
        			 								s = ((JoinRelation)con).getDecisionNr();
        			 							else
        			 								s += ", "+((JoinRelation)con).getDecisionNr();
        									}
        			 					}
        							} else if (con.getTarget() instanceof Transition &&
        								zoomed.hasChildRecursively(((Transition)con.getTarget()).getTarget())) {
        								
        								((JoinRelation)con).setAbstrectedIn(jrToAdd);
        			 					if (((JoinRelation)con).getDecision() != null) {
        			 						dec.add(((JoinRelation)con).getDecision());
        			 						
        			 						if (!s.contains(((JoinRelation)con).getDecisionNr())) {
        			 							if (s == "")
        			 								s = ((JoinRelation)con).getDecisionNr();
        			 							else
        			 								s += ", "+((JoinRelation)con).getDecisionNr();
        									}
        			 					}
        								
        							}
        						} 
        					}
        				}
        			}
    			}
    			jrToAdd.setAbstractedDecNrs(s);
         	}
     	}
     }
     
     /**
      * Add all Decisions of {@link JoinRelation}s of children of "zoomed" which end/start in the ConnectableElement, to the label of the newly created JR. 
      * 
      * @param map HashMap<ConnectableElement, JoinRelation> stores the newly created {@link JoinRelation}s and their {@link ConnectableElement}
      * on the end outside of "zoomed".
      * @param zoomed the {@link ZoomableNodeContainer} which gets zoomed
      */private void calculateDecision2(HashMap<ConnectableElement, JoinRelation> map, ZoomableNodeContainer zoomed) {
      	
      	List<Decision> dec;
      	//ele: newJR connects zoomed <-> ele
      	for (ConnectableElement ele : map.keySet()) {
      		String s = "";
          	dec = new ArrayList<Decision>();

 			//--TARGET-CONNECTIONS----------------------
           	for (Connection c : ele.getTargetConnections()) {
      			if (!(c  instanceof JoinRelation))
      				continue;
      			
      			//here all decision of JRs from children of zoomed to ele are added to the map(ele) connection
  				if (zoomed.hasChildRecursively(c.getSource())) {
  					((JoinRelation)c).setAbstrectedIn(map.get(ele));
  					if (((JoinRelation)c).getDecision() != null) {
  						dec.add(((JoinRelation)c).getDecision());
  						if (!s.contains(((JoinRelation)c).getDecisionNr())) {
  	     					if (s == "")
  	     						s = ((JoinRelation)c).getDecisionNr();
  	     					else
  	     						s += ", "+((JoinRelation)c).getDecisionNr();
  						}
 					}
  				}	
      		}
           	
         	//in the case where ele is zoomed out, we have to add all decisions from JRs of its children (incl. Transitions)
          	//to children of zoomed
 			if (ele instanceof ZoomableNodeContainer && !((ZoomableNodeContainer)ele).isZoomedIn()) {
 				Iterator<ModelElement> itr = ((ZoomableNodeContainer)ele).getRecursiveChildrenIterator();
     			while (itr.hasNext()) {
     				ModelElement me = itr.next();
     				if (me instanceof Node)  {
     					for (Connection con : ((Node)me).getTargetConnections()) {
     						if (con instanceof JoinRelation) {
     							//if (zoomed.hasChildRecursively(con.getSource())) {
     							
     							ModelElement tmpE = null;
     							if (map.get(ele).getTarget() != ele)
     								tmpE = map.get(ele).getTarget();
     							else 
     								tmpE = map.get(ele).getSource();
     							
     							if (tmpE == con.getSource() || tmpE instanceof ZoomableNodeContainer && 
     									((ZoomableNodeContainer)tmpE).hasChildRecursively(con.getSource())) {
     								((JoinRelation)con).setAbstrectedIn(map.get(ele));
     			 					if (((JoinRelation)con).getDecision() != null) {
     			 						dec.add(((JoinRelation)con).getDecision());
     			 						if (!s.contains(((JoinRelation)con).getDecisionNr())) {
     			 	     					if (s == "")
     			 	     						s = ((JoinRelation)con).getDecisionNr();
     			 	     					else
     			 	     						s += ", "+((JoinRelation)con).getDecisionNr();
     			 						}
     			 					}
     							} 
     						} 
     						else if (con instanceof Transition) {
     							for (Connection tmpC : con.getTargetConnections()) {
     								if (tmpC instanceof JoinRelation) {
     									if (zoomed.hasChildRecursively(tmpC.getSource())) {
     										((JoinRelation)tmpC).setAbstrectedIn(map.get(ele));
     					 					if (((JoinRelation)tmpC).getDecision() != null) {
     	    			 						dec.add(((JoinRelation)tmpC).getDecision());
     	    			 						if (!s.contains(((JoinRelation)tmpC).getDecisionNr())) {
     	    			 	     					if (s == "")
     	    			 	     						s = ((JoinRelation)tmpC).getDecisionNr();
     	    			 	     					else
     	    			 	     						s += ", "+((JoinRelation)tmpC).getDecisionNr();
     	    			 						}
     	    			 					}
     	    							}
     								}
     							}
     						}
     					}
     				}
     			}
 			}
 					
 		  	
 			//--SOURCE-CONNECTIONS----------------------
 			for (Connection c : ele.getSourceConnections()) {
      			if (!(c  instanceof JoinRelation))
      				continue;
      			
      			//here all decision of JRs from children of zoomed to ele are added to the map(ele) connection
  				if (zoomed.hasChildRecursively(c.getTarget())) {
  					((JoinRelation)c).setAbstrectedIn(map.get(ele));
  					if (((JoinRelation)c).getDecision() != null) {
  						dec.add(((JoinRelation)c).getDecision());
  						if (!s.contains(((JoinRelation)c).getDecisionNr())) {
      						if (s == "")
      							s = ((JoinRelation)c).getDecisionNr();
      						else
      							s += ", "+((JoinRelation)c).getDecisionNr();
  						}
 					}
  				}	
  				
  				//Transition Case
  				if (c.getTarget() instanceof Transition) {
  					if (zoomed.hasChildRecursively(((Transition)c.getTarget()).getSource())) {
  						((JoinRelation)c).setAbstrectedIn(map.get(ele));
  	 					if (((JoinRelation)c).getDecision() != null) {
 		 						dec.add(((JoinRelation)c).getDecision());
 		 					if (!s.contains(((JoinRelation)c).getDecisionNr())) {
 		 	     				if (s == "")
 		 	     					s = ((JoinRelation)c).getDecisionNr();
 		 	     				else
 		 	     					s += ", "+((JoinRelation)c).getDecisionNr();
 		 					}
 		 				}
  					}
  				}
      		}
           	
         	//in the case where ele is zoomed out, we have to add all decisions from JRs of its children
          	//to children of zoomed
 			if (ele instanceof ZoomableNodeContainer && !((ZoomableNodeContainer)ele).isZoomedIn()) {
 				Iterator<ModelElement> itr = ((ZoomableNodeContainer)ele).getRecursiveChildrenIterator();
     			while (itr.hasNext()) {
     				ModelElement me = itr.next();
     				if (me instanceof Node)  {
     					for (Connection con : ((Node)me).getSourceConnections()) {
     						if (con instanceof JoinRelation) {
     							//if (zoomed.hasChildRecursively(con.getTarget())) {
     							
     							ModelElement tmpE = null;
     							if (map.get(ele).getTarget() != ele)
     								tmpE = map.get(ele).getTarget();
     							else 
     								tmpE = map.get(ele).getSource();
     							
     							if (con.getTarget() == tmpE  || tmpE instanceof ZoomableNodeContainer 
     									&& ((ZoomableNodeContainer)tmpE).hasChildRecursively(con.getTarget())) {
     							
     								((JoinRelation)con).setAbstrectedIn(map.get(ele));
     			 					if (((JoinRelation)con).getDecision() != null) {
     			 						dec.add(((JoinRelation)con).getDecision());
     			 						
     			 						if (!s.contains(((JoinRelation)con).getDecisionNr())) {
     			 							if (s == "")
     			 								s = ((JoinRelation)con).getDecisionNr();
     			 							else
     			 								s += ", "+((JoinRelation)con).getDecisionNr();
     									}
     			 					}
     							} else if (con.getTarget() instanceof Transition &&
     								zoomed.hasChildRecursively(((Transition)con.getTarget()).getTarget())) {
     								
     								((JoinRelation)con).setAbstrectedIn(map.get(ele));
     			 					if (((JoinRelation)con).getDecision() != null) {
     			 						dec.add(((JoinRelation)con).getDecision());
     			 						
     			 						if (!s.contains(((JoinRelation)con).getDecisionNr())) {
     			 							if (s == "")
     			 								s = ((JoinRelation)con).getDecisionNr();
     			 							else
     			 								s += ", "+((JoinRelation)con).getDecisionNr();
     									}
     			 					}
     								
     							}
     						} 
     					}
     				}
     			}
 			}
 			map.get(ele).setAbstractedDecNrs(s);
      	}
      }

}