package ch.unizh.ifi.adora.representation.management;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.gef.commands.Command;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationNodeBloat;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationNodeShrink;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNode;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class is responsible for generating (composing) the Implementation
 * Proposal.
 * 
 * @author 			Jaro Habr
 * @copyright 		Requirements Engineering Research Group
 *            		University of Zurich
 *            		Department of Informatics
 * @history 		2008-08-08 JH First Version
 * @version			$Date: 2008-09-20 15:08:56 +0200 (Sat, 20 Sep 2008) $, $Author: jaro $
 */
public class ImplPropComposer {

	/**
	 * Singleton Object
	 */
	private static ImplPropComposer composer;
	
	/**
	 * Holds the model elements in the inverted order.
	 */
	private ArrayList<ConnectableElement> invertedModelElements;
	
	/**
	 * Holds the visibility value of the model elements before the composition
	 * has been started.
	 */
	private ArrayList<Boolean> visibilityStates;
	
	/**
	 * Holds the zoomedId states of the nodes.
	 */
	private ArrayList<Boolean> zoomedInStates;
	
	/**
	 * Holds the size of the nodes (dimension).
	 */
	private ArrayList<Dimension> nodeSizes;
		
	/**
	 * Used to implement property change support.
	 */
	private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

	/**
	 * Returns the viewManager. If it doesn't exist yet, this method instantiates
	 * one.
	 * 
	 * @return ViewManager A singleton instance.
	 */
	public static ImplPropComposer getInstance() {
		if (composer == null) {
			composer = new ImplPropComposer();
		}
		return composer;
	}
	
	/**
	 * Starts the filtering process.
	 * 
	 * @param Vector<ConnectableElement> selectedModelElements
	 * @param boolean visible
	 */
	public void run(Vector<ConnectableElement> selectedModelElements,
			boolean visible) {
		firePropertyChange(ViewManagementConstants.NODE_FILTER_START, null,	null);

		if (visible == true) {
			processModelElements();
		}

		applyFilter(selectedModelElements, visible);
		firePropertyChange(ViewManagementConstants.NODE_FILTER_END, null, null);
	}
	
	/**
	 * Applies the filter on the current adora model.
	 * 
	 * @param Vector<ConnectableElement> selectedModelElements
	 */
	private void applyFilter(Vector<ConnectableElement> selectedModelElements,
			boolean visible) {
		if (visible == true) {
			compose(selectedModelElements, visible);
		} else if (visible == false) {
			decompose(invertedModelElements);
		}
	}
	
	/**
	 * Sets the adora model to the original state.
	 * 
	 * @param ArrayList<ConnectableElement> invertedModelElements
	 */
	private void decompose(ArrayList<ConnectableElement> invertedModelElements) {
		List<ZoomableNodeContainer> parents = new ArrayList<ZoomableNodeContainer>();
		ZoomableNode zoomableNode = null;
		int i = invertedModelElements.size();
		while (i != 0) {
			Node node = (Node) invertedModelElements.get(i - 1);
			if (node.isVisible() != visibilityStates.get(i - 1)) {
				node.setVisible(visibilityStates.get(i - 1));
			}

			// if node is shrinked now and wasn't shrinked before composition -> bloat
			if ((node.getSize().height == 1)
					&& (nodeSizes.get(i - 1).height != 1)) {
				bloatNode(node);
			// if node is bloated now and was shrinked before composition -> shrink
			} else if ((node.getSize().height != 1)
					&& (nodeSizes.get(i - 1).height == 1)) {
				shrinkNode(node);
			}

			if (node.isZoomedIn() != zoomedInStates.get(i - 1)) {
				zoomableNode = (ZoomableNode) node;
				zoomableNode.setZoomedIn(zoomedInStates.get(i - 1));
			}

			if (node.getParent() instanceof ZoomableNodeContainer
					&& !parents.contains(node.getParent())) {
				parents.add((ZoomableNodeContainer) node.getParent());
			}
			i--;
		}
		for (ZoomableNodeContainer container : parents) {
			ConnectionAbstractionAlgorithm.getInstance().run(container);
		}
	}
	
	/**
	 * Processes the filter and composes the implementation proposal.
	 * 
	 * @param Vector<ConnectableElement> selectedModelElements
	 * @param boolean visible
	 */
	@SuppressWarnings("unchecked")
	private void compose(Vector<ConnectableElement> selectedModelElements,
			boolean visible) {
		Node currentNode = null;
		Iterator iterator = invertedModelElements.iterator();
		List<ZoomableNodeContainer> parents = new ArrayList<ZoomableNodeContainer>();
		while (iterator.hasNext()) {
			currentNode = (Node) iterator.next();
			if (!selectedModelElements.contains(currentNode)
					&& !currentNode.wasVisited()) {
				if (currentNode.isVisible() == visible) {
					currentNode.setVisible(!visible);
					shrinkNode(currentNode);
				} else if (currentNode.isVisible() != visible) {
					if (currentNode.getSize().height != 1
							&& currentNode.getSize().width != 1) {
						shrinkNode(currentNode);
					}
				}
				// get parents
				if (currentNode.getParent() instanceof ZoomableNodeContainer
						&& !parents.contains(currentNode.getParent())) {
					parents.add((ZoomableNodeContainer) currentNode.getParent());
				}
			} else if (currentNode.wasVisited()) {
				currentNode.setVisited(false);
			} else {
				// views: in case node is invisible and shrinked and has to be
				// visible
				if (currentNode.isVisible() != visible) {
					if (currentNode.getSize().height == 1
							&& currentNode.getSize().width == 1) {
						bloatNode(currentNode);
					}
					// if fisheye has been used too
					currentNode.setVisible(visible);
					currentNode.getFirstVisibleParentContainer().invalidate();
				}
				processParents(currentNode, visible);
			}
		}
		// calculate abstract connections
		for (ZoomableNodeContainer container : parents) {
			ConnectionAbstractionAlgorithm.getInstance().run(container);
		}
	}
	
	/**
	 * Processes all children of the "fisheye" parent and makes them visible,
	 * if not visible yet.
	 * 
	 * @param ZoomableNode node
	 * @param boolean visible
	 */
	@SuppressWarnings("unused")
	private void processChildren(ZoomableNode node, boolean visible,
			Vector<ConnectableElement> selectedModelElements) {
		ZoomableNode parent = node;
		@SuppressWarnings("unused")
		ZoomableNode child = null;

		int index = invertedModelElements.indexOf(parent);

		for (int i = index - 1; i > -1; i--) {
			if (invertedModelElements.get(i) instanceof ZoomableNode) {
				try {
					child = (ZoomableNode) invertedModelElements.get(i);
				} catch (ClassCastException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	/**
	 * Returns an arrayList with inverted order of model elements.
	 * 
	 * @return ArrayList<ConnectableElement> modelElements
	 */
	@SuppressWarnings("unchecked")
	private void processModelElements() {		
		Iterator childIterator = AdoraPlugin.getActiveModel()
				.getRecursiveChildrenIterator();
		Vector<ZoomableNode> zoomedInElements = new Vector<ZoomableNode>();
		invertedModelElements = new ArrayList<ConnectableElement>();
		visibilityStates = new ArrayList<Boolean>();
		zoomedInStates = new ArrayList<Boolean>();
		nodeSizes = new ArrayList<Dimension>();
		ConnectableElement element = null;
		
		while (childIterator.hasNext()) {
			element = (ConnectableElement) childIterator.next();
			invertedModelElements.add(0, element);
			visibilityStates.add(0, element.isVisible());
			zoomedInStates.add(0, ((IZoomNode) element).isZoomedIn());
			nodeSizes.add(0, element.getSize());
			
			if (!(element instanceof Actor)) {
				if (!((ZoomableNode) element).isZoomedIn()) {
					zoomedInElements.add((ZoomableNode) element);
				}
			}	
		}
		if (zoomedInElements.size() != 0) {
			zoomInNodes(zoomedInElements);
		}
	}
	
	/**
	 * Disables the fisheye zoom temporally, if used.
	 * 
	 * @param Vector<ZoomableNode> zoomedInElements
	 */
	private void zoomInNodes(Vector<ZoomableNode> zoomedInElements) {
		if (zoomedInElements.size() != 0) {
			for (ZoomableNode node : zoomedInElements) {
				node.setZoomedIn(true);
			}
		}
	}
	
	/**
	 * Sets the parent of the current node to visible, if current node was
	 * set to visible.
	 * 
	 * @param Node currentNode
	 * @param boolean visible
	 */
	private void processParents(Node currentNode, boolean visible) {
		Node parent = null;
		while (!(currentNode.getParent() instanceof AdoraModel)) {
			parent = (Node) currentNode.getParent();
			if (parent.isVisible() != visible) {
				parent.setVisible(visible);
				bloatNode(parent);
			}
			parent.setVisited(true);
			currentNode = parent;
		}
	}
	
	/**
	 * Bloats the node and refreshes its parent.
	 * 
	 * @param Node currentNode
	 */
	private void bloatNode(Node currentNode) {
		Command command = new RepresentationNodeBloat(currentNode);
		executeCommandOnStack(command);
		currentNode.getFirstVisibleParentContainer().invalidate();
	}
	
	/**
	 * Shrinks the node and refreshes its parent.
	 * 
	 * @param Node currentNode
	 */
	private void shrinkNode(Node currentNode) {
		Command command = new RepresentationNodeShrink(currentNode);
		executeCommandOnStack(command);
		currentNode.getFirstVisibleParentContainer().invalidate();
	}	
	
	/**
	 * Adds a listener.
	 * 
	 * @param PropertyChangeListener listener
	 */
	public synchronized void addPropertyChangeListener(
			PropertyChangeListener listener) {
		if (listener == null) {
			throw new IllegalArgumentException();
		}
		propertyChangeSupport.addPropertyChangeListener(listener);
	}
	
	/**
	 * Removes the listener.
	 * 
	 * @param PropertyChangeListener listener
	 */
	public synchronized void removePropertyChangeListener(
			PropertyChangeListener listener) {
		if (listener != null) {
			propertyChangeSupport.removePropertyChangeListener(listener);
		}
	}
	
	/**
	 * Report a property change to registered listeners (for example edit
	 * parts).
	 * 
	 * @param String property
	 * @param Object oldValue
	 * @param Object newValue
	 */
	public void firePropertyChange(String property, Object oldValue,
			Object newValue) {
		if (propertyChangeSupport.hasListeners(property)) {
			propertyChangeSupport.firePropertyChange(property, oldValue,
					newValue);
		}
	}
	
	/**
	 * Executes a given {@link org.eclipse.gef.commands.Command} on
	 * the {@link org.eclipse.gef.commands.CommandStack} of the editor.
	 * 
	 * @param Command command
	 */
	protected void executeCommandOnStack(Command command) {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				((AdoraModelEditor) editor).getCommandStack().execute(command);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
