package ch.unizh.ifi.adora.representation.management;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationNodeBloat;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationNodeShrink;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.CrosscuttingConcern;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.IConcealableConnection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.utils.ObservablePropertyContainer;

/**
 * This manager is resonsible for updating the visibility of
 * all hidable model elements. 
 *
 * @author              Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             04.04.2006 IV First Version
 *                      06.06.2006 IV Reintroduced the use of RepresentationNodeBloat
 *                                    and RepresentationNodeShrink
 *                      24.10.2007 SM Adaption for the view filtering of aspects, Bug #835
 * @version             $Date: 2009-07-01 15:32:13 +0200 (Wed, 01 Jul 2009) $, $Author: jehle $, $Revision: 4419 $
 */
public class ViewManager extends ObservablePropertyContainer {

	/**
	 * Singleton Object
	 */
	private static ViewManager viewManager;

	/**
	 * Returns the BehavorialView object, if it doesn't exist, this method instantiates one.
	 * @return ViewManager A singleton instance.
	 */
	public static ViewManager getInstance() {
		if (viewManager == null) {
			viewManager = new ViewManager();
		}
		return viewManager;
	}

	/**
	 * Instantiates a new ViewManager object. 
	 */
	private ViewManager() {
	}

	/**
	 * Applies view filters on the model.
	 * 
	 * @param type Determins which nodetypes have to be filtered out.
	 */
	public void applyViewFilter(Class type)
	{
		if (type.equals(Association.class) || type.equals(JoinRelation.class))
		{
			applyConnectionFiter();
		}
		else
		{
			Class[] types;
			
			// if the behavioral view is hidden
			// then filter states and exit points
			if (type.equals(State.class))
			{
				types = new Class[2];				
				types[1] = ExitPoint.class;
			}
			else if (type.equals(CrosscuttingConcern.class))
			{
				types = new Class[] {CrosscuttingConcern.class};
			}
			else
			{
				// else filter the given element type
				types = new Class[1];				
			}
			
			types[0] = type;
			firePropertyChange(ViewManagementConstants.NODE_FILTER_START, type, null);
			applyNodeFilter(getActualModel().getViewFilterState(type), types);
			firePropertyChange(ViewManagementConstants.NODE_FILTER_END, type, null);
		}
	}

	/**
	 * Connections have to be dealt seperatly since they aren't
	 * directly known by the model.
	 * Therefore each node has to be checked for associations.
	 */
	private void applyConnectionFiter() {
		Iterator<ModelElement> childIter = getActualModel().getRecursiveChildrenIterator();
		Iterator<Connection> conIter;
		Connection con;
		while (childIter.hasNext()) {
			conIter = ((Node) childIter.next()).getConnectionsIterator();
			while (conIter.hasNext()) {
				con = (Connection) conIter.next();
				if (con instanceof IConcealableConnection) {
					((IConcealableConnection) con).updateVisibility();
				}
			}
		}
	}

	/**
	 * Applies viewfilters which affects node elements.
	 * 
	 * @param hide Determines if the node has to be hidden or shown.
	 * @param type Determines which node types have to be filtered out.
	 */
	private void applyNodeFilter(boolean hide, Class... type)
	{
		// get all children in the model
		Iterator<ModelElement> childIter = AdoraPlugin.getActiveModel().getRecursiveChildrenIterator();

		Command cmd;
		Node currentNode;

		// list which notes all parent whose children were hidden
		// needed for applying the line abstraction algorithm later.
		List<ZoomableNodeContainer> parents = new ArrayList<ZoomableNodeContainer>();
		
		while (childIter.hasNext())
		{
			currentNode = ((Node) childIter.next());
			
			boolean match = false;
			
			// the decision if a node is a crosscutting concern is made here.
			// if not, do the normal filter procedure
			if (type[0].equals(CrosscuttingConcern.class))
			{
				if (currentNode.getClass().equals(AspectContainer.class))
				{
					match = !((AspectContainer) currentNode).isVariability();
				}
			}
			else
			{
				match = checkIfTypeIsInList(currentNode.getClass(),type);
			}
			if (match)
			{
				if (hide)
				{
					cmd = new RepresentationNodeShrink(currentNode);
					executeCommandOnStack(cmd);
					currentNode.updateVisibility();
				}
				else
				{
					currentNode.updateVisibility();
					cmd = new RepresentationNodeBloat(currentNode);
					executeCommandOnStack(cmd);
				}
				
				// Don't register a parent node a second time
				// as this hampers otherwise the performance
				if (currentNode.getParent() instanceof ZoomableNodeContainer && 
					!parents.contains(currentNode.getParent()))
				{
					parents.add((ZoomableNodeContainer)currentNode.getParent());
				}
				else 
					// The actor is not a zoomable node container although it can have an
					// association connected, therefore check.
					if (type.equals(Actor.class) /*|| type.equals(AspectContainer.class)*/)
					{
    					Iterator<Association> iter = ((Node)currentNode).getAssociationsIterator();
    					Association asso;
    					Node n;

    					while (iter.hasNext())
    					{
    						asso = (Association)iter.next();
    						n = asso.getSource() == currentNode ? asso.getTarget() : asso.getSource();
    						
    						if (n instanceof ZoomableNodeContainer)
    						{
    							parents.add((ZoomableNodeContainer)n);
    						}
    						else if (!(n.getFirstVisibleParentContainer() instanceof AdoraModel))
    						{
    							parents.add((ZoomableNodeContainer)n.getFirstVisibleParentContainer());
    						}
    					}
				}
			}
		}
		
		// apply the 
		for (ZoomableNodeContainer container : parents) {
			ConnectionAbstractionAlgorithm.getInstance().run(container);
		}
	}

	
	/**
	 * Checks wheter a given class type is contained in the given array of class
	 * types.
	 * 
	 * @pre       type != null && checkType != null
	 * @post      true
	 * @param type The type to check.
	 * @param checkTypes The array with which the given type is compared.
	 * @return Returns true if the given type is contained in the given array.
	 */
	private boolean checkIfTypeIsInList(Class type, Class... checkTypes) {
		boolean retVal = false;
		for (int i = 0; i < checkTypes.length; i++) {
			if (type.equals(checkTypes[i])) {
				retVal = true;
				break;
			}
		}
		return retVal;
	}
	
	
	 /**
     * Executes a given {@link org.eclipse.gef.commands.Command} on the {@link org.eclipse.gef.commands.CommandStack} 
     * of the editor.
     *
     * @param cmd The {@link org.eclipse.gef.commands.Command} which gets executed on the 
     * {@link org.eclipse.gef.commands.CommandStack}.
     */
    protected void executeCommandOnStack(Command cmd){
    	IWorkbenchPage page = PlatformUI.getWorkbench()
		.getActiveWorkbenchWindow().getActivePage();
		
    	try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				((AdoraModelEditor)editor).getCommandStack().execute(cmd);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
    }

	/**
	 * @return The adora model on which the filteraction is applied.
	 */
	private AdoraModel getActualModel() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				return ((AdoraModelEditor) editor).getModel();
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}
}