package ch.unizh.ifi.adora.representation.routing.algorithm;

import java.util.Iterator;

import org.eclipse.draw2d.AbstractRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.LayoutListener;

/**
 * A GEF router that listens for layout changes and informs revalidates the
 * connection figure if it has to be rerouted.
 * 
 * Subclasses can override processConnection() for a different behavior.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-04-03 TR First Version
 * @version				$Date: 2009-10-28 09:29:27 +0100 (Wed, 28 Oct 2009) $, $Author: treinhard $, $Revision: 4902 $
 *
 */
public abstract class LayoutObservingRouter extends AbstractRouter {

    /**
     * The line that has to be routed
     */
    protected Connection line;
    
    /**
     * The listener that is informed about layout changes
     */
	private LayoutListener listener = new LayoutTracker();
	
	/**
	 * The layer that contains all the connections of the model.
	 */
	protected Layer connectionLayer;
    
    /**
     * Listener that is informed about layout changes (added, moved, removed nodes)
     * add reroutes the affected connections.
     *
     */
    private class LayoutTracker extends LayoutListener.Stub {
    	
       /**
    	* Called when the node given as argument is removed.
    	* 
    	* @param child the removed node
    	*/
		public void remove(IFigure child) {
			child.removeLayoutListener(listener);
		}

		/**
		 * Called when the argument child is added or moved.
		 * 
		 * @param child the newly added or moved node
		 * @param constraint the new bounds of the node child
		 */
		public void setConstraint(IFigure child, Object constraint) {
			if(constraint == null) {
				child.addLayoutListener(listener);
			} 
			else if(!constraint.equals(child.getBounds())) {

				Iterator iter = connectionLayer.getChildren().iterator();
    			while(iter.hasNext()) {
    				Connection con = (Connection)iter.next();
    				if(isAffectedBy(con, child)) {
    					con.revalidate();
    				}
    			}
    			
			}
			
		}
		
	}
    
    
    /**
     * Constructor for the tile maze router. Registers the layout listener at all
     * existing figures in the model.
     * 
     * @param root the root element in the model
     * @param connectionLayer the layer that contains all the connections
     */
    public LayoutObservingRouter(IFigure root, Layer connectionLayer) {
  		
    		this.connectionLayer = connectionLayer;
    		registerAll(root);
 	
    }

    
    /**
     * Registers recursively the layout listener at the figure given as argument and
     * all its children.
     * 
     * @param figure the figure to register the layout listener recursively at
     */
    private void registerAll(IFigure figure) {
    	
    		figure.addLayoutListener(listener);
    	
    		Iterator iter = figure.getChildren().iterator();
    		while(iter.hasNext()) {
    			IFigure child = (IFigure)iter.next();
    			registerAll(child);
    		}
    	
    }
    
    
    /**
     * Returns true if the connection <i>con</i> has to be rerouted if the bounds
     * of node <i>node</i> change and false otherwise.
     * 
     * @param con the connection that has to be checked
     * @param node the node whose bounds have been changed
     * @return true if the connection has to be routed, false otherwise
     */
    protected boolean isAffectedBy(Connection con, IFigure node) {
    	
    	IFigure source = con.getSourceAnchor().getOwner();
		IFigure target = con.getTargetAnchor().getOwner();
		
		// This may happen during the loading of the model
		if(source == null || target == null) {
			
			return false;
			
		}
		
		// Don't route a line if its source and target have the same parent
		// and this parent is not the parent of the changed node.
		// This simple rule already excludes almost all transitions.
		if(source.getParent() == target.getParent() && source.getParent() != node.getParent()
				&& source.getParent() != node) {
			
			return false;
			
		}
		
		// This routes more lines than necessary! But a full check for which
		// node are hierarchically affected seems to be slower than the router
		// itself!
		
		return true;
  	
    }

}