package ch.unizh.ifi.adora.representation.routing.algorithm.shortestpath;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.draw2d.AbstractRouter;
import org.eclipse.draw2d.Bendpoint;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.FigureListener;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.LayoutListener;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.PrecisionPoint;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.view.figures.label.NodeLabel;
import ch.unizh.ifi.adora.view.layers.MultipleRoutersConnectionLayer;

/**
 * This class is responsible for routing connections on the shortes path that
 * they don't intersect any {@link ch.unizh.ifi.adora.model.representation.Node}s,
 * except the anchestors.
 * <p>
 * More precisely this class only delegates the routing callculations to the
 * {@link ch.unizh.ifi.adora.representation.routing.algorithm.shortestpath.SPRouter}
 * associated with the {@link org.eclipse.draw2d.Connection} to route. For this
 * mapping ({@link org.eclipse.draw2d.Connection} ->
 * {@link ch.unizh.ifi.adora.representation.routing.algorithm.shortestpath.SPRouter})
 * the {@link #algoMap} is used.
 * <p>
 * This router has to be implemented from scratch although draw2d offers the
 * {@link org.eclipse.draw2d.ShortestPathConnectionRouter}. The problem with
 * this router is that it is only capable of routing
 * {@link org.eclipse.draw2d.Connection}s in non hierarchical models.
 * 
 * 
 * @author Whitney Sorenson, Randy Hudson, Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 04.08.2006 IV First Version
 * @version $Date: 2009-10-20 10:36:35 +0200 (Tue, 20 Oct 2009) $, $Author: benz $, $Revision: 4871 $
 */
public final class SPManager extends AbstractRouter {

	private class LayoutTracker extends LayoutListener.Stub {
		public void postLayout(IFigure container) {
			processLayout();
		}

		public void remove(IFigure child) {
			removeMeAndyMyChildren(child);
		}

		public void setConstraint(IFigure child, Object constraint) {
			addChild(child);
		}
	}

	/**
     * This Map is the main difference to the
     * {@link org.eclipse.draw2d.ShortestPathConnectionRouter} of draw2d. It
     * stores the
     * {@link ch.unizh.ifi.adora.representation.routing.algorithm.shortestpath.SPRouter}
     * for each {@link org.eclipse.draw2d.Connection}.
     */
	private Map<PolylineConnection, SPRouter> algoMap;

	private Map<Connection, Object> constraintMap = new HashMap<Connection, Object>();

	private Map<IFigure, Obstacle> figuresToBounds;

	private Map<PolylineConnection, Path> connectionToPaths;

	private boolean isDirty;

	private IFigure container;

	private Set<Connection> staleConnections = new HashSet<Connection>();

	private LayoutListener listener = new LayoutTracker();

	private FigureListener figureListener = new FigureListener() {
		public void figureMoved(IFigure source) {
			Rectangle newBounds = source.getBounds().getCopy();
			if (figuresToBounds == null)
				return;
			if (updateObstacle(source, newBounds)) {
				queueSomeRouting();
				isDirty = true;
			}
			figuresToBounds.put(source, new Obstacle(newBounds));
		}

	};

	private boolean ignoreInvalidate;

	/**
     * The layer that contains all the connections of the model.
     */
	protected Layer connectionLayer;

	/**
     * Creates a new shortest path router with the given container. The
     * container contains all the figure's which will be treated as obstacles
     * for the connections to avoid. Any time a child of the container moves,
     * one or more connections will be revalidated to process the new obstacle
     * locations. The connections being routed must not be contained within the
     * container.
     * 
     * @param container
     *            the container
     */
	public SPManager(IFigure container, Layer connectionLayer) {
		// changed
		algoMap = new HashMap<PolylineConnection, SPRouter>();
		isDirty = false;
		this.container = container;
		this.connectionLayer = connectionLayer;
	}

	/* package */void queueSomeRouting() {
		if (connectionToPaths == null || connectionToPaths.isEmpty())
			return;
		try {
			ignoreInvalidate = true;
			((Connection) connectionToPaths.keySet().iterator().next())
					.revalidate();
		} finally {
			ignoreInvalidate = false;
		}
	}

	/**
     * Adds the
     * <li>rootChild</li>
     * and all it's children to the obstacles.
     * 
     * @param rootChild
     *            Child which gets added.
     */
	/* package */void addChild(IFigure rootChild) {
		List<IFigure> l = getMeAndMyOffspring(rootChild);
		Iterator<IFigure> iter = l.iterator();
		IFigure child = null;

		if (figuresToBounds.containsKey(rootChild))
			return;

		while (iter.hasNext()) {
			child = iter.next();
			if (connectionToPaths == null)
				continue;
			if (child instanceof NodeLabel) {
				continue;
			}
			Rectangle bounds = child.getBounds().getCopy();

			addObstacle(child);

			figuresToBounds.put(child, new Obstacle(bounds));

			// testAndDirtyPaths(bounds);

			child.addFigureListener(figureListener);
			child.addLayoutListener(listener);
			isDirty = true;
		}
	}

	/**
     * Removes the
     * <li>rootChild</li>
     * and all it's children to the obstacles.
     * 
     * @param rootChild
     *            Child which gets added.
     */
	private void removeMeAndyMyChildren(IFigure rootChild) {
		List<IFigure> l = getMeAndMyOffspring(rootChild);
		Iterator<IFigure> iter = l.iterator();
		IFigure child = null;

		while (iter.hasNext()) {
			child = iter.next();

			if (connectionToPaths == null)
				return;
			Obstacle bounds = new Obstacle(child.getBounds().getCopy());

			// boolean change = algorithm.removeObstacle(bounds);

			figuresToBounds.remove(child);

			boolean change = removeObstacle(bounds);

			child.removeFigureListener(figureListener);
			child.removeLayoutListener(listener);
			if (change) {
				isDirty = true;
				queueSomeRouting();
			}
			iter.remove();
		}
	}

	private void hookAll() {
		figuresToBounds = new HashMap<IFigure, Obstacle>();
		addChild(container);
	}

	private void unhookAll() {
		container.removeLayoutListener(listener);
		if (figuresToBounds != null) {
			Iterator<IFigure> figureItr = figuresToBounds.keySet().iterator();
			while (figureItr.hasNext()) {
				IFigure child = (IFigure) figureItr.next();
				child.removeLayoutListener(listener);
			}
			figuresToBounds = null;
		}
	}

	/**
     * Gets the constraint for the given {@link Connection}. The constraint is
     * the paths list of bend points for this connection.
     * 
     * @param connection
     *            The connection whose constraint we are retrieving
     * @return The constraint
     */
	public Object getConstraint(Connection connection) {
		return constraintMap.get(connection);
	}

	/**
     * @see org.eclipse.draw2d.ConnectionRouter#invalidate(org.eclipse.draw2d.Connection)
     */
	public void invalidate(Connection connection) {
		if (ignoreInvalidate)
			return;
		staleConnections.add(connection);
		isDirty = true;
	}

	/**
     * @see org.eclipse.draw2d.ConnectionRouter#remove(org.eclipse.draw2d.Connection)
     */
	public void remove(Connection connection) {
		staleConnections.remove(connection);
		constraintMap.remove(connection);
		if (connectionToPaths == null)
			return;
		Path path = (Path) connectionToPaths.remove(connection);
		removePath(connection, path);
		isDirty = true;
		if (connectionToPaths.isEmpty()) {
			unhookAll();
			connectionToPaths = null;
		} else {
			// Make sure one of the remaining is revalidated so that we can
			// re-route again.
			queueSomeRouting();
		}
		// changed
		algoMap.remove(connection);

	}

	private void processLayout() {
		if (staleConnections.isEmpty())
			return;
		((Connection) staleConnections.iterator().next()).revalidate();
	}

	/**
     * @see ConnectionRouter#route(Connection)
     */
	public void route(Connection conn) {

		// Routes reflexiv connections orthogonal
		if (conn.getSourceAnchor().getOwner() == conn.getTargetAnchor()
				.getOwner()) {

			((MultipleRoutersConnectionLayer) connectionLayer)
					.getSpecificRouter(
							MultipleRoutersConnectionLayer.OrthogonalRouter)
					.route(conn);
			return;
		}

		if (isDirty) {

			ignoreInvalidate = true;
			processStaleConnections();

			isDirty = false;
			List<?> updated = solve(conn);
			Connection current;

			for (int i = 0; i < updated.size(); i++) {
				Path path = (Path) updated.get(i);
				current = (Connection) path.data;
				if (current.getSourceAnchor().getOwner() == current
						.getTargetAnchor().getOwner()) {
					continue;
				}

				PointList points = path.getPoints().getCopy();
				Point ref1, ref2, start, end;

				ref1 = new PrecisionPoint(points.getPoint(1));
				ref2 = new PrecisionPoint(points.getPoint(points.size() - 2));

				current.translateToAbsolute(ref1);
				current.translateToAbsolute(ref2);

				start = current.getSourceAnchor().getLocation(ref1).getCopy();
				end = current.getTargetAnchor().getLocation(ref2).getCopy();

				current.translateToRelative(start);
				current.translateToRelative(end);

				points.setPoint(start, 0);
				points.setPoint(end, points.size() - 1);

				current.setPoints(points);

			}
			ignoreInvalidate = false;
		}
	}

	/**
     * @since 3.0
     */
	private void processStaleConnections() {
		Iterator iter = staleConnections.iterator();
		if (iter.hasNext() && connectionToPaths == null) {
			connectionToPaths = new HashMap();
			hookAll();
		}

		while (iter.hasNext()) {
			PolylineConnection conn = (PolylineConnection) iter.next();

			Path path = (Path) connectionToPaths.get(conn);
			if (path == null) {
				path = new Path(conn);
				connectionToPaths.put(conn, path);
				addPath(conn, path);
			}

			List constraint = (List) getConstraint(conn);
			if (constraint == null)
				constraint = Collections.EMPTY_LIST;

			Point start = conn.getSourceAnchor().getReferencePoint().getCopy();
			Point end = conn.getTargetAnchor().getReferencePoint().getCopy();

			container.translateToRelative(start);
			container.translateToRelative(end);

			path.setStartPoint(start);
			path.setEndPoint(end);

			if (!constraint.isEmpty()) {
				PointList bends = new PointList(constraint.size());
				for (int i = 0; i < constraint.size(); i++) {
					if (!(constraint.get(i) instanceof Bendpoint)) {
						continue;
					}
					Bendpoint bp = (Bendpoint) constraint.get(i);
					bends.addPoint(bp.getLocation());
				}
				path.setBendPoints(bends);
			} else
				path.setBendPoints(null);

			isDirty |= path.isDirty;
		}
		staleConnections.clear();
	}

	/**
     * @see org.eclipse.draw2d.ConnectionRouter#setConstraint(org.eclipse.draw2d.Connection,
     *      java.lang.Object)
     */
	public void setConstraint(Connection connection, Object constraint) {
		// Connection.setConstraint() already calls revalidate, so we know that
		// a
		// route() call will follow.
		staleConnections.add(connection);
		constraintMap.put(connection, constraint);
		isDirty = true;
	}

	// changed ----------------------------
	/**
     * Evaluates the
     * <li>figure</li>
     * and all it's children to the obstacles.
     * 
     * @param figure
     *            Figure where all children are evaluated.
     * @return List, containing all Children of
     *         <li>figure</li>
     *         and the
     *         <li>figure</li>
     *         itself.
     */
	private List<IFigure> getMeAndMyOffspring(IFigure figure) {
		List<IFigure> l = new ArrayList<IFigure>();

		l.add(figure);
		if (figure.getChildren().size() == 0
				|| figure.getChildren().get(0) instanceof NodeLabel)
			return l;

		Iterator iter = figure.getChildren().iterator();
		while (iter.hasNext()) {
			l.addAll(getMeAndMyOffspring((IFigure) iter.next()));
		}
		return l;
	}

	/**
     * Called, each time an existing {@link IFigure} is moved.
     * 
     * @param movedFigure
     *            moved {@link IFigure}
     * @param newBounds
     *            new Bounds
     * @return true, if something changed.
     */
	private boolean updateObstacle(IFigure movedFigure, Rectangle newBounds) {
		boolean changed = false;
		Rectangle oldBounds = figuresToBounds.get(movedFigure);
		for (PolylineConnection c : algoMap.keySet()) {
			changed |= algoMap.get(c).updateObstacle(oldBounds, newBounds);
		}
		return changed;
	}

	/**
     * Called, each time an existing {@link IFigure} is removed.
     * 
     * @param bounds
     *            new Bounds.
     * @return true, if something changed.
     */
	private boolean removeObstacle(Rectangle bounds) {
		boolean changed = false;
		for (SPRouter algo : algoMap.values()) {
			changed |= algo.removeObstacle(bounds);
		}
		return changed;
	}

	/**
     * Called, each time a {@link Path} has to be added
     * 
     * @param conn
     *            {@link PolylineConnection} for which this {@link Path} is
     *            added.
     * @param path
     *            {@link Path} to add.
     */
	private void addPath(PolylineConnection conn, Path path) {
		SPRouter algo;
		algo = algoMap.get(conn);

		if (algo == null) {
			List l = new ArrayList<Obstacle>(cleanObstacles(figuresToBounds,
					conn).values());
			algo = new SPRouter(l, conn);
			algoMap.put(conn, algo);

		}
		algo.addPath(path);
	}

	/**
     * Removes all figures which are children of a source or target node.
     * 
     * @param map
     * @param conn
     * @return
     */
	private Map<IFigure, Obstacle> cleanObstacles(Map<IFigure, Obstacle> map,
			Connection conn) {
		Map<IFigure, Obstacle> cleandmap = new HashMap<IFigure, Obstacle>();
		IFigure source = conn.getSourceAnchor().getOwner();
		IFigure target = conn.getTargetAnchor().getOwner();

		for (IFigure f : map.keySet()) {
			if (f == source || f == target || !isAoffspringOfB(f, source)
					&& !isAoffspringOfB(f, target)) {
				cleandmap.put(f, map.get(f));
			}
		}
		return cleandmap;
	}

	/**
     * Checks if a figure <code>a</code> is an offspring of an other figure
     * <code>b</code>.
     * 
     * @param a
     *            possible child.
     * @param b
     *            possible parent.
     * @return <code>true</code> if a is an offspring of b.
     */
	private boolean isAoffspringOfB(IFigure a, IFigure b) {
		if (a == null) {
			return false;
		}
		if (a == b) {
			return true;
		}
		return isAoffspringOfB(a.getParent(), b);
	}

	/**
     * Does the actual routing.
     * 
     * @param conn
     * @return List containing all updated connections.
     */
	@SuppressWarnings("unchecked")
	private List solve(Connection con) {
		List updated = new ArrayList();

		for (SPRouter algo : algoMap.values()) {
			updated.addAll(algo.solve());
		}

		return updated;
	}

	/**
     * Called, each time a new {@link IFigure} is added.
     * 
     * @param figure
     *            {@link IFigure} which was added.
     * @return true, if something changed.
     */
	private void addObstacle(IFigure figure) {
		for (PolylineConnection c : algoMap.keySet()) {
			if (!isAoffspringOfB(figure, c.getSourceAnchor().getOwner())
					&& !isAoffspringOfB(figure, c.getTargetAnchor().getOwner())) {
				algoMap.get(c).addObstacle(figure.getBounds());
			}
		}
	}

	/**
     * Called, each time an existing {@link Path} has to be deleted
     * 
     * @param conn
     *            {@link PolylineConnection} for which this {@link Path} is
     *            deleted.
     * @param path
     *            {@link Path} to delete.
     */
	private void removePath(Connection conn, Path path) {
		if (algoMap.get(conn) != null)
			algoMap.get(conn).removePath(path);
	}
}