package ch.unizh.ifi.adora.ui;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.ide.IDE;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.util.ModelFileProperties;
import ch.unizh.ifi.adora.model.util.ModelFileProperty;
import ch.unizh.ifi.adora.model.util.ModelSerializer;

/**
 * This WizardPage can create an empty ADORA model file.
 * 
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Aug 30, 2006 AG First Version Jun 18, 2006 SM Extended version for
 *          storing file format.
 * @version $Date: 2007-06-05 14:07:40 +0200 (Tue, 05 Jun 2007) $, $Author: schoeni $, $Revision: 2749 $
 */
class AdoraFileCreationPage extends WizardNewFileCreationPage {
	/**
     * The extension of the model files.
     */
	private static final String DEFAULT_EXTENSION = ".adora";

	/**
     * Reference to the workbench.
     */
	private final IWorkbench workbench;

	/**
     * holds all the ui elements for choosing the file type.
     */
	private Composite fileTypeArea;

	/**
     * Combo box describing the type to choose for file format.
     */
	private Combo fileTypeCombo;

	/**
     * Stores the file type which was chosen to store the model.
     */
	private int fileFormat = ModelFileProperty.FILE_FORMAT_NO_TYPE_SET;

	/**
     * Create a new wizard page instance.
     * 
     * @param workbench
     *            the current workbench
     * @param selection
     *            the current object selection
     * @see AdoraModelCreationWizard#init(IWorkbench, IStructuredSelection)
     */
	public AdoraFileCreationPage(IWorkbench workbench,
			IStructuredSelection selection) {
		super("shapeCreationPage1", selection);
		this.workbench = workbench;
		setTitle("Create a new " + DEFAULT_EXTENSION + " file");
		setDescription("Create a new " + DEFAULT_EXTENSION + " file");
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#createControl(org.eclipse.swt.widgets.Composite)
     */
	public void createControl(Composite parent) {
		super.createControl(parent);
		setFileName("adoraModel" + AdoraModelCreationWizard.getfileCount()
				+ DEFAULT_EXTENSION);
		setPageComplete(validatePage());
	}

	/**
     * Returns an empty ADORA Model.
     * 
     * @return Returns a new and empty ADORA model.
     */
	private AdoraModel createDefaultContent() {
		return new AdoraModel();
	}

	/**
     * This method will be invoked, when the "Finish" button is pressed.
     * 
     * @see AdoraModelCreationWizard#performFinish()
     */
	public boolean finish() {
		// create a new file, result != null if successful
		fileFormat = this.fileTypeCombo.getSelectionIndex();

		IFile newFile = createNewFile();
		// get the filename of the create file.
		String absFileName = newFile.getLocation().toString();

		ModelFileProperties.getInstance().setProperty(absFileName,
				createDefaultContent(), this.getFileFormat());

		AdoraModelCreationWizard.increaseFileCount();

		// open newly created file in the editor
		IWorkbenchPage page = workbench.getActiveWorkbenchWindow()
				.getActivePage();

		if (newFile != null && page != null) {
			try {
				IDE.openEditor(page, newFile, true);
			} catch (PartInitException e) {
				e.printStackTrace();
				return false;
			}
		}

		return true;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#getInitialContents()
     */
	protected InputStream getInitialContents() {
		ByteArrayInputStream bais = null;
		try {
			ByteArrayOutputStream out = new ByteArrayOutputStream();

			switch (fileFormat) {
			case ModelFileProperty.FILE_FORMAT_XML_GZIPPED:
				ModelSerializer.serializeModelToXMLOutputStream(
						createDefaultContent(), out, true);
				break;
			case ModelFileProperty.FILE_FORMAT_XML:
				ModelSerializer.serializeModelToXMLOutputStream(
						createDefaultContent(), out, false);
				break;
			case ModelFileProperty.FILE_FORMAT_SERIALIZED:
				ModelSerializer.serializeModelToStream(out,
						createDefaultContent());
				break;
			}
			out.flush();

			bais = new ByteArrayInputStream(out.toByteArray());
			out.close();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return bais;
	}

	/**
     * Return true, if the file name entered in this page is valid.
     */
	private boolean validateFilename() {
		if (getFileName() != null && getFileName().endsWith(DEFAULT_EXTENSION)) {
			return true;
		}
		setErrorMessage("The 'file' name must end with " + DEFAULT_EXTENSION);
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#createAdvancedControls(org.eclipse.swt.widgets.Composite)
     */
	protected void createAdvancedControls(Composite parent) {
		fileTypeArea = new Composite(parent, SWT.NONE);
		GridLayout gl = new GridLayout();
		gl.numColumns = 2;
		fileTypeArea.setLayout(gl);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		// grid data settings for the fiel type area --> use the horizontal
		// space
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;

		fileTypeArea.setLayoutData(gd);
		fileTypeArea.setFont(parent.getFont());

		Label fileTypeComboLabel = new Label(fileTypeArea, SWT.LEFT);
		fileTypeComboLabel.setText("File Format: ");
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		// gd.grabExcessHorizontalSpace = true;
		fileTypeComboLabel.setLayoutData(gd);

		fileTypeCombo = new Combo(fileTypeArea, SWT.DROP_DOWN | SWT.READ_ONLY);
		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = true;

		fileTypeCombo.setLayoutData(gd);
		fileTypeCombo.add("XML GZipped", 0);
		fileTypeCombo.add("XML Uncompressed", 1);
		fileTypeCombo.add("Java Serialized", 2);
		fileTypeCombo.select(0);
		super.createAdvancedControls(parent);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#validatePage()
     */
	protected boolean validatePage() {
		return super.validatePage() && validateFilename();
	}

	/**
     * Returns the chosen file format.
     * 
     * @return Returns the chosen file format.
     */
	public int getFileFormat() {
		return this.fileFormat;
	}
}