package ch.unizh.ifi.adora.ui;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.SelectionSynchronizer;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.help.IWorkbenchHelpSystem;
import org.eclipse.ui.part.IPageSite;

import ch.unizh.ifi.adora.parts.tree.AdoraTreeEditPartFactory;

/**
 * Represents the outline pagebook for this editor.
 * 
 * @author smeier
 * @author gygax
 * 
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * 
 * @history June 30, 2007 SM Proper header comments in the class introduced.
 *          Corrected bug, the model in the treeview was not replaced properly
 *          which was causing exceptions when working on the replaced model.
 *          08.04.2008 CJ Fix bug 853 (copy/cut/paste do not work in the outline
 *          view)
 * @version $Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 * 
 */
public class AdoraModelOutlinePage extends ContentOutlinePage {

	/**
	 * Create a new outline page for the Adora editor.
	 * 
	 * @param viewer
	 *            a viewer (TreeViewer instance) used for this outline page
	 * @throws IllegalArgumentException
	 *             if editor is null
	 */

	private EditDomain domain;

	private SelectionSynchronizer synch;

	private ActionRegistry registry;

	private Object model;

	public AdoraModelOutlinePage(EditPartViewer viewer, EditDomain domain,
			SelectionSynchronizer synch, ActionRegistry registry, Object model) {

		super(viewer);

		this.domain = domain;
		this.synch = synch;
		this.registry = registry;
		this.model = model;
	}

	/**
	 * Registers the help for the outline.
	 */
	private void registerHelp() {
		// Register help
		IWorkbenchHelpSystem wh = PlatformUI.getWorkbench().getHelpSystem();
		wh.setHelp(getViewer().getControl(),
				"ch.unizh.ifi.adora.editor.OutlineViewHelp");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.IPage#createControl(org.eclipse.swt.widgets.Composite
	 * )
	 */
	public void createControl(Composite parent) {
		// create outline viewer page
		getViewer().createControl(parent);
		// configure outline viewer
		getViewer().setEditDomain(domain);
		getViewer().setEditPartFactory(new AdoraTreeEditPartFactory());
		// configure & add context menu to viewer
		ContextMenuProvider cmProvider = new AdoraTreeContextMenuProvider(
				getViewer(), registry);
		getViewer().setContextMenu(cmProvider);
		getSite().registerContextMenu("ch.unizh.ifi.adora.outline.contextmenu",
				cmProvider, getSite().getSelectionProvider());
		// hook outline viewer
		synch.addViewer(getViewer());
		// initialize outline viewer with model
		getViewer().setContents(model);
		registerHelp();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPage#dispose()
	 */
	public void dispose() {
		
		// unhook outline viewer		
		synch.removeViewer(getViewer());
	
		
		MenuManager ctxMenu = getViewer().getContextMenu();

		getViewer().getContextMenu().dispose();
		getViewer().setContextMenu(null);
		ctxMenu = null;


		domain.removeViewer(getViewer());
		
		
		domain = null;
		model = null;
		synch = null;
		registry = null;
		
		// dispose
		super.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPage#getControl()
	 */
	public Control getControl() {
		return getViewer().getControl();
	}

	/*
	 * @see
	 * org.eclipse.ui.part.IPageBookViewPage#init(org.eclipse.ui.part.IPageSite)
	 */
	public void init(IPageSite pageSite) {
		super.init(pageSite);

		/*
		 * Registers the Adora editor specific actions, ie. undo, redo, delete,
		 * copy, paste, to the globalActionHandler. CJ: Added "CUT", see bug 853
		 */
		IActionBars bars = pageSite.getActionBars();
		String id = ActionFactory.UNDO.getId();
		bars.setGlobalActionHandler(id, registry.getAction(id));
		id = ActionFactory.REDO.getId();
		bars.setGlobalActionHandler(id, registry.getAction(id));
		id = ActionFactory.DELETE.getId();
		bars.setGlobalActionHandler(id, registry.getAction(id));
		id = ActionFactory.COPY.getId();
		bars.setGlobalActionHandler(id, registry.getAction(id));
		id = ActionFactory.PASTE.getId();
		bars.setGlobalActionHandler(id, registry.getAction(id));
		id = ActionFactory.CUT.getId();
		bars.setGlobalActionHandler(id, registry.getAction(id));
	}

	public Object getModel() {
		return model;
	}

	/**
	 * This method is used to set a new model.
	 * 
	 * @param model
	 *            The new model to set.
	 */
	public void setModel(Object model) {
		this.model = model;

		// The initialization of this object has to be queried
		// otherwise a exception occurs.
		if (isInitialized()) {
			getViewer().setContents(model);
		}
	}

	/**
	 * Method returns whether the {@link #createControl(Composite)} method of
	 * this object was already called.
	 * 
	 * @return Returns true in the case that this object is initialized
	 *         properly.
	 */
	public boolean isInitialized() {
		return (getViewer() != null && getViewer().getEditPartFactory() != null);
	}

	public ActionRegistry getRegistry() {
		return registry;
	}

	public void setRegistry(ActionRegistry registry) {
		this.registry = registry;
	}

	public SelectionSynchronizer getSynch() {
		return synch;
	}

	public void setSynch(SelectionSynchronizer synch) {
		this.synch = synch;
	}

	public EditDomain getDomain() {
		return domain;
	}

	public void setDomain(EditDomain domain) {
		this.domain = domain;
	}
	
	
}