package ch.unizh.ifi.adora.ui;

import org.eclipse.gef.editparts.ZoomListener;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Slider;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IPartService;
import org.eclipse.ui.IWorkbenchPart;

/**
 * A ControlContribution that uses a {@link org.eclipse.swt.widgets.Slider} as
 * its control
 *
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             27.10.2006 SM First Version
 * @version             $Date: 2007-02-22 17:08:23 +0100 (Thu, 22 Feb 2007) $, $Author: stoiber $, $Revision: 2461 $
 *
 */
public class SliderContributionItem extends ContributionItem implements
		ZoomListener {

	/**
	 * The width of the slider.
	 */
	private static final int SLIDER_WIDTH = 30;

	/**
	 * The height of the slider.
	 */
	private static final int SLIDER_HEIGHT = 20;

	/**
	 * The slider widget representing the zoom factor.
	 */
	private Slider slider;

	/**
	 * The label indicating the zoom factor in percents.
	 */
	private Label zoomPercentage;

	/**
	 * The container containing the zoom factor label and the slider.
	 */
	private Composite container;

	/**
	 * The tool item.
	 */
	private ToolItem toolitem;

	/**
	 * The zoom manager to which this slider contribution belongs to.
	 */
	private ZoomManager zoomManager;

	private IPartService service;

	private IPartListener partListener;

	/**
     * Constructor for ComboToolItem.
     *
     * @param partService
     *            used to add a PartListener
     * @param initStrings
     *            the initial string displayed in the combo
     */
	public SliderContributionItem(IPartService partService) {
		super(GEFActionConstants.ZOOM_TOOLBAR_WIDGET);

		service = partService;
		Assert.isNotNull(partService);
		partService.addPartListener(partListener = new IPartListener() {
			public void partActivated(IWorkbenchPart part) {
				setZoomManager((ZoomManager) part.getAdapter(ZoomManager.class));
			}

			public void partBroughtToTop(IWorkbenchPart p) {
			}

			public void partClosed(IWorkbenchPart p) {
			}

			public void partDeactivated(IWorkbenchPart p) {
			}

			public void partOpened(IWorkbenchPart p) {
			}
		});
	}

	/**
	 * Refreshes the slider state.
	 * 
	 * @param refreshSlider
	 */
	void refresh(boolean refreshSlider) {
		if (slider == null || slider.isDisposed())
			return;

		try {
			if (zoomManager == null) {
				slider.setEnabled(false);
				zoomPercentage.setText(""); //$NON-NLS-1$
			} else {
				this.zoomPercentage.setText("" + Math.round(zoomManager.getZoom() * 100) + "%");
				if (refreshSlider) {
					slider.setEnabled(true);
					//the slider state also needs a rounded integer to be reset to the specified value.
					slider.setSelection((int) Math.round(zoomManager.getZoom() * 100));
				}
			}
		} catch (SWTException exception) {

		}
	}

	/** 
	 * Initializes the slider widget which visualizes the zoom factor.
	 */
	public void initSlider() {
		if (zoomManager != null) {

			slider.setMinimum((int) (zoomManager.getMinZoom() * 100));
			slider.setMaximum((int) (zoomManager.getMaxZoom() * 100));

			slider.setIncrement(1);
			slider.setSelection((int) zoomManager.getZoom() * 100);
			slider.setEnabled(true);
		}
	}

	/**
     * Creates and returns the control for this contribution item under the
     * given parent composite.
     *
     * @param parent
     *            the parent composite
     * @return the new control
     */
	protected Control createControl(Composite parent) {
		container = new Composite(parent, SWT.NONE);

		GridLayout gl = new GridLayout();
		gl.numColumns = 2;
		container.setLayout(gl);

		zoomPercentage = new Label(container, SWT.LEFT);
		zoomPercentage.setText("100%");

		slider = new Slider(container, SWT.HORIZONTAL);
		slider.setSize(SLIDER_WIDTH, SLIDER_HEIGHT);
		slider.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				handleWidgetSelected(e);
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				handleWidgetDefaultSelected(e);
			}
		});

		slider.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				// do nothing
			}

			public void focusLost(FocusEvent e) {
				refresh(true);
			}
		});
		int width = container.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).x;

		toolitem.setWidth(width);
		initSlider();
		return container;
	}

	/**
     * @see org.eclipse.jface.action.ContributionItem#dispose()
     */
	public void dispose() {
		if (partListener == null)
			return;
		service.removePartListener(partListener);
		if (zoomManager != null) {
			zoomManager.removeZoomListener(this);
			zoomManager = null;
		}
		slider = null;
		zoomPercentage = null;
		container = null;
		partListener = null;
	}

	/**
     * The control item implementation of this <code>IContributionItem</code>
     * method calls the <code>createControl</code> framework method.
     * Subclasses must implement <code>createControl</code> rather than
     * overriding this method.
     *
     * @param parent
     *            The parent of the control to fill
     */
	public final void fill(Composite parent) {
		createControl(parent);
	}

	/**
     * The control item implementation of this <code>IContributionItem</code>
     * method throws an exception since controls cannot be added to menus.
     *
     * @param parent
     *            The menu
     * @param index
     *            Menu index
     */
	public final void fill(Menu parent, int index) {
		Assert.isTrue(false, "Can't add a control to a menu");//$NON-NLS-1$
	}

	/**
     * The control item implementation of this <code>IContributionItem</code>
     * method calls the <code>createControl</code> framework method to create
     * a control under the given parent, and then creates a new tool item to
     * hold it. Subclasses must implement <code>createControl</code> rather
     * than overriding this method.
     *
     * @param parent
     *            The ToolBar to add the new control to
     * @param index
     *            Index
     */
	public void fill(ToolBar parent, int index) {
		toolitem = new ToolItem(parent, SWT.SEPARATOR, index);
		Control control = createControl(parent);
		toolitem.setControl(control);
	}

	/**
     * Returns the zoomManager.
     *
     * @return ZoomManager
     */
	public ZoomManager getZoomManager() {
		return zoomManager;
	}

	/**
     * Sets the ZoomManager and adjustes the slider to the appropriate state 
     * selected before, or to 100 percent as default.
     *
     * @param zm
     *            The ZoomManager
     */
	public void setZoomManager(ZoomManager zm) {
		if (zoomManager == zm)
			return;
		if (zoomManager != null)
			zoomManager.removeZoomListener(this);

		zoomManager = zm;
		refresh(true);
		
		if (zoomManager != null)
			zoomManager.addZoomListener(this);
	}

	/**
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(SelectionEvent)
     */
	private void handleWidgetDefaultSelected(SelectionEvent event) {
		if (zoomManager != null) {
			zoomManager.setZoom((double)slider.getSelection() / 100.0);
		}
		refresh(false);
	}

	/**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(SelectionEvent)
     */
	private void handleWidgetSelected(SelectionEvent event) {
		handleWidgetDefaultSelected(event);
	}

	/**
     * @see ZoomListener#zoomChanged(double)
     */
	public void zoomChanged(double zoom) {
		refresh(false);
	}
}
