package ch.unizh.ifi.adora.ui.preferences;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.help.IWorkbenchHelpSystem;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * This class creates the preferences page for the choosing 
 * the router parameter.
 * 
 * @author              Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             Feb 13, 2006 TR First Version
 *                      May 20, 2006 SM Extended for join relationship router
 * @version             $Date: 2009-12-01 09:57:52 +0100 (Tue, 01 Dec 2009) $, $Author: treinhard $, $Revision: 4970 $
 * @responsibilities    Preference page of the ADORA plugin
 *
 */
public class RoutingPreferencePage extends PreferencePage implements
		IWorkbenchPreferencePage {
    
    private StringFieldEditor nodeCostEditor;
    
    private StringFieldEditor abstractNodeCostEditor;
	
	private StringFieldEditor lineCostEditor;
	
	private BooleanFieldEditor showTileBool;
	
	private BooleanFieldEditor colorTileBool;
	
	private BooleanFieldEditor printBool;
	
	private BooleanFieldEditor placeLabels;
	
	private BooleanFieldEditor showLabelPositions;
    
    private Combo assoCombo, transCombo, scenarioCombo, joinRelCombo;
    
    private List<String> encodedRoutingKeys;
	 
   	/**
	 * Constructor for the preference page.
	 */
	public RoutingPreferencePage() {
		setPreferenceStore(AdoraPlugin.getDefault().getPreferenceStore());
		
		encodedRoutingKeys = new ArrayList<String>();
		
		encodedRoutingKeys.add(IPreferenceValueConstants.ADORA_REPRESENTATION_ROUTER_SHORTESTPATH);
		encodedRoutingKeys.add(IPreferenceValueConstants.ADORA_REPRESENTATION_ROUTER_DIRECTLINE);
		encodedRoutingKeys.add(IPreferenceValueConstants.ADORA_REPRESENTATION_ROUTER_NEWORTHOGONAL);
	}
	
	/**
     * Registers the editor canvas and its palette (the editor) at the help.
     */
    private void registerHelp() {
        // Register help
        IWorkbenchHelpSystem wh = PlatformUI.getWorkbench().getHelpSystem();
        wh.setHelp(getControl(), "ch.unizh.ifi.adora.editor.AdoraRoutingPreferencePage");   
    }
	
    protected void checkState() {
		if(!isValid()) {
			return;
		}
		try {
			Float.parseFloat(nodeCostEditor.getStringValue());
			Float.parseFloat(abstractNodeCostEditor.getStringValue());
			Float.parseFloat(lineCostEditor.getStringValue());
		} catch (NumberFormatException e) {
			setErrorMessage("Cost factors must be of type float");
			setValid(false);
			return;
		}
		
		setErrorMessage(null);
		setValid(true);
	}
	
	public void propertyChange(PropertyChangeEvent evt) {
		if(evt.getProperty().equals(FieldEditor.VALUE)) {
			if(evt.getSource() == nodeCostEditor
					|| evt.getSource() == lineCostEditor
					|| evt.getSource() == abstractNodeCostEditor) {
				checkState();
			}
		}
	}
	
	@Override
	protected Control createContents(Composite parent) {
		
		Composite comp = new Composite(parent, SWT.NONE);
		comp.setLayout(new GridLayout(1, true));
    
	    GridData gd;
		gd = new GridData(GridData.FILL_BOTH);
		comp.setLayoutData(gd);
	    
	    
		Group routersGroup = new Group(comp, SWT.NONE);
		routersGroup.setText("Line routing style");
		routersGroup.setLayout(new GridLayout(2, false));
	    
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		routersGroup.setLayoutData(gd);
	    
	    assoCombo = createRoutingContent(routersGroup, "Association:", 
	    		IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_ASSOCIATION);
	    transCombo = createRoutingContent(routersGroup, "Transition:", 
	    		IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_TRANSITION);
	    scenarioCombo= createRoutingContent(routersGroup, "Scenario Connection:", 
	    		IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_SCENARIOCONNECTION);
	    joinRelCombo = createRoutingContent(routersGroup, "Join Relationship:", 
	    		IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_JOIN_RELATIONSHIP);
		
	 	IPreferenceStore ps = AdoraPlugin.getDefault().getPreferenceStore();
	 	
	 	
	 	Group debugGroup = new Group(comp, SWT.NONE);
	 	debugGroup.setText("Debug informations");
	 	
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		debugGroup.setLayoutData(gd);
	 	
	 	showTileBool = new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES,
					"&Show tile structure",
					debugGroup);
	 	showTileBool.setPreferenceStore(ps);
	 	showTileBool.load();
	 	
	 	colorTileBool = new BooleanFieldEditor(
				IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_COLOR_TILES,
				"&Color tile structure",
				debugGroup);
	 	colorTileBool.setPreferenceStore(ps);
	 	colorTileBool.load();

	 	printBool = new BooleanFieldEditor(
	 					IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PRINT_SOLUTIONS,
						"&Print solutions",
						debugGroup);
	 	printBool.setPreferenceStore(ps);
	 	printBool.load();
	 	
	 	placeLabels = new BooleanFieldEditor(
				IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PLACE_LABELS,
			"Place line &labels automatically (EXPERIMENTAL)",
			debugGroup);
	 	placeLabels.setPreferenceStore(ps);
	 	placeLabels.load();
	 	
	 	showLabelPositions = new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_LABEL_POSITIONS,
				"Show label &positions",
				debugGroup);
	 	showLabelPositions.setPreferenceStore(ps);
	 	showLabelPositions.load();
	 	
	 	Group costComp = new Group(comp, SWT.NONE);
	 	costComp.setText("Cost functions");
	 	
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		costComp.setLayoutData(gd);
		
		nodeCostEditor = new StringFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_NODE_COSTFACTOR,
					"Node cost factor:",
					costComp);
		nodeCostEditor.setPreferenceStore(ps);
		nodeCostEditor.load();
	 	
		abstractNodeCostEditor = new StringFieldEditor(
				IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_ABSTRACT_NODE_COSTFACTOR,
				"Abstract node cost factor:",
				costComp);
		abstractNodeCostEditor.setPreferenceStore(ps);
		abstractNodeCostEditor.load();
		
		lineCostEditor = new StringFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_LINE_COSTFACTOR,
					"Line cost factor:",
					costComp);
		lineCostEditor.setPreferenceStore(ps);
		lineCostEditor.load();
	
		registerHelp();
		
		return comp;
	}
	
	/**
	 * Instantiates Combo, which contain the routing algorithms. 
	 * 
	 * @param parent	Parent 
	 * @param connectionType Type of connection 
	 * @param preferenceName
	 * @return Combo, which got instantiated.
	 */
	protected Combo createRoutingContent(Composite parent,
			String connectionType, 
			String preferenceName) {
		
		Label lbl = new Label(parent, SWT.NONE);
		lbl.setText(connectionType);
		
		GridData gd;
		gd = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		gd.horizontalSpan = 1;
		lbl.setLayoutData(gd);
		
		Combo combo = new Combo(parent, SWT.READ_ONLY);
		combo.setItems(
				new String[]   {"Shortest path router", 
								"Direct line router",
								"Orthogonal router"}
				);
		
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		combo.setLayoutData(gd);
	
		IPreferenceStore ps = AdoraPlugin.getDefault().getPreferenceStore();
		String routingType = ps.getString(preferenceName);
	
		combo.select(encodedRoutingKeys.indexOf(routingType));
		
		return combo;
	}
	
    /*
     *  (non-Javadoc)
     * @see org.eclipse.jface.preference.IPreferencePage#performOk()
     */
    public boolean performOk() {
    	IPreferenceStore ps = AdoraPlugin.getDefault().getPreferenceStore();

    	ps.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_ASSOCIATION, 
    			encodedRoutingKeys.get(assoCombo.getSelectionIndex()));
    	ps.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_TRANSITION, 
    			encodedRoutingKeys.get(transCombo.getSelectionIndex()));
    	ps.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_SCENARIOCONNECTION, 
    			encodedRoutingKeys.get(scenarioCombo.getSelectionIndex()));
    	ps.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_JOIN_RELATIONSHIP, 
    			encodedRoutingKeys.get(joinRelCombo.getSelectionIndex()));
    
     	nodeCostEditor.store();
     	abstractNodeCostEditor.store();
        lineCostEditor.store();
        showTileBool.store();
        colorTileBool.store();
        printBool.store();
        placeLabels.store();
        showLabelPositions.store();

    	return true;
    }
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.jface.preference.PreferencePage#performApply()
     */
    protected void performApply() {
        performOk();
    }

    /**
     * Performs special processing when this page's Defaults button has been pressed.
     * <p>
     * This is a framework hook method for subclasses to do special things when
     * the Defaults button has been pressed.
     * Subclasses may override, but should call <code>super.performDefaults</code>.
     * </p>
     */
    protected void performDefaults() {
    
    	IPreferenceStore ps = AdoraPlugin.getDefault().getPreferenceStore();
		
		assoCombo.select(encodedRoutingKeys.indexOf(ps.getDefaultString(IPreferenceNameConstants.
				ADORA_REPRESENTATION_ROUTER_ASSOCIATION)));
		
		transCombo.select(encodedRoutingKeys.indexOf(ps.getDefaultString(IPreferenceNameConstants.
				ADORA_REPRESENTATION_ROUTER_TRANSITION)));
		
		scenarioCombo.select(encodedRoutingKeys.indexOf(ps.getDefaultString(IPreferenceNameConstants.
				ADORA_REPRESENTATION_ROUTER_SCENARIOCONNECTION)));
		
		joinRelCombo.select(encodedRoutingKeys.indexOf(ps.getDefaultString(IPreferenceNameConstants.
				ADORA_REPRESENTATION_ROUTER_JOIN_RELATIONSHIP)));
		
     	nodeCostEditor.loadDefault();
     	abstractNodeCostEditor.loadDefault();
        lineCostEditor.loadDefault();
        showTileBool.loadDefault();
        colorTileBool.loadDefault();
        printBool.loadDefault();
        placeLabels.loadDefault();
        showLabelPositions.loadDefault();
    }

    /*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}
}