package ch.unizh.ifi.adora.ui.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.Observable;

import org.eclipse.gef.RootEditPart;
import org.eclipse.ui.IPropertyListener;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.parts.AdoraRootEditPart;
import ch.unizh.ifi.adora.properties.ZoomableNodeContainerProperties;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
//import ch.unizh.ifi.adora.ui.views.decisiontable.DecisionTableViewPart;

/**
 * This is a mediator between the {@link DecisionTableViewPart} and the editor /
 * model. A class can register itself on this mediator as an observer and gets
 * every propertyChange from the model.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-03-05 PS First Version
 * @version $Date: 2010-02-15 01:59:36 +0100 (Mon, 15 Feb 2010) $, $Author: vigan $, $Revision: 5136 $
 */
public class ModelObserverMediator extends Observable implements
		PropertyChangeListener, IPropertyListener {
	/**
     * This string indicated, that the model has changed in the editor.
     */
	public static String MODEL_CHANGED = "model.changed";

	/**
     * The instance (singleton).
     */
	private static ModelObserverMediator instance;

	/**
     * Returns the instance of this class.
     * 
     * @return The single instance of this class.
     */
	public static ModelObserverMediator getInstance() {
		if (instance == null) {
			instance = new ModelObserverMediator();
		}
		
		return instance;
	}
	
	
	/**
     * The current {@link AdoraModelEditor}.
     */
	private AdoraModelEditor observedAdoraModelEditor;

	/**
     * The constructor (private because of the singleton).
     */
	private ModelObserverMediator() {
	}

	/**
     * Adds all listeners to the {@link AdoraModel}.
     * 
     * @param model
     *            The {@link AdoraModel}.
     */
	private void addListeners(AdoraModel model) {
		model.addPropertyChangeListener(this);

		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();
		while (iterator.hasNext()) {
			addListener(iterator.next());
		}
	}

	/**
     * Adds all listeners on a {@link ModelElement} and its outgoing
     * connections.
     * 
     * @param modelElement
     *            The {@link ModelElement} to observe.
     */
	private void addListener(ModelElement modelElement) {
		if (!(modelElement instanceof ObservableModelElement)) {
			return;
		}

		ObservableModelElement observable = (ObservableModelElement) modelElement;
		observable.addPropertyChangeListener(this);

		if (observable instanceof ConnectableElement) {
			ConnectableElement connectable = (ConnectableElement) observable;
			for (Connection connection : connectable.getSourceConnections()) {
				connection.addPropertyChangeListener(this);
			}
		}
	}

	/**
     * Removes all listeners from the {@link AdoraModel}.
     * 
     * @param model
     */
	private void removeListeners(AdoraModel model) {
		if (model == null)
			return;
		
		model.removePropertyChangeListener(this);

		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();
		while (iterator.hasNext()) {
			removeListener(iterator.next());
		}
	}

	/**
     * Removes a listener from a specific {@link ModelElement}.
     * 
     * @param modelElement
     *            The {@link ModelElement} where the listeners should be
     *            deactivated.
     */
	private void removeListener(ModelElement modelElement) {
		if (!(modelElement instanceof ObservableModelElement)) {
			return;
		}

		ObservableModelElement observable = (ObservableModelElement) modelElement;
		observable.removePropertyChangeListener(this);

		if (observable instanceof ConnectableElement) {
			ConnectableElement connectable = (ConnectableElement) observable;
			for (Connection connection : connectable.getSourceConnections()) {
				connection.removePropertyChangeListener(this);
			}
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
	public void propertyChange(PropertyChangeEvent arg0) {
		setChanged();

		if (ZoomableNodeContainerProperties.DELETE_ELEMENT_PROP.equals(arg0
				.getPropertyName())) {
			removeListener((ModelElement) arg0.getNewValue());
		}

		if (INodeContainer.CHILD_ADDED_PROP.equals(arg0.getPropertyName())) {
			addListener((ModelElement) arg0.getNewValue());
		}
		
		if (ConnectableElementPropNames.SOURCECONNECTION_NEW_PROP.equals(arg0.getPropertyName())) {
			addListener((ModelElement) arg0.getNewValue());
		}

		notifyObservers(arg0);
	}

	/**
     * Returns the {@link AdoraModelEditor}.
     * 
     * @return The current {@link AdoraModelEditor}.
     */
	public AdoraModelEditor getAdoraModelEditor() {
		return observedAdoraModelEditor;
	}

	/**
     * Sets the {@link AdoraModelEditor}. There is a check, if the editor has
     * changed. if yes, it removes all listeners from the old editor / model and
     * adds listeners to the new editor and its model.
     * 
     * @param adoraModelEditor
     *            The current {@link AdoraModelEditor}.
     */
	public void setAdoraModelEditor(AdoraModelEditor adoraModelEditor) {
		if (this.observedAdoraModelEditor == adoraModelEditor) {
			return;
		}

		setChanged();

		if (this.observedAdoraModelEditor != null) {
			this.observedAdoraModelEditor.removePropertyListener(this);
			removeListeners(this.observedAdoraModelEditor.getModel());
		}

		this.observedAdoraModelEditor = adoraModelEditor;
		addListeners(this.observedAdoraModelEditor.getModel());
		adoraModelEditor.addPropertyListener(this);

		notifyObservers(MODEL_CHANGED);
	}

	/**
     * Implemented to listen, if the editor reloads the model (for example for
     * the weaving).
     * 
     * @param source
     * @param propId
     */
	public void propertyChanged(Object source, int propId) {
		setChanged();

		if (propId == AdoraModelEditor.PROP_MODEL_CHANGE) {
			removeListeners(observedAdoraModelEditor.getModel());
			addListeners(observedAdoraModelEditor.getModel());

			notifyObservers(MODEL_CHANGED);
		}
	}
	
	/**
	 * Refreshes the model view of the model.
	 */
	public static void modelRefresher() {
		RootEditPart rep = AdoraPlugin.getActiveEditor().getViewer()
		.getRootEditPart();

		AdoraRootEditPart arep = (AdoraRootEditPart) rep;
		arep.getZoomManager();
		
		arep.getZoomManager().setZoom(arep.getZoomManager().getZoom() + 0.000001);
		
	}
}