package ch.unizh.ifi.adora.ui.views.explosiveZoom;

import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;


/**
 * This class handles the explosive zoom function.
 * It opens a new editor tab and shows the content of the zoomed
 * element inside it.
 * 
 * The content of the zoomed element is temporarily saved in an
 * adora file. The temporary file is removed as soon as the
 * corresponding editor tab is closed.
 * 
 * @author Michael Jehle
 */
public class ExplosiveZoomHandler
{
	/**
	 * Counter for the temporary files (is used in their file names to avoid
	 * duplicated filenames).
	 */
	public static int EXPLOSIVE_ZOOMS_OPEN_COUNTER = 0;
	
	/**
	 * Basic file name for the temporary files.
	 */
	public static final String EXPLOSIVE_ZOOM_TEMP_FILENAME = "TemporaryFileForExplosiveZoom";
	
	/**
	 * The model element on which the explosive zoom was called.
	 */
	private ZoomableNodeContainer zoomTarget;
	
	/**
	 * List of all the files created by the explosive zoom.
	 * They are remembered, to be able to delete them, when 
	 * the corresponding editor is closed.
	 * 
	 * @key: Integer, the hashCode of the editor, the file is opened in.
	 * @value: IFile, the files opened by the explosive zoom.
	 */
	private HashMap<Integer, IFile> openedEditors = new HashMap<Integer, IFile>();

	/**
	 * Constructor. Takes the model element on which the explosive zoom
	 * was called as an argument.
	 * 
	 * @param zoomTarget ZoomableNodeContainer
	 */
	public ExplosiveZoomHandler(ZoomableNodeContainer zoomTarget)
	{
		this.zoomTarget = zoomTarget;
	}

	/**
	 * Creates the content of the new editor tab. This is basically the
	 * content of the zoomed model element.
	 * 
	 * @return AdoraModel
	 */
	private AdoraModel createModel()
	{
		//first, find the original adora model by going
		//upwards the model element hierarchy
		IContainer lookAt = this.zoomTarget.getParent();
		
		while (!(lookAt instanceof AdoraModel))
		{
			lookAt = lookAt.getParent();
		}
		
		AdoraModel theSourceModel = null;
		
		try
		{
			theSourceModel = (AdoraModel) lookAt;
		}
		catch (ClassCastException e)
		{
			System.err.println("AdoraModel of the zoom target not found!");
			e.printStackTrace();
		}

		//clone the model
		AdoraModel modelElementSource = theSourceModel.modelCopy();
		AdoraModel newModel = new AdoraModel();
		
		//iterate down the model until the zoom target is found
		Iterator<ModelElement> childIterator = modelElementSource.getRecursiveChildrenIterator();
		
		// search the zoom target in the modelElementSource model.
		while (childIterator.hasNext())
		{
			ModelElement child = childIterator.next();
			
			// equals method fails (not the same instance of the model element),
			// therefore the lazy manual implementation here
			if (child instanceof ZoomableNodeContainer)
			{
				if (((ZoomableNodeContainer) child).getName().equals(this.zoomTarget.getName()) &&
					((ZoomableNodeContainer) child).getBounds().equals(this.zoomTarget.getBounds()) &&
					((ZoomableNodeContainer) child).getAbsoluteLocation().equals(this.zoomTarget.getAbsoluteLocation()) &&
					((ZoomableNodeContainer) child).getBorderDimension().equals(this.zoomTarget.getBorderDimension()) &&
					((ZoomableNodeContainer) child).getBorderInsets().equals(this.zoomTarget.getBorderInsets()) &&
					((ZoomableNodeContainer) child).getChildCount() == this.zoomTarget.getChildCount() &&
					((ZoomableNodeContainer) child).getComment().equals(this.zoomTarget.getComment()) &&
					((ZoomableNodeContainer) child).getConnectionsCount() == this.zoomTarget.getConnectionsCount() &&
					((ZoomableNodeContainer) child).getLocation().equals(this.zoomTarget.getLocation()))
				{
					this.zoomTarget = (ZoomableNodeContainer) child;
				}
			}
		}

		Iterator<ModelElement> addChildrenIterator = this.zoomTarget.getChildrenIterator();

		while (addChildrenIterator.hasNext())
		{
			ModelElement temp = addChildrenIterator.next();
			newModel.addChild(temp);
			
			if (temp instanceof VisualizableModelElement)
			{
				((VisualizableModelElement) temp).setVisible(true);
			}
		}
		
		return newModel;
	}

	/**
	 * This method creates a temporary adora file which stores
	 * the content of the new editor tab (the content of the
	 * zoomed model element).
	 * 
	 * @return IFile
	 */
	private IFile makeFileForZoomInput()
	{
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		IWorkbenchPage page = window.getActivePage();

		IFile theFile = null;
		
		FileEditorInput theEditorInput = (FileEditorInput) page.getActiveEditor().getEditorInput();
		theFile = theEditorInput.getFile();
		
		String newFileName = ExplosiveZoomHandler.EXPLOSIVE_ZOOM_TEMP_FILENAME + ExplosiveZoomHandler.EXPLOSIVE_ZOOMS_OPEN_COUNTER + ".adora";
		
		String theFilesPath = theFile.getFullPath().toString();
		theFilesPath = theFilesPath.substring(0, theFilesPath.lastIndexOf(IPath.SEPARATOR));
		
		IPath theNewPath = new Path(theFilesPath + IPath.SEPARATOR + newFileName);
		
		ExplosiveZoomHandler.EXPLOSIVE_ZOOMS_OPEN_COUNTER++;
		
		try
		{
			theFile.copy(theNewPath, false, null);
		}
		catch (CoreException e1)
		{
			e1.printStackTrace();
		}
		
		// This string is used to find the copied file
		String theNewPathString = theNewPath.toString();
		
		if (theNewPathString.charAt(0) == IPath.SEPARATOR)
		{
			//if the path begins with a /, remove it
			theNewPathString = theNewPathString.substring(1);
		}
		
		// remove the project name from the path string, since we already know where to search.
		theNewPathString = theNewPathString.substring(theNewPathString.indexOf(IPath.SEPARATOR));

		//get the temporary file reference from the workspace.
		IFile theNewFile = theFile.getProject().getFile(new Path(theNewPathString));

		if (!theNewFile.exists())
		{
			try
			{
				theNewFile.refreshLocal(IResource.DEPTH_INFINITE, null);
			}
			catch (CoreException e)
			{
				e.printStackTrace();
			}
			
			if (!theNewFile.exists())
			{
				System.err.println("The temporary file was not created properly.");
			}
		}

		return theNewFile;
	}

	/**
	 * Opens a new Adora-Editor-Tab with the content of the model element
	 * on which the explosive zoom was called.
	 */
	public void openEditor()
	{
		IFile theNewFile = this.makeFileForZoomInput();
		
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		IWorkbenchPage page = window.getActivePage();
		
		AdoraModelEditor theOpenedEditor = null;
		
		try
		{
			theOpenedEditor = (AdoraModelEditor) page.openEditor(new FileEditorInput(theNewFile), "Adora Model Editor", true);
		}
		catch (PartInitException e)
		{
			e.printStackTrace();
		}

		AdoraModel theZoomedInModel = this.createModel();
		
		theOpenedEditor.setModel(theZoomedInModel);
		
		// register the opened editor and the file displayed in it
		// to be able to delete the temporary file, when the tab is closed
		this.openedEditors.put(theOpenedEditor.hashCode(), theNewFile);
		
		page.addPartListener(new IPartListener()
		{
			public void partActivated(IWorkbenchPart part)
			{
				// do nothing
			}

			public void partBroughtToTop(IWorkbenchPart part)
			{
				// do nothing	
			}

			/**
			 * If one of the explosive zoom editor tabs is closed
			 * the corresponding temporary file needs to be deleted.
			 */
			public void partClosed(IWorkbenchPart part)
			{
				if (openedEditors.containsKey(part.hashCode()))
				{
					IFile theFileToDelete = openedEditors.get(part.hashCode());

					if (!theFileToDelete.isSynchronized(IResource.DEPTH_INFINITE))
					{
						try
						{
							theFileToDelete.refreshLocal(IResource.DEPTH_INFINITE, null);
						}
						catch (CoreException e)
						{
							e.printStackTrace();
						}
					}
					
					try
					{
						theFileToDelete.delete(true, null);
					}
					catch (CoreException e)
					{
 						e.printStackTrace();
					}

					openedEditors.remove(part.hashCode());
				}
			}

			public void partDeactivated(IWorkbenchPart part)
			{
				// do nothing
			}

			public void partOpened(IWorkbenchPart part)
			{
				// do nothing		
			}
		});
	}	
}