package ch.unizh.ifi.adora.ui.views.modelsearch;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.ReduceModelVisibility;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class implements a view part which allows to search the
 * model elements of an adora model.
 * When a search is performed, all the model elements which don't belong
 * to the result are set invisible.
 * 
 * @author Michael Jehle
 * @date 26.01.2009
 */
public class ModelSearchView extends ViewPart implements IPartListener
{
	private Text searchFor;
	private Table found;
	private Button reset;
	
	private TableViewer tableViewer;
	private String[] columnNames = new String[] {"Model Elements found", "Type"};
	private int[] columnWidths = new int[] {200, 200};
	private int[] columnWeights = new int[] {30, 70};
	
	private static final String MODEL_ELEMENT_FOUND_PROPERTY = "MODEL_ELEMENT_FOUND";
	private static final String MODEL_ELEMENT_FOUND_TYPE_PROPERTY = "MODEL_ELEMENT_FOUND_TYPE";
	
	private Stack<ReduceModelVisibility> reduceVisibilityCommands = new Stack<ReduceModelVisibility>();
	private String lastSearchText;
	
	public ModelSearchView()
	{
		super();
	}

	/**
	 * This method is called by eclipse, when the view
	 * is activated. It builds up the GUI components.
	 */
	@Override
	public void createPartControl(Composite parent)
	{
		GridLayout viewLayout = new GridLayout();
		viewLayout.numColumns = 2;
		viewLayout.horizontalSpacing = 1;
		viewLayout.verticalSpacing = 1;
		viewLayout.makeColumnsEqualWidth = false;

		parent.setLayout(viewLayout);

		this.createSearchField(parent);
		this.createResetButton(parent);
		this.createFoundTable(parent);

		parent.pack();
		
		this.getSite().getPage().addPartListener(this);
	}
	
	/**
	 * Initializes the reset button which can be used to
	 * reset the model visibility.
	 * 
	 * @param parent Composite
	 */
	private void createResetButton(Composite parent)
	{
		this.reset = new Button(parent, SWT.PUSH);
		this.reset.setText("Reset");
		
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		this.reset.setLayoutData(gridData);
		
		this.reset.addMouseListener(new MouseListener()
		{
			public void mouseDoubleClick(MouseEvent e)
			{
			}

			public void mouseDown(MouseEvent e)
			{
			}

			public void mouseUp(MouseEvent e)
			{
				searchFor.setText("");
				clearSearch();
			}
		});
	}

	/**
	 * Initials the table, which displays the search results.
	 * @param parent Composite
	 */
	private void createFoundTable(Composite parent)
	{
		int style = SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION;
		
		this.found = new Table(parent, style);
		this.found.setHeaderVisible(true);
		this.found.setLinesVisible(true);
		
		GridData gridData = new GridData(GridData.FILL_BOTH);
		
		gridData.horizontalSpan = 2;
		
		this.found.setLayoutData(gridData);
				
		TableLayout layout = new TableLayout();
		
		for (int c = 0; c < this.columnNames.length; c++)
		{
		    layout.addColumnData(new ColumnWeightData(this.columnWeights[c], this.columnWidths[c], true));
			
			TableColumn modelElementcolumn = new TableColumn(this.found, SWT.LEFT, c);
			modelElementcolumn.setText(this.columnNames[c]);
			modelElementcolumn.setWidth(this.columnWidths[c]);
			modelElementcolumn.setResizable(true);
		}
		
	    this.found.setLayout(layout);

		tableViewer = new TableViewer(this.found);
        tableViewer.setUseHashlookup(true);
        tableViewer.setColumnProperties(new String[] {ModelSearchView.MODEL_ELEMENT_FOUND_PROPERTY, ModelSearchView.MODEL_ELEMENT_FOUND_TYPE_PROPERTY});
        
        this.attachCellModifier();
        this.attachLabelProvider();
	}
	
	/**
	 * Removes all the entries in the table, which displays the
	 * elements found by a previous model search.
	 */
	private void clearTable()
	{
		int t = 0;
		
		while (tableViewer.getElementAt(t) != null)
		{
			tableViewer.remove(this.tableViewer.getElementAt(t));
		}
	}

	/**
	 * This method is called, every time the search results have
	 * to be updated. It initializes the command which reduces
	 * the visibility of the model elements.
	 */
	private void updateSearch()
	{
		this.clearTable();
		
		ArrayList<VisualizableModelElement> result = (ArrayList<VisualizableModelElement>) searchModelElement(searchFor.getText());

		this.updateTable(result);
		
		AdoraModelEditor activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
		AdoraModel model = activeAdoraEditor.getModel();
		
		ReduceModelVisibility reduceModelVisibilityCommand = new ReduceModelVisibility();
		
		if (this.reduceVisibilityCommands.size() > 0)
		{
			reduceModelVisibilityCommand.setSearchResultBeforeCommand(this.reduceVisibilityCommands.peek().getSearchResult());
		}
		else
		{
			reduceModelVisibilityCommand.setSearchResultBeforeCommand(new ArrayList<VisualizableModelElement>());
		}
		
		reduceModelVisibilityCommand.setSearchResult(result);
		reduceModelVisibilityCommand.setModel(model);
		reduceModelVisibilityCommand.setPartTitle(activeAdoraEditor.getTitle());
		reduceModelVisibilityCommand.execute();
		
//		System.out.println("Command executed and pushed: " + reduceModelVisibilityCommand);
		this.reduceVisibilityCommands.push(reduceModelVisibilityCommand);
	}
	
	private void updateTable(List<VisualizableModelElement> result)
	{
		for (int f = 0; f < result.size(); f++)
		{
			tableViewer.add(new EditableTableItem(result.get(f).getName(),
						                          result.get(f).getClass().getSimpleName()));
		}
	}
	
	/**
	 * This method sets up the text field where the user can enter
	 * the search text.
	 * 
	 * @param parent Composite
	 */
	private void createSearchField(Composite parent)
	{
		this.searchFor = new Text(parent, SWT.BORDER);
		this.searchFor.setSize(200, 20);
		
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		
		this.searchFor.setLayoutData(gridData);
		
		this.searchFor.addKeyListener(new KeyListener()
		{
			public void keyPressed(KeyEvent e)
			{

			}

			public void keyReleased(KeyEvent e)
			{
				// Ignore 65536: is some kind of event, that comes with every key
				// Ignore 131072: shift key
				if (e.keyCode != 65536 && e.keyCode != 131072)
				{
					// only if the content of the text field changed, actions are triggered.
					if (!searchFor.getText().equals(lastSearchText))
					{
    					// Key code 8 is the backspace
    					if (e.keyCode == 8)
    					{
    						clearTable();
    						
    						if (reduceVisibilityCommands.size() > 0)
    						{
    							updateTable(reduceVisibilityCommands.peek().getSearchResultBeforeCommand());
    						
    							reduceVisibilityCommands.pop().undo();
    							lastSearchText = searchFor.getText();
    						}
       					}
    					else
    					{
    						if (searchFor.getText().length() > 0)
    						{
    							updateSearch();
    							lastSearchText = searchFor.getText();
    						}
    						else
    						{
    							if (!reduceVisibilityCommands.isEmpty())
    							{
    								IllegalStateException exception = new IllegalStateException("All commands should be undone and the stack should be empty!");
    								throw exception;
    							}
    						}
    					}
					}
				}
//				System.out.println("Key pressed, stack size: " + reduceVisibilityCommands.size());
			}
		});
	}

	@Override
	public void setFocus()
	{
		
	}

	/**
	 * Use this method to search the model. It compares the names of the model elements
	 * with the search-string and adds the model elements which match this string to
	 * the list which is returned.
	 * 
	 * @param name String to search for.
	 * @return List<VisualizableModelElement>
	 */
	private List<VisualizableModelElement> searchModelElement(String name)
	{
		ArrayList<VisualizableModelElement> result = new ArrayList<VisualizableModelElement>();
		
		AdoraModelEditor activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
		AdoraModel model = activeAdoraEditor.getModel();
		
		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();
		
		while (iterator.hasNext())
		{
			ModelElement temp = iterator.next();
			
			if (temp instanceof VisualizableModelElement)
			{
				int substrEnd = java.lang.Math.min(name.length(), ((VisualizableModelElement) temp).getName().length());
				
				String compareTo = ((VisualizableModelElement) temp).getName().substring(0, substrEnd);
				
				if (name.equals(compareTo))
				{
					result.add((VisualizableModelElement) temp);
				}
			}
		}
		
		return result;
	}
	
	/**
	 * Attaches a CellModifier to the TableViewer.
	 */
	private void attachCellModifier()
	{
		this.tableViewer.setCellModifier(new ICellModifier()
        {
			public boolean canModify(Object element, String property)
			{
				return true;
			}

			public Object getValue(Object element, String property)
			{
				if (property.equals(ModelSearchView.MODEL_ELEMENT_FOUND_PROPERTY))
				{
					return ((EditableTableItem) element).modelElementFound;
				}
				else if (property.equals(ModelSearchView.MODEL_ELEMENT_FOUND_TYPE_PROPERTY))
				{
					return ((EditableTableItem) element).modelElementFoundType;
				}
				return null;
			}

			public void modify(Object element, String property, Object value)
			{
				TableItem tableItem = (TableItem) element;
				EditableTableItem data = (EditableTableItem) tableItem.getData();
				
				if (property.equals(ModelSearchView.MODEL_ELEMENT_FOUND_PROPERTY))
				{
					data.modelElementFound = value.toString();
				}
				if (property.equals(ModelSearchView.MODEL_ELEMENT_FOUND_TYPE_PROPERTY))
				{
					data.modelElementFoundType = value.toString();
				}
			}
		});
	}

	/**
	 * Attaches a LableProvider to the TableViewer.
	 */
	private void attachLabelProvider()
	{
		tableViewer.setLabelProvider(new ITableLabelProvider()
        {
			public Image getColumnImage(Object element, int columnIndex)
			{
				return null;
			}

			public String getColumnText(Object element, int columnIndex)
			{
				switch (columnIndex)
				{
		        	case 0:
		        		return ((EditableTableItem) element).modelElementFound;
		        	case 1:
		        		return ((EditableTableItem) element).modelElementFoundType;
		        	default:
		        		return "Invalid column: " + columnIndex;
		        }
			}

			public void addListener(ILabelProviderListener listener)
			{

			}

			public void dispose()
			{
				
			}

			public boolean isLabelProperty(Object element, String property)
			{
				return false;
			}

			public void removeListener(ILabelProviderListener listener)
			{
				
			}
		});
	}


	/**
	 * Inner class which models an EditableTableElement.
	 * This table element defines, how an entry in the table looks like.
	 * 
	 * @author Michael Jehle
	 */
	class EditableTableItem
	{
		public String modelElementFound;
		public String modelElementFoundType;

		public EditableTableItem(String modelElementFound, String modelElmentFoundType)
		{
			this.modelElementFound = modelElementFound;
			this.modelElementFoundType = modelElmentFoundType;
		}
	}
	
	/**
	 * If the user switches the adora model (switches the tab or
	 * opens another model), the search selection is reset (the model
	 * visibility is reset to its initial state).
	 */
	private void clearSearch()
	{
		for (int c = 0; c < this.reduceVisibilityCommands.size(); c++)
		{
			this.reduceVisibilityCommands.pop().undo();
		}
		
		this.clearTable();
		this.searchFor.setText("");
	}


	public void partActivated(IWorkbenchPart part)
	{
		try
		{
		if (!part.getTitle().equals("Model Search") &&
			this.reduceVisibilityCommands.size() > 0 &&
			!part.getTitle().equals(this.reduceVisibilityCommands.peek().getPartTitle()))
		{
			this.clearSearch();
		}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void partBroughtToTop(IWorkbenchPart part)
	{
		//do nothing
	}

	public void partClosed(IWorkbenchPart part)
	{
		//do nothing
	}

	public void partDeactivated(IWorkbenchPart part)
	{
		//do nothing
	}

	public void partOpened(IWorkbenchPart part)
	{
		//do nothing
	}
}