package ch.unizh.ifi.adora.ui.views.modelstatistics;

import java.beans.PropertyChangeEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.visitors.ElementCountVisitor;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.utils.mediator.ModelMediator;

/**
 *  This class implements an eclipse view which shows statistics about the
 *  active model.
 *  It implements PropertyChangeListener to be notified every time the model changes.
 *  
 * @author Michael Jehle
 */
public class StatisticsView extends ViewPart implements Observer, IPartListener
{
	private AdoraModel adoraModel;
	private Table elementsTable;
	private TableViewer tableViewer;
	private String[] columnNames = new String[] {"Model Element", "Count"};
	private int[] columnWidths = new int[] {120, 50};
	private int[] columnWeights = new int[] {25, 75};
	private static final String MODEL_ELEMENT_PROPERTY = "modelElement";
	private static final String ELEMENT_COUNT_PROPERTY = "elementCount";

	/**
	 * Empty constructor which initializes the view.
	 */
	public StatisticsView()
	{
		super();
	}

	@Override
	/**
	 * This method initializes the GUI-components of the view and
	 * registers itself as a PropertyChangeListener at the adoraModel.
	 */
	public void createPartControl(Composite parent)
	{
		this.initTable(parent);
		
		AdoraModelEditor activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
		if(activeAdoraEditor != null) {
			this.adoraModel = activeAdoraEditor.getModel();
			this.countModelElements();
		} else {
			elementsTable.setVisible(false);
		}
		
		this.getSite().getPage().addPartListener(this);

		ModelMediator.getInstance().addEventToReactOn(IContainer.CHILD_ADDED_PROP);
		ModelMediator.getInstance().addEventToReactOn(IContainer.CHILD_REMOVED_PROP);
		ModelMediator.getInstance().addEventToReactOn(ModelMediator.MODEL_CHANGED);
		ModelMediator.getInstance().addEventToReactOn(ConnectableElementPropNames.SOURCE_CONNECTIONS_PROP);
		
		ModelMediator.getInstance().addObserver(this);
		ModelMediator.getInstance().setAdoraModelEditor(activeAdoraEditor);
		
	}
	
	/**
	 * Initializes the view's table.
	 * 
	 * @param parent Composite
	 */
	private void initTable(Composite parent)
	{
		int style = SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION;
		
		this.elementsTable = new Table(parent, style);
		this.elementsTable.setHeaderVisible(true);
		this.elementsTable.setLinesVisible(true);
		
		TableLayout layout = new TableLayout();

		for (int c = 0; c < this.columnNames.length; c++)
		{
		    layout.addColumnData(new ColumnWeightData(this.columnWeights[c], this.columnWidths[c], true));
			
			TableColumn modelElementcolumn = new TableColumn(elementsTable, SWT.LEFT, c);
			modelElementcolumn.setText(this.columnNames[c]);
			modelElementcolumn.setWidth(this.columnWidths[c]);
			modelElementcolumn.setResizable(true);
		}
		
	    elementsTable.setLayout(layout);
		
		tableViewer = new TableViewer(elementsTable);
        tableViewer.setUseHashlookup(true);
        tableViewer.setColumnProperties(new String[] {StatisticsView.MODEL_ELEMENT_PROPERTY, StatisticsView.ELEMENT_COUNT_PROPERTY});
        
        this.attachCellModifier();
        this.attachLabelProvider();
	}

	/**
	 * Attaches a LableProvider to the TableViewer.
	 */
	private void attachLabelProvider()
	{
		tableViewer.setLabelProvider(new ITableLabelProvider()
        {
			public Image getColumnImage(Object element, int columnIndex)
			{
				return null;
			}

			public String getColumnText(Object element, int columnIndex)
			{
				switch (columnIndex)
				{
		        	case 0:
		        		return ((EditableTableItem) element).modelElement;
		        	case 1:
		        		return ((EditableTableItem) element).count.toString();
		        	default:
		        		return "Invalid column: " + columnIndex;
		        }
			}

			public void addListener(ILabelProviderListener listener)
			{

			}

			public void dispose()
			{
				
			}

			public boolean isLabelProperty(Object element, String property)
			{
				return false;
			}

			public void removeListener(ILabelProviderListener listener)
			{
				
			}
		});
	}

	/**
	 * Attaches a CellModifier to the TableViewer.
	 */
	private void attachCellModifier()
	{
		tableViewer.setCellModifier(new ICellModifier()
        {
			public boolean canModify(Object element, String property)
			{
				return true;
			}

			public Object getValue(Object element, String property)
			{
				if (property.equals(StatisticsView.MODEL_ELEMENT_PROPERTY))
				{
					return ((EditableTableItem) element).modelElement;
				}
				else if (property.equals(StatisticsView.ELEMENT_COUNT_PROPERTY))
				{
					return ((EditableTableItem) element).count;
				}
				return null;
			}

			public void modify(Object element, String property, Object value)
			{
				TableItem tableItem = (TableItem) element;
				EditableTableItem data = (EditableTableItem) tableItem.getData();
				
				if (property.equals(StatisticsView.MODEL_ELEMENT_PROPERTY))
				{
					data.modelElement = value.toString();
				}
				else if (property.equals(StatisticsView.ELEMENT_COUNT_PROPERTY))
				{
					data.count = (Integer) value;
				}
			}
		});
	}

	@Override
	public void setFocus()
	{
		
	}

	/**
	 * This method is responsible for counting the model elements using the visitor pattern.
	 */
	private void countModelElements()
	{
		Iterator<ModelElement> it = this.adoraModel.getRecursiveChildrenIterator();
		
		ElementCountVisitor visitor = new ElementCountVisitor();
		
		while (it.hasNext())
		{
			((ModelElement) it.next()).acceptElementVisitor(visitor);
		}
		
		HashMap<String, Integer> counts = visitor.getElementCount();
		
		for (int i = 0; i < ElementCountVisitor.identifiers.length; i++)
		{
			EditableTableItem element = new EditableTableItem(this.cutClassName(ElementCountVisitor.identifiers[i]), counts.get(ElementCountVisitor.identifiers[i]));
			
			for (int e = 0; e < ElementCountVisitor.identifiers.length; e++)
			{
				EditableTableItem tempTableItem = (EditableTableItem) this.tableViewer.getElementAt(e);
				
				if (tempTableItem != null)
				{
					String tempTableItemName = tempTableItem.modelElement;
				
					if (tempTableItemName.equals(element.modelElement))
					{
						this.tableViewer.remove(this.tableViewer.getElementAt(e));
					}
				}
			}
			
			this.tableViewer.add(element);
		}
	}
	
	/**
	 * Helper method which cuts off the package name of a class string.
	 * 
	 * @param className String
	 * @return className String
	 */
	private String cutClassName(String className)
	{
		String shortName = className.substring(className.lastIndexOf(".") + 1);
		
		if (shortName.equals("Component"))
		{
			return "Abstract Object";
		}
		else
		{
			return shortName;
		}
	}

	/**
	 * When the model changes, the model count is refreshed.
	 */
	public void propertyChange(PropertyChangeEvent arg0)
	{
		this.countModelElements();
	}
	
	/**
	 * Inner class which models an EditableTableElement.
	 * 
	 * @author Michael Jehle
	 */
	class EditableTableItem
	{
		public String modelElement;
		public Integer count;

		public EditableTableItem(String modelElement, Integer count)
		{
			this.modelElement = modelElement;
			this.count = count;
		}
	}

	/**
	 * This method is called by the mediator, when the model is changed.
	 * 
	 * @param observable Observable
	 * @param Object the event
	 */
	public void update(Observable observable, Object o)
	{
		if (!(o instanceof PropertyChangeEvent))
		{
			return;
		}

		this.countModelElements();
	}

	/**
	 * When a model editor part is activated, the statistics table
	 * is actualized to that new model.
	 */
	public void partActivated(IWorkbenchPart part)
	{
		if (part instanceof AdoraModelEditor && !elementsTable.isDisposed())
		{
			this.adoraModel = ((AdoraModelEditor) part).getModel();
			ModelMediator.getInstance().setAdoraModelEditor((AdoraModelEditor) part);
			this.countModelElements();
		}
	}

	/**
	 * No action required, when a part is brought to top.
	 */
	public void partBroughtToTop(IWorkbenchPart part)
	{
		
	}

	/**
	 * If a model is closed, the statistics table is disposed.
	 */
	public void partClosed(IWorkbenchPart part)
	{
		
		if (part instanceof AdoraModelEditor && !elementsTable.isDisposed())
		{
			
			IEditorReference[] editors = getSite().getPage().getEditorReferences();
			
			if(editors != null &&  editors.length == 0) {
				elementsTable.setVisible(false);
			}
		}
	}

	/**
	 * If a part is deactivated, no action is required.
	 */
	public void partDeactivated(IWorkbenchPart part)
	{
		
	}

	/**
	 * If a new model is opened, the statistics table
	 * is set to visible again.
	 */
	public void partOpened(IWorkbenchPart part)
	{
		if (part instanceof AdoraModelEditor && !elementsTable.isDisposed())
		{
			elementsTable.setVisible(true);
		}
	}

	/**
	 * Overrides the parents dispose method.
	 * It is only necessary to dispose the view's table, when
	 * a model tab is closed, the empty statistics view tab
	 * remains visible.
	 */
	public void dispose()
	{
		ModelMediator.getInstance().deleteObserver(this);
		
		if(!elementsTable.isDisposed()) {
			this.elementsTable.dispose();
		}
	}
}