package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import java.util.ArrayList;

/**
 /**
 * Represents a node with a label, image and children in the syntax error view tree.
 * In the current implementation, this node shows the parsed text.
 * 
 * @author 				Pascal Schoeni
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				21.09.2007 SM heavy refactoring.
 * 						24.01.2008 CR total refactoring.
 * @version				$Date: 2008-01-24 13:08:40 +0100 (Thu, 24 Jan 2008) $, $Author: cramer $, $Revision: 3498 $
 * 
 */
public class ContainerTreeNode extends LeafTreeNode {

	/**
	 * Contains the child nodes in the tree. In the current implementation, the child nodes are
	 * the error messages of the parsed text.
	 */
	private ArrayList<TreeNode> childNodes = new ArrayList<TreeNode>();

	
	
	/**
	 * Instantiates a new container node in the tree with a parent, a description and an image.
	 * 
	 * @param parent The parent node of this tree node. Null if this container is a root in the 
	 *        tree.
	 * @param description The description label which is shown in the tree.
	 * @param imageName The image which is shown in the tree. 
	 *                  The image must be a relative file path from the package {@link ch.unizh.ifi.adora}.
	 */
	public ContainerTreeNode(ContainerTreeNode parent, String description, String imageName) {
		super(parent, description, imageName);
	}

	/**
	 * Instantiates a new container root node in the tree with a description and an image.
	 * 
	 * @param description The description label which is shown in the tree.
	 * @param imageName The image which is shown in the tree. 
	 *                  The image must be a relative file path from the package {@link ch.unizh.ifi.adora}.
	 */
	public ContainerTreeNode(String description, String imageName) {
		super(null, description, imageName);
	}
	
	/**
	 * Instantiates a new container root node in the tree with a description but without image.
	 * @param description The image which is shown in the tree. 
	 *                    The image must be a relative file path from the package {@link ch.unizh.ifi.adora}.
	 */
	public ContainerTreeNode(String description) {
		super(null, description);
	}
	
	/**
	 * Instantiates a new container root node in the tree with a description but without image.
	 * @param description The image which is shown in the tree. 
	 *                    The image must be a relative file path from the package {@link ch.unizh.ifi.adora}.
	 */
	public ContainerTreeNode(ContainerTreeNode parent, String description) {
		super(parent, description);
	}

	/**
     * Adds a child node to this Node. If the mode is an instance of a
     * ParseRuleTreeNode, it is just added. Else, there is a
     * ParseRuleEntryTreeNode object generated, which has the model in it.
     * 
     * @param entry Adds a tree node as child.
     */
	public void addNode(TreeNode entry) {
		if (!this.childNodes.contains(entry)) {
			this.childNodes.add(entry);
		}
	}

	/**
	 * Removes a given child tree node
	 * 
	 * @param entry The child tree node to remove.
	 */
	public void removeNode(Object entry) {
		childNodes.remove(this.getNode(entry));
	}

	/**
	 * Getting all childNodes. Returns the child nodes as an array list.
	 * 
	 * @return Returns an arraylist containing all child nodes.
	 */
	public ArrayList<TreeNode> getChildNodes() {
		return this.childNodes;
	}

	/**
	 * Returns true, if there are more than zero child nodes.
	 * @return Returns true, if more than zero child nodes are existing.
	 */
	public boolean hasChildNodes() {
		return this.childNodes.size() > 0;
	}
	
	/**
	 * Removes all child nodes of this node.
	 * @pre       true
	 * @post      true
	 * @obligation
	 */
	public void removeNodes() {
		childNodes.clear();
	}
	
	/**
	 * Returns true, if the object given as argument is contained in one of the child nodes.
	 * The object can be the description lable (String) of the child node, the model part which 
	 * is represented by a {@link ch.unizh.ifi.adora.ui.views.syntaxerrors.ModelContainerTreeNode}
	 * or a {@link ch.unizh.ifi.adora.ui.views.syntaxerrors.TreeNode} object.
	 * 
	 * @param tn The object to look for.
	 * @return Returns true if the given object is found.
	 */
	public boolean containsNode(Object tn) {
		boolean retVal = (this.getNode(tn) != null);
		return retVal;
	}
	
	/**
	 * Returns an object for which the search argument object fits. The search argument can be the 
	 * description lable (String) of the child node, the model part which 
	 * is represented by a {@link ch.unizh.ifi.adora.ui.views.syntaxerrors.ModelContainerTreeNode}
	 * or a {@link ch.unizh.ifi.adora.ui.views.syntaxerrors.TreeNode} object.
	 * 
	 * @param tn The search argument to look for.
	 * @return Returns the found object which matches the search argument.
	 */
	public TreeNode getNode(Object tn) {				
		TreeNode retVal = null;
		for (TreeNode t : childNodes) {
			
			if (t.equals(tn)) {
				retVal = t;
				break;
			}
			
		}
		return retVal;
	}	
	
	
	/**
	 * Removes all the children from this node. 
	 */
	public void removeChildren(){
		removeNodes();
		}
	}

