package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * 
 * This represents a leaf node in the syntax error view hierarchy. The leaf node
 * can have a description and an image, but no children.
 * 
 * @author Pascal Schoeni
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.09.2007 SM Heavy refactoring.
 * 			24.01.2008 CR total refactoring again
 * @version $Date: 2008-01-24 13:08:40 +0100 (Thu, 24 Jan 2008) $, $Author: cramer $, $Revision: 3498 $
 * 
 */
public class LeafTreeNode implements TreeNode {

	/**
	 * Constant for the relative location an alert icon.
	 */
	public static final String ALERT_IMAGE = "icons/alert_obj.gif";

	/**
	 * Constant for the relative location an error icon.
	 */
	public static final String ERROR_IMAGE = "icons/error.gif";

	/**
	 * The description of this node.
	 */
	private String description = "";

	/**
	 * The parent node of this node. If no parent is given, i.e., the parent is
	 * <i>null</i>, this node is a root node.
	 */
	private ContainerTreeNode parent;

	/**
	 * Defaultly no image is used for the entry.
	 */
	private String imageName = "";

	/**
	 * Constructor initializes a leaf node with a parent, a description and an
	 * image.
	 * 
	 * @param parent
	 *            The parent node. This node is a root node if the parent is
	 *            <i>null</i>.
	 * @param description
	 *            The description lable of this node. It is displayed in the
	 *            tree view.
	 * @param imgName
	 *            The name of the image.
	 */
	public LeafTreeNode(ContainerTreeNode parent, String description,
			String imgName) {
		this.parent = parent;
		this.description = description;
		setImageName(imgName);
	}

	/**
	 * Constructor which initializes a leaf node with a parent and a description
	 * but with no image.
	 * 
	 * @param parent
	 *            The parent node of this node. This node is a root node if the
	 *            parent is <i>null</i>.
	 * @param description
	 *            The description lable of this node. It is displayed in the
	 *            tree view.
	 */
	public LeafTreeNode(ContainerTreeNode parent, String description) {
		this.parent = parent;
		this.description = description;
	}

	/**
	 * Returns the description which is returned by this node.
	 * 
	 * @return Returns the description lable.
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Sets the description lable of this node. It is displayed in the tree
	 * viewer.
	 * 
	 * @param description
	 *            The description lable of this node.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Returns the parent of this node. If this node is a root node, the parent
	 * is <i>null</i>.
	 * 
	 * @return Returns the parent of this node or <i>null</i>.
	 */
	public ContainerTreeNode getParent() {
		return this.parent;
	}

	/**
	 * Returns the image of this node. The image is specified with the method
	 * {@link #setImageName(String)}.
	 * 
	 * @return Returns the image of this node.
	 */
	public Image getImage() {
		Image img = null;
		if (!imageName.equals("")) {
			img = ImageDescriptor.createFromFile(AdoraPlugin.class, imageName)
					.createImage();
		}
		return img;
	}

	/**
	 * Overrides the equals method. It enables to compare this node with various
	 * other object. The following rule apply:
	 * <ol>
	 * <li>If the argument object is a
	 * {@link ch.unizh.ifi.adora.views.syntaxerrors.LeafTreeNode} object the
	 * method compares if its description is equal to the description of this
	 * object. The result of the comparison is returned.</li>
	 * <li>If the argument object is a {@link java.lang.String} object, it is
	 * compared to the description of this object. The result of the comparison
	 * is returned.</li>
	 * <li>Otherwise false is returned.
	 * </ol>
	 */
	@Override
	public boolean equals(Object obj) {
		boolean retVal = false;
		if (obj instanceof LeafTreeNode) {
			retVal = ((LeafTreeNode) obj).description.equals(this.description);
		} else if (obj instanceof String) {
			retVal = obj.equals(description);
		} else {
			retVal = false;
		}
		return retVal;
	}

	/**
	 * Returns the relative image file name. The filename path is relative to
	 * the directory structure of the package {@link ch.unizh.ifi.adora}.
	 * 
	 * @return Returns the relative filename of the image icon.
	 */
	public String getImageName() {
		return imageName;
	}

	/**
	 * Sets the image name of the image shown in the tree view. The filename
	 * path is relative to the directory structure of the package
	 * {@link ch.unizh.ifi.adora}.
	 * 
	 * @param imgName
	 *            The file name of the image shown in the tree view.
	 */
	public void setImageName(String imgName) {
		this.imageName = imgName;
	}

	/**
	 * Sets the parent node of this node. The parent can be <i>null</i> in this
	 * case this node is a root node.
	 * 
	 * @param parent
	 *            The parent node of this node.
	 */
	public void setParent(ContainerTreeNode parent) {
		this.parent = parent;
	}

	/**
	 * Returns the model object of the first found direct or indirect parent
	 * node (ancestor) in the syntax error tree view which associates a valid
	 * adora model object. This method can be used to determine the model object
	 * in which the error occurred.
	 * 
	 * @return Returns the first found model object of an direct or indirect
	 *         parent.
	 */
	public Object getAncestorNodeModel() {
		TreeNode tn = parent;
		Object retVal = null;
		while (tn != null) {
			if (tn instanceof ModelContainerTreeNode) {
				retVal = ((ModelContainerTreeNode) tn).getModel();
				if (retVal != null)
					break;
			}
			if (tn instanceof ContainerTreeNode) {
				tn = ((ContainerTreeNode) tn).getParent();
			}
		}
		return retVal;
	}

	/**
	 * Return false as a LeafTreeNode cannot have children.
	 */
	public boolean hasChildren() {
		return false;
	}


	
	/**
	 * Return the ModelElement corresponding to this tree entry. 
	 * @return
	 */
	public Object getModelElement() {
		return getAncestorNodeModel();
	}

	/**
	 * Do nothing as LeafTreeNode cannot have Children.
	 */
	public void removeChildren() {
		// TODO Auto-generated method stub
		
	}
}
