package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;

import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxProblem;
import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxProblemsSorter;

/** 
 * This class is an auxiliary class for {@link SyntaxErrorsView}.
 * It provides {@link ITreeContentProvider} services to it.
 * 
 * Basically, it provides a tree from a {@link SyntaxProblemsSorter}
 * input. The tree structure is:
 * 
 *  root ({@link SyntaxProblemsSorter}, not displayed)
 *  |_ syntax error name
 *     |_ error entries ({@link SyntaxProblem})
 * 
 * This class listens to its input and forwards changes to the
 * tree viewer it provides content for (via {@link Observer}/{@link Observable}
 * mechanism).
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ First Version
 * @version   $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */
class SyntaxErrorsContentProvider implements ITreeContentProvider, Observer {

	/**
	 * An empty array to return, in case there is no child
	 */
	private static final Object[] EMPTY_ARRAY = new Object[0];
	
	/**
	 * Cache a reference to the tree viewer (to echo change events)
	 */
	private TreeViewer viewer;
	
	/**
	 * The current "input" this tree is derived from
	 */
	private SyntaxProblemsSorter errors;

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object element)
	{
		Object[] children = EMPTY_ARRAY;
		if (element instanceof SyntaxProblemsSorter) {
			children = errors.getViolatedRules().toArray();
		}
		
		if (element instanceof String) {
			String violatedRuleName = (String) element;
			children = errors.getEntries(violatedRuleName).toArray();
		}
		
		return children;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element)
	{
		if (element instanceof SyntaxProblemsSorter) {
			return null;
		}
		
		if (element instanceof String) {
			return errors;
		}
		
		if (element instanceof SyntaxProblem) {
			SyntaxProblem entry = (SyntaxProblem) element;
			return entry.getSyntaxRuleName();
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element)
	{
		boolean result = false;
		if (element instanceof SyntaxProblemsSorter) {
			result = !errors.getViolatedRules().isEmpty();
		}
		
		if (element instanceof String) {
			result = true;
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement)
	{
		return getChildren(inputElement);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose()
	{
		this.viewer = null;
		if(errors != null)
		{
    		this.errors.deleteObserver(this);
    		this.errors = null;
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
	{
		// unregister from the oldInput
		if (oldInput != null && oldInput instanceof Observable)
		{
			Observable observable = (Observable) oldInput;
			observable.deleteObserver(this);
		}
		
		// register to the new input
		if (newInput != null && newInput instanceof SyntaxProblemsSorter)
		{
			this.errors = (SyntaxProblemsSorter) newInput;
			errors.addObserver(this);
		}
		
		this.viewer = (TreeViewer) viewer;
	}

	/* (non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable arg0, Object arg1)
	{
		viewer.refresh();
	}

}
