package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.BaseLabelProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.swt.graphics.Image;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxProblem;

/**
 * This class is an auxiliary class for {@link SyntaxErrorsView}.
 * It provides {@link ITableLabelProvider} services to it.
 * 
 * Basically, it provides label and icons for {@link SyntaxProblem}.
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ Refactoring of syntax error view
 * @version   $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */
class SyntaxErrorsLabelProvider extends BaseLabelProvider implements ITableLabelProvider {
	
	/**
	 * Constant for the relative location an alert icon.
	 */
	private static final String ALERT_IMAGE = "icons/alert_obj.gif";
	/**
	 * Constant for the relative location an error icon.
	 */
	private static final String ERROR_IMAGE = "icons/error.gif";
	/**
	 * Constant for the relative location of the transition icon.
	 */
	private static final String TRANSITION_IMAGE="icons/transition16.gif";
	/**
	 * Constant for the relative location of the rectangle icon.
	 */
	private static final String COMPONENT_IMAGE="icons/rectangle16.gif";
	/**
	 * Constant for the relative location an ellipse icon.
	 */
	private static final String SCENARIO_IMAGE="icons/ellipse16.gif";
	/**
	 * Constant for the relative location an aspect container icon.
	 */
	private static final String ASPECTCONTAINER_IMAGE="icons/AspectContainer.gif";
	/**
	 * Constant for the relative location an joint relation icon.
	 */
	private static final String JOINTRELATION_IMAGE = "icons/JoinRelation16.gif";
	
	/**
	 * A cache for images (mapping from name to {@link Image})
	 */
	private Map<String, Image> imageCache;
	
	/**
	 * Constructor for this class.
	 * Should be called by {@link SyntaxErrorsView} only
	 */
	SyntaxErrorsLabelProvider()
	{
		imageCache = new HashMap<String, Image>();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
	 */
	public Image getColumnImage(Object element, int columnIndex)
	{
		if(element instanceof SyntaxProblem)
		{
			SyntaxProblem entry = (SyntaxProblem)element;
			if(columnIndex == SyntaxErrorsView.MODEL_ELEMENT_COL)
			{
				return getModelElementImage(entry.getModelElement());
			}
			else if(columnIndex == SyntaxErrorsView.SYNTAX_RULE_ID_COL)
			{
				if(SyntaxProblem.Severity.Error == entry.getSeverity())
				{
					return getImage(ERROR_IMAGE);
				}
				if(SyntaxProblem.Severity.Warning == entry.getSeverity())
				{
					return getImage(ALERT_IMAGE);
				}
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
	 */
	public String getColumnText(Object element, int columnIndex)
	{
		if(element instanceof SyntaxProblem)
		{
			SyntaxProblem entry = (SyntaxProblem)element;
			if(columnIndex == SyntaxErrorsView.MODEL_ELEMENT_COL)
			{
				return entry.getModelElement().toString();
			}
			else if(columnIndex == SyntaxErrorsView.ERROR_MSG_COL)
			{
				return entry.getErrorMessage();
			}
			else if(columnIndex == SyntaxErrorsView.SYNTAX_RULE_ID_COL)
			{
				return entry.getSyntaxRuleName();
			}
		}
		else if(element instanceof String)
		{
			String syntaxID = (String) element;
			if(columnIndex == SyntaxErrorsView.SYNTAX_RULE_ID_COL)
			{
				return syntaxID;
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.BaseLabelProvider#dispose()
	 */
	@Override
	public void dispose()
	{
		for (Iterator<Image> it = imageCache.values().iterator(); it.hasNext();) {
			it.next().dispose();
		}
		imageCache.clear();
		super.dispose();
	}

	/**
	 * Get the image of a model element
	 * @param element a model element
	 * @return an image corresponding to the given model element
	 */
	private Image getModelElementImage(Object element)
	{
		String imgName = null;
		if(element instanceof Component)
		{
			imgName = COMPONENT_IMAGE;
		}
		else if(element instanceof AspectContainer)
		{
			imgName = ASPECTCONTAINER_IMAGE;
		}
		else if(element instanceof Scenario)
		{
			imgName = SCENARIO_IMAGE;
		}
		else if(element instanceof JoinRelation)
		{
			imgName = JOINTRELATION_IMAGE;
		}
		else if(element instanceof Transition)
		{
			imgName = TRANSITION_IMAGE;
		}
		return getImage(imgName);
	}

	/**
	 * Obtain an {@link Image} from its name
	 * and store it in the cache.
	 * 
	 * @param imgName the name of the requested image
	 * @return an {@link Image} according to the given name
	 */
	private Image getImage(String imgName)
	{
		Image image = null;
		if(imgName != null)
		{
			image = imageCache.get(imgName);
			if (image == null)
			{
				ImageDescriptor desc = ImageDescriptor.createFromFile(AdoraPlugin.class, imgName);
				image = desc.createImage();
				imageCache.put(imgName, image);
			}
		}
		return image;
	}
}
