package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IPropertyListener;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationRule;
import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxProblem;
import ch.unizh.ifi.adora.properties.longtextproperty.ILongTextEditorInput;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class is a view that displays the syntax error of an {@link AdoraModelEditor}.
 *
 * It is a {@link TreeViewer} with several columns.
 * 
 * This class listen to the current editor in case the editor changes the
 * model it edits.
 *
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ First version (Refactoring of syntax error view)
 * @version   $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */

public class SyntaxErrorsView extends ViewPart implements IPropertyListener {

	/**
	 * The ID of this view
	 */
	public static final String VIEW_ID = "ch.unizh.ifi.adora.editor.views.SyntaxErrorsView";
	
	/**
	 * The name of the columns
	 * Make sure the index matches with the one defined right after
	 */
	static final String[] COL_NAMES = {"Syntax Rule","Invalid Model element", "Error Message"};
	
	
	/**
	 * The column index for the name of the violated
	 * {@link ISyntaxVerificationRule}.
	 */
	static final int SYNTAX_RULE_ID_COL = 0;

	/**
	 * The column index for the name of the invalid
	 * {@link ModelElement}
	 */
	static final int MODEL_ELEMENT_COL = 1;

	/**
	 * The column index for the message
	 */
	static final int ERROR_MSG_COL = 2;
	
	/**
	 * The {@link AdoraModelEditor} currently active.
	 */
	private AdoraModelEditor currentEditor;
	
	/**
	 * The {@link TreeViewer} this view relies on
	 */
	private TreeViewer treeViewer;
	
	/**
	 * The {@link IAdapterFactory} that translates
	 * {@link SyntaxProblem} to {@link ILongTextEditorInput}.
	 */
	private EditorInputAdapterFactory editorInputFactory;
	
	/**
	 * An {@link IPartListener} that will keep track
	 * of the current {@link AdoraModelEditor}.
	 */
	private IPartListener partListener;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent)
	{
		treeViewer = new TreeViewer(parent);
			
		treeViewer.setAutoExpandLevel(TreeViewer.ALL_LEVELS);
		treeViewer.addSelectionChangedListener(new SyntaxErrorsSelectionListener(this));
		treeViewer.setContentProvider(new SyntaxErrorsContentProvider());
		treeViewer.setLabelProvider(new SyntaxErrorsLabelProvider());
		
		treeViewer.getTree().setHeaderVisible(true);
		treeViewer.getTree().setLinesVisible(true);
		treeViewer.getTree().addMouseListener(new SyntaxErrorsMouseListener(this));
		
		for (String columnName : COL_NAMES) {
			TreeColumn column = new TreeColumn(treeViewer.getTree(),SWT.LEFT);
			column.setText(columnName);
			column.setWidth(200);
		}
		
		setCurrentEditor(AdoraPlugin.getActiveEditor());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.ViewPart#init(org.eclipse.ui.IViewSite)
	 */
	@Override
	public void init(IViewSite site) throws PartInitException
	{
		super.init(site);
		
		editorInputFactory = new EditorInputAdapterFactory();
		partListener = new SyntaxErrorsPartListener(this);

		Platform.getAdapterManager().registerAdapters(editorInputFactory, SyntaxProblem.class);
		site.getPage().addPartListener(partListener);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#dispose()
	 */
	@Override
	public void dispose()
	{
		getSite().getPage().removePartListener(partListener);
		Platform.getAdapterManager().unregisterAdapters(editorInputFactory);
		
		treeViewer = null;
		editorInputFactory = null;
		partListener = null;
		
		super.dispose();
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPropertyListener#propertyChanged(java.lang.Object, int)
	 */
	public void propertyChanged(Object source, int propId)
	{
		if(propId == AdoraModelEditor.PROP_MODEL_CHANGE)
		{
			setCurrentEditor((AdoraModelEditor) source);
		}
	}

	/**
	 * @return the current {@link AdoraModelEditor}
	 */
	AdoraModelEditor getCurrentEditor() {
		return currentEditor;
		
	}

	/**
	 * Change the {@link AdoraModelEditor}
	 * @param part the new active {@link AdoraModelEditor}
	 */
	void setCurrentEditor(AdoraModelEditor part)
	{
		if(currentEditor != null)
		{
			currentEditor.removePropertyListener(this);
		}
		
		currentEditor = part;
		
		if(currentEditor != null)
		{
			currentEditor.addPropertyListener(this);
			treeViewer.setInput(part.getErrors());
		}
		else
		{
			treeViewer.setInput(null);
		}
	}

	/**
	 * Get the selection of the tree viewer
	 * @return selected elements
	 */
	IStructuredSelection getSelection()
	{
		return (IStructuredSelection) treeViewer.getSelection();
	}
	

}
