package ch.unizh.ifi.adora.utils;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * Base class that can be used to register listeners and notify them about
 * property changes.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-01-10 TR First Version
 * @version				$Date: 2008-01-10 10:01:21 +0100 (Thu, 10 Jan 2008) $, $Author: treinhard $, $Revision: 3446 $
 * @responsibilities	Provides operations for observing property changes.
 *
 */
public abstract class ObservablePropertyContainer {

	/** 
	 * Delegate used to implemenent property-change-support. 
	 */
	private transient PropertyChangeSupport pcsDelegate;

	/**
	 * The constructur of the model element class which initializes its
	 * attributes.
	 * 
	 * @pre       true
	 * @post      true
	 */
	public ObservablePropertyContainer() {
        pcsDelegate = new PropertyChangeSupport(this);
	}
	
	/**
	 * Attach a non-null PropertyChangeListener to this object.
	 * 
	 * @param l
	 *            a non-null PropertyChangeListener instance
	 * @throws IllegalArgumentException
	 *             if the parameter is null
	 */
	public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
		if (l == null) {
			throw new IllegalArgumentException();
		}
		pcsDelegate.addPropertyChangeListener(l);
	}
	
	
	/**
	 * Attach a non-null PropertyChangeListener to this object that listens only to a given property
	 * 
	 * @param l a non-null PropertyChangeListener instance
	 * @param property the property that the listener shall be informed about
	 * @throws IllegalArgumentException
	 *             if the parameter is null
	 */
	public synchronized void addPropertyChangeListener(String property,PropertyChangeListener l) {
		if (l == null) {
			throw new IllegalArgumentException();
		}
		pcsDelegate.addPropertyChangeListener(property,l);
	}

	/**
 	 * 
	 * Remove a PropertyChangeListener from this component.
	 * 
	 * @param l A PropertyChangeListener instance
	 */
	public synchronized void removePropertyChangeListener(
			PropertyChangeListener l) {
		if (l != null) {
			pcsDelegate.removePropertyChangeListener(l);
		}
	}
	
	/**
	 * Children should override this. The default implementation does nothing.
	 */
	public void resetPropertyValue(Object id) {
		// do nothing
	}


	/**
	 * Report a property change to registered listeners (for example edit
	 * parts).
	 * 
	 * @param property The programmatic name of the property that changed
	 * @param oldValue The old value of this property
	 * @param newValue The new value of this property
	 */
	public  void firePropertyChange(String property, Object oldValue,
			Object newValue) {
		if (pcsDelegate.hasListeners(property)) {
			pcsDelegate.firePropertyChange(property, oldValue, newValue);
		}
	}

}

