package ch.unizh.ifi.adora.utils;

import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.core.runtime.Preferences.PropertyChangeEvent;
import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceValueConstants;
import ch.unizh.ifi.adora.zoom.berner.algorithm.LayoutAlgorithm;
import ch.unizh.ifi.adora.zoom.berner.algorithm.MinRadial;
import ch.unizh.ifi.adora.zoom.berner.algorithm.NewBerner;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;

/**
 * Singleton to manage the different zoom algorithms. The zoom algorithm that
 * is currently set in the preferences can be obtained by calling getZoomAlgorithm.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-03-19 TR First Version
 * @version				$Date: 2009-07-20 15:44:11 +0200 (Mon, 20 Jul 2009) $, $Author: treinhard $, $Revision: 4528 $
 *
 */
public class ZoomAlgorithmsManager implements IPropertyChangeListener {

	/**
	 * The currently set zoom algorithm.
	 */
	private LayoutAlgorithm zoomAlgorithm;
	
	/**
	 * The layout algorithm that is called if a node is inserted
	 */
	private LayoutAlgorithm insertAlgorithm;
	
	/**
	 * The layout algorithm that is called if a node is removed
	 */
	private LayoutAlgorithm removeAlgorithm;
	
	/**
	 * The layout algorithm that is called if a node is moved
	 */
	private LayoutAlgorithm moveAlgorithm;
	
	/**
	 * The layout algorithm that is called if a view filter is applied
	 */
	private LayoutAlgorithm filterAlgorithm;
	
	/**
	 * Singleton variable.
	 */
	private static ZoomAlgorithmsManager instance;
	
	/**
     * The preference store (used to get the routers that are set in the preferences)
     */
    private IPreferenceStore store;
	
	/**
	 * Private constructor.
	 */
	private ZoomAlgorithmsManager() {
				
		store  = AdoraPlugin.getDefault().getPreferenceStore();
        AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);
        
        setZoomAlgorithm();
        setInsertAlgorithm();
        setRemoveAlgorithm();
        setMoveAlgorithm();
        setFilterAlgorithm();
		
	}
	
	
	/**
	 * Returns the singleton instance of this class.
	 * 
	 * @return the singleton instance
	 */
	public static ZoomAlgorithmsManager getInstance() {
		
		if(instance == null) {
			instance = new ZoomAlgorithmsManager();
		}
		
		return instance;
		
	}
	
	
	/**
	 * Returns the zoom algorithm that is currently selected in the
	 * preferences.
	 * 
	 * @return the currently selected zoom algorithm
	 */
	public LayoutAlgorithm getZoomAlgorithm() {
		
		return zoomAlgorithm;
		
	}
	
	/**
	 * Returns the layout algorithm that handles the insertion of a node.
	 * 
	 * @return the layout algorithm that handles insertions
	 */
	public LayoutAlgorithm getInsertAlgorithm() {
		
		return insertAlgorithm;
		
	}
	
	
	/**
	 * Returns the layout algorithm that handles the removal a node.
	 * 
	 * @return the layout algorithm that handles removals
	 */
	public LayoutAlgorithm getRemoveAlgorithm() {
		
		return removeAlgorithm;
		
	}
	
	
	/**
	 * Returns the layout algorithm that handles the movement of a node.
	 * 
	 * @return the layout algorithm that handles movements
	 */
	public LayoutAlgorithm getMoveAlgorithm() {
		
		return moveAlgorithm;
		
	}
	
	
	/**
	 * Returns the layout algorithm that handles view filters.
	 * 
	 * @return the layout algorithm that handles view filters
	 */
	public LayoutAlgorithm getFilterAlgorithm() {
		
		return filterAlgorithm;
		
	}

	
	/**
	 * Called by the preference store to become informed about changes in the
	 * preferences.
	 * 
	 * @param event the event
	 */
	public void propertyChange(PropertyChangeEvent event) {
		
		if(event.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ZOOM_ALGORITHM)) {
			
			setZoomAlgorithm();
			
		}
		
		else if(event.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_INSERT_LAYOUT_ALGORITHM)) {
			
			setInsertAlgorithm();
			
		}
		
		else if(event.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_REMOVE_LAYOUT_ALGORITHM)) {
			
			setRemoveAlgorithm();
			
		}
		
		else if(event.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_MOVE_LAYOUT_ALGORITHM)) {
			
			setMoveAlgorithm();
			
		}
		
		else if(event.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_FILTER_LAYOUT_ALGORITHM)) {
			
			setFilterAlgorithm();
			
		}
		
		else if(event.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SCALE_GAPS)) {
		
			((AdoraZoomAdapter)filterAlgorithm).setScaleGapIntervals(store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SCALE_GAPS));
		}
	}
	
	
	/**
	 * Sets the zoom algorithm to the one selected in the preferences.
	 */
	private void setZoomAlgorithm() {
		
		String zoomName = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_ZOOM_ALGORITHM);
		
		if(zoomName.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_BERNER)) {
			
			zoomAlgorithm = NewBerner.getInstance();
			
		}
		else if(zoomName.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS)) {
			
			zoomAlgorithm = AdoraZoomAdapter.getInstance();
			
		}
		
	}
	
	
	/**
	 * Sets the node insertion algorithm to the one selected in the preferences.
	 */
	private void setInsertAlgorithm() {
		
		String name = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_INSERT_LAYOUT_ALGORITHM);
		
		if(name.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_MINRADIAL)) {
			
			insertAlgorithm = MinRadial.getInstance();
			
		}
		
		else if(name.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS)) {
			
			insertAlgorithm = AdoraZoomAdapter.getInstance();
			
		}
		
	}
	
	
	/**
	 * Sets the node removal algorithm to the one selected in the preferences.
	 */
	private void setRemoveAlgorithm() {
		
		String name = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_REMOVE_LAYOUT_ALGORITHM);
		
		if(name.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_BERNER)) {
			
			removeAlgorithm = NewBerner.getInstance();
			
		}
		
		else if(name.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS)) {
			
			removeAlgorithm = AdoraZoomAdapter.getInstance();
			
		}
		
	}
	
	
	/**
	 * Sets the node move algorithm to the one selected in the preferences.
	 */
	private void setMoveAlgorithm() {
		
		String name = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_MOVE_LAYOUT_ALGORITHM);
		
		if(name.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_MINRADIAL)) {
			
			moveAlgorithm = MinRadial.getInstance();
			
		}
		
		else if(name.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS)) {
			
			moveAlgorithm = AdoraZoomAdapter.getInstance();
			
		}
		
	}
	
	
	/**
	 * Sets the filter layout algorithm to the one selected in the preferences.
	 */
	private void setFilterAlgorithm() {
		
		String name = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_FILTER_LAYOUT_ALGORITHM);
		
		if(name.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_BERNER)) {
			
			filterAlgorithm = NewBerner.getInstance();
			
		}
		
		else if(name.equals(IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS)) {
			
			filterAlgorithm = AdoraZoomAdapter.getInstance();
			
		}
		
	}
	
	
}
