package ch.unizh.ifi.adora.utils.mediator;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Observable;

import org.eclipse.ui.IPropertyListener;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class acts as a metiator between the adora model and its
 * observers.
 * The events on which the mediator reacts can be defined by the user of
 * this class.
 * 
 * @author Michael Jehle
 */
public class ModelMediator extends Observable implements PropertyChangeListener, IPropertyListener
{
	/**
	 * Identifier, which indicates that the model has changed.
	 */
	public static String MODEL_CHANGED = "model.changed";

	/**
	 * The instance (singleton pattern).
	 */
	private static ModelMediator instance;
	
	/**
     * The current {@link AdoraModelEditor}.
     */
	private AdoraModelEditor observedAdoraModelEditor;
	
	private ArrayList<Object> eventsToReactOn;

	private ModelMediator()
	{
		this.eventsToReactOn = new ArrayList<Object>();
	}

	/**
	 * Returns the instance of the {@link ModelMediator} (singleton).
	 * 
	 * @return {@link ModelMediator}
	 */
	public static ModelMediator getInstance()
	{
		if (ModelMediator.instance == null)
		{
			ModelMediator.instance = new ModelMediator();
		}
		
		return ModelMediator.instance;
	}
	
	/**
     * Sets the {@link AdoraModelEditor}. If the model editor has changed, it
     * removes all listeners from the old editor / model and
     * adds listeners to the new editor and its model.
     * 
     * @param adoraModelEditor
     *            The current {@link AdoraModelEditor}.
     */
	public void setAdoraModelEditor(AdoraModelEditor adoraModelEditor)
	{
		if (this.observedAdoraModelEditor == adoraModelEditor)
		{
			return;
		}

		if (this.observedAdoraModelEditor != null)
		{
			this.observedAdoraModelEditor.removePropertyListener(this);
			removeListeners(this.observedAdoraModelEditor.getModel());
		}

		this.observedAdoraModelEditor = adoraModelEditor;
		addListeners(this.observedAdoraModelEditor.getModel());
		adoraModelEditor.addPropertyListener(this);
		
		setChanged();
		notifyObservers(MODEL_CHANGED);
	}
	
	/**
	 * This mediator notifies its observers only on predefined
	 * events. They can be set through this method.
	 * 
	 * @param eventName Object
	 */
	public void addEventToReactOn(Object eventName)
	{
		this.eventsToReactOn.add(eventName);
	}
	
	/**
	 * If the observers of this mediator should no longer be notified on
	 * events of a certain type, remove the event name from the observer
	 * by using this method.
	 * 
	 * @param eventName Object
	 */
	public void removeEventToReactOn(Object eventName)
	{
		if (this.eventsToReactOn.contains(eventName))
		{
			this.eventsToReactOn.remove(eventName);
		}
	}

	/**
	 * Removes all the listeners from the {@link AdoraModel}.
	 * @param model {@link AdoraModel}
	 */
	private void removeListeners(AdoraModel model)
	{
		model.removePropertyChangeListener(this);
		
		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();
		
		while (iterator.hasNext())
		{
			removeListener(iterator.next());
		}
	}
	
	/**
     * Removes a listener from a specific {@link ModelElement}.
     * 
     * @param modelElement The {@link ModelElement} where the
     * listeners should be deactivated.
     */
	private void removeListener(ModelElement modelElement)
	{
		if (!(modelElement instanceof ObservableModelElement))
		{
			return;
		}

		ObservableModelElement observable = (ObservableModelElement) modelElement;
		observable.removePropertyChangeListener(this);

		if (observable instanceof ConnectableElement)
		{
			ConnectableElement connectable = (ConnectableElement) observable;
			for (Connection connection : connectable.getConnections())
			{//fr alle connectionstypen
				connection.removePropertyChangeListener(this);
			}
		}
	}

	/**
     * Adds all listeners to the {@link AdoraModel}.
     * 
     * @param AdoraModel model.
     */
	private void addListeners(AdoraModel model)
	{
		model.addPropertyChangeListener(this);

		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();
		
		while (iterator.hasNext())
		{
			addListener(iterator.next());
		}
	}

	/**
	 * Registers the mediator at a specific model element.
	 * 
	 * @param element ModelElement
	 */
	private void addListener(ModelElement element)
	{
		if (element instanceof ObservableModelElement)
		{
			ObservableModelElement observableModelElement = (ObservableModelElement) element;
			observableModelElement.addPropertyChangeListener(this);	
		
			if (observableModelElement instanceof ConnectableElement)
			{
				ConnectableElement connectable = (ConnectableElement) observableModelElement;

				for (Connection connection : connectable.getConnections())
				{ //alle arten von verbindungen zu connectable element
					connection.addPropertyChangeListener(this);
				}
			}
		}
	}

	public void propertyChange(PropertyChangeEvent event)
	{
		String eventName = event.getPropertyName();
		
		if (eventName.equals(IContainer.CHILD_ADDED_PROP))
		{
			this.addListener((ModelElement) event.getNewValue());
		}
		
		if (eventName.equals(IContainer.CHILD_REMOVED_PROP))
		{
			this.removeListener((ModelElement) event.getNewValue());
		}

		if (this.eventsToReactOn.contains(eventName))
		{
			setChanged();
			notifyObservers(event);
		}
	}

	/**
     * Implemented to listen, if the editor reloads the model (for example for
     * the weaving).
     * 
     * @param source
     * @param propId
     */
	public void propertyChanged(Object source, int propId)
	{

		if (propId == AdoraModelEditor.PROP_MODEL_CHANGE)
		{
			removeListeners(observedAdoraModelEditor.getModel());
			addListeners(observedAdoraModelEditor.getModel());

			setChanged();
			notifyObservers(MODEL_CHANGED);
		}
	}	
}