package ch.unizh.ifi.adora.view.anchors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.view.figures.ConnectionFigure;
import ch.unizh.ifi.adora.view.figures.geometry.LineSeg;
import ch.unizh.ifi.adora.view.figures.geometry.PointListUtilities;
import ch.unizh.ifi.adora.view.figures.geometry.LineSeg.KeyPoint;
import ch.unizh.ifi.adora.view.figures.geometry.PointListUtilities.LocateInfo;

/**
 * A {@link org.eclipse.draw2d.ConnectionAnchor} for
 * {@link ch.unizh.ifi.adora.view.figures.TransitionFigure}s. 
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             09.08.2006 IV First Version
 *                       29.09.2006 JS Forcing click points to be on the line
 * @version             $Date: 2007-04-20 09:42:41 +0200 (Fri, 20 Apr 2007) $, $Author: treinhard $, $Revision: 2607 $
 */
public class ConstrainableConnectionAnchor extends ClickPointAnchor {

   	/**
	 * The owner.
	 */
    private final PolylineConnection owner;
    
    /**
     * Stores the ratio startpoint to endpoint compared to 
     * startpoint to joinRelation point.
     */
    private double clickpoint2lineRation;
    
    /**
	 * Instantiates a new Anchor for the given <li>owner</li>.
	 * 
	 * @param owner The figure for which this Anchor is been 
	 * Instantiated. 
	 */
	public ConstrainableConnectionAnchor(PolylineConnection owner) {
		super(owner);
        this.owner = owner;
	}
	
	/**
	 * Returns the midpoint of the owner connection.
	 */
	public Point getLocation(Point reference) {
		
		Point p = getUnscaledLocation(reference);
		
		getOwner().translateToAbsolute(p);
		
		return p;
	}
	
	/**
	 * Returns the absolute and unscaled coordinates of the point on the
	 * figure's border where the connection should start or end. The 
	 * point is defined by the interscetion between the line segment formed
	 * by the click point inside the node and the point given as argument and
	 * the border of the figure.
	 * The returned coordinates are unscaled (i.e. not adjsuted to the
	 * viewport and the scale factor).
	 * 
	 * @param reference the reference point outside the figure's border
	 * @return the point on the figure's border
	 */
	public Point getUnscaledLocation(Point reference) {
		
		Point p = new Point();
		LocateInfo locateInfo = new LocateInfo();
		
		PointListUtilities.locateSegment(
				PointListUtilities.getLineSegments(getOwner().getPoints()), 
				clickpoint2lineRation,
				KeyPoint.ORIGIN,
				locateInfo
		);
			 
		List<LineSeg> segs = new ArrayList<LineSeg>();
		segs.add(locateInfo.theSegment);
			 
		p = PointListUtilities.pointOn(
				segs, 
				locateInfo.remainingDist, 
				KeyPoint.ORIGIN, 
				p
			);
		
		return p;	
		
	}
	
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.anchors.ClickPointAnchor#setClickPoint(org.eclipse.draw2d.geometry.Point)
	 */
	public void setClickPoint(Point p) {

		Rectangle realBounds = ((ConnectionFigure)owner).getRealBounds();
		
		float xRatio = p.x / (float)realBounds.width;
		float yRatio = p.y / (float)realBounds.height;
		
		setClickPoint(xRatio, yRatio);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.anchors.ClickPointAnchor#setClickPoint(float, float)
	 */
	public void setClickPoint(float xRatio, float yRatio) {
		
		Rectangle realBounds = ((ConnectionFigure)owner).getRealBounds();
		
		Point clickPoint = new Point(
				realBounds.getLocation().x + 
					 	(int) (xRatio * realBounds.width), 
					 	realBounds.getLocation().y + 
					 	(int) (yRatio * realBounds.height));
			
		clickpoint2lineRation = PointListUtilities.distanceAlong(
				 PointListUtilities.getLineSegments(getOwner().getPoints()), clickPoint
		);
		// Make sure that the click point really is on the line, not just near it
		LocateInfo locateInfo = new LocateInfo();
		PointListUtilities.locateSegment(
				PointListUtilities.getLineSegments(getOwner().getPoints()), 
				clickpoint2lineRation,
				KeyPoint.ORIGIN,
				locateInfo
		);
		// ... by finding a point which is on the segment nearest to the original clickPoint
		locateInfo.theSegment.pointOn(locateInfo.remainingDist, KeyPoint.ORIGIN, clickPoint);
		xRatio = (clickPoint.x - realBounds.getLocation().x) / (float)realBounds.width;
		yRatio = (clickPoint.y - realBounds.getLocation().y) / (float)realBounds.height;

		this.xRatio = xRatio;
		this.yRatio = yRatio;
	}
    
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.ConnectionAnchor#getOwner()
	 */
    public PolylineConnection getOwner() {
		return owner;
	}
    
    
	/**
	 * @return The ClickPoint.
	 */
	public Point getClickPoint() {
		
		Rectangle realBounds = ((ConnectionFigure)owner).getRealBounds();
		
		int x = Math.round(realBounds.width * xRatio);
		int y = Math.round(realBounds.height * yRatio);
		
		return new Point(x, y);
	}
}