package ch.unizh.ifi.adora.view.anchors;


import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.view.figures.ConnectionFigure;
import ch.unizh.ifi.adora.view.figures.ConstraintConnectionFigure;
import ch.unizh.ifi.adora.view.figures.geometry.GeometryUtils;
import ch.unizh.ifi.adora.view.figures.geometry.PointListUtilities;
import ch.unizh.ifi.adora.view.figures.geometry.LineSeg.KeyPoint;
import ch.unizh.ifi.adora.view.figures.geometry.PointListUtilities.LocateInfo;

/**
 * A {@link org.eclipse.draw2d.ConnectionAnchor} for a {@link ConstraintConnectionFigure}
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             09.12.2008 IV First Version
 * @version             $Date: 2008-12-09 16:23:32 +0100 (Tue, 09 Dec 2008) $, $Author: vigan $, $Revision: 3956 $
 */
public class ConstraintConnectionAnchor extends ClickPointAnchor {

   	/**
	 * The owner.
	 */
    private final ConstraintConnectionFigure owner;
    
    /**
     * Stores the ratio startpoint to endpoint compared to 
     * startpoint to joinRelation point.
     */
    private double clickpoint2lineRation;
    
    /**
	 * Instantiates a new Anchor for the given <li>owner</li>.
	 * 
	 * @param owner The figure for which this Anchor is been 
	 * Instantiated. 
	 */
	public ConstraintConnectionAnchor(PolylineConnection owner) {
		super(owner);
        this.owner = (ConstraintConnectionFigure)owner;
	}
	
	/**
	 * Returns the midpoint of the owner connection.
	 */
	public Point getLocation(Point reference) {
		
		Point p = getUnscaledLocation(reference);
		
		getOwner().translateToAbsolute(p);
		
		return p;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.anchors.ClickPointAnchor#setClickPoint(org.eclipse.draw2d.geometry.Point)
	 */
	public void setClickPoint(Point p) {

		Rectangle realBounds = ((ConnectionFigure)owner).getRealBounds();
		
		float xRatio = p.x / (float)realBounds.width;
		float yRatio = p.y / (float)realBounds.height;
		
		setClickPoint(xRatio, yRatio);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.anchors.ClickPointAnchor#setClickPoint(float, float)
	 */
	public void setClickPoint(float xRatio, float yRatio) {
		
		Rectangle realBounds = ((ConnectionFigure)owner).getRealBounds();
		
		Point clickPoint = new Point(
				realBounds.getLocation().x + 
					 	(int) (xRatio * realBounds.width), 
					 	realBounds.getLocation().y + 
					 	(int) (yRatio * realBounds.height));
			
		clickpoint2lineRation = PointListUtilities.distanceAlong(
				 PointListUtilities.getLineSegments(getOwner().getPoints()), clickPoint
		);
		// Make sure that the click point really is on the line, not just near it
		LocateInfo locateInfo = new LocateInfo();
		PointListUtilities.locateSegment(
				PointListUtilities.getLineSegments(getOwner().getPoints()), 
				clickpoint2lineRation,
				KeyPoint.ORIGIN,
				locateInfo
		);
		// ... by finding a point which is on the segment nearest to the original clickPoint
		locateInfo.theSegment.pointOn(locateInfo.remainingDist, KeyPoint.ORIGIN, clickPoint);
		xRatio = (clickPoint.x - realBounds.getLocation().x) / (float)realBounds.width;
		yRatio = (clickPoint.y - realBounds.getLocation().y) / (float)realBounds.height;

		this.xRatio = xRatio;
		this.yRatio = yRatio;
	}
    
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.ConnectionAnchor#getOwner()
	 */
    public ConstraintConnectionFigure getOwner() {
		return owner;
	}
    
    
	/**
	 * @return The ClickPoint.
	 */
	public Point getClickPoint() {
		
		Rectangle realBounds = ((ConnectionFigure)owner).getRealBounds();
		
		int x = Math.round(realBounds.width * xRatio);
		int y = Math.round(realBounds.height * yRatio);
		
		return new Point(x, y);
	}

	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.anchors.ClickPointAnchor#getUnscaledLocation(org.eclipse.draw2d.geometry.Point)
	 */
	public Point getUnscaledLocation(Point reference) {

		Rectangle r = Rectangle.SINGLETON;
		int dia = ConstraintConnectionFigure.ellipseDiameter;
		

		r.setBounds(new Rectangle(getOwner().getMidpoint().getTranslated(-dia/2, -dia/2), new Dimension(dia, dia)));

		if(getOwner().getParent() == null){
			return reference;
		}

		Point inside = getOwner().getMidpoint();
		Point p = GeometryUtils.intersectEllipse(r, inside, reference);
		
		if(p == null) {
			p = inside;
		}
		
		return p;
	}
}