package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.model.representation.Cardinality;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.view.figures.geometry.GeometryUtils;
import ch.unizh.ifi.adora.view.figures.label.NodeLabel;

/**
 * Represents the figure of an actor which has a sexangle shape.
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             29.09.2005 SM First Version
 *                      31.03.2006 SM Comments, Bug 550, some renamed methods
 * @version             $Date: 2007-12-21 13:53:37 +0100 (Fri, 21 Dec 2007) $, $Author: treinhard $, $Revision: 3435 $
 * @responsibilities	   Represents a figure of an actor.

 */
public class ActorFigure extends CommentableNodeFigure implements INodeProperties {
	
	/**
	 * The label that shows the name of the scenario
	 */
	private NodeLabel nameLabel;
	
	/**
	 * Stores the cardinality of this component.
	 */
	private Cardinality card = new Cardinality();
	
	private NodeLabel cardinalityLabel;
	
	/**
	 * Initializes the actor figure.
	 * 
	 * @param minCardinality Min cardinality of this component.
	 * @param maxCardinality Max cardinality of this component.
	 */
	public ActorFigure(String minCardinality, String maxCardinality) {
		
		nameLabel = new NodeLabel();
		nameLabel.setBorder(new MarginBorder(5, 20, 5, 20));
		nameLabel.setHorizontalAligment(PositionConstants.CENTER);
		nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
		add(nameLabel);
		
		cardinalityLabel = new NodeLabel();
		cardinalityLabel.setVerticalAlignment(PositionConstants.TOP);
		cardinalityLabel.setHorizontalAligment(PositionConstants.CENTER);
		cardinalityLabel.setBorder(new MarginBorder(5, 5, 5, 5));
		add(cardinalityLabel);
		
		setMinCardinality(minCardinality);
		setMaxCardinality(maxCardinality);
	
	}

    /**
     * Returns <code>true</code> if the given point (x,y) is contained within this ellipse.
     * @param x the x coordinate
     * @param y the y coordinate
     * @return <code>true</code>if the given point is contained
     */
    public boolean containsPoint(int x, int y) {
        return GeometryUtils.sexangleContains(x, y, this.getBounds());
    }
    

	/**
	 * This method layouts the figure and the border.
	 */
	public void relayout(){
		layout();
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.Shape#fillShape(Graphics graphics)
	 */
	protected void fillShape(Graphics graphics) {
		
		super.fillShape(graphics);
		
		graphics.fillPolygon(GeometryUtils.getPolylineSexanglePoints(getLocation(), getSize(), lineWidth));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.Shape#outlineShape(Graphics graphics)
	 */	
	protected void outlineShape(Graphics graphics) {
		
		graphics.drawPolyline(GeometryUtils.getPolylineSexanglePoints(
				getLocation(),
				getSize(), 
				lineWidth));
		
		if(!card.isDefaultCard()) {
			
			Rectangle r = getBounds();
			// draws the specif lines of an ObjectSet
			int width = r.width - lineWidth;
			int height = r.height - lineWidth;
			// draws the rectangles
			Point point = new Point(r.x, r.y);
			Dimension dim = new Dimension(width, height);

			point.x -= 4;
			point.y -= 3;

			graphics.drawPolyline(GeometryUtils.getBorderSexangleStackPolylinePoints(
					point, dim, lineWidth));

			point.x -= 5;
			point.y -= 4;

			graphics.drawPolyline(GeometryUtils.getBorderSexangleStackPolylinePoints(
					point, dim, lineWidth));
			
		}
		
	}

		
	/*
     *  (non-Javadoc)
     * @see org.eclipse.draw2d.Figure#layout()
	 */
	public void layout() {

		super.layout();
		
		Insets newBorder = new Insets(0, 0, 0, 0);
		
		// Set: Add cardinality label
		if(!card.isDefaultCard()) {
			
			newBorder.right = 20;
			
			int width = getSize().width - newBorder.left - newBorder.right;
			
			cardinalityLabel.setSize(width, cardinalityLabel.getMinimalHeight(width));
			Point loc = getLocation().getTranslated(newBorder.left, getSize().height - cardinalityLabel.getSize().height);
			cardinalityLabel.setLocation(loc);
			
			newBorder.bottom = 20;
			
		}
		
		// Add the name label
		Point labelLoc = getLocation().getTranslated(newBorder.left, newBorder.top);
		nameLabel.setLocation(labelLoc);
		
		int width = getSize().width - newBorder.left - newBorder.right;
		int height = getSize().height - newBorder.top - newBorder.bottom;
		
		nameLabel.setText(getText());
		nameLabel.setSize(width, height);
		nameLabel.setHorizontalAligment(PositionConstants.CENTER);
		nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
	
	}
	
	/**
     * Returns the maximum cardinality.
     * 
	 * @return The maxCardinality.
	 */
	public String getMaxCardinality() {
		return card.getMaxCard();
	}

	/**
     * Sets the maximum cardinality.
     * 
	 * @param maxCardinality The maxCardinality to be set.
	 */
	public void setMaxCardinality(String maxCardinality) {
		card.setMaxCard(maxCardinality);
		
		cardinalityLabel.setVisible(!card.isDefaultCard());
		cardinalityLabel.setText(card.getMinCard() + ":" + card.getMaxCard());
	}

	/**
     * Returns the minimum cardinality.
	 * @return The minCardinality.
	 */
	public String getMinCardinality() {
		return card.getMinCard();
	}

	/**
     * Sets minimum cardinality.
     * 
	 * @param minCardinality The minCardinality to set.
	 */
	public void setMinCardinality(String minCardinality) {
		card.setMinCard(minCardinality);
		
		cardinalityLabel.setVisible(!card.isDefaultCard());
		cardinalityLabel.setText(card.getMinCard() + ":" + card.getMaxCard());
	}


    /**
     * Returns the label of this figure.
     * 
     * @return The label of this figure.
     */
	public IFigure getNameLabel() {
		return nameLabel;
	}
    
    /**
     * Returns the name of the actor.
     * 
     * @return name of the actor
     */
    public String toString() {
        return getText();
    }


	public boolean isZoomed() {

		return true;
	}
	
}