package ch.unizh.ifi.adora.view.figures;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.model.representation.Cardinality;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties;
import ch.unizh.ifi.adora.view.figures.geometry.GeometryUtils;
import ch.unizh.ifi.adora.view.figures.label.NodeLabel;

/**
 * This class is responisble for creating a visual representation of an
 * component.
 * 
 * @author 			vigan
 * @copyright 		Requirements Engineering Research Group<br>
 *            		Department of Informatics<br>
 *            		University of Zurich<br>
 * @history 		Feb 15, 2006 SM First Version 
 * 					Aug 04, 2006 JS This Figure will be filled now
 * 
 * @version 		$Date: 2008-06-17 16:01:29 +0200 (Tue, 17 Jun 2008) $, $Author: treinhard $, $Revision: 3639 $
 */
public class ComponentFigure extends StartStateFigure implements
		IZoomableFigure, INodeContainerProperties {

	/**
     * Stores the partial information.
     */
	private boolean external;

	/**
     * Stores whether this node is zoomed in (true) or zoomed out (false).
     */
	private boolean zoomed;

	/**
     * Stores the type of the
     * {@link ch.unizh.ifi.adora.model.representation.Component}.
     */
	private String type = "";

	/**
     * Stores the cardinality of this component.
     */
	private Cardinality card = new Cardinality();
	
	private boolean calculatedPartial;
	
	/**
	 * Fires a PropertyChange message, if the have insets changed.
	 */
	protected PropertyChangeSupport pcsDelegate;
	
	private int minBorder = 5;

	/**
     * The label that shows the name of this component
     */
	private NodeLabel nameLabel;
	
	private NodeLabel cardinalityLabel;
	
	private Insets border = new Insets(0, 0, 0, 0);
	


	/**
     * Intializes the abstract object figure with an xy layout.
     * 
     * @param minCardinality
     *            {@link #card}
     * @param maxCardinality
     *            {@link #card}
     * @param startState
     *            {@link #startState}
     * @param partial
     *            {@link #partial}
     * @param external
     *            {@link #external}
     */
	public ComponentFigure(String minCardinality, String maxCardinality,
			boolean startState, boolean partial, boolean external) {
		
		setLayoutManager(new XYLayout());

		pcsDelegate = new PropertyChangeSupport(this);

		nameLabel = new NodeLabel();
		nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
		nameLabel.setHorizontalAligment(PositionConstants.CENTER);
		nameLabel.setBorder(new MarginBorder(5, 5, 5, 5));
		add(nameLabel);
		
		cardinalityLabel = new NodeLabel();
		cardinalityLabel.setVerticalAlignment(PositionConstants.TOP);
		cardinalityLabel.setHorizontalAligment(PositionConstants.RIGHT);
		cardinalityLabel.setBorder(new MarginBorder(5, 5, 5, 5));
		add(cardinalityLabel);
		
		startStateIndicator = new StartStateIndicatorFigure();
		Point loc = getLocation().getTranslated(0, 5);
		startStateIndicator.setLocation(loc);
		add(startStateIndicator);
		
		setStartState(startState);
		setPartial(partial);
		setMinCardinality(minCardinality);
		setMaxCardinality(maxCardinality);
		setExternal(external);

	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties#setZoom(boolean
     *      z)
     */
	public void setZoomed(boolean z) {
		zoomed = z;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties#isZoomed()
     */
	public boolean isZoomed() {
		return zoomed;
	}


	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.draw2d.Figure#layout()
     */
	public void layout() {

		super.layout();
		
		Insets newBorder = new Insets(0, 0, 0, 0);

		// Add start state indicator if this is a start state
		if(isStartState()) {
			
			newBorder.left = startStateIndicator.getSize().width;
			
		}
		
		// Set: Add cardinality label
		if(!card.isDefaultCard()) {
			
			newBorder.right = 10;
			
			int width = getSize().width - newBorder.left - newBorder.right;
			
			cardinalityLabel.setSize(width, cardinalityLabel.getMinimalHeight(width));
			Point loc = getLocation().getTranslated(newBorder.left, getSize().height - cardinalityLabel.getSize().height);
			cardinalityLabel.setLocation(loc);
			
			newBorder.bottom = cardinalityLabel.getBounds().height;
			
		}
		
		// Add the name label
		Point labelLoc = getLocation().getTranslated(newBorder.left, newBorder.top);
		nameLabel.setLocation(labelLoc);
		
		int width = getSize().width - newBorder.left - newBorder.right;
		int height = getSize().height - newBorder.top - newBorder.bottom;
		
		nameLabel.setText(getFullNameString());
		if(getChildren().size() > 3) {
			
			nameLabel.setSize(width, nameLabel.getMinimalHeight(width));
			nameLabel.setHorizontalAligment(PositionConstants.LEFT);
			nameLabel.setVerticalAlignment(PositionConstants.TOP);
			
			if(!nameLabel.getText().equals("")) {
				newBorder.top = nameLabel.getBounds().height - minBorder - 10;
			}
		}
		else {
			nameLabel.setSize(width, height);
			nameLabel.setHorizontalAligment(PositionConstants.CENTER);
			nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
		}
		
		
		newBorder.top += minBorder;
		newBorder.left += minBorder;
		newBorder.bottom += minBorder;
		newBorder.right += minBorder;
		
		if(!newBorder.equals(border)) {

			border = newBorder;
			pcsDelegate.firePropertyChange(INSETS_CHANGED_PROP, null, newBorder);
			
		}
		
	}
	
	
	private String getFullNameString() {
		
		// Compose the name string
		String s = getText();
		if (!(type.equals(""))) {
			s = s.concat(":" + type);
		}
		if (!isZoomed() || isPartial() || isCalculatedPartial()) {
			s = s.concat("...");
		}
		if (external) {
			s = s.concat(":external");
		}
		
		return s;
		
	}
	

	/**
     * @return The {@link NodeLabel} situated inside this figure.
     */
	public IFigure getNameLabel() {

		return nameLabel;
		
	}

	/**
     * Returns the maximum cardinality.
     * 
     * @return The maxCardinality.
     */
	public String getMaxCardinality() {
		return card.getMaxCard();
	}

	/**
     * Sets the maximum cardinality.
     * 
     * @param maxCardinality
     *            The maxCardinality to be set.
     */
	public void setMaxCardinality(String maxCardinality) {
		card.setMaxCard(maxCardinality);
		
		cardinalityLabel.setVisible(!card.isDefaultCard());
		cardinalityLabel.setText(card.getMinCard() + ":" + card.getMaxCard());
	}

	/**
     * Returns the minimum cardinality.
     * 
     * @return The minCardinality.
     */
	public String getMinCardinality() {
		return card.getMinCard();
	}

	/**
     * Sets minimum cardinality.
     * 
     * @param minCardinality
     *            The minCardinality to be set.
     */
	public void setMinCardinality(String minCardinality) {
		card.setMinCard(minCardinality);
		
		cardinalityLabel.setVisible(!card.isDefaultCard());
		cardinalityLabel.setText(card.getMinCard() + ":" + card.getMaxCard());
	}

	/**
     * Sets he external boolean, and relayouts this figure.
     * 
     * @param external
     *            The external boolean to be set.
     */
	public void setExternal(boolean external) {
		this.external = external;
		relayout();
	}

	/**
     * Returns <code>true</code> if the given point (x,y) is contained within
     * this ellipse.
     * 
     * @param x
     *            the x coordinate
     * @param y
     *            the y coordinate
     * @return <code>true</code>if the given point is contained
     */
	public boolean containsPoint(int x, int y) {
		if (startState) {
			Rectangle rec = new Rectangle(getBounds());
			rec.x += startStateIndicator.getBounds().width;
			rec.width -= startStateIndicator.getBounds().width;

			return rec.contains(x, y);
		}
		return super.containsPoint(x, y);
	}

	/**
     * Returns the name of the component.
     * 
     * @return name of the state
     */
	public String toString() {
		return getText();
	}

	/**
     * Returns the {@link ch.unizh.ifi.adora.model.representation.AdoraType} of
     * this {@link ch.unizh.ifi.adora.model.representation.Component}.
     * 
     * @return The {@link ch.unizh.ifi.adora.model.representation.AdoraType} of
     *         this {@link ch.unizh.ifi.adora.model.representation.Component}.
     */
	public String getType() {
		return type;
	}

	/**
     * Sets the {@link ch.unizh.ifi.adora.model.representation.AdoraType} of
     * this {@link ch.unizh.ifi.adora.model.representation.Component} and calls
     * {@link ComponentFigure#relayout()}.
     * 
     * @param type
     *            The {@link ch.unizh.ifi.adora.model.representation.AdoraType}
     *            of this
     *            {@link ch.unizh.ifi.adora.model.representation.Component}.
     */
	public void setType(String type) {
		this.type = type;
		relayout();
	}
	
	
	public void setCalculatedPartial(boolean isPartial) {
		calculatedPartial = isPartial;
	}
	
	
	public boolean isCalculatedPartial() {
		return calculatedPartial;
	}


	/*
     * This method is overriden because of the special border of a component
     * which is a start state. The background is not as big as the whole shape,
     * so that it's position must be new calculated.
     */
	@Override
	protected void fillShape(Graphics graphics) {
		
		super.fillShape(graphics);
		
		Rectangle r = getBounds();
		int x = r.x + lineWidth / 2;
		int y = r.y + lineWidth / 2;
		int w = r.width - Math.max(1, lineWidth);
		int h = r.height - Math.max(1, lineWidth);
		
		if(isStartState()) {
			x += startStateIndicator.getSize().width;
			w -= startStateIndicator.getSize().width;
		}
		
		if(!card.isDefaultCard()) {
			
			int[] p = GeometryUtils.getPolylineObjectSetRectangle(x, y, w - lineWidth + 1, h + 1);
			graphics.fillPolygon(p);
			
			// Fill the stacked rectangles
			p = GeometryUtils.getObjectSetHiddenPolygon(x, y, w, r.height, 0, 3);
			graphics.fillPolygon(p);
			p = GeometryUtils.getObjectSetHiddenPolygon(x, y, w, r.height, 3, 6);
			graphics.fillPolygon(p);
			
		} else {
			
			graphics.fillRectangle(x, y, w, h);
			
		}
	}
	
	
	/**
	 * @see Shape#outlineShape(Graphics)
	 */
	protected void outlineShape(Graphics graphics) {
		
		Rectangle r = getBounds();
		int x = r.x + lineWidth / 2;
		int y = r.y + lineWidth / 2;
		int w = r.width - Math.max(1, lineWidth);
		int h = r.height - Math.max(1, lineWidth);
		
		if(isStartState()) {
			x += startStateIndicator.getSize().width;
			w -= startStateIndicator.getSize().width;
		}
		
		if(!card.isDefaultCard()) {
			
			int[] p = GeometryUtils.getPolylineObjectSetRectangle(x, y, w - lineWidth + 1, h + 1);
			graphics.drawPolygon(p);
			// draws the partly hidden rectangles
			p = GeometryUtils.getObjectSetHiddenPolygon(x, y, w, r.height, 0, 3);
			graphics.drawPolygon(p);

			p = GeometryUtils.getObjectSetHiddenPolygon(x, y, w, r.height, 3, 6);
			graphics.drawPolygon(p);
		
		} else {
			
			graphics.drawRectangle(x, y, w, h);
			
		}
	}
	
	
	/**
	 * Adds a PropertyChangeListener to this Border. This is used to update the Border bounds
	 * eachtime the paint method is called. 
	 */
	public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
		if (l == null) {
			throw new IllegalArgumentException();
		}
		pcsDelegate.addPropertyChangeListener(l);
	}

	/**
	 * Remove a PropertyChangeListener from this component.
	 * 
	 * @param l a PropertyChangeListener instance
	 */
	public synchronized void removePropertyChangeListener(
			PropertyChangeListener l) {
		if (l != null) {
			pcsDelegate.removePropertyChangeListener(l);
		}
	}

	/**
	 * TODO: Remove if the IZoomableNode interface is removed
	 */
	public void relayout() {
		
		layout();
		
	}

}