package ch.unizh.ifi.adora.view.figures;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLineLabel;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.linerouting.utils.HierarchyUtilities;
import ch.unizh.ifi.adora.view.anchors.ClickPointAnchor;
import ch.unizh.ifi.adora.view.figures.label.ToolTipLabel;

/**
 * This is the base class of all connection figures represented in an ADORA
 * diagram. A connection figure is a {@link org.eclipse.draw2d.PolylineConnection}
 * extended by the ability to store the information of the clickpoints, i.e. theses
 * both points where the user clicked to manifest the connection. The first clickpoint
 * represents the relative  position (i.e. the position relative to the left upper corner
 * of the figure) where the user clicked to start drawing the line.<br>
 * The second clickpoint is the relative position where the user clicked a second
 * time to finish drawing the connection.
 *      
 *
 * @author		      	smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             27.09.2005 SM First Version
 *                      12.12.2005 TR Use ratios to calculate click points
 * @version             $Date: 2009-10-30 14:58:38 +0100 (Fri, 30 Oct 2009) $, $Author: treinhard $, $Revision: 4917 $
 * @responsibilities	Encapsulates and represents the information 
 *                      about a (visualized) connection between node elements.
 *
 */
public abstract class ConnectionFigure extends PolylineConnection implements ICommentableFigure, IRoutingLine {
	
	/**
	 * Stores the key for the LINE_ROUTED property.
	 */
	public static final String LINE_ROUTED = "ConnectionFigure.routed";
	
	/**
	 * The constraints (bendpoint ratios) for the routing
	 */
	protected Object routingConstraints;
	
	/**
	 * Stores the ToolTipLabelwhich gets displayed if 
	 * one moves the mousecursor over the figure.
	 */
	private ToolTipLabel toolTipLabel;

	/**
	 * Stores the comment for this Figure.
	 */
	private String comment = "";
	
	/**
	 * The bounds of the source anchor; used to filter the anchorMoved(...)
	 * events.
	 */
	private Rectangle sourceAnchorBounds = new Rectangle();
	
	/**
	 * The bounds of the target anchor; used to filter the anchorMoved(...)
	 * events.
	 */
	private Rectangle targetAnchorBounds = new Rectangle();
	

	/**
	 * Instantiates a new ConnectionFigure and sets it's color to gray.
	 */
	public ConnectionFigure(){
		setForegroundColor(new Color(null, 125, 125, 125));
		toolTipLabel = new ToolTipLabel();
	}
	
	
	/**
	 * Sets the first click point of this connection, i.e. the point
	 * where the user clicked to begin when drawing the connection. The position
	 * of the click point is determined by the x and y ratio of the point defined
	 * by the x respectively y location inside the node divided through the width 
	 * respectively height of the source node.
	 * 
	 * @pre    0.0 <= xRatio && xRatio <= 1.0 && 0.0 <= yRatio && yRatio <= 1.0
	 * @post   true
	 * 
	 * @param  xRatio x coordinate relative to source node / width of the source node
	 * @param  yRatio y coordinate relative to source node / height of source node
	 */
	public void setSourceClickPoint(float xRatio, float yRatio) {
		
		if(getSourceAnchor() instanceof ClickPointAnchor) {
			((ClickPointAnchor)getSourceAnchor()).setClickPoint(xRatio, yRatio);
		}
		
		revalidate();
	}
	
	
	/**
	 * Sets the first click point of this connection, i.e. the point
	 * where the user clicked at the end when drawing the connection. The position
	 * of the click point is determined by the x and y ratio of the point defined
	 * by the x respectively y location inside the node devided through the width 
	 * respectively height of the target node.
	 * 
	 * @pre    0.0 <= xRatio && xRatio <= 1.0 && 0.0 <= yRatio && yRatio <= 1.0
	 * @post   true
	 * 
	 * @param  xRatio x coordinate relative to target node / width of the target node
	 * @param  yRatio y coordinate relative to target node / height of target node
	 */
	public void setTargetClickPoint(float xRatio, float yRatio) {
		
		if(getTargetAnchor() instanceof ClickPointAnchor) {
			((ClickPointAnchor)getTargetAnchor()).setClickPoint(xRatio, yRatio);
		}
		
		revalidate();
	}
	
	
	/**
	 * Returns the constraints (i.e. bendpoint ratios) for the routing of this
	 * line. This method is necessary because the routing constraints are
	 * propagated from the model to the routing algorithm before the specific
	 * router is set.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return the constraints for the routing of this line
	 */
	public Object getRoutingConstraint() {
		if(getConnectionRouter().getConstraint(this) != null) {
			return getConnectionRouter().getConstraint(this);
		}
		
		return routingConstraints;
	}
	
	
	/**
	 * Method to set the routing constraints for this line. Overrides
	 * Connection.setRoutingConstraints(...) and stores the constraints in the
	 * figure.
	 * 
	 * @pre cons != null
	 * @post getRoutingConstraint() == cons
	 * 
	 * @param cons the routing constraints for this line
	 */
	public void setRoutingConstraint(Object cons) {
		routingConstraints = cons;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.Figure#layout()
	 */
	public void layout() {

		// GEF breaks without this because the bounds may not be initialized if
		// this method hasn't been called once.
		getBounds();
		
		super.layout();
		if(getSourceAnchor().getOwner() != null && getTargetAnchor().getOwner() != null) {
			firePropertyChange(LINE_ROUTED, null, this);
		}
	}
    
    /**
     * Returns a string representation of this connection figure.
     * 
     * @return string representation of this figure
     */
    public String toString() {
        
        return "Figure of Connection from " + getSourceAnchor().getOwner() 
                  + " to " + getTargetAnchor().getOwner();
        
    }
    
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.figures.ICommentableFigure#getComment()
	 */
	public String getComment() {
		return comment;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.figures.ICommentableFigure#setComment(java.lang.String)
	 */
	public void setComment(String comment) {
		this.comment = comment;
		updateToolTip();
	}
	
	/**
	 * Updates the text on the tooltip.
	 */
	private void updateToolTip(){
		if (comment.equals("")) {
			setToolTip(null);
		} else {
			setToolTip(toolTipLabel);
			toolTipLabel.setText(comment);
		}
	}
	
	
	/**
	 * Returns the node from which the line originates.
	 * 
	 * @return the source node
	 */
	public IRoutingNode getRoutingSource() {
		
		if(getSourceAnchor().getOwner() instanceof IRoutingNode) {
			
			return (IRoutingNode)getSourceAnchor().getOwner();
			
		}
		
		return null;
		
	}
	
	
	/**
	 * Returns the node where the line ends.
	 * 
	 * @return the target node
	 */
	public IRoutingNode getRoutingTarget() {
		
		if(getTargetAnchor().getOwner() instanceof IRoutingNode) {
			
			return (IRoutingNode)getTargetAnchor().getOwner();
			
		}
		
		return null;
		
	}
	
	
	/**
	 * Returns a list containing all the bendpoints inside the line. The points 
	 * have coordinates that are relative to the first common anchestor of 
	 * the source and the target node of this line.
	 * 
	 * @return the list containing all the bendpoints in the line
	 */
	public List<java.awt.Point> getRoutingBendpoints() {
		
		return (List<Point>)routingConstraints;
		
	}
	
	
	/**
	 * Used by the routing algorithm to set the new bendpoints after the line
	 * has been routed. The points have coordinates that are relative to the 
	 * first common anchestor of the source and the target node of this line.
	 * 
	 * @param bendpoints the new bendpoints
	 */
	public void setRoutingBendpoints(List<Point> bendpoints) {
		
		routingConstraints = bendpoints;
		
		IRoutingNode parent = HierarchyUtilities.getInstance().commonAncestor(getRoutingSource(), getRoutingTarget());
		// Reflexive connection
		if(getRoutingSource() == getRoutingTarget()) {
			parent = getRoutingSource().getRoutingParent();
		}
		
		adjustStartPoint(bendpoints);
		adjustEndPoint(bendpoints);
		
		PointList plist = getPoints();
		plist.removeAllPoints();
		
		org.eclipse.draw2d.geometry.Point parentLocation = ((INestedNode)parent).getAbsoluteLocation();
		
		Iterator<Point> iter = bendpoints.iterator();
		while(iter.hasNext()) {
			
			Point p = iter.next();
			org.eclipse.draw2d.geometry.Point point = new org.eclipse.draw2d.geometry.Point(p.x, p.y);
			
			// Make the coordinates absolute
			point.translate(parentLocation);
			((Figure)parent).translateToAbsolute(point);
			
			translateToRelative(point);
			plist.addPoint(point);
			
		}
	
		setPoints(plist);
		
	}
	
	
	/**
	 * Returns the start point of the line. The coordinates are relative to
	 * the source node.
	 * 
	 * @return the start point of the line
	 */
	public Point getRoutingStartPoint() {
		
		org.eclipse.draw2d.geometry.Point start = ((ClickPointAnchor)getSourceAnchor()).getClickPoint();
		
		// TODO TR: Ugly hack to correct the start state indicator offset.
		if(getSourceAnchor().getOwner() instanceof IZoomableFigure) {
			
			if(((IZoomableFigure)getSourceAnchor().getOwner()).isStartState()) {
				start.x -= StartStateIndicatorFigure.WIDTH;
			}
			
		}
		
		return new Point(start.x, start.y);
		
	}
	
	
	/**
	 * Returns the end point of the line. The coordinates are relative to the
	 * target node.
	 * 
	 * @return the end point of the line
	 */
	public Point getRoutingEndPoint() {
		
		org.eclipse.draw2d.geometry.Point end = ((ClickPointAnchor)getTargetAnchor()).getClickPoint();
		
		
		// TODO TR: Ugly hack to correct the start state indicator offset.
		if(getTargetAnchor().getOwner() instanceof IZoomableFigure) {
			
			if(((IZoomableFigure)getTargetAnchor().getOwner()).isStartState()) {
				end.x -= StartStateIndicatorFigure.WIDTH;
			}
			
		}
		
		return new Point(end.x, end.y);
		
	}
	
	
	/**
	 * Calculates the intersection between the start node and the line and sets
	 * the first point of the line.
	 * 
	 * @param bendpoints the bendpoints inside the line
	 */
	private void adjustStartPoint(List<Point> bendpoints) {
		
		IRoutingNode parent = HierarchyUtilities.getInstance().commonAncestor(getRoutingSource(), getRoutingTarget());
		org.eclipse.draw2d.geometry.Point parentLocation = ((INestedNode)parent).getAbsoluteLocation();
		
		ClickPointAnchor sourceAnchor = (ClickPointAnchor)getSourceAnchor();
		
		org.eclipse.draw2d.geometry.Point originalClickPoint = sourceAnchor.getClickPoint();
		
		org.eclipse.draw2d.geometry.Point p = new org.eclipse.draw2d.geometry.Point(bendpoints.get(0).x, bendpoints.get(0).y);
		org.eclipse.draw2d.geometry.Point relativeAnchorLocation = sourceAnchor.getOwner().getBounds().getLocation();
		relativeAnchorLocation.translate(parentLocation.getNegated());
		p.translate(relativeAnchorLocation.getNegated());
		sourceAnchor.setClickPoint(p);
		
		org.eclipse.draw2d.geometry.Point reference = new org.eclipse.draw2d.geometry.Point(bendpoints.get(1).x, bendpoints.get(1).y);
		reference.translate(parentLocation);
		org.eclipse.draw2d.geometry.Point start = sourceAnchor.getUnscaledLocation(reference);
		
		sourceAnchor.setClickPoint(originalClickPoint);
	
		start.translate(parentLocation.getNegated());
		bendpoints.set(0, new Point(start.x, start.y));	
	}
	
	
	/**
	 * Calculates the intersection between the end node and the line and sets
	 * the last point of the line.
	 * 
	 * @param bendpoints the bendpoints inside the line
	 */
	private void adjustEndPoint(List<Point> bendpoints) {
		
		IRoutingNode parent = HierarchyUtilities.getInstance().commonAncestor(getRoutingSource(), getRoutingTarget());
		org.eclipse.draw2d.geometry.Point parentLocation = ((INestedNode)parent).getAbsoluteLocation();
		
		ClickPointAnchor targetAnchor = (ClickPointAnchor)getTargetAnchor();
		
		org.eclipse.draw2d.geometry.Point originalClickPoint = targetAnchor.getClickPoint();
		
		org.eclipse.draw2d.geometry.Point p = new org.eclipse.draw2d.geometry.Point(bendpoints.get(bendpoints.size() - 1).x, bendpoints.get(bendpoints.size() - 1).y);
		org.eclipse.draw2d.geometry.Point relativeAnchorLocation = targetAnchor.getOwner().getBounds().getLocation();
		relativeAnchorLocation.translate(parentLocation.getNegated());
		p.translate(relativeAnchorLocation.getNegated());
		targetAnchor.setClickPoint(p);
		
		org.eclipse.draw2d.geometry.Point reference = new org.eclipse.draw2d.geometry.Point(bendpoints.get(bendpoints.size() - 2).x, bendpoints.get(bendpoints.size() - 2).y);
		reference.translate(parentLocation);
		org.eclipse.draw2d.geometry.Point end = targetAnchor.getUnscaledLocation(reference);
		
		targetAnchor.setClickPoint(originalClickPoint);
			
		end.translate(parentLocation.getNegated());
		bendpoints.set(bendpoints.size() - 1, new Point(end.x, end.y));	
	}
	
	
	/**
	 * Returns the bounds of the line. The bounds are defined by the bendpoints
	 * in the line. This method doesn't (in contrast to the getBounds() method)
	 * add any decorators or labels to the bounds of the line.
	 * 
	 * @return the bounds of the line (without labels or decorators)
	 */
	public Rectangle getRealBounds() {
		
		Rectangle rect = getPoints().getBounds().getExpanded(lineWidth / 2, lineWidth / 2);
		
		return rect;
		
	}
	
	
	/**
	 * Override to filter the events so that the line is only rerouted if the
	 * bounds of the source or target anchor have really changed. The standard
	 * GEF implementation reroutes *all* lines every time the model is scrolled!
	 */
	public void anchorMoved(ConnectionAnchor anchor) {

		if (getSourceAnchor().getOwner() == null ||  getTargetAnchor().getOwner() == null )
			return;
		// The anchor may not (yet) have an owner during the initial routing
		if(getSourceAnchor().getOwner().getBounds() == null || getTargetAnchor().getOwner() == null) {
			super.anchorMoved(anchor);
			return;
		}
		
		if(anchor == getSourceAnchor() && !getSourceAnchor().getOwner().getBounds().equals(sourceAnchorBounds)) {
			super.anchorMoved(anchor);
			sourceAnchorBounds = getSourceAnchor().getOwner().getBounds().getCopy();
		}
		else if(anchor == getTargetAnchor() && !getTargetAnchor().getOwner().getBounds().equals(targetAnchorBounds)) {
			super.anchorMoved(anchor);
			targetAnchorBounds = getTargetAnchor().getOwner().getBounds().getCopy();
		}
		
	}
	
	
	public List<IRoutingLineLabel> getLineLabels() {
		
		return new ArrayList<IRoutingLineLabel>(0);
		
	}
	
}