package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Polyline;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;

/**
 *  This {@link Figure} represents the {@link Constraint}s between {@link Decision}s, 
 *  by connecting {@link Decision}s involved in the 
 *  logical expression of the {@link Constraint} with this special {@link Connection}. 
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2.12.2008 IV First Version
 */
public class ConstraintConnectionFigure extends ConnectionFigure{

	/**
	 * The radius of the {@link Ellipse}.
	 */
	public static final int ellipseDiameter  = 20;
	
	/**
	 * The {@link Ellipse} which represents the boolean operator
	 */
	private Ellipse e;
	
	/**
	 * Draws a "X" in the {@link Ellipse}.
	 */
	private final PointList pointsXOR1 = new PointList(new int[]{ellipseDiameter/4,ellipseDiameter/4, 
												3*ellipseDiameter/4,3*ellipseDiameter/4});
	
	private final PointList pointsXOR2 = new PointList(new int[]{ellipseDiameter/4,3*ellipseDiameter/4, 
												3*ellipseDiameter/4,ellipseDiameter/4});

	/**
	 * Draws a "V" in the {@link Ellipse}.
	 */
	private final PointList pointsAND = new PointList(new int[]{ellipseDiameter/4,3*ellipseDiameter/4,
												  ellipseDiameter/2,ellipseDiameter/4,
												  3*ellipseDiameter/4,3*ellipseDiameter/4});
	/**
	 * Draws a wedge inside the {@link Ellipse}.
	 */
	private final PointList pointsOR = new PointList(new int[]{ellipseDiameter/4,ellipseDiameter/4,
													ellipseDiameter/2,3*ellipseDiameter/4,
													3*ellipseDiameter/4,ellipseDiameter/4});
	

	/**
	 * True if the from variable is used as a negative literal.
	 */
	private boolean negFrom;
	
	/**
	 * True if the to variable is used as a negative literal.
	 */
	private boolean negTo;
	
	private Polyline poly;
	
	private Polyline poly2  = null;
	
	private PointList pointList;
	int opCode;
	
	/**
	 * Generates a new dashed line
	 * 
	 * @param opCode Determines whether the {@link ConstraintConnection} represents a XOR, and AND, or an OR oppeartion
	 * @param negFrom True if the from variable is used as a negative literal.
	 * @param negTo True if the to variable is used as a negative literal.
	 */
	public ConstraintConnectionFigure(int opCode, boolean negFrom, boolean negTo){
		this.opCode = opCode;
		this.negFrom = negFrom;
		this.negTo = negTo;
		poly = new Polyline();
		poly2 = new Polyline();
		setLineStyle(Graphics.LINE_DASH);
		
		e = new Ellipse();
		e.setForegroundColor(ColorConstants.black);
		e.setBackgroundColor(ColorConstants.red);
	
		poly.setForegroundColor(ColorConstants.black);
		if (opCode == ConstraintConnection.AND)
			pointList = pointsAND;
		else if (opCode == ConstraintConnection.OR)
			pointList = pointsOR;
		else {
			pointList = pointsXOR1;
			poly2.setPoints(pointsXOR2);
			poly.setFill(false);
			e.add(poly2);
		}
		
		poly.setPoints(pointList);
		e.add(poly);
		
		repaint();
	}

	/**
	 * Determines the Mid-{@link Point} of the {@link ConstraintConnectionFigure}.
	 * 
	 * @param points The {@link PointList} defining the {@link ConstraintConnectionFigure}
	 * @return The Mid-{@link Point}
	 */
	private Point getMidpoint(PointList points) {
		Point pStart = points.getFirstPoint();
		Point pEnd = points.getLastPoint();
		
		Point pMid = new Point();
		pMid.x = pStart.x + (pEnd.x - pStart.x)/2;
		pMid.y = pStart.y + (pEnd.y - pStart.y)/2;
		return 	pMid;
	}
	
	public Point getMidpoint() {
		//return getMidpoint(getPoints()).getTranslated(-ellipseDiameter/2, -ellipseDiameter/2);
		return getMidpoint(getPoints());
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.draw2d.Polyline#outlineShape(org.eclipse.draw2d.Graphics)
	 */
	protected void outlineShape(Graphics g) {
		PointList fromPoints = new PointList();
		fromPoints.addPoint(getPoints().getFirstPoint());
		fromPoints.addPoint(getMidpoint(getPoints()));
		if (negFrom) 
			g.setForegroundColor(ColorConstants.red);	
		else
			g.setForegroundColor(ColorConstants.green);
		
		g.drawPolyline(fromPoints);

		PointList toPoints = new PointList();
		toPoints.addPoint(getMidpoint(getPoints()));
		toPoints.addPoint(getPoints().getLastPoint());
		if (negTo) 
			g.setForegroundColor(ColorConstants.red);	
		else
			g.setForegroundColor(ColorConstants.green);
		g.drawPolyline(toPoints);
		
		Point midP = getMidpoint(getPoints()).getTranslated(-ellipseDiameter/2, -ellipseDiameter/2);
		Rectangle gBounds = new Rectangle();
		
		g.getClip(gBounds);
		Rectangle r = gBounds.getUnion(
				new Rectangle(midP, new org.eclipse.draw2d.geometry.Dimension(ellipseDiameter, ellipseDiameter)));
		
		g.setClip(r);
		e.setBounds(new Rectangle(midP, new org.eclipse.draw2d.geometry.Dimension(ellipseDiameter, ellipseDiameter)));
		
		PointList pl = new PointList();
		if (opCode == ConstraintConnection.AND)
			pl.addAll(pointsAND);
		else if (opCode == ConstraintConnection.OR)
			pl.addAll(pointsOR);
		else {
			pl.addAll(pointsXOR1);
			PointList pl2 = new PointList();
			pl2.addAll(pointsXOR2);
			pl2.translate(e.getLocation());
			poly2.setPoints(pl2);
			e.add(poly2);

		}
		
		pl.translate(e.getLocation());
		poly.setPoints(pl);
		poly.setForegroundColor(ColorConstants.black);
		
		e.setBackgroundColor(ColorConstants.white);
		e.setFill(true);
		e.paint(g);
	}

	public void setNegFrom(boolean negFrom) {
		this.negFrom = negFrom;
	}

	public void setNegTo(boolean negTo) {
		this.negTo = negTo;
	}

	public void setOpCode(int opCode) {
		this.opCode = opCode;
	}
}