package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.view.figures.label.NodeLabel;

/**
 * This class is a superclass for all {@link org.eclipse.draw2d.IFigure}s
 * with an exit-point attribute. It is responsible for drawing the 
 * exit-point indicator (= o->) inside the {@link org.eclipse.draw2d.IFigure}'s 
 * border.
 *
 * @author              Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             27.04.2006 SM First Version
 *                      04.08.2006 JS This Figure will be filled now
 * @version             $Date: 2008-03-06 14:44:33 +0100 (Thu, 06 Mar 2008) $, $Author: treinhard $, $Revision: 3549 $
 */
public class ExitPointFigure extends CommentableNodeFigure implements INodeProperties  {
  
	/** 
	 * The width and height radii applied to each corner. 
	 */
	protected Dimension corner = new Dimension(8, 8);
	
    /**
     * The label that shows the name of this component
     */
    private NodeLabel nameLabel;

    /**
     * Proportional factor for edge height
     */
    private final static float ARC_PROP = 0.09f;

    /**
     * Minimum height for the arc at the edges
     */
    private final static int MIN_ARC = 4;

	/**
     * Initializes the ExitPointFigure figure.
     * 
     * @param string the name of this figure.
     */
    public ExitPointFigure(String name) {

    	setLayoutManager(new XYLayout());
    	
        nameLabel = new NodeLabel();
        nameLabel.setHorizontalAligment(PositionConstants.CENTER);
        nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
        nameLabel.setBorder(new MarginBorder(5, 5, 5, 5));
        nameLabel.setVisible(true);
        add(nameLabel);
        
        setText(name);
        
   }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.draw2d.Shape#fillShape(Graphics graphics)
     */
    protected void fillShape(Graphics graphics) {
    	
    	super.fillShape(graphics);
    	
		graphics.fillRoundRectangle(getBounds(), corner.width, corner.height);;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.draw2d.Shape#outlineShape(Graphics graphics)
     */
    protected void outlineShape(Graphics graphics) {
        int radius = compArcSize(ARC_PROP, MIN_ARC);
        this.corner.width = radius;
        this.corner.height = radius;
        Rectangle f = Rectangle.SINGLETON;
    	Rectangle r = getBounds();
    	f.x = r.x + lineWidth / 2;
    	f.y = r.y + lineWidth / 2;
    	f.width = r.width - lineWidth;
    	f.height = r.height - lineWidth;
    	graphics.drawRoundRectangle(f, corner.width, corner.height);
        
		Rectangle rec = new Rectangle(
				getBounds().x + getBorderWidth(),
				getBounds().y + getBorderWidth(),
				getBounds().width - 2 * getBorderWidth(), getBounds().height - 2 * getBorderWidth());

		graphics.drawRoundRectangle(rec, getCornerDimension().width, getCornerDimension().height);
    }

    /**
     * Computes the size of the arc of an edge. The method uses the maximum of
     * the width and the height to calculate with the arcPropFactor a base
     * length for the edge arc size. arcMinSize is also added.
     * 
     * @param arcPropFactor
     *            proportional factor as a fraction of the length of the whole
     *            edge.
     * @param arcMinSize
     *            minimum size of the edge length
     */
    protected int compArcSize(float arcPropFactor, int arcMinSize) {
        int arcSize;
        // get the max size
        arcSize = Math.max(this.getSize().width - 3, this.getSize().height - 3);
        // calculate the arc length
        arcSize = (int) (arcPropFactor * arcSize) + arcMinSize;
        
		// Max size for the arcs to avoid that big parts of the shape become
		// useless border.
		if(arcSize > 25) {
			arcSize = 25;
		}
        
        return arcSize;
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.draw2d.Figure#layout()
     */
    public void layout() {

        super.layout();
        nameLabel.setText(getText());
        nameLabel.setBounds(getBounds().getCopy());

    }   

    /**
     * Returns the {@link NodeLabel} situated inside this figure.
     * 
	 * @return The {@link NodeLabel} situated inside this figure. 
	 */
	public IFigure getNameLabel() {
       return nameLabel;
    }
    
	/**
	 * Returns the width and height radii applied to each corner 
	 * of the Rectangle.
	 * 
	 * @return The width and height radii applied to each corner 
	 * of the Rectangle.
	 */
	public Dimension getCornerDimension(){
        return corner;
    }
	
	
	/**
	 * Returns the distance between the two lines of the double border.
	 * 
	 * @return distance between the two lines of the outline
	 */
	private int getBorderWidth() {
		
		int width = (int) (getBounds().width * 0.1);
		if(width > 7) {
			width = 7;
		}
		
		return width;
		
	}
	
	
	/**
	 * Sets the dimensions of each corner. This will form the radii of the arcs which form the
	 * corners.
	 *
	 * @param d the dimensions of the corner
	 * @since 2.0
	 */
	public void setCornerDimensions(Dimension d) {
		corner.width = d.width;
		corner.height = d.height;
	}

	public boolean isZoomed() {
		
		return true;
		
	}
}