package ch.unizh.ifi.adora.view.figures;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.RGB;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.view.figures.label.ToolTipLabel;

/**
 * Base class for all node figures. Handles the shading of the hierarchy and
 * the line routing stuff.
 * 
 * TODO: Extend class hierarchy with clear responsibilities.
 *
 * @author		        Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2007-11-30 TR First Version
 * @version             $Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 */
public abstract class NodeFigure extends Shape implements IRoutingNode, IPropertyChangeListener, INestedNode  {

	/**
	 * The color the node is shaded with
	 */
	private Color shadingColor;
	
	/**
	 * The font for the node label
	 */
	private Font labelFont;

	/**
	 * Stores the partial information.
	 */
	private boolean partial;

	/**
	 * Stores the name of the
	 * 
	 * @{link ch.unizh.ifi.adora.model.representation.Component}.
	 */
	private String name = "";

	/**
	 * Stores the ToolTipLabelwhich gets displayed if one moves the mousecursor
	 * over the figure.
	 */
	protected ToolTipLabel toolTipLabel;
	
	
	/**
	 * Default constructor.
	 *
	 */
	public NodeFigure() {
		
		toolTipLabel = new ToolTipLabel();
		
		AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);
		refreshFont();
		refreshShadingColor();
		
	}
	
	
	/**
	 * Returns true if this figure should be shaded.
	 * 
	 * @return true if the figure should be shaded, false otherwise
	 */
	private boolean isShaded() {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		return store.getBoolean(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY)
				&& getHierarchyLevel() % 2 == 0;
		
	}
	
	
	/**
	 * Returns an integer indicating the level on the hierarchy this figure
	 * belongs to. The adora model figure is on level one.
	 * 
	 * @return the hierarchy level this figure belongs to
	 */
	private int getHierarchyLevel() {
		
		int level = 0;
		
		IFigure parent = getParent();
		while(parent instanceof Shape) {
			level++;
			parent = parent.getParent();
		}
		
		return level;
		
	}
	
	
	/**
     * Get the font from the preference store and update the visuals
     * accordingly.
     */
	private void refreshFont() {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		Font font = new Font(null, PreferenceConverter.getFontData(store, IPreferenceNameConstants.ADORA_VIEW_NODE_LABEL_FONT));
		
		setFont(font);
		
		if(labelFont != null && !labelFont.isDisposed()) {
			labelFont.dispose();
		}
		
		labelFont = font;
		
	}
	
	/**
	 * Get the shading color from the preference store and store it.
	 */
	private void refreshShadingColor() {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		RGB color = PreferenceConverter.getColor(store, IPreferenceNameConstants.ADORA_VIEW_SHADE_COLOR);
		
		if(shadingColor != null && !shadingColor.isDisposed()) {
			shadingColor.dispose();
		}
		
		shadingColor = new Color(null, color);
		repaint();
		
	}
	
	
	/**
     * Handler method for preference changes. This method listens for font
     * property changes in the preferences.
     */
	public void propertyChange(Preferences.PropertyChangeEvent evt) {
		if (evt.getProperty().equals(
				IPreferenceNameConstants.ADORA_VIEW_NODE_LABEL_FONT)) {
			refreshFont();
		}
		else if(evt.getProperty().equals(
				IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY)) {
			repaint();
		}
		else if(evt.getProperty().equals(
				IPreferenceNameConstants.ADORA_VIEW_SHADE_COLOR)) {
			refreshShadingColor();
		}
	}
	
	
	/**
     * Returns the bounds of this node. The coordinates of the bounds have to be
     * relative to the parent of this node.
     * 
     * @return the relative bounds of the node
     */
	public java.awt.Rectangle getRoutingBounds() {

		Rectangle rect = ((INestedNode)getParent()).getRelativeBounds(this);
		
		return new java.awt.Rectangle(rect.x, rect.y, rect.width, rect.height);

	}
	
	
	/**
	 * Returns the relative coordinates of the node given as argument with
	 * respect to this node.
	 * 
	 * @param child the node to get the relative coordinates for
	 * @return the relative coordinates of the child
	 */
	public Rectangle getRelativeBounds(IFigure child) {
		
		Rectangle rect = child.getBounds().getCopy();
		rect.translate(-getBounds().x, -getBounds().y);
		
		return rect;
		
	}
	
	
	/**
	 * Returns the absolute (GEF) coordinates of this node.
	 * 
	 * @return the absolute coordinates.
	 */
	public Point getAbsoluteLocation() {
		
		return getLocation().getCopy();
		
	}
	
	

	/**
     * Returns all the children of this node. The method returns an iterator
     * over an empty collection if this node doesn't have any children.
     * 
     * @return the contained children
     */
	public Iterator<IRoutingNode> getRoutingChildren() {

		List<IRoutingNode> children = new ArrayList<IRoutingNode>();
		Iterator<Figure> figures = getChildren().iterator();
		while (figures.hasNext()) {
			Figure fig = figures.next();
			if (fig instanceof IRoutingNode) {
				children.add((IRoutingNode) fig);
			}
		}

		return children.iterator();

	}

	/**
     * Returns the parent of this node.
     * 
     * @return the parent node
     */
	public IRoutingNode getRoutingParent() {

		if (getParent() instanceof IRoutingNode) {
			return (IRoutingNode) getParent();
		}

		return null;

	}
	
	
	/**
	 * Sets the backgroud color depending on whether this node should be
	 * shaded or not.
	 * 
	 * @param graphics the graphics context to draw on
	 */
	protected void fillShape(Graphics graphics) {
		
		if(isShaded()) {
			graphics.setBackgroundColor(shadingColor);
		}
		
	}
	
	
	/**
	 * Disposes the os resources (fonts and colors).
	 */
	public void finalize() {
		
		if(labelFont != null && !labelFont.isDisposed()) {
			labelFont.dispose();
		}
		
		if(shadingColor != null && !shadingColor.isDisposed()) {
			shadingColor.dispose();
		}
		
	}

	
	/**
	 * Returns true if the node that is represented by this figure is partial,
	 * false otherwise.
	 * 
	 * @return true if the node is partial, false otherwise
	 */
	public boolean isPartial() {
		return partial;
	}


	/**
	 * Set to true if the figure should be marked as partial.
	 * 
	 * @param isPartial true if the figure is partial, false otherwise
	 */
	public void setPartial(boolean isPartial) {
		this.partial = isPartial;
		layout();
	}


	/**
	 * Sets the name of the figure.
	 * 
	 * @param name the name of the figure
	 */
	public void setText(String name) {
		this.name = name;
		updateToolTip();
		layout();
		repaint();
	}


	/**
	 * Returns the name of the figure.
	 * 
	 * @return the name of the figure.
	 */
	public String getText() {
		return name;
	}
	
	
	/**
	 * Updates the tool tip so that the name of the figure is shown in it.
	 *
	 */
	protected void updateToolTip() {
		if (name.equals("")) {
			setToolTip(null);
		} else {
			setToolTip(toolTipLabel);
			toolTipLabel.setText(name);
		}
	}
	
	@Override
	public void removeNotify() {
		AdoraPlugin.getDefault().getPluginPreferences().removePropertyChangeListener(this);
		super.removeNotify();
	}
}
