package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Shell;

import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.view.figures.label.NodeLabel;

/**
 * Represents the figure of a scenario which has a ellipse shape.
 *
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             29.09.2005 SM First Version
 * @version             $Date: 2008-04-29 11:51:04 +0200 (Tue, 29 Apr 2008) $, $Author: treinhard $, $Revision: 3627 $
 */
public class ScenarioFigure extends CommentableNodeFigure implements INodeProperties {

	/**
	 * Stores the type indicators of a scenario 
	 * (none, alternative, switch)
	 */
	private final static String typeSigns[] = { "", "o", "||", ""};

	/**
	 * Stores the sequence number.
	 */
	private int seqNr;

	/**
	 * Determins if the scenario is zommed in or not.
	 */
	private boolean zoomed = true;

	/**
	 * Determins if this scenario is an iterator.
	 */
	private boolean iteraton;
	
	/**
	 * Marks the scenario as partial if one of its direct children is not visible
	 */
	private boolean calculatedPartial;

	/**
	 * The label that shows the name of the scenario
	 */
	private NodeLabel label;

	/**
	 * Stores the scenarion type.
	 */
	private int type;

	/**
	 * Instantiates a ScenarioFigure with a given type and seqNr. 
	 * 
	 * @param type The scenarion type.
	 * @param seqNr The sequence number.
	 */
	public ScenarioFigure(int type, int seqNr, boolean iteraton) {
		this.type = type;
		this.seqNr = seqNr;
		this.iteraton = iteraton;
		
		label = new NodeLabel();

		label.setBorder(new MarginBorder(0, 0, 0, 0));
		label.setHorizontalAligment(PositionConstants.CENTER);
		label.setVerticalAlignment(PositionConstants.MIDDLE);
		
		add(label);
		
	}


	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.Figure#layout()
	 */
	public void layout() {

		super.layout();

		//		 variable for the optimal coordinates of the label
		float xopt = 0.0f;
		float yopt = 0.0f;

		// variables for the real screen coordinates (relative to the component)
		float realx = 0.0f;
		float realy = 0.0f;

		// variables for the length of the label
		float w = 0.0f;
		float h = 0.0f;

		// variable is used for calculations
		float sqrtOfTwo = (float) Math.sqrt(2.0);

		// half x axis of ellipse
		float a = ((float) (getSize().width - 2)) / 2;

		// half y axis of ellipse
		float b = ((float) (getSize().height - 2)) / 2;

		// calculate optimal left upper corner in quandrant zero
		xopt = a / sqrtOfTwo;
		yopt = b / sqrtOfTwo;

		// caluclate label component position by shifting coordinates
		realx = a - xopt;
		realy = b - yopt;

		// calculate optimal width and height of the label
		w = (getSize().width - 2) - 2 * realx;
		h = (getSize().height - 2) - 2 * realy;

		String s = getText();
		if (!isZoomed() || isPartial() || isCalculatedPartial()) {
			s = s.concat("...");
		}
		label.setText(s);
		label.setLocation(new Point((int) realx, (int) realy + 6)
				.translate(getBounds().getLocation()));
		label.setSize((int) w, (int) h);

		Dimension d = label.getPreferredSize((int) w, -1);
		if (d.height < h) {
			label.setSize((int) w, d.height);
			label.setLocation(new Point((int) realx + 1,
					(getBounds().height - d.height) / 2).translate(getBounds()
					.getLocation()));
		}
	}

	/**
	 * Used for direct editing.
	 * 
	 * @return The Label situated inside the figure.
	 */
	public IFigure getNameLabel() {
		return label;
	}
	
	
    protected void fillShape(Graphics graphics) {
    	
    	super.fillShape(graphics);
    	
		graphics.fillOval(getBounds());
    }
	

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.Shape#outlineShape(org.eclipse.draw2d.Graphics)
	 */
	protected void outlineShape(Graphics graphics) {
		String displayedStr;
		displayedStr = getType() > 0 ? typeSigns[getType()] : String
				.valueOf(getSeqNr());

		
		graphics.drawString(displayedStr,
				getLocation().x + getSize().width / 3, getLocation().y + (int)(getSize().height * 0.05));

		FontData fd = graphics.getFont().getFontData()[0];
		
		Font initialFont = graphics.getFont();
		fd.setHeight(fd.getHeight() + 5);
		Font f = new Font(null, fd);
		graphics.setFont(f);

		Shell shell = new Shell();
		
		GC gc = new GC(shell);
		
		if (isIteraton()) { 
			graphics.drawString("*",
				getLocation().x + getSize().width / 3 + gc.stringExtent(displayedStr).x, getLocation().y + (int)(getSize().height * 0.05));
		} 
		
		gc.dispose();
		shell.dispose();
		f.dispose();
		
		graphics.setFont(initialFont);
		
		Rectangle r = getBounds();
		
		int x = r.x + lineWidth / 2;
		int y = r.y + lineWidth / 2;
		int w = r.width - Math.max(1, lineWidth);
		int h = r.height - Math.max(1, lineWidth);
		
		graphics.drawOval(x, y, w, h);
	}

	/**
	 * @return Returns the type.
	 */
	public int getType() {
		return type;
	}

	/**
	 * @param type The type to set.
	 */
	public void setType(int type) {
		this.type = type;
	}

	/**
	 * @return Returns the seqNr.
	 */
	public int getSeqNr() {
		return seqNr;
	}

	/**
	 * @param seqNr The seqNr to set.
	 */
	public void setSeqNr(int seqNr) {
		this.seqNr = seqNr;
	}

	/**
	 * Returns <li>true</li>, if this Scenario is an iteration.
	 * 
	 * @return <li>true</li>, if iteraton.
	 */
	public boolean isIteraton() {
		return iteraton;
	}

	/**
	 * @param iteraton The iteraton to set.
	 */
	public void setIteraton(boolean iteraton) {
		this.iteraton = iteraton;
	}

	/**
	 * Used to determine, if "..." have to be appended to the displayed text.
	 * 
	 * @return Returns the zoomedIn.
	 */
	public boolean isZoomed() {
		return zoomed;
	}

	/**
	 * Called from the {@link ch.unizh.ifi.adora.parts.ScenarioEditPart} 
	 * after a PropertyChange happened, to repaint this {@link ScenarioFigure}.
	 * 
	 * @param zoomedIn The zoomedIn to set.
	 */
	public void setZoomed(boolean zoomed) {
		this.zoomed = zoomed;
	}
	
    /**
     * Used to mark the figure as partial if one of its children is hidden by
     * the view mechanism.
     * 
     * @param isPartial
     */
    public void setCalculatedPartial(boolean isPartial) {
    	calculatedPartial = isPartial;
    }
	
	
    /**
     * Returns true if the figure is marked as partial because one of its
     * children is hidden by the view mechanism.
     * 
     * @return
     */
	public boolean isCalculatedPartial() {
		return calculatedPartial;
	}


	/**
	 * Returns the name of the scenario.
	 * 
	 * @return name of the scenario
	 */
	public String toString() {
		return getText();
	}
	
	
	/**
	 * Returns <code>true</code> if the given point (x,y) is contained within this ellipse.
	 * @param x the x coordinate
	 * @param y the y coordinate
	 * @return <code>true</code>if the given point is contained
	 */
	public boolean containsPoint(int x, int y) {
		if (!super.containsPoint(x, y))
			return false;
		Rectangle r = getBounds();
		long ux = x - r.x - r.width / 2;
		long uy = y - r.y - r.height / 2;
		return ((ux * ux) << 10) / (r.width * r.width) 
			 + ((uy * uy) << 10) / (r.height * r.height) <= 256;
	}
}