package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Base class for those node figures that can be marked as start states.
 *
 * @author		        Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2007-12-21 TR First Version
 * @version             $Date: 2008-06-13 15:23:55 +0200 (Fri, 13 Jun 2008) $, $Author: treinhard $, $Revision: 3638 $
 */
public abstract class StartStateFigure extends CommentableNodeFigure {

	/**
	 * Stores whether this figure represents a start state or not.
	 */
	protected boolean startState;
	
	/**
	 * The figure that draws the start state indicator (the circle with the
	 * outgoing arrow).
	 */
	protected StartStateIndicatorFigure startStateIndicator;

	
	/**
	 * Returns true if this component figure represents a start state.
	 * 
	 * @return true if the component is a start state, false otherwise
	 */
	public boolean isStartState() {
		return startState;
	}

	
	/**
	 * Marks this component figure as a start state.
	 * 
	 * @param startState true if the figure should be marked as start state,
	 *        false otherwise
	 */
	public void setStartState(boolean startState) {
		
		this.startState = startState;
		startStateIndicator.setVisible(startState);
		
	}

	
	/**
     * Returns the bounds of this node. The coordinates of the bounds have to be
     * relative to the parent of this node.
     * 
     * @return the relative bounds of the node
     */
	public java.awt.Rectangle getRoutingBounds() {

		java.awt.Rectangle bounds = super.getRoutingBounds();

		if(isStartState()) {
			bounds.x += startStateIndicator.getSize().width;
			bounds.width -= startStateIndicator.getSize().width;
		}
		
		return bounds;
	}
	
	
	/**
	 * Returns the relative coordinates of the node given as argument with
	 * respect to this node. Overriden to take the start state indicator
	 * into account.
	 * 
	 * @param child the node to get the relative coordinates for
	 * @return the relative coordinates of the child
	 */
	public Rectangle getRelativeBounds(IFigure child) {
		
		Rectangle rect = super.getRelativeBounds(child);
		if(isStartState()) {
			rect.translate(-startStateIndicator.getSize().width, 0);
		}
		
		return rect;
		
	}
	
	
	/**
	 * Returns the absolute (GEF) coordinates of this node. Overriden to take 
	 * the start state indicator into account.
	 * 
	 * @return the absolute coordinates.
	 */
	public Point getAbsoluteLocation() {
		
		Point location = getLocation().getCopy();
		if(isStartState()) {
			location.translate(startStateIndicator.getSize().width, 0);
		}
		return location;
		
	}
	

}
