package ch.unizh.ifi.adora.view.figures;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties;
import ch.unizh.ifi.adora.view.figures.label.NodeLabel;

/**
 * Represents the figure of a state which has a rounded rectangleshape.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             24.07.2006 IV First Version
 *                      04.08.2006 JS This Figure will be filled now
 * @version             $Date: 2008-06-17 16:01:29 +0200 (Tue, 17 Jun 2008) $, $Author: treinhard $, $Revision: 3639 $
 */
public class StateFigure extends StartStateFigure implements IZoomableFigure,
		INodeContainerProperties  {

	/** 
	 * The width and height radii applied to each corner. 
	 */
	protected Dimension corner = new Dimension(8, 8);
	
    /**
	 * Stores whether this node is zoomed in (true) or zoomed out (false).
	 */
	private boolean zoomed;

	/**
     * The label that shows the name of this component
     */
    private NodeLabel nameLabel;

    /**
	 * Proportional factor for edge height
	 */
	public final static float ARC_PROP = 0.09f;

	/**
	 * Minimum height for the arc at the edges
	 */
	public final static int MIN_ARC = 4;
    
    /**
	 * Fires a PropertyChange message, if the have insets changed.
	 */
	protected PropertyChangeSupport pcsDelegate;
	
	private int minBorder = 5;
	
	private Insets border = new Insets(0, 0, 0, 0);


    /**
     * Instantiates a State. 
     *
     * @param startState Determines whether this node is a startstate or not.
     * @param partial Is the state partial.
     */
	public StateFigure(boolean startState, boolean partial) {
		
		pcsDelegate = new PropertyChangeSupport(this);
		
		setLayoutManager(new XYLayout());	
        
		nameLabel = new NodeLabel();
		nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
		nameLabel.setHorizontalAligment(PositionConstants.CENTER);
		nameLabel.setBorder(new MarginBorder(5, 5, 5, 5));
		add(nameLabel);
        
		startStateIndicator = new StartStateIndicatorFigure();
		Point loc = getLocation().getTranslated(0, 5);
		startStateIndicator.setLocation(loc);
		add(startStateIndicator);
        
        setStartState(startState);
        setPartial(partial);
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.Shape#fillShape(Graphics graphics)
	 */
	protected void fillShape(Graphics graphics) {
		
		super.fillShape(graphics);
		
		int radius = compArcSize(ARC_PROP, MIN_ARC);
		this.corner.width = radius;
		this.corner.height = radius;
		
		Rectangle f = Rectangle.SINGLETON;
		Rectangle r = getBounds();
		f.x = r.x + lineWidth / 2;
		f.y = r.y + lineWidth / 2;
		f.width = r.width - lineWidth;
		f.height = r.height - lineWidth;
		
		if(isStartState()) {
			f.x += startStateIndicator.getSize().width;
			f.width -= startStateIndicator.getSize().width;
		}
		
		graphics.fillRoundRectangle(f, corner.width, corner.height);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.Shape#outlineShape(Graphics graphics)
	 */
	protected void outlineShape(Graphics graphics) {
		int radius = compArcSize(ARC_PROP, MIN_ARC);
		this.corner.width = radius;
		this.corner.height = radius;
		
		Rectangle f = Rectangle.SINGLETON;
		Rectangle r = getBounds();
		f.x = r.x + lineWidth / 2;
		f.y = r.y + lineWidth / 2;
		f.width = r.width - lineWidth;
		f.height = r.height - lineWidth;
		
		if(isStartState()) {
			f.x += startStateIndicator.getSize().width;
			f.width -= startStateIndicator.getSize().width;
		}
		
		graphics.drawRoundRectangle(f, corner.width, corner.height);
		
		
	}

	/**
	 * Computes the size of the arc of an edge. The method uses the maximum of
	 * the width and the height to calculate with the arcPropFactor a base
	 * length for the edge arc size. arcMinSize is also added.
	 * 
	 * @param arcPropFactor
	 *            proportional factor as a fraction of the length of the whole
	 *            edge.
	 * @param arcMinSize
	 *            minimum size of the edge length
	 */
	protected int compArcSize(float arcPropFactor, int arcMinSize) {
		int arcSize;
		// get the max size
		arcSize = Math.max(this.getSize().width - 3, this.getSize().height - 3);
		// calculate the arc length
		arcSize = (int) (arcPropFactor * arcSize) + arcMinSize;
		
		// Max size for the arcs to avoid that big parts of the shape become
		// useless border.
		if(arcSize > 25) {
			arcSize = 25;
		}
		
		return arcSize;
	}
	

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties#setZoom(boolean
	 *      z)
	 */
	public void setZoomed(boolean z) {
		zoomed = z;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties#isZoomed()
	 */
	public boolean isZoomed() {
		return zoomed;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.Figure#layout()
	 */
	public void layout() {

	    super.layout();
	    
		Insets newBorder = new Insets(0, 0, 0, 0);

		// Add start state indicator if this is a start state
		if(isStartState()) {
			
			newBorder.left = startStateIndicator.getSize().width;
			
		}
        
        String s = getText();
        if(!isZoomed() || isPartial()) {
            s = s.concat("...");
        }
        
		// Add the name label
		Point labelLoc = getLocation().getTranslated(newBorder.left, newBorder.top);
		nameLabel.setLocation(labelLoc);
		
		int width = getSize().width - newBorder.left - newBorder.right;
		int height = getSize().height - newBorder.top - newBorder.bottom;
		
		nameLabel.setText(s);
		if(getChildren().size() > 2) {
			
			nameLabel.setSize(width, nameLabel.getMinimalHeight(width));
			nameLabel.setHorizontalAligment(PositionConstants.LEFT);
			nameLabel.setVerticalAlignment(PositionConstants.TOP);
			
			if(!nameLabel.getText().equals("")) {
				newBorder.top = nameLabel.getBounds().height - minBorder - 10;
			}
		}
		else {
			nameLabel.setSize(width, height);
			nameLabel.setHorizontalAligment(PositionConstants.CENTER);
			nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
		}
		
		
		newBorder.top += minBorder;
		newBorder.left += minBorder;
		newBorder.bottom += minBorder;
		newBorder.right += minBorder;
		
		if(!newBorder.equals(border)) {

			border = newBorder;
			pcsDelegate.firePropertyChange(INSETS_CHANGED_PROP, null, newBorder);
			
		}
    }
	
	
    /**
     * This method layouts the figure and the border.
     */
    public void relayout(){
        layout();
    }

    
    /**
     * Returns the label of this figure.
     * 
     * @return The label of this figure.
     */
	public IFigure getNameLabel() {
        return nameLabel;
    }

    
    /**
     * Returns <code>true</code> if the given point (x,y)
     *  is contained within this Rounded Rectangle.
     * @param x the x coordinate
     * @param y the y coordinate
     * @return <code>true</code>if the given point is contained
     */
    public boolean containsPoint(int x, int y) {
        if(startState){
            Rectangle rec = new Rectangle(getBounds());
            rec.x += startStateIndicator.getBounds().width;
            rec.width -= startStateIndicator.getBounds().width;
            
            return rec.contains(x,y);            
        }
        return super.containsPoint(x,y);
    }
    
    /**
     * Returns the name of the state.
     * 
     * @return name of the state
     */
    public String toString() {
        return getText();
    }
	
	
	/**
	 * Adds a PropertyChangeListener to this Border. This is used to update the Border bounds
	 * eachtime the paint method is called. 
	 */
	public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
		if (l == null) {
			throw new IllegalArgumentException();
		}
		pcsDelegate.addPropertyChangeListener(l);
	}

	/**
	 * Remove a PropertyChangeListener from this component.
	 * 
	 * @param l a PropertyChangeListener instance
	 */
	public synchronized void removePropertyChangeListener(
			PropertyChangeListener l) {
		if (l != null) {
			pcsDelegate.removePropertyChangeListener(l);
		}
	}

	
	/**
	 * Sets the dimensions of each corner. This will form the radii of the arcs which form the
	 * corners.
	 *
	 * @param d the dimensions of the corner
	 * @since 2.0
	 */
	public void setCornerDimensions(Dimension d) {
		corner.width = d.width;
		corner.height = d.height;
	}

	/**
	 * Returns always true because no child of a state can be filtered out
	 * by the view mechanism.
	 * 
	 * @return true
	 */
	public boolean isCalculatedPartial() {
		return false;
	}

	/**
	 * Doesn't do anything because no child of a state can be filtered out
	 * by the view mechanism.
	 */
	public void setCalculatedPartial(boolean isPartial) {
		
		// Don't do anything because no child of a state can be filtered out
		// by the view mechanism.
		
	}
}