package ch.unizh.ifi.adora.view.figures;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLineLabel;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.linerouting.utils.HierarchyUtilities;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.view.layers.MultipleRoutersConnectionLayer;

/**
 * This object is responsible for creating a visual connection between two
 * objects, connected via an transition. 
 * Furthermore it stores the {@link LabelPlacementAlgorithm} used to position the
 * Transition description represented in the 
 * {@link ch.unizh.ifi.adora.view.figures.label.ConnectionLabel}.     
 *
 * @author		        smeier, 
 * 						vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             27.09.2005 SM First Version
 *                      07.09.2006 JS the figure is layouted when the size of the 
 *                                    arrow changes
 * @version             $Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 */
public class TransitionFigure extends LabeledConnection implements IPropertyChangeListener, IRoutingNode {

	/**
	 * Stores the {@link PolylineDecoration} 
	 * used to position the arrowhead of this transition..
	 */
	private PolylineDecoration decoration;
	
	/**
	 * Constructs a new TransitionFigure.
	 */
	public TransitionFigure() {
		
		decoration = new PolylineDecoration();
		
		AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);
		
		IPreferenceStore store  = AdoraPlugin.getDefault().getPreferenceStore();
		int size = store.getInt(IPreferenceNameConstants.ADORA_VIEW_TRANSITION_ARROW_SIZE);
		decoration.setScale(size, size / 2);
		
		setTargetDecoration(decoration); 
		
	}
	
	/**
	 * Handler method for preference changes.
	 */
	public void propertyChange(Preferences.PropertyChangeEvent evt) {
		
		if(evt.getProperty().equals(IPreferenceNameConstants.ADORA_VIEW_TRANSITION_ARROW_SIZE)) {		
			IPreferenceStore store  = AdoraPlugin.getDefault().getPreferenceStore();
			int size = store.getInt(IPreferenceNameConstants.ADORA_VIEW_TRANSITION_ARROW_SIZE);
			decoration.setScale(size, size / 2);
			layout();
		}
	}
	
	
	/**
	 * Returns the parent of this node or null if this is the root node.
	 * 
	 * @return the parent node
	 */
	public IRoutingNode getRoutingParent() {
		
		IRoutingNode parent = HierarchyUtilities.getInstance().commonAncestor(getRoutingSource(), getRoutingTarget());
		
		return parent;
	}
	
	
	/**
	 * The method returns an iterator over an empty collection because a
	 * line doesn't have any children.
	 * 
	 * @return the contained children
	 */
	public Iterator<IRoutingNode> getRoutingChildren() {
		
		return new ArrayList<IRoutingNode>().iterator();
		
	}
	
	
	/**
	 * Returns the bounds of this line. The coordinates of the bounds are 
	 * relative to the first common ancestor of the source and target node
	 * of this line (i.e. the node this line is completely contained in).<br>
	 * 
	 * The bounds are required the calculate the clickpoint if this line is
	 * the target of another line.
	 * 
	 * @return the relative bounds of the line
	 */
	public java.awt.Rectangle getRoutingBounds() {
		
		if(getRoutingBendpoints() == null) {
			return new java.awt.Rectangle(getRealBounds().x, getRealBounds().y, getRealBounds().width, getRealBounds().height);
		}
		
		// Use the unscaled bendpoints that are calculated by the router.
		java.awt.Point p = getRoutingBendpoints().get(0);
		
		java.awt.Rectangle rect = new java.awt.Rectangle(p.x, p.y, 0, 0);
		
		Iterator<java.awt.Point> iter = getRoutingBendpoints().iterator();
		while(iter.hasNext()) {
			rect.add(iter.next());
		}
		
		
		if(getRoutingParent() instanceof MultipleRoutersConnectionLayer) {
			
			if(getRoutingParent().getRoutingBounds().x < 0) {
				int x = rect.x - getRoutingParent().getRoutingBounds().x;
				rect.setLocation(x, rect.y);
			}
			
			if(getRoutingParent().getRoutingBounds().y < 0) {
				int y = rect.y - getRoutingParent().getRoutingBounds().y;
				rect.setLocation(rect.x, y);
			}
			
		}
		
		return rect;
		
	}

	public boolean isZoomed() {

		return true;
	}
	
	
	public List<IRoutingLineLabel> getLineLabels() {
		
		List<IRoutingLineLabel> labels = new ArrayList<IRoutingLineLabel>(1);
		for(Object child: getChildren()) {
			if(child instanceof IRoutingLineLabel) {
				labels.add((IRoutingLineLabel)child);
			}
			
		}
		return labels;
		
	}
	
	@Override
	public void removeNotify() {
		AdoraPlugin.getDefault().getPluginPreferences().removePropertyChangeListener(this);
		super.removeNotify();
	}
	
}