package ch.unizh.ifi.adora.view.figures.geometry;

import java.awt.geom.Line2D;

import org.eclipse.draw2d.Polygon;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.parts.geometryfactory.GeometryFactory;
/**
 * Provides several geometrical functions to perform for 
 * drawing shapes of a diagram.
 *      
 * @author              smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             30.08.2005 SM First Version
 *                      31.03.2006 SM Comments, Bug 550, some renamed methods 
 *                      28.06.2006 JS changed intersectEllipse(ellipse, inside, outside) 
 *                                    to behave properly when inside and outside have the
 *                                    same x coordinate; added solveQuadraticEquation method
 *                      13.07.2006 JS Bug 596: reimplemented the methods 
 *                                    intersectBeveledRectangle and intersectSextangle, 
 *                                    added intersectPolyline and beveledRectangleContains, 
 *                                    fixed distance
 *                      20.07.2006 JS fixed intersectPolyline to return the 
 *                                    correct corner if more than one corner lies on the 
 *                                    intersecting line
 *                      26.07.2006 JS comments, bug 596: intersectPolyline doesn't require
 *                                    any more that one point be inside of the polygon
 *                                 JS bug 684: different formula for the sexangle to avoid
 *                                    candy-shaped actors
 * @version             $Date: 2007-12-21 11:50:59 +0100 (Fri, 21 Dec 2007) $, $Author: treinhard $, $Revision: 3434 $
 *                 
 * @responsibilities    Contains several geometrical functions.
 */
public class GeometryUtils {

    /**
     * Returns a polyline in form of a beveled rectangle, 
     * used by the Aspect Containers.
     *
     * @param upperLeftCorner
     * @param d the size of the bounding rectangle
     * @param lineWidth The thickness of the line
     * 
     * @return array of integers [ firstX, firstY, secondX, secondY, ... ]
     */
    public static int[] getPolylineBeveledRectanglePoints(Point upperLeftCorner, Dimension d) {
        int absoluteX = upperLeftCorner.x;
        int absoluteY = upperLeftCorner.y;
        
        int width = d.width;
        int height = d.height;
        
        int deltaX = width/8;
        int deltaY = height/4;
        
        if(deltaX > 40) {
        	deltaX = 40;
        }
        
        if(deltaY > 70) {
        	deltaY = 70;
        }
        
        
        int[] p = {
        		absoluteX + deltaX, absoluteY,
                absoluteX + width, absoluteY,
                absoluteX + width, absoluteY + height - deltaY,
                absoluteX + width - deltaX, absoluteY + height,
                absoluteX, absoluteY + height,
                absoluteX, absoluteY + deltaY,
                absoluteX + deltaX, absoluteY
         };
        
        return p;
    }
    
    /**
     * Computes if a given point by x and y corrdinate is contained in a 
     * beveled rectangle figure which is defined by the beveledRectangleBounds.
     * 
     * @param x The x corrdinate of the point which is potentially in the beveled rectangle.
     * @param y The y corrdinate of the point which is potentially in the beveled rectangle.
     * @param beveledRectangleBounds The bounds of the beveled rectangle.
     * @return Returns true if the point given by x and y is contained in the 
     *         beveled rectangle defined by the given beveledRectangleBounds.
     */
    public static boolean beveledRectangleContains(int x, int y, Rectangle beveledRectangleBounds) {
        Polygon poly = new Polygon();
        poly.setPoints(new PointList(getPolylineBeveledRectanglePoints(beveledRectangleBounds.getLocation(), beveledRectangleBounds.getSize())));
        
        return poly.containsPoint(x,y);
    }
    
    
    /**
     * Computes the intersection of a beveled rectangle with a line. 
     * The line must be defined by an outsidePoint, i.e. a point that is
     * located outside of the shape and an insidePoint, i.e. a point that
     * is located inside of the shape or on the other side of the shape. 
     * The bounds denotes the rectilinear area around the shape. 
     * 
     * @param bounds the bounds of the beveled rectangle
     * @param insidePoint  a point that inside beveled rectangle or on the opposite side of it
     * @param outsidePoint a point that must be outside the beveled rectangle
     * @return Returns a point which denotes the insersection between
     *         the sexangle and a line, or null if there is no intersection
     */
    public static Point intersectBeveledRectangle(Rectangle bounds, Point insidePoint,
            Point outsidePoint) {

		return intersectPolyline(getPolylineBeveledRectanglePoints(bounds.getLocation(), bounds.getSize()), insidePoint, outsidePoint);
    }
            
    
    /**
     * Returns the polyline coordinates for drawing a sexangle.
     *  
     * @param upperLeftCorner The location of the sexangle.
     * @param d The rectilinear dimension of the sexangle.
     * @param lineWidth The thickness of the line. 
     * @return Returns an array containing (x,y) pair values that 
     *         represent the corners of the sexangle.
     */
    public static int[] getPolylineSexanglePoints(Point upperLeftCorner, Dimension d,
            int lineWidth) {
        int relativeX = lineWidth / 2;
        int relativeY = lineWidth / 2;
        int width = d.width - lineWidth;
        int height = d.height - lineWidth;
        int bend = width/3;
        if(bend > 25) {
        	bend = 25;
        }
        
        int mid1 = relativeX + bend;//height / 2; // relative x position of the left upper/lower corner
        int mid3 = relativeX + width - bend;//width - mid1; // relative x position of the right upper/lower corner
        int midY = height / 2; // relative y position of the left corner

        int absoluteX = upperLeftCorner.x;
        int absoluteY = upperLeftCorner.y;

        int[] p = { 
        		relativeX + absoluteX, midY + absoluteY, 
        		mid1 + absoluteX, relativeY + absoluteY, 
        		mid3 + absoluteX, relativeY + absoluteY,
                width + absoluteX, midY + absoluteY, 
                mid3 + absoluteX, height + absoluteY, 
                mid1 + absoluteX, height + absoluteY,
                relativeX + absoluteX, midY + absoluteY
        };
        
        return p;
    }
    
    /**
     * Returns the border polyline of a sexangle stack.
     * 
     * @param upperLeftCorner The location of the sexangle stack.
     * @param d The dimension of the sexangle stack.
     * @param lineWidth The line width of the sexangle stack.
     * @return Returns an array of int containing a set of point pairs (x,y) where
     *         x denotes the x coordinate and y the y coordinate of a 
     *         corner point of the sexangle stack figure.
     */
	public static int[] getBorderSexangleStackPolylinePoints(Point upperLeftCorner, Dimension d,
			int lineWidth) {
	
		int width = d.width - lineWidth;
		int height = d.height - lineWidth;
        int bend = width/3;
        if(bend > 25) {
        	bend = 25;
        }
        int mid1 = bend;//height / 2; // relative x position of the left upper/lower corner
        int mid3 = width - bend;//width - mid1; // relative x position of the right upper/lower corner
        int midY = height / 2; // relative y position of the left corner

		int absoluteX = upperLeftCorner.x;
		int absoluteY = upperLeftCorner.y;

		int[] p = {
				mid1 + absoluteX, height + absoluteY, 
				mid3 + absoluteX, height + absoluteY, 
				width + absoluteX, midY + absoluteY
		};
		return p;
	}

    /**
     * Computes if a given point by x and y corrdinate is contained in a 
     * sexangle figure which is defined by the sexangleBounds.
     * 
     * @param x The x corrdinate of the point which is potentially in the sexangle.
     * @param y The y corrdinate of the point which is potentially in the sexangle.
     * @param sexangleBounds The bounds of the sexangle.
     * @return Returns true if the point given by x and y is contained in the 
     *         sexangle defined by the given sexangleBounds.
     */
    public static boolean sexangleContains(int x, int y, Rectangle sexangleBounds) {

        int xPos = sexangleBounds.x;
        int yPos = sexangleBounds.y;
        double midY = sexangleBounds.height / 2;
        int bend = sexangleBounds.width/3;
        if(bend > 25) {
        	bend = 25;
        }
        double six1X = bend; //midY;
        double six5X = sexangleBounds.width - bend; //sexangleBounds.width - six1X;
        boolean contains = false;
        
        if (x >= six1X + xPos && x <= six5X + xPos && 
                y > 0 + yPos && y <= sexangleBounds.height + yPos) {
        	// point is in the rectangle in the middle of he sexangle
            contains = true;
        } else if (x <= 0 + xPos || x >= sexangleBounds.width + xPos || 
                   y <= 0 + yPos || y >= sexangleBounds.height + yPos) {
        	// point is outside/exactly on the bounds of the rectangle
            contains = false; // to prevent divide by zero
        } else {
            double slope = (midY / six1X);
            if (x < six1X + xPos) {
                contains = Math.abs((y - yPos - midY) / (x - xPos)) < slope;
            } else {
                contains = Math.abs((y - yPos - midY) / ((sexangleBounds.width) - x + xPos)) < slope;
            }
        }
        return contains;
    }
    
    /**
     * Computes the intersection of a rectangle with a line. 
     * The line must be defined by an outside point, i.e. a point that is
     * located outside of the shape and an inside point, i.e. a point that
     * is located inside of the shape or on the other side of the shape. 
     * The bounds denotes the rectilinear area around the shape. 
     * 
     * @param rect the bounds of the rectangle
     * @param inside  a point inside the rectangle or on the opposite side of it
     * @param outside a point that must be outside the rectangle
     * @return Returns a point which denotes the insersection between
     *         the rectangle and a line, or null if there is no intersection
     */
    public static ch.unizh.ifi.adora.model.geometry.Point intersectRectangle(
                                    ch.unizh.ifi.adora.model.geometry.Rectangle rect, 
                                    ch.unizh.ifi.adora.model.geometry.Point inside, 
                                    ch.unizh.ifi.adora.model.geometry.Point outside) {
        
        
        org.eclipse.draw2d.geometry.Rectangle draw2dRect =  
            (org.eclipse.draw2d.geometry.Rectangle)
                GeometryFactory.getDraw2dElement(rect);
        
        org.eclipse.draw2d.geometry.Point draw2dInside =  
            (org.eclipse.draw2d.geometry.Point) 
                GeometryFactory.getDraw2dElement(inside);

        org.eclipse.draw2d.geometry.Point draw2dOutside = 
            (org.eclipse.draw2d.geometry.Point) 
                GeometryFactory.getDraw2dElement(outside);

        
        org.eclipse.draw2d.geometry.Point draw2dReturn = 
            GeometryUtils.intersectRectangle(draw2dRect,draw2dInside,draw2dOutside);
        
        return (ch.unizh.ifi.adora.model.geometry.Point)
            GeometryFactory.getAdoraElement(draw2dReturn);
                
 
    }

    /**
     * Computes the intersection of a sexangle with a line. 
     * The line must be defined by an outsidePoint, i.e. a point that is
     * located outside of the shape and an insidePoint, i.e. a point that
     * is located inside or on the other side of the shape. The bounds 
     * denotes the rectilinear area around the sexangle. 
     * 
     * @param bounds the bounds of the sexangle
     * @param insidePoint  a point that inside sexangle or on the opposite side of it
     * @param outsidePoint a point that must be outside the sexangle
     * @return Returns a point which denotes the insersection between
     *         the sexangle and a line, or null if there is no intersection
     */
	public static Point intersectSextangle(Rectangle bounds, Point insidePoint,
			Point outsidePoint) {

		return intersectPolyline(getPolylineSexanglePoints(bounds.getLocation(), bounds.getSize(), 0), insidePoint, outsidePoint);
	}

	/**
	 * Computes the line intersection between two lines and returns it, if there
	 * is one, otherwise a null reference is returned.
	 * 
	 * @param p1
	 *            start point of line 1
	 * @param p2
	 *            end point of line 1
	 * @param p3
	 *            start point of line 2
	 * @param p4
	 *            end point of line 2
	 * @return Returns a new Point object of the point of intersection. If no
	 *         such point exists, a null reference will be returned.
	 */
	public static Point intersectLines(final Point p1, final Point p2,
			final Point p3, final Point p4) {

		/*
		 * Implementation taken from
		 * http://forum.java.sun.com/thread.jsp?thread=179628&forum=20&message=747493
		 */

		final Line2D.Float line1 = new Line2D.Float(p1.x, p1.y, p2.x, p2.y);
		final Line2D.Float line2 = new Line2D.Float(p3.x, p3.y, p4.x, p4.y);

		if (!line1.intersectsLine(line2)) {
			return null;
		}

		final double px = line1.getX1(), py = line1.getY1(), rx = line1.getX2()
				- px, ry = line1.getY2() - py;

		final double qx = line2.getX1(), qy = line2.getY1(), sx = line2.getX2()
				- qx, sy = line2.getY2() - qy;

		final double det = sx * ry - sy * rx;
		if (det == 0) {
			return null;
		} else {
			double z = (sx * (qy - py) + sy * (px - qx)) / det;
			if (z == 0 || z == 1) {
				return null; // intersection at end point!
			}

			final int x = (int) (px + z * rx);
			final int y = (int) (py + z * ry);
			return new Point(x, y);
		}
	}

    /**
     * Returnst the Euclid's distance between two points. 
     * @param p1 Point 1 for computing the distance.
     * @param p2 Point 2 for computing the distance
     * @return Returns the distance in points.
     */
	public static double distance(Point p1, Point p2) {
		long dx = (p2.x - p1.x);
		long dy = (p2.y - p1.y);
		return Math.sqrt(dx * dx + dy * dy);
	}

    /**
     * Returns a polyline describing the top object shape of a 
     * object set shape.
     * 
     * @param x The left upper corner x coordinate of the shape.
     * @param y The left upper corner y coordinate of the shape.
     * @param width The width of the object set.
     * @param height The height of the object set.
     * @return Returnst the poly line for the top object of an
     *         object set shape. 
     */
	public static int[] getPolylineObjectSetRectangle(int x, int y, int width,
			int height) {
		// draws the specif lines of an ObjectSet
		int w = width - 8;
		int h = height - 8;

		// draws the rectangles
		int[] p = { 
				1 + x, 1 + y, 
				1 + x, h + y, 
				w + x, h + y, 
				w + x, 1 + y 
		};
		return p;
	}

    /**
     * Returns a polyline describing the hidden objects shape.
     * 
     * @param x The left upper corner x coordinate of the shape.
     * @param y The left upper corner y coordinate of the shape.
     * @param width The width of the object set.
     * @param height The height of the object set.
     * @param d0 Denotes the distance between the top object and the second
     *           object shape on the stack.
     * @param d1 Denotes the distance between the second object and the 
     *           third object shape on the stack.
     * @return Returns the shape of two partially hidden objects on an object
     *         set.
     */
	public static int[] getObjectSetHiddenPolygon(int x, int y, int width,
			int height, int d0, int d1) {
		// draws the specif lines of an ObjectSet
		int w = width - 8;
		int h = height - 8;

		// draws the partly hidden rectangles
		int[] p = { 
				d1 + x, h + d0 + y, 
				d1 + x, h + d1 + y, 
				w + d1 + x, h + d1 + y, 
				w + d1 + x, d1 + y, 
				w + d0 + x, d1 + y, 
				w + d0 + x, h + d0 + y
		};
		return p;
	}

	/**
	 * Computes the intersection of a line with an ellipse.
	 * <p>
	 * The line goes from the point <code>inside</code> to the point
	 * <code>outside</code>. The rectangle <code>ellipse</code> describes
	 * the outer bounds of the ellipse.
	 * 
	 * @param ellipse
	 *            the outer bounds of the ellipse.
	 * @param inside
	 *            ending point of the line inside of the ellipse.
	 * @param outside
	 *            ending point of the line outside of the ellipse.
	 * @return the intersection point of the line with the ellipse. Null, if
	 *         there's no intersection point.
	 * @throws NullPointerException
	 *             if any of the parameters is null.
	 */
	public static Point intersectEllipse(final Rectangle ellipse,
			final Point inside, final Point outside)
			throws NullPointerException {

		if (ellipse == null)
			throw new NullPointerException("parameter ellipse must not be null");
		if (inside == null)
			throw new NullPointerException("parameter inside must not be null");
		if (outside == null)
			throw new NullPointerException("parameter outside must not be null");

		final Point convIn = convertToMathCoord(ellipse.getCenter(), inside);
		final Point convOut = convertToMathCoord(ellipse.getCenter(), outside);

		// Determine constants for the equation.
		final float a = ellipse.width / 2f;
		final float b = ellipse.height / 2f;

		final float deltaY = convOut.y - convIn.y;
		float deltaX = convOut.x - convIn.x;
		
		if (deltaX == 0) // If the intersection line has an infinite slope
		{
			deltaX = 1.0f;
			
			double x = convIn.x; // X coordinate of intersection point
			double y; // Y coordinate of intersection point
			
			// Solve the quadratic equation
			final float qa = 1f / (b * b);
			final float qb = 0;
			final float qc = (float) ((x *  x) / (a * a)) -1f;
			double intersectionYValues[] = solveQuadraticEquation(qa, qb, qc);
			
			if(intersectionYValues == null) {
				return null;
			}
			
			if(deltaY > 0) {
				y = intersectionYValues[1];
			}
			else {
				y = intersectionYValues[0];
			}
			
			final Point intersect = new Point((int) Math.round(x), (int) Math.round(y));
			
			return convertToJavaCoord(ellipse.getCenter(), intersect);
		}
		final float c = deltaY / deltaX; // line slope
		final float d = c * (0 - convIn.x) + convIn.y;
		// line translation

		// Solve the quadratic equation
		final float qa = 1f / (a * a) + (c * c) / (b * b);
		final float qb = ((2 * c * d) / (b * b));
		final float qc = ((d * d) / (b * b)) - 1f;
		
		double intersectionXValues[] = solveQuadraticEquation(qa, qb, qc);

		if(intersectionXValues == null) {
			return null;
		}
		
		double x = 0; // X coordinate of intersection point
		double y = 0; // Y coordinate of intersection point
		
		if ((c > 0 && deltaY > 0) || (c < 0 && deltaY < 0)
				|| (c == 0 && deltaX > 0)) {
			x = intersectionXValues[1];
			y = c * x + d;
		} else if ((c < 0 && deltaY > 0) || (c > 0 && deltaY < 0)
				|| (c == 0 && deltaX < 0)) {
			x = intersectionXValues[0];
			y = c * x + d;
		}

		final Point intersect = new Point((int) Math.round(x), (int) Math
				.round(y));

		// If there's no intersection point return null
		if (intersect.x == 0 && intersect.y == 0) {
			return null;
		}

		// Convert intersection Point from mathematical coordinate system to
		// Java coordinate system.
		return convertToJavaCoord(ellipse.getCenter(), intersect);
	}

	/**
	 * Converts a point from the Java coordinate system (root at the top-left, y
	 * grows from top to bottom) to the mathematic coordinate system (root is at
	 * center, y grows from bottom to top).
	 * 
	 * @param reference
	 *            The <code>point</code>'s y coordinate will be mirrored on
	 *            the horizontal line going through this point's y coordinate.
	 * @param point
	 *            The point whose coordinates shall be converted.
	 * @return A new point with the converted coordinates.
	 */
	private static Point convertToMathCoord(final Point reference,
			final Point point) {

		final Point mirrored = new Point(point.x, reference.y
				+ (reference.y - point.y));

		final Point translated = new Point(mirrored.x - reference.x, mirrored.y
				- reference.y);

		return translated;
	}

	/**
	 * Converts a point from the mathematic coordinate system (root is at
	 * center, y grows from bottom to top) to the Java coordinate system (root
	 * at the top-left, y grows from top to bottom).
	 * 
	 * @param reference
	 *            The <code>point</code>'s y coordinate will be mirrored on
	 *            the horizontal line going through this point's y coordinate.
	 * @param point
	 *            The point whose coordinates shall be converted.
	 * @return A new point with the converted coordinates.
	 */
	private static Point convertToJavaCoord(final Point reference,
			final Point point) {

		final Point translated = new Point(point.x + reference.x, point.y
				+ reference.y);

		final Point mirrored = new Point(translated.x, reference.y
				- (translated.y - reference.y));

		return mirrored;
	}
    
	/**
	 * Solves a quadratic equation of the form a*x*x + b*x + c = 0, or returns null
	 * if it's not solvable.
	 * 
	 * @param a the coefficient of x*x
	 * @param b the coefficient of x
	 * @param c the last part of the equation
	 * @return both solutions for x (may be identical) with the smaller x first, or null
	 */
    private static double[] solveQuadraticEquation(double a, double b, double c) {
    	
    	double[] result = new double[2];
    	
		final double discrim = b * b - 4 * a * c;
		
		if(a == 0) { // the equation is not quadratic...
			if(b != 0)
			{
				result[0] = -c / b;
				result[1] = -c / b;
			}
			else // the 'equation' is of the sort c = 0, so no solution for x...
			{
				return null;
			}
		}
		else if(discrim >= 0) { // a normal quadratic equation
			result[0] = (-b - Math.sqrt(discrim)) / (2 * a);
			result[1] = (-b + Math.sqrt(discrim)) / (2 * a);
		}
		else { // unsolvable
			result = null;
		}
		return result;
    }
    
    /**
     * Computes the intersection of a line with a rectangle.
     * <p>
     * The line goes from the point <code>inside</code> to the point
     * <code>outside</code>.
     * 
     * @param rect
     *            the outer bounds of the ellipse.
     * @param inside
     *            ending point of the line inside of the ellipse.
     * @param outside
     *            ending point of the line outside of the ellipse.
     * @return the intersection point of the line with the rectangle. Null, if
     *         there's no intersection point.
     */
    public static Point intersectRectangle(Rectangle rect, Point inside, Point outside) {
        
    	Point p = GeometryUtils.intersectLines(rect.getTopLeft(), 
    	rect.getTopRight(), inside, outside);
    			
    	Point intersection = GeometryUtils.intersectLines(rect.getTopRight(), 
    	rect.getBottomRight(), inside, outside);
    	        
    	if(intersection != null && (p == null || intersection.getDistance(outside) < p.getDistance(outside))) {
    		p = intersection;
    	}
    			
    	intersection = GeometryUtils.intersectLines(rect.getBottomLeft(), 
    			rect.getBottomRight(), inside, outside);
    	        
    	if(intersection != null && (p == null || intersection.getDistance(outside) < p.getDistance(outside))) {
    		p = intersection;
    	}
    			
    	intersection = GeometryUtils.intersectLines(rect.getTopLeft(), 
    			rect.getBottomLeft(), inside, outside);
    	        
    	if(intersection != null && (p == null || intersection.getDistance(outside) < p.getDistance(outside))) {
    		p = intersection;
    	}
    			
    	if(p == null) {
    		p = inside;
    	}
    	
    	return p;
    }

    /**
     * Computes the intersection of a polygon with a line from a point outside and
     * a point inside or on the other side of the polygon. <br>
     * This method has been designed for a convex polygon.
     * 
     * @param polyline an array with coordinate pairs; the first and last point
     *                 should be the same in order to close the polygon
     * @param inside  a point that inside polyline or on the opposite side of polyline
     * @param outside a point that must be outside polyline
     * @pre polyline[0] == polyline[polyline.length-2] && polyline[1] == polyline[polyline.length-1] && polyline.length >= 2
     * @return the intersection nearest to outside, or null if there is none
     */
	public static Point intersectPolyline(int[] polyline, Point inside, Point outside) {
	
		PointList intersections = new PointList();

		float almostZero = 0.001f;
		double lineLength = distance(inside, outside);
		
		// find the intersections between the lines of the polygon and the line between
		// inside and outside, and find intersection of this line with the corners
		Point lineBegin;
		Point lineEnd = new Point(polyline[0], polyline[1]);
		for(int i = 0; i<polyline.length - 2; i += 2) {
			
			lineBegin = lineEnd;
			lineEnd = new Point(polyline[i+2], polyline[i+3]);
			
			Point currentIntersection = intersectLines(lineBegin, lineEnd, inside, outside);
			
			double currentDistance = distance(inside, lineEnd) + distance(outside, lineEnd);

			if(currentIntersection != null) { // intersection with a side of the polygon
				intersections.addPoint(currentIntersection);
			}
			else if(Math.abs(currentDistance - lineLength) < almostZero) {
				// An intersection corner lies on a straight line between inside and 
				// outside, whereas the other ones form a triangle with lineEnd and lineBegin.
				// The total length of two sides of a triangle is greater than any single side
				intersections.addPoint(lineEnd);
			}
		}

		// return the intersection that is nearest to outside
		double minDistance = Double.MAX_VALUE;
		Point intersection = null;
		for(int i = 0; i<intersections.size(); ++i) {
			
			Point currentIntersection = intersections.getPoint(i);
			
			double currentDistance = distance(outside, currentIntersection);
			if(currentDistance < minDistance) { 
				minDistance = currentDistance;
				intersection = currentIntersection;
			}
		}

		return intersection;
	}
}