package ch.unizh.ifi.adora.view.figures.label;


import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLineLabel;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.linerouting.utils.HierarchyUtilities;
import ch.unizh.ifi.adora.parts.callback.ConnectionLabelConstraintCallBack;
import ch.unizh.ifi.adora.parts.callback.ConnectionLabelConstraintCallBackImpl;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.view.figures.ConnectionFigure;
import ch.unizh.ifi.adora.view.figures.INestedNode;
import ch.unizh.ifi.adora.view.figures.geometry.PointListUtilities;
import ch.unizh.ifi.adora.view.figures.geometry.LineSeg.KeyPoint;

/**
 * The connection label is the class used for all
 * connections that have a label annotation. The class
 * {@link ch.unizh.ifi.adora.view.figures.label.associationlabel.AssociationLabel}
 * is a specialized class of this class which additionally
 * shows an arrow following the slope of the association. 
 * 
 * TODO It would be nice to inherit {@link org.eclipse.draw2d.Figure}
 * instead of {@link org.eclipse.draw2d.Shape}.
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				18.07.2006 SM First Version
 * 						19.07.2006 SM Comments reformatting, 
 *									  correcting and checking.
 *						21.09.2007 CR changes to allow setting size according to length of text
 * @version				Date: 2006/07/18 15:34:43 $, $Author: treinhard $, $Revision: 4182 $
 */
public class ConnectionLabel extends Shape implements IPropertyChangeListener,  INodeProperties, IRoutingLineLabel {

	/**
	 * Stores the width of the line.
	 */
	protected static int LINEWITH = 1;
	
	/**
	 * Stores the {@link ch.unizh.ifi.adora.view.figures.label.WordWrappingLabel} 
	 * which contains the displayed text.
	 */
	
	private ToolTipLabel toolTipLabel;
	
	private WordWrappingLabel wrappingLabel;
	
     /**
     * Stores the text which will be displayed in this label.
     */
	protected String name;
	
	/**
	 * True, if the figure is selected.
	 */
	private boolean selected;

	private ConnectionLabelConstraintCallBack callback;
	
	private Font labelFont;
	
	private Point midpointOffset = null;

	/**
     * Instantiates a new NestedAssociationLabel for a given association.
     * @param name Stores the text which will be displayed in this label.
     */
    public ConnectionLabel(String name){
		this.setOpaque(true);
    	wrappingLabel = new WordWrappingLabel("");
		add(wrappingLabel);
		this. name = name;
		wrappingLabel.setText(name);
		toolTipLabel = new ToolTipLabel(name);
		setToolTip(toolTipLabel);
		init();
 	}
    
    public ConnectionLabel(ConnectionLabelConstraintCallBackImpl callback) {
    	this.callback=callback;
    	this.setOpaque(true);
    	wrappingLabel = new WordWrappingLabel("");
    	wrappingLabel.setText("");
		add(wrappingLabel);
		toolTipLabel = new ToolTipLabel("");
		setToolTip(toolTipLabel);
		init();
	}

	/**
     * Used to visually denote that a label
     * got selected by a user.
     *
     * This is done, by drawing a black rectangle around label.
     *
     * @param selected if <li>true</li>, outline this shape.
     */
    public void changeSelectionState(boolean selected){
    	this.selected = selected;
    	repaint();
    }
    
    /**
     * It has to be latched, because each time the label has to be repainted, 
     * the position of the text has to be changed.
     */
    public void paint(Graphics graphics) {
    	super.paint(graphics);
    	updateChildrenBounds();
    }
    
    /**
     * Updates the text position after the label got resized, because the text
     * has to be placed centered relating to the vertical axis. 
     */
    protected void updateChildrenBounds(){

    	Dimension textSize = new Dimension(getSize());
    	
    	textSize.width -= 2*LINEWITH;
    	
		Point textPosition = new Point(getLocation().x + 2*LINEWITH,
										getLocation().y + getVerticalLabelPosition());

		wrappingLabel.setBounds(new Rectangle(textPosition, textSize));
    }
    
    /**
     * @return The y coordinate of the label's position.
     * This is used to place the label centered in y- direction.
     */
    protected int getVerticalLabelPosition(){
    	int textPositionY = 0;
    	
    	if(wrappingLabel.getLabelHeight() > 0){
    		textPositionY  = Math.max((getSize().height -
									wrappingLabel.getLabelHeight()) / 2, 0);
    	}
    	return textPositionY;
    }
    
	/*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties#setName(java.lang.String)
     */
    public void setText(String name){
    	this.name = name;
    	wrappingLabel.setText(name);
    	if(callback!=null){
    		Rectangle r=this.getBounds();
    		int height=wrappingLabel.getMinimalHeight(r.width);
    		r.setSize(r.width, height);
    		callback.setConstraint(this.getLocation(), r);
    	}
    	if (name.equals("")) {
			setToolTip(null);
		} else {
			setToolTip(toolTipLabel);
			toolTipLabel.setText(name);
		}
    }
    
    
    /**
     * setting the text when a ConnectionLabel is new. 
     * @param name
     */
    public void setInitialText(String name){
    	this.name=name;
    	wrappingLabel.setText(name);
    	if(callback!=null){
    	if(this.getBounds().equals(new Rectangle(0,0,ch.unizh.ifi.adora.model.representation.ConnectionLabel.DEFAULT_WIDTH,ch.unizh.ifi.adora.model.representation.ConnectionLabel.DEFAULT_HEIGHT))){
    	int width=wrappingLabel.getInitWidth(name);
    	Rectangle r=this.getBounds();
    	int height=wrappingLabel.getMinimalHeight(width);
    	r.setSize(width, height);
    	callback.setConstraint(new Point(0,0), r);
    	}
    	}
    	if (name.equals("")) {
			setToolTip(null);
		} else {
			setToolTip(toolTipLabel);
			toolTipLabel.setText(name);
		}
    }
    
   
    
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties#getName()
     */
	public String getText() {
		return name;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.figures.label.nested.IConnectionLabel#getNameLabel()
	 */
	public WordWrappingLabel getWrappingLabel() {
		return wrappingLabel;
	}	
	
    private void init(){
        AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);
        refreshFont();
    }

    /**
	 * Handler method for preference changes.
	 */
	public void propertyChange(Preferences.PropertyChangeEvent evt) {
		
		if(evt.getProperty().equals(IPreferenceNameConstants.ADORA_VIEW_CONNECTION_LABEL_FONT)) {		
			refreshFont();		
		}
	}
	
	/**
	 * Get the font from the preference store and update the visuals accordingly.
	 */
	private void refreshFont() {
		
		IPreferenceStore store  = AdoraPlugin.getDefault().getPreferenceStore();
		Font font = new Font(null, PreferenceConverter.getFontData(store, IPreferenceNameConstants.ADORA_VIEW_CONNECTION_LABEL_FONT));
		
		setFont(font);
		wrappingLabel.setFont(font);
		
		if(labelFont != null && !labelFont.isDisposed()) {
			labelFont.dispose();
		}
		
		labelFont = font;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.Shape#outlineShape(org.eclipse.draw2d.Graphics)
	 */
	protected void outlineShape(Graphics graphics) {
		if(selected){
			Rectangle r = getBounds();
			int x = r.x + lineWidth / 2;
			int y = r.y + lineWidth / 2;
			int w = r.width - Math.max(1, lineWidth);
			int h = r.height - Math.max(1, lineWidth);
			graphics.setForegroundColor(new Color(null, 0, 125, 125));
			graphics.drawRectangle(x, y, w, h);
			graphics.setForegroundColor(new Color(null, 0, 0, 0));
		}
		setForegroundColor(new Color(null, 0, 0, 0));
	}
	
	/**
	 * Does nothing.
	 */
	protected void fillShape(Graphics graphics) {
	}
	
	/*
	 * IRoutingLineLabel interface
	 * 
	 * @see ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLineLabel
	 */

	/**
	 * Returns the current bounds of this label. The coordinates
	 * are relative to the the first common ancestor of the 
	 * source and target node of the line this label belongs to.
	 * 
	 * @return current boundaries of the label
	 */
	public java.awt.Rectangle getRoutingBounds() {
		
		Rectangle rect = getBounds().getCopy();
		
		ConnectionFigure line = (ConnectionFigure)getParent();		
		IRoutingNode parent = HierarchyUtilities.getInstance().commonAncestor(line.getRoutingSource(), line.getRoutingTarget());
		Point parentLocation = ((INestedNode)parent).getAbsoluteLocation();
		
		// Make the coordinates absolute
		rect.translate(parentLocation.negate());
		((Figure)parent).translateToAbsolute(rect);
		// and then relative
		translateToRelative(rect);

		return new java.awt.Rectangle(rect.x, rect.y, rect.width, rect.height);
		
	}

	/**
	 * Sets the bounds (i.e., position and size) of this label to
	 * the values given as argument. The coordinates must be relative 
	 * to the the first common ancestor of the source and target node 
	 * of the line this label belongs to.
	 * 
	 * @param bounds the new bounds of the label
	 */
	public void setRoutingBounds(java.awt.Rectangle bounds) {
		
		Rectangle rect = new Rectangle(bounds.x, bounds.y, bounds.width, bounds.height);
		
		ConnectionFigure line = (ConnectionFigure)getParent();
		
		Point midpoint = new Point();
        PointListUtilities.pointOn(line.getPoints(), 0, KeyPoint.MIDPOINT, midpoint);
		
		IRoutingNode parent = HierarchyUtilities.getInstance().commonAncestor(line.getRoutingSource(), line.getRoutingTarget());
		Point parentLocation = ((INestedNode)parent).getAbsoluteLocation();
		
		// Make the coordinates absolute
		rect.translate(parentLocation);
		((Figure)parent).translateToAbsolute(rect);
		// and then relative
		translateToRelative(rect);

		setSize(rect.width, rect.height);
		
		int xOffset = rect.x - midpoint.x + rect.width / 2;
		int yOffset = rect.y - midpoint.y + rect.height / 2;
		Point p = new Point(xOffset, yOffset);
		midpointOffset = p;
		
	}

	/**
	 * Returns the minimal height this label has if its width is the one
	 * given as argument. The height is calculated by wrapping the
	 * text in the label accordingly.
	 * 
	 * @param width the width the minimal height should be given for
	 * @return minimal height with width as constraint
	 */
	public int getMinimalHeight(int width) {
		
		if(width < 20) {
			width = 20;
		}
		
		return wrappingLabel.getMinimalHeight(width);
	}

	/**
	 * Returns the actual width of this label if its maximal width is
	 * the one given as argument.
	 * 
	 * @param maxWidth the maximal width the label can have
	 * @return the actual width under the constraint maxWidth
	 */
	public int getWidth(int maxWidth) {
		
		if(maxWidth < 20) {
			maxWidth = 20;
		}
		
		return wrappingLabel.getWidth(maxWidth);
		
	}
	
	/**
	 * Returns the offset of the label from the lines midpoint. This value is
	 * set during the automatic placement of the label.
	 * 
	 * @return the offset from the line's midpoint
	 */
	public Point getMidpointOffset() {
		return midpointOffset;
	}

	/**
	 * Returns the desired position of the label on the line.
	 * 
	 * @return desired position of the label
	 */
	public int getLabelPosition() {
		return ch.unizh.ifi.adora.linerouting.labels.LabelConstants.MIDDLE;
	}
	
}