package ch.unizh.ifi.adora.view.figures.label;

import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Shell;

import ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping.BracketDelimiter;
import ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping.CapitalCharStringDelimiter;
import ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping.DesciptionDelimiter;
import ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping.SpaceCharStringDelimiter;
import ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping.StringDelimiter;
import ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping.UnderlineCharStringDelimiter;
import ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping.WordWrappingStringLayouter;

/**
 * This label is capable of wrapping it's text according to specified
 * delimitting policies.
 * 
 * @author 			vigan
 * @copyright 			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history 			Feb 15, 2006 SM First Version
 * 						Jul 19, 2006 SM Comments reformatting, 
 * 										checking and correcting.
 * 										Renaming of setName(..) and getName().
 * 						Aug 17, 2006 JS Fixed calculation of labelHeight
 * @version $Date: 2009-12-15 14:57:08 +0100 (Tue, 15 Dec 2009) $, $Author: treinhard $, $Revision: 4999 $
 * 
 * 
 */
public class WordWrappingLabel extends Shape{

	/**
	 * String which saves the label text.
	 */
	private String text;

	/**
	 * Stores the height of the label.
	 * This variable is used to determine 
	 * whether the label is big enough 
	 * to display the whole text or not.
	 */
	private int labelHeight;

	/**
	 * Constants representing cardinal directions and relative positions.
	 */
	private int horizontalAlignment = PositionConstants.LEFT;

	/**
	 * Used to hide this label during direct edit.
	 */
	private boolean hide;

	/**
	 * Constants representing cardinal directions and relative positions.
	 */
	private int verticalAlignment = PositionConstants.TOP;

	/**
	 * The graphics context (GC) class is used as helpe for drawing. 
	 */
	private static GC gc;
	
	/**
	 * Stores the displacement between each line.
	 */
	final int lineDisplacement = 5;

	/**
	 * Instantiates a WordWrappingLabel with an empty text.
	 */
	public WordWrappingLabel() {
		text = "";
		setBorder(new MarginBorder(0, 0, 0, 0));
	}
	
	/**
	 * Creates a new Label with a given text.
	 * 
	 * @param text Text wich will be displayed.
	 * @param showCardinality Decides if the cardinality should be shown.
	 */
	public WordWrappingLabel(String text) {
		this();
		this.text = text;
	}

	/**
	 * Draws the text of the label.
	 * @param graphics The graphics canvas to draw the text.
	 */
	protected void outlineShape(Graphics graphics) {
		drawWrappedText(graphics);
	}

	/**
	 * Sets the text of the label.
	 * @param text The text to draw.
	 */
	public void setText(String text) {
		this.text = text;
		layout();
		repaint();
	}
	
	/**
	 * Hides the label.
	 */
	public void hide(){
		hide = true;
		repaint();
	}

	/**
	 * Shows the label
	 */
	public void show(){
		hide = false;
		repaint();
	}
	
	/**
	 * Gets the text of the label.
	 * @return Returns the text of the label.
	 */
	public String getText() {
		return text;
	}

	/**
	 * Sets the horitontal aligment of the text in the label. 
	 * Valid values are:
	 * <ul>
	 *   <li>{@link org.eclipse.draw2d.PositionConstants#LEFT}</li>
	 *   <li>{@link org.eclipse.draw2d.PositionConstants#RIGHT}</li>
	 *   <li>{@link org.eclipse.draw2d.PositionConstants#CENTER}</li>
	 * </ul>
	 * @param value the aligment
	 * @see #getHorizontalAligment()
	 */
	public void setHorizontalAligment(int value) {
		if (value == horizontalAlignment)
			return;
		horizontalAlignment = value;
		repaint();
	}

	/**
	 * Sets the vertical aligment of the text in the label. 
	 * Valid values are:
	 * <ul>
	 *   <li>{@link org.eclipse.draw2d.PositionConstants#TOP}</li>
	 *   <li>{@link org.eclipse.draw2d.PositionConstants#MIDDLE}</li>
	 *   <li>{@link org.eclipse.draw2d.PositionConstants#BOTTOM}</li>
	 * </ul>
	 * @param value the aligment
	 * @see #getHorizontalAligment()
	 */
	public void setVerticalAlignment(int value) {
		if (value == verticalAlignment)
			return;
		verticalAlignment = value;
		repaint();
	}
	
	
	public int getMinimalHeight(int width) {
		
		List<String> list = generateWrappedText(width - getInsets().left - getInsets().right);
		
		int min = (getGC().getFontMetrics().getHeight() + lineDisplacement) * list.size();
		
		return min + getInsets().top + getInsets().bottom;
		
	}
	
	
	public int getWidth(int maxWidth) {
		
		int max = 0;
		List<String> list = generateWrappedText(maxWidth - getInsets().left - getInsets().right);
		for(String line: list) {
			int length = gc.stringExtent(line).x + 5;
			if(length > max) {
				max = length;
			}
		}
		
		return max;
	}
	
	
	public int getInitWidth(String name){
		int avg=gc.getFontMetrics().getAverageCharWidth();
		int letters=name.length();
		if(name.contains("send ")&&!(name.startsWith("send"))){//TODO: need something better here!
			letters=name.substring(0,name.indexOf("send")).length();
		}
		return avg*letters;
	}
	
	/**
	 * Generates a list containing the Wrapped Text.
	 *  
	 * @param ins The insets of the parent in which this label is contained.
	 * @return List containing all text-rows.
	 */
	protected List<String> generateWrappedText(int width){
		
		/*
		 * List containing all text-rows.
		 */
		List<String> list;
		
		//Specifies which tokens should be considered to delimit a line.
		StringDelimiter[] stringDelimiters = {  
				new CapitalCharStringDelimiter(),
				new SpaceCharStringDelimiter(),
				new UnderlineCharStringDelimiter(),
				new BracketDelimiter(),
				new DesciptionDelimiter()};
		
		WordWrappingStringLayouter layouter = new WordWrappingStringLayouter(
				stringDelimiters);
		
		layouter.setFont(getFont());
		
		//This list contains all textlines.
		list = layouter.layoutLabelString(text, width);
		
		return list;
	
	}

	/**
	 * Writes the text onto the label. The text will be wraped with the help of
	 * a WordWrappingStringDelimiter.
	 * 
	 * @param graphics The graphics in which the text is drawn.
	 */
	public void drawWrappedText(Graphics graphics) {

		/*
		 * List containing all text-rows.
		 */
		List<String> list;

		/*
		 * Stores the displacement between each line.
		 */
		final String cuttedIndicator = "\u00BB";
		
		/*
		 * Stores the height of a text line.
		 */
		final int lineHeight = graphics.getFontMetrics().getHeight() + lineDisplacement;
		
		/*
		 * Stores the y coordinate of label's starting point. 
		 */
		final int yStartingPoint = getLocation().y;
		
		/*
		 * Stores the height of all processed text lines.
		 */
		int totalHeight;

		/*
		 * Current line's text.
		 */
		String s = "";

		/*
		 * Current location of the line starting position.
		 */
		Point location = getLocation();
		
		Insets ins = new Insets(0, 0, 0, 0);

		ins = getInsets();
		
		if (hide) 
			graphics.setForegroundColor(ColorConstants.white);
		else
			graphics.setForegroundColor(ColorConstants.black);
		
		list = generateWrappedText(getSize().width - ins.left - ins.right);
		
		totalHeight = lineHeight * list.size();

		//Specifies whether the text should be drawn left-aligned 
		//or centered.
		if (verticalAlignment == PositionConstants.MIDDLE
				&& totalHeight < (getSize().height - ins.top - ins.bottom)) {
			location.y += (getSize().height - ins.top - ins.bottom - totalHeight) / 2;
			location.y += ins.top;
		} else if (verticalAlignment == PositionConstants.BOTTOM
				&& totalHeight < (getSize().height - ins.top - ins.bottom)) {
			location.y += getSize().height - ins.top - ins.bottom - totalHeight;
			location.y += ins.top;
		}

		// Paints each line with an vertical offset of 5 pixels.
//		location.y += lineDisplacement;
		
		location.y += ins.top;
		location.x += ins.left;
		
		for (int lineNr = 0; lineNr < list.size(); lineNr++) {

			s = list.get(lineNr);

			int x = location.x;
			
			GC gc = getGC();
			gc.setFont(getFont());

			if (horizontalAlignment == PositionConstants.CENTER) {
				x += (getSize().width - ins.left - ins.right - gc
						.stringExtent(s).x) / 2;

			} else if (horizontalAlignment == PositionConstants.RIGHT) {
				x += getSize().width - ins.left - ins.right
						- gc.stringExtent(s).x;
			}

			//enough space for text?
			int labelHeight = getBounds().height
					- getParent().getInsets().bottom
					- getParent().getInsets().top;

			//If this label is used in a node, children
			//the whole text has to be displayed at the top of the node.
			if (labelHeight >= 
					location.y - yStartingPoint + 2 * lineHeight - lineDisplacement  || 
					list.size() - 1 == lineNr || 
					((this instanceof NodeLabel && ((NodeLabel) this)
							.getParent().getChildren().size() > 1))) {

				graphics.drawString(s, x, location.y);
				location.y += lineHeight;
			} else {
				String concat = s.substring(0, s.length() - 1) + cuttedIndicator;
				graphics.drawString(concat, x, location.y);
				location.y += lineHeight;
				break;
			}
		}

		//calculates the actual height of the label.
		if (location.y + lineDisplacement - yStartingPoint > 0) {
			labelHeight = location.y + lineDisplacement - yStartingPoint;
		}
	}
	
	/**
	 * Returns the GC used as helper for drawing graphics elements.
	 * @pre       true
	 * @post      return != null
	 * @return	The graphics context is returned.
	 */
	protected static GC getGC() {
		if (gc == null) {
			gc = new GC(new Shell());
		}
		return gc;
	}
	

	/**
	 * Used to determine the height of the Label in a node with children.
	 * Because then, the whole text has to be displayed at the top of the node.
	 * 
	 * @return the height of this label.
	 */
	public int getLabelHeight() {
		return labelHeight;
	}

	/**
	 * Sets the currently used font for drawing the text.
	 * @param font The font to use.
	 */
	public void setFont(Font font) {
		super.setFont(font);
		getGC().setFont(font);
	}
	
	

	/**
	 * Labels are not opaque, therefore this method does nothing.
	 *
	 * @see org.eclipse.draw2d.Shape#fillShape(org.eclipse.draw2d.Graphics)
	 */
	protected void fillShape(Graphics graphics) {
	}
}