package ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping;

import java.awt.FontMetrics;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Shell;


/**
 * This class provides a layouter which layouts a string into substrings that
 * can be drawn without any border crossing in a canvas of a given
 * {@link WordWrappingJLabel}object. For this this class uses the FontMetrics
 * object of the {@link WordWrappingJLabel}object.
 * <p>
 * The layouting is done by breaking the strings into substrings. The string is
 * broken at a specified delimiter character. Which kind of character designates
 * a delimiter is defined via the {@link StringDelimiter}class. This class has
 * to be inherited to a sub class and there there has to be defined a concrete
 * delimiter class as it is done in the
 * {@link view.Role.CapitalCharStringDelimiter}class.
 * <p>
 * The delimiting is done by using a precedence delimiter, i.e. first the are
 * the more preferable delimiters used to layout the string. If there is not
 * found any of this delimiter, so another layouter is called which implements
 * the layouting by a less preferred delimiter.
 * <p>
 * 
 * @author		Silvio Meier
 * @history	2004-10-27 SM 	History deleted.
 *				2004-10-27 AG 	Some refactoring. Added a check to
 *								{@link #findBestWrapPosition(String,
 *								FontMetrics, int, IntList)} to prevent further line
 *								wrappings if the line fits the label's width. Fixes bug #14
 * 
 * @version $Date: 2009-12-18 14:26:58 +0100 (Fri, 18 Dec 2009) $, $Author: treinhard $, $Revision: 5003 $
 */
public class WordWrappingStringLayouter {

    /**
     * Delimiters identifying potential positions for line breaks.
     */
    private final StringDelimiter delimiters[];

    /**
     * Holds the lines of the wrapped text.
     */
    private List<String> textContainer;
    
    /**
     * The graphic context which is used as 
     * helper class for drawing graphical elements.
     */
    private static GC gc;

    /**
     * Initializes a WordWrappingStringLayouter with an array of delimiters
     * for text-wrapping.
     * 
     * @param stringDelimiters The delimiters to use for wrapping the text.
     */
    public WordWrappingStringLayouter(final StringDelimiter[] stringDelimiters) {
        if (stringDelimiters == null) {
            throw new NullPointerException(
                    "Parameter stringDelimters must not be null");
        }
        delimiters = stringDelimiters;
        textContainer = new ArrayList<String>();
    }

    /**
     * Constructor initializes the String layouter with 
     * no delimiter.
     */
    public WordWrappingStringLayouter() {
        this(new StringDelimiter[0]);
    }

    /**
     * Returns the textcontainer which is used to store the layouted text of the
     * label.
     * 
     * @return  An unmodifiable list with Strings representing the lines.
     */
    public synchronized List<String> getTextContainer() {
        return Collections.unmodifiableList(textContainer);
    }
    
    /**
     * Returns the width of the longest line.
     * 
     * @return The width of the longest line or 0 if no line exists.
     */
    public synchronized int getLongestLineWidth() {
        int maxWidth = 0;
        final Iterator<String> iterator = textContainer.iterator();
        while (iterator.hasNext()) {
            final String element = iterator.next();
            final int length = getGC().stringExtent(element).x;
            maxWidth = Math.max(maxWidth, length);
        }
        return maxWidth;
    }

    /**
     * This method is called by the {@link WordWrappingJLabelLayouter}class to
     * get a layouted text string, in form of an array. First there is the
     * layout of the manually set line breaks, after that the smart layout with
     * the delimiters is done.
     * 
     * @param fullString The full string which should be layouted by the
     *            layouter
     * @param fontMetrics FontMetrics object of the font to layout the string
     *            with
     */
    public synchronized List<String> layoutLabelString(final String fullString,
             final int labelWith) {
        textContainer = new ArrayList<String>();
        
        final boolean textIsEmpty = (fullString == null) || fullString.equals("");
        if (!textIsEmpty) {
            //First convert the linebreaks in the String to entries in a list
            final List<String> manuallyWrappedStrings = manuallyWrapString(fullString);

            // iterate over all manually broken text lines and try to layout
            // them using StringDelimiters
            final Iterator<String> iter = manuallyWrappedStrings.iterator();
            
            while (iter.hasNext()) {
                final String tempStr = iter.next();

                if (fitsWidth(tempStr, labelWith)) {
                    textContainer.add(tempStr);
                } else {
                    // if the string fits not entirely on one line in the drawing
                    // canvas, try to
                    // use sophisticated layouting methods
                	
                    final List<String> delimiterWrappedLines = delimiterWrapString(tempStr,
                    		labelWith);
                    
                    textContainer.addAll(delimiterWrappedLines);
                }
            }
        }
        
        return textContainer;
    }

    /**
     * This method layouts a given string by doing dynamically line breaks for a
     * given set of delimiters.
     * 
     * @param text String to wrap.
     * @param fontMetrics Font metrics, which is used to calculate string size
     *            in points.
     * @param labelWidth The width of the label.
     */
    private List<String> delimiterWrapString(String text,
             final int labelWidth) {
        final List<String> outStrings = new ArrayList<String>();
        
        // as long as string consists of characters.....
        while (text.length() != 0) {
        	
            //Find possible wrap positions using delimiters
        	final Vector<Integer> positions = findPossibleWrapPos(text);
        	
            //Find the wrap position producing a string which just fits the
            // label's width
            int savedCharIndex = findBestWrapPosition(text, 
                    labelWidth, positions);
            
            //If delimiters didn't produce usable wrap position every character
            //can be wrapped
            if (savedCharIndex == -1) {
                savedCharIndex = findFallbackWrapPos(text,
                        labelWidth);
            }
            
            // fallback if fallback delimiter does not work, i.e.
            // the string can not be layouted in width!!!!
            if (savedCharIndex == -1) {
                // string can not be handled.
                text = "";
                outStrings.add(" ");
            } else {
                // add the found string to the output strings
                final String addString = text.substring(0, savedCharIndex);
                
                outStrings.add(addString);
                // the remaining string is the new fullString
                text = text.substring(savedCharIndex, text.length()).trim();
                //Only continue if the text does not fit the label's width
                if (fitsWidth(text, labelWidth)) {
                    outStrings.add(text);
                    break;
                }
            }
        }
        
        return outStrings;
    }

    /**
     * Finds the best position to wrap the text. The best position is the one
     * which maximizes the text width while not exceeding the given labelWidth.
     * 
     * @param text The text to investigate.
     * @param labelWidth The width of the label.
     * @param positions The positions which can be used to wrap the text.
     * @return The index of the best wrap position.
     */
    private int findBestWrapPosition(final String text,
            final int labelWidth,
            final Vector<Integer> positions) {
        int savedCharIndex = -1;
        
        Collections.sort(positions);
        
        int l = 0;
        int r = positions.size() - 1;
        while(l < r) {
        	int x = (l + r) / 2;
        	int delimiterPosition = (positions.get(x)).intValue();
        	int stringWidth = getGC().stringExtent(text.substring(0, delimiterPosition)).x;
        	if(stringWidth < labelWidth) {
        		l = x + 1;
        	} else {
        		r = x - 1;
        	}
        }
        
        if(l > 0) {
        	int stringWidth = getGC().stringExtent(text.substring(0, (positions.get(l)).intValue())).x;
        	if(stringWidth < labelWidth) {
        		savedCharIndex = (positions.get(l)).intValue();
        	} else if (l > 1){
        		savedCharIndex = (positions.get(l-1)).intValue();
        	}
        }
        
        return savedCharIndex;
    }

    /**
     * Finds the best wrap position by considering every character as a
     * possibility. The best position is the one which maximizes the text width
     * while not exceeding the given labelWidth.
     * 
     * @param text The text to investigate.
     * @param labelWidth The width of the label.
     * @return The best position to wrap.
     */
    private int findFallbackWrapPos(String text, final int labelWidth) {

    	// Do a binary search to find best wrap position.        
        int savedCharIndex = -1;
        int l = 0;
        int r = text.length() - 1;
        while(l < r) {
        	int x = (l + r) / 2;
        	String currentText = text.substring(0, x);
        	if(!fitsWidth(currentText, labelWidth)) {
        		r = x - 1;
        	} else {
        		l = x + 1;
        	}
        }
        
        if(l > 0 && l >= r) {
        	savedCharIndex = r;
        }
        
        return savedCharIndex;
    }

    /**
     * Find positions to wrap the given text using the delimiters.
     * 
     * @param text Text to be wrapped.
     * @return A list of positions where the text could be wrapped.
     */
    private Vector<Integer> findPossibleWrapPos(final String text) {
        final Vector<Integer> positions = new Vector<Integer>();

        for (int i = 0; i < delimiters.length; i++) {
            int index = 0;
            final StringDelimiter stringDelimiter = delimiters[i];

            // get all potential line break positions
            while (index != StringDelimiter.CONTAINS_NO_DELIMITER) {
                index = stringDelimiter.nextIndexOfDelimiter(text, index + 1);
                if (index != StringDelimiter.CONTAINS_NO_DELIMITER) {
                    positions.add(new Integer(index));
                }
            }
        }
        return positions;
    }

    /**
     * Tests if a text fits within a given width.
     * 
     * @param text The text to test.
     * @param textFontMetrics The metrics to be used for computing the text
     *            width.
     * @param width The width to be tested against.
     * @return True if the rendered text width is smaller than the given width.
     */
    boolean fitsWidth(final String text, final int width) {
        final int stringWidth = getGC().stringExtent(text).x;
        return stringWidth <= width;
    }

    /**
     * This method tries to do (manually set) linebreaks in a given string.
     * 
     * @param fullString The string which contains the manual set line breaks
     * @return Returns a text container, containing all the text lines which
     *         were delimited in the fullString argument by a line feed.
     */
    List<String> manuallyWrapString(final String fullString) {
        final List<String> lines = new ArrayList<String>();
        final char MANUAL_LINE_BREAK = '\n';
        final StringTokenizer tokenizer = new StringTokenizer(fullString, ""
                + MANUAL_LINE_BREAK);
        while (tokenizer.hasMoreTokens()) {
            lines.add(tokenizer.nextToken());
        }

        return lines;
    }
    
    /**
     * Sets the font for the layouting.
     * @pre       font != null
     * @post      true 
     * @param font The font used for the layouting
     *				of the string.
     */
    public void setFont(Font font) {
        getGC().setFont(font);
    }
    
    /**
     * Returns the helper graphic context which is used
     * for the layouting of the string.
     *  
     * @pre       true
     * @post      return != null
     * @return Returns the graphic context helper 
     *			class which can be used for graphical operation.
     */
    private static GC getGC() {
        if(gc == null) {
            gc = new GC(new Shell());
        }
        return gc;
    }

}