package ch.unizh.ifi.adora.view.figures.label.associationlabel;


import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.parts.callback.ConnectionLabelConstraintCallBack;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.view.figures.AssociationFigure;
import ch.unizh.ifi.adora.view.figures.label.ConnectionLabel;
import ch.unizh.ifi.adora.view.figures.label.WordWrappingLabel;

/**
 * This association label is capable of wrapping its text according to 
 * specified delimiting policies.
 * Furthermore, it displays the cardinality as well as 
 * an arrowhead which indicated the affiliation of an association label
 * to a node (source or target node).      
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				17.07.2006 IV First Version
 * 						19.07.2006 SM Comments reformatting, correcting 
 * 									  and checking
 * 						25.09.2007 CR changes to allow resizing of label if text changes
 * @version				Date: 2006/07/18 15:34:43 $, $Author: treinhard $, $Revision: 4917 $
 */
public class AssociationLabel extends ConnectionLabel implements INodeProperties{
	
	/**
	 * Stores the {@link ArrowDecoration} which contains
	 * the arrow to indicate the direction.
	 */
	private ArrowDecoration arrowFigure;
	
	/**
	 * Stores the defaultCardinality.
	 */
	private final String defaultCardinality = "1";
	
	/**
     * Stores the minimal Cardinality.
     */
    private String minCard = defaultCardinality;
    
    /**
     * Stores the maximal Cardinality.
     */
    private String maxCard = defaultCardinality;
    
    /**
     * gap between arrow and text
     */
	final int arrowTextGap = 10; 
    /**
     * to resize AssociationLabel(model) from view
     */
    private ConnectionLabelConstraintCallBack callback;
    
    /**
     * Position of the label on the line. Required for the automatic label placement
     */
    private int position;
  
    
    /**
     * Instantiates a new NestedAssociationLabel for a given association.
     * 
     * @param association Association, where this label belongs to.
     * @param minCard Stores the minimal Cardinality.
     * @param maxCard Stores the maximal Cardinality.
     * @param name Stores the text which will be displayed in this label.
     * @param isTo Determine if the label is attached to the source- or targetnode.
     * @param callback 
     */
    public AssociationLabel(AssociationFigure association, 
    		String minCard, 
    		String maxCard, 
    		String name, 
    		boolean isTo, ConnectionLabelConstraintCallBack callback){
		
    	super(name);
    	this.callback=callback;
    	arrowFigure = new ArrowDecoration();
    	add(arrowFigure);
        
    	//Appends the arrow figure to the right end of the association.
    	if(isTo){
    		association.setTargetDecoration(arrowFigure);
    		position = ch.unizh.ifi.adora.linerouting.labels.LabelConstants.START;
    	} else{
    		association.setSourceDecoration(arrowFigure);
    		position = ch.unizh.ifi.adora.linerouting.labels.LabelConstants.END;
    	}
		
		setMinCard(minCard); 
        setMaxCard(maxCard); 
        refreshText();
        refreshSize();
 	}
    
    
    /**
     * used to calculate the width of the total text
     * @return the total width needed for text, spaces, arrow
     */
    private int getTotalTextWidth(){
    	String txt=getWrappingLabel().getText();
    	int txtwidth=getWrappingLabel().getInitWidth(txt);
    	int width=arrowFigure.getSize().width + arrowTextGap;
    	return width+txtwidth;
    }
    
    
    /**
     * Used to reposition the {@link ArrowDecoration} in a way, that it
     * is located in the vertical center of the text. 
     * 
     * @return The height of the {@link WordWrappingLabel}
     */
    public int getTextHeight(){
    	return Math.max(getSize().height/2 - 5, 0);
    }
	  
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.view.figures.label.ConnectionLabel#updateChildrenBounds()
     */
    protected void updateChildrenBounds(){


    	
    	Dimension textSize = new Dimension(getSize().width - 
									arrowFigure.getSize().width - arrowTextGap,
									getSize().height);
    	
		Point textPosition = new Point(getLocation().x +
									arrowFigure.getSize().width + arrowTextGap,
									getLocation().y + getVerticalLabelPosition());

		getWrappingLabel().setBounds(new Rectangle(textPosition, textSize));
			
    }
    
	/**
	 * Sets the maximal cardinality.
	 * 
     * @param maxCard The maxCard to set.
     */
    public void setMaxCard(String maxCard) {
        this.maxCard = maxCard;
        refreshText();
        refreshSize();
        
    }

    /**
     * Sets the minimal cardinality.
     * 
     * @param minCard The minCard to set.
     */
    public void setMinCard(String minCard) {
        this.minCard = minCard;
        refreshText();
        refreshSize();
    }
    
    
    /**
     * refreshes the size of the label according to the text showing in the label
     */
    public void refreshSize(){
    	 int width=getTotalTextWidth();
         callback.setConstraint(new Point(0,0), new Rectangle(getLocation().x,getLocation().y,width,getWrappingLabel().getMinimalHeight(width)));
    }
    
     /**
      * Returns true, if the default cardinality is set.
     * @return True if the current Cardinality equals 1:1.
     */
    public boolean isDefaultCardinality(){
        return maxCard.equalsIgnoreCase(defaultCardinality) && 
        	minCard.equalsIgnoreCase(defaultCardinality);
    }
    
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties#setName(java.lang.String)
     */
    public void setText(String name){
    	super.setText(name);
    	refreshText();
    	refreshSize();
    }
   
    /**
     * Refreshed the displayed text. 
     */
    private void refreshText(){
        if (isDefaultCardinality()) {
        	getWrappingLabel().setText(name);
    	} else{
    		getWrappingLabel().setText(
    				name + "   (" + minCard + ":" + maxCard + ")");
    	}
     }
    
    public void setLocation(Point p) {
    	if (getLocation().equals(p)) 
    		return;
    	Rectangle r = new Rectangle(getBounds());
    	r.setLocation(p);
    	setBounds(r);
    }
    
	public int getLabelPosition() {
		return position;
	}
}