package ch.unizh.ifi.adora.view.layers;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.view.figures.INestedNode;

/**
 * Replaces the standard GEF FreeformLayer because this is the root figure of
 * the model and the line router expects the methods defined in IRoutingNode
 * to be implemented on the root element.
 *
 * @author		        Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2007-04-19 TR First Version
 * @version             $Date: 2009-02-12 15:36:05 +0100 (Thu, 12 Feb 2009) $, $Author: treinhard $, $Revision: 4010 $
 * @responsibilities	Root figure for the ADORA model
 *
 */
public class AdoraFreeformLayer extends FreeformLayer implements IRoutingNode, INestedNode {

	/**
	 * Returns the parent of this node or null if this is the root node.
	 * 
	 * @return the parent node
	 */
	public IRoutingNode getRoutingParent() {
		
		return null;
		
	}
	
	/**
	 * Returns all the children of this node. The method returns an iterator
	 * over an empty collection if this node doesn't have any children.
	 * 
	 * @return the contained children
	 */
	public Iterator<IRoutingNode> getRoutingChildren() {
		
		List<IRoutingNode> children = new ArrayList<IRoutingNode>();
		Iterator<Figure> figures = getChildren().iterator();
		while(figures.hasNext()) {
			Figure fig = figures.next();
			if(fig instanceof IRoutingNode) {
				children.add((IRoutingNode)fig);
			}
		}
		
		return children.iterator();
		
	}
	
	
	/**
	 * Returns the bounds of this node. The coordinates of the bounds have to
	 * be relative to the parent of this node.
	 * 
	 * @return the relative bounds of the node
	 */
	public Rectangle getRoutingBounds() {
		
		org.eclipse.draw2d.geometry.Rectangle bounds = getFreeformExtent();
		
		int x = bounds.x;
		int y = bounds.y;
		
		int width = bounds.width;
		if(bounds.x > 0) {
			width += bounds.x;
			x = 0;
		}

		int height = bounds.height;
		if(bounds.y > 0) {
			height += bounds.y;
			y = 0;
		}
		
		width += 30;
		height += 30;
		
		return new Rectangle(x, y, width, height);
		
	}

	
	public boolean isZoomed() {
		return true;
	}

	
	/**
	 * Returns the relative coordinates of the node given as argument with
	 * respect to this node. The method makes sure that the coordinates are
	 * always positive as it is expected by the line routing algorithm.
	 * 
	 * @param child the node to get the relative coordinates for
	 * @return the relative coordinates of the child
	 */
	public org.eclipse.draw2d.geometry.Rectangle getRelativeBounds(IFigure child) {
		
		org.eclipse.draw2d.geometry.Rectangle rect = child.getBounds().getCopy();
		
		if (getRoutingBounds().x < 0) {
			rect.translate(-getRoutingBounds().x, 0);
		}

		if (getRoutingBounds().y < 0) {
			rect.translate(0, -getRoutingBounds().y);
		}
		
		return rect;
		
	}
	
	
	/**
	 * Returns the absolute (GEF) coordinates of this layer. This method makes
	 * sure that the origin of the layer is always at (0,0) as it is expected
	 * by the line routing algorithm.
	 * 
	 * @return the absolute coordinates.
	 */
	public Point getAbsoluteLocation() {
		
		Point location = getFreeformExtent().getLocation().getCopy();
		if(location.x > 0) {
			location.x = 0;
		}

		if(location.y > 0) {
			location.y = 0;
		}
		
		return location;
		
	}
	
	
}
