package ch.unizh.ifi.adora.view.layers;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.Polyline;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.parts.geometryfactory.GeometryFactory;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.zoom.ZoomEventConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;
import ch.unizh.ifi.adora.zoom.berner.algorithm.MinRadial;
import ch.unizh.ifi.adora.zoom.berner.algorithm.NewBerner;

/**
 * This class shows auxiliary figures used for layouting, such as original
 * position of a component moved by an algorithm or collision rectangles.
 * 
 * Both the preferences and the layout algorithms are observed for this purpose.
 * 
 * @author Juerg Schlaefli (JS)
 * @copyright Requirements Engineering Research Group Department of Computer
 *            Science University of Zurich
 * @history 2006-05-03 JS First version
 *          2006-05-31 JS added visualization for movements of child components
 *          2006-08-04 JS added setEnabled(true),so that the user can click 
 *                        "through" this layer
 * 
 * @version $Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 * @responsibilities Shows auxiliary information about the layout managers
 */
public class LayoutVisualizationLayer extends FreeformLayer implements PropertyChangeListener, ZoomEventConstants, IPropertyChangeListener
{
	/**
	 * Size of the head of an arrow
	 */
	private final static int ARROW_HEAD_SIZE = 10;
	
	/**
	 * Size of the starting at the start point of an arrow
	 */
	private final static int ARROW_RECT_SIZE = 3; 
	
	/**
	 * Layer to store arrows in (movement vectors of individually moved nodes)
	 */
	private FreeformLayer arrows;
	
	/**
	 * Layer to store original positions in (of nodes that were moved 
	 * individually)
	 */
	private FreeformLayer originalPositions;
	
	/**
	 * Layer to store original positions in (of nodes that were moved 
	 * when all children of a ZoomableNodeContainer were moved)
	 */
	private FreeformLayer originalChildrenPositions;
	
	/**
	 * Layer to store occlusion rectangles in 
	 */
	private FreeformLayer occlusions;
	
	/**
	 * Layer to show the zoom holes
	 */
	private FreeformLayer zoomHoles;
	
	/**
	 * Layer to show the horizontal intervals of the interval structure
	 */
	private FreeformLayer horizontalIntervals;
	
	/**
	 * Layer to show the vertical intervals of the interval structure
	 */
	private FreeformLayer verticalIntervals;
	
	private IPreferenceStore store;
	

    /**
     * The constructor adds itself as a property change listener to the 
     * Layout algorithms and the plugin preferences.
     */
    public LayoutVisualizationLayer()
    {
    	store = AdoraPlugin.getDefault().getPreferenceStore();
    	
    	setEnabled(false);
        MinRadial.getInstance().addPropertyChangeListener(this);
        NewBerner.getInstance().addPropertyChangeListener(this);
        AdoraZoomAdapter.getInstance().addPropertyChangeListener(this);
        AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);

        arrows = new FreeformLayer();
    	arrows.setForegroundColor(ColorConstants.green);
    	arrows.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ARROWS));
    	arrows.setOpaque(false);
        add(arrows);
        
    	originalPositions = new FreeformLayer();
    	originalPositions.setForegroundColor(ColorConstants.green);
    	originalPositions.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_POSITIONS));
    	originalPositions.setOpaque(false);
    	add(originalPositions);
    	
    	originalChildrenPositions = new FreeformLayer();
    	originalChildrenPositions.setForegroundColor(ColorConstants.blue);
    	originalChildrenPositions.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_CHILD_POSITIONS));
    	originalChildrenPositions.setOpaque(false);
    	add(originalChildrenPositions);
    	
    	occlusions = new FreeformLayer();
    	occlusions.setForegroundColor(ColorConstants.red);
    	occlusions.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_OCCLUSIONS));
    	occlusions.setOpaque(false);
    	add(occlusions);
    	
    	horizontalIntervals = new FreeformLayer();
    	RGB color = PreferenceConverter.getColor(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS_COLOR);
    	refreshForegroundColor(horizontalIntervals, color);
    	horizontalIntervals.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS));
    	horizontalIntervals.setOpaque(false);
    	add(horizontalIntervals);
    	
    	verticalIntervals = new FreeformLayer();
    	color = PreferenceConverter.getColor(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS_COLOR);
    	refreshForegroundColor(verticalIntervals, color);
    	verticalIntervals.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS));
    	verticalIntervals.setOpaque(false);
    	add(verticalIntervals);
    	
    	zoomHoles = new FreeformLayer();
    	color = PreferenceConverter.getColor(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES_COLOR);
    	refreshForegroundColor(zoomHoles, color);
    	zoomHoles.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES));
    	zoomHoles.setOpaque(false);
    	add(zoomHoles);
    	
        setOpaque(false);
        setVisible(true);
    }
    
    
    /**
     * Create the figures that represents the horizontal intervals of the
     * interval structure of the node given as argument and add them to the
     * layer.
     * 
     * @param node the node to create the figures for
     */
    private void createHorizontalIntervalFigures(IZoomNode node) {
    	
    	if(node.isZoomedIn()) {
    		Iterator<java.awt.Rectangle> iter = AdoraZoomAdapter.getInstance().getHorizontalIntervals(node).iterator();
    		while(iter.hasNext()) {
    			java.awt.Rectangle rect = iter.next();
            
    			if(node instanceof Node) {
    				java.awt.Point location = getAbsoluteLocation(node);
    				rect.translate(location.x, location.y);
    			}

    			Polyline leftLine = new Polyline();
    			leftLine.setStart(new Point(rect.x, rect.y));
    			leftLine.setEnd(new Point(rect.x, rect.y + rect.height));
    			//leftLine.setLineStyle(Graphics.LINE_DOT);

    			horizontalIntervals.add(leftLine);

    			if(!iter.hasNext()) {

    				Polyline lastLine = new Polyline();
    				lastLine.setStart(new Point(rect.x + rect.width, rect.y));
    				lastLine.setEnd(new Point(rect.x + rect.width, rect.y + rect.height));
    				lastLine.setLineStyle(Graphics.LINE_DOT);

    				horizontalIntervals.add(lastLine);
    			}

    		}
    	}
    	
    }
    
    
    /**
     * Create the figures that represents the vertical intervals of the
     * interval structure of the node given as argument and add them to the
     * layer.
     * 
     * @param node the node to create the figures for
     */
    private void createVerticalIntervalFigures(IZoomNode node) {
    	
    	if(node.isZoomedIn()) {
    		Iterator<java.awt.Rectangle> iter = AdoraZoomAdapter.getInstance().getVerticalIntervals(node).iterator();
    		while(iter.hasNext()) {
    			java.awt.Rectangle rect = iter.next();

    			if(node instanceof Node) {
    				java.awt.Point location = getAbsoluteLocation(node);
    				rect.translate(location.x, location.y);
    			}

    			Polyline topLine = new Polyline();
    			topLine.setStart(new Point(rect.x, rect.y));
    			topLine.setEnd(new Point(rect.x + rect.width, rect.y));
    			//topLine.setLineStyle(Graphics.LINE_DOT);

    			verticalIntervals.add(topLine);

    			if(!iter.hasNext()) {

    				Polyline lastLine = new Polyline();
    				lastLine.setStart(new Point(rect.x, rect.y + rect.height));
    				lastLine.setEnd(new Point(rect.x + rect.width, rect.y + rect.height));
    				lastLine.setLineStyle(Graphics.LINE_DOT);

    				verticalIntervals.add(lastLine);
    			}

    		}
    	}
    	
    }
    
    
    /**
     * Create the figures that represents the zoom holes of the
     * interval structure of the node given as argument and add them to the
     * layer.
     * 
     * @param node the node to create the figures for
     */
    private void createZoomHoleFigures(IZoomNode node) {
    	
    	if(node.isZoomedIn()) {
    		Iterator<java.awt.Rectangle> iter = AdoraZoomAdapter.getInstance().getZoomHoles(node).iterator();
    		while(iter.hasNext()) {
    			java.awt.Rectangle rect = iter.next();
            
    			if(node instanceof Node) {
    				java.awt.Point location = getAbsoluteLocation(node);
    				rect.translate(location.x, location.y);
    			}
            
    			RectangleFigure figure = new RectangleFigure();
    			figure.setLineWidth(2);
    			figure.setLocation(new Point(rect.x, rect.y));
    			figure.setSize(rect.width, rect.height);
    			figure.setOpaque(false);
    			figure.setFill(false);
            
    			zoomHoles.add(figure);
    		}
    		
    	}
    	
    }
    
    
    /**
     * Return the absolute location of the node given as argument without
     * concerning the borders.
     * 
     * @param node the node to get the absolute location of
     * @return the absolute location
     */
    private java.awt.Point getAbsoluteLocation(IZoomNode node) {
    	
    	java.awt.Point location = node.getZoomBounds().getLocation();
    	IZoomNode parent = node.getZoomParent();
    	while(parent != null) {
    		java.awt.Point parentLoc = parent.getZoomBounds().getLocation();
    		location.translate(parentLoc.x, parentLoc.y);
    		parent = parent.getZoomParent();
    	}
    	
    	return location;
    	
    }
    

    /**
     * Depending on the event, a new figure will be added to this layer, or
     * all figures are removed in case of a NEW_LAYOUT_OPERATION event (i.e.
     * when a new layout operation begins).
     *  
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent event)
    {
        String eventType = event.getPropertyName();
        Object oldValue = event.getOldValue();
        Object newValue = event.getNewValue();

        if(eventType.equals(NEW_LAYOUT_OPERATION))
        {
        	arrows.removeAll();
        	originalPositions.removeAll();
        	occlusions.removeAll();
        	originalChildrenPositions.removeAll();
        	
        	horizontalIntervals.removeAll();
        	verticalIntervals.removeAll();
        	zoomHoles.removeAll();

        }
        else if(eventType.equals(OCCLUSION) && newValue instanceof ch.unizh.ifi.adora.model.geometry.Rectangle)
        {
            occlusions.add(createOcclusionFigure((Rectangle) GeometryFactory.getDraw2dElement(newValue)));
        }
        else if(eventType.equals(POSITION_CHANGE) && oldValue instanceof ch.unizh.ifi.adora.model.geometry.Rectangle && newValue instanceof ch.unizh.ifi.adora.model.geometry.Rectangle)
        {
            Rectangle oldPosition = (Rectangle) GeometryFactory.getDraw2dElement(oldValue);
            Rectangle newPosition = (Rectangle) GeometryFactory.getDraw2dElement(newValue);
            
            originalPositions.add(createOriginalPositionFigure(oldPosition));
            arrows.add(createArrow(oldPosition.getCenter(), newPosition.getCenter()));
        }
        else if(eventType.equals(CHILD_POSITION_CHANGE) && oldValue instanceof ch.unizh.ifi.adora.model.geometry.Rectangle && newValue instanceof ch.unizh.ifi.adora.model.geometry.Rectangle)
        {
            Rectangle oldPosition = (Rectangle) GeometryFactory.getDraw2dElement(oldValue);
            originalChildrenPositions.add(createOriginalPositionFigure(oldPosition));
         }
        else if(eventType.equals(INTERVALS_HORIZONTAL)) {
        	createHorizontalIntervalFigures((IZoomNode)newValue);
        }
        else if(eventType.equals(INTERVALS_VERTICAL)) {
        	createVerticalIntervalFigures((IZoomNode)newValue);
        }
        else if(eventType.equals(ZOOM_HOLES)) {
        	createZoomHoleFigures((IZoomNode)newValue);
        }
    }
    
    /**
     * Observes the preferences to check if the visualization should be hidden/shown.
     * 
     *  @see org.eclipse.core.runtime.Preferences.IPropertyChangeListener#propertyChange(org.eclipse.core.runtime.Preferences.PropertyChangeEvent)
     */
    public void propertyChange(org.eclipse.core.runtime.Preferences.PropertyChangeEvent event)
	{
		String eventType = event.getProperty();
        if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ARROWS))
        {
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ARROWS);
        	arrows.setVisible(visible);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_POSITIONS))
        {
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_POSITIONS);
        	originalPositions.setVisible(visible);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_OCCLUSIONS))
        {
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_OCCLUSIONS);
        	occlusions.setVisible(visible);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_CHILD_POSITIONS))
        {
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_CHILD_POSITIONS);
        	originalChildrenPositions.setVisible(visible);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS))
        {
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS);
        	horizontalIntervals.setVisible(visible);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS_COLOR)) {
    		RGB color = PreferenceConverter.getColor(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS_COLOR);
    		refreshForegroundColor(horizontalIntervals, color);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS))
        {
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS);
        	verticalIntervals.setVisible(visible);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS_COLOR)) {
    		RGB color = PreferenceConverter.getColor(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS_COLOR);
    		refreshForegroundColor(verticalIntervals, color);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES))
        {
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES);
        	zoomHoles.setVisible(visible);
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES_COLOR)) {
    		RGB color = PreferenceConverter.getColor(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES_COLOR);
    		refreshForegroundColor(zoomHoles, color);
        }
	}
    
     /**
      * Creates a figure that visualizes an occlusion.
      * 
      * @param occlusion the intersected area
      * @return an IFigure representing the occlusion
      * @pre occlusion != null
      */
    private IFigure createOcclusionFigure(Rectangle occlusion)
    {
        RectangleFigure rect = new RectangleFigure();
        rect.setOpaque(false);
        rect.setFill(false);
        rect.setBounds(occlusion);
        return rect;
    }
    
    /**
     * Creates a figure that visualizes the original position of a component.
     * 
     * @param originalRect the original position of the component
     * @return an IFigure representing the original position
     * @pre originalRect != null
     */
    private IFigure createOriginalPositionFigure(Rectangle originalRect)
    {
        RectangleFigure rect = new RectangleFigure();
        rect.setOpaque(false);
        rect.setFill(false);
        rect.setBounds(originalRect);
        return rect;
    }

    /**
     * Creates an arrow (used to visualize movement of components).
     * 
     * @param start the start of the arrow
     * @param end the end of the arrow
     * @return an arrow pointing from start to end
     * @pre start != null && end != null
     */
    private IFigure createArrow(Point start, Point end)
    {
    	IFigure arrow = new FreeformLayer();
    	
    	// the start of the arrow
    	RectangleFigure rect = new RectangleFigure();
    	Point rectLocation = start.getTranslated(-ARROW_RECT_SIZE/2, -ARROW_RECT_SIZE/2);
    	rect.setLocation(rectLocation);
    	rect.setSize(ARROW_RECT_SIZE, ARROW_RECT_SIZE);
    	arrow.add(rect);
    	
    	// the line of the arrow
    	Polyline line = new Polyline();
    	line.setEndpoints(start, end);
    	arrow.add(line);
    	
    	// the head of the arrow
    	PolylineDecoration decoration = new PolylineDecoration();
    	decoration.setScale(ARROW_HEAD_SIZE, ARROW_HEAD_SIZE / 2);
    	Dimension arrowDimension = end.getDifference(start);
    	double arrowLength = Math.sqrt(arrowDimension.width*arrowDimension.width+arrowDimension.height*arrowDimension.height);
    	// if the length is 0, just use an arbitrary value for the 
    	// direction of the arrow
    	if(arrowLength == 0)
    	{
    		arrowLength = 1;
    	}
    	double angle = Math.acos(arrowDimension.width/arrowLength);
    	// the angle gets calculated the wrong way if the arrow points down,
    	// so switch its direction in this case
    	if(arrowDimension.height<0)
    	{
    		angle *= -1;
    	}
    	decoration.setRotation(angle);
    	decoration.setLocation(end);
    	arrow.add(decoration);
    	
    	arrow.setOpaque(false);
    	
    	return arrow;
    }
    
    
    private void refreshForegroundColor(Layer layer, RGB newColor) {
    	
    	Color oldColor = layer.getForegroundColor();
		
		if(oldColor != null && !oldColor.isDisposed()) {
			oldColor.dispose();
		}
		
		layer.setForegroundColor(new Color(null, newColor));
    	
    }
    
	@Override
	public void removeNotify() {
		AdoraPlugin.getDefault().getPluginPreferences().removePropertyChangeListener(this);
		super.removeNotify();
	}
}
