package ch.unizh.ifi.adora.view.layers;

import java.util.HashMap;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.FreeformLayeredPane;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.gef.LayerConstants;
import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.representation.routing.algorithm.OrthogonalRouter;
import ch.unizh.ifi.adora.representation.routing.algorithm.directline.DirectLineRouter;
import ch.unizh.ifi.adora.representation.routing.algorithm.shortestpath.SPManager;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.view.figures.AssociationFigure;
import ch.unizh.ifi.adora.view.figures.ConnectionFigure;
import ch.unizh.ifi.adora.view.figures.JoinRelationFigure;
import ch.unizh.ifi.adora.view.figures.ScenarioConnectionFigure;
import ch.unizh.ifi.adora.view.figures.TransitionFigure;

/**
 * Replacement for the gef connection layer that allows the usage of different line
 * routers for the different connection types.   
 *
 * @author             Tobias Reinhard
 * @copyright          Requirements Engineering Research Group<br>
 *                     Department of Informatics<br>
 *                     University of Zurich<br>
 * @history            2006-04-25 TR First Version
 *                     2006-05-20 SM Extended version for join relationship router
 * @version            $ Date: 2006/04/20 11:59:51 $, $ Author: vigan $, $ Revision: 1.34 $
 * @responsibilities   Connection layer with multiple different routers.
 *
 */
public class MultipleRoutersConnectionLayer extends ConnectionLayer implements IPropertyChangeListener {
 
    
	/**
	 * Key for the BendpointConnectionRouter.
	 */
	public static String BendpointConnectionRouter = "BendpointConnectionRouter";
	
	/**
	 * Key for the ShortestPathConnectionRouter.
	 */
	public static String ShortestPathConnectionRouter = "ShortestPathConnectionRouter";
	
	/**
	 * Key for the DirectLineRouter.
	 */
	public static String DirectLineRouter = "DirectLineRouter";
	
	/**
	 * Key for the orthogonal router (replacement for the OrthogonalTileMazeRouter)
	 */
	public static String OrthogonalRouter = "OrthogonalRouter";
	
	/**
     * Mapping from the router names to the actual router instances
     */
    private HashMap<String, ConnectionRouter> routers;
    
    /**
     * The preference store (used to get the routers that are set in the preferences)
     */
    private IPreferenceStore store;
    
    
    /**
     * Constructor
     *
     */
    public MultipleRoutersConnectionLayer() {
        
        super();
        
        store  = AdoraPlugin.getDefault().getPreferenceStore();
        AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);   
        
    }
    
    /**
     * Stores the different routers in the name to routers map.
     *
     *@pre true
     *@post true
     */
    public void initializeRouters() {
        
    	FreeformLayeredPane layeredPane = (FreeformLayeredPane)getParent();
        Layer primaryLayer = layeredPane.getLayer(LayerConstants.PRIMARY_LAYER);
    	
        routers = new HashMap<String, ConnectionRouter>();
        routers.put(BendpointConnectionRouter, new BendpointConnectionRouter());
        routers.put(ShortestPathConnectionRouter, new SPManager((IFigure)primaryLayer.getChildren().get(0), this));
        routers.put(DirectLineRouter, new DirectLineRouter((IFigure)primaryLayer.getChildren().get(0), this));
        routers.put(OrthogonalRouter, new OrthogonalRouter((IFigure)primaryLayer.getChildren().get(0), this));
        
    }
    
    
    /**
     * Adds the given figure with the given constraint at the given index. If the figure is a
     * {@link Connection}, its {@link ConnectionRouter} is set.
     *
     * @param figure  Figure being added
     * @param constraint  Constraint of the figure being added
     * @param index  Index where the figure is to be added
     * @since 2.0 
     */
    public void add(IFigure figure, Object constraint, int index) {
        super.add(figure, constraint, index);
        
        // If the connection layout manager is set, then every
        // figure added should use this layout manager.
        if (figure instanceof Connection) {
            
            Connection connection = (Connection)figure;
            
            connection.setConnectionRouter(getRouter(figure.getClass()));
        } 
        
    }
    
    
    /**
     * Handler method for preference changes.
     * 
     */
    public void propertyChange(Preferences.PropertyChangeEvent evt) {
        
        if(evt.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_ASSOCIATION)) {       
            setRouterForType(AssociationFigure.class, getRouter(AssociationFigure.class));
        }
        else if (evt.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_TRANSITION)) {
            setRouterForType(TransitionFigure.class, getRouter(TransitionFigure.class));     
        }
        else if(evt.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_SCENARIOCONNECTION)) {
            setRouterForType(ScenarioConnectionFigure.class, getRouter(ScenarioConnectionFigure.class)); 
        }      
        else if(evt.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_JOIN_RELATIONSHIP)) {
            setRouterForType(JoinRelationFigure.class, getRouter(JoinRelationFigure.class)); 
        }      

    }
    
    /**
     * Used to get the router for reflexive connections.
     * 
     * @param router HashKey
     * @return router according to the passed key.
     */
    public ConnectionRouter getSpecificRouter(String router) {
        return routers.get(router);
    }
    
    /**
     * Returns an instance of the router that is responsible for the routing of a
     * connection of the type <i>connectionType</i>.
     * 
     * @param connectionType the type of the connection
     * @return an instance of the responsible line router
     */
    public ConnectionRouter getRouter(Class connectionType) {
        
        if(routers == null) {        
            initializeRouters();          
        }
        
        if(connectionType == AssociationFigure.class) {
            String routerName = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_ASSOCIATION);
            return routers.get(routerName);
        }
        else if(connectionType == TransitionFigure.class) {
            String routerName = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_TRANSITION);
            return routers.get(routerName);
        }
        else if(connectionType == ScenarioConnectionFigure.class) {
            String routerName = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_SCENARIOCONNECTION);
            return routers.get(routerName);
        } else if(connectionType == JoinRelationFigure.class) {
            String routerName = store.getString(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_JOIN_RELATIONSHIP);
            return routers.get(routerName);
        }
        else if(connectionType == PolylineConnection.class) {
            return routers.get("ShortestPathConnectionRouter");
        }
        
        return routers.get("OrthogonalTileMazeRouter");
        
    }
    
    /**
     * Set the router <i>router</i> for all the existing connections of the type 
     * <i>connectionType</i>.
     * 
     * @param connectionType the type of the connection
     * @param router an instance of the responsible router
     */
    private void setRouterForType(Class connectionType, ConnectionRouter router) {
        
        FigureIterator iter = new FigureIterator(this);
        IFigure figure;
        while (iter.hasNext()) {
            figure = iter.nextFigure();
            if (figure.getClass() == connectionType)
                ((ConnectionFigure)figure).setConnectionRouter(router);
        }     
    }
    
    
    public void validate() {
    	if(isVisible()) {
    		super.validate();
    	}
    }

	@Override
	public void removeNotify() {
		AdoraPlugin.getDefault().getPluginPreferences().removePropertyChangeListener(this);
		super.removeNotify();
	}
}