package ch.unizh.ifi.adora.empiricism.adoraeditor.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStackEvent;
import org.eclipse.gef.commands.CommandStackEventListener;
import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.empiricism.Activator;
import ch.unizh.ifi.adora.empiricism.model.DataBaseSQLStrings;
import ch.unizh.ifi.adora.empiricism.model.Database;
import ch.unizh.ifi.adora.empiricism.preferences.PreferencesConstants;

/**
 * This class is used to save the {@link Command}s - specified in the preference page - 
 * into the database.  
 *
 * @author		        vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             25.11.2008 IV First Version
 * @version             $Date: 2008-12-02 14:44:15 +0100 (Tue, 02 Dec 2008) $, $Author: vigan $
 */
public class AdoraCommandListener implements CommandStackEventListener{

	/**
	 * Stores the {@link Database} object.
	 */
	private Database db = null;
	
	/**
	 * Stores the ID of the user.
	 */
	private int userID = -1;
	
	/**
	 * Singleton instance of this class.
	 */
	private static AdoraCommandListener instance;
	
	/**
	 * The {@link IPreferenceStore} which stores whether a {@link Command} should be saved
	 * into the DB or not. 
	 */
	IPreferenceStore store = Activator.getDefault().getPreferenceStore();
	
	/**
	 * Private constructor to insure single instance of this class.
	 */
	private AdoraCommandListener() {}
	 
	/**
	 * Static method to implement the Singleton-Pattern. 
	 * 
	 * @return the singleton instance of this class.
	 */
    public synchronized static AdoraCommandListener getInstance() {
        if (instance == null) {
            instance = new AdoraCommandListener();
        }
        return instance;
    }
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.CommandStackEventListener#stackChanged(org.eclipse.gef.commands.CommandStackEvent)
	 */
	public void stackChanged(CommandStackEvent event) {
		String loggedCmdStr = store.getString(PreferencesConstants.LOGGED_COMMANDS);
        String[] logCmds = loggedCmdStr.split(",");
        String cmdClassName = event.getCommand().getClass().getSimpleName();
        for (int i = 0; i < logCmds.length; i++) {
        	if (cmdClassName.equals(logCmds[i].trim())) {
        		saveToDB(event.getCommand().toString());
        		break;
        	}
        }
	}
	
	/**
	 * Saves the {@link Command}, the userID and a timestamp 
	 * into a table of the {@link Database}. 
	 * 
	 * @param cmdName The name of the {@link Command} to be saved.
	 */
	private void saveToDB(String cmdName) {
		if (db == null)
			return;
		
		Connection con = db.getConnection();
		
		PreparedStatement pstmt;
		try {
			pstmt = con.prepareStatement(DataBaseSQLStrings.SQL_CREATE_LOGGED_COMMANDS);
			pstmt.setInt(1, userID);  
			pstmt.setString(2, cmdName);
			
			pstmt.execute();
			pstmt.close();
		
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Sets the {@link Database} in which the {@link Command}s get stored.
	 * 
	 * @param adoraDB {@link Database} where we store things.
	 */
	public void setDB(Database adoraDB) {
		if (db != null) 
			return;
		db = adoraDB;
	}
	
	/**
	 * Sets the ID of the user which executes the {@link Command}s.
	 * 
	 * @param userID The ID of the user.
	 */
	public void setUserID(int userID) {
		this.userID = userID;
	}
}