package ch.unizh.ifi.adora.empiricism.model;


public abstract class AbstractTopic extends DatabaseObject implements TopicDataTypes{

	/**
	 * The id of the question. This id is used to uniquely identify the correspoinding 
	 * tuple in the database.
	 */
	protected int id = -1;	
	
	/**
	 * The question as string.
	 */
	protected String question = "";
	
	/**
	 * The correct answer as string. This answer is only set unequal to the empty string 
	 * if  the {@link #answerType} is equal to one of the following values:
	 * {@link #T_FLOAT}, {@link #T_INT} or {@link #T_STRING}. If the {@link #answerType} is
	 * {@link #T_MC} or {@link #T_OPTION} then this attribute should be the empty string.
	 */
	protected String answer   = "";
	
	/**
	 * The type of the answer. If one of {@link #T_FLOAT}, {@link #T_INT} or {@link #T_STRING}
	 * is set as value, this means that there is the correct answer represented as string in the 
	 * attribute {@link #answer}. Otherwise the possible answers are mutlitple choice answers 
	 * (see {@link ch.unizh.ifi.adora.empiricism.model.QuestionMcItem}) associated with this question. 
	 */
	protected int answerType  = T_NO;
	
	/**
	 * This is the test suite which this question belongs to.
	 */
	protected TestSuite suite = null;
	
	/**
	 * Creates a new question including also a new tuple in the database
	 * associated with a given test suite in a given database.
	 * 
	 * @param db The database to which this question belongs to.
	 * @param suite The TestSuite object to which this question belongs to.
	 */
	public AbstractTopic(Database db, TestSuite suite) {
		super(db);
		this.suite = suite;
	}
	
	/**
	 * This constructor initializes a new Question object with the content of a
	 * question tuple in the database.
	 * 
	 * @param id The id of the question tuple in the database.
	 * @param db The database to which the question belongs.
	 */
	public AbstractTopic(int id, Database db) {
		super(db);
		this.id = id;
	}
	
	public AbstractTopic(Database db) {
		super(db);
	}
	
	/**
	 * Returns the unique id of this question object.
	 * 
	 * @return Teturns the unique id of this question object.
	 */
	public int getId() {
		return id;
	}
	
	/**
	 * Returns the question string. 
	 * 
	 * @return Returns the question string.
	 */
	public String getQuestion() {
		return question;		
	}
	
	/**
	 * Sets the question string.
	 * @param question Sets the question string.
	 */
	public void setQuestion(String question) {
		this.question = question;
		changed = true;
	}
	
	/**
	 * Returns the answer string of the correct answwre which is set 
	 * if the {@link #getAnswerType()} method returns
	 * one of the type {@link #T_FLOAT}, {@link #T_INT} or {@link #T_STRING}.
	 * 
	 * @return Returns the answer string of the correct answer. 
	 */
	public String getAnswer() {
		return answer;
	}
	
	/**
	 * Sets the answer string of the correct answer. This string is should only be set if
	 * the {@link #getAnswerType()}  method returns
	 * one of the type {@link #T_FLOAT}, {@link #T_INT} or {@link #T_STRING}.
	 * @param answer The string of the correct answer.
	 */
	public void setAnswer(String answer) {
		this.answer = answer;
		changed = true;
	}
	
	/**
	 * Returns the type of the answer. If the type is one of 
	 * {@link #T_FLOAT}, {@link #T_INT} or {@link #T_STRING} there should be
	 * the correct answer represented as string which can be get by calling {@link #getAnswer()}.
	 * If the type of the answer is either {@link #T_MC} or {@link #T_OPTION}, there should be 
	 * {@link ch.unizh.ifi.adora.empiricism.model.QuestionMcItem} objects instead of an answer given by
	 * the method {@link #getAnswer()}.
	 * 
	 * @return Returns the type of the answer. 
	 */
	public int getAnswerType() {
		return this.answerType;
	}
	
	/**
	 * Sets the type of the answer. If the type is one of 
	 * {@link #T_FLOAT}, {@link #T_INT} or {@link #T_STRING} there should be
	 * the correct answer represented as string which can be get by calling {@link #getAnswer()}.
	 * If the type of the answer is either {@link #T_MC} or {@link #T_OPTION}, there should be 
	 * {@link ch.unizh.ifi.adora.empiricism.model.QuestionMcItem} objects instead of an answer given by
	 * the method {@link #getAnswer()}.
	 * 
	 * @param type Sets the type of the answer.
	 */
	public void setAnswerType(int type) {
		this.answerType = type;
		changed = true;
	}
	
	/**
	 * Returns the test suite associated with this question.
	 * @return Returns the test suite associated with this question.
	 */
	public TestSuite getTestSuite() {
		return suite;
	}
	
	/**
	 * Sets the test suite associated with this Question.
	 * 
	 * @param suite The test suite which is associated with this question.  
	 */
	public void setTestSuite(TestSuite suite) {
		this.suite = suite;
		changed = true;
	}
}

