package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * An adora property object represents a propertyName, propertyValue pair, 
 * associated with the adora model of a quesiton. 
 * The propertyName, propertyValue pair is used to change the preferences of 
 * the displayed adora model. (e.g. Linerouting) 
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             30.06.2006 SM First Version
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 */
public class AdoraProperty extends DatabaseObject {

	/**
	 * Question, for whose adora model the property is used.
	 */
	private Question q;
	
	/**
	 * Name of the property.
	 */
	private String propertyName  = "";
	
	/**
	 * Value of the property.
	 */
	private String propertyValue = "";
	
	/**
	 * Creates a AdoraProperty object from an existing AdoraProperty record in the database.
	 * 
	 * @param db The database where the  AdoraProperty is stored..
	 * @param q Question, for whose adora model the property is used.
	 * @param propertyName Name of property.
	 */
	public AdoraProperty(Database db, Question q, String propertyName) {
		super(db);
		this.q = q;
		this.propertyName = propertyName;
		refreshData();
	}
	
	/**
	 * Creates a new AdoraProperty in a given database.
	 * 
	 * @param db The database in which the AdoraProperty will be created.
	 * @param q
	 */
	public AdoraProperty(Database db, Question q) {
		super(db);
		this.q = q;
		createObject();
	}	
	
	@Override
	protected String getNextIdSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_LAST_ADORA_PROPERTY_ID;
	}

	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_ADORA_PROPERTY;
	}

	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_ADORA_PROPERTY;
	}

	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_ADORA_PROPERTY;
	}

	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_ADORA_PROPERTY;
	}

	@Override
	protected void prepareCache() {
		// TODO Auto-generated method stub
	}

	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, q.getId());
		statement.setString(2, propertyName);
		statement.setString(3, propertyValue);
	}

	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, q.getId());
		statement.setString(2, propertyName);
	}

	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, q.getId());
		statement.setString(2, propertyName);
	}

	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setInt(1, q.getId());
		statement.setString(2, propertyName);
		statement.setString(3, propertyValue);
	}

	@Override
	protected void readData(ResultSet result) throws SQLException {
		propertyName  = result.getString(1);
		propertyValue = result.getString(2);
	}

	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		//this.
	}

	/**
	 * @return Name of the property.
	 */
	public String getPropertyName() {
		return propertyName;
	}

	/**
	 * @return Value of the property.
	 */
	public String getPropertyValue() {
		return propertyValue;
	}

	/**
	 * Question, for whose adora model the property is used.
	 * 
	 * @return Question, for whose adora model the property is used.
	 */
	public Question getQuestion() {
		return q;
	}

	/**
	 * Sets the name of the AdoraProperty.
	 * @param propertyName Name of the AdoraProperty.
	 */
	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
		changed = true;
	}

	/**
	 * Sets the value of the AdoraProperty.
	 * @param propertyValue Vvalue of the AdoraProperty.
	 */
	public void setPropertyValue(String propertyValue) {
		this.propertyValue = propertyValue;
		changed = true;
	}
}