package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

/**
 * This class represents a set of data objects, that are mapped to a relational
 * database.
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 24, 2006 SM First Version
 *                      May 28, 2006 SM Some refactoring
 *                      May 29, 2006 SM Comments
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 *
 */
public abstract class DBObjectSet extends ArrayList {

	/**
	 * The reference to the database.
	 */
	protected Database db = null;

	/**
	 * Creates the object set by setting the database.
	 *
	 * @pre       db != null
	 * @post      true
	 * @param db  The database object which represents the relational database object.
	 */
	protected DBObjectSet(Database db) {
		this.db = db;
	}

	/**
	 * This method rereads all objects from the database associated with
	 * this object set.
	 */
	public void refreshObjects() {
		this.clear();
		ResultSet resultSet = getDBObjects();
		addObjects(resultSet);
	}

	/**
	 * Adds all tuples as database object to this list by
	 * iterating over a result set. The result set was retrieved
	 * by executing the query given by the method {@link #getSelectStatement()}.
	 *
	 * @param resultSet The resultset containing the data for the creation of the
	 *                  objects.
	 */
	protected void addObjects(ResultSet resultSet) {
		try {
			while (resultSet.next()) {
				Object o = createObject(resultSet);
				this.add(o);
			}
			resultSet.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * This method creates one object in the set from a given resultset. The given
	 * result set is positioned correctly, i.e. the current tuple represents the
	 * data for the creation of the object.
	 */
	protected Object createObject(ResultSet resultSet) {
		Object u=null;
		try {
			u = createThisObject(resultSet);
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return u;
	}

	/**
	 * This method returns a result set from the data base which contains the
	 * data for creating the set of objects. This data is retrieved by executing the
	 * SQL statement given by the method {@link #getSelectStatement()} and prepared
	 * by the method {@link #prepareSelectStatement(PreparedStatement)}.
	 * @return Returns a result set with data that is used to construct the set of
	 *         objects.
	 */
	protected ResultSet getDBObjects() {
		ResultSet resultSet = null;
		try {
			PreparedStatement statement = db.getConnection().prepareStatement(getSelectStatement());
			prepareSelectStatement(statement);
			resultSet = statement.executeQuery();
			statement.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return resultSet;
	}

	/**
	 * Returns a select statement which retrieves the data needed for constructing
	 * this object set. The preparation of the select statement is done by the method
	 * {@link #prepareSelectStatement(PreparedStatement)}.
	 *
	 * @return Returns a SQL select statement containing the SQL that is used to
	 *         retrieve the data needed to construct this object set.
	 */
	protected abstract String getSelectStatement();

	/**
	 * This method prepares the select statement given by the method {@link #getSelectStatement()}
	 * for the executing in the database. The preparation can be used for parametrizing the
	 * query, i.e. setting some speicific value. This is done by using the given
	 * {@link java.sql.PreparedStatement}.
	 *
	 * @pre statement != null
	 * @post true
	 * @param statement The prepared statement which can be used to parametrize the
	 *        select query given by the method {@link #getSelectStatement()}.
	 */
	protected abstract void prepareSelectStatement(PreparedStatement statement);

	/**
	 * Creates a speicific object from the current tupel given by the object rs.
	 * The position in the record set is set to the tupel to create. The tupel for
	 * each database object to create results in exactly <i>one</i> call of this method.
	 *
	 * @pre rs != null
	 * @post true
	 * @param  rs The record set which contains data for the database object to create.
	 * @return Returns the created object which is added to this list.
	 */
	protected abstract Object createThisObject(ResultSet rs) throws SQLException;
}
