package ch.unizh.ifi.adora.empiricism.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Class with helper methods for creating a database schema.
 *
 * @author		        smeier, Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 24, 2006 SM First Version
 *                      May 29, 2006 SM Comments
 *                      Oct 13, 2006 SM Refactoring, implemented the access for the
 *                      content of an inquery from a http server.
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 *
 */
public class DataBaseCreator {

	/**
	 * Resource path, where the sql files are stored.
	 */
	public static final String sqlFilePath		= "/ch/unizh/ifi/adora/empiricism/model/dbschema/";

	/**
	 * Indicator for a comment.
	 */
	private static final String commentIndicator 	= "--";

	/**
	 * Indicates the start and the end of a macro filename,
	 * contained in a sql file.
	 */
	private static final String macroIndicator 	= "@";

	/**
	 * Stores the used character code.
	 */
	private static final  String textEncoding		= "UTF-8";

	/**
	 * Temporary buffer variable that is used for storing the characters after a
	 * semicolon.
	 */
	private StringBuffer temp= new StringBuffer("");

	/**
	 *
	 */
	private Connection connection = null;

	/**
	 *
	 */
	private String contentBaseDirectory = "";

	/**
	 *
	 * @param c
	 */
	public DataBaseCreator(Connection c, String contentBaseDirectory) {
		this.connection = c;
		if (!contentBaseDirectory.equals("")) this.contentBaseDirectory = contentBaseDirectory + ((contentBaseDirectory.endsWith("/"))? "" : "/");
	}

	/**
	 * This method reads a given text input stream and executes each block separated
	 * by a semicolon as a database sql statement.
	 *
	 * @pre inputFile != null && db != null
	 * @post true
	 * @param inputFile The input text file containing the semicolon separated
	 *        SQL statements.
	 * @param db The database for which the sql statements should be executed.
	 */
	public void executeSQLScript(InputStream inputFile) {
		// The input stream wrapped in a reader.
		InputStreamReader isr = null;
		StringBuffer line = new StringBuffer("");
		// get the connection from the database.

		try {
			isr = new InputStreamReader(inputFile, textEncoding);
		} catch (UnsupportedEncodingException e1) {
			e1.printStackTrace();
		}

		// the input stream as buffered reader.
		BufferedReader br = new BufferedReader(isr);
		// temp var for the line content.


		// --- Set up the database ---
		try {

			while (br.ready()) {
				// Create a Statement object to execute the queries on,

				line = readNextStatement(br, connection); // read all lines until the next semicolon is found

				executeCommand(line);
			}
			br.close();
			isr.close();
			// close database connection, no longer used
		} catch (SQLException e) {
			System.out.println("The query: " + line + " caused an exception!");
			System.out.println("An error occured\n"
					+ "The SQLException message is: " + e.getMessage());
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Executes one single line that was read from the input stream.
	 * If an sql command is contained in that line, the command is searched
	 * for makros indicated by @makroname@.
	 * If a makro is found, an sql statement is prepared which contains files
	 * that are read from a given location. If a {@link #contentBaseDirectory}
	 * is given, the content is read by creating an url from the
	 * {@link #contentBaseDirectory} plus the name
	 * of the current makro. Otherwise the name of the macro is searched in the
	 * local classpath in the directory given by {@link #sqlFilePath}.
	 *
	 * @pre       line !0 null
	 * @post      true
	 * @param line The StringBuffer which contains the sql command to execute.
	 */
	public void executeCommand(StringBuffer line) throws Exception {
		Statement statement = null;


		InputStream macroStream=null;


		String currMakro = "";
		Vector<String> makros = new Vector<String>();


		//Extract Macros
		currMakro = extractMacro(line);
		makros.clear();

		// first get all makro names
		while (!currMakro.equals("")) {
			makros.add(currMakro);
			currMakro = extractMacro(line);
		}

		// second load the particular files into the prepared statement.
		if (makros.size() > 0) {
			// the sql command contains makros, ie. binary files to include
			// create a prepared statement.
			statement = connection.prepareStatement(line.toString());
			Enumeration<String> e = makros.elements();

			while (e.hasMoreElements()) {
				try {
					currMakro = e.nextElement();
					if (contentBaseDirectory.equals("")) {
						macroStream = Class.forName("ch.unizh.ifi.adora.empiricism.model.DataBaseCreator").getResourceAsStream(sqlFilePath + currMakro);
					} else {
						File f = new File(contentBaseDirectory + currMakro);
						System.out.println(f);
						if (f.exists()) {
							macroStream = new java.io.FileInputStream(f);
						} else {
							URLConnection connection;
							connection = (new URL(contentBaseDirectory + currMakro)).openConnection();
							connection.connect();
							macroStream = connection.getInputStream();
						}
					}

					if (macroStream != null){
						//number of bytes to be read.
						int cntBytes = macroStream.available();
						((PreparedStatement)statement).setBinaryStream(1, macroStream, cntBytes);
						macroStream.close();
						macroStream = null;
					}
				} catch (ClassNotFoundException ex) {
					ex.printStackTrace();
				}
			}
			((PreparedStatement)statement).execute();
		} else {
			if (!line.equals("")) {
				statement = connection.createStatement();
				statement.executeQuery(line.toString());
			}
		}

		// Close the statement and the connection.
		statement.close();
	}

	/**
	 * This method reads the next statement block for the creation of the database.
	 *
	 * @param br The buffered reader from which the next statement block
	 *           should be read.
	 * @return Returns a string which contains the next statement block to
	 *         execute.
	 */
	public StringBuffer readNextStatement(BufferedReader br, Connection connection) {
		// get the remainings of the last read line as start for the input.
		StringBuffer outLine = new StringBuffer(temp);
		if (!temp.toString().trim().equals("")) {
			outLine.append("\n");
		}

		StringBuffer rl;

		temp = new StringBuffer("");
		boolean reading = true;
		try {
			while (br.ready() && reading) {
				rl = new StringBuffer(br.readLine());

				// check for comments and remove them.
				int startIndex = rl.indexOf(commentIndicator);
				if (startIndex > -1) {
					rl = new StringBuffer(rl.substring(0, startIndex));
				}

				if (rl.lastIndexOf(";") >= 0) {

					temp = new StringBuffer(rl.substring(rl.lastIndexOf(";") + 1, rl.length()));
					outLine.append(" " + rl.substring(0, rl.lastIndexOf(";")));
					reading = false;
				} else {
					outLine.append("\n" + rl);
				}
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return outLine;
	}


	/**
	 * Extracts the makro from the sql command and replaces it by a question mark,
	 * for the creation of a prepared statement.
	 *
	 * @param line The line which gets analysed if it contains a macro filename.
	 */
	public String extractMacro(StringBuffer line){
		String fileName="";
		int startIndex = line.indexOf(macroIndicator);
		int endIndex = line.indexOf(macroIndicator, startIndex + 1);

		if (startIndex > -1 && endIndex > startIndex ) {
			fileName = line.substring(startIndex+1, endIndex);
			line.replace(startIndex, endIndex + 1, " ? ");
		}
		return fileName;
	}
}