package ch.unizh.ifi.adora.empiricism.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.empiricism.Activator;
import ch.unizh.ifi.adora.empiricism.adoraeditor.model.AdoraCommandListener;
import ch.unizh.ifi.adora.empiricism.model.cache.AdoraPropertyCache;
import ch.unizh.ifi.adora.empiricism.model.cache.ModelForQuestionCache;
import ch.unizh.ifi.adora.empiricism.model.cache.QuestionCache;
import ch.unizh.ifi.adora.empiricism.model.cache.QuestionMcItemCache;
import ch.unizh.ifi.adora.empiricism.model.cache.TestSuiteCache;
import ch.unizh.ifi.adora.empiricism.model.cache.UserAnswerOfQuestionCache;
import ch.unizh.ifi.adora.empiricism.model.cache.UserAnswersOfMcItemCache;
import ch.unizh.ifi.adora.empiricism.model.cache.UserAnswersOfUserInfoCache;
import ch.unizh.ifi.adora.empiricism.model.cache.UserAnswersOfUserInfoMcAnswersCache;
import ch.unizh.ifi.adora.empiricism.model.cache.UserCache;
import ch.unizh.ifi.adora.empiricism.model.cache.UserInfoCache;
import ch.unizh.ifi.adora.empiricism.model.cache.UserInfoMcItemCache;
import ch.unizh.ifi.adora.empiricism.preferences.PreferencesConstants;

/**
 * This class represents the database of the persistent model. It is used througout the
 * classes which represent persistent data.
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 22, 2006 SM First Version
 *                      May 29, 2006 SM Comments
 *                      Oct 13, 2006 SM Implemented the access for the content of an inquery
 *                      from a http server.
 *
 * @version             $Date: 2008-11-28 11:41:23 +0100 (Fri, 28 Nov 2008) $, $Author: vigan $, $Revision: 3907 $
 */
public class Database implements PreferencesConstants {

	/**
	 * Cache for a Question.
	 */
	private QuestionCache questionCache = new QuestionCache();


	/**
	 * Cache for a Question.
	 */
	private ModelForQuestionCache modelForQuestionCache = new ModelForQuestionCache();


	/**
	 * Cache for a QuestionMcItem.
	 */
	private QuestionMcItemCache questionMcItemCache = new QuestionMcItemCache();

	/**
	 * Cache for a TestSuite.
	 */
	private TestSuiteCache testSuiteCache = new TestSuiteCache();

	/**
	 * Cache for a UserAnswerOfQuestion.
	 */
	private UserAnswerOfQuestionCache userAnswerOfQuestionCache = new UserAnswerOfQuestionCache();

	/**
	 * Cache for a User.
	 */
	private UserCache userCache = new UserCache();

	/**
	 * Cache for a UserAnswersOfMcItem.
	 */
	private UserAnswersOfMcItemCache userAnswersOfMcItemCache = new UserAnswersOfMcItemCache();

	/**
	 * Cache for a UserAnswersOfUserInfo.
	 */
	private UserAnswersOfUserInfoCache userAnswersOfUserInfoCache = new UserAnswersOfUserInfoCache();

	/**
	 * Cache for a UserAnswersOfUserInfoMcAnswers.
	 */
	private UserAnswersOfUserInfoMcAnswersCache userAnswersOfUserInfoMcAnswersCache = new UserAnswersOfUserInfoMcAnswersCache();

	/**
	 * Cache for a UserInfoMcItem.
	 */
	private UserInfoMcItemCache userInfoMcItemCache = new UserInfoMcItemCache();

	/**
	 * Cache for a UserInfo.
	 */
	private UserInfoCache userInfoCache = new UserInfoCache();

	/**
	 * Cache for a adoraProperty.
	 */
	private AdoraPropertyCache adoraPropertyCache = new AdoraPropertyCache();

	/**
	 * Indicates whether the database is shut down when closing
	 * the connection or not.
	 */
	private boolean shutDownOnCloseConnection = false;

	/**
	 * The used connection String which describes how to access the database.
	 */
	private String connectionString = "";

	/**
	 * User name of the database user.
	 */
	private String userid = "";
	
	/**
	 * Password of the database user.
	 */
	private String password = "";
	
	/**
	 * The connection to the database.
	 */
	private Connection connection;

	/**
	 * This flag indicates whether the database schema should be
	 * initialized or not.
	 */
	private boolean initDatabase = false;

	/**
	 * Checks for the jdbc connection to the database and creates the table
	 * structure iff not existant.
	 */
	public Database() {
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		String driverName = store.getString(PreferencesConstants.DB_DRIVER_NAME);
		try {
			Class.forName(driverName).newInstance();
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		refreshConnectionString();
		checkDBSchema();
		
		AdoraCommandListener.getInstance().setDB(this);
	}

	/**
	 * Sets the flag which causes the database to initialize.
	 *
	 * @pre       true
	 * @post      true
	 * @param init Indicates whether the database should be reinitialized
	 *         or not.
	 */
	public void setInitDatabase(boolean init) {
		this.initDatabase = init;
	}

	/**
	 * Returns whether the database will be initializes when the Database
	 * object is initialized or not.
	 *
	 * @pre       true
	 * @post      true
	 * @return	Returns either true or false indicating
	 */
	public boolean isInitDatabaseSet() {
		return this.initDatabase;
	}


	/**
	 * Returns the currently used connection string.
	 *
	 * @pre true
	 * @post true
	 * @return Returns the currently used connection string.
	 */
	public String getConnectionString() {
		return connectionString;
	}

	/**
	 * Reads the connection string from the plugin's storage.
	 * @pre true
	 * @post true
	 */
	protected void refreshConnectionString() {
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		this.connectionString = (String)store.getString(DB_CONNECTION_PARAM);
		this.userid = (String)store.getString(DB_USER_NAME);
		this.password = (String)store.getString(Database.DB_USER_PASSWORD);
		
		if (this.connectionString == null || this.connectionString.equals("") ){
			this.connectionString = "jdbc:hsqldb:file:" + System.getProperty("user.home") + "/adora/empiricaldb";
			store.setValue(DB_CONNECTION_PARAM, connectionString);
		}
	}

	/**
	 * This method checks for the existance of the used database structure.
	 * If the data base schema does not exist, it will be created.
	 */
	protected void checkDBSchema() {
		boolean createDB = false;
		try {
			Connection conn = this.getConnection();
			Statement statement = conn.createStatement();
			ResultSet rs = statement.executeQuery("SELECT * FROM INFORMATION_SCHEMA.SYSTEM_TABLES WHERE TABLE_NAME='USER'");
			createDB =! rs.next();
			rs.close();
			statement.close();

		} catch (Exception ex) {
			ex.printStackTrace();
		}

		if (createDB || initDatabase) {
			createDB();
		}
	}

	/**
	 * Creates the database schema in the case of non existance.
	 */
	protected void createDB() {
		InputStream is;
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		URLConnection connection;


		// database content and location.
		String contentDirectory = store.getString(PreferencesConstants.DB_CONTENT_DIRECTORY);
		String contentFile      = store.getString(PreferencesConstants.DB_CONTENT_FILE_NAME);
		if (!contentDirectory.equals("")) contentDirectory += ((contentDirectory.endsWith("/"))? "": "/");

		DataBaseCreator dbc = new DataBaseCreator(this.getConnection(),contentDirectory);

		try {
			is = getClass().getResourceAsStream(DataBaseCreator.sqlFilePath + "createDbSchema.sql");

			//--------------------
			// create the schema
			//--------------------
			dbc.executeSQLScript(is);
			is.close();


			// create content
			// if nothing defined, take the test elements which are located in the class path
			// for filling the db with content.
			if (contentDirectory.equals("")) {
				is = getClass().getResourceAsStream(DataBaseCreator.sqlFilePath + contentFile);
			} else {
				File f = new File(contentDirectory + contentFile);
				if (f.exists()){
					is = new FileInputStream(f);
				} else {
					connection = (new URL(contentDirectory + contentFile)).openConnection();
					connection.connect();
					is = connection.getInputStream();
				}
			}

			//--------------------
			// create the content
			//--------------------
			dbc.executeSQLScript(is);
			is.close();

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Opens the connection to the database. An existing connection will be closed first.
	 */
	public void open(){
		refreshConnectionString();
		try {
			if (connection != null && !connection.isClosed()) {
				connection.close();
			}
			connection = DriverManager.getConnection(connectionString, userid, password);
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Returns the connection to the database.
	 * @return Returns the currently opened connection to the database.
	 */
	public Connection getConnection() {
		try {

			if (connection == null || connection.isClosed()) {
				connection = null;
				open();
			}
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return connection;
	}

	/**
	 * Returns a new user that will go through a given test suite.
	 *
	 * @param The age of the user.
	 * @return Returns a new user object with a given age.
	 */
	public User getNewUser(int age) {
		User user = null;
		user = new User(this);
		user.setAge(age);
		user.writeChangedData();
		return user;
	}

	/**
	 * Returns a newly test suite that can contain a set of user specific and test specific
	 * questions.
	 *
	 * @return  Returns a newly created test suite that can contain a set of user
	 *          specific and tet specific questions.
	 */
	public TestSuite getNewTestSuite() {
		TestSuite suite = new TestSuite(this);
		return suite;
	}

	/**
	 * Sets whether the database is closed on shut down.
	 * @param sdOnClose
	 */
	public void setShutDownOnClose(boolean sdOnClose) {
		this.shutDownOnCloseConnection = sdOnClose;
	}

	/**
	 * Returns whether the database is shut down after the closing of the
	 * connection.
	 * @return Returns whether the database is shut down after the closing of the
	 *         connection.
	 */
	public boolean doesShutDownOnClose() {
		return this.shutDownOnCloseConnection;
	}

	/**
	 * Closes the database. This method has to be called before the database object is
	 * disposed. For each database object associated with the database which has unstored
	 * data has to be called the method
	 * {@link ch.unizh.ifi.adora.empiricism.model.DatabaseObject#writeChangedData()} first,
	 * before calling this method.
	 */
	public void close() {
		try {
			if (this.shutDownOnCloseConnection) {
				Statement statement = this.getConnection().createStatement();
				statement.execute("SHUTDOWN");
				statement.close();
			}
			this.getConnection().close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * @return Returns the questionCache.
	 */
	public QuestionCache getQuestionCache() {
		return questionCache;
	}

	/**
	 * @return Returns the model for question cache.
	 */
	public ModelForQuestionCache getModelForQuestionCache() {
		return modelForQuestionCache;
	}

	/**
	 * @return Returns the questionMcItemCache.
	 */
	public QuestionMcItemCache getQuestionMcItemCache() {
		return questionMcItemCache;
	}

	/**
	 * @return Returns the shutDownOnCloseConnection.
	 */
	public boolean isShutDownOnCloseConnection() {
		return shutDownOnCloseConnection;
	}

	/**
	 * @return Returns the testSuiteCache.
	 */
	public TestSuiteCache getTestSuiteCache() {
		return testSuiteCache;
	}

	/**
	 * @return Returns the userAnswerOfQuestionCache.
	 */
	public UserAnswerOfQuestionCache getUserAnswerOfQuestionCache() {
		return userAnswerOfQuestionCache;
	}

	/**
	 * @return Returns the userAnswersOfMcItemCache.
	 */
	public UserAnswersOfMcItemCache getUserAnswersOfMcItemCache() {
		return userAnswersOfMcItemCache;
	}

	/**
	 * @return Returns the userAnswersOfUserInfoCache.
	 */
	public UserAnswersOfUserInfoCache getUserAnswersOfUserInfoCache() {
		return userAnswersOfUserInfoCache;
	}

	/**
	 * @return Returns the userAnswersOfUserInfoMcAnswersCache.
	 */
	public UserAnswersOfUserInfoMcAnswersCache getUserAnswersOfUserInfoMcAnswersCache() {
		return userAnswersOfUserInfoMcAnswersCache;
	}

	/**
	 * @return Returns the userCache.
	 */
	public UserCache getUserCache() {
		return userCache;
	}

	/**
	 * @return Returns the userInfoCache.
	 */
	public UserInfoCache getUserInfoCache() {
		return userInfoCache;
	}

	/**
	 * @return Returns the userInfoMcItemCache.
	 */
	public UserInfoMcItemCache getUserInfoMcItemCache() {
		return userInfoMcItemCache;
	}

	public AdoraPropertyCache getAdoraPropertyCache() {
		return adoraPropertyCache;
	}
}
