package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * This class represents a tupel in a database table. A sub class has to implement
 * the abstract methods which describe in detail what kind of tuple from which table
 * is represented by an object of that class.
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 22, 2006 SM First Version
 *                      May 29, 2006 SM Comments
 *                      Jan 21, 2007 SM Changes for loading more than one model per question.
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 *
 */
public abstract class DatabaseObject {

	/**
	 * True if this Object exists in the database.
	 */
	private boolean tupelExists = false;

	/**
	 * Represents the database to which this DatabaseObject belongs to.
	 */
	protected Database db = null;

	/**
	 * This flag indicates whether the attribute of the database object were changed
	 * or not. In the case of change, the method {@link #writeChangedData()} has to be
	 * called.
	 *
	 */
	protected boolean changed = false;

	/**
	 * This flag represents that during the insertion (creation) of a tupel a unique
	 * id is generated by the database and has to be read by this database object after
	 * insertion. In this case the methods {@link #readNextId()} and
	 * {@link #getNextIdSelectStatement()} havae to be implemented.
	 */
	protected boolean idCreatedDuringInsert = true;

	/**
	 * The constructor initializes this object with a given database to which it
	 * belongs to.
	 *
	 * @pre       db != null
	 * @post      true
	 * @param db  The database to which this database object belongs to.
	 */
	public DatabaseObject(Database db) {
		this.db = db;
	}

	/**
	 * Creates the object by first creating a SQL insert statement which is got from
	 * the method {@link #getCreateStatement()} and initialized by the method
	 * {@link #prepareCreateStatement(PreparedStatement)}.<br>
	 * After executing this SQL
	 * statement, the last id generated in the table is read {@link #readNextId()}, ie.
	 * this is only done, if the attribute {@link #idCreatedDuringInsert} is set to true
	 * which is dependent on the structure of the table and which will be set in the sub
	 * class. <br>
	 * After reading the last generated id, the cache is prepared. The cache of a given type
	 * of database object contains all elements that are read into non persistent memory.
	 * Therefore the method {@link #prepareCache()} has to be implemented by the sub
	 * class.
	 *
	 */
	protected void createObject() {
		try {
			PreparedStatement statement = db.getConnection().prepareStatement(getCreateStatement());
			readNextId();
			prepareCreateStatement(statement);
			statement.execute();
			statement.close();
			
			prepareCache();
			this.refreshData();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Returns the last user id that was given to a user entry. This method gets a
	 * particular  SQL statment from the method {@link #getNextIdSelectStatement()}
	 * which has to be executed for getting the last id that was created in the table.
	 *
	 * @return Returns the last id that was given to a user.
	 */
	public void readNextId() {
		if (idCreatedDuringInsert) {
			try {
				Statement statement = db.getConnection().createStatement();
				ResultSet resultSet = statement.executeQuery(getNextIdSelectStatement());

				if (resultSet.next()) {
					readNextId(resultSet);
				}
				resultSet.close();
				statement.close();
			} catch (SQLException ex) {
				ex.printStackTrace();
			}
		}
	}

	/**
	 * Deletes this object from the database. This method gets a particular SQL delete statement
	 * from the method {@link #getDeleteStatement()}. After calling this method, this
	 * database object should be used any more.
	 */
	public void delete() {
		try {
			PreparedStatement statement = db.getConnection().prepareStatement(getDeleteStatement());
			prepareDeleteStatement(statement);
			statement.execute();
			statement.close();

		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Refreshes the non-persistent data of this database object by reading the data from the
	 * database. For this purpose a particular SQL statement is got from the method
	 * {@link #getSelectStatement()} and then prepared by the method
	 * {@link #prepareSelectString(PreparedStatement)}.
	 */
	public void refreshData() {
		tupelExists = false;
		writeChangedData();
		try {
			PreparedStatement statement = db.getConnection().prepareStatement(getSelectStatement());
			prepareSelectString(statement);
			prepareCache();
			ResultSet result = statement.executeQuery();
			if (result.next()) {
				readData(result);
				tupelExists = true;
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * This method writes the changed data of a database object back to the persistant
	 * tuple in a particular database table. This is done by getting a update SQL statement
	 * which is provided by the method {@link #getUpdateStatement()} and prepared for the
	 * update by the method {@link #prepareWriteData(PreparedStatement)}.<br>
	 * The data is only written back if there is a change in the attributes, indicated
	 * by the attribute {@link #changed}. This attribute has to set to true in subclasses
	 * if attribute data was changed.
	 */
	public void writeChangedData() {
		if (changed) {
			try {
				PreparedStatement statement = db.getConnection().prepareStatement(getUpdateStatement());
				prepareWriteData(statement);
				statement.execute();
				statement.close();
			} catch (SQLException ex) {
				ex.printStackTrace();
			}
			changed = false;
		}
	}


	/**
	 * Returns the SQL select statement for reading the last uniquely generated ID by
	 * the database  after the insertion of a new tupel. The method should only return
	 * such a select statement if the corresponding database table has such an automatically
	 * generated ID on insert. In this case this statement is used to retrieve and stored this
	 * ID for representation puposes.
	 *
	 * @return Returns a SQL select statement for retrieving an ID which was
	 *         generated automatically after the insert of a tupel in the database.
	 */
	protected abstract String getNextIdSelectStatement() ;

	/**
	 * Returns an SQL delete statement for deleting the corresponding tupel
	 * in the database. This delete statement has to prepared by the method
	 * {@link #prepareDeleteStatement(PreparedStatement)}, which parameterizes
	 * the query.
	 *
	 * @return Returns an SQL delete statement which deletes the tuple in the database
	 *         represented by this object.
	 */
	protected abstract String getDeleteStatement();

	/**
	 * Returns an SQL insert statement which creates the tupel corresponding to this
	 * database object in the database by inserting data into the database.
	 * This SQL statement will be parametrized by
	 * the method {@link #prepareCreateStatement(PreparedStatement)}.
	 *
	 * @return Returns an SQL insert statement which creates the tupel in the
	 *         corresponding database table.
	 */
	protected abstract String getCreateStatement();

	/**
	 * Returns an SQL update statement which writes the data of this database object
	 * back in the corresponding database tupel. This SQL statement will be prepared
	 * by the method {@link #prepareWriteData(PreparedStatement)}.
	 *
	 * @return Returns the SQL update statement which writes back the data of this
	 *         database object to the corresponding database tupel.
	 */
	protected abstract String getUpdateStatement();

	/**
	 * This method returns an SQL select statement which retrieves the data for
	 * this database object from the corresponding database tupel. This select
	 * statement will be parametrized by the
	 * method {@link #prepareSelectString(PreparedStatement)}.
	 *
	 * @return Returns an SQL select statement.
	 */
	protected abstract String getSelectStatement();

	/**
	 * This method is called after the instantiation of a new database object.
	 * It provides the object to a database cache which can be used to avoid multiple
	 * copies of this database object if it is retrieved several times from the corresponding
	 * database tuple.
	 */
	protected abstract void prepareCache();

	/**
	 * Prepares the SQL insert statement which is returned by the method
	 * {@link #getCreateStatement()}. The prepared <i>statement</i> (see
	 * {@link java.sql.PreparedStatement}) can be used for parametrizing variable
	 * parameters in the statement.
	 *
	 * @param statement The insert statement returned by the method {@link #getCreateStatement()}
	 *        which is parameterized by this method.
	 */
	protected abstract void prepareCreateStatement(PreparedStatement statement) throws SQLException;

	/**
	 * Prepares the SQL insert statement which is returned by the method
	 * {@link #prepareDeleteStatement(PreparedStatement)}. The prepared
	 * <i>statement</i> (see {@link java.sql.PreparedStatement}) can be used for
	 * parametrizing variable parameters in the statement.
	 * @param statement The insert statement returned by the method {@link #getDeleteStatement()}
	 *        which is parametrized by this method.
	 */
	protected abstract void prepareDeleteStatement(PreparedStatement statement) throws SQLException;

	/**
	 * Prepares the SQL select statement which is returned by the method
	 * {@link #getSelectStatement()}. The prepared
	 * <i>statement</i> (see {@link java.sql.PreparedStatement}) can be used for
	 * parametrizing variable parameters in the statement.
	 * @param statement The select statement returned by the method {@link #getSelectStatement()}
	 *        which is parametrized by this method.
	 */
	protected abstract void prepareSelectString(PreparedStatement statement) throws SQLException;

	/**
	 * Prepares the SQL update statement which is returned by the method
	 * {@link #getUpdateStatement()}. The prepared
	 * <i>statement</i> (see {@link java.sql.PreparedStatement}) can be used for
	 * parametrizing variable parameters in the statement.
	 * @param statement The update statement returned by the method {@link #getUpdateStatement()}
	 *        which is parametrized by this method.
	 */
	protected abstract void prepareWriteData(PreparedStatement statement) throws SQLException;

	/**
	 * Reads the data read from a tuple in the database, returned by the
	 * select statement given by {@link #getSelectStatement()} and parametrized by
	 * the method {@link #prepareSelectString(PreparedStatement)}. The
	 * query results in a {@link java.sql.ResultSet} object and provides the results.
	 * These results can be retrieved and set to attribues of this data object.
	 * This method is only called if there is data retrieved from the database. The
	 * result set is ready to be read, ie. the resultset is positioned correctly on the
	 * found, first tuple.
	 *
	 * @param The result set containing the data retrieved fromt the query given by the
	 *        method {@link #getSelectStatement()}
	 */
	protected abstract void readData(ResultSet result) throws SQLException;

	/**
	 * Reads the id that is generated when inserting a record into the database.
	 * This method is only used, if the table structure generates
	 * automatically an id when inserting a tuple. The id is retrived by the select
	 * SQL query given by the method {@link #getNextIdSelectStatement()}.
	 *
	 * @param resultSet The result set containing the id that is retrieved after the
	 *        insertion of a new tuple.
	 */
	protected abstract void readNextId(ResultSet resultSet) throws SQLException;

	/**
	 * @return Returns the tupelExists.
	 */
	public boolean doesTupelExists() {
		return tupelExists;
	}
}
