package ch.unizh.ifi.adora.empiricism.model;

import java.io.InputStream;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * A question is a part of a {@link ch.unizh.ifi.adora.empiricism.model.TestSuite} which
 * askes a particular question about an ADORA model. This question can be answered by a
 * {@link ch.unizh.ifi.adora.empiricism.model.User} either as a singel value of  the type
 * {@link TopicDataTypes#T_FLOAT}, {@link TopicDataTypes#T_INT} or {@link TopicDataTypes#T_STRING}
 * or as a answering several mc item if the type of the question is
 * {@link TopicDataTypes#T_MC} or {@link TopicDataTypes#T_OPTION}.
 *
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 29, 2006 SM First Version
 * 						Jan 21, 2007 SM Changes for loading more than one model per question.
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 *
 */
public class ModelForQuestion extends DatabaseObject implements TopicDataTypes{

	/**
	 * The id of the model for the question.
	 */
	private int id = 0;

	/**
	 * Stores the description which will be displayed prior to the question.
	 */
	private String name = "";

	/**
	 * The suite, associated with this question.
	 */
	private Question question;

	/**
	 * Stream, in which the xml formated adoramodel is contained,
	 * if a model is associated with this question.
	 */
	private InputStream adoraModelStream = null;

	/**
	 * Initializes a new user info object with the data of an existing tuple in the database.
	 *
	 * @param id The id of the tuple from the database.
	 * @param db The database where the element is stored in.
	 */
	public ModelForQuestion(int id, Database db, Question q) {
		super(db);
		this.id = id;
		this.question = q;
		refreshData();
	}

	/**
	 * Initializes a new Question object and creates a new tuple in the database.
	 *
	 * @param db The database where the element is stored in.
	 * @param question The suite, associated with this question.
	 */
	public ModelForQuestion(Database db, Question question) {
		super(db);
		this.question = question;
		createObject();
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {
		//ID, NAME, QUESTION_ID, ADORA_MODEL
		this.name   = result.getString(2);
		this.question      = lookUpQuestion(result.getInt(3));
		adoraModelStream= result.getBinaryStream(4);
	}

	/**
	 * Looks up the test suite associated with this question.
	 * @param id The unique id in the database of the test suite.
	 * @return Returns a test suite object with the given id.
	 */
	private Question lookUpQuestion(int id) {
		Question q = db.getQuestionCache().getDBObject(id);

		if (q == null) {
			q = new Question(id, db);
		}
		return q;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_MODEL_FOR_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_MODEL_FOR_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_NEXT_MODEL_FOR_QUESTION_ID;
	}


	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_MODEL_FOR_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, this.id);
		statement.setString(2, this.name);
		statement.setInt(3, this.question.getId());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		id = resultSet.getInt(1);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setString(2, this.name);
		statement.setInt(3, question.getId());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getModelForQuestionCache().setDBObject(id, this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_MODEL_FOR_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/**
	 * @return Returns the description.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param description The description to set.
	 */
	public void setName(String name) {
		this.name = name;
		changed = true;
	}

	/**
	 * Returns the adora model in form of a stream
	 * @return adora model in form of a stream
	 */
	public InputStream getAdoraModelStream() {
		return adoraModelStream;
	}

	/**
	 * Sets the adora model in form of a stream.
	 *
	 * @param adoraModelStream 	 Adora model in form of a stream.
	 */
	public void setAdoraModelStream(InputStream adoraModelStream) {
		this.adoraModelStream = adoraModelStream;
		changed = true;
	}
}