package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * A question is a part of a {@link ch.unizh.ifi.adora.empiricism.model.TestSuite} which
 * askes a particular question about an ADORA model. This question can be answered by a
 * {@link ch.unizh.ifi.adora.empiricism.model.User} either as a singel value of  the type
 * {@link TopicDataTypes#T_FLOAT}, {@link TopicDataTypes#T_INT} or {@link TopicDataTypes#T_STRING}
 * or as a answering several mc item if the type of the question is
 * {@link TopicDataTypes#T_MC} or {@link TopicDataTypes#T_OPTION}.
 *
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 29, 2006 SM First Version
 * 						Jan 21, 2007 SM Changes for loading more than one model per question.
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 *
 */
public class Question extends AbstractTopic implements TopicDataTypes{

	/**
	 * Stores the description which will be displayed prior to the question.
	 */
	private String description = "";

	/**
	 * The suite, associated with this question.
	 */
	private TestSuite suite;

	/**
	 * Initializes a new user info object with the data of an existing tuple in the database.
	 *
	 * @param id The id of the tuple from the database.
	 * @param db The database where the element is stored in.
	 */
	public Question(int id, Database db) {
		super(id,db);
		refreshData();
	}

	/**
	 * Initializes a new Question object and creates a new tuple in the database.
	 *
	 * @param db The database where the element is stored in.
	 * @param suite The suite, associated with this question.
	 */
	public Question(Database db, TestSuite suite) {
		super(db, suite);
		this.suite = suite;
		createObject();
	}

	/**
	 * Returns an array list that contains all multiple choice answers associated with this
	 * question. The array list should have more than one element if the  method {@link #getAnswerType()}
	 * returns either {@link #T_MC} or {@link #T_OPTION}.
	 *
	 * @return Returns a list of all multiple choice answers.
	 */
	public QuestionMcItemSet getAllMcAnswer() {
		QuestionMcItemSet qs = new QuestionMcItemSet(this, db);
		return qs;
	}

	/**
	 * Creates a new multiple choice answer which is associated with this question.
	 *
	 * @return Returns a new multiple choice answer which is associated with
	 *         this question.
	 */
	public QuestionMcItem createMcAnswer() {
		return new QuestionMcItem(this,db);
	}

	/**
	 * Returns the answer of this question for a given user. If the answer does not
	 * exist in the database, the tupel in the database is created.
	 *
	 * @param user The user who answers this question.
	 * @return Returns the user answer of this question.
	 */
	public UserAnswersOfQuestion getUserAnswerOfQuestion(User user) {
		return new UserAnswersOfQuestion(db, user, this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {
		this.question   = result.getString(2);
		this.answerType = result.getInt(3);
		this.answer     = result.getString(4);
		this.suite      = lookUpSuite(result.getInt(5));
		this.description= result.getString(6);
	}

	/**
	 * Looks up the test suite associated with this question.
	 * @param id The unique id in the database of the test suite.
	 * @return Returns a test suite object with the given id.
	 */
	private TestSuite lookUpSuite(int id) {
		TestSuite suite = db.getTestSuiteCache().getDBObject(id);

		if (suite == null) {
			suite = new TestSuite(id, db);
		}
		return suite;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_NEXT_QUESTION_ID;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, this.id);
		statement.setString(2, this.question);
		statement.setInt(3, this.answerType);
		statement.setString(4, this.answer);
		statement.setInt(5, (suite != null) ? this.suite.getId() : -1);
		statement.setString(6, this.description);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		id = resultSet.getInt(1);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setString(1, this.question);
		statement.setInt(2, this.answerType);
		statement.setString(3, this.answer);
		statement.setInt(4, this.suite.getId());
		statement.setString(5, this.description);
		statement.setInt(6, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getQuestionCache().setDBObject(id, this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/**
	 * @return Returns the description.
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description The description to set.
	 */
	public void setDescription(String description) {
		this.description = description;
		changed = true;
	}

	/**
	 * Return a set of all models for this question.
	 * @return Returns a set of all models for this question.
	 */
	public ModelForQuestionSet getModelForQuestionSet() {
		return new ModelForQuestionSet(this, db);
	}
}