package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Represents a multiple choice answer or optional answer of a 
 * {@link ch.unizh.ifi.adora.empiricism.model.Question}. A multiple choice {@link #answer}
 * consists of an answer string which gives a possible answer to a  
 * {@link ch.unizh.ifi.adora.empiricism.model.Question}. The answer can be {@link #correct}
 * or not. A mutliple choice answer has a unique {@link #id} which identifies it whithin all
 * exisiting multiple choice answers. There is the {@link #question} to which the answer
 * belongs to.<br>
 * This multiple can be answered be answered by a {@link ch.unizh.ifi.adora.empiricism.model.User}
 * and the user answer is stored in a {@link ch.unizh.ifi.adora.empiricism.model.UserAnswersOfMcItems}.
 * 
 *      
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 26, 2006 SM First Version
 *                      May 29, 2006 SM Comments
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 *
 */
public class QuestionMcItem extends McItem {
	
	
	/**
	 * The question to which this multiple choice answer belongs.
	 */
	private Question question = null;

	/**
	 * Describes whether
	 */
	private boolean correct = false;
	
	/**
	 * Creates a new multiple choice answer for a given question for a given database,
	 * by creating a new record for the multiple choice answer.
	 * 
	 * @param question The question for which a new multiple choice answer should be
	 *                 created.
	 * @param db       The database in which the multiple choice will be created.
	 */
	public QuestionMcItem(Question question, Database db) {
		super(db);
		this.question = question;
		createObject();
	}
	
	/**
	 * Creates a new multiple choice object from an existing multiple choice answer tuple
	 * in the database. The id of the existing element must be given.
	 * 
	 * @param id The id of the existing tupel in the database for which a new 
	 *           database object should be created.
	 * @param db The database which contains the tupel.
	 */
	public QuestionMcItem(int id, Database db) {
		super(db);
		this.id = id;
		this.refreshData();
	}
	
	/**
	 * Sets the multiple choice answer with which this multiple choice answer is 
	 * associated.
	 * 
	 * @param  q The question with which this multiple choice answer is associatied. 
	 */
	public void setQuestion(Question q) {
		changed = true;
		question = q;
	}
	
	/**
	 * Returns the question with which this 
	 * multiple choice question is associated.
	 * 
	 * @return Returns the question with which this 
	 *         multiple choice question is associated.
	 */
	public Question getQuestion() {
		return question;
	}
	
	/**
	 * Returns whether the given answer is correct or not.
	 * @return Returns if the given answer is correct or not.
	 */
	public boolean isCorrect() {
		return correct;
	}
	
	/**
	 * Sets whether this multiple choice answwer is correct or not.
	 * @param c indicates whether the answer is correct or not.
	 */
	public void setCorrect(boolean c) {
		correct = c;
		changed = true;
	}
	
	/**
	 * Returns the answer of a mc item for a given user. 
	 * If the answer does not exist in the database, the tupel in 
	 * the database is created.
	 * 
	 * @param user The user which answers the mc item.
	 * @return Returns the user answer of this multiple choice item.
	 */
	public UserAnswersOfMcItems getUserAnswerOfQuestion(User user) {
		return new UserAnswersOfMcItems(db, user, this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {
		 this.description = result.getString(2);
		 this.question = lookUpQuestion(result.getInt(3));
		 this.correct = result.getBoolean(4);		
	}

	/**
	 * Retrieves a question database object by its id.
	 * 
	 * @param questionId The id of the question to look up.
	 * @return  Returns the question database object that was looked up.
	 */
	protected Question lookUpQuestion(int questionId) {
		Question q = db.getQuestionCache().getDBObject(questionId);
		if (q == null) {
			q = new Question(questionId, db);
		}		
		return q;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_NEXT_MC_ITEM_ID;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getQuestionMcItemCache().setDBObject(id, this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, this.id);
		statement.setString(2, description);
		statement.setInt(3, question.getId());
		statement.setBoolean(4, correct);
	}

	/*
	 * 
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		id = resultSet.getInt(1);		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setString(1, description);
		statement.setInt(2, question.getId());
		statement.setBoolean(3, correct);		
		statement.setInt(4, id);
	}
}