package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Represents a test suite in an empirical test. A test suite consists of a set of 
 * {@link ch.unizh.ifi.adora.empiricism.model.Question} which can be either questions
 * with a simple answer (see {@link ch.unizh.ifi.adora.empiricism.model.Question#getAnswer()}} or a
 * set of multiple choice answers {@link ch.unizh.ifi.adora.empiricism.model.McAnswers}. 
 * 
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 23, 2006 SM First Version
 *                      May 29, 2006 SM Comments
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 */
public class TestSuite extends DatabaseObject {

	/**
	 * Stores the description which will be displayed after all questions
	 * were answered..
	 */
	private String descriptionStartUserInfo = "";
	
	/**
	 * Stores the description which will be displayed prior to the TestSuite.
	 */
	private String descriptionStartQuestion = "";
	
	/**
	 * Stores the description which will be displayed after all questions
	 * were answered..
	 */
	private String descriptionEndSuite = "";

	/**
	 * The reference to the test suite.
	 */
	private int id;
	
	/**
	 * The name of the test suite describes the purpose of the test.
	 */
	private String suiteName = "";
	
	/**
	 * This attribute indicates how often this test suite was done by different users.
	 */
	private int numberOfExecutedTest=0;
	
	/**
	 * Creates a new test suite, i.e. inserts a new tupel into the corresponding database.
	 * @param db The database with which this test suite is associated.
	 */
	public TestSuite(Database db) {
		super(db);
		createObject();
	}
	
	/**
	 * Creates a test suite object from an existing test suite tupel in the database.
	 * 
	 * @param id The id of test suite tuple in the database.
	 * @param db The database in which the test suite is stored.
	 */
	public TestSuite(int id, Database db) {
		super(db);
		this.id = id;
		refreshData();
	}
	
	/**
	 * Returns the test suite id.
	 * 
	 * @return Returens the test suite id. 
	 */
	public int getId() {
		return id;
	}
	
	/**
	 * Returns the suite name of the test suite.
	 * @return Returns the suite name of the test suite.
	 */
	public String getSuiteName() {
		return suiteName;
	}
	
	/**
	 * Returns the Number Of Executed Test.
	 * 
	 * @pre       true
	 * @post      true
	 * @obligation 
	 * @return NumberOfExecutedTest
	 */
	public int getNumberOfExecutedTest(){
		return numberOfExecutedTest;
	}
	
	/**
	 * Inrements the number Of Executed Test
	 * and write the new value back to the db.
	 * 
	 * @pre       true
	 * @post      true
	 * @obligation
	 */
	public void writeIncrementNrOfExeTest(){
		numberOfExecutedTest++;
		changed = true;
		writeChangedData();
	}

	
	/**
	 * Sets the name of the suite which describes the test in a rough way.
	 * 
	 * @param suiteName The name of the suite.
	 */
	public void setSuiteName(String suiteName) {
		this.suiteName = suiteName;
	}
	
	/**
	 * Returns all the question objects in a set.
	 * 
	 * @return Returns a question set object which contains all questions of this test suite.
	 */
	public QuestionSet getAllQuestions() {
		return new QuestionSet(this, db);
	}
	
	/**
	 * Returns all questions about a user executing the test, which are associated with this test suite.
	 * 
	 * @return Returns all questions about a user executing the test, which are associated with this test suite.
	 */
	public UserInfoSet getAllUserInfos() {
		return new UserInfoSet(this, db);
	}

	/**
	 * Creates a new question in the test suite, i.e. a new question tuple in the database is created.
	 * 
	 * @return Creates a new question object and returns it.
	 */
	public Question createQuestion() {
		return new Question(db, this);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {
		this.suiteName = result.getString(2);
		this.numberOfExecutedTest = result.getInt(3);
		this.descriptionStartUserInfo = result.getString(4);
		this.descriptionStartQuestion = result.getString(5);
		this.descriptionEndSuite= result.getString(6);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setString(1, suiteName);
		statement.setInt(2, numberOfExecutedTest);
		statement.setString(3, descriptionStartUserInfo);
		statement.setString(4, descriptionStartQuestion);
		statement.setString(5, descriptionEndSuite);
		statement.setInt(6, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_TEST_SUITE;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_TEST_SUITE;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_TEST_SUITE;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_NEXT_TEST_SUITE_ID;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		this.id = resultSet.getInt(1);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, this.id);
		statement.setString(2,this.suiteName);
		statement.setInt(3, this.numberOfExecutedTest);
		statement.setString(4, descriptionStartUserInfo);
		statement.setString(5, descriptionStartQuestion);
		statement.setString(6, descriptionEndSuite);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getTestSuiteCache().setDBObject(id,this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_TEST_SUITE;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {		
		statement.setInt(1, id);
	}

	/**
	 * @return Returns the descriptionEndSuite.
	 */
	public String getDescriptionEndSuite() {
		return descriptionEndSuite;
	}

	/**
	 * @return Returns the descriptionStartQuestion.
	 */
	public String getDescriptionStartQuestion() {
		return descriptionStartQuestion;
	}

	/**
	 * @return Returns the descriptionStartUserInfo.
	 */
	public String getDescriptionStartUserInfo() {
		return descriptionStartUserInfo;
	}

	/**
	 * @param descriptionEndSuite The descriptionEndSuite to set.
	 */
	public void setDescriptionEndSuite(String descriptionEndSuite) {
		this.descriptionEndSuite = descriptionEndSuite;
		changed = true;
	}

	/**
	 * @param descriptionStartQuestion The descriptionStartQuestion to set.
	 */
	public void setDescriptionStartQuestion(String descriptionStartQuestion) {
		this.descriptionStartQuestion = descriptionStartQuestion;
		changed = true;
	}

	/**
	 * @param descriptionStartUserInfo The descriptionStartUserInfo to set.
	 */
	public void setDescriptionStartUserInfo(String descriptionStartUserInfo) {
		this.descriptionStartUserInfo = descriptionStartUserInfo;
		changed = true;
	}
}