package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;


/**
 * An object of this class represents the answer from a 
 * {@link ch.unizh.ifi.adora.empiricism.model.UserInfo}
 * for a {@link ch.unizh.ifi.adora.empiricism.model.TestSuite}.
 *
 * @author		       	vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 28, 2006 SM First Version
 *                      May 30, 2006 SM Comments
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 */
public class TestSuiteUserInfo extends WeakIdentityDatabaseObject {
	
	/**
	 * UserInfo
	 */
	private UserInfo userInfo;
	
	/**
	 * TestSuite
	 */
	private TestSuite suite;
	
	/**
	 * Reads the content of the corresponding database tupel or 
	 * creates a new TestSuite tuple (if the tuple does not exist) of 
	 * a UserInfo {@link ch.unizh.ifi.adora.empiricism.model.UserInfo}
	 * for a given   {@link ch.unizh.ifi.adora.empiricism.model.TestSuite}.
	 *
	 * @param db This is the database in which the elements are stored.
	 * @param userInfo The userInfo.
	 * @param suite The TestSuite.
	 */
	public TestSuiteUserInfo(Database db, UserInfo userInfo, TestSuite suite) {
		super(db);
		this.userInfo     = userInfo;
		this.suite = suite;
		this.idCreatedDuringInsert=false;
		if (!this.checkIfDatabaseTupleExists()) {
			createObject();
		} else {
			this.refreshData();
		}
	}
	
	/**
	 * Looks up a user with a given id in the user cache.
	 * If it does not exist, a new object is created, otherwise
	 * the exisiting object is returned.
	 * 
	 * @param id the Id of the user.
	 * @return Returns the user object which is associated with the given id. 
	 */
	private UserInfo lookUpUserInfo(int id) {
		UserInfo userInfo = db.getUserInfoCache().getDBObject(id);
		if (userInfo == null) {
			userInfo = new UserInfo(id, db);
		}
		return userInfo;
	}

	/**
	 * Looks up a user with a given id in the user cache.
	 * If it does not exist, a new object is created, otherwise
	 * the exisiting object is returned.
	 * 
	 * @param id the Id of the user.
	 * @return Returns the user object which is associated with the given id. 
	 */
	private TestSuite lookUpTestSuite(int id) {
		TestSuite suite = db.getTestSuiteCache().getDBObject(id);
		if (suite == null) {
			suite = new TestSuite(id, db);
		}
		return suite;
	}

	/**
	 * Returns the user who answered the question.
	 * 
	 * @return Returns the user who answreed the question.
	 */
	public UserInfo getUserInfo() {
		return userInfo;
	}
	
	/**
	 * Sets the user who answered the question.
	 * @param user The user.
	 */
	public void setUserInfo(UserInfo userInfo) {
		this.userInfo = userInfo;
		changed = true;
	}
	
	/**
	 * Returns the question which is answered.
	 *   
	 * @return Returns the question which is answered. 
	 */
	public TestSuite getTestSuite() {
		return suite;
	}
	
	/**
	 * Sets the question which is answered.
	 * 
	 * @param q The question which is answered. 
	 */
	public void setTestSuite(TestSuite suite) {
		this.suite = suite;
		changed = true;
	}
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {			
		suite = this.lookUpTestSuite(result.getInt(1));
		userInfo = this.lookUpUserInfo(result.getInt(2));
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_TEST_SUITE_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_TEST_SUITE_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return "";
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_TEST_SUITE_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_TEST_SUITE_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getUserAnswerOfQuestionCache().setDBObject(suite.getId() + "" + userInfo.getId(), this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, suite.getId());
		statement.setInt(2, userInfo.getId());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, suite.getId());
		statement.setInt(2, userInfo.getId());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		// does nothing, does not generate own ids during insert.
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, suite.getId());
		statement.setInt(2, userInfo.getId());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setInt(1, suite.getId());
		statement.setInt(2, userInfo.getId());
	}
}