package ch.unizh.ifi.adora.empiricism.model;

import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A user object represents the person who is executing a test suite, i.e. is asked questions about
 * a given model.      
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 22, 2006 SM First Version
 *                      May 30, 2006 SM Comments
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 *
 */
public class User extends DatabaseObject {

	/**
	 * TestSuite which is associated with the user.
	 */
	private TestSuite suite;
	
	/**
	 * Identifies uniquely the user which is executing a test suite.
	 */
	private int id = -1;
	
	/**
	 * The age of the user is the only attribute which is mandatory for the user. Additional
	 * questions about the user have to be done by creating particular {@link ch.unizh.ifi.adora.empiricism.model.UserInfo}
	 * objects which contain questions about a user.
	 */
	private int age = 0;
	
	/**
	 * Contains the date when the user was created.
	 */
	private Date created;
	
	/**
	 * Creates a new user in a given database, i.e. a new user tuple is created in the database.
	 * 
	 * @param db The database in which the user should be created.
	 */
	public User(Database db) {
		super(db);
		createObject();
	}
	
	/**
	 * Creates a user object from an existing user record in the database.
	 * 
	 * @param id The unique identifier of the user.
	 * @param db The database where the user is located.
	 */
	public User(int id, Database db) {
		super(db);
		this.id = id;
		this.refreshData();
	}

	/**
	 * Returns the least used TestSuite for this user.
	 * 
	 * @pre       true
	 * @post      true
	 * @obligation 
	 * @return
	 */
	public TestSuite getLeastUsedTestSuit(){
		if (suite != null) {
			return suite;
		}
		TestSuite currentTs;
		UnansweredTestSuiteSet tss = new UnansweredTestSuiteSet(db,this);
		Iterator tssIter = tss.iterator();
		while (tssIter.hasNext()) {
			currentTs = (TestSuite)tssIter.next();
			if (suite == null || 
					currentTs.getNumberOfExecutedTest() < suite.getNumberOfExecutedTest()) {
				suite = currentTs;
				//doesn't make sense to search more if nr=0;
				if (suite.getNumberOfExecutedTest() == 0)
					return suite;
			}
		}
		return suite;
	}
	
	/**
	 * Returns a list of UserInfos, which this user hasn't answered yet.
	 * 
	 * @pre       true
	 * @post      true
	 * @obligation 
	 * @param suite
	 * @return
	 */
	public List<UserInfo> getUnansweredUserInfos(TestSuite suite){
		List<UserInfo> unansweredUIs = new ArrayList<UserInfo>();
		AnsweredUserInfoSet answerSet = new AnsweredUserInfoSet(db, this);
		
		UserInfoSet uis = new UserInfoSet(suite, db);
		Iterator uisIter = uis.iterator();
		UserInfo ui = null;
		while (uisIter.hasNext()) {
			ui = (UserInfo)uisIter.next();
			if (!answerSet.contains(ui)) {
				unansweredUIs.add(ui);
			}
		}
		return unansweredUIs;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_USER;
	}
	
	/**
	 * Sets the age of the user.
	 * @param age The age of the user.
	 */
	public void setAge(int age) {
		this.age = age;
		changed = true;
	}
	
	/**
	 * Returns the age of the user.
	 * 
	 * @return Returns the age of the user. 
	 */
	public int getAge() {
		return age;
	}
	
	/**
	 * Returns the id of the user.
	 * @return Returns the id of the user. 
	 */
	public int getID() {
		return id;
	}
	
	/**
	 * Returns the date when the user was created.
	 * @return Returns the date (timestamp) when the user was create.
	 */
	public Date getCreatedDate() {
		return created;
	}

	/**
	 * Returns the answer of a question for this user. If the answer does not
	 * exist in the database, the tupel in the database is created.
	 * 
	 * @param question The question which is answered by this user.
	 * @return Returns the user answer of this question.
	 */
	public UserAnswersOfQuestion getUserAnswerOfQuestion(Question question) {
		return new UserAnswersOfQuestion(db, this, question);
	}

	/**
	 * Returns the answer of a mc item for this user. 
	 * If the answer does not exist in the database, the tupel in 
	 * the database is created.
	 * 
	 * @param mcItem The mc item that is answered by this user.
	 * @return Returns the user answer of a given multiple choice item answered
	 *         by this user.
	 */
	public UserAnswersOfMcItems getUserAnswerOfQuestion(QuestionMcItem mcItem) {
		return new UserAnswersOfMcItems(db, this, mcItem);
	}

	/**
	 * Returns the answer of a given user info for this user. 
	 * If the answer does not exist in the database, the tupel in 
	 * the database is created.
	 * 
	 * @param userInfo The userInfo which is answered by this user.
	 * @return Returns the user answer of a given user info.
	 */
	public UserAnswersOfUserInfo getUserAnswerOfQuestion(UserInfo userInfo) {
		return new UserAnswersOfUserInfo(db, this, userInfo);
	}
	
	/**
	 * Returns the answer of  this user info mc item for this user. 
	 * If the user info mc item does not exist in the database, the tupel in 
	 * the database is created.
	 * 
	 * @param userInfoMcItem The user info mc item which is answered by this user.
	 * @return Returns the user answer of user info mc answer.
	 */
	public UserAnswersOfUserInfoMcItems getUserAnswerOfQuestion(UserInfoMcItem userInfoMcItem) {
		return new UserAnswersOfUserInfoMcItems(db, this, userInfoMcItem);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setInt(1, age);
		statement.setDate(2, created);
		statement.setInt(3, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {
		age = result.getInt(2);
		created = result.getDate(3);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_USER;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_USER;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		id = resultSet.getInt(1);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_NEXT_USER_ID;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, this.id);
		statement.setInt(2, age);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getUserCache().setDBObject(id,this);		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_USER;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}
}