package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Instances of this object represent a multiple choice answer of a user 
 * {@link ch.unizh.ifi.adora.empiricism.model.User} to a given 
 * multiple choice item (see {@link ch.unizh.ifi.adora.empiricism.model.QuestionMcItem}) of 
 * a {@link ch.unizh.ifi.adora.empiricism.model.Question}.
 * 
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 28, 2006 SM First Version
 *                      May 30, 2006 SM Comments.
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 */
public class UserAnswersOfMcItems extends WeakIdentityDatabaseObject {
	
	/**
	 * The user who answered the multiple choice question.
	 */
	private User user = null;
	
	/**
	 * The multiple choice item that was answered.
	 */
	private QuestionMcItem mcItem = null;
	
	/**
	 * Indicates whether the user ticked this item or not.
	 */
	private boolean correct = false;
	
	/**
	 * Reads the content of the corresponding database tuple or creates a new answer tupel
	 * (if the tuple does not exist) to a multiple choice item the answering user and the 
	 * correspoinding answer. It will be created a persitant record for the answer.
	 * 
	 * @param db The database in whch the user answer will be created.
	 * @param user The user object representing the user who ansered the multiple choice
	 *             item.
	 * @param item The multiple choice item that was answered. 
	 */
	public UserAnswersOfMcItems(Database db, User user, QuestionMcItem item) {
		super(db);
		this.user     = user;
		this.mcItem = item;
		this.idCreatedDuringInsert=false;
		if (!this.checkIfDatabaseTupleExists()) {
			createObject();
		} else {
			this.refreshData();
		}
	}
	
	/**
	 * Returns if the user answered the multiple choice answer as correct or not.
	 * @return Returns if the user ticked this answer as correct or not.
	 */
	public boolean isCorrect() {
		return correct;
	}
	
	/**
	 * Sets whether the mc item was ticked as correct or as wrong by the user. 
	 * @param c Indicates if the user ticket the item or not.
	 */
	public void setCorrect(boolean c) {
		correct = c;
		changed = true;
	}
	
	/**
	 * Returns the user who gave the answer.
	 *  
	 * @return Returns the user who gave the answer. 
	 */
	public User getUser() {
		return user;
	}
	
	/**
	 * Sets the user who gave the answer. 
	 * @param u The user who gave the answer.
	 */
	public void setUser(User u) {
		user = u;
		changed = true;
	}
	
	/**
	 * Returns the mc item that was answered.
	 * @return Returns the mc item that was answered. 
	 */
	public QuestionMcItem getAnswer() {
		return mcItem;
	}
	
	/**
	 * Sets the mc item that was answered.
	 * 
	 * @param a The mc item that was answered. 
	 * 
	 */
	public void setAnswer(QuestionMcItem a) {
		mcItem = a;
		changed = true;
	}
	
	/**
	 * Looks up the user with the given id. If the user doesn't exists as object
	 * a new user object is created (without creating a new user tuple in the db).
	 * 
	 * @param id The unique id of the user which should be looked up. 
	 * @return Returns the user object representing the user who answered the 
	 *         mc item.
	 */
	private User lookUpUser(int id) {
		User user = db.getUserCache().getDBObject(id);
		if (user == null) {
			user = new User(id, db);
		}
		return user;
	}

	/**
	 * Looks up the mc item with the given id. If the user doesn't exists as object
	 * a new mc item object is created (without creating a new user tuple in the db).
	 * 
	 * @param id The unique id of the mc item which should be looked up.
	 *  
	 * @return Returns the user object representing the user who answered the 
	 *         mc item.
	 */
	private QuestionMcItem lookUpMcAnswer(int id) {
		QuestionMcItem answer = db.getQuestionMcItemCache().getDBObject(id);
		if (answer == null) {
			answer = new QuestionMcItem(id, db);
		}
		return answer;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {	
		this.user     = lookUpUser(result.getInt(1));
		this.mcItem = lookUpMcAnswer(result.getInt(2));
		this.correct  = result.getBoolean(3);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_USER_ANSWERS_OF_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_USER_ANSWERS_OF_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return "";
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_USER_ANSWERS_OF_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_USER_ANSWERS_OF_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getUserAnswersOfMcItemCache().setDBObject(user.getID() + "/" + mcItem.getId(), true);		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, mcItem.getId());
		statement.setBoolean(3, correct);		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, mcItem.getId());		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		// does nothing, because no id is generate when creating the record.
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, mcItem.getId());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, mcItem.getId());
		statement.setBoolean(3, correct);		
		statement.setInt(4, user.getID());
		statement.setInt(5, mcItem.getId());
	}
}