package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;


/**
 * An object of this class represents the answer from a 
 * {@link ch.unizh.ifi.adora.empiricism.model.User}
 * for a {@link ch.unizh.ifi.adora.empiricism.model.Question}.
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 28, 2006 SM First Version
 *                      May 30, 2006 SM Comments
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 *
 */
public class UserAnswersOfQuestion extends WeakIdentityDatabaseObject {	
	
	/**
	 * The user who answered.
	 */
	private User user=null;
	
	/**
	 * The question which was answered.
	 */
	private Question question = null;
	
	/**
	 * The time that was needed to answer the question (in seconds).
	 */
	private int usedTime = -1;
	
	/**
	 * The timestamp describing when the user started to find an answer for the question.
	 */
	private Timestamp timestamp = null;
	
	/**
	 * The answer of the user for the question. This field contains only a value
	 * if the method {@link ch.unizh.ifi.adora.empiricism.model.Question#getAnswerType()} is one of the following types:
	 * {@link ch.unizh.ifi.adora.empiricism.model.Question#T_FLOAT},
	 * {@link ch.unizh.ifi.adora.empiricism.model.Question#T_INT} or
	 * {@link ch.unizh.ifi.adora.empiricism.model.Question#T_STRING}.
	 */
	private String userAnswer = ""; 
	
	/**
	 * Reads the content of the corresponding database tupel or 
	 * creates a new answer tuple (if the tuple does not exist) of 
	 * a user question for a given {@link ch.unizh.ifi.adora.model.adora.empiricism.User} 
	 * and a given {@link ch.unizh.ifi.adora.model.adora.empiricism.Question}.
	 *
	 * @param db This is the database in which the elements are stored.
	 * @param user The user who answers the question.
	 * @param question The question which is answered.
	 */
	public UserAnswersOfQuestion(Database db, User user, Question question ) {
		super(db);
		this.user     = user;
		this.question = question;
		this.idCreatedDuringInsert=false;
		if (!this.checkIfDatabaseTupleExists()) {
			createObject();
		} else {
			this.refreshData();
		}
	}
	
	/**
	 * Looks up a user with a given id in the user cache.
	 * If it does not exist, a new object is created, otherwise
	 * the exisiting object is returned.
	 * 
	 * @param id the Id of the user.
	 * @return Returns the user object which is associated with the given id. 
	 */
	private User lookUpUser(int id) {
		User user = db.getUserCache().getDBObject(id);
		if (user == null) {
			user = new User(id, db);
		}
		return user;
	}

	/**
	 * Looks up a user with a given id in the user cache.
	 * If it does not exist, a new object is created, otherwise
	 * the exisiting object is returned.
	 * 
	 * @param id the Id of the user.
	 * @return Returns the user object which is associated with the given id. 
	 */
	private Question lookUpQuestion(int id) {
		Question question = db.getQuestionCache().getDBObject(id);
		if (question == null) {
			question = new Question(id, db);
		}
		return question;
	}
	
	/**
	 * Returns the user who answered the question.
	 * 
	 * @return Returns the user who answreed the question.
	 */
	public User getUser() {
		return user;
	}
	
	/**
	 * Sets the user who answered the question.
	 * @param user The user.
	 */
	public void setUser(User user) {
		this.user = user;
		changed = true;
	}
	
	/**
	 * Returns the question which is answered.
	 *   
	 * @return Returns the question which is answered. 
	 */
	public Question getQuestion() {
		return question;
	}
	
	/**
	 * Sets the question which is answered.
	 * 
	 * @param q The question which is answered. 
	 */
	public void setQuestion(Question q) {
		this.question = q;
		changed = true;
	}
	
	/**
	 * Returns the time in seconds which was used to answer the question.
	 *
	 * @return  Returns the time used for answering the question.
	 */
	public int getUsedTime() {
		return usedTime;
	}
	
	/**
	 * Sets the time used for answering the quesion. The time measure is seconds.
	 * 
	 * @param time The time in seconds which was used to answer the question.
	 */
	public void setUsedTime(int time) {
		this.usedTime = time;
		changed = true;
	}
	
	/**
	 * Sets the time when the user started to answer the question.
	 * 
	 * @param t The time when the user started to answer the question.
	 */
	public void setTime(Timestamp t) {
		this.timestamp = t;
		changed = true;
	}
	
	/**
	 * Returns the time when the user started to answer the question.
	 * 
	 * @return Returns the time stamp when the user started to answer the question.
	 */
	public java.util.Date getTime() {
		return this.timestamp;
	}
	
	/**
	 * Returns the answer that is answered. The answer should only return a string
	 * unequal to null or the empty string the method {@link ch.unizh.ifi.adora.empiricism.model.Question#getAnswerType()} 
	 * returns one of the following values describing the type of the answer.
	 * 
	 * {@link ch.unizh.ifi.adora.empiricism.model.Question#T_FLOAT},
	 * {@link ch.unizh.ifi.adora.empiricism.model.Question#T_INT} or
	 * {@link ch.unizh.ifi.adora.empiricism.model.Question#T_STRING}.
	 * 
	 * If the method {@link ch.unizh.ifi.adora.empiricism.model.Question#getAnswerType()} returns
	 * {@link ch.unizh.ifi.adora.empiricism.model.Question#T_MC} or 
	 * {@link ch.unizh.ifi.adora.empiricism.model.Question#T_OPTION} 
	 * there are multiple choice answers of the user represented by 
	 * {@link ch.unizh.ifi.adora.empiricism.model.UserAnswersOfMcItems}. 
	 * 
	 * @return Returns the answer that was answered.
	 */
	public String getUserAnswer() {
		return userAnswer;
	}
	
	/**
	 * Sets the answer which was given by the user to the question. 
	 * Sets the answer that was given for the question.
	 * 
	 * @param userAnswer The answer given for the question
	 * @see #getUserAnswer()
	 */
	public void setUserAnswer(String userAnswer) {
		this.userAnswer = userAnswer;
		changed = true;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {			
		user = this.lookUpUser(result.getInt(1));
		question = this.lookUpQuestion(result.getInt(2));
		this.usedTime = result.getInt(3);
		this.timestamp = result.getTimestamp(4);
		this.userAnswer = result.getString(5);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_USER_ANSWERS_OF_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_USER_ANSWERS_OF_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return "";
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_USER_ANSWERS_OF_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_USER_ANSWERS_OF_QUESTION;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getUserAnswerOfQuestionCache().setDBObject(user.getID() + "" + question.getId(), this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, question.getId());
		statement.setInt(3, this.usedTime);
		statement.setTimestamp(4, this.timestamp);
		statement.setString(5, this.userAnswer);		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, this.question.getId());				
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		// does nothing, does not generate own ids during insert.
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, this.question.getId());		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, question.getId());
		statement.setInt(3, this.usedTime);
		statement.setTimestamp(4, this.timestamp);
		statement.setString(5, this.userAnswer);
		statement.setInt(6, user.getID());
		statement.setInt(7, this.question.getId());		
	}

}