package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Represents the answer to a {@link ch.unizh.ifi.adora.empiricism.model.UserInfo} question
 * of a {@link ch.unizh.ifi.adora.empiricism.model.User}. 
 * 
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 28, 2006 SM First Version
 *                      May 30, 2006 SM Comments
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 */
public class UserAnswersOfUserInfo extends WeakIdentityDatabaseObject {
	
	/**
	 * The answer given by the user. This string has a content only if 
	 */
	private String answer = "";
	
	/**
	 * The user who answered the user info question.
	 */
	private User user;
	
	/**
	 * The user info which is answered by this user answer.
	 */
	private UserInfo userInfo;
	
	/**
	 * Creates a new user answer tuple of a user info for a specific user or if
	 * the tuple already exists, the content of the tupel is read from the database.
	 * 
	 * @param db The database in which the user and the user info is entered.
	 * @param user
	 * @param userInfo
	 */
	public UserAnswersOfUserInfo(Database db, User user, UserInfo userInfo) {
		super(db);
		this.idCreatedDuringInsert = false;
		this.user = user;
		this.userInfo = userInfo;
		if (!this.checkIfDatabaseTupleExists()) {
			createObject();
		} else {
			this.refreshData();
		}
	}

	/**
	 * Returns the user who answered the user info question.
	 * 
	 * @return Returns the user who answered the user info.
	 */
	public User getUser() {
		return user;
	}
	
	/**
	 * Sets the user who answered the user info question.
	 * @param  u The user who answered the user info question.
	 */
	public void setUser(User u) {
		user = u;
		changed = true;
	}
	
	/**
	 * Returns the user info which is answered.
	 * 
	 * @return Returns the user info which is answered.
	 */
	public UserInfo getUserInfo() {
		return userInfo;
	}
	
	/**
	 * Sets the user info question which is answered by this question.
	 * @param ui UserInfo object which is answered by this user answer. 
	 */
	public void setUserInfo(UserInfo ui) {
		userInfo = ui;
		changed = true;
	}
	
	/**
	 * Returns the answer as a string. This method returns onyl a value unequal to the 
	 * empty string or null if the method {@link Question#getAnswerType()} returns either
	 * {@link TopicDataTypes#T_FLOAT}, {@link TopicDataTypes#T_INT} or 
	 * {@link TopicDataTypes#T_STRING}. If the method {@link Question#getAnswerType()}
	 * returns {@link TopicDataTypes#T_MC} or {@link TopicDataTypes#T_OPTION}, then
	 * this method returns the empty string or null.
	 * @return Returns the answer of the user info question as a string.
	 */
	public String getAnswer() {
		return answer;
	}
	
	/**
	 * Sets the answer to the user info question as a string.
	 * 
	 * @param answer The answer to set as a string.
	 * @see #getAnswer()
	 */
	public void setAnswer(String answer) {
		this.answer = answer;
		changed = true;
	}
	

	/**
	 * Looks up a user with given user id. First the user 
	 * is searched in the cache. If it is found the existing object is returned
	 * otherwise a new user bject is created.
	 * 
	 * @param id The id of the user tuple. 
	 * @return Returns the user object with the given id.	
	 */ 
	private User lookUpUser(int id) {
		User user = db.getUserCache().getDBObject(id);
		if (user == null) {
			user = new User(id, db);
		}
		return user;
	}
	
	/**
	 * Looks up a user info with a given user info id. First the user info
	 * is searched in the cache. If it is found the existing object is returned
	 * otherwise a new user info object is created.
	 * 
	 * @param id The id of the user info tuple. 
	 * @return Returns the user info object with the given id.
	 */
	private UserInfo lookUpUserInfo(int id) {
		UserInfo uiAnswer = db.getUserInfoCache().getDBObject(id);
		if (uiAnswer == null) {
			uiAnswer = new UserInfo(id, db);
		}
		return uiAnswer;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {
		user = lookUpUser(result.getInt(1));
		userInfo = lookUpUserInfo(result.getInt(2));
		answer = result.getString(3);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {		
		return DataBaseSQLStrings.SQL_CREATE_USER_ANSWERS_OF_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_USER_ANSWERS_OF_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return "";
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_USER_ANSWERS_OF_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_USER_ANSWERS_OF_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getUserAnswersOfUserInfoCache().setDBObject(user.getID() + "/" + userInfo.getId(), this);		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, userInfo.getId());
		statement.setString(3, answer);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, userInfo.getId());		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		// do nothing, because no id is generated during  
		// the insertion of a new record.		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, userInfo.getId());		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, userInfo.getId());
		statement.setString(3, answer);
		statement.setInt(4, user.getID());
		statement.setInt(5, userInfo.getId());		
	}
}