package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Describes the user answers of a {@link ch.unizh.ifi.adora.empiricism.User} for a
 * given multiple choice item (see {@link ch.unizh.ifi.adora.empiricism.QuestionMcItem}. 
 * 
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 30, 2006 SM First Version
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 */
public class UserAnswersOfUserInfoMcItems extends WeakIdentityDatabaseObject {
	
	/**
	 * The user who answered this multiple choice items of the user info. 
	 */
	private User user = null;
	
	/**
	 * The multiple choice item of the user info that was answered.
	 */
	private UserInfoMcItem userInfo = null;
	
	/**
	 * The indicates if the user ticked this multiple choice item as correct.
	 */
	private boolean correct = false;
	
	/**
	 * Creates a new user answer object  (tuple in the database) of a user 
	 * info multiple choice item, i.e. it creates a new tuple in the database, 
	 * or if the tuple already exists, it reads the tuple from the database.
	 *   
	 * @param db The database where the tuple will be stored in.
	 * @param user The user who answered the multiple choice item.
	 * @param userInfo The user info which is answered.
	 */
	public UserAnswersOfUserInfoMcItems(Database db, User user, UserInfoMcItem userInfo) {
		super(db);
		this.idCreatedDuringInsert = false;
		this.user = user;
		this.userInfo = userInfo;
		if (!this.checkIfDatabaseTupleExists()) {
			createObject();
		} else {
			this.refreshData();
		}		
	}
	
	/**
	 * Returns the user info multiple choice item which is answered by this
	 * user answer.
	 *  
	 * @return Returns the user info multiple choice item which is answered by this
	 * user answer. 
	 */
	public UserInfoMcItem getUserInfoMcItem() {
		return userInfo;
	}

	/**
	 * Sets a user info multiple choice item which is answered by this user answer.
	 * 
	 * @param  uio The user info multiple choice item which is answered by this user answer.
	 */
	public void setUserInfoMcItem(UserInfoMcItem uio) {
		userInfo = uio;
		changed = true;
	}
	
	/**
	 * Returns the user which answered the multiple choice item.
	 * 
	 * @return Returns the user which answered the multiple choice item. 
	 */
	public User getUser() {
		return user;
	}

	/**
	 * Sets the user which answered the multiple choice item.
	 * 
	 * @param u The user which answered the multiple choice item. 
	 */
	public void setUser(User u) {
		this.user = u;
		changed = true;
	}
	
	/**
	 * This method returns whether the user ticked the multiple choice answer as correct or not.
	 * @return Returns whether the user ticked the multiple choice answer as correct or not.
	 */
	public boolean isCorrect() {
		return correct;
	}
	
	/**
	 * This method sets whether the user ticked the multiple choice answer as correct or not.
	 * @param c Indicates whether the user ticked the multiple choice answer as correct.
	 */
	public void setCorrect(boolean c) {
		this.correct = c;
		changed = true;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {
		user = lookUpUser(result.getInt(1));
		userInfo = lookUpUserInfoMcAnswer(result.getInt(2));
		correct  = result.getBoolean(3);
	}

	/**
	 * Return the user, associated with this answer.
	 * 
	 * @pre       true
	 * @post      true
	 * @param id User to look up.
	 * @return User, associated with this answer.
	 */
	private User lookUpUser(int id) {
		User user = db.getUserCache().getDBObject(id);
		if (user == null) {
			user = new User(id, db);
		}
		return user;
	}
	
	/**
	 * Returns the UserInfoMcItem for the passed id.
	 * 
	 * @pre       true
	 * @post      true
	 * @param id Id of UserInfoMcItem
	 * @return UserInfoMcItem which contains the passed id. 
	 */
	private UserInfoMcItem lookUpUserInfoMcAnswer(int id) {
		UserInfoMcItem userInfo = db.getUserInfoMcItemCache().getDBObject(id);
		if (userInfo == null) {
			userInfo = new UserInfoMcItem(id, db);
		}
		return userInfo;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_USER_ANSWERS_OF_USER_INFO_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_USER_ANSWERS_OF_USER_INFO_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_USER_ANSWERS_OF_USER_INFO_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_USER_ANSWERS_OF_USER_INFO_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getUserAnswersOfUserInfoMcAnswersCache().setDBObject(user.getID()+"/"+user.getID(),this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, userInfo.getId());
		statement.setBoolean(3, correct);
		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, userInfo.getId());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		// does nothing, no id is created when the record is initially
		// created.		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, userInfo.getId());		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setInt(1, user.getID());
		statement.setInt(2, userInfo.getId());
		statement.setBoolean(3, correct);
		statement.setInt(4, user.getID());
		statement.setInt(5, userInfo.getId());
	}
}