package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * A user info question describes an additional question about the 
 * {@link ch.unizh.ifi.adora.empiricism.model.User} who answers 
 * {@link ch.unizh.ifi.adora.empiricism.model.User}.  
 * 
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 23, 2006 SM First Version
 *                      May 31, 2006 SM Comments
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 */
public class UserInfo extends AbstractTopic implements TopicDataTypes{

	/**
	 * Initializes a new user info object with the data of an existing tuple in the database.
	 * 
	 * @param id The id of the tuple from the database.
	 * @param db The database where the element is stored in.
	 */
	public UserInfo(int id, Database db) {
		super(id,db);
		refreshData();
	}
	
	/**
	 * Initializes a new user info object and creates a new tuple in the database.
	 * 
	 * @param db The database in which the user info is stored persitently.
	 */
	public UserInfo(Database db) {
		super(db);
		createObject();
	}

	/**
	 * Creates a new multiple choice item for this user info question in the database and
	 * returns the corresponding user info multiple choice item object.
	 * A multiple choice item should only be associated with this user info object
	 * if the {@link #getAnswerType()} method returns either {@link TopicDataTypes#T_MC}
	 * or {@link TopicDataTypes#T_OPTION}.
	 * 
	 * @return 
	 */
	public UserInfoMcItem createUserInfoMcAnswer() {
		return new UserInfoMcItem(this, db);
	}
	
	/**
	 * Returns a list of all user info multiple choice items that are associated with this
	 * user info question.
	 * 
	 * @return Returns a set of all multiple choice items that are associated with this
	 *         user info question. 
	 */
	public UserInfoMcItemSet getAllUserInfoMcAnswers() {
		return new UserInfoMcItemSet(this, db);
	}

	/**
	 * Returns the answer of a user info for a given user. 
	 * If the answer does not exist in the database, the tupel in 
	 * the database is created.
	 * 
	 * @param user The user which answers this user info.
	 * @return Returns the user answer of user info.
	 */
	public UserAnswersOfUserInfo getUserAnswerOfQuestion(User user) {
		return new UserAnswersOfUserInfo(db, user, this);
	}
	
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readData(java.sql.ResultSet)
	 */
	@Override
	protected void readData(ResultSet result) throws SQLException {
		question = result.getString(2);
		answerType = result.getInt(3);
	}

	
	
	/*
	 * 
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_NEXT_USER_INFO_ID;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getUserInfoCache().setDBObject(id, this);	
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, this.id);
		statement.setString(2, question);
		statement.setInt(3, answerType);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		id = resultSet.getInt(1);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setString(1, this.question);
		statement.setInt(2,this.answerType);
		statement.setInt(3, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_USER_INFO;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}
}