package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * A user info multiple choice item is associated with a {@link ch.unizh.ifi.adora.empiricism.model.UserInfo}
 * object and formulates a part or the user info question by providing an item which can 
 * be ticked true or false by the {@link ch.unizh.ifi.adora.empiricism.model.User} who 
 * answers the question.
 * 
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 24, 2006 SM First Version
 *                      May 31, 2006 SM Comments
 * @version             $Date: 2007-01-22 18:51:53 +0100 (Mon, 22 Jan 2007) $, $Author: smeier $, $Revision: 2392 $
 */
public class UserInfoMcItem extends McItem {

	/**
	 * The user info qustion to which this multiple choice item is associated.
	 */
	private UserInfo userInfo = null;
	
	/**
	 * Creates a new tuple for a user info item in the database and initializes the 
	 * corresponding object with the corresponding information.
	 *   
	 * @param userInfo The user info to which this multiple choice item is associated.
	 * @param db The database in which this item is stored.
	 */
	public UserInfoMcItem(UserInfo userInfo, Database db) {
		super(db);
		this.userInfo = userInfo;
		createObject();
	}
	
	/**
	 * Creates a new multiple choice object from an existing tuple in the database.
	 * 
	 * @param id The id of the user info multiple choice tuple in the database.
	 * @param db The database in which the information is stored.
	 */
	public UserInfoMcItem(int id, Database db) {
		super(db);
		this.id = id;
		this.refreshData();
	}
	
	/**
	 * Returns the user info which is associated with this multiple choice item.
	 * @return Returns the user info which is associated with this multiple choice item. 
	 */
	public UserInfo getUserInfo() {
		return userInfo;
	}
	
	/**
	 * Sets the user info that is associated with this multiple choice item.
	 * @param  userInfo The user info to associate this mulitple choice item with.
	 */
	public void setUserInfo(UserInfo userInfo) {
		this.userInfo = userInfo;
		changed = true;
	}
	
	/**
	 * Creates a user answer to this multiple choice item which is given by a specific
	 * user.
	 * 
	 * @param user The user who answers this multiple choice answer. 
	 * @return Returns the new user answer object.
	 */
	public UserAnswersOfUserInfoMcItems createUserAnswer(User user) {
		return new UserAnswersOfUserInfoMcItems(db, user, this);
	}
	
	/**
	 * Returns the answer of a user info mc item for a given user. 
	 * If the user info mc item does not exist in the database, the tupel in 
	 * the database is created.
	 * 
	 * @param user The user which answers the user info mc item.
	 * @return Returns the user answer of user info mc answer.
	 */
	public UserAnswersOfUserInfoMcItems getUserAnswerOfQuestion(User user) {
		return new UserAnswersOfUserInfoMcItems(db, user, this);
	}
	
	
	@Override
	protected void readData(ResultSet result) throws SQLException {
		this.description = result.getString(2);
		this.userInfo = lookUpUserInfo(result.getInt(3));		
	}

	/**
	 * Look up a user info object with given id. The user info tuple must already
	 * exist in the database. First it is looked up in the cache, if the object is already
	 * existing. If it is not, a new object from the database tuple is created and 
	 * returned.  
	 * 
	 * @param uiId The id of the user info tuple in the database.
	 * @return Returns a user info object of the tuple with the given id.
	 */
	protected UserInfo lookUpUserInfo(int uiId) {
		UserInfo ui = db.getUserInfoCache().getDBObject(uiId);
		if (ui == null) {
			ui = new UserInfo(uiId, db);
		}		
		return ui;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getCreateStatement()
	 */
	@Override
	protected String getCreateStatement() {
		return DataBaseSQLStrings.SQL_CREATE_USER_INFO_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getDeleteStatement()
	 */
	@Override
	protected String getDeleteStatement() {
		return DataBaseSQLStrings.SQL_DELETE_USER_INFO_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getLastIdSelectStatement()
	 */
	@Override
	protected String getNextIdSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_NEXT_USER_INFO_MC_ITEM_ID;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_SELECT_USER_INFO_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#getUpdateStatement()
	 */
	@Override
	protected String getUpdateStatement() {
		return DataBaseSQLStrings.SQL_UPDATE_USER_INFO_MC_ITEM;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCache()
	 */
	@Override
	protected void prepareCache() {
		db.getUserInfoMcItemCache().setDBObject(id, this);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareCreateStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareCreateStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, this.id);
		statement.setString(2, description);
		statement.setInt(3, this.userInfo.getId());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareDeleteStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareDeleteStatement(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#readLastId(java.sql.ResultSet)
	 */
	@Override
	protected void readNextId(ResultSet resultSet) throws SQLException {
		id = resultSet.getInt(1);		
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareSelectString(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectString(PreparedStatement statement) throws SQLException {
		statement.setInt(1, id);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DatabaseObject#prepareWriteData(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareWriteData(PreparedStatement statement) throws SQLException {
		statement.setString(1, description);
		statement.setInt(2, this.userInfo.getId());
		statement.setInt(3, id);
	}
}