package ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager;

import java.util.List;

import ch.unizh.ifi.adora.empiricism.model.AbstractTopic;
import ch.unizh.ifi.adora.empiricism.model.TestSuite;
import ch.unizh.ifi.adora.empiricism.model.TopicDataTypes;
import ch.unizh.ifi.adora.empiricism.model.User;

/**
 * This class operates as a memory for the whole wizard state machine. This
 * memory is used to decide which page will be shown next, or which page whill
 * be shown previously.
 * 
 * During a questioning cycle, up to 2 PageManagers get instaciated. The first
 * gets instatiated before the first user info is asked. The secound gets
 * instatiated before the first question is asked.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 12.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class PageManager {

	/**
	 * Testsuite, which is dealt durring this session.
	 */
	private TestSuite suite;

	/**
	 * This list is the actual memory of the process. It contains all topics
	 * associated with this suite
	 */
	private List set;

	/**
	 * User which answers the topics.
	 */
	private User user;

	/**
	 * Index of the Current DataBaseElement in the ObjectSet. With the help of
	 * this variable, the whole process is controlled.
	 */
	private int currentPageNr = -1;

	/**
	 * PageManager Constructor
	 * 
	 * @param set
	 *            This list is the actual memory of the process. It contains all
	 *            topics associated with this suite
	 * @param user
	 *            User which answers the topics.
	 * @param suite
	 *            Testsuite, which is dealt durring this session
	 */
	public PageManager(List set, User user, TestSuite suite) {
		this.set = set;
		this.user = user;
		this.suite = suite;
	}

	/**
	 * Checks if the state machine is already at the final state.
	 * 
	 * @return True, if more topics exist.
	 */
	public boolean hasNextPage() {
		return currentPageNr + 1 < set.size();
	}

	/**
	 * Returns the next Question/UserInfo Tupel.
	 * 
	 * @return
	 */
	public AbstractTopic getNextTopic() {
		currentPageNr++;

		if (currentPageNr < set.size()) {
			return getCurrentTopic();
		}

		return null;
	}

	/**
	 * Used to determin what kind of page will be instantiated next.
	 * 
	 * @return true if the next question's answer is a multiplechoice answer
	 */
	public boolean isNextMultipleChoice() {
		if (currentPageNr + 1 >= set.size()) {
			return false;
		}

		int answerType = ((AbstractTopic) set.get(currentPageNr + 1))
				.getAnswerType();

		if (answerType == TopicDataTypes.T_MC
				|| answerType == TopicDataTypes.T_OPTION) {
			return true;
		}

		return false;
	}

	/**
	 * Peeks the next Topic without incrementing the {@link #currentPageNr}.
	 * 
	 * @return next Topic or null
	 */
	public AbstractTopic peekNextTopic() {
		AbstractTopic topic = getNextTopic();
		currentPageNr--;
		return topic;
	}

	/**
	 * @return the current Question/UserInfo Tupel or null, if no more Topics
	 *         exists
	 */
	public AbstractTopic getCurrentTopic() {
		if (currentPageNr < set.size()) {
			return (AbstractTopic) set.get(currentPageNr);
		}

		return null;
	}

	/**
	 * Returns the previous Question/UserInfo Tupel or null if the current page
	 * is the first page.
	 * 
	 * @return
	 */
	public AbstractTopic getPreviousTopic() {
		currentPageNr--;

		if (currentPageNr > -1) {
			return getCurrentTopic();
		}

		return null;
	}

	/**
	 * Return the current user.
	 * 
	 * @pre true
	 * @post true
	 * @obligation
	 * @return current user
	 */
	public User getUser() {
		return user;
	}

	/**
	 * @return Returns the suite, which is dealt durring this session.
	 */
	public TestSuite getSuite() {
		return suite;
	}

	/**
	 * @param suite
	 *            The suite to set.
	 */
	public void setSuite(TestSuite suite) {
		this.suite = suite;
	}
}