package ch.unizh.ifi.adora.empiricism.views.wizard.pages.loginpages;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import ch.unizh.ifi.adora.empiricism.adoraeditor.model.AdoraCommandListener;
import ch.unizh.ifi.adora.empiricism.model.TestSuite;
import ch.unizh.ifi.adora.empiricism.model.User;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.AbstractEmpericismPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.uimessages.UIMessages;

/**
 * Supplies methods used by all LoginPages.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 16.06.2006 SM First Version
 * @version $Date: 2008-11-28 11:41:23 +0100 (Fri, 28 Nov 2008) $, $Author: vigan $, $Revision: 3907 $
 */
public abstract class AbstractLoginPage extends AbstractEmpericismPage {

	/**
	 * Text, contained in the label, situated left of the text box. Used to
	 * describe the textbox' function.
	 */
	private String textBoxText;

	/**
	 * User has to enter ID or age in here.
	 */
	private Text textField;

	/**
	 * Handles the navigation thru the question- and user info pages.
	 */
	private PageManager pageManager;

	/**
	 * Constructs a AbstractLoginPage.
	 * 
	 * @param textBoxText
	 *            Text, contained in the label, situated left of the text box.
	 *            Used to describe the textbox' function.
	 * @param wizard
	 *            Wizard, to whom this Page belongs.
	 */
	public AbstractLoginPage(String textBoxText, IWizard wizard) {
		super(wizard);

		this.textBoxText = textBoxText;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {

		GridData gd;
		Composite composite = new Composite(parent, SWT.NULL);

		// create the desired layout for this wizard page
		GridLayout gl = new GridLayout();
		int numCols = 2;
		gl.numColumns = numCols;
		composite.setLayout(gl);

		gd = new GridData();
		gd.horizontalAlignment = GridData.BEGINNING;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalSpan = numCols;

		Label lbl = new Label(composite, SWT.WRAP | SWT.LEFT);
		lbl.setText(getDescriptionText());
		lbl.setLayoutData(gd);

		createLine(composite, numCols);

		new Label(composite, SWT.LEFT).setText(textBoxText);

		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = true;

		textField = new Text(composite, SWT.SINGLE | SWT.BORDER);
		textField.setLayoutData(gd);
		textField.setFocus();

		// set the composite as the control for this page
		setControl(composite);
		addListeners();
	}

	/**
	 * Returns a String, used to describe the action which has to be taken.
	 * 
	 * @pre true
	 * @post true
	 * @obligation
	 * @return
	 */
	protected abstract String getDescriptionText();

	/**
	 * Adds a listener to the {@link org.eclipse.swt.widgets.Text}. This is
	 * used to check the data during entry.
	 * 
	 * @pre true
	 * @post true
	 * @obligation
	 */
	private void addListeners() {
		textField.addListener(SWT.KeyUp, this);
	}

	/**
	 * Sets the completed field on the wizard class when all the information is
	 * entered and the wizard can be completed
	 */
	public boolean isPageComplete() {
		return isNumber(textField.getText());
	}

	/**
	 * Process the events: when the user has entered all information the wizard
	 * can be finished
	 */
	public void handleEvent(Event event) {

		if (event.widget == textField) {
			conditionalShowStatusMessage(UIMessages.NOT_NEG_ERR_MSG,
					!isNumber(textField.getText()));
		}
		setPageComplete(isPageComplete());
		getWizard().getContainer().updateButtons();
	}

	/**
	 * Creates a new user and choses a testsuite to answer and the corresponding
	 * UserInfos.
	 * 
	 * @return true if a testSuite exists
	 */
	protected abstract boolean saveDataToModel();

	/**
	 * This method is used to pass the pagemanager to the next page.
	 * 
	 * @return Returns the PageManager.
	 */
	public PageManager getPageManager() {
		return pageManager;
	}

	/**
	 * @param pm
	 *            The PageManager to be set.
	 */
	public void setPageManager(PageManager pm) {
		this.pageManager = pm;
	}

	/**
	 * @return Returns the textField.
	 */
	public Text getTextField() {
		return textField;
	}

	/**
	 * @param textField
	 *            The textField to set.
	 */
	public void setTextField(Text textField) {
		this.textField = textField;
	}

	/**
	 * Creates the PageManager for a given user. This manager contains all
	 * userInfos (if existent) or all questions.
	 * 
	 * @param user
	 *            User, for whom the {@link PageManager} is been created.
	 */
	protected boolean createPageManager(User user) {
		AdoraCommandListener.getInstance().setUserID(user.getID());
		TestSuite ts = user.getLeastUsedTestSuit();
		// all suites answered
		if (ts == null) {
			return false;
		}
		setPageManager(new PageManager(user.getUnansweredUserInfos(ts), user,
				ts));
		return true;
	}
}