package ch.unizh.ifi.adora.figureexporter;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.Path;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayeredPane;
import org.eclipse.draw2d.Shape;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.editor.test.AdoraEditorTestCase;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.parts.AdoraModelEditPart;
import ch.unizh.ifi.adora.parts.AdoraRootEditPart;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceValueConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;
import de.unikassel.imageexport.exporters.ImageExporterDescriptor;
import de.unikassel.imageexport.exporters.ImageExporters;

/**
 * The FigureExporter can be used as base class to export ADORA models as images
 * (svg, pdf, ...). It builds on the eclipse plug-in test framework and the
 * imageexporter of the University of Kassel (de.unikassel.imageexport).
 * 
 * Maximal width of a node == 870
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-01-16 TR First Version
 * @version				$Date: 2009-11-12 11:02:59 +0100 (Thu, 12 Nov 2009) $, $Author: treinhard $, $Revision: 4944 $
 *
 */
public class FigureExporter extends AdoraEditorTestCase {
	
	protected void setUp() {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
	
		// Set the continuous zoom as layout algorithm for all layout operations
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ZOOM_ALGORITHM, 
				IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
	
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_INSERT_LAYOUT_ALGORITHM, 
				IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
	
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_REMOVE_LAYOUT_ALGORITHM, 
				IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
	
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_MOVE_LAYOUT_ALGORITHM, 
				IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
	
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_FILTER_LAYOUT_ALGORITHM, 
				IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
		
		// Shade the hierarchy
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, true);
		// And set the shading color
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_COLOR, "240,240,240");
		
		// Set the font for the node labels
        FontData fontData = new FontData();
        fontData.setName("arial");
        fontData.setHeight(22);
        PreferenceConverter.setValue(store, IPreferenceNameConstants.ADORA_VIEW_NODE_LABEL_FONT, fontData);
        
        // Set the font for the connection labels
        fontData = new FontData();
        fontData.setName("arial");
        fontData.setHeight(14);
        PreferenceConverter.setValue(store, IPreferenceNameConstants.ADORA_VIEW_CONNECTION_LABEL_FONT, fontData);
		
		// Don't add the nodes centered
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_INSERT_CENTERED, true);
	
	}
	
	
	/**
	 * Export the model given as argument <i>editor</i> to a pdf file with the
	 * filename given as argument <i>filename</i>.
	 * 
	 * @param editor the model that should be exported
	 * @param fileName the name of the exported pdf file
	 */
	protected void exportModel(IEditorPart editor, String fileName) {
        
        GraphicalViewer viewer = (GraphicalViewer)editor.getAdapter(GraphicalViewer.class);
		AdoraRootEditPart rootEditPart = (AdoraRootEditPart)viewer.getRootEditPart();
		String imageFormat = fileName.substring(fileName.lastIndexOf(".") + 1);
		
		viewer.flush();
		
		AdoraModelEditPart modelEditPart = (AdoraModelEditPart)rootEditPart.getChildren().get(0);
		LayeredPane layeredPane = (LayeredPane)modelEditPart.getFigure().getParent().getParent();
		
		List<IFigure> figures = new ArrayList<IFigure>();
		figures.add(layeredPane);
		
		// Adjust the line width (the figure will be scaled with the factor 0.5)
		setLineWidth(layeredPane, 1);
		
		List<ImageExporterDescriptor> exporterDescs = ImageExporters.get().getImageExporterDescriptors();
		
		ImageExporterDescriptor exporterDescriptor = null;
		
		for(ImageExporterDescriptor desc : exporterDescs) {
			if(desc.getFileExtension().equals(imageFormat)) {
				exporterDescriptor = desc;
			}
		}
		
		
		if(exporterDescriptor != null) {
			
			String folderPath = System.getProperty("user.home") + Path.SEPARATOR + "figures";
			
			File directory = new File(folderPath);
			if(!directory.exists()) {
				new File(folderPath).mkdirs();
			}
			
			File imageFile = new File(folderPath + Path.SEPARATOR + fileName);
			boolean antialiasing = true;
		
			exporterDescriptor.createExporter().export(figures, imageFile, antialiasing);
		}
		
		// Reset the line width
		setLineWidth(layeredPane, 0.5);
		
	}
	
	
	/**
	 * Add a new component to the given <i>parent</i> with the given <i>bounds</i>
	 * and the given <i>name</i>. The method returns the newly added node.
	 * 
	 * @param parent the parent the node should be added to
	 * @param bounds the bounds of the new node
	 * @param name the name of the new node
	 * @return the newly added nodes
	 */
	protected Component addComponent(INodeContainer parent, Rectangle bounds, String name) {
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		Component comp = new Component();
		comp.setLocation(bounds.getLocation());
		comp.setSize(bounds.getSize());
		comp.setName(name);
		
		parent.addChild(comp);
		zoom.insertNode(comp);
		
		return comp;
		
	}
	
	
	/**
	 * Method to add an association between <i>source</i> and <i>target</i>
	 * with the clickpoints <i>cp1</i> and <i>cp2</i> respectively. Returns the
	 * newly created associaton.
	 * 
	 * @param source the sourne node of the association
	 * @param cp1 the clickpoint inside the source node (relative coordinates)
	 * @param target the target node of the association
	 * @param cp2 the clickpoint inside the target node (relative coordinates)
	 * @return the newly created association
	 */
	protected Association addAssociation(Node source, Point cp1,
			Node target, Point cp2) {
		
		Association asso = new Association(source, target);
		asso.setSourceClickPoint(cp1);
		asso.setTargetClickPoint(cp2);
		
		return asso;
	}
	
	
	/**
	 * Adjusts the line width of the figure <i>parent</i> and all its direct
	 * and indirect children by the <i>factor</i>.
	 * 
	 * This method is used for exported models that are later scaled down.
	 * 
	 * @param parent the figure to start with
	 * @param factor the factor to adjust the line width
	 */
	private void setLineWidth(IFigure parent, double factor) {
		
		if(parent instanceof Shape) {
			Shape shape = (Shape)parent;
			shape.setLineWidth((int)Math.round(factor * shape.getLineWidth()));
		}
		
		for(int i = 0; i < parent.getChildren().size(); i++) {
			IFigure child = (IFigure)parent.getChildren().get(i);
			setLineWidth(child, factor);
		}
		
	}

}
