package ch.unizh.ifi.adora.linerouting.algorithm;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.linerouting.tilestructure.Tile;

/**
 * The OrthogonalBendpointCalculator uses the tile path constructed by the 
 * TileMazeRouter to calculate the bend points in the line for an orthogonal routing.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-03-21 TR First Version
 * @version				$Date: 2009-04-29 16:41:27 +0200 (Wed, 29 Apr 2009) $, $Author: treinhard $, $Revision: 4138 $
 *
 */
public class OrthogonalBendpointCalculator {
	
	/**
	 * The path consisting of tiles the bend points is calculated for
	 */
	private List<Tile> path;
	
	/**
	 * The start point of the line (coordinates are relative to the tile structure)
	 */
	private Point start;
	
	/**
	 * The end point of the line (coordinates are relative to the tile structure)
	 */
	private Point end;
	
	/**
	 * The tile that contains the start point of the line
	 */
	private Tile startTile;
	
	/**
	 * The right boundary of the horizontal interval
	 */
	private int rightX;
	
	/**
	 * The left boundary of the horizontal interval
	 */
	private int leftX;
	
	/**
	 * The top boundary of the vertical interval
	 */
	private int topY;
	
	/**
	 * The bottom boundary of the vertical interval
	 */
	private int bottomY;
	
	/**
	 * The direction the line enters the current tile from
	 * (constant from RoutingConstants)
	 */
	private int fromSide = -1;
	
	/**
	 * The direction the line leaves the current tile to
	 * (constant from RoutingConstants)
	 */
	private int toSide = -1;
	
	/**
	 * The last vertex in the line
	 */
	private Point lastVertex;
	
	/**
	 * List that contains all the bend points of the line (including start and end
	 * point)
	 */
	private List<Point> candidate;
	
	private int borderDist = 2;
	
	
	/**
	 * Constructor
	 * 
	 * @param start the start point (coordinates are relative to the tile structure)
	 * @parent end the end point (coordinates are relative to the tile structure)
	 * @param path the path consisting of the space tiles the line goes through
	 * 
	 */
	public OrthogonalBendpointCalculator(Point start, Point end, List<Tile> path) {
		
		this.path = path;
		this.start = start;
		this.end = end;
		
		startTile = path.get(0);
		
		candidate = new ArrayList<Point>();
		
	}
	

	/**
	 * This method does the orthogonal routing of a line for the path
	 * consisting of space tiles that was set in the constructor. The method 
	 * returns a list that contains all the line vertices that have to be
	 * inserted into the line to route it orthogonal.
	 * 
	 * @return a list containing all the bend points of the line
	 * 
	 */
	public List<Point> getBendpoints() {

		candidate.add(start);
		candidate.add(end);	

		// Set the borders of the vertical and the horizontal interval to the
		// boundaries of the start tile
		rightX = startTile.getBottomRight().x;
		leftX = startTile.getBottomLeft().x;
		topY = startTile.getTopLeft().y;
		bottomY = startTile.getBottomLeft().y;

		// The direction the line enters and leaves the current tile
		fromSide = -1;
		toSide = -1;

		for(int i = 0; i < path.size() - 1; i++) {

			Tile currentTile = path.get(i);
			Tile nextTile = path.get(i+1);
			lastVertex = candidate.get(candidate.size() - 2);

			// The direction from the currentTile to the next one
			toSide = direction(currentTile, nextTile);

			if (currentTile == startTile) {

				handleStartTile(currentTile, nextTile);

			}

			// CASE 1: The line crosses the tile vertical or enters it from
			// north or south and leaves it in the same direction
			if ((fromSide == RoutingConstants.NORTH || fromSide == RoutingConstants.SOUTH)
					&& (toSide == RoutingConstants.NORTH || toSide == RoutingConstants.SOUTH)) {

				handleVerticalCrossing(currentTile, nextTile);

			}

			// CASE 2: The line crosses the tile horizontal or enters it from
			// west or east and leaves it in the same direction.
			else if ((fromSide == RoutingConstants.WEST || fromSide == RoutingConstants.EAST)
					&& (toSide == RoutingConstants.WEST || toSide == RoutingConstants.EAST)) {

				handleHorizontalCrossing(currentTile, nextTile);
			}

			// CASE 3: The line changes it's direction
			else if (((fromSide == RoutingConstants.NORTH || fromSide == RoutingConstants.SOUTH) 
					  && (toSide == RoutingConstants.WEST || toSide == RoutingConstants.EAST))
					|| ((fromSide == RoutingConstants.WEST || fromSide == RoutingConstants.EAST) 
					  && (toSide == RoutingConstants.NORTH || toSide == RoutingConstants.SOUTH))) {

				handleDirectionChange(currentTile, nextTile);

			}

			// The line enters the next tile from the opposite direction than
			// it left the current tile
			fromSide = (toSide + 2) % 4;
		}

		// Adjust the last segment of the line
		handleEndTile();
		
		
		return candidate;

	}

	
	/**
	 * Adjusts the bend points for the first tile in the path (i.e., the tile that
	 * contains the start point of the line).
	 * 
	 * @param currentTile the current tile in the path (that's the start tile)
	 * @param nextTile the next tile the line has to go through
	 */
	private void handleStartTile(Tile currentTile, Tile nextTile) {
		
		Rectangle currentBounds = currentTile.getBounds();			
		Rectangle nextBounds = nextTile.getBounds();
		
		if (toSide == RoutingConstants.WEST || toSide == RoutingConstants.EAST) {					
			
			if (nextBounds.y > currentBounds.y) {
				topY = nextBounds.y;
			}

			if (nextBounds.getMaxY() < currentBounds.getMaxY()) {
				bottomY = (int)nextBounds.getMaxY();
			} 
			
			
			// The start point lies outside the vertical interval
			if(start.y < topY + borderDist || start.y > bottomY - borderDist) {
				
				int y = topY + ((bottomY - topY) / 2);
				Point p = new Point(start.x, y);

				candidate.set(0, p);
				
			}

		}

		else if (toSide == RoutingConstants.NORTH || toSide == RoutingConstants.SOUTH) {

			if (nextBounds.x > currentBounds.x) {
				leftX = nextBounds.x;
			}

			if (nextBounds.getMaxX() < currentBounds.getMaxX()) {
				rightX = (int)nextBounds.getMaxX();
			}

			// The start point lies outside the horizontal interval
			if(start.x < leftX + borderDist || start.x > rightX - borderDist) {
				int x = leftX + ((rightX - leftX) / 2);
				Point p = new Point(x, start.y);
				candidate.set(0, p);
			}

		}
		
	}
	
	
	/**
	 * Adjusts the bend points for the last tile in the path (i.e., the tile that
	 * contains the end point of the line).
	 */
	private void handleEndTile() {
		
		if (fromSide == RoutingConstants.NORTH || fromSide == RoutingConstants.SOUTH) {
			
			// We can connect the start and end point by a straight horizontal line
			if(candidate.size() == 2) {
				int x = (start.x + end.x) / 2;
				if(x < leftX + borderDist || x > rightX - borderDist) {
					x = (leftX + rightX) / 2;
				}
				candidate.set(1, new Point(x, end.y));
				candidate.set(0, new Point(x, start.y));
			}
			// The end point lies within the horizontal interval
			else if(end.x > leftX && end.x < rightX) {
				// Set the horizontal coordinate of the last bend point to the 
				// horizontal coordinate of the end point.
				Point p = new Point(end.x, candidate.get(candidate.size() - 2).y);
				candidate.set(candidate.size() - 2, p);
			} 
			// The end point doesn't lie within the horizontal interval
			else {
				// Set the horizontal coordinate of the end point to the
				// horizontal coordinate of the last bend point.
				candidate.set(candidate.size() - 1, new Point(candidate.get(candidate.size() - 2).x, end.y));
			}
			
		}
		else if(fromSide == RoutingConstants.WEST || fromSide == RoutingConstants.EAST) {
			
			// We can connect the start and end point by a straight vertical line
			if(candidate.size() == 2) {
				int y = (start.y + end.y) / 2;
				if(y < topY + borderDist || y > bottomY - borderDist) {
					y = (topY + bottomY) / 2;
				}
				candidate.set(1, new Point(end.x, y));
				candidate.set(0, new Point(start.x, y));
			}
			// The end point lies within the vertical interval
			else if(end.y > topY && end.y < bottomY) {
				// Set the vertical coordinate of the last bend point to the 
				// vertical coordinate of the end point.
				Point p = new Point(candidate.get(candidate.size() - 2).x, end.y);
				candidate.set(candidate.size() - 2, p);
			} 
			// The end point doesn't lie within the vertical interval
			else {
				// Set the vertical coordinate of the end point to the
				// horizontal coordinate of the last bend point.
				candidate.set(candidate.size() - 1, new Point(end.x, candidate.get(candidate.size() - 2).y));
			}
			
		}
		
	}
	
	
	/**
	 * Adjusts the bend points if the line crosses a tile vertically or enters
	 * it from north or south and leaves it in the same direction.
	 * 
	 * @param currentTile the current tile in the path
	 * @param nextTile the next tile the line has to go through
	 */
	private void handleVerticalCrossing(Tile currentTile, Tile nextTile) {
		
		Rectangle currentBounds = currentTile.getBounds();			
		Rectangle nextBounds = nextTile.getBounds();
		
		// The position of the horizontal line segment
		int y = currentTile.getCenter().y;

		// Maintain a maximal distance if the line is routed around an
		// obstacle
		if (fromSide == toSide
			&& currentTile.getHeight() > RoutingConstants.MAX_SIZE_TO_CENTER) {
			
			if (fromSide == RoutingConstants.NORTH) {
				y = currentBounds.y + RoutingConstants.DIST_TO_NODE;
			} else {
				y = (int)currentBounds.getMaxY() - RoutingConstants.DIST_TO_NODE;
			}
		}

		// No part of the next tile lies inside the interval.
		// So we need two more line vertices.
		if ((nextBounds.x > rightX && nextTile.getBottomRight().x > rightX)
			|| (nextBounds.x < leftX && nextTile.getBottomRight().x < leftX)) {

			// Maintain a maximal distance if the line is routed around
			// an obstacle
			int lastX = lastVertex.x;
			if (candidate.size() > 2 && rightX - leftX > RoutingConstants.MAX_SIZE_TO_CENTER) {
				
				if (candidate.get(candidate.size() - 3).x < lastVertex.x
					&& (nextBounds.x < leftX && nextTile.getBottomRight().x < leftX)) {
					lastX = leftX + RoutingConstants.DIST_TO_NODE;
				} else if (candidate.get(candidate.size() - 3).x > lastVertex.x
						   && (nextBounds.x > rightX
						   && nextBounds.getMaxX() > rightX)) {
					lastX = rightX - RoutingConstants.DIST_TO_NODE;
				}
			}

			// Make the interval as tight as possible, take the left
			// border of that tile (current or next) that has the bigger x
			// coordinate
			if (nextBounds.x > currentBounds.x) {
				leftX = nextBounds.x;
			} else {
				leftX = currentBounds.x;
			}

			// Make the interval as tight as possible, take the right
			// border of that tile (current or next) that has the smaller x
			// coordinate
			if (nextBounds.getMaxX() < currentTile.getBottomRight().x) {
				rightX = (int)nextBounds.getMaxX();
			} else {
				rightX = (int)currentBounds.getMaxX();
			}

			if (lastX != lastVertex.x) {
				candidate.set(candidate.size() - 2, new Point(lastX, candidate
						.get(candidate.size() - 2).y));
			}

			// Insert the two line vertices
			candidate.add(candidate.size() - 1, new Point(lastX, y));
			candidate.add(candidate.size() - 1, new Point((leftX + rightX) / 2, y));
		}

		// The new and the previous interval overlap, so we can move the
		// last vertex
		else {

			if (nextBounds.x > leftX) {
				leftX = nextBounds.x;
			}
			
			if (nextBounds.getMaxX() < rightX) {
				rightX = (int)nextBounds.getMaxX();
			}

			if(!(candidate.size() == 2 && lastVertex.x > leftX && lastVertex.x < rightX)) {
			
				Point p = new Point((leftX + rightX) / 2, lastVertex.y);
				candidate.set(candidate.size() - 2, p);
			}
		}
		
	}
	
	
	/**
	 * Adjusts the bend points if the line crosses a tile horizontally or enters
	 * it from west or east and leaves it in the same direction.
	 * 
	 * @param currentTile the current tile in the path
	 * @param nextTile the next tile the line has to go through
	 */
	private void handleHorizontalCrossing(Tile currentTile, Tile nextTile) {
		
		Rectangle currentBounds = currentTile.getBounds();			
		Rectangle nextBounds = nextTile.getBounds();
		
		// The next tile lies outside the current vertical interval
		if ((nextBounds.y < topY && nextBounds.getMaxY() < topY)
			|| (nextBounds.y > bottomY && nextBounds.getMaxY() > bottomY)) {

			int x = currentTile.getCenter().x;
			
			// Reflexive line
			if (fromSide == toSide
					&& currentTile.getWidth() > RoutingConstants.MAX_SIZE_TO_CENTER) {
				if (fromSide == RoutingConstants.WEST) {
					x = currentBounds.x + RoutingConstants.DIST_TO_NODE;
				} else {
					x = (int)currentBounds.getMaxX() - RoutingConstants.DIST_TO_NODE;
				}
			}
			
			// Add two new bend points
			candidate.add(candidate.size() - 1, new Point(x, lastVertex.y));
			candidate.add(candidate.size() - 1, new Point(x, currentTile.getCenter().y));

			// The vertical interval is set to the vertical bounds of the next tile
			topY = nextBounds.y;
			bottomY = (int)nextBounds.getMaxY();

		}
		
		// The next tile lies within the current vertical interval or overlaps
		// partially or completely with the current vertical interval.
		else if((nextBounds.y < topY && nextBounds.getMaxY() > topY) 
				|| (nextBounds.y < bottomY && nextBounds.getMaxY() > bottomY)
				|| (nextBounds.y >= topY && nextBounds.getMaxY() <= bottomY)) {
			
			if (nextBounds.y > topY) {
				topY = nextBounds.y;
			}

			if (nextBounds.getMaxY() < bottomY) {
				bottomY = (int)nextBounds.getMaxY();
			}

			if(!(candidate.size() == 2 && start.y > topY && start.y < bottomY)) {
			
				Point p = new Point(lastVertex.x, (topY + bottomY) / 2);
				candidate.set(candidate.size() - 2, p);
			}
			
		}
	}
	
	
	private void handleDirectionChange(Tile currentTile, Tile nextTile) {
		
		Rectangle currentBounds = currentTile.getBounds();			
		Rectangle nextBounds = nextTile.getBounds();
		
		// Line leaves the current tile to the north or south
		// (i.e., it enters the tile from the west or east)
		if (toSide == RoutingConstants.NORTH || toSide == RoutingConstants.SOUTH) {

			if (nextBounds.x > currentBounds.x) {
				leftX = nextBounds.x;
			} else {
				leftX = currentBounds.x;
			}

			if (nextBounds.getMaxX() < currentBounds.getMaxX()) {
				rightX = (int)nextBounds.getMaxX();
			} else {
				rightX = (int)currentBounds.getMaxX();
			}

			int x = leftX + ((rightX - leftX) / 2);
			Point p = new Point(x, lastVertex.y);

			candidate.add(candidate.size() - 1, p);

		}

		// Line leaves the current tile to the west or east
		// (i.e., it enters the tile from the north or south)
		else if (toSide == RoutingConstants.WEST || toSide == RoutingConstants.EAST) {

			if (nextBounds.y > currentBounds.y) {
				topY = nextBounds.y;
			} else {
				topY = currentBounds.y;
			}

			if (nextBounds.getMaxY() < currentBounds.getMaxY()) {
				bottomY = (int)nextBounds.getMaxY();
			} else {
				bottomY = (int)currentBounds.getMaxY();
			}

			// Maintain a maximal distance if the line is routed around
			// an obstacle
			int lastX = lastVertex.x;
			if (candidate.size() > 2 && rightX - leftX > RoutingConstants.MAX_SIZE_TO_CENTER) {
				if (candidate.get(candidate.size() - 3).x < lastVertex.x
						&& candidate.get(candidate.size() - 1).x < lastVertex.x) {
					lastX = leftX + RoutingConstants.DIST_TO_NODE;
				} else if (candidate.get(candidate.size() - 3).x > lastVertex.x
						&& candidate.get(candidate.size() - 1).x > lastVertex.x) {
					lastX = rightX - RoutingConstants.DIST_TO_NODE;
				}
				candidate.set(candidate.size() - 2, new Point(lastX, lastVertex.y));
			}

			int y = topY + ((bottomY - topY) / 2);
			Point p = new Point(lastX, y);

			candidate.add(candidate.size() - 1, p);

		}
		
	}
	
	
	/**
	 * Returns the direction from the tile currentTile in which the tile
	 * neighbourTile lies. The return value is one of the constants NORTH,
	 * SOUTH, EAST or WEST or -1 if the two tiles are not neighbors.
	 * 
	 * @param currentTile
	 *            the tile to check the neighborhood
	 * @param neighbourTile
	 *            the tile of which the direction should be returned
	 * @return the direction in which the neighbourTile lies
	 * 
	 * @pre currentTile != null
	 * @post true
	 * 
	 */
	private int direction(Tile currentTile, Tile neighbourTile) {
		// Currently this method is implemented by checking all the corner
		// stitches of the tile currentTile.

		// Check top neighbors
		Tile neighbor = currentTile.getLt();
		while (neighbor != null) {
			// Return if the neighbourTile is a top neighbor of the current
			// tile
			if (neighbor == neighbourTile) {
				return RoutingConstants.NORTH;
			}
			// There are no more top neighbors
			if (neighbor == currentTile.getRt()) {
				break;
			}
			neighbor = neighbor.getBr();
		}

		// Check bottom neighbors
		neighbor = currentTile.getLb();
		while (neighbor != null) {
			// Return if the neighbourTile is a bottom neighbor of the current
			// tile
			if (neighbor == neighbourTile) {
				return RoutingConstants.SOUTH;
			}
			// There are no more bottom neighbors
			if (neighbor == currentTile.getRb()) {
				break;
			}
			neighbor = neighbor.getTr();
		}

		// Check left neighbors
		neighbor = currentTile.getTl();
		while (neighbor != null) {
			// Return if the neighbourTile is a left neighbor of the current
			// tile
			if (neighbor == neighbourTile) {
				return RoutingConstants.WEST;
			}
			// There are no more left neighbors
			if (neighbor == currentTile.getBl()) {
				break;
			}
			neighbor = neighbor.getRb();
		}

		// Check right neighbors
		neighbor = currentTile.getTr();
		while (neighbor != null) {
			// Return if the neighbourTile is a right neighbor of the current
			// tile
			if (neighbor == neighbourTile) {
				return RoutingConstants.EAST;
			}
			// There are no more right neighbors
			if (neighbor == currentTile.getBr()) {
				break;
			}
			neighbor = neighbor.getLb();
		}

		// The neighbourTile isn't a neighbor of the currentTile
		return -1;
	}

}
