package ch.unizh.ifi.adora.linerouting.algorithm;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import ch.unizh.ifi.adora.linerouting.factors.BoundaryFactor;
import ch.unizh.ifi.adora.linerouting.factors.InfluenceFactor;
import ch.unizh.ifi.adora.linerouting.factors.NodeCrossingFactor;
import ch.unizh.ifi.adora.linerouting.factors.UniformityFactor;
import ch.unizh.ifi.adora.linerouting.factors.VertexFactor;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.linerouting.labels.OrthogonalLabelPlacement;
import ch.unizh.ifi.adora.linerouting.tilestructure.InvalidLayoutException;
import ch.unizh.ifi.adora.linerouting.tilestructure.LabelTileStructure;
import ch.unizh.ifi.adora.linerouting.tilestructure.LineTile;
import ch.unizh.ifi.adora.linerouting.tilestructure.LineTileStructure;
import ch.unizh.ifi.adora.linerouting.tilestructure.NodeTile;
import ch.unizh.ifi.adora.linerouting.tilestructure.Tile;
import ch.unizh.ifi.adora.linerouting.tilestructure.TileStructure;
import ch.unizh.ifi.adora.linerouting.utils.HierarchyUtilities;


/**
 * Implementation of an orthogonal routing algorithm that is based on the
 * {@link routing.algorithm.TileMazeRouter}.
 * 
 * Use of composition instead of inheritance so that this algorithm can be used
 * for different algorithms that find the tiles paths.
 * 
 * @author Tobias Reinhard (TR)
 * @copyright Requirements Engineering Research Group Department of Computer
 *            Science University of Zurich
 * @history 2005-03-07 TR First version 2005-03-18 TR Measure and print the time
 *          that is needed to construct the tile structure, print all solutions
 *          2005-03-21 TR Integrate the terminal selection in the algorithm
 *          2005-05-18 TR Decoupled from the main adora package 
 *          2005-05-19 TR Some small bugfixes 
 *          2005-06-14 TR Extended tile structure implemented
 *          2006-08-04 JS The constructionLayer is now on top of the other layers
 *                        and is disabled, so it will ignore mouse events
 *          2006-09-06 JS Routing is canceled if the target or source aren't inside
 *                        the bounds of the routing layer
 *          
 * 
 * @version $Date: 2009-12-14 15:59:25 +0100 (Mon, 14 Dec 2009) $, $Author: treinhard $, $Revision: 4993 $
 * @responsibilities Routes the lines orthogonal
 */
public class OrthogonalTileMazeRouter {
	
	/**
	 * Logger for this class
	 */
	private static final Logger logger = 
				Logger.getLogger(OrthogonalTileMazeRouter.class.getName());

	/**
	 * The starting point of the connection
	 */
	private Point start;

	/**
	 * The end point of the connection
	 */
	private Point end;

	/**
	 * The parent node of the current line (i.e. the node the line is completely
	 * contained in)
	 */
	private IRoutingNode parentNode;
	
    /**
     * The corner stitching data structure.
     */
    protected TileStructure structure;

    /**
     * The line that has to be routed
     */
    protected IRoutingLine line;

	/**
	 * The singleton instance
	 */
	private static OrthogonalTileMazeRouter instance;
	
	/**
	 * Helper methods to compute hierarchical relations
	 */
	private HierarchyUtilities hu = HierarchyUtilities.getInstance();


	/**
	 * Hide the constructor because the router is a singleton.
	 */
	private OrthogonalTileMazeRouter() {	

		super();

	}
	
	
	/**
	 * Method to get the singleton instance.
	 * 
	 * @return the singleton instance
	 */
	public static OrthogonalTileMazeRouter getInstance() {
		
		if(instance == null) {
			instance = new OrthogonalTileMazeRouter();
		}
		
		return instance;
		
	}

	
	/**
	 * Routes the line given as argument <i>line</i> orthogonal. The start
	 * and end point of the line must be the first and last element in the
	 * bend points list (IRoutingLine.getRoutingBendpoints()) of the line. The
	 * coordinates of the start and end point have to be relative to the start/
	 * end node.<br>
	 * 
	 * @param line line the line that has to be routed orthogonal
	 */
	public void route(IRoutingLine line) {
		
		route(line, new ArrayList<IRoutingLine>(0));
		
	}
	
	
	/**
	 * Routes the line given as argument <i>line</i> orthogonal while
	 * considering the lines in the list <i>lines</i> as obstacles. The start
	 * and end point of the line are obtained by calling getRoutingStartPoint() 
	 * and getRoutingEndPoint on the line. The coordinates of the start and end 
	 * point have to be relative to the start/end node.<br>
	 * 
	 * The coordinates of the lines in the list <i>lines</i> have to be relative
	 * to the common ancestor of their start and end node.
	 * 
	 * @param line the line that has to be routed orthogonal
	 * @param lines the lines for which overlaps, crossings should be avoided
	 */
	public void route(IRoutingLine line, List<IRoutingLine> lines) {

		List<Point> solution = null;
		
		long startTime;
		long structureTime;

		this.line = line;

		try {
			startTime = System.nanoTime();
			structure = new LineTileStructure(line, lines);
			structureTime = System.nanoTime() - startTime;
		} 
		catch (InvalidLayoutException e) {
			logger.warning("Error while trying to route " + line + ": Invalid layout");
			return;
		}

		parentNode = structure.getParent();

		start = line.getRoutingStartPoint();
		end = line.getRoutingEndPoint();

		Rectangle relativeSourceBounds = hu.getRelativeBounds(line
				.getRoutingSource(), parentNode);
		Rectangle relativeTargetBounds = hu.getRelativeBounds(line
				.getRoutingTarget(), parentNode);

		// Translate the start and end point relative to the parent node
		// (i.e., relative to the tile structure).
		start.translate(relativeSourceBounds.x, relativeSourceBounds.y);
		end.translate(relativeTargetBounds.x, relativeTargetBounds.y);

		Tile startTile = structure.findTileAtPoint(start);
		Tile endTile = structure.findTileAtPoint(end);
		
		try
		{
		
			// Special cost factor in the start and end tile for reflexive lines.
			if (line.getRoutingSource() == line.getRoutingTarget()) {
	
				startTile.setCostFactor(10.0f);
				endTile.setCostFactor(10.0f);
	
			}
			// Set the ABSTRACT_NODE_COSTFACTOR for both the start and end  tile
			// if a connection is (calculated) abstract.
			else if(!line.getRoutingSource().isZoomed() || !line.getRoutingTarget().isZoomed()) {
				startTile.setCostFactor(RoutingConstants.ROUTING_ABSTRACT_NODE_COSTFACTOR);
				endTile.setCostFactor(RoutingConstants.ROUTING_ABSTRACT_NODE_COSTFACTOR);
			}
			else {
	
				int delta = 10;
				
				if(start.x < relativeSourceBounds.x + delta || start.x > relativeSourceBounds.getMaxX() - delta
						|| start.y < relativeSourceBounds.y + 10 || start.y > relativeSourceBounds.getMaxY() - delta) {
					
					startTile.setCostFactor(RoutingConstants.ROUTING_NODE_COSTFACTOR);
					
				} else {
					startTile.setCostFactor(1.0f);
				}
				
				if(end.x < relativeTargetBounds.x + delta || end.x > relativeTargetBounds.getMaxX() - delta
						|| end.y < relativeTargetBounds.y + 10 || end.y > relativeTargetBounds.getMaxY() - delta) {
					
					endTile.setCostFactor(RoutingConstants.ROUTING_NODE_COSTFACTOR);
					
				} else {
					endTile.setCostFactor(1.0f);
				}
				
			}
	
			// Propagate the wave, i.e. calculate the costs for the tiles
			startTime = System.nanoTime();
			Wave wave = new Wave(structure);
			wave.propagate(start, end);
			long waveTime = System.nanoTime() - startTime;
	
			// Construct the solutions by retracing from the end point to the
			// start point. The solutions consist of a path consisting of tiles.
			startTime = System.nanoTime();
			List<List<Tile>> solutions = wave.retracePaths();
			long retraceTime = System.nanoTime() - startTime;
			
			if(RoutingConstants.SHOW_ROUTING_SOLUTIONS) {
				
				System.out.println();
				
				System.out.println("*** Routing line from '" + ((NodeTile)startTile).getNode() + 
						           "' to '" + ((NodeTile)endTile).getNode() + "' ***");
				
				countTiles();
				
				System.out.println("Number of cost calculations: " + wave.numberOfCostCalculations);
				
				System.out.println();
				
				System.out.println("Time to construct structure: " + (structureTime / 1000000.0) + " ms");
				System.out.println("Time for expansion: " + (waveTime / 1000000.0) + " ms");
				System.out.println("Time for retracing: " + (retraceTime / 1000000.0) + " ms");
				
			}
	
			// The algorithm has found at least one solution!
			if (solutions.size() > 0) {
	
				// Find the optimal solution out of the distance equivalent
				// solutions. This includes the construction of the actual line
				// i.e. the bend points.
				solution = evaluateSolutions(solutions.iterator());
	
				line.setRoutingBendpoints(solution);
				
				// Arrange the line labels
				if(RoutingConstants.PLACE_LABELS == true) {
					
					try {
						// TODO: Routing has to be done on the label structure as
						// soon as it's stable enough.
						structure = new LabelTileStructure(line, lines);
					} 
					catch (InvalidLayoutException e) {
						logger.warning("Error while trying to place labels for " + line + ": Invalid layout");
						return;
					}
					
					OrthogonalLabelPlacement.getInstance().arrangeLabels(line, structure);
				}
	
			}
			// There exists no path between the start and end point!
			else {
				logger.warning("No path found!");
			}
		}
		catch (NullPointerException e)
		{
			// do nothing
			// sometimes the model is in an unclean state
			// eg during the zoom, in this case the routing leads to
			// errors which should be absorbed, because the routing
			// is immediately repeated as soon as the state is fixed.
		}
	}
	
	
	/**
	 * Selects one path from the solutions given as argument and returns a
	 * Candidate containing all the line vertices that have to be inserted into
	 * the line to route it accordingly.
	 * 
	 * @param solutions
	 *            all the solutions
	 * @return Candidate representing the selected (e.g. best) solution
	 * 
	 * @pre solutions != null
	 * @post true
	 * 
	 */
	private List<Point> evaluateSolutions(Iterator<List<Tile>> solutions) {

		List<Point> best = null;
		double lowestCost = Double.MAX_VALUE;
		
		Rectangle startNodeBounds = structure.findTileAtPoint(start).getBounds();
		Rectangle endNodeBounds = structure.findTileAtPoint(end).getBounds();
		
		List<InfluenceFactor> factors = new ArrayList<InfluenceFactor>();
		
		// Number of bend points
		factors.add(new VertexFactor());
		
		// Total length of the line
		// Disabled because the moving of the start and end node in the new
		// OTMR may distort the result.
		//factors.add(new LengthFactor());
		
		// Uniformity of the line segment lengths
		factors.add(new UniformityFactor());
		
		// Distance the line lies inside the start and end node
		// This has only an influence if the ROUTING_NODE_COSTFACTOR == 1.0
		factors.add(new NodeCrossingFactor(startNodeBounds, endNodeBounds));
		
		// The following factor punishes solutions for which the line router
		// has moved the start or end point too close to the boundary of the
		// start or end node.
		factors.add(new BoundaryFactor(startNodeBounds, endNodeBounds));
		
		
		// Show the exact influence of the factors on the console
		if(RoutingConstants.SHOW_ROUTING_SOLUTIONS) {
			
			System.out.println();
			
			String title = "| Solution    ";
			for(InfluenceFactor factor: factors) {
				title += String.format("| %-15s", factor.getName());
			}
			title += "| Total          ";
			title += "| Tiles";
			
			System.out.println(title);
		}
		
		
		List<double[]> values = new ArrayList<double[]>();
		int index = 0;

		while(solutions.hasNext()) {

			List<Tile> currentPath = solutions.next();
			index++;

			// Get the bend points for the line
			OrthogonalBendpointCalculator obc = new OrthogonalBendpointCalculator(start, end, currentPath);
			List<Point> cand = obc.getBendpoints();
			centerHorizontally(cand);
			avoidBorderOverlappings(cand);
			
			// Calculate the score for each factor
			double[] results = new double[factors.size() + 1];
			for(int i = 0; i < factors.size(); i++) {
				
				InfluenceFactor factor = factors.get(i);
				
				List<Point> copy = new ArrayList<Point>(cand.size());
				copy.addAll(cand);
				
				results[i] = factor.getValue(cand);
				
			}
			
			// Calculate the total cost for each solution
			double overallCost = 0;
			for(int i = 0; i < results.length; i++) {
				overallCost += results[i];
			}
			
			results[results.length - 1] = overallCost;
			values.add(results);

			// Select the cheapest path
			if (overallCost < lowestCost) {
				best = cand;
				lowestCost = overallCost;
			}
			
			// Show the exact influence of the factors on the console
			if(RoutingConstants.SHOW_ROUTING_SOLUTIONS) {
				
				String s = String.format("| Solution %2d ", index);
					
				for(int j = 0; j < results.length; j++) {
					s += String.format("| %14.2f ", results[j]);
				}
					
				s += "|";
				
				// Print all the tiles contained in the current solution
				Iterator<Tile> tiles = currentPath.iterator();
				while(tiles.hasNext()) {
					s += " " + tiles.next().getName();
				}
				
				System.out.println(s);
				
			}

		}

		return best;

	}

	/**
	 * This method centers all the horizontal segments of the line given as
	 * argument. The vertical segments are directly centered by the
	 * retracePath() method.
	 * 
	 * @param candidate
	 *            the line to center the horizontal segments
	 * 
	 * @pre candidate != null
	 * @post true
	 * 
	 */
	private void centerHorizontally(List<Point> candidate) {

		for (int i = 2; i < candidate.size() - 1; i++) {
			Point v1 = candidate.get(i - 1);
			Point v2 = candidate.get(i);

			// The current segment is a horizontal one
			if (v1.y == v2.y && structure.findTileAtPoint(v1).isSpace()
					&& structure.findTileAtPoint(v2).isSpace()) {
				int topY, bottomY;

				Rectangle rect = getMaxBounds(v1, v2);
				topY = rect.y;
				bottomY = (int)rect.getMaxY();

				int y = topY + ((bottomY - topY) / 2);
				;
				if (bottomY - topY > RoutingConstants.MAX_SIZE_TO_CENTER) {
					if (candidate.get(i - 2).y > y
							&& candidate.get(i + 1).y > y) {
						y = bottomY - RoutingConstants.DIST_TO_NODE;
					} else if (candidate.get(i - 2).y < y
							&& candidate.get(i + 1).y < y) {
						y = topY + RoutingConstants.DIST_TO_NODE;
					}
				}

				candidate.set(i - 1, new Point(v1.x, y));
				candidate.set(i, new Point(v2.x, y));
			}
		}
	}
	
	
	private void avoidBorderOverlappings(List<Point> candidate) {
		
		for(int i = 2; i < candidate.size() - 1; i++) {
			
			Point v1 = candidate.get(i - 1);
			Point v2 = candidate.get(i);
			
			// Horizontal segment
			if(v1.y == v2.y) {
				
				// Source node
				IRoutingNode nodeParent = line.getRoutingSource().getRoutingParent();
				while(nodeParent != structure.getParent()) {
					
					Rectangle rect = nodeParent.getRoutingBounds();
					
					if(v1.y >= rect.y - RoutingConstants.DIST_TO_PARENT_BORDER
							&& v1.y <= rect.y + RoutingConstants.DIST_TO_PARENT_BORDER) {
						
						v1.translate(0, -RoutingConstants.DIST_TO_PARENT_BORDER);
						v2.translate(0, -RoutingConstants.DIST_TO_PARENT_BORDER);
						
						candidate.set(i - 1, v1);
						candidate.set(i, v2);
					}
					else if(v1.y >= rect.getMaxY() - RoutingConstants.DIST_TO_PARENT_BORDER
							&& v1.y <= rect.getMaxY() + RoutingConstants.DIST_TO_PARENT_BORDER) {
						
						v1.translate(0, RoutingConstants.DIST_TO_PARENT_BORDER);
						v2.translate(0, RoutingConstants.DIST_TO_PARENT_BORDER);
						
						candidate.set(i - 1, v1);
						candidate.set(i, v2);
					}
					
					nodeParent = nodeParent.getRoutingParent();
				}

				// Target node
				nodeParent = line.getRoutingTarget().getRoutingParent();
				while(nodeParent != structure.getParent()) {
					
					Rectangle rect = nodeParent.getRoutingBounds();
					
					if(v1.y >= rect.y - RoutingConstants.DIST_TO_PARENT_BORDER
							&& v1.y <= rect.y + RoutingConstants.DIST_TO_PARENT_BORDER) {
						v1.translate(0, -RoutingConstants.DIST_TO_PARENT_BORDER);
						v2.translate(0, -RoutingConstants.DIST_TO_PARENT_BORDER);
						
						candidate.set(i - 1, v1);
						candidate.set(i, v2);
					}
					else if(v1.y >= rect.getMaxY() - RoutingConstants.DIST_TO_PARENT_BORDER
							&& v1.y <= rect.getMaxY() + RoutingConstants.DIST_TO_PARENT_BORDER) {
						v1.translate(0, RoutingConstants.DIST_TO_PARENT_BORDER);
						v2.translate(0, RoutingConstants.DIST_TO_PARENT_BORDER);
						
						candidate.set(i - 1, v1);
						candidate.set(i, v2);
					}
					
					nodeParent = nodeParent.getRoutingParent();
				}
				
			}
			
			// Vertical Segment
			if(v1.x == v2.x) {
				
				// Source node
				IRoutingNode nodeParent = line.getRoutingSource().getRoutingParent();
				while(nodeParent != structure.getParent()) {
					
					Rectangle rect = nodeParent.getRoutingBounds();
					
					if(v1.x >= rect.x - RoutingConstants.DIST_TO_PARENT_BORDER
							&& v1.x <= rect.x + RoutingConstants.DIST_TO_PARENT_BORDER) {
						v1.translate(-RoutingConstants.DIST_TO_PARENT_BORDER, 0);
						v2.translate(-RoutingConstants.DIST_TO_PARENT_BORDER, 0);
						
						candidate.set(i - 1, v1);
						candidate.set(i, v2);
					}
					else if(v1.x >= rect.getMaxX() - RoutingConstants.DIST_TO_PARENT_BORDER
							&& v1.x <= rect.getMaxX() + RoutingConstants.DIST_TO_PARENT_BORDER) {
						v1.translate(RoutingConstants.DIST_TO_PARENT_BORDER, 0);
						v2.translate(RoutingConstants.DIST_TO_PARENT_BORDER, 0);
						
						candidate.set(i - 1, v1);
						candidate.set(i, v2);
					}
					
					nodeParent = nodeParent.getRoutingParent();
				}
				
				// Target node
				nodeParent = line.getRoutingTarget().getRoutingParent();
				while(nodeParent != structure.getParent()) {
					
					Rectangle rect = nodeParent.getRoutingBounds();
					
					if(v1.x >= rect.x - RoutingConstants.DIST_TO_PARENT_BORDER
							&& v1.x <= rect.x + RoutingConstants.DIST_TO_PARENT_BORDER) {
						v1.translate(-RoutingConstants.DIST_TO_PARENT_BORDER, 0);
						v2.translate(-RoutingConstants.DIST_TO_PARENT_BORDER, 0);
						
						candidate.set(i - 1, v1);
						candidate.set(i, v2);
					}
					else if(v1.x >= rect.getMaxX() - RoutingConstants.DIST_TO_PARENT_BORDER
							&& v1.x <= rect.getMaxX() + RoutingConstants.DIST_TO_PARENT_BORDER) {
						v1.translate(RoutingConstants.DIST_TO_PARENT_BORDER, 0);
						v2.translate(RoutingConstants.DIST_TO_PARENT_BORDER, 0);
						
						candidate.set(i - 1, v1);
						candidate.set(i, v2);
					}
					
					nodeParent = nodeParent.getRoutingParent();
				}
				
			}
			
		}
		
	}
	

	/**
	 * Calculates the place around a line segment that is not occupied by a node
	 * (e.g. the free space around the horizontal or vertical line segment).
	 * This method is reflexive, e.g. it doesn't matter witch of the two points
	 * is the start and end point of the segment.
	 * 
	 * @param vertex1
	 *            the first point of the segment
	 * @param vertex2
	 *            the second point of the segment
	 * @return the rectangle that represents the free space
	 * 
	 * @pre vertex1 != null && vertex2 != null
	 * @post true
	 * 
	 */
	private Rectangle getMaxBounds(Point vertex1, Point vertex2) {
		int leftX, rightX;
		int topY, bottomY;

		Point leftVertex, rightVertex;

		/*
		 * Case 2: Horizontal segment
		 */
		if (vertex1.y == vertex2.y) {
			if (vertex1.x < vertex2.x) {
				leftVertex = vertex1;
				rightVertex = vertex2;
			} else {
				leftVertex = vertex2;
				rightVertex = vertex1;
			}

			Tile startTile = structure.findTileAtPoint(leftVertex);
			Tile endTile = structure.findTileAtPoint(rightVertex);

			if (startTile == endTile) {
				return horizontalPart(leftVertex, rightVertex);
			}

			int top = 0;
			int bottom = structure.getHeight();
			int left = leftVertex.x;
			int right = rightVertex.x;

			Tile currentTile = startTile;
			while (true) {
				rightVertex = new Point(currentTile.getBottomRight().x,
						leftVertex.y);

				if (currentTile == endTile) {
					rightVertex = new Point(right, leftVertex.y);
				}

				if (!currentTile.isLineTile()) {
					Rectangle rect = horizontalPart(leftVertex, rightVertex);
					if (rect.y > top) {
						top = rect.y;
					}
					if (rect.getMaxY() < bottom) {
						bottom = (int)rect.getMaxY();
					}
				}

				if (currentTile == endTile) {
					return new Rectangle(left, top, right - left, bottom - top);
				}
				rightVertex.translate(1, 0);
				currentTile = structure.findTileAtPoint(rightVertex);
				rightVertex.translate(1, 0);
				leftVertex = rightVertex;
			}

		}

		/*
		 * Case 2: Vertical segment
		 */
		else if (vertex1.x == vertex2.x) {
			Tile startTile, endTile;

			if (vertex1.y < vertex2.y) {
				topY = vertex1.y;
				bottomY = vertex2.y;

				startTile = structure.findTileAtPoint(vertex1);
				endTile = structure.findTileAtPoint(vertex2);
			} else {
				topY = vertex2.y;
				bottomY = vertex1.y;

				startTile = structure.findTileAtPoint(vertex2);
				endTile = structure.findTileAtPoint(vertex1);
			}

			leftX = startTile.getTopLeft().x;
			rightX = startTile.getBottomRight().x;

			// Because of the organization of the corner stitching structure, we
			// don't have
			// to check the right and left neighbors.

			Tile currentTile = startTile;
			while (currentTile != endTile) {
				Tile neighbour = currentTile.getLb();
				while (neighbour != null) {
					if (vertex1.x >= neighbour.getTopLeft().x
							&& vertex1.x <= neighbour.getBottomRight().x) {
						if (neighbour.getTopLeft().x > leftX) {
							leftX = neighbour.getTopLeft().x;
						}
						if (neighbour.getBottomRight().x < rightX) {
							rightX = neighbour.getBottomRight().x;
						}

						currentTile = neighbour;
						break;
					}
					neighbour = neighbour.getTr();
				}
			}

			return new Rectangle(leftX, topY, rightX - leftX, bottomY - topY);
		}

		// This means that the two points don't form a vertical or horizontal
		// segment
		return null;

	}

	/**
	 * Returns a rectangle that denotes the free the segment from p1 to p2 can
	 * be moved in.
	 * 
	 * @param p1
	 *            starting point of the segment
	 * @param p2
	 *            ending point of the segment
	 * @return a rectangle that denotes the free space
	 * 
	 * @pre p1 != null && p2 != null
	 * @post true
	 */
	private Rectangle horizontalPart(Point p1, Point p2) {
		int leftX = p1.x;
		int rightX = p2.x;
		int topY, bottomY;

		Tile startTile = structure.findTileAtPoint(p1);
		topY = startTile.getTopLeft().y;
		bottomY = startTile.getBottomRight().y;

		// Check the bottom neighbors
		Tile currentTile = startTile;
		while (currentTile != null) {
			Tile neighbor = currentTile.getLb();
			Tile candidate = null;
			while (neighbor != null) {
				if (neighbor.isSpace() && !neighbor.isLineTile()
				// && !(neighbor instanceof BorderTile)
						&& p1.x >= neighbor.getTopLeft().x
						&& p1.x <= neighbor.getBottomRight().x
						&& p2.x >= neighbor.getTopLeft().x
						&& p2.x <= neighbor.getBottomRight().x) {
					bottomY = neighbor.getBottomRight().y;
					candidate = neighbor;
				}
				if (neighbor == currentTile.getRb()) {
					break;
				}
				neighbor = neighbor.getTr();
			}
			currentTile = candidate;
		}

		// Check the top neighbors
		currentTile = startTile;
		while (currentTile != null) {
			Tile neighbor = currentTile.getLt();
			Tile candidate = null;
			while (neighbor != null) {
				if (neighbor.isSpace() && !neighbor.isLineTile()
						&& p1.x >= neighbor.getTopLeft().x
						&& p1.x <= neighbor.getBottomRight().x
						&& p2.x >= neighbor.getTopLeft().x
						&& p2.x <= neighbor.getBottomRight().x) {
					topY = neighbor.getTopLeft().y;
					candidate = neighbor;
				}
				if (neighbor == currentTile.getRt()) {
					break;
				}
				neighbor = neighbor.getBr();
			}
			currentTile = candidate;
		}

		return new Rectangle(leftX, topY, rightX - leftX, bottomY - topY);
	}
	
	
	private void countTiles() {
		
		System.out.println();
		
		int nodeTiles = 0;
		int spaceTiles = 0;
		int lineTiles = 0;
		int associatedCostValue = 0;
		
		Iterator<Tile> tiles = structure.getAllTiles();
		while(tiles.hasNext()) {
			
			Tile tile = tiles.next();
			if(tile instanceof NodeTile) {
				nodeTiles++;
			}
			else if(tile instanceof LineTile) {
				lineTiles++;
			}
			else {
				spaceTiles++;
			}
			
			if(tile.getCost() > 0) {
				associatedCostValue++;
			}
			
		}
		
		System.out.println("Number of node tiles: " + nodeTiles);
		System.out.println("Number of line tiles: " + lineTiles);
		System.out.println("Number of space tiles: " + spaceTiles);
		System.out.println("Number of tiles: " + (nodeTiles + lineTiles + spaceTiles));
		
		System.out.println();
		System.out.println("Number of tiles with associated cost value: " + associatedCostValue);
		
	}

	
	/**
	 * Calculates the Euclidean distance between the points p1 and p2. This
	 * method is transitive (i.e. distance(p1, p2) == distance(p2, p1)).
	 * 
	 * @param p1
	 *            the first point
	 * @param p2
	 *            the second point
	 * @return the Euclidean distance between p1 and p2
	 * 
	 * @pre p1 != null && p2 != null
	 * @post true
	 */
	public static double distance(Point p1, Point p2) {
		long dx = (p2.x - p1.x);
		long dy = (p2.y - p1.y);
		return Math.sqrt(dx * dx + dy * dy);
	}
	
	
	/**
	 * Returns the tile structure that was most recently used. This method can
	 * be used to visualize the tile structure.
	 * 
	 * @return the most recently used tile structure.
	 */
	public TileStructure getCurrentTileStructure() {
		
		return structure;
		
	}

}
