package ch.unizh.ifi.adora.linerouting.algorithm;

import java.util.LinkedList;
import java.util.List;

import ch.unizh.ifi.adora.linerouting.tilestructure.Tile;

/**
 * Represents a path consisting of space tiles that has been found by the
 * {@link routing.algorithm.TileMazeRouter}. The path is represented
 * as tree of paths.
 * 
 * @author              Tobias Reinhard (TR)
 * @copyright           Requirements Engineering Research Group
 *                      Department of Computer Science
 *                      University of Zurich
 * @history
 * 2005-03-07           TR First version
 * 2005-03-18           TR Methods getTiles() and contains(...) added
 * 
 * @version             $Date: 2008-08-11 17:33:06 +0200 (Mon, 11 Aug 2008) $, $Author: treinhard $, $Revision: 3790 $
 * @responsibilities    Represents a path found by the tile maze routing algorithm
 */
public class TilePath {

    /**
     * The tile the path ends at
     */
    private Tile tile;
    
    /**
     * Reference to the next path that brings us  one tile closer to the target.
     * The nextPath is null for the last path, i.e. the path that contains the
     * end tile.
     */
    private TilePath nextPath;
    
    
    /**
     * Constructor for a path structure. The parameter nextPath can be null
     * if the path doesn't have a next path (i.e. the path has the tile the line
     * ends in as end tile).
     * 
     * @pre tile != null
     * @post getTile() == tile
     * 
     * @param tile the tile that path ends in
     * @param nextPath the next path that brings us closer to the target
     */
    public TilePath(Tile tile, TilePath nextPath) {
    	    this.tile = tile;
    	    this.nextPath = nextPath;
    }
    
    
    /**
     * Returns the tile this path ends in.
     * 
     * @pre true
     * @post true
     * 
     * @return the tile the paths ends in.
     */
    public Tile getTile() {
    	    return tile;
    }
    
    
    /**
     * Returns the parent path of this path.
     * 
     * @pre true
     * @post true
     * 
     * @return the parent path of this path
     */
    public TilePath getNextPath() {
    	    return nextPath;
    }
    
    
    /**
     * Returns all the tiles in this path. E.g. all the tiles a path from the start point
     * of the line has to go through to get to the end tile of this path.
     * 
     * @pre true
     * @post true
     * 
     * @return all the tiles in this path
     */
    public List<Tile> getTiles() {
        LinkedList<Tile> tiles = new LinkedList<Tile>();
        TilePath currentPath = this;
        while(currentPath != null)
        {
            tiles.add(currentPath.getTile());
            currentPath = currentPath.getNextPath();
        }

        return tiles;
    }
    
    
    /**
     * Returns a string representation of this path. The string contains
     * all the names of the tiles that are contained in the path (i.e. in
     * this path or one of it's parent paths).
     * 
     * @pre true
     * @post true
     * 
     * @return a string representation o this path
     */
    public String toString() {
    	    String s = "TilePath:";
    	    TilePath p = this;
        while(p != null) {
        	    s += " " + p.getTile().getName();
        	    p = p.getNextPath();
        }
        
        return s;
    }
    
}
