package ch.unizh.ifi.adora.linerouting.algorithm;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.tilestructure.Tile;
import ch.unizh.ifi.adora.linerouting.tilestructure.TileStructure;
import ch.unizh.ifi.adora.linerouting.utils.SortedList;

public class Wave {

    /**
     * The corner stitching data structure.
     */
    protected TileStructure structure;

    /**
     * The line that has to be routed
     */
    protected IRoutingLine line;

    /**
     * Heap data structure to hold the tiles
     */
    private SortedList<Tile> tileHeap;

    /**
     * The end point of the line (inside the end node)
     */
    private Point endPoint;

    /**
     * The start tile (the tile that is occupied by the start node)
     */
    private Tile startTile;

    /**
     * The end tile (the tile that is occupied by the end node)
     */
    private Tile endTile;
    
    /**
     * Stores the numbers of tiles that have been visited during the expansion
     */
    public int numberOfCostCalculations = 0;
    
    /**
     * Constructor.
     * 
     * @param structure the tile structure the wave should expand on
     */
    public Wave(TileStructure structure) {
    	
    	this.structure = structure;
    	
    }
    
    
	
    /**
     * This method performs the first step of Lee's algorithm: It calculates the
     * costs (i.e. distance values) for the space tiles starting from the start
     * point. The coordinates of the start and end point have to be relative
     * to the tile structure.
     * 
     * @param start the start point of the line (inside the start node)
     * @param end the end point of the line (inside the end node)
     */
    public void propagate(Point start, Point end) {

        endPoint = end;

        startTile = structure.findTileAtPoint(start);
        endTile = structure.findTileAtPoint(end);

        startTile.setCostPoint(start);
        startTile.setSourceCost(0);
        startTile.setDestinationCost(calcManhattanDist(start, end));

        tileHeap = new SortedList<Tile>();
        tileHeap.add(startTile);

        while (tileHeap.size() > 0) {
        	
            Tile currentTile = (Tile) tileHeap.first();
            tileHeap.remove(currentTile);

            if (currentTile == endTile) {
                return;
            }
            
            processTile(currentTile);

        }

    }
    
    
    /**
     * The method performs the second step of Lee's algorithm by retracing a
     * path through the space tiles accordingly to their costs. It returns a
     * list containing all the optimal (i.e. shortest) paths. The returned
     * list is empty if no solution exists
     * 
     * @return list containing all the optimal (i.e. shortest) solutions
     */
    public List<List<Tile>> retracePaths() {

        Stack<TilePath> paths = new Stack<TilePath>();
        ArrayList<TilePath> solutions = new ArrayList<TilePath>(3);

        TilePath path = new TilePath(endTile, null);

        paths.push(path);
        while (!paths.isEmpty()) {

            TilePath currentPath = paths.pop();

            if (currentPath.getTile() == startTile) {
                solutions.add(currentPath);
            } else {
                
                Tile[] tiles = currentPath.getTile().getOrigins();
                for (int i = 0; i < tiles.length; i++) {                 
                       
                    paths.push(new TilePath(tiles[i], currentPath));

                }

            }
        }

        List<List<Tile>> foundPaths = new ArrayList<List<Tile>>(solutions.size());
        
        for(TilePath solution: solutions) {
        	
        	foundPaths.add(solution.getTiles());
        	
        }
        
        return foundPaths;
    }
    
    
    /**
     * Calculates the costs for all the space tiles that are neighbors to the
     * tile given as argument.
     * 
     * @pre current != null
     * @post true
     * 
     * @param current
     *            the current tile
     */
    private void processTile(Tile current) {

        Iterator<Tile> neighborIterator = current.getNeighbors();

        while (neighborIterator.hasNext()) {

            Tile neighbor = (Tile) neighborIterator.next();

            if (neighbor.isSpace() || neighbor == endTile) {
                Point currentCostPoint = current.getCostPoint();
                Point nextCostPoint = findCostPoint(current, neighbor);

                ++numberOfCostCalculations;
                
                int sourceDist = current.getSourceCost();
                sourceDist += Math.round(calcManhattanDist(currentCostPoint, nextCostPoint)
                        * current.getCostFactor());
                int destinationDist = calcManhattanDist(nextCostPoint, endPoint);
                
                if(neighbor == endTile) {
                    
                    endTile.setCostPoint(endPoint);
                    Point p = findCostPoint(endTile, current);
                    sourceDist = current.getSourceCost();
                    
                    sourceDist += (int)(calcManhattanDist(currentCostPoint, p)
                                  * current.getCostFactor());
                    
                    sourceDist += (int)(calcManhattanDist(endPoint, p)
                                  * neighbor.getCostFactor());
                    
                    destinationDist = 0;
                    
                }

                if (!neighbor.visited()
                        || (sourceDist + destinationDist) < neighbor.getCost()) {

                    neighbor.setCostPoint(nextCostPoint);
                    neighbor.setSourceCost(sourceDist);
                    neighbor.setDestinationCost(destinationDist);

                    neighbor.resetOrigins();

                    if(tileHeap.contains(neighbor)) {
                        tileHeap.remove(neighbor);
                    }
                    
                    tileHeap.add(neighbor);
                }

                if ((sourceDist + destinationDist) == neighbor.getCost()
                    && !neighbor.hasOrigin(current)) {
                    neighbor.addOrigin(current);
                }
            }

        }
    }

	
    /**
     * This method returns the cost point in the tile given as parameter
     * neighbor. The cost point is that point inside the tile neighbor for
     * which the Euclidean distance to the cost point in the tile given as
     * parameter current is minimal.
     * 
     * @pre current != null && neighbor != null && current.getCostPoint() != null
     * @post true
     * 
     * @param current
     *            tile that contains the reference cost point
     * @param neighbour
     *            tile to calculate the cost point for
     * @return the cost point inside the tile neighbor
     */
    protected Point findCostPoint(Tile current, Tile neighbour) {

        Point currentCostPoint = current.getCostPoint();
        int x, y;

        int direction = directionToNeighbour(current, neighbour);

        // Top or bottom neighbor
        if (direction == RoutingConstants.NORTH || direction == RoutingConstants.SOUTH) {

            if (direction == RoutingConstants.NORTH) {
                y = neighbour.getBottomRight().y;
            } else {
                y = neighbour.getTopLeft().y;
            }

            // The current cost point lies inside the vertical range of the
            // neighbor tile
            if (currentCostPoint.x >= neighbour.getTopLeft().x
                    && currentCostPoint.x <= neighbour.getBottomRight()
                            .x) {

                x = currentCostPoint.x;

            }
            // The horizontal position of the next cost point has to be
            // adjusted
            else {

                // The top left corner of the neighbor tile is closer to
                // the current cost point
                if (Math.abs(currentCostPoint.x
                        - neighbour.getTopLeft().x) < Math
                        .abs(currentCostPoint.x
                                - neighbour.getBottomRight().x)) {
                    x = neighbour.getTopLeft().x;
                }
                // The bottom right corner of the neighbor tile is closer
                // to the current cost point
                else {
                    x = neighbour.getBottomRight().x;
                }
            }

        }
        // left or right neighbor
        else {

            if (direction == RoutingConstants.WEST) {
                x = neighbour.getBottomRight().x;
            } else {
                x = neighbour.getTopLeft().x;
            }

            // The current cost point lies inside the vertical range of the
            // neighbor tile
            if (currentCostPoint.y >= neighbour.getTopLeft().y
                    && currentCostPoint.y <= neighbour.getBottomRight()
                            .y) {

                y = currentCostPoint.y;

            }
            // The horizontal position of the next cost point has to be
            // adjusted
            else {

                // The top left corner of the neighbor tile is closer to
                // the current cost point
                if (Math.abs(currentCostPoint.y
                        - neighbour.getTopLeft().y) < Math
                        .abs(currentCostPoint.y
                                - neighbour.getBottomRight().y)) {
                    y = neighbour.getTopLeft().y;
                }
                // The bottom right corner of the neighbor tile is closer
                // to the current cost point
                else {
                    y = neighbour.getBottomRight().y;
                }
            }

        }

        return new Point(x, y);
    }

    
    /**
     * Returns one of the constants from {@link layout.data.GeometricConstants}
     * to indicate the direction from the tile current the tile neighbor lies
     * in. The tile neighbor has to be a neighbor of the tile current to,
     * otherwise this method doesn't work correctly.
     * 
     * @pre current != null && neighbor != null
     * @post true
     * 
     * @param current
     *            the reference tile
     * @param neighbour
     *            the neighboring tile
     * @return the direction from current to neighbor
     */
    private int directionToNeighbour(Tile current, Tile neighbour) {
        
        Iterator<Tile> top = current.getTopNeighbors();
        while(top.hasNext()) {
            if(top.next() == neighbour) {
                return RoutingConstants.NORTH;
            }                
        }
        
        Iterator<Tile> bottom = current.getBottomNeighbors();
        while(bottom.hasNext()) {
            if(bottom.next() == neighbour) {
                return RoutingConstants.SOUTH;
            }                
        }
        
        Iterator<Tile> left = current.getLeftNeighbors();
        while(left.hasNext()) {
            if(left.next() == neighbour) {
                return RoutingConstants.WEST;
            }                
        }
        
        Iterator<Tile> right = current.getRightNeighbors();
        while(right.hasNext()) {
            if(right.next() == neighbour) {
                return RoutingConstants.EAST;
            }                
        }
        
        
        return -1;

    }

    
    /**
     * Calculates the Manhattan distance between the two points given as
     * parameters. This method is reflexive (i.e. the order of the points
     * doesn't matter).
     * 
     * @param p1
     *            first point
     * @param p2
     *            second point
     * @return the Manhattan distance between the two points
     */
    protected int calcManhattanDist(Point p1, Point p2) {

        int x = Math.abs(p1.x - p2.x);
        int y = Math.abs(p1.y - p2.y);

        return x + y;

    }

    
	
}
