package ch.unizh.ifi.adora.linerouting.factors;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.List;

/**
 * Criteria that adds a cost penalty if the start or end point has been moved 
 * by the router and lies now too close to the boundary of the start or end node.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-04-08 TR First Version
 * @version				$Date: 2009-12-14 15:59:25 +0100 (Mon, 14 Dec 2009) $, $Author: treinhard $, $Revision: 4993 $
 * @responsibilities	Returns a cost penalty if the (moved) start or end point
 *                      lies too close to the boundary of the node.
 *
 */
public class BoundaryFactor extends InfluenceFactor {
	
	/**
	 * The start node of the routing (stays the same for all solutions)
	 */
	private Rectangle startNodeBounds;
	
	/**
	 * The end node of the routing (stays the same of all solutions)
	 */
	private Rectangle endNodeBounds;
	
	/**
	 * Size of the zone inside the boundaries for which the lines are punished
	 */
	private int punishementZoneSize = 10;

	
	/**
	 * Constructor. Sets the start and end node
	 * 
	 * @param startNodeBounds the start node of the routing
	 * @param endNodeBounds the end node of the routing
	 */
	public BoundaryFactor(Rectangle startNodeBounds, Rectangle endNodeBounds) {
		
		super("Boundary", 5.0);
		
		this.startNodeBounds = startNodeBounds;
		this.endNodeBounds = endNodeBounds;
		
	}
	
	
	/**
	 * Returns the cost penalty for the line solution given as argument.
	 * 
	 * @param candidate the routed line that should be checked
	 * @return the cost penalty (or 0.0 if there's none) 
	 */
	public double getValue(List<Point> candidate) {
		
		Point start = candidate.get(0);
		Point end = candidate.get(candidate.size() - 1);
		
		double cost = 0;

		// First segment is horizontal
		if(start.y == candidate.get(1).y) {
		
			if(Math.abs(start.y - startNodeBounds.y) < punishementZoneSize 
					|| Math.abs(startNodeBounds.getMaxY() - start.y) < punishementZoneSize) {
				cost += 20;
			}

		// Last segment is vertical
		} else {
			
			if(Math.abs(start.x - startNodeBounds.x) < punishementZoneSize 
					|| Math.abs(startNodeBounds.getMaxX() - start.x) < punishementZoneSize) {
				cost += 20;
			}
			
		}
		
		// Last segment is horizontal
		if(end.y == candidate.get(candidate.size() - 2).y) {
			
			if(Math.abs(end.y - endNodeBounds.y) < punishementZoneSize 
					|| Math.abs(endNodeBounds.getMaxY() - end.y) < punishementZoneSize) {
				cost += 20;
			}
			
		// Last segment is vertical
		} else {

			if(Math.abs(end.x - endNodeBounds.x) < punishementZoneSize 
					|| Math.abs(endNodeBounds.getMaxX() - end.x) < punishementZoneSize) {
				cost += 20;
			}
			
		}
		
		return cost * factor;
	}

}
