package ch.unizh.ifi.adora.linerouting.factors;

import java.awt.Point;
import java.util.List;

/**
 * Abstract base class for the criteria that are used to select one of possible
 * many line routings as the final solution.
 * 
 * All factors (criteria) that are registered in the OrthgonalTileMazeRouter
 * are evaluated in turn by calling the getValue(...) method. The total cost
 * that is calculated for each routing is then used to select the best one.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-01-11 TR First Version
 * @version				$Date: 2008-08-19 15:49:22 +0200 (Tue, 19 Aug 2008) $, $Author: treinhard $, $Revision: 3813 $
 * @responsibilities	Abstract base class for all selecting criteria.
 *
 */
public abstract class InfluenceFactor {

	/**
	 * The name of the factor (used for debugging)
	 */
	private String name;
	
	/**
	 * The factor the value is multiplied by before it is returned
	 */
	protected double factor;
	
	
	/**
	 * Constructor. Sets the name and the factor
	 * 
	 * @param name the name of the factor
	 * @param factor the multiplying factor
	 */
	public InfluenceFactor(String name, double factor) {
		
		this.name = name;
		this.factor = factor;
		
	}
	
	
	/**
	 * Returns the name of this factor.
	 * 
	 * @return the name of the factor
	 */
	public String getName() {
		
		return name;
		
	}
	
	
	/**
	 * Method that returns the calculated value for the routing given as argument
	 * <i>candidate</i>. A lower value means a better solution (it's a cost and
	 * not a score approach).
	 * 
	 * @param candidate the line routing to calculate the value for
	 * @return the value
	 */
	public abstract double getValue(List<Point> candidate);
	
}
