package ch.unizh.ifi.adora.linerouting.factors;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.List;

/**
 * Criteria that evaluates how far a line is routed inside the start- and
 * endnode.
 * 
 * This criterion makes only sense for routings where the 
 * RoutingConstants.ROUTING_NODE_COSTFACTOR == 1.0
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-01-11 TR First Version
 * @version				$Date: 2008-12-02 18:08:40 +0100 (Tue, 02 Dec 2008) $, $Author: treinhard $, $Revision: 3933 $
 * @responsibilities	Evaluates how far the line is routed in the start- and endnode
 *
 */
public class NodeCrossingFactor extends InfluenceFactor {

	/**
	 * The start node of the routing (stays the same for all solutions)
	 */
	private Rectangle startNodeBounds;
	
	/**
	 * The end node of the routing (stays the same of all solutions)
	 */
	private Rectangle endNodeBounds;
	
	
	/**
	 * Constructor. Sets the start and end node
	 * 
	 * @param startNodeBounds the start node of the routing
	 * @param endNodeBounds the end node of the routing
	 */
	public NodeCrossingFactor(Rectangle startNodeBounds, Rectangle endNodeBounds) {
		
		super("Node crossing", 1.0);
		
		this.startNodeBounds = startNodeBounds;
		this.endNodeBounds = endNodeBounds;
		
	}
	
	
	/**
	 * Method that returns the calculated value for the routing given as argument
	 * <i>candidate</i>. A lower value means a better solution.
	 * 
	 * @param candidate the line routing to calculate the value for
	 * @return the calculated distance multiplied by a constant factor
	 */
	public double getValue(List<Point> cand) {
		
		double cost = 0;
		
		Line2D.Double segment = new Line2D.Double(cand.get(0), cand.get(1));
		Point v1 = getLineRectangleIntersection(startNodeBounds, segment);
		Point v2 = cand.get(0);
		
		cost += factor * v1.distance(v2);
		
		segment = new Line2D.Double(cand.get(cand.size() - 1), cand.get(cand.size() - 2));
		v1 = getLineRectangleIntersection(endNodeBounds, segment);
		v2 = cand.get(cand.size() - 1);
		
		if(v1 == null || v2 == null) {
			return 0;
		}
		
		cost += factor * v1.distance(v2);

		return cost;
	}
	
	
	private Point getLineRectangleIntersection(Rectangle rect, Line2D.Double line) {
		
		Point intersection = null;
		
		Point2D.Double topLeft = new Point2D.Double(rect.x, rect.y);
		Point2D.Double topRight = new Point2D.Double(rect.x + rect.width, rect.y);
		Point2D.Double bottomLeft = new Point2D.Double(rect.x, rect.y + rect.height);
		Point2D.Double bottomRight = new Point2D.Double(rect.x + rect.width, rect.y + rect.height);
		
		// Top side
		Line2D.Double side = new Line2D.Double(topLeft, topRight);
		intersection = getLineLineIntersection(side, line);
		
		// Left side
		if(intersection == null) {
			side = new Line2D.Double(topLeft, bottomLeft);
			intersection = getLineLineIntersection(side, line);
		}
		
		// Bottom side
		if(intersection == null) {
			side = new Line2D.Double(bottomLeft, bottomRight);
			intersection = getLineLineIntersection(side, line);
		}
		
		// Right side
		if(intersection == null) {
			side = new Line2D.Double(bottomRight, topRight);
			intersection = getLineLineIntersection(side, line);
		}
		
		return intersection;
		
	}
	
	
	private Point getLineLineIntersection(Line2D.Double l1, Line2D.Double l2) {
		
		if (!l1.intersectsLine(l2))
			return null;

		double x1 = l1.getX1(), y1 = l1.getY1(), x2 = l1.getX2(), y2 = l1
				.getY2(), x3 = l2.getX1(), y3 = l2.getY1(), x4 = l2.getX2(), y4 = l2
				.getY2();

		Point2D.Double intersection = new Point2D.Double();
		
		intersection.x = det(det(x1, y1, x2, y2), x1 - x2, det(x3, y3, x4, y4),
				x3 - x4) / det(x1 - x2, y1 - y2, x3 - x4, y3 - y4);
		intersection.y = det(det(x1, y1, x2, y2), y1 - y2, det(x3, y3, x4, y4),
				y3 - y4) / det(x1 - x2, y1 - y2, x3 - x4, y3 - y4);

		return new Point((int)Math.round(intersection.getX()), (int)Math.round(intersection.getY()));
	}

	
	private double det(double a, double b, double c, double d) {
		
		return a * d - b * c;
		
	}

}
