package ch.unizh.ifi.adora.linerouting.labels;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLineLabel;
import ch.unizh.ifi.adora.linerouting.tilestructure.Tile;
import ch.unizh.ifi.adora.linerouting.tilestructure.TileStructure;

public class OrthogonalLabelPlacement {
	
	public static final int MINIMAL_DISTANCE = 10;
	
	public static final int OPTIMAL_WIDTH = 200;
	
	/**
	 * Singleton instance
	 */
	private static OrthogonalLabelPlacement instance;
	
	/**
	 * The tile structure for the line we're currently calculating
	 * the labels for.
	 */
	private TileStructure structure;
	
	/**
	 * Stores rectangles representing the candidate bounds for the labels.
	 * Only used for debugging / demonstration.
	 */
	private List<Rectangle> labelPlaces = new ArrayList<Rectangle>();
	
	/**
	 * Stores the scores for all candidate label positions.
	 * Only used for debugging / demonstration.
	 */
	private List<Double> labelScores = new ArrayList<Double>();
	
	/**
	 * Hide the constructor as this is a Singleton
	 */
	private OrthogonalLabelPlacement() {
		super();
	}
	
	/**
	 * Method to get the singleton instance.
	 * 
	 * @return the singleton instance
	 */
	public static OrthogonalLabelPlacement getInstance() {
		
		if(instance == null) {
			instance = new OrthogonalLabelPlacement();
		}
		
		return instance;
		
	}

	
	public void arrangeLabels(IRoutingLine line, TileStructure structure) {
		
		this.structure = structure;
		labelPlaces = new ArrayList<Rectangle>();
		labelScores = new ArrayList<Double>();
		
		for(IRoutingLineLabel label: line.getLineLabels()) {
			
			Rectangle bestBounds = null;
			double bestScore = Double.NEGATIVE_INFINITY;
			
			Point segmentStart = line.getRoutingBendpoints().get(0);
			for(int i = 1; i < line.getRoutingBendpoints().size(); i++) {
				
				Point segmentEnd = line.getRoutingBendpoints().get(i);
				List<Rectangle> positions = new ArrayList<Rectangle>();
				
				// Horizontal segment
				if(segmentStart.y == segmentEnd.y) {
					positions = getPositionsAroundHorizontalSegment(segmentStart, segmentEnd, label);
				} 
				// Vertical segment
				else if(segmentStart.x == segmentEnd.x) {
					positions = getPositionsAroundVerticalSegment(segmentStart, segmentEnd, label);
				}
				
				for(Rectangle rect: positions) {

					double score = getScore(rect, segmentStart, segmentEnd, line);
					labelPlaces.add(rect);
					labelScores.add(new Double(score));
					if(score > bestScore) {
						bestScore = score;
						bestBounds = rect;
					}

				}
				
				segmentStart = segmentEnd;
				
			}
			
			if(bestBounds != null) {
				label.setRoutingBounds(bestBounds);
			}
			
		}
		
	}
	
	
	private List<Rectangle> getPositionsAroundVerticalSegment(Point segmentStart, Point segmentEnd, IRoutingLineLabel label) {
		
		Point midpoint = midpoint(segmentStart, segmentEnd);
		List<Rectangle> positions = new ArrayList<Rectangle>(3);
		
		// Calculate maximal free space on the right side of the segment.
		Tile t = structure.findTileAtPoint(midpoint);
		Rectangle space = new Rectangle(midpoint.x, t.getBounds().y, t.getBottomRight().x - midpoint.x, t.getBounds().height);
		while(t.getTopLeft().y - 1 >= 0) {
			t = structure.findTileAtPoint(new Point(midpoint.x, t.getTopLeft().y - 1));
			if(t.getBounds().x <= space.x && t.getBottomRight().x >= space.getMaxX()) {
				space.y = t.getTopLeft().y;
				space.height += t.getHeight();
			} else {
				break;
			}
		}
		t = structure.findTileAtPoint(midpoint);
		while(t.getBottomRight().y + 1 <= structure.getHeight()) {
			t = structure.findTileAtPoint(new Point(midpoint.x, t.getBottomRight().y + 1));
			if(t.getBounds().x <= space.x && t.getBottomRight().x >= space.getMaxX()) {
				space.height += t.getHeight();
			} else {
				break;
			}
		}
		
		// Restrict search space to the length of the vertical line segment.
		int top;
		int bottom;
		if(segmentStart.y < segmentEnd.y) {
			top = segmentStart.y;
			bottom = segmentEnd.y;
		} else {
			top = segmentEnd.y;
			bottom = segmentStart.y;
		}
		if(space.y > top) {
			top = space.y;
		}
		if(space.getMaxY() < bottom) {
			bottom = (int)space.getMaxY();
		}
		
		// Try to place the label on the right side of the line
		int width = (int)space.getMaxX() - midpoint.x - 2 * MINIMAL_DISTANCE;
		int height = label.getMinimalHeight(width);
		width = label.getWidth(width);
		Rectangle rect = new Rectangle();
		rect.x = midpoint.x + MINIMAL_DISTANCE;
		rect.y = top + (bottom - top - height) / 2;
		rect.width = width;
		rect.height = height;
		positions.add(rect);
		
		if(space.getMaxX() - midpoint.x > OPTIMAL_WIDTH) {
			
			width = OPTIMAL_WIDTH - 2 * MINIMAL_DISTANCE;
			height = label.getMinimalHeight(width);
			width = label.getWidth(width);
			rect = new Rectangle();
			rect.x = midpoint.x + MINIMAL_DISTANCE;
			rect.y = top + (bottom - top - height) / 2;
			rect.width = width;
			rect.height = height;
			positions.add(rect);
			
		}
		
		// Calculate maximal free space on the left side of the segment.
		t = structure.findTileAtPoint(midpoint);
		space = new Rectangle(t.getBounds().x, t.getBounds().y, midpoint.x - t.getBounds().x, t.getBounds().height);
		while(t.getTopLeft().y - 1 >= 0) {
			t = structure.findTileAtPoint(new Point(midpoint.x, t.getTopLeft().y - 1));
			if(t.getBounds().x <= space.x && t.getBottomRight().x >= space.getMaxX()) {
				space.y = t.getTopLeft().y;
				space.height += t.getHeight();
			} else {
				break;
			}
		}
		t = structure.findTileAtPoint(midpoint);
		while(t.getBottomRight().y + 1 <= structure.getHeight()) {
			t = structure.findTileAtPoint(new Point(midpoint.x, t.getBottomRight().y + 1));
			if(t.getBounds().x <= space.x && t.getBottomRight().x >= space.getMaxX()) {
				space.height += t.getHeight();
			} else {
				break;
			}
		}
		
		if(segmentStart.y < segmentEnd.y) {
			top = segmentStart.y;
			bottom = segmentEnd.y;
		} else {
			top = segmentEnd.y;
			bottom = segmentStart.y;
		}
		if(space.y > top) {
			top = space.y;
		}
		if(space.getMaxY() < bottom) {
			bottom = (int)space.getMaxY();
		}
		
		// Try to place the label on the left side of the line
		width = space.width - 2 * MINIMAL_DISTANCE;
		height = label.getMinimalHeight(width);
		width = label.getWidth(width);
		rect = new Rectangle();
		rect.x = midpoint.x - width - MINIMAL_DISTANCE;
		rect.y = top + (bottom - top - height) / 2;
		rect.width = width;
		rect.height = height;
		positions.add(rect);
		
		if(space.width > OPTIMAL_WIDTH) {
			
			width = OPTIMAL_WIDTH - 2 * MINIMAL_DISTANCE;
			height = label.getMinimalHeight(width);
			width = label.getWidth(width);
			rect = new Rectangle();
			rect.x = midpoint.x - width - MINIMAL_DISTANCE;
			rect.y = top + (bottom - top - height) / 2;
			rect.width = width;
			rect.height = height;
			positions.add(rect);
			
		}
		
		space = structure.getFreeSpace(midpoint);
		
		if(segmentStart.y < segmentEnd.y) {
			top = segmentStart.y;
			bottom = segmentEnd.y;
		} else {
			top = segmentEnd.y;
			bottom = segmentStart.y;
		}
		if(space.y > top) {
			top = space.y;
		}
		if(space.getMaxY() < bottom) {
			bottom = (int)space.getMaxY();
		}
		
		// Try to place the label at the midpoint of the line
		// (i.e., the line crosses the label).
		width = space.width - 2 * MINIMAL_DISTANCE;
		height = label.getMinimalHeight(width);
		width = label.getWidth(width);
		rect = new Rectangle();
		// Try to center the label horizontally around the vertical line segment
		int x = midpoint.x - width / 2 - MINIMAL_DISTANCE;
		if(x < space.x) {
			x = space.x + MINIMAL_DISTANCE;
		}
		if(x + width > space.getMaxX()) {
			x = (int)space.getMaxX() - width - MINIMAL_DISTANCE;
		}
		rect.x = x;
		rect.y = top + (bottom - top - height) / 2;
		rect.width = width;
		rect.height = height;
		positions.add(rect);
		
		return positions;
		
	}
	
	private List<Rectangle> getPositionsAroundHorizontalSegment(Point segmentStart, Point segmentEnd, IRoutingLineLabel label) {

		Point midpoint = midpoint(segmentStart, segmentEnd);
		List<Rectangle> positions = new ArrayList<Rectangle>(3);
		
		Rectangle space = structure.getFreeSpace(midpoint);
		
		// Restrict search space to the length of the horizontal line segment.
		int start;
		int end;
		if(segmentStart.x < segmentEnd.x) {
			start = segmentStart.x;
			end = segmentEnd.x;
		} else {
			start = segmentEnd.x;
			end = segmentStart.x;
		}
		if(space.x > start) {
			start = space.x;
		}
		if(space.getMaxX() < end) {
			end = (int)space.getMaxX();
		}
		
		int width = end - start - 2 * MINIMAL_DISTANCE;
		
		int height = label.getMinimalHeight(width);
		width = label.getWidth(width);
		
		// Try to center the label horizontally within the horizontal segment
		int x = start + (end - start - width) / 2;
		if(x < start) {
			x = start + MINIMAL_DISTANCE;
		}
		if(x > end) {
			x = end - width - MINIMAL_DISTANCE;
		}
		
		// Try to place the label above the line
		Rectangle rect = new Rectangle();
		rect.x = x;
		rect.y = midpoint.y - height;
		rect.width = width;
		rect.height = height;
		positions.add(rect);
		
		// Try to place the label below the line
		rect = new Rectangle();
		rect.x = x;
		rect.y = midpoint.y;
		rect.width = width;
		rect.height = height;
		positions.add(rect);
		
		// Try to place the label at the midpoint of the line
		rect = new Rectangle();
		rect.x = x;
		rect.y = midpoint.y - height / 2;
		rect.width = width;
		rect.height = height;
		positions.add(rect);
		
		return positions;
		
	}
	
	
	private Point midpoint(Point p1, Point p2) {
		
		return new Point((p1.x + p2.x)/2, (p1.y + p2.y)/2);
		
	}
	
	
	private double getScore(Rectangle labelBounds, Point segmentStart, Point segmentEnd, IRoutingLine line) {
		
		Point midpoint = midpoint(segmentStart, segmentEnd);
		Rectangle space = structure.getFreeSpace(midpoint);
		boolean vertical = segmentStart.x == segmentEnd.x;
		double score = 0;

		// Distance from reference point to label's top left point
		double dist1 = midpoint.distance(labelBounds.getX(), labelBounds.getY());
		// Distance from reference point to label's bottom right point
		double dist2 = midpoint.distance(labelBounds.getMaxX(), labelBounds.getMaxY());
		score -= Math.min(dist1, dist2) / 20.0;
		
		// Width/height ratio if height is bigger than width.
		if(labelBounds.height > labelBounds.width) {
			double rate = (double) labelBounds.width / labelBounds.height;
			score -= 30.0 * rate;
		}
		else if(labelBounds.width / labelBounds.height > 15) {
			score -= 5.0;
		}

		// Punish labels which do not fit into the white space
		if(!structure.isFreeSpace(labelBounds)) {
			Rectangle intersection = (Rectangle)space.createIntersection(labelBounds);
			int diff = labelBounds.width * labelBounds.height - intersection.width * intersection.height;
			score -= diff * 10.0;
		}

		if(vertical == true) {
			// Punish labels which are crossed by the line
			if(labelBounds.x < segmentStart.x && labelBounds.getMaxX() > segmentStart.x) {
				score -= 60.0;
			}

			// Prefer labels to the right of the line
			if(labelBounds.x > segmentStart.x) {
				score += 2.0;
			}
		}
		else {
			// Punish labels which are crossed by the line
			if(labelBounds.y < segmentStart.y && labelBounds.getMaxY() > segmentStart.y) {
				score -= 10.0;
			}
			
			// Prefer labels above the line
			if(labelBounds.getMaxY() <= segmentStart.y) {
				score += 2.0;
			}
		}
		
		return score;
		
	}
	
	
	public List<Rectangle> getAllLabelPlaces() {
		
		return labelPlaces;
		
	}
	
	
	public List<Double> getAllScores() {
		
		return labelScores;
		
	}
	
}
