package ch.unizh.ifi.adora.linerouting.tilestructure;

import java.awt.Point;

import ch.unizh.ifi.adora.linerouting.algorithm.RoutingConstants;

/**
 * This class represents a tile that is occupied by a line. That means this tile
 * can be used for the routing of another line, but the costs to travel the tile
 * are higher than the costs to travel a space tile.
 * 
 * @author Tobias Reinhard
 * @history 2004-12-16 TR First version 2005-01-05 TR Line tiles implemented
 *          2005-01-26 TR Represent the orientation of the line segments
 *          2005-02-18 TR Crossings and bends as new orientations added
 *          2005-06-14 TR Extended tile structure implemented
 * 
 * @version $Date: 2009-12-08 09:19:27 +0100 (Tue, 08 Dec 2009) $, $Author: treinhard $, $Revision: 4983 $
 */
public class LineTile extends Tile {

	/**
	 * The line tile represents a horizontal line segment
	 */
	public static final int HORIZONTAL = 1;

	/**
	 * The line tile represents a vertical line segment
	 */
	public static final int VERTICAL = 2;

	/**
	 * The line tile represents a crossing between two lines
	 */
	public static final int CROSSING = 3;

	/**
	 * The line segment represents a bend of the line
	 */
	public static final int BEND = 4;

	/**
	 * The orientation of the line segment that is represented by this line tile
	 */
	private int orientation;

	/**
	 * Constructor for a line tile.
	 * 
	 * @param topLeft the top left corner of this tile
	 * @param bottomRight the bottom right corner of this tile
	 * @param lt the lt corner stitch (can be null if there's no neighbor)
	 * @param tl the tl corner stitch (can be null if there's no neighbor)
	 * @param rt the rt corner stitch (can be null if there's no neighbor)
	 * @param tr the tr corner stitch (can be null if there's no neighbor)
	 * @param rb the rb corner stitch (can be null if there's no neighbor)
	 * @param br the br corner stitch (can be null if there's no neighbor)
	 * @param lb the lb corner stitch (can be null if there's no neighbor)
	 * @param bl the bl corner stitch (can be null if there's no neighbor)
	 * @param orientation the orientation of the line segment (horizontal or vertical)
	 * 
	 */
	public LineTile(Point topLeft, Point bottomRight, Tile lt, Tile tl,
			Tile rt, Tile tr, Tile rb, Tile br, Tile lb, Tile bl,
			int orientation) {
		super(topLeft, bottomRight, lt, tl, rt, tr, rb, br, lb, bl);
		this.orientation = orientation;
		
		costFactor = RoutingConstants.ROUTING_LINE_COSTFACTOR;

	}

	/**
	 * Returns true because this is a line tile.
	 * 
	 * @return true
	 * 
	 */
	public boolean isLineTile() {
		return true;
	}

	/**
	 * Returns the orientation of the line segment that is represented by this
	 * line tile.
	 * 
	 * @return orientation
	 * 
	 */
	public int getOrientation() {
		return orientation;
	}

	
	/**
	 * Clones the tile (i.e., returns new line tile with the same neighbors
	 * and orientation)
	 * 
	 * @return the clone
	 * 
	 */
	public Object clone() {

		LineTile t = new LineTile(getTopLeft(), getBottomRight(), getLt(),
				getTl(), getRt(), getTr(), getRb(), getBr(), getLb(), getBl(),
				getOrientation());

		return t;

	}
	
	
	/**
	 * Returns true as line line tiles have to be split.
	 * 
	 * @return always true
	 * 
	 */
	public boolean hasToBeSplitted() {
		return true;
	}
	
	
    /**
     * Returns true if the two tiles top and bottom given as
     * argument can be merged vertical.
     * 
     * @param bottom the bottom tile
     * @return true if the two tiles can be merged
     */
    protected boolean canBeMergedWith(Tile bottom) {

    		return (bottom.isLineTile()
                && getTopLeft().x == bottom.getTopLeft().x
                && getBottomRight().x == bottom.getBottomRight().x
                && getBl() == bottom.getTl() && getBr() == bottom.getBr());

    }
}
