package ch.unizh.ifi.adora.linerouting.tilestructure;

import java.awt.Point;
import java.util.Iterator;
import java.util.List;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.linerouting.utils.HierarchyUtilities;


/**
 * This class represents the corner stitching data structure contained in a
 * specific node. The structure can be constructed for a node (e.g. for all it's
 * children) or for a connection (e.g. for the parent node of the connection and
 * all the nodes that can be an obstacle for the line routing). This tile structure
 * has additional line tiles that are used to avoid line intersections.
 * 
 * @author Tobias Reinhard
 * @history
 * 2004-12-16  TR  First version
 * 2005-01-05  TR  Line tiles implemented
 * 2005-01-26  TR  Represent the orientation of the line segments
 * 2005-02-17  TR  Adjust the size of the line segment if there's not enough space
 * 2005-02-18  TR  Bends on crossings of lines are not explicitly represented
 * 2005-05-18  TR  Decoupled from the main adora package
 * 2005-06-14  TR Extended tile structure implemented
 * 
 * @version $Date: 2009-12-11 14:55:00 +0100 (Fri, 11 Dec 2009) $, $Author: treinhard $, $Revision: 4986 $
 */
public class LineTileStructure extends TileStructure {

	/**
     * This is the size of the line tiles (e.g. the additional space around
     * the tiles).
     * 
     * CAUTION: If this value is changed, the insertion of line tiles may
     *          no longer work properly, because there may be not enough free
     *          space around the line tile to insert it.
	 */
	private int distToLine = 0;
	
	
	/**
	 * Constructor to build a new corner stitching structure for the node node (e.g. for all
	 * the children of this node).
	 * 
	 * @param node the node the structure has to be constructed for
	 * 
	 */
	public LineTileStructure(IRoutingNode node) throws InvalidLayoutException {	
		super(node);	
	}
	
	
    /**
     * Constructor to build a new corner stitching structure for a connection
     * (i.e., for the parent node of the connection).
     * 
     * @param line the line the structure has to be constructed for
     *  
     */
	public LineTileStructure(IRoutingLine line) throws InvalidLayoutException {	
		super(line);	
	}
	
	
	/**
	 * Constructor to build a new corner stitching structure for a connection (e.g. for the
	 * parent node of the connection).
	 * 
	 * @param line the line the structure has to be constructed for
     * @param lines the existing lines that are obstacles for the new line
	 * 
	 */
	public LineTileStructure(IRoutingLine line, List<IRoutingLine> lines) throws InvalidLayoutException {	
		super(line);
        
        Iterator<IRoutingLine> iter = lines.iterator();
        while(iter.hasNext()) {
            IRoutingLine current = iter.next();
            if(current == line) {
            	return;
            }
            insertLine(current);       
        }
        
	}
	
	
    /**
     * Method to insert the line given as parameter into the corner stitching structure.
     * 
     * @param line the line to insert into the structure
     * 
     */
	protected void insertLine(IRoutingLine line) throws InvalidLayoutException {
		
		if(!isOrthogonal(line)) {
			return;
		}
		
		List<Point> plist = line.getRoutingBendpoints();
		int segment = 0;
		
		Point start = new Point(plist.get(0).x, plist.get(0).y);
		Point end;
		
		// Adjust the position of the start point of the line segment
		// relative to the tile structure 
		IRoutingNode lineParent = HierarchyUtilities.getInstance().commonAncestor(line.getRoutingSource(), line.getRoutingTarget());
		
		Point absLineParentLoc = HierarchyUtilities.getInstance().getAbsoluteLocation(lineParent);
		Point absParentLoc = HierarchyUtilities.getInstance().getAbsoluteLocation(parentNode);
		
		start.translate(absLineParentLoc.x, absLineParentLoc.y);
        start.translate(-absParentLoc.x, -absParentLoc.y);
        
		// Iterate over the segments of the line
        for(int i = 1; i < plist.size(); i++) {
			
			end = new Point(plist.get(i).x, plist.get(i).y);
			
			// Adjust the position of the end point of the line segment
			// relative to the tile structure
			end.translate(absLineParentLoc.x, absLineParentLoc.y);
            end.translate(-absParentLoc.x, -absParentLoc.y);
			
			Point from = null;
			Point to = null;
			
			// Check whether the start and end point lie inside the tile structure
			boolean startInside = start.x > 0 && start.x < getWidth() && start.y > 0 && start.y < getHeight();
			boolean endInside = end.x > 0 && end.x < getWidth() && end.y > 0 && end.y < getHeight();
			
			// The segment lies outside the tile structure.
			// Don't do anything, just go on to the next line segment
			if(!startInside && !endInside) {
				
			}
			
			// The segment lies inside a node (i.e. a child of the parent node of this structure)
			// Don't do anything, just go on to the next line segment
			else if(startInside && !findTileAtPoint(start).isSpace() &&
					endInside && !findTileAtPoint(end).isSpace() &&
					((NodeTile)findTileAtPoint(start)).getNode() == ((NodeTile)findTileAtPoint(end)).getNode()) {
				
			}
			
			/*
			 * Segment from north to south
			 */ 
			else if(start.x == end.x && start.y < end.y) {
				
				// Only part of the line segment lies in the tile structure
				if(start.y < 0) {
					from = new Point(start.x, 0);
				}
				// One part of the segment lies inside a node (e.g. a child of the parent node)
				else if(!findTileAtPoint(start).isSpace()) {
					Tile tile = findTileAtPoint(start);
					from = new Point(start.x, tile.getBottomRight().y + 1);
					while(!findTileAtPoint(from).isSpace()) {
						from = new Point(start.x, findTileAtPoint(from).getBottomRight().y + 1);
					}
				}
				// This is the first segment of the line
				else if(segment == 0) {
					from = start; 
				}
				else {
					from = new Point(start.x, start.y + distToLine + 1);
				}
				
				// Only part of the line segment lies in the tile structure
				if(end.y > getHeight()) {
					to = new Point(start.x, getHeight());
				}
				// One part of the segment lies inside a node (e.g. a child of the parent node)
				else if(!findTileAtPoint(end).isSpace()) {
					Tile tile = findTileAtPoint(end);
					to = new Point(start.x, tile.getTopLeft().y - 1);
				}
				// This is the last segment of the line
				else if(segment == line.getRoutingBendpoints().size() - 2) {
                    to = end;
				}
				else {
					to = new Point(end.x, end.y - distToLine - 1);
                }
				
				adjustIntersectionsVerticalSegment(from, to);
			}
			
			/*
			 * Segment from south to north
			 */ 
			else if(start.x == end.x && start.y > end.y) {
				
				// Check the start point of the segment
				
				if(start.y > getHeight()) {
					to = new Point(start.x, getHeight());
				}
				
				else if(!findTileAtPoint(start).isSpace()) {
					Tile tile = findTileAtPoint(start);
					to = new Point(start.x, tile.getTopLeft().y - 1);
					while(!findTileAtPoint(to).isSpace()) {
						to = new Point(start.x, findTileAtPoint(to).getTopLeft().y - 1);
					}
				}
				
				else if(segment == 0) {
                    to = start;
				}
				else {
					to = new Point(start.x, start.y - distToLine - 1);
				}
				
				// Check the end point of the segment
				
				if(end.y < 0) {
					from = new Point(end.x, 0);
				}
				
				else if(!findTileAtPoint(end).isSpace()) {
					Tile tile = findTileAtPoint(end);
					from = new Point(start.x, tile.getBottomRight().y + 1);
				}
				
				else if(segment == line.getRoutingBendpoints().size() - 2)
				{
					from = end;
				}
				else {
					from = new Point(end.x, end.y + distToLine);                   
				}
				
				adjustIntersectionsVerticalSegment(from, to);
			}
			
			/*
			 * Segment from west to east
			 */ 
			else if(start.y == end.y && start.x < end.x) {
				
				// Check the start point of the segment
				
				if(start.x < 0) {
					from = new Point(0, start.y);
				}
				
				else if(!findTileAtPoint(start).isSpace()) {
					Tile tile = findTileAtPoint(start);
					from = new Point(tile.getBottomRight().x + 1, start.y);
					while(!findTileAtPoint(from).isSpace()) {
						from = new Point(findTileAtPoint(from).getBottomRight().x + 1, start.y);
					}
				}
				
				else if(segment == 0) {
					from = start;
				}
				else {
					from = new Point(start.x + distToLine + 1, start.y);
					Point p1 = new Point(start.x - distToLine, start.y - distToLine);
					Point p2 = new Point(start.x + distToLine, start.y + distToLine);
					Tile newLineTile = new LineTile(p1, p2, null, null, null, null,
				                     null, null, null, null, LineTile.BEND);
					insert(newLineTile);
				}
				
				// Check the end point of the segment
				
				if(end.x > getWidth()) {
					to = new Point(getWidth(), end.y);
				}
				
				else if(!findTileAtPoint(end).isSpace()) {
					Tile tile = findTileAtPoint(end);
					to = new Point(tile.getTopLeft().x - 1, end.y);
				}
				
				else if(segment == line.getRoutingBendpoints().size() - 2) {
					to = new Point(end.x, end.y);
				}
				else {
				    to = new Point(end.x - distToLine - 1, end.y);
				    Point p1 = new Point(end.x -distToLine, end.y - distToLine);
				    Point p2 = new Point(end.x + distToLine, end.y + distToLine);
				    Tile newLineTile = new LineTile(p1, p2, null, null, null, null,
		                     null, null, null, null, LineTile.BEND);
				    insert(newLineTile);
				}
				
				adjustIntersectionsHorizontalSegment(from, to);
			}
			
			/*
			 * Segment from east to west
			 */ 
			else if(start.y == end.y && start.x > end.x) {
				
				// Check the start point of the segment
				
				if(start.x > getWidth()) {
					to = new Point(getWidth(), start.y);
				}
				
				else if(!findTileAtPoint(start).isSpace()) {
					Tile tile = findTileAtPoint(start);
					to = new Point(tile.getTopLeft().x - 1, start.y);
					while(!findTileAtPoint(to).isSpace()) {
						to = new Point(findTileAtPoint(to).getTopLeft().x - 1, start.y);
					}
				}
				
				else if(segment == 0) {
                    to = start;
				}
				else {
					to = new Point(start.x - distToLine - 1, start.y);
					Point p1 = new Point(start.x - distToLine, start.y - distToLine);
					Point p2 = new Point(start.x + distToLine, start.y + distToLine);
					Tile newLineTile = new LineTile(p1, p2, null, null, null, null,
		                     null, null, null, null, LineTile.BEND);
					insert(newLineTile);
				}

				// Check the end point of the segment				
				
				if(end.x < 0) {
					from = new Point(0, end.y);
				}
				
				else if(!findTileAtPoint(end).isSpace()) {
					Tile tile = findTileAtPoint(end);
					from = new Point(tile.getBottomRight().x + 1, end.y);
				}
				
				else if(segment == line.getRoutingBendpoints().size() - 2) {
					from = end;
				}
				else {
					from = new Point(end.x + distToLine, end.y);
					Point p1 = new Point(end.x - distToLine, end.y - distToLine);
					Point p2 = new Point(end.x + distToLine, end.y + distToLine);
					Tile newLineTile = new LineTile(p1, p2, null, null, null, null,
		                     null, null, null, null, LineTile.BEND);
					insert(newLineTile);
				}
				
				adjustIntersectionsHorizontalSegment(from, to);
			}
					
			start = end;
			segment++;
		}
		
	}
	
	
	/**
	 * This method splits the line tiles of a horizontal segment if the line crosses
	 * another line.
	 * 
	 * @param from start point of horizontal segment
	 * @param to end point of horizontal segment
	 * 
	 */
	private void adjustIntersectionsHorizontalSegment(Point from, Point to) throws InvalidLayoutException{
		
		Point topLeft = new Point(from.x, from.y - distToLine);
		Point bottomRight = new Point(to.x, to.y + distToLine);
		
		Tile leftTile = findTileAtPoint(from);
		Tile rightTile = findTileAtPoint(to);
		
        Tile currentTile = leftTile;
        
        if(!currentTile.isSpace()) {
        	return;
        }
        
		if(currentTile == rightTile) {
			
			if(!(topLeftTile.isLineTile())) {
              
				Tile newLineTile = new LineTile(topLeft, bottomRight, null, null, null, null,
	                     null, null, null, null, LineTile.HORIZONTAL);
				insert(newLineTile);			
			}
			
		}
		
		else {
		
			while(currentTile != null && currentTile != rightTile) {
			
				if(currentTile.isLineTile()) {
                    
				}

                else if(currentTile == leftTile) {
                    
                    Point p2 = new Point(currentTile.getBottomRight().x, bottomRight.y);
                    Tile newLineTile = new LineTile(topLeft, p2, null, null, null, null,
   	                     null, null, null, null, LineTile.HORIZONTAL);
                    insert(newLineTile);
                    currentTile = newLineTile;
                }
			
				else {
                   
                    Point p1 = new Point(currentTile.getTopLeft().x, topLeft.y);
                    Point p2 = new Point(currentTile.getBottomRight().x, bottomRight.y);
				
                    Tile newLineTile = new LineTile(p1, p2, null, null, null, null,
   	                     null, null, null, null, LineTile.HORIZONTAL);
                    insert(newLineTile);							
                }		
			
				Point p = new Point(currentTile.getBottomRight().x + 1, from.y);
				if(p.x > bottomRight.x) {
					break;
				}             
                
				currentTile = findTileAtPoint(p);
                
			}
			
			if(!(currentTile.isLineTile())) {
               
                Point p = new Point(currentTile.getTopLeft().x, topLeft.y);
			
                Tile newLineTile = new LineTile(p, bottomRight, null, null, null, null,
	                     null, null, null, null, LineTile.HORIZONTAL);
                insert(newLineTile);
		
			}
			
		}
		
	}
	
	
	/**
	 * This method splits the line tiles of a vertical segment if the line crosses
	 * another line.
	 * 
	 * @param from start point of vertical segment
	 * @param to end point of vertical segment
	 * 
	 */
	private void adjustIntersectionsVerticalSegment(Point from, Point to) throws InvalidLayoutException {
			
		Point topLeft = new Point(from.x - distToLine, from.y);
		Point bottomRight = new Point(to.x + distToLine, to.y);
        
		Tile topTile = findTileAtPoint(from);
		Tile bottomTile = findTileAtPoint(to);
		
		Tile currentTile = topTile;
		
		// Return if the height of the vertical line segment is zero
		if(topLeft.y >= bottomRight.y + 1 || !currentTile.isSpace()) {
			return;
		}
      
        while(currentTile != null) {
        	
            if(currentTile == bottomTile) {
                
                bottomRight = new Point(to.x + distToLine, to.y);
                Tile newTile = new LineTile(topLeft, bottomRight, null, null, null, null,
	                     null, null, null, null, LineTile.VERTICAL);
                insert(newTile);
                return;
                
            } else {
                bottomRight = new Point(from.x + distToLine, currentTile.getBottomRight().y);
            
                Tile newTile = new LineTile(topLeft, bottomRight, null, null, null, null,
	                     null, null, null, null, LineTile.VERTICAL);
                insert(newTile);
            
                Point p = new Point(from.x, newTile.getBottomRight().y + 1);
                
                currentTile = findTileAtPoint(p);
                topLeft = new Point(p.x - distToLine, p.y);
            }
            
        }
        
        String msg = "Error while trying to adjust vertical segments!";
    	throw new InvalidLayoutException(msg);
       
	}
	
	
	/**
	 * Returns true if the line given as argument is orthogonal (i.e., consists
	 * of vertical and horizontal segments only), false otherwise.
	 * 
	 * @param line the line to check
	 * @return true if the line is orthogonal, false otherwise
	 */
	protected boolean isOrthogonal(IRoutingLine line) {
		
		List<Point> points = line.getRoutingBendpoints();
		
		if(points == null) {
			return false;
		}
		
		if(points.size() > 0) {
			Point p1 = points.get(0);
			for(int i = 1; i < points.size(); i++) {
				Point p2 = points.get(i);
				if(!(p1.x == p2.x || p1.y == p2.y)) {
					return false;
				}
				p1 = p2;
			}
		}
		
		return true;
	}
	
	
}
