package ch.unizh.ifi.adora.linerouting.tilestructure;

import java.util.Iterator;

/**
 * This class is an implementation of an {@link java.util.Iterator} that iterates
 * over all neighbors of a tile. The neighbors are visited in clockwise order
 * (i.e. first the top neighbors, then the right, bottom and left neighbors).
 * 
 * @author              Tobias Reinhard (TR)
 * @copyright           Requirements Engineering Research Group
 *                      Department of Computer Science
 *                      University of Zurich
 * @history
 * 2005-03-04           TR First version
 * 
 * @version             $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 * @responsibilities    Iterates over all neighbors of a tile.
 */
class NeighborsIterator implements Iterator<Tile> {
    
    /**
     * Iterator to iterate over all the top neighbors
     */
    private TopNeighborsIterator ti;
    
    /**
     * Iterator to iterate over all the right neighbors
     */
    private RightNeighborsIterator ri;
    
    /**
     * Iterator to iterate over all the bottom neighbors
     */
    private BottomNeighborsIterator bi;
    
    /**
     * Iterator to iterate over all the left neighbors
     */
    private LeftNeighborsIterator li;
    

    /**
     * Constructor for an iterator that iterates over all the neighbors
     * of the tile given as argument.
     * 
     * @pre tile != null
     * @post true
     * 
     * @param tile the tile over whose neighbors the iterator iterates
     */
    public NeighborsIterator(Tile tile) {
        ti = new TopNeighborsIterator(tile);
        ri = new RightNeighborsIterator(tile);
        bi = new BottomNeighborsIterator(tile);
        li = new LeftNeighborsIterator(tile);
    }

    
    /**
     * Returns true if the iteration has more elements. This method can return
     * false during the first invocation if the tile doesn't have any top
     * neighbors (i.e. the tile lies at the border of the tile structure).
     * 
     * @pre true
     * @post true
     * 
     * @return true if the iteration has more elements
     */
    public boolean hasNext() {
        return ti.hasNext() || ri.hasNext() || bi.hasNext() || li.hasNext();
    }

    
    /**
     * Returns the next element in the iteration and advances the iteration to
     * the next element. Returns null if the iteration doesn't have a next element.
     * 
     * @pre true
     * @post true
     * 
     * @return the next element in the iteration
     */
    public Tile next() {
        if(ti.hasNext()) {
        	    return ti.next();
        }
        if(ri.hasNext()) {
        	    return ri.next();
        }
        if(bi.hasNext()) {
        	    return bi.next();
        }
        if(li.hasNext()) {
        	    return li.next();
        }
        return null;
    }

    
    /**
     * Standard iterator method to remove from the underlying collection the 
     * last element returned by the  iterator. This method doesn't do anything
     * in this implementation because the removal of a tile would destroy the
     * whole tile structure and doesn't make any sense.
     * 
     * @pre true
     * @post true
     * 
     */
    public void remove() {
    }

}
