package ch.unizh.ifi.adora.linerouting.tilestructure;

import java.awt.Point;

import ch.unizh.ifi.adora.linerouting.algorithm.RoutingConstants;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;

/**
 * This class represents a tile that is occupied by a node.
 * 
 * @author Tobias Reinhard
 * @history
 * 2004-12-16  TR  First version
 * 2005-01-05  TR  Line tiles implemented
 * 2005-05-18  TR  Decoupled from the main adora package
 * 
 * @version $Date: 2008-03-17 12:20:55 +0100 (Mon, 17 Mar 2008) $, $Author: treinhard $, $Revision: 3570 $
 */
public class NodeTile extends Tile  {

	/**
	 * The node this tile is occupied by. Null if the tile is a space tile.
	 */
	protected IRoutingNode node = null;
	
	
	/**
	 * Constructor for a node tile.
	 * 
	 * @param layout the node that occupies this tile, null if the tile is a space tile
	 * @param topLeft the top left corner of this tile
	 * @param bottomRight the bottom right corner of this tile
	 * @param lt the lt corner stitch (can be null if there's no neighbour)
	 * @param tl the tl corner stitch (can be null if there's no neighbour)
	 * @param rt the rt corner stitch (can be null if there's no neighbour)
	 * @param tr the tr corner stitch (can be null if there's no neighbour)
	 * @param rb the rb corner stitch (can be null if there's no neighbour)
	 * @param br the br corner stitch (can be null if there's no neighbour)
	 * @param lb the lb corner stitch (can be null if there's no neighbour)
	 * @param bl the bl corner stitch (can be null if there's no neighbour)
	 * 
	 * @pre topLeft != null && bottomRight != null
	 * @post true
	 * 
	 */
	public NodeTile(IRoutingNode node, Point topLeft, Point bottomRight, 
		    Tile lt, Tile tl, Tile rt, Tile tr, Tile rb, Tile br, Tile lb, Tile bl)
	{
	
		super(topLeft, bottomRight, lt, tl, rt, tr, rb, br, lb, bl);
		this.node = node;
		
		// Use the cost factor to control the line routing inside the start
		// and end node (i.e. the start and end point of the line)
		costFactor = RoutingConstants.ROUTING_NODE_COSTFACTOR;
	}
	
	
	/**
	 * Returns always false because this tile is not a space tile.
	 * 
	 * @return true because this is not a space tile
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public boolean isSpace()
	{
		return false;
	}
	
	
	/**
	 * Returns a clone of this node tile. The clone is occupied by the same node,
	 * has the same location and size and the same neighbours as this node
	 * tile.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return a clone of this node tile
	 */
	public Object clone() {

		NodeTile t = new NodeTile(node, getTopLeft(), getBottomRight(), getLt(), getTl(),
				getRt(), getTr(), getRb(), getBr(), getLb(), getBl());

		return t;

	}
	
	
	/**
	 * Returns the node (i.e. the IFigure) that occupies this node tile.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return the node that occupies this node tile
	 */
	public IRoutingNode getNode() {
		return node;
	}
	
	
    /**
     * Returns true if the two tiles top and botttom given as
     * argument can be merged vertical.
     * 
     * @pre bottom != null
     * @post true
     * 
     * @param bottom the bottom tile
     * @return true if the two tiles can be merged
     */
    protected boolean canBeMergedWith(Tile bottom) {

        return false;

    }
	
}
