package ch.unizh.ifi.adora.linerouting.tilestructure;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.Iterator;
import java.util.Vector;


/**
 * This class represents a tile in the corner stitching data structure. This is
 * the superclass of all the tile types in the tile structure. It represents a
 * space tile.
 * 
 * The corner stitches are named as follows:
 * 
 *         lt                    rt 
 *      tl ------------------------ tr 
 *         |                      | 
 *         |                      | 
 *         |                      |
 *         |                      | 
 *      bl ------------------------ br 
 *         lb                    rb
 * 
 * 
 * @author Tobias Reinhard
 * @history 2005-10-28 TR Comments updated
 * 
 * @version $Date: 2008-08-11 16:16:12 +0200 (Mon, 11 Aug 2008) $, $Author: treinhard $, $Revision: 3789 $
 */
public class Tile implements Comparable<Tile> {

	/**
	 * The corner stitch in the top left corner of the tile that points to the
	 * top neighbor
	 */
	private Tile lt;

	/**
	 * The corner stitch in the top left corner of the tile that points to the
	 * left neighbor
	 */
	private Tile tl;

	/**
	 * The corner stitch in the top right corner of the tile that points to the
	 * top neighbor
	 */
	private Tile rt;

	/**
	 * The corner stitch in the top right corner of the tile that points to the
	 * right neighbor
	 */
	private Tile tr;

	/**
	 * The corner stitch in the bottom right corner of the tile that points to
	 * the bottom neighbor
	 */
	private Tile rb;

	/**
	 * The corner stitch in the bottom right corner of the tile that points to
	 * the right neighbor
	 */
	private Tile br;

	/**
	 * The corner stitch in the bottom left corner of the tile that points to
	 * the bottom neighbor
	 */
	private Tile lb;

	/**
	 * The corner stitch in the bottom left corner of the tile that points to
	 * the left neighbor
	 */
	private Tile bl;

	/**
	 * The coordinates of the top left corner of this tile (relative to the tile
	 * structure)
	 */
	private Point topLeft;

	/**
	 * The coordinates of the bottom right corner of this tile (relative to the
	 * tile structure)
	 */
	private Point bottomRight;

	/**
	 * The name of this tile. Only for debugging purposes.
	 */
	private String name;

	/**
	 * Static variable to generate unique ids for all the tiles. The name of the
	 * tile is the id.
	 */
	static int numberOfTiles = 1;

	/**
	 * The constant cost factor that is multiplied with the distance the line
	 * covers in this tile. The cost factor can become adjusted by the
	 * subclasses (e.g. the costs to travel trough a line tile are higher than
	 * the costs for a space tile.
	 */
	protected float costFactor;

	/**
	 * Stores the source costs (i.e the distance to the start point)
	 */
	private int sourceCost = -1;

	/**
	 * Stores the destination costs (i.e. the distance to the end point)
	 */
	private int destinationCost = 0;

	/**
	 * The position of the cost point inside this tile
	 */
	private Point costPoint;

	/**
	 * Stores all tiles that are predecessors of this tile during the expansion
	 * of the wave.
	 */
	private Vector<Tile> origins;

	/**
	 * Constructor for a tile.
	 * 
	 * @param topLeft
	 *            the top left corner of this tile
	 * @param bottomRight
	 *            the bottom right corner of this tile
	 * @param lt
	 *            the lt corner stitch (can be null if there's no neighbor)
	 * @param tl
	 *            the tl corner stitch (can be null if there's no neighbor)
	 * @param rt
	 *            the rt corner stitch (can be null if there's no neighbor)
	 * @param tr
	 *            the tr corner stitch (can be null if there's no neighbor)
	 * @param rb
	 *            the rb corner stitch (can be null if there's no neighbor)
	 * @param br
	 *            the br corner stitch (can be null if there's no neighbor)
	 * @param lb
	 *            the lb corner stitch (can be null if there's no neighbor)
	 * @param bl
	 *            the bl corner stitch (can be null if there's no neighbor)
	 * 
	 * @pre topLeft != null && bottomRight != null
	 * @post true
	 * 
	 */
	public Tile(Point topLeft, Point bottomRight, Tile lt, Tile tl, Tile rt,
			Tile tr, Tile rb, Tile br, Tile lb, Tile bl) {
		this.topLeft = topLeft;
		this.bottomRight = bottomRight;
		this.setLt(lt);
		this.setTl(tl);
		this.setRt(rt);
		this.setTr(tr);
		this.setRb(rb);
		this.setBr(br);
		this.setLb(lb);
		this.setBl(bl);

		name = "" + numberOfTiles++;

		// No special cost factor for a space tile (use just the distance)
		costFactor = 1;

		origins = new Vector<Tile>(2);
	}

	/**
	 * Set the corner stitch in the top left corner of the tile that points to
	 * the top neighbor
	 * 
	 * @param lt
	 *            the lt corner stitch
	 * 
	 * @pre lt != null
	 * @post true
	 * 
	 */
	public void setLt(Tile lt) {
		this.lt = lt;
	}

	/**
	 * Returns the corner stitch in the top left corner of the tile that points
	 * to the top neighbor
	 * 
	 * @return the lt corner stitch
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Tile getLt() {
		return lt;
	}

	/**
	 * Set the corner stitch in the top left corner of the tile that points to
	 * the left neighbor
	 * 
	 * @param tl
	 *            the tl corner stitch
	 * 
	 * @pre tl != null
	 * @post true
	 * 
	 */
	public void setTl(Tile tl) {
		this.tl = tl;
	}

	/**
	 * Returns the corner stitch in the top left corner of the tile that points
	 * to the left neighbor
	 * 
	 * @return the tl corner stitch
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Tile getTl() {
		return tl;
	}

	/**
	 * Set the corner stitch in the top right corner of the tile that points to
	 * the top neighbor
	 * 
	 * @param the
	 *            rt corner stitch
	 * 
	 * @pre rt != null
	 * @post true
	 * 
	 */
	public void setRt(Tile rt) {
		this.rt = rt;
	}

	/**
	 * Returns the corner stitch in the top right corner of the tile that points
	 * to the top neighbor
	 * 
	 * @return the rt corner stitch
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Tile getRt() {
		return rt;
	}

	/**
	 * Set the corner stitch in the top right corner of the tile that points to
	 * the right neighbor
	 * 
	 * @param tr
	 *            the tr corner stitch
	 * 
	 * @pre tr != null
	 * @post true
	 * 
	 */
	public void setTr(Tile tr) {
		this.tr = tr;
	}

	/**
	 * Returns the corner stitch in the top right corner of the tile that points
	 * to the right neighbor
	 * 
	 * @return the tr corner stitch
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Tile getTr() {
		return tr;
	}

	/**
	 * Set the corner stitch in the bottom right corner of the tile that points
	 * to the bottom neighbor
	 * 
	 * @param rb
	 *            the rb corner stitch
	 * 
	 * @pre rb != null
	 * @post true
	 * 
	 */
	public void setRb(Tile rb) {
		this.rb = rb;
	}

	/**
	 * Returns the corner stitch in the bottom right corner of the tile that
	 * points to the bottom neighbor
	 * 
	 * @return the rb corner stitch
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Tile getRb() {
		return rb;
	}

	/**
	 * Set the corner stitch in the bottom right corner of the tile that points
	 * to the right neighbor
	 * 
	 * @param br
	 *            the br corner stitch
	 * 
	 * @pre br != null
	 * @post true
	 * 
	 */
	public void setBr(Tile br) {
		this.br = br;
	}

	/**
	 * Returns the corner stitch in the bottom right corner of the tile that
	 * points to the right neighbor
	 * 
	 * @return the br corner stitch
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Tile getBr() {
		return br;
	}

	/**
	 * Set the corner stitch in the bottom left corner of the tile that points
	 * to the bottom neighbor
	 * 
	 * @param lb
	 *            the lb corner stitch
	 * 
	 * @pre lb != null
	 * @post true
	 * 
	 */
	public void setLb(Tile lb) {
		this.lb = lb;
	}

	/**
	 * Returns the corner stitch in the bottom left corner of the tile that
	 * points to the bottom neighbor
	 * 
	 * @return the lb corner stitch
	 * 
	 * @pre true
	 * @post true
	 */
	public Tile getLb() {
		return lb;
	}

	/**
	 * Set the corner stitch in the bottom left corner of the tile that points
	 * to the left neighbor
	 * 
	 * @param bl
	 *            the bl corner stitch
	 * 
	 * @pre bl != null
	 * @post true
	 * 
	 */
	public void setBl(Tile bl) {
		this.bl = bl;
	}

	/**
	 * Returns the corner stitch in the bottom left corner of the tile that
	 * points to the left neighbor
	 * 
	 * @return the bl corner stitch
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Tile getBl() {
		return bl;
	}

	/**
	 * Returns the coordinates of the top left corner of this tile (relative to
	 * the corner stitching structure).
	 * 
	 * @return the top left corner
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Point getTopLeft() {
		return topLeft;
	}

	/**
	 * Sets the coordinates of the top left corner of this tile (relative to the
	 * corner stitching structure).
	 * 
	 * @param topLeft
	 *            the top left corner
	 * 
	 * @pre topLeft != null
	 * @post this.getTopLeft() == topLeft
	 * 
	 */
	public void setTopLeft(Point topLeft) {
		this.topLeft = topLeft;
	}

	/**
	 * Returns the coordinates of the bottom right corner of this tile (relative
	 * to the corner stitching structure).
	 * 
	 * @return the bottom right corner
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Point getBottomRight() {
		return bottomRight;
	}

	/**
	 * Sets the coordinates of the bottom right corner of this tile (relative to
	 * the corner stitching structure).
	 * 
	 * @param bottomRight
	 *            the bottom right corner
	 * 
	 * @pre bottomRight != null
	 * @post this.getBottomRight() == bottomRight
	 * 
	 */
	public void setBottomRight(Point bottomRight) {
		this.bottomRight = bottomRight;
	}
	
	/**
	 * Returns the top right point of the tile. The coordinates are relative to
	 * the tile structure.
	 * 
	 * @return top right point of the tile
	 */
	public Point getTopRight() {
		return new Point(bottomRight.x, topLeft.y);
	}	
	
	/**
	 * Returns the bottom left point of the tile. The coordinates are relative to
	 * the tile structure.
	 * 
	 * @return bottom left point of the tile
	 */
	public Point getBottomLeft() {
		return new Point((int)getBounds().getMinX(), (int)getBounds().getMaxY());
	}
	
	/**
	 * Returns a rectangle representing the tiles position relative to the tile
	 * structure and its size.
	 * 
	 * @return rectangle representing the position and size
	 */
	public Rectangle getBounds() {
		
		int width = bottomRight.x - topLeft.x;
		int height = bottomRight.y - topLeft.y;
		
		return new Rectangle(topLeft.x, topLeft.y, width, height);
		
	}

	/**
	 * Returns the coordinates of the center of this tile (relative to the tile
	 * structure).
	 * 
	 * @return the center of the tile
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public Point getCenter() {
		int x = topLeft.x + (getWidth() / 2);
		int y = topLeft.y + (getHeight() / 2);

		return new Point(x, y);
	}

	/**
	 * Returns the width of this tile.
	 * 
	 * @return the width of the tile
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public int getWidth() {
		return bottomRight.x - topLeft.x;
	}

	/**
	 * Returns the height of the tile.
	 * 
	 * @return the height of the tile
	 * 
	 * @pre true
	 * @post true
	 */
	public int getHeight() {
		return bottomRight.y - topLeft.y;
	}

	/**
	 * Returns true if this tile is a space tile (e.g. not occupied by a node),
	 * false otherwise.
	 * 
	 * @return true if the tile is a space tile, false otherwise
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public boolean isSpace() {
		return true;
	}

	/**
	 * Returns true if this tile is a line tile (e.g. not occupied by a line),
	 * false otherwise.
	 * 
	 * @return true if the tile is a line tile, false otherwise
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public boolean isLineTile() {
		return false;
	}

	/**
	 * Returns the name (e.g. number) of this tile. This name is also drawn for
	 * each tile if the while tile structure is drawn inside a node.
	 * 
	 * @return the name of the tile
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns the cost factor of this tile.
	 * 
	 * @return the cost factor
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public float getCostFactor() {
		return costFactor;
	}
	
	
	/**
	 * Method to set the cost factor of this node.
	 * 
	 * @pre true
	 * @post getCostFactor() == costFactor
	 * 
	 * @param costFactor
	 */
	public void setCostFactor(float costFactor) {
		this.costFactor = costFactor;
	}
	

	/**
	 * Method to set the destination cost (i.e. the distance to the end point)
	 * 
	 * @pre cost >= 0
	 * @post true
	 * 
	 * @param cost
	 *            the destination cost
	 */
	public void setDestinationCost(int cost) {
		destinationCost = cost;
	}

	/**
	 * Method that returns the destination cost (i.e. the distance to the end
	 * point)
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return the destination cost
	 */
	public int getDestinationCost() {
		return destinationCost;
	}

	/**
	 * Method that returns the source cost (i.e. the distance to the start
	 * point)
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return the source cost
	 */
	public int getSourceCost() {
		return sourceCost;
	}

	/**
	 * Method to set the source cost (i.e. the distance to the start point)
	 * 
	 * @pre cost >= 0
	 * @post getSourceCost() == cost
	 * 
	 * @param cost
	 *            the source cost
	 */
	public void setSourceCost(int cost) {
		sourceCost = cost;
	}

	/**
	 * Returns the costs that are associated with this tile. The costs contain
	 * the source and the destination costs.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return the costs of this tile
	 */
	public int getCost() {
		return sourceCost + destinationCost;
	}

	/**
	 * Sets the position of the cost point inside this tile.
	 * 
	 * @pre point != null
	 * @post getCostPoint() == point
	 * 
	 * @param point
	 *            the position of the cost point
	 */
	public void setCostPoint(Point point) {
		costPoint = point;
	}

	/**
	 * Returns the position of the cost point inside this tile.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return the position of the cost point
	 */
	public Point getCostPoint() {
		return costPoint;
	}

	/**
	 * Returns true if this tile has already been visited.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return true if the tile has already been visited, false otherwise
	 */
	public boolean visited() {
		return sourceCost >= 0;
	}

	/**
	 * Comparable methods to sort the tiles in a sorted data structure (e.g. a
	 * heap). The tiles are sorted ascending by their costs.
	 * 
	 * @pre o != null
	 * @post true
	 * 
	 * @return <0 if this.getCost() < o.getCost() 0 if this.getCost() ==
	 *         o.getCost() >0 if this.getCost() > o.getCost()
	 * 
	 */
	public int compareTo(Tile o) {
		// return t.getCost() - getCost();

		int cost = getCost() - o.getCost();
		if (cost == 0) {
			return getSourceCost() - o.getSourceCost();
		}
		return cost;
	}

	/**
	 * Returns an iterator to iterate through all the top neighbors of this
	 * tile. The tiles are visited from left to right.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return iterator to iterate through all the top neighbors
	 */
	public Iterator<Tile> getTopNeighbors() {
		return new TopNeighborsIterator(this);
	}

	/**
	 * Returns an iterator to iterate through all the bottom neighbors of this
	 * tile. The tiles are visited from left to right.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return iterator to iterate through all the bottom neighbors
	 */
	public Iterator<Tile> getBottomNeighbors() {
		return new BottomNeighborsIterator(this);
	}

	/**
	 * Returns an iterator to iterate through all the left neighbors of this
	 * tile. The tiles are visited from top to bottom.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return iterator to iterate through all the left neighbors
	 */
	public Iterator<Tile> getLeftNeighbors() {
		return new LeftNeighborsIterator(this);
	}

	/**
	 * Returns an iterator to iterate through all the right neighbors of this
	 * tile. The tiles are visited from top to bottom.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return iterator to iterate through all the right neighbors
	 */
	public Iterator<Tile> getRightNeighbors() {
		return new RightNeighborsIterator(this);
	}

	/**
	 * Returns an iterator to iterate through all the neighbors (in top, right,
	 * bottom, left order) of this tile.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return iterator to iterate through all the neighbors of this tile
	 */
	public Iterator<Tile> getNeighbors() {
		return new NeighborsIterator(this);
	}

	/**
	 * Returns a string representation of this tile.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return string representation of this tile
	 */
	public String toString() {
		return "Tile " + getName() + " [(" + topLeft.x + ", " + topLeft.y + 
				"), (" + bottomRight.x + ", " + bottomRight.y + ")]";
	}

	/**
	 * Register a tile as the predecessor during the wave expansion.
	 * 
	 * @param origin the tile that has to be registered
	 * 
	 * @pre origin != null
	 * @post true
	 */
	public void addOrigin(Tile origin) {
		origins.add(origin);
	}

	/**
	 * Reset (i.e. remove) all the tiles that are registered as predecessors of
	 * this tile during the wave expansion.
	 * 
	 * @pre true
	 * @post getOrigins().length == 0
	 */
	public void resetOrigins() {
		origins.clear();
	}

	/**
	 * Returns an array that contains all the tiles that are registered as
	 * predecessors of this tile during the wave expansion.
	 * 
	 * @return am array containing all the predecessors
	 * 
	 * @pre true
	 * @post true
	 */
	public Tile[] getOrigins() {

		Tile[] tiles = new Tile[origins.size()];
		origins.toArray(tiles);
		return tiles;

	}

	/**
	 * Returns true is the tile given as argument is already registered as a
	 * predecessor of this tile.
	 * 
	 * @param tile the tile to check for
	 * @return true if the tile is already registered as predecessor, false
	 *         otherwise
	 * 
	 * @pre true
	 * @post true
	 */
	public boolean hasOrigin(Tile tile) {
		return origins.contains(tile);
	}

	/**
	 * Splits this horizontal at the y coordinate given as argument yCoord. 
	 * The method returns the bottom part of the split. The top
	 * part is the current tile.
	 * 
	 * @pre canBeSplitted() && yCoord >= getTopLeft().y && yCoord <= getBottomRight().y
	 * 
	 * @param yCoord
	 *            the y coordinate where the tile has to be split
	 * @return the bottom part of the split
	 */
	public Tile splitHorizontal(int yCoord) {

		Tile bottom = (Tile) clone();
		bottom.setTopLeft(new Point(getTopLeft().x, yCoord));
		bottom.setLt(this);
		bottom.setTl(null);
		bottom.setRt(this);
		bottom.setTr(null);
		bottom.setRb(getRb());
		bottom.setBr(null);
		bottom.setLb(getLb());
		bottom.setBl(null);

		// Adjust the top part
		setBottomRight(new Point(getBottomRight().x, yCoord - 1));

		// Adjust the left-top and right-top corner stitches of the tiles below
		// the split one
		Iterator<Tile> neighbors = getBottomNeighbors();
		while (neighbors.hasNext()) {
			Tile neighbour = (Tile) neighbors.next();

			if (neighbour.getTopLeft().x >= bottom.getTopLeft().x
					&& neighbour.getTopLeft().x <= bottom.getBottomRight().x) {
				neighbour.setLt(bottom);
			}

			if (neighbour.getBottomRight().x >= bottom.getTopLeft().x
					&& neighbour.getBottomRight().x <= bottom.getBottomRight().x) {
				neighbour.setRt(bottom);
			}
		}

		// Adjust the top-right and bottom-right corner stitches of the left
		// neighbors
		neighbors = getLeftNeighbors();
		while (neighbors.hasNext()) {
			Tile neighbour = (Tile) neighbors.next();

			if (neighbour.getTopLeft().y >= bottom.getTopLeft().y
					&& neighbour.getTopLeft().y <= bottom.getBottomRight().y) {
				neighbour.setTr(bottom);
			}

			if (neighbour.getBottomRight().y >= bottom.getTopLeft().y
					&& neighbour.getBottomRight().y <= bottom.getBottomRight().y) {
				neighbour.setBr(bottom);
			}
		}

		// Adjust the top-left and bottom-left corner stitches of the right
		// neighbors
		neighbors = getRightNeighbors();
		while (neighbors.hasNext()) {
			Tile neighbour = (Tile) neighbors.next();
			if (neighbour.getTopLeft().y >= bottom.getTopLeft().y
					&& neighbour.getTopLeft().y <= bottom.getBottomRight().y) {
				neighbour.setTl(bottom);
			}
			if (neighbour.getBottomRight().y >= bottom.getTopLeft().y
					&& neighbour.getBottomRight().y <= bottom.getBottomRight().y) {
				neighbour.setBl(bottom);
			}
		}

		// Adjust the top-left and bottom-left corner stitches of the bottom
		// part
		neighbors = getLeftNeighbors();
		while (neighbors.hasNext()) {
			Tile neighbour = (Tile) neighbors.next();
			if (bottom.getTopLeft().y >= neighbour.getTopLeft().y
					&& bottom.getTopLeft().y <= neighbour.getBottomRight().y) {
				bottom.setTl(neighbour);
			}
			if (bottom.getBottomRight().y >= neighbour.getTopLeft().y
					&& bottom.getBottomRight().y <= neighbour.getBottomRight().y) {
				bottom.setBl(neighbour);
			}
		}

		// Adjust the top-right and bottom-right corner stitches of the bottom
		// part
		neighbors = getRightNeighbors();
		while (neighbors.hasNext()) {
			Tile neighbour = (Tile) neighbors.next();
			if (bottom.getTopLeft().y >= neighbour.getTopLeft().y
					&& bottom.getTopLeft().y <= neighbour.getBottomRight().y) {
				bottom.setTr(neighbour);
			}
			if (bottom.getBottomRight().y >= neighbour.getTopLeft().y
					&& bottom.getBottomRight().y <= neighbour.getBottomRight().y) {
				bottom.setBr(neighbour);
			}
		}

		// Adjust the bottom-left corner stitch of the top part
		neighbors = getLeftNeighbors();
		while (neighbors.hasNext()) {
			Tile neighbour = (Tile) neighbors.next();
			if (getBottomRight().y >= neighbour.getTopLeft().y
					&& getBottomRight().y <= neighbour.getBottomRight().y) {
				setBl(neighbour);
				break;
			}
		}

		// Adjust the bottom-right corner stitch of the top part
		neighbors = getRightNeighbors();
		while (neighbors.hasNext()) {
			Tile neighbour = (Tile) neighbors.next();
			if (getBottomRight().y >= neighbour.getTopLeft().y
					&& getBottomRight().y <= neighbour.getBottomRight().y) {
				setBr(neighbour);
				break;
			}
		}

		// Adjust the left-bottom and right-bottom corner stitches of the top
		// part of the split tile
		setLb(bottom);
		setRb(bottom);

		return bottom;

	}

	
	/**
	 * Returns a clone of this tile, i.e. a tile that points to the same 
	 * neighbors. The neighbors still point to the original tile and
	 * not to the clone.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @ return the clone
	 */
	public Object clone() {

		Tile t = new Tile(getTopLeft(), getBottomRight(), getLt(), getTl(),
				getRt(), getTr(), getRb(), getBr(), getLb(), getBl());

		return t;

	}
	
	
	/**
	 * This method has to return true if the tile has to be split if
	 * a neighbored tile is split.  This method can be overriden by
	 * subclasses.
	 * 
	 * @pre true
	 * @post true
	 * 
	 * @return true of the tile has to be split, false otherwise
	 */
	public boolean hasToBeSplitted() {
		return false;
	}
	
	
    /**
     * Returns true if the two tiles top and bottom given as
     * argument can be merged vertical.
     * 
     * @pre bottom != null
     * @post true
     * 
     * @param bottom the bottom tile
     * @return true if the two tiles can be merged
     */
    protected boolean canBeMergedWith(Tile bottom) {
    	
        return (bottom != null && getClass().equals(bottom.getClass())
                && getTopLeft().x == bottom.getTopLeft().x 
                && getBottomRight().x == bottom.getBottomRight().x);

    }
    
    
    /**
     * Method to merge to space tiles vertically. The method returns true if the
     * two tiles have been merged and false if the merger wasn't possible (i.e.
     * because one tile wasn't a space tile).
     * 
     * @param top
     *            the top tile of the two tiles to merge
     * @param bottom
     *            the bottom tile of the two tiles to merge
     * @return true if the two tiles have been merged, false otherwise
     * 
     * @pre top != null && bottom != null
     * @post true
     *  
     */
    public boolean mergeWith(Tile bottom) {
        if (canBeMergedWith(bottom)) {
            setBottomRight(bottom.getBottomRight());

            // Adjust the top-right and bottom-right corner stitches of the left
            // neighbors
            Iterator<Tile> neighbors = bottom.getLeftNeighbors();
            while (neighbors.hasNext()) {
                Tile neighbour = (Tile) neighbors.next();
                if (neighbour.getTopLeft().y >= getTopLeft().y
                        && neighbour.getTopLeft().y <= getBottomRight().y) {
                    neighbour.setTr(this);
                }
                if (neighbour.getBottomRight().y >= getTopLeft()
                        .y
                        && neighbour.getBottomRight().y <= getBottomRight().y) {
                    neighbour.setBr(this);
                }
            }

            // Adjust the top-left and bottom-left corner stitches of the right
            // neighbors
            neighbors = bottom.getRightNeighbors();
            while (neighbors.hasNext()) {
                Tile neighbour = (Tile) neighbors.next();
                if (neighbour.getTopLeft().y >= getTopLeft().y
                        && neighbour.getTopLeft().y <= getBottomRight().y) {
                    neighbour.setTl(this);
                }
                if (neighbour.getBottomRight().y >= getTopLeft()
                        .y
                        && neighbour.getBottomRight().y <= getBottomRight().y) {
                    neighbour.setBl(this);
                }
            }

            // Adjust the left-top and right-top corner stitches of the bottom
            // neighbors
            neighbors = bottom.getBottomNeighbors();
            while (neighbors.hasNext()) {
                Tile neighbour = (Tile) neighbors.next();
                if (neighbour.getTopLeft().x >= bottom.getTopLeft().x
                        && neighbour.getTopLeft().x <= bottom
                                .getBottomRight().x) {
                    neighbour.setLt(this);
                }
                if (neighbour.getBottomRight().x >= bottom.getTopLeft()
                        .x
                        && neighbour.getBottomRight().x <= bottom
                                .getBottomRight().x) {
                    neighbour.setRt(this);
                }
            }

            // Adjust the left-bottom and right-bottom corner stitches of the
            // merged tile
            setLb(bottom.getLb());
            setRb(bottom.getRb());

            // Adjust the bottom-left and bottom-right corner stitches of the
            // merged tile
            setBl(bottom.getBl());
            setBr(bottom.getBr());

            return true;
        }

        return false;
    }

}