package ch.unizh.ifi.adora.linerouting.tilestructure;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.linerouting.utils.HierarchyUtilities;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;


/**
 * This class represents the corner stitching data structure contained in a
 * specific node. The structure can be constructed for a node (e.g. for all it's
 * children) or for a connection (e.g. for the parent node of the connection and
 * all the nodes that can be an obstacle for the line routing). The class also
 * provides some basic operations on the corner stitching structure.
 * 
 * @author Tobias Reinhard
 * @history 
 * 2004-11-17 TR First version 
 * 2004-11-18 TR Comments updated
 * 2004-11-22 TR Tiles for the drawing of its shapes 
 * 2004-11-26 TR Support for tile naming added 
 * 2004-12-16 TR Node tiles are now represented by a subclass of its own 
 * 2005-02-18 TR Don't merge line tiles (even though they're space tiles) 
 * 2005-05-18 TR Decoupled from the main adora package
 * 2005-06-14 TR Extended tile structure implemented
 * 
 * @version $Date: 2009-12-09 14:44:04 +0100 (Wed, 09 Dec 2009) $, $Author: treinhard $, $Revision: 4984 $
 */
public class TileStructure {
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(TileStructure.class
			.getName());
	
    /** 
     * The node this Tile structure is contained in 
     */
    protected IRoutingNode parentNode;

    /**
     * Reference to the top left tile in the structure. From this tile all the
     * other tiles can be obtained by following the corner stitches.
     */
    protected Tile topLeftTile;

    /** 
     * The width of this corner stitching structure 
     */
    private int width;

    /** 
     * The height of this corner stitching structure
     */
    private int height;
    
    /**
     * Contains all the tiles of the structure (for visualization and debugging)
     */
    private List<Tile> allTiles;

    /**
     * Constructor to build a new corner stitching structure for the node node
     * (e.g. for all the children of this node).
     * 
     * @param node the node the structure has to be constructed for
     *  
     */
    public TileStructure(IRoutingNode node) throws InvalidLayoutException {
        // Reset the numbering (e.g. naming) of the tiles
        Tile.numberOfTiles = 1;
        allTiles = new ArrayList<Tile>();

        initStructure(node);

        Iterator<IRoutingNode> iter = node.getRoutingChildren();
        while(iter.hasNext()) {
        	IRoutingNode child = iter.next();
            Tile tile = insertNode(child);
            if(tile == null) {
            	throw new InvalidLayoutException(child + " can't be added to the tile structure");
            }
        }
    }

    /**
     * Constructor to build a new corner stitching structure for a connection
     * (e.g. for the parent node of the connection).
     * 
     * @param line the line the structure has to be constructed for
     *  
     */
    public TileStructure(IRoutingLine line) throws InvalidLayoutException {
        // Reset the numbering for the naming of the tiles
        Tile.numberOfTiles = 1;
        allTiles = new ArrayList<Tile>();

        IRoutingNode source = line.getRoutingSource();
        IRoutingNode target = line.getRoutingTarget();
                
        parentNode = HierarchyUtilities.getInstance().commonAncestor(source, target);
        
        if(parentNode == null) {
        	logger.warning("The source and the target node don't have a common anchestor!");
        }
        
        initStructure(parentNode);
        addInvolvedNodes(line);

    }

    /**
     * Insert a node into the corner stitching structure. The method doesn't
     * check if there's enough free space to insert the node. This has to be
     * done by the caller. The method returns the tile that is occupied by the
     * inserted node.
     * 
     * @param node
     *            the node that had to be inserted into the structure
     * @return the tile occupied by the node node
     *  
     */
    protected Tile insertNode(IRoutingNode node) throws InvalidLayoutException  {
  
    	Rectangle relativeBounds = HierarchyUtilities.getInstance().getRelativeBounds(node, parentNode);
    	
    	Point topLeftCorner = relativeBounds.getLocation();
    	Point bottomRightCorner = new Point((int)relativeBounds.getMaxX(), (int)relativeBounds.getMaxY());
    	
    	
        NodeTile nt = new NodeTile(node, topLeftCorner, bottomRightCorner, null, 
        							 null, null, null, null, null, null, null);
        
        
        // Don't try to insert the nodes if they lie outside the bounds of the
        // parent node. This can happen during the zooming.
        if(topLeftCorner.x < 0 || topLeftCorner.y < 0 
        		|| bottomRightCorner.x > parentNode.getRoutingBounds().width 
        		|| bottomRightCorner.y > parentNode.getRoutingBounds().height) {
        		return null;
        }
        
        return insert(nt);      
        
    }
  
    
    /**
     * Returns a list that contains all the tiles in the structure. The order of the
     * tiles in the structure is undefined.
     * 
     * @return list containing all the tiles in the structure.
     */
    public Iterator<Tile> getAllTiles() {
    		
    		return allTiles.iterator();
    	
    }
    
    
    /**
     * Returns the number of tiles in the structure.
     * 
     * @return the number of tiles
     */
    public int getNumberOfTiles() {
    	
    	return allTiles.size();
    	
    }

    
    /**
     * Helper method to insert a tile into the structure. Method doesn't check
     * if there's enough free space available to insert the node.
     * 
     * @param newTile The tile that should be inserted
     * @return The tile occupied by the node node
     *  
     */
    protected Tile insert(Tile newTile) throws InvalidLayoutException {

        // Find the space tile that contains the top edge of the solid tile to
        // be inserted
        Tile topTile = findTileAtPoint(newTile.getTopLeft());
        
        if(topTile == null) {
        	String msg = "Top tile for " + newTile + " not found!";
        	throw new InvalidLayoutException(msg);
        }

        if (!topTile.isSpace()) {
        	String msg = "Top tile for " + newTile + " is a node tile!";
        	throw new InvalidLayoutException(msg);
        }

        Tile firstVerticalSplit, lastVerticalSplit;
        if (newTile.getTopLeft().y != topTile.getTopLeft().y) {
            Tile bottomSplit = topTile.splitHorizontal(newTile.getTopLeft().y);

            // The bottom part of the split tile is the first tile that has to
            // be split vertically
            firstVerticalSplit = bottomSplit;
            
            Tile cur = bottomSplit.getTl();
            while(cur != null && cur.hasToBeSplitted()
                  && newTile.getTopLeft().y != cur.getTopLeft().y) {
                cur.splitHorizontal(newTile.getTopLeft().y);
                cur = cur.getTl();
            }
            
            cur = bottomSplit.getTr();
            while(cur != null && cur.hasToBeSplitted()
                  && newTile.getTopLeft().y != cur.getTopLeft().y) {
                cur.splitHorizontal(newTile.getTopLeft().y);
                cur = cur.getTr();
            }
            
            allTiles.add(bottomSplit);

        }

        // This means that we can't split the space tile because the vertical
        // position of
        // the solid tile to be inserted is equal to the vertical position of
        // the tile to be split.
        else {
            // We don't have to adjust the left-bottom and right-bottom corner
            // stitches of
            // the top neighbors even though they change.
            // The adjustment is done after the insertion of the new solid tile.

            firstVerticalSplit = topTile;
        }

        /* */

        // Find the space tile that contains the bottom edge of the solid tile
        // to be inserted
        Tile bottomTile = findTileAtPoint(newTile.getBottomRight());

        if (!bottomTile.isSpace()) {
            String msg = "Bottom tile for " + newTile + " is a node tile!";
        	throw new InvalidLayoutException(msg);
        }

        if (newTile.getBottomRight().y != bottomTile.getBottomRight().y) {
            Tile bottomSplit = bottomTile.splitHorizontal(newTile.getBottomRight().y + 1);
            
            Tile cur = bottomSplit.getTl();
            while(cur != null && cur.hasToBeSplitted()
                  && newTile.getBottomRight().y + 1 != cur.getTopLeft().y) {
                cur.splitHorizontal(newTile.getBottomRight().y + 1);
                cur = cur.getTl();
            }
            
            cur = bottomSplit.getTr();
            while(cur != null && cur.hasToBeSplitted()
                  && newTile.getBottomRight().y + 1 != cur.getTopLeft().y) {
                cur.splitHorizontal(newTile.getBottomRight().y + 1);
                cur = cur.getTr();
            }
            
            allTiles.add(bottomSplit);
            
        }

        lastVerticalSplit = bottomTile;

        /* */

        // Adjust the left-top and right-top corner stitch of the new solid tile
        // We can't just take the topTile here because it's possible that the
        // top tile hasn't been split.
        Tile neighbour = firstVerticalSplit.getLt();
        while (neighbour != null) {
            if (newTile.getTopLeft().x >= neighbour.getTopLeft()
                    .x
                    && newTile.getTopLeft().x <= neighbour
                            .getBottomRight().x) {
                newTile.setLt(neighbour);
            }
            if (newTile.getBottomRight().x >= neighbour.getTopLeft()
                    .x
                    && newTile.getBottomRight().x <= neighbour
                            .getBottomRight().x) {
                newTile.setRt(neighbour);
            }
            neighbour = neighbour.getBr();
        }

        // Adjust the left-bottom and right-bottom corner stitch of the new
        // solid tile
        // We can't just take the bottomTile here because it's possible that the
        // bottom tile hasn't been split.
        neighbour = lastVerticalSplit.getLb();
        while (neighbour != null) {
            if (newTile.getTopLeft().x >= neighbour.getTopLeft()
                    .x
                    && newTile.getTopLeft().x <= neighbour
                            .getBottomRight().x) {
                newTile.setLb(neighbour);
            }
            if (newTile.getBottomRight().x >= neighbour.getTopLeft()
                    .x
                    && newTile.getBottomRight().x <= neighbour
                            .getBottomRight().x) {
                newTile.setRb(neighbour);
            }
            neighbour = neighbour.getTr();
        }

        // Adjust the left-bottom and right-bottom corner stitches of the top
        // neighbors
        neighbour = firstVerticalSplit.getLt();
        while (neighbour != null) {
            if (neighbour.getTopLeft().x >= newTile.getTopLeft()
                    .x
                    && neighbour.getTopLeft().x <= newTile
                            .getBottomRight().x) {
                neighbour.setLb(newTile);
            }
            if (neighbour.getBottomRight().x >= newTile.getTopLeft()
                    .x
                    && neighbour.getBottomRight().x <= newTile
                            .getBottomRight().x) {
                neighbour.setRb(newTile);
            }
            neighbour = neighbour.getBr();
        }

        // Adjust the left-top and right-top corner stitches of the bottom
        // neighbors
        neighbour = lastVerticalSplit.getLb();
        while (neighbour != null) {
            if (neighbour.getTopLeft().x >= newTile.getTopLeft()
                    .x
                    && neighbour.getTopLeft().x <= newTile
                            .getBottomRight().x) {
                neighbour.setLt(newTile);
            }
            if (neighbour.getBottomRight().x >= newTile.getTopLeft()
                    .x
                    && neighbour.getBottomRight().x <= newTile
                            .getBottomRight().x) {
                neighbour.setRt(newTile);
            }
            neighbour = neighbour.getTr();
        }

        /*
         * Here starts the vertical splitting and horizontal merging of the
         * space tiles
         */

        // Split the space tiles to a left and right part and merge vertical
        // adjacent space tiles if possible
        Tile currentSplit = firstVerticalSplit;
        while (currentSplit != null) {
            // Find the next space tile that has to be split vertically.
            // This is necessary because the direct following of the corner
            // stitch can lead to 
            // a horizontal misalignment.
            Tile nextTile = currentSplit.getLb();
            while (nextTile != null
                    && !(nextTile.getTopLeft().x <= newTile
                            .getTopLeft().x && nextTile.getBottomRight()
                            .x >= newTile.getBottomRight().x)) {
                nextTile = nextTile.getTr();
            }

            // Check if there indeed exists a space tile to the right of the new
            // solid tile
            if (newTile.getBottomRight().x != currentSplit.getBottomRight().x) {
                // This is the new space tile to the right of the new solid tile               
                Tile rightSplit = (Tile)currentSplit.clone();
                rightSplit.setTopLeft(new Point(newTile.getBottomRight().x + 1, 
                		currentSplit.getTopLeft().y));
                rightSplit.setBottomRight(currentSplit.getBottomRight());
                rightSplit.setTl(newTile);
                rightSplit.setTr(currentSplit.getTr());
                rightSplit.setBr(currentSplit.getBr());
                rightSplit.setBl(newTile);
                
                allTiles.add(rightSplit);

                // Adjust the left-bottom and right-bottom corner stitches of
                // the top neighbors
                neighbour = currentSplit.getRt();
                while (neighbour != null) {
                    if (neighbour.getTopLeft().x >= rightSplit.getTopLeft().x
                            && neighbour.getTopLeft().x <= rightSplit
                                    .getBottomRight().x) {
                        neighbour.setLb(rightSplit);
                    }
                    if (neighbour.getBottomRight().x >= rightSplit
                            .getTopLeft().x
                            && neighbour.getBottomRight().x <= rightSplit
                                    .getBottomRight().x) {
                        neighbour.setRb(rightSplit);
                    }
                    neighbour = neighbour.getBl();
                }

                // Adjust the left-top and right-top corner stitches of the
                // bottom neighbors
                neighbour = currentSplit.getRb();
                while (neighbour != null) {
                    if (neighbour.getTopLeft().x >= rightSplit
                            .getTopLeft().x
                            && neighbour.getTopLeft().x <= rightSplit
                                    .getBottomRight().x) {
                        neighbour.setLt(rightSplit);
                    }
                    if (neighbour.getBottomRight().x >= rightSplit
                            .getTopLeft().x
                            && neighbour.getBottomRight().x <= rightSplit
                                    .getBottomRight().x) {
                        neighbour.setRt(rightSplit);
                    }
                    neighbour = neighbour.getTl();
                }

                // Adjust the left-top and right-top corner stitches of the new
                // space tile
                neighbour = currentSplit.getRt();
                while (neighbour != null) {
                    if (rightSplit.getTopLeft().x >= neighbour
                            .getTopLeft().x
                            && rightSplit.getTopLeft().x <= neighbour
                                    .getBottomRight().x) {
                        rightSplit.setLt(neighbour);
                    }
                    if (rightSplit.getBottomRight().x >= neighbour
                            .getTopLeft().x
                            && rightSplit.getBottomRight().x <= neighbour
                                    .getBottomRight().x) {
                        rightSplit.setRt(neighbour);
                    }
                    neighbour = neighbour.getBl();
                }

                // Adjust the left-bottom and right-bottom corner stitches of
                // the new space tile
                neighbour = currentSplit.getRb();
                while (neighbour != null) {
                    if (rightSplit.getTopLeft().x >= neighbour
                            .getTopLeft().x
                            && rightSplit.getTopLeft().x <= neighbour
                                    .getBottomRight().x) {
                        rightSplit.setLb(neighbour);
                    }
                    if (rightSplit.getBottomRight().x >= neighbour
                            .getTopLeft().x
                            && rightSplit.getBottomRight().x <= neighbour
                                    .getBottomRight().x) {
                        rightSplit.setRb(neighbour);
                    }
                    neighbour = neighbour.getTl();
                }

                // Adjust the top-left and bottom-left corner stitches of the
                // right neighbors
                neighbour = currentSplit.getTr();
                while (neighbour != null) {
                    if (neighbour.getTopLeft().y >= rightSplit
                            .getTopLeft().y
                            && neighbour.getTopLeft().y <= rightSplit
                                    .getBottomRight().y) {
                        neighbour.setTl(rightSplit);
                    }
                    if (neighbour.getBottomRight().y >= rightSplit
                            .getTopLeft().y
                            && neighbour.getBottomRight().y <= rightSplit
                                    .getBottomRight().y) {
                        neighbour.setBl(rightSplit);
                    }
                    neighbour = neighbour.getLb();
                }

                // The top-left and bottom-left corner stitches point always to
                // the new solid tile
                // so we don't have to check the left site explicitly.

                // Adjust the top-right and bottom-right corner stitches of the
                // new solid tile
                // if this is the first or last space tile that was split
                // horizontally.
                if (currentSplit == firstVerticalSplit) {
                    newTile.setTr(rightSplit);
                }
                if (currentSplit == lastVerticalSplit) {
                    newTile.setBr(rightSplit);
                }

                // Try to merge the current space tile with it's bottom
                // neighbor
                // This is necessary if there's no space between two solid tiles
                
                Tile lb = rightSplit.getLb();
                if(rightSplit.mergeWith(rightSplit.getLb())) {
                		allTiles.remove(lb);
                }
                Tile cur = currentSplit.getTr();
                while(cur != null && (lb = cur.getLb()) != null && cur.mergeWith(cur.getLb())) {
                	   allTiles.remove(lb);
                    cur = cur.getTr();
                }

                // Try to merge the current space tile with it's top neighbor
                if(rightSplit.getLt() != null) {
                		
                		if(rightSplit.getLt().mergeWith(rightSplit)) {
                			allTiles.remove(rightSplit);
                		}                		
                		cur = currentSplit.getTr();                
                		while(cur != null && cur.getLt() != null && cur.getLt().mergeWith(cur)) {
                			allTiles.remove(cur);
                			cur = cur.getLt().getTr();
                		}
                }

            }

            // This means that there is not enough place for a space tile to the
            // right of the new
            // solid tile.
            else {
                // Adjust the top-left and bottom-left corner stitches of the
                // right neighbors
                neighbour = currentSplit.getTr();
                while (neighbour != null) {
                    if (neighbour.getTopLeft().y >= currentSplit
                            .getTopLeft().y
                            && neighbour.getTopLeft().y <= currentSplit
                                    .getBottomRight().y) {
                        neighbour.setTl(newTile);
                    }
                    if (neighbour.getBottomRight().y >= currentSplit
                            .getTopLeft().y
                            && neighbour.getBottomRight().y <= currentSplit
                                    .getBottomRight().y) {
                        neighbour.setBl(newTile);
                    }
                    neighbour = neighbour.getLb();
                }

                // Adjust the top-right and bottom-right corner stitches of the
                // new solid tile
                // if this is the first or last space tile that was split
                // horizontally.
                if (currentSplit == firstVerticalSplit) {
                    newTile.setTr(currentSplit.getTr());
                }
                if (currentSplit == lastVerticalSplit) {
                    newTile.setBr(currentSplit.getBr());
                }

            }

            // Check if there indeed exists a space tile to the left of the new
            // solid tile
            if (newTile.getTopLeft().x != currentSplit.getTopLeft().x) {
                // Adjust the size of the current space tile so that it's now
                // the left part of the split.
                currentSplit.setBottomRight(new Point(newTile.getTopLeft().x - 1, 
                		currentSplit.getBottomRight().y));

                // Adjust the top-right and bottom-right corner stitches to the
                // new solid tile
                currentSplit.setTr(newTile);
                currentSplit.setBr(newTile);

                // Adjust the right-top corner stitch of the left space tile
                // We don't have to adjust the left-top one because it stays the
                // same
                neighbour = currentSplit.getLt();
                while (neighbour != null) {
                    if (currentSplit.getBottomRight().x >= neighbour
                            .getTopLeft().x
                            && currentSplit.getBottomRight().x <= neighbour
                                    .getBottomRight().x) {
                        currentSplit.setRt(neighbour);
                    }
                    neighbour = neighbour.getBr();
                }

                // Adjust the right-bottom corner stitch of the left space tile
                // We don't have to adjust the left-bottom one because it stays
                // the same
                neighbour = currentSplit.getLb();
                while (neighbour != null) {
                    if (currentSplit.getBottomRight().x >= neighbour
                            .getTopLeft().x
                            && currentSplit.getBottomRight().x <= neighbour
                                    .getBottomRight().x) {
                        currentSplit.setRb(neighbour);
                    }
                    neighbour = neighbour.getTr();
                }

                // Adjust the top-left and bottom-left corner stitches of the
                // new solid tile
                // if this is the first or last space tile that was split
                // horizontally.
                if (currentSplit == firstVerticalSplit) {
                    newTile.setTl(currentSplit);
                }
                if (currentSplit == lastVerticalSplit) {
                    newTile.setBl(currentSplit);
                }

                // Try to merge the current space tile with it's bottom neighbor.
                // This is necessary if there's no space between two solid tiles
                Tile rb = currentSplit.getRb();
                if(currentSplit.mergeWith(rb)) {
                		allTiles.remove(rb);
                }
                Tile cur = currentSplit.getTl();
                while(cur != null && (rb = cur.getRb()) != null && cur.mergeWith(cur.getRb())) {
//                	   allTiles.remove(rb);
                    cur = cur.getTl();
                }

                // Try to merge the current tile with it's top neighbor
                if(currentSplit.getRt() != null) {
                		if(currentSplit.getRt().mergeWith(currentSplit)) {
                			allTiles.remove(currentSplit);
                		}                		
                		cur = currentSplit.getTl();
                		while(cur != null && cur.getRt() != null && cur.getRt().mergeWith(cur)) {
//                			allTiles.remove(cur);
                			cur = cur.getRt().getTl();
                		}
                }

            }

            // This means that there is not enough place for a space tile to the
            // left of the new
            // solid tile.
            else {

                // Adjust the top-right and bottom-right corner stitches of the
                // left neighbor
                neighbour = currentSplit.getTl();
                while (neighbour != null) {
                    if (neighbour.getTopLeft().y >= currentSplit
                            .getTopLeft().y
                            && neighbour.getTopLeft().y <= currentSplit
                                    .getBottomRight().y) {
                        neighbour.setTr(newTile);
                    }
                    if (neighbour.getBottomRight().y >= currentSplit
                            .getTopLeft().y
                            && neighbour.getBottomRight().y <= currentSplit
                                    .getBottomRight().y) {
                        neighbour.setBr(newTile);
                    }
                    neighbour = neighbour.getRb();
                }

                // Adjust the top-left and bottom-left corner stitches of the
                // new solid tile
                // if this is the first or last space tile that was split
                // horizontally.
                if (currentSplit == firstVerticalSplit) {
                    newTile.setTl(currentSplit.getTl());
                }
                if (currentSplit == lastVerticalSplit) {
                    newTile.setBl(currentSplit.getBl());
                }
                
                allTiles.remove(currentSplit);
            }

            // Check if this was the last space tile we had to split horizontally
            if (currentSplit == lastVerticalSplit) {
                currentSplit = null;
            }
            // Go on with the next tile
            else {
                currentSplit = nextTile;
            }
        }

        allTiles.add(newTile);
        
        if(newTile.getTopLeft().x == 0 && newTile.getTopLeft().y == 0) {
        	topLeftTile = newTile;
        }
        
        return newTile;

    }


    /**
     * This method returns the tile at the position given as the parameter
     * point. The tile can either be a space tile or a tile that is occupied by
     * a node.
     * 
     * @param point the relative position inside the structure
     * @return the tile at the position given by point
     * @throws InvalidLayoutException 
     *  
     */
    public Tile findTileAtPoint(Point point)  {

    	if(point.x < 0 || point.x > width || point.y < 0 || point.y > height) {
    		
    		String msg = point + " lies outside the tile structure.";
    		logger.warning(msg);
    		return null;
    		
    	}
    	
    	
        Tile currentTile = topLeftTile;
        
        while (!(currentTile.getTopLeft().x <= point.x
                && currentTile.getBottomRight().x >= point.x
                && currentTile.getTopLeft().y <= point.y && currentTile
                .getBottomRight().y >= point.y)) {
        	
            // move up or down
            while (!(currentTile.getTopLeft().y <= point.y && currentTile
                    .getBottomRight().y >= point.y)) {
            	
                if (currentTile.getBottomRight().y < point.y) {
                		currentTile = currentTile.getLb();
                }
                    
                else {
                		currentTile = currentTile.getLt();
                }         
                
            }

            // move left or right
            while (!(currentTile.getTopLeft().x <= point.x && currentTile
                    .getBottomRight().x >= point.x)) {
            	
                if (currentTile.getBottomRight().x < point.x) {
                	currentTile = currentTile.getTr();
                }
                
                else {
                	currentTile = currentTile.getTl();
                }
                
            }

        }
       
        return currentTile;
    }
    

    /**
     * Returns the width of this tile structure.
     * 
     * @return the width of the tile structure
     *  
     */
    public int getWidth() {
        return width;
    }

    /**
     * Returns the height of the tile structure.
     * 
     * @return the height of the tile structure
     *  
     */
    public int getHeight() {
        return height;
    }

    /**
     * Helper method to initialize this corner stitching structure. Adds the
     * first space tile to the structure.
     * 
     * @param node the parent node of the tile structure
     *  
     */
    private void initStructure(IRoutingNode node) {
        parentNode = node;

        width = node.getRoutingBounds().width;
        height = node.getRoutingBounds().height;

        Point topLeft = new Point(0, 0);
        Point bottomRight = new Point(width, height);

        topLeftTile = new Tile(topLeft, bottomRight, null, null, null, null,
                null, null, null, null);
        
        allTiles.add(topLeftTile);
    }

    /**
     * Helper method to add all the nodes that could be an obstacle for the
     * routing of the line connection to the tile structure.
     * 
     * @param line the line the nodes could intersect
	 *
     */
    protected void addInvolvedNodes(IRoutingLine line) throws InvalidLayoutException {

		IRoutingNode from = line.getRoutingSource();
		IRoutingNode to = line.getRoutingTarget();

		// We have to split the start node of a reflexive node in to nodes so
        // that the start and end node are not the same.
		if (to == from) {

			Point start = line.getRoutingStartPoint();
			Point end = line.getRoutingEndPoint();

			int x = Math.abs(start.x - end.x) / 2;
			int y = Math.abs(start.y - end.y) / 2;

			// Split the tile vertical
			if (x > y) {

				x += from.getRoutingBounds().x + Math.min(start.x, end.x);
				
				DummyNode r1 = new DummyNode(from.getRoutingParent());

				Point leftPoint = new Point(x - 1, (int)from.getRoutingBounds().getMaxY());
				Point rightPoint = new Point(x + 2, from.getRoutingBounds().y);
					
				r1.setBounds(from.getRoutingBounds().getLocation(),	leftPoint);
				insertNode(r1);
				
				r1 = new DummyNode(from.getRoutingParent());

				// That's the middle (extreme expensive) tile
				r1.setBounds(new Point(x, from.getRoutingBounds().y), 
						new Point(x + 1, (int)from.getRoutingBounds().getMaxY()));
				insertNode(r1);

				// Insert the same IFigure for a second time so that the two
                // node tiles seem to be occupied by the same figure.
				// This doesn't have any side effects because the tiles store
				// their size themselves and just hold a reference to the
                // IRoutingNode.
				r1.setBounds(rightPoint, new Point((int)from.getRoutingBounds().getMaxX(), (int)from.getRoutingBounds().getMaxY()));
				insertNode(r1);

			// Split horizontally
			} else {

				y += from.getRoutingBounds().y + Math.min(start.y, end.y);
				DummyNode r1 = new DummyNode(from.getRoutingParent());

				Point topPoint = new Point((int)from.getRoutingBounds().getMaxX(), y - 1);
				Point bottomPoint = new Point(from.getRoutingBounds().x, y + 2);

				r1.setBounds(from.getRoutingBounds().getLocation(),	topPoint);
				insertNode(r1);
				
				r1 = new DummyNode(from.getRoutingParent());

				// That's the middle (extreme expensive) tile
				r1.setBounds(new Point(from.getRoutingBounds().x, y), 
						new Point((int)from.getRoutingBounds().getMaxX(), y + 1));
				insertNode(r1);

				// Insert the same IFigure for a second time so that the two
                // node tiles seem to be occupied by the same IRoutingNode.
				// This doesn't have any side effects because the tiles store
				// their size themselves and just hold a reference to the
                // IRoutingNode.
				r1.setBounds(bottomPoint, new Point((int)from.getRoutingBounds().getMaxX(), (int)from.getRoutingBounds().getMaxY()));
				insertNode(r1);

			}

		} else {
			
			// Don't insert the source node if it is itself a line.
			if(!(from instanceof IRoutingLine)) {
				insertNode(from);
			}
			// Don't insert the target node if it is itself a line.
			if(!(to instanceof IRoutingLine)) {
				insertNode(to);
			}
		}

		Iterator<IRoutingNode> iter = parentNode.getRoutingChildren();
        
        // Calculate (top down) all the nodes, that the line could intersect
        // (depends on hierarchy)
        while (iter.hasNext()) {
            IRoutingNode child = iter.next();
            if (HierarchyUtilities.getInstance().isAncestorOf(child, from) || child == from) {
            	collectNodes(child, from);
            } else if (HierarchyUtilities.getInstance().isAncestorOf(child, to) || child == to) {
            	collectNodes(child, to);
            } else {
            	insertNode(child);
            }       
        }
        
    }

    /**
     * Helper method to collect recursively all the nodes by going down the
     * hierarchy from node start to node leaf and collecting on every level of
     * the hierarchy all the brothers.
     * 
     * @param start start point in the hierarchy
     * @param leaf end point in the hierarchy
     *  
     */
    protected void collectNodes(IRoutingNode start, IRoutingNode leaf) throws InvalidLayoutException {
        Iterator<IRoutingNode> iter = start.getRoutingChildren();
        while(iter.hasNext()) {
            IRoutingNode child = iter.next();
            if (HierarchyUtilities.getInstance().isAncestorOf(child, leaf) || child == leaf) {
                collectNodes(child, leaf);
            }
            // The children of the start or the end are not effected
            // Necessary for the orthogonal line (lines start in the center of
            // the node)
            else if (!HierarchyUtilities.getInstance().isAncestorOf(leaf, child)) {
                insertNode(child);
            }
        }
    }

    
    /**
     * Returns the parent IFigure of this structure (i.e. the IFigure this structure
     * is completely contained in).
     * 
     * @return the parent IFigure
     */
    public IRoutingNode getParent() {
        return parentNode;
    }

    
    /**
     * Returns the maximal rectangular space around the point given as argument. The
     * method tries to extend the bounds of the tile point p lies in into all 
     * directions until it hits an obstacle.
     * 
     * @param p the point around which space should be looked for
     * @return the maximal rectangular space 
     */
    public Rectangle getFreeSpace(Point p) {
    	
    	Tile t = findTileAtPoint(p);
    	Rectangle bounds = t.getBounds();
    	Tile current = t;
    	// Try to extend with tiles above
    	while(current.getLt() != null && current.getLt() == current.getRt()) {
    		current = current.getLt();
    		bounds.y = current.getTopLeft().y;
    		bounds.height += current.getHeight();
    		
    	}
    	current = t;
    	// Try to extend with tiles below
    	while(current.getLb() != null && current.getLb() == current.getRb()) {
    		current = current.getLb();
    		bounds.height += current.getHeight();
    	}
    	
    	// Because of the tile structure's organization as maximal horizontal strips,
    	// we don't have to extend to the left and right.
    	
    	return bounds;
    	
    }
    
    
    /**
     * Returns true if the space represented by the rectangle given as argument is
     * white space (i.e., not occupied by a node or any other graphical feature) and
     * false otherwise. The coordinates of the rectangle given as argument have to be
     * relative to the structure.
     * 
     * @param rect the space to check for (with relative coordinates)
     * @return true if white space, false otherwise
     */
    public boolean isFreeSpace(Rectangle rect) {
    	
    	if(rect.x < 0 || rect.y < 0 || rect.x > getWidth() || rect.y > getHeight()
    			|| rect.x + rect.width > getWidth() || rect.y + rect.height > getHeight()) {
    		return false;
    	}
    	
    	Tile t = findTileAtPoint(rect.getLocation());
    	while(t != null) {
    		if(t.getTopLeft().y > rect.y + rect.height) {
    			return true;
    		}
    		if(!t.isSpace() || t.getBottomRight().x < rect.x + rect.width) {
    			return false;
    		}
    		if(t.getBottomRight().y >= rect.y + rect.height) {
    			return true;
    		}
    		if(t.getBottomRight().y + 1 > getHeight()) {
    			return false;
    		}
    		
    		t = findTileAtPoint(new Point(rect.getLocation().x, t.getBottomRight().y + 1));
    	}
    	
    	return true;
    	
    }
    
}