package ch.unizh.ifi.adora.linerouting.utils;

import java.awt.Point;
import java.awt.Rectangle;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;

/**
 * Provides some utility functions to work on the IRoutingNode hierarchy.
 * This methods should actually be provided by the IRoutingNode but are in
 * a class of its own to keep the IRoutingNode interface small.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-03-21 TR First Version
 * @version				$Date: 2009-09-28 10:54:59 +0200 (Mon, 28 Sep 2009) $, $Author: treinhard $, $Revision: 4766 $
 *
 */
public class HierarchyUtilities {

	/**
	 * The singleton instance
	 */
	private static HierarchyUtilities instance;
	
	/**
	 * Hide the constructor because this is a singleton.
	 */
	private HierarchyUtilities() {
		
	}
	
	
	/**
	 * Returns the singleton instance.
	 * 
	 * @return the singleton instance
	 */
	public static HierarchyUtilities getInstance() {
		
		if(instance == null) {
			instance = new HierarchyUtilities();
		}
		
		return instance;
		
	}
	
	
	/**
	 * Returns true if the node <i>father<i> is a direct or indirect ancestor
	 * of the node <i>son</i>. The method returns false if father == son.
	 * 
	 * @param father the ancestor
	 * @param son the son
	 * @return true if father is an ancestor of son
	 */
    public boolean isAncestorOf(IRoutingNode father, IRoutingNode son) {
        return commonAncestor(son, father) == father;
    }
    
    
    /**
     * Returns the first common ancestor of the two nodes given as argument.
     * This method is symmetrical. The method returns null if the two nodes
     * don't have a common ancestor or node1 == node2 and node1 is the root
     * of the hierarchy.
     * 
     * @param node1
     * @param node2
     * @return the first common ancestor of the two nodes
     */
    public IRoutingNode commonAncestor(IRoutingNode node1, IRoutingNode node2) {
    	
    	if(node1 == node2) {
    		return node1.getRoutingParent();
    	}
    	
    	IRoutingNode anchestor = findAncestor(node1, node2);
    	
    	if(anchestor == null) {	
    		anchestor = findAncestor(node2, node1);
    	}
    	
    	return anchestor;
    	
    }
    
    
    private IRoutingNode findAncestor(IRoutingNode father, IRoutingNode son) {
        IRoutingNode result = null;
        IRoutingNode parent = father.getRoutingParent();
        if (father == son) {
            result = father;

        } else if (parent != null) {
            IRoutingNode res = findAncestor(parent, son);
            if (res == null && son.getRoutingParent() != null) {
                res = findAncestor(parent, son.getRoutingParent());
            }
            result = res;

        } else {
            result = null;
        }
        
        return result;
    }
	
	
    /**
     * Returns the bounds of the node <i>node</i> relative to the bounds of the
     * node <i>reference</i>. The method returns null if the reference is not an
     * ancestor of the node.
     * 
     * @param node
     * @param reference
     * @return the relative bounds (null if reference isn't an ancestor of node) 
     */
	public Rectangle getRelativeBounds(IRoutingNode node, IRoutingNode reference) {
		
		if(reference == node) {
			return node.getRoutingBounds();
		}
		
		if(isAncestorOf(reference, node)) {
			
			Rectangle bounds = new Rectangle(node.getRoutingBounds());
			
			IRoutingNode current = node.getRoutingParent();
			while(current != reference) {
				bounds.setLocation(bounds.x + current.getRoutingBounds().x, bounds.y + current.getRoutingBounds().y);
				current = current.getRoutingParent();
			}
			
			return bounds;
			
		} else {
			return null;
		}
		
	}
	
	
	/**
	 * Returns the absolute location of the node given as argument. The
	 * method returns the node's location if the node doesn't belong to a
	 * hierarchy.
	 * 
	 * @param node the node to obtain the absolute location of
	 * @return the absolute location of the node
	 */
	public Point getAbsoluteLocation(IRoutingNode node) {
		
		Point location = node.getRoutingBounds().getLocation();
		
		while(node.getRoutingParent() != null) {
			
			node = node.getRoutingParent();
			location.translate(node.getRoutingBounds().x, node.getRoutingBounds().y);
			
		}
		
		return location;
		
	}
	
	
}
