package ch.unizh.ifi.adora.linerouting.utils;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * This class provides a simple sorted list data structure, that is used by the
 * tile based routing algorithms to find a path in the tile structure. The
 * elements in the heap have to implement the Observable interface. The methods
 * of this class aren't thread safe (e.g. synchronized)!
 * 
 * @author Tobias Reinhard
 * 
 * @copyright Requirements Engineering Research Group 
 *            Department of Computer Science
 *            University of Zurich
 * 
 * @history 
 * 2005-05-11 TR First version
 * 2005-05-19 TR Additional check in the method add(...)
 * 2005-06-14 TR Small bugfix in the contains(...) method
 * 
 * @version $Date: 2008-08-06 09:54:18 +0200 (Wed, 06 Aug 2008) $, $Author: treinhard $, $Revision: 3777 $
 * 
 * @responsibilities Implementation of a sorted list data structure
 */
public class SortedList<E extends Comparable<? super E>> implements Collection<E> {

    /**
     * The elements in the list
     */
    private LinkedList<E> elements;
    
    
    /**
     * Constructor for the sorted list.
     * 
     * @pre true
     * @post true
     */
    public SortedList() {
        
        elements = new LinkedList<E>();
        
    }
    
    
    /**
     * Returns the number of elements that are currently in the list.
     * 
     * @return the number of elements in the list
     * 
     * @pre true
     * @post true
     */
    public int size() {

        return elements.size();
    }

    
    /**
     * Returns true if the list is empty (i.e. contains no elements), 
     * false otherwise.
     * 
     * @return true if the list is empty, false otherwise.
     * 
     * @pre true
     * @post true
     */
    public boolean isEmpty() {

        return elements.isEmpty();
    }

    
    /**
     * Returns true if this collection contains the specified  element. 
     * More formally, returns true if and only if this collection contains 
     * at least one element e such that  (o==null ? e==null : o.equals(e))
     * 
     * @param o element whose presence in this collection is to be tested.
     * @return true if object o is in thel list, false otherwise
     * 
     * @pre o != null
     * @post true
     */
    public boolean contains(Object o) {

        return elements.contains(o);
        
    }


    /**
     * Returns an iterator to traverse the objects in the list.
     * 
     * @return iterator to traverse the objects
     * 
     * @pre true
     * @post true
     */
    public Iterator<E> iterator() {

        return elements.iterator();
    }


    /**
     * Returns an array containing all of the elements in this list.
     * 
     * @return an array containing all of the elements in this collection
     * 
     * @pre true
     * @post true
     */
    public Object[] toArray() {

        return elements.toArray();
    }


    /**
     * Returns an array containing all of the elements in this collection; 
     * the runtime type of the returned array is that of the specified array. 
     * If the collection fits in the specified array, it is returned therein. 
     * Otherwise, a new array is allocated with the runtime type of the 
     * specified array and the size of this collection.
     * 
     * If this collection fits in the specified array with room to spare  
     * (i.e., the array has more elements than this collection), the element
     * in the array immediately following the end of the collection is set to
     * null. This is useful in determining the length of this  collection only
     * if the caller knows that this collection does  not contain any null elements.)
     * 
     * @param o the array into which the elements of this collection are to be
     *   stored, if it is big enough; otherwise, a new array of the same
     *   runtime type is allocated for this purpose.
     * 
     * @return an array containing the elements of this collection
     * 
     * @pre o != null
     * @post size() == o.length
     */
    public <T> T[] toArray(T[] o) {

        return elements.toArray(o);
    }


    /**
     * Ensures that this collection contains the specified element. 
     * Returns true if this collection changed as a  result of the call.
     * 
     * @param o the object to add
     * @return true if the addition was successfull
     * 
     * @pre o != null
     * @post this.size() == this@pre.size() + 1
     */
    public boolean add(E o) {

    	if(contains(o)) {
    		return false;
    	}
    	
        int index = Collections.binarySearch(elements, o);
        if(index == 0) {
            elements.add(0, o);
            return true;
        }
        if (index < 0) {
            index = -index;
        }
        
        elements.add(index-1, o);
        return true;
    }


    /**
     * Removes a single instance of the specified element from this
     * collection, if it is present. More formally, removes an element 
     * e such that (o==null ? e==null :  o.equals(e)), if this 
     * collection contains one or more such elements. Returns true if this 
     * collection contained the specified element (or equivalently, if 
     * this collection changed as a result of the  call)
     * 
     * @param o the object to remove
     * @return true if the object has been removed successfully
     * 
     * @pre o != null
     * @post this.size() == this@pre.size() - 1
     */
    public boolean remove(Object o) {

        return elements.remove(o);
    }


    /**
     * Returns true if this collection contains all of the elements
     * in the specified collection.
     * 
     * @param c collection to be checked for containment in this collection.
     * @return true if this collection contains all of the elements
     *   in the specified collection
     * 
     * @pre c != null
     * @post true
     */
    public boolean containsAll(Collection<?> c) {

        Iterator<?> iter = c.iterator();
        while(iter.hasNext()) {
            if(!contains(iter.next())) {
                return false;
            }
        }
        return true;
    }


    /**
     * Adds all of the elements in the specified collection to this collection
     * (optional operation). The behavior of this operation is undefined if
     * the specified collection is modified while the operation is in progress.
     * (This implies that the behavior of this call is undefined if the
     * specified collection is this collection, and this collection is  nonempty.)
     * 
     * @param c the collection containing all the elements to add
     * @return true if all the elements have been added successfully
     * 
     * @pre c != null
     * @post this.size() == this@pre.size() + c.size()
     */
    public boolean addAll(Collection<? extends E> c) {

        Iterator<? extends E> iter = c.iterator();
        while(iter.hasNext()) {
            add(iter.next());
        }
        return true;
    }


    /**
     * Removes all this collection's elements that are also contained in the
     * specified collection. After this call returns, this collection will 
     * contain no elements in common with the specified collection.
     * 
     * @param c elements to be removed from this collection.
     * @return true if this collection changed as a result of the call
     * 
     * @pre c != null
     * @post true
     */
    public boolean removeAll(Collection<?> c) {

        Iterator<?> iter = c.iterator();
        while(iter.hasNext()) {
            remove(iter.next());
        }
        return true;
        
    }


    /**
     * Retains only the elements in this collection that are contained in the
     * specified collection. In other words, removes from this collection all
     * of its elements that are not contained in the specified collection.
     * 
     * @param elements to be retained in this collection.
     * @return true if this collection changed as a result of the call
     * 
     * @pre c != null
     * @post true
     */
    public boolean retainAll(Collection<?> c) {

        return elements.retainAll(c);
    }


    /**
     * Removes all of the elements from this collection. This collection will
     * be empty after this method returns unless it throws an exception.
     * 
     * @pre true
     * @post this.size() == 0
     */
    public void clear() {
        elements.clear();
    }
    
    
    /**
     * Returns the first element of the sorted list.
     * 
     * @return the first element
     * 
     * @pre true
     * @post true
     */
    public Object first() {
        return elements.get(0);
    }

}
