package ch.unizh.ifi.adora.productline.ui.views.constraintstable;

import java.io.StringBufferInputStream;
import java.util.HashMap;
import java.util.regex.Pattern;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.widgets.TableItem;
import org.sat4j.specs.ISolver;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.SetConstraintsTableListAntecedent;
import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.parser.ConstraintParser;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.SatSolver;
import ch.unizh.ifi.adora.ui.constrainthint.*;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * This class is the {@link ICellModifier} for the {@link VariationPointViewPart}.
 * It returns, if a column can be modified, what the current value is and it
 * modifies the entry.
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-10-16 RS First Version
 * @version 
 */
public class ConstraintsCellModifier implements ICellModifier {
	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
     *      java.lang.String)
     */
	public boolean canModify(Object element, String property) {
		
		if (property.equals(ConstraintsViewPart.COL_OPERATION)) {
			return true;
		}

		if (property.equals(ConstraintsViewPart.COL_DESCRIPTION)) {
			return true;
		}

		if (property.equals(ConstraintsViewPart.COL_ANTECEDENT)
				|| property.equals(ConstraintsViewPart.COL_CONSEQUENT)) {
			return true;
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
     *      java.lang.String)
     */
	public Object getValue(Object element, String property) {
		Constraint constraint = (Constraint) element;
		if (property.equals(ConstraintsViewPart.COL_DESCRIPTION)) {
			return constraint.getDescription();
		}

		if (property.equals(ConstraintsViewPart.COL_ANTECEDENT)) {
			return constraint.getAntecedent();
		}
		
		if (property.equals(ConstraintsViewPart.COL_CONSEQUENT)) {
			return constraint.getConsequent();
		}
		
		if (property.equals(ConstraintsViewPart.COL_OPERATION)) {
			if (constraint.getOperation().equals("=>"))
				return 0;
			return 1;
		}

		return null;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
     *      java.lang.String, java.lang.Object)
     */
	public void modify(Object element, String property, Object value) {
		Command command = null;
		Constraint constraint = (Constraint) ((TableItem) element).getData();

		if (property.equals(ConstraintsViewPart.COL_DESCRIPTION)) {
			try {
				command = new SetProperty(constraint, value, constraint
						.getClass().getMethod("setDescription",
								new Class[] { String.class }), constraint
						.getClass().getMethod("getDescription", new Class[] {}));
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}
		
		if (property.equals(ConstraintsViewPart.COL_ANTECEDENT)) {
			
			if (!isConstraintValid((String)value))
					return;
			
			command = new SetConstraintsTableListAntecedent((String) value, constraint);
		}

		if (property.equals(ConstraintsViewPart.COL_OPERATION)) {
			String newVal = "";
			if ((Integer) value == 0) 
				newVal = "=>";
			else
				newVal = "<=>";
			try {
				command = new SetProperty(constraint, newVal, constraint
						.getClass().getMethod("setOperation",
								new Class[] { String.class }), constraint
						.getClass().getMethod("getOperation", new Class[] {}));
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}
		
		if (property.equals(ConstraintsViewPart.COL_CONSEQUENT)) {
			
			if (!isConstraintValid((String)value))
				return;

			try {
				command = new SetProperty(constraint, value, constraint
						.getClass().getMethod("setConsequent",
								new Class[] { String.class }), constraint
						.getClass().getMethod("getConsequent", new Class[] {}));
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}

		if (command != null) {
			ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getCommandStack().execute(command);
		}

		//Update Constraints
		AdoraModel model = ModelObserverMediator.getInstance()
						.getAdoraModelEditor().getModel();

		if (!property.equals(ConstraintsViewPart.COL_DESCRIPTION)) { 
			new SatSolver(model).solve();
		}
	}
	
	/**
	 * Check if a given logical expression is valid.
	 * An expression is valid <=> its syntax is ok and it only involves
	 * {@link Decisions} which actually exist.
	 * 
	 * @param expr The expression to be checked.
	 * @return True if expr is a valid expression
	 */
	private boolean isConstraintValid(String expr) {
		
		String errorDec 	= "The expression involves a nonexisting Decision.";
		String errorSyntax 	= "The expression contains syntax errors.";
				
		//check if Decision actually occurs in model.
        Pattern p = Pattern.compile("(D)[0-9]+");
        java.util.regex.Matcher m = p.matcher(expr);
        HashMap<Integer, Decision> map = AdoraPlugin.getActiveModel().getDecisionsMap();
        
        while (m.find()) {
            String[] splitted = m.group().split("D");
            if (map.get(Integer.parseInt(splitted[1])) == null) {
            	ConstraintHint.displayConstraintHint(errorDec);
            	return false;
            }
        }
        
    	AdoraModel model = ModelObserverMediator.getInstance()
								.getAdoraModelEditor().getModel();

    	
        //check syntax
        ISolver solver = new ConstraintParser(new StringBufferInputStream(expr
											.concat("\n")), null, false, model).satParser();
        if (solver == null) {
        	ConstraintHint.displayConstraintHint(errorSyntax);
        	return false;
        }
        ConstraintHint.displayConstraintHint("");
    	return true;
	}
}