package ch.unizh.ifi.adora.productline.ui.views.constraintstable;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.model.commands.ConstraintAdd;
import ch.unizh.ifi.adora.model.commands.ConstraintRemove;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.SatSolver;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.AdoraModelEditorPaletteFactory;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointContentProvider;

/**
 * This class is the view, which is used to create and maintain
 * the <b>Constraints</b> for the Software Product Line Platform/Domain.
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-10-16 RS First Version
 * @version 
 */
public class ConstraintsViewPart extends ViewPart {
	
	/**
     * The ID of this view.
     */
	public static final String VIEW_ID = "ch.unizh.ifi.adora.productline.views.Constraints";

	/**
     * The identifier of the Nr Column of this tableview.
     */
	protected static final String COL_NR = "NR";

	/**
     * The identifier of the Description Column of this tableview.
     */
	protected static final String COL_DESCRIPTION = "Description";

	/**
     * The first term for the constraint, called the <i>antecedent</i>.
     */
	protected static final String COL_ANTECEDENT = "Antecedent";

	/**
     * The <i>logical operation</i> (function) between antecedent and consequent. 
     * This is per default a logical implication (e.g. A implies B);
     */
	protected static final String COL_OPERATION = "LogicalOperation";

	/**
     * The second term for the constraint, called the <i>consequent</i> of 
     * the logical operation.
     */
	protected static final String COL_CONSEQUENT = "Consequent";

	/**
     * The {@link TableViewer} used.
     */
	private TableViewer viewer;

	/**
     * The {@link VariationPointContentProvider} used.
     */
	private ConstraintsContentProvider contentProvider;
	
	/**
	 * Toolbar Actions
	 */
	private Action deleteConstraintAction;
	private Action addConstraintAction;
	
	/**
	 * The Action used to show the Variation Points associated with the selected Decisions.
	 */
	private Action showConstraintCon;

	
	private AdoraModel adoraModel;

	/**
     * The constructor.
     */
	public ConstraintsViewPart() {
	}
	
	/**
     * @see IViewPart.init(IViewSite)
     */
	@Override
	public void init(IViewSite site) throws PartInitException {
            super.init(site);
            // Normally we might do other stuff here.
    }


	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
	@Override
	public void createPartControl(Composite parent) {
		
		this.addChildControls(parent);

		Table table = new Table(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		TableLayout layout = new TableLayout();
		layout.addColumnData(new ColumnWeightData(2, true)); // number
		layout.addColumnData(new ColumnWeightData(20, true)); // description
		//TODO: type-checking for these values ...!!!!!!!!!
		layout.addColumnData(new ColumnWeightData(15, true)); // antecedent
		layout.addColumnData(new ColumnWeightData(3, true)); // logical operation
		layout.addColumnData(new ColumnWeightData(15, true)); // consequent

		table.setLayout(layout);

		TableColumn nrCol = new TableColumn(table, SWT.NONE);
		nrCol.setText(COL_NR);
		nrCol.setResizable(true);

		TableColumn descriptionCol = new TableColumn(table, SWT.NONE);
		descriptionCol.setText(COL_DESCRIPTION);
		descriptionCol.setResizable(true);

		TableColumn minCardCol = new TableColumn(table, SWT.NONE);
		minCardCol.setText(COL_ANTECEDENT);
		minCardCol.setResizable(true);

		TableColumn maxCardCol = new TableColumn(table, SWT.NONE);
		maxCardCol.setText(COL_OPERATION);
		maxCardCol.setResizable(true);

		TableColumn decisionsCol = new TableColumn(table, SWT.NONE);
		decisionsCol.setText(COL_CONSEQUENT);
		decisionsCol.setResizable(true);

		viewer = new TableViewer(table);
		contentProvider = new ConstraintsContentProvider(viewer, getSite()
				.getPage());
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(new ConstraintsLabelProvider());
		viewer.setInput(getViewSite());
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@SuppressWarnings("unchecked")
			public void selectionChanged(SelectionChangedEvent event) {
				
				//TODO: Implement the highlighting for 'all involved decisions' that are 
				//			part of the 'constraints currently selected'.
				
			}

			/**
             * Checks, if the list of selectedModelElements is already selected
             * in the editor.
             * 
             * @param ame
             * @param selectedModelElements
             * 
             * @return true if already selected.
             */
			private boolean alreadySelected(AdoraModelEditor ame, List selectedModelElements) {
				
				//TODO --- dito like before...
				
				return true; //this is not implemented yet
			}

		});

		CellEditor[] editors = new CellEditor[5];

		//Nr
		TextCellEditor nrEditor = new TextCellEditor(table);
		((Text) nrEditor.getControl()).setTextLimit(5);
		editors[0] = nrEditor;

		//description
		TextCellEditor descriptionEditor = new TextCellEditor(table);
		((Text) descriptionEditor.getControl()).setTextLimit(200);
		editors[1] = descriptionEditor;

		//antecedent
		TextCellEditor antecedentEditor = new TextCellEditor(table);
		((Text) antecedentEditor.getControl()).setTextLimit(60);
		editors[2] = antecedentEditor;

		//logical operation
		String[] choices = { "=>", "<=>"};
		ComboBoxCellEditor decision = new ComboBoxCellEditor(table, choices);
		editors[3] = decision;

		//consequent
		TextCellEditor consequentEditor = new TextCellEditor(table);
		((Text) consequentEditor.getControl()).setTextLimit(60);
		editors[4] = consequentEditor;

		String[] columnProperties = { COL_NR, COL_DESCRIPTION,
				COL_ANTECEDENT, COL_OPERATION, COL_CONSEQUENT };

		viewer.setColumnProperties(columnProperties);
		viewer.setCellEditors(editors);
		viewer.setCellModifier(new ConstraintsCellModifier());
	}

	/**
     * Returns the {@link VariationPointContentProvider} of this view.
     * 
     * @return The {@link VariationPointContentProvider} of this view.
     */
	public ConstraintsContentProvider getContentProvider() {
		return contentProvider;
	}

	/**
     * Passing the focus request to the viewer's control.
     */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	/**
	 * Refreshes this viewer completely with information freshly obtained from this viewer's model.
	 */
	public void refresh() {
		viewer.refresh();
	}
	
	
	
	/**
	 * creates the controls for the parent composite
	 * @param Composite parent
	 */
	private void addChildControls(Composite parent){

		createToolbarActions(parent);
		createToolBar();
		
	}
	
	/**
	 * creates the view toolbar and adds actions to it
	 */
	private void createToolBar(){
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();
		mgr.add(deleteConstraintAction);
		mgr.add(addConstraintAction);
		mgr.add(showConstraintCon);
		
	}
	
	/**
	 * creates the toolbar actions and sets the action image
	 */
	public void createToolbarActions (final Composite parent){		


		showConstraintCon = new Action("Show Constraints") {
			
			/*
			 * (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				//get selected  constraints
				//first hide all ConstraintConnections
				
				List<Constraint> selectedConst = new ArrayList<Constraint>();
				
				adoraModel = AdoraPlugin.getActiveModel();
				TableItem[] selection = viewer.getTable().getSelection();
			
				for (int i = 0; i < selection.length; i++) {
					int constNr = Integer.parseInt(selection[i].getText(0).substring(1));
					for (Constraint con : adoraModel.getConstraints()) {
						if (con.getNr() == constNr) 
							selectedConst.add(con);
					}
				}
				
				UpdateConstraintConnections constCreate = new UpdateConstraintConnections(adoraModel, selectedConst);
				AdoraPlugin.getActiveEditor().getCommandStack().execute(constCreate);
				
				return;
				
			}
		};
		
		
		deleteConstraintAction = new Action("DeleteConstraint") {
			
			@Override
			public void run() {
				
				adoraModel = AdoraPlugin.getActiveModel();
				
				
				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
//				ArrayList<VariationPoint> variationPointsToDelete = new ArrayList<VariationPoint>();
				
				System.out.println("Selection: "+viewer.getSelection().toString());
				
				ArrayList<Constraint> constraintsToRemove = new ArrayList<Constraint>();
				constraintsToRemove.clear();

				TableItem[] selection = viewer.getTable().getSelection();
				
				for (TableItem item : selection) {
					System.out.println("item: "+item.getData().toString());
					if (item.getData() instanceof Constraint) {
						System.out.println("added: "+((Constraint)item.getData()).toString());
						
						constraintsToRemove.add((Constraint) item.getData());

						if (((Constraint)item.getData()).getImplCon() != null) {
							ame.getCommandStack().execute(new ConnectionDelete(((Constraint)item.getData()).getImplCon()));
						}
						for (ConstraintConnection con : ((Constraint)item.getData()).getConnections()) {
							ame.getCommandStack().execute(new ConnectionDelete(con));
						}
					}
				}
				
				ConstraintRemove removeCommand = new ConstraintRemove(adoraModel, constraintsToRemove);
				ame.getCommandStack().execute(removeCommand);
				
				ConstraintsViewPart constraintViewPart = (ConstraintsViewPart) ame.getSite().getPage().findView(VIEW_ID);
				//to activate the variation points view...
				constraintViewPart.refresh();
				constraintViewPart.setFocus();

				AdoraModel model = ModelObserverMediator.getInstance()
										.getAdoraModelEditor().getModel();
				new SatSolver(model).solve();
				
				viewer.refresh();
			}
		};
		//deleteConstraintAction.setImageDescriptor(ImageDescriptor.createFromFile(AdoraPlugin.class, "icons/add.gif"));
		
		addConstraintAction = new Action("AddConstraint") {
			
			@Override
			public void run() {
				
				adoraModel = AdoraPlugin.getActiveModel();
				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
				
				System.out.println(ame.getCommandStack().getCommands().toString());
				
				if (adoraModel == null) System.out.println("severe fatal error!!");
				ConstraintAdd addCommand = new ConstraintAdd(adoraModel);
				ame.getCommandStack().execute(addCommand);
				System.out.println(ame.getCommandStack().getCommands().toString());

				ConstraintsViewPart constraintsViewPart = (ConstraintsViewPart) ame.getSite().getPage().findView("ch.unizh.ifi.adora.productline.views.Constraints");
				constraintsViewPart.refresh();
				constraintsViewPart.setFocus();

				viewer.refresh();
				
				System.out.println("the constraints: "+adoraModel.getConstraints().toString());
			}
		};
	}
	
	/**
	 * This method is responsible for enabling/disabling the {@link Constraint} creation /deletion buttons.
	 * 
	 * @param pdStart  true  => palette-elements get hidden
	 * 				   false => palette-elements get shown 
	 */
	public void updateProductDerivation(boolean derivationStarted) {
		deleteConstraintAction.setEnabled(!derivationStarted);
		addConstraintAction.setEnabled(!derivationStarted);
	}
}