package ch.unizh.ifi.adora.productline.ui.views.constraintstable;

import java.io.StringBufferInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.HideAllConstraintConnections;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.ConstraintImplicationConnection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.parser.ConstraintParser;


/**
 * This {@link Command} first deletes all visible {@link Constraint}s in the
 * model and then shows the selected {@link Constraint}s
 * in the ConstraintsViewPart.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-11-19 IV First Version
 * @version
 */
public class UpdateConstraintConnections extends Command {

	/**
	 * The AdoraModel.
	 */
	private AdoraModel adoraModel;

	/**
	 * The {@link List} of the selected {@link Constraint}s.
	 */
	private List<Constraint> constraints;

	/**
	 * Generates this {@link Command} out of selected {@link Constraint}s in the
	 * {@link ConstraintsViewPart}.
	 * 
	 * @param adoraModel
	 *            The AdoraModel.
	 * @param constraints
	 *            Contains a list of selected {@link Constraint}s
	 */
	public UpdateConstraintConnections(AdoraModel adoraModel,
			List<Constraint> constraints) {
		this.adoraModel = adoraModel;
		this.constraints = constraints;
	}

	/**
	 * Generates this {@link Command} out of selected {@link Constraint}s in the
	 * {@link ConstraintsViewPart}.
	 * 
	 * @param adoraModel2
	 *            The AdoraModel.
	 * @param selectedDecisions
	 *            Stores the selected {@link Decision}s.
	 */
	public UpdateConstraintConnections(AdoraModel adoraModel,
			ArrayList<Decision> selectedDecisions) {

		this.adoraModel = adoraModel;
		this.constraints = getSelectedConstraints(selectedDecisions);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	@Override
	@SuppressWarnings("deprecation")
	public void execute() {

		HideAllConstraintConnections hideAllCommand = new HideAllConstraintConnections(
				adoraModel);
		hideAllCommand.execute();

		ConnectableElement fromCon, toCon;

		for (Constraint con : constraints) {

			//here we take into account that some of the variables may already be set.
			
			String conAntec = con.getAntecedent().trim().concat("\n");
			
			ConstraintParser cp = new ConstraintParser(new StringBufferInputStream(conAntec), null, false, adoraModel);
			
			conAntec = cp.simplifyParser();
			
			if (conAntec.equals("false"))
				continue;

			if (!con.isRelevant())
				continue;

			// check to make constraint invisible if a decision is set to true
			
			con.setVisible(true);

			// ---Consequent--------------

			String conCons = con.getConsequent().concat("\n");
			
			conCons = new ConstraintParser(new StringBufferInputStream(conCons), null, false, adoraModel)
					.simplifyParser();
		
			toCon = new ConstraintParser(new StringBufferInputStream(conCons.concat("\n")), con, false, adoraModel)
					.connectionParser(); 
	
			if (toCon instanceof JoinRelation) {
				if (!((JoinRelation) toCon).isCalculatedAbstracted()) {
					Decision dec = ((JoinRelation) toCon).getDecision();
					if (dec.getFamilyLines().size() == 0)
						dec.generateFamilyLines();
				} else {
					String[] decisionStr = ((JoinRelation) toCon)
							.getAbstractedDecNrs().toLowerCase().split(",");
					for (int k = 0; k < decisionStr.length; k++) {
						if (adoraModel.getDecisionsMap().get(
								Integer.parseInt(decisionStr[k].trim()
										.substring(1))).getFamilyLines().size() == 0) {
							adoraModel.getDecisionsMap().get(
									Integer.parseInt(decisionStr[k].trim()
											.substring(1)))
									.generateFamilyLines();
						}
					}
				}
			} 

			// ---antecedent--------------
			if (conAntec.toLowerCase().trim().equals("true"))
				continue;
			
			fromCon = new ConstraintParser(new StringBufferInputStream(conAntec.concat("\n")), con, true, adoraModel)
					.connectionParser();

			if (fromCon instanceof JoinRelation) {
				if (!((JoinRelation) fromCon).isCalculatedAbstracted()) {
					Decision dec = ((JoinRelation) fromCon).getDecision();
					if (dec.getFamilyLines().size() == 0)
						dec.generateFamilyLines();
				} else {
					String[] decisionStr = ((JoinRelation) fromCon)
							.getAbstractedDecNrs().toLowerCase().split(",");
					for (int k = 0; k < decisionStr.length; k++) {
						if (adoraModel.getDecisionsMap().get(
								Integer.parseInt(decisionStr[k].trim()
										.substring(1))).getFamilyLines().size() == 0) {
							adoraModel.getDecisionsMap().get(
									Integer.parseInt(decisionStr[k].trim()
											.substring(1)))
									.generateFamilyLines();
						}
					}
				}
			}

			// ---implication---------------
			if (fromCon != toCon && fromCon != null && toCon != null
					&& fromCon.isVisible() && toCon.isVisible()) {
				ConstraintImplicationConnection implConn = new ConstraintImplicationConnection(
						fromCon, toCon, con.getOperation().equals("=>"), con
								.getNr());
				con.setImplCon(implConn);
			}
		}
	}

	private boolean evaluateConstraintVisibility(String antecedent,
			String consequent) {
		List<Integer> constraintDecisionNumbers = new ArrayList<Integer>();
		Pattern p = Pattern.compile("(D)[0-9]+");
		java.util.regex.Matcher m = p.matcher(antecedent.concat(consequent));
		while (m.find()) {
			String[] splitted = m.group().split("D");
			constraintDecisionNumbers.add(Integer.parseInt(splitted[1]));
		}

		for (int i = 0; i < constraintDecisionNumbers.size(); i++)
			for (Decision dec : AdoraPlugin.getActiveModel().getDecisions()) {
				if (dec.getNr() == constraintDecisionNumbers.get(i)) {
					if (dec.getDecisionValue() != null) {
						return true;
					}
					for(JoinRelation joinRelation:dec.getJoinRelations()){
						if(joinRelation.isVisible()==false){
							return true;
						}
					}
				}
			}

		return false;
	}

	/**
	 * Extracts the {@link Constraint}s from the {@link Decision}s stored in the
	 * {@link List}
	 * 
	 * @param decisions
	 *            {@link List} storing the selected {@link Decision}s.
	 * @return {@link List} of the corresponding {@link Constraint}s.
	 */
	private List<Constraint> getSelectedConstraints(
			ArrayList<Decision> decisions) {
		List<Constraint> selectedConsts = new ArrayList<Constraint>();

		List<Constraint> consts = adoraModel.getConstraints();

		for (Constraint c : consts) {
			for (Decision dec : decisions) {
				// add the Constraint in which the selected Decisions are
				// involved.
				if (c.containsDecision(dec) && !selectedConsts.contains(c)) {
					selectedConsts.add(c);
				}
			}
		}
		return selectedConsts;
	
	}
}
