package ch.unizh.ifi.adora.productline.ui.views.constraintstable.parser;

import java.util.HashMap;

import org.sat4j.core.VecInt;
import org.sat4j.minisat.SolverFactory;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVecInt;
import org.sat4j.tools.GateTranslator;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;

/**
 * Visitor Pattern for creating the syntax tree which builds the basis for
 * the satisfiability checking.  
 * 
 * Goal: This visitor returns a cnf formula representing the {@link Constraint}s.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2.12.2008 IV First Version
 */
public class SATvisitor implements ConstraintsVisitor{

	/**
	 * Stores the number of the Gate which 
	 * was generated last.
	 */
	private int varcnt;
	
 	/**
 	 * {@link GateTranslator} used to build CNF formula.
 	 */
 	private GateTranslator gt; 
 	
 	/**
 	 * The Solver
 	 */
 	private ISolver solver;
	
 	/**
	 * Stores the nr->decision relation.
	 */
	private HashMap<Integer, Decision> decMap;
	
	/**
	 * The {@link AdoraModel} on which the {@link Constraint}s get checked.
	 */
	private AdoraModel model; 

	/**
	 * Constructs a new InfixVisitor
	 * 
	 * @param model the {@link AdoraModel} 
	 */
	public SATvisitor(AdoraModel model) {
		this.model = model;
		decMap = model.getDecisionsMap();
		solver = SolverFactory.newDefault();
		gt = new GateTranslator(solver);
		varcnt = getHighestDecisionNumber()+1;
	}
	
	/**
	 * Generic method to check the whether the current {@link Constraint}setting
	 * is satisfiable.  
	 * 
	 * @param node The {@link Node}
	 * @param data not used
	 * @param opCode Determines whether the operation is an AND, OR, XOR
	 * @return new variable in the cnf formula
	 * @throws ContradictionException 
	 */
	private Object operationVisitor(Node node, Object data, int opCode) throws ContradictionException {
		
		Node a = node.jjtGetChild(0);
		Node b = node.jjtGetChild(1);
		
		Object aVal = a.jjtAccept(this, data);
		Object bVal = b.jjtAccept(this, data);
		
		int aInt = -1;
		boolean aNot = false;
		
		if (aVal instanceof String) { 
			int numIndex = 1;
			if (((String) aVal).substring(0,1).equals("-")) {
				aNot = true;
				numIndex++;
			}
			aInt = Integer.parseInt(((String) aVal).substring(numIndex));
		}
		else if (aVal instanceof Integer)
			aInt = (Integer)aVal;
		
		if (!(aVal instanceof Boolean)) {
			
    		//enforce that the truthvalues of the Decisions get considered
    		if (decMap.get(aInt) != null && decMap.get(aInt).getDecisionValue() != null) {
    			
    			aVal = decMap.get(aInt).getDecisionValue().booleanValue();
    			if (aNot)
    				aVal = !(Boolean)aVal;
    		}
    		if (aNot)
    			aInt *= -1;
		} 

		
		int  bInt = -1;
		boolean bNot = false;
		
		if (bVal instanceof String) {
			int numIndex = 1;
			if (((String) bVal).substring(0,1).equals("-")) {
				numIndex++;
				bNot = true;
			}
			bInt = Integer.parseInt(((String) bVal).substring(numIndex));
		}
		else if (bVal instanceof Integer)
			bInt = (Integer)bVal;

		if (!(bVal instanceof Boolean)) {
			
    		//enforce that the truthvalues of the Decisions get considered
    		if (decMap.get(bInt) != null && decMap.get(bInt).getDecisionValue() != null) {
    			
    			bVal = decMap.get(bInt).getDecisionValue().booleanValue();
    			if (bNot)
    				bVal = !(Boolean)bVal;
    			
    		}
			if (bNot)
				bInt *= -1;
		}
		
		if (aVal instanceof Boolean && bVal instanceof Boolean ) {
			if (opCode == ConstraintConnection.AND) {
				if (node.jjtGetParent() == null && !((Boolean)aVal && (Boolean)bVal)) {
					solver = null;
					return null;
				}
				return (Boolean)aVal && (Boolean)bVal;
			}
			if (opCode == ConstraintConnection.OR) {
				if (node.jjtGetParent() == null && !((Boolean)aVal || (Boolean)bVal)) {
					solver = null;
					return null;
				}
				return (Boolean)aVal || (Boolean)bVal;
			}
			if (opCode == ConstraintConnection.XOR) {
				if (node.jjtGetParent() == null && 
						!(((Boolean)aVal || (Boolean)bVal) && !(((Boolean)aVal && (Boolean)bVal)))) {
					solver = null;
					return null;
				}
				return ((Boolean)aVal || (Boolean)bVal) && !(((Boolean)aVal && (Boolean)bVal));
			}
				
				
		} else if (aVal instanceof Boolean) {
			if (opCode == ConstraintConnection.AND) {
				if ((Boolean)aVal)
					gt.iff(yNr(),  new VecInt(new int[]{bInt}));
				else {
					if (node.jjtGetParent() == null) {
						solver = null;
						return null;
					}
					return false;
				}
			} if (opCode == ConstraintConnection.OR) {
				if ((Boolean)aVal) {
					return true;
				}
				else
					gt.iff(yNr(),  new VecInt(new int[]{bInt}));
			} if (opCode == ConstraintConnection.XOR) {
				if ((Boolean)aVal)
					gt.iff(yNr(),  new VecInt(new int[]{-bInt}));
				else
					gt.iff(yNr(),  new VecInt(new int[]{bInt}));
			}
			return varcnt-1;
			
		} else if (bVal instanceof Boolean) {
			if (opCode == ConstraintConnection.AND) {
				if ((Boolean)bVal)
					gt.iff(yNr(),  new VecInt(new int[]{aInt}));
				else {
					if (node.jjtGetParent() == null) {
						solver = null;
						return null;
					}
					return false;
				}
			} else if (opCode == ConstraintConnection.OR) {
				if ((Boolean)bVal) {
					return true;
				}
				else{
					int nr = yNr();
					gt.iff(nr,  new VecInt(new int[]{aInt}));
				}
			} else if (opCode == ConstraintConnection.XOR) {
				if ((Boolean)bVal)
					gt.iff(yNr(),  new VecInt(new int[]{-aInt}));
				else
					gt.iff(yNr(),  new VecInt(new int[]{aInt}));
			}
			return varcnt-1;
		}

		
		IVecInt vc = new VecInt(new int[]{aInt, bInt});
				
		if (opCode == ConstraintConnection.AND)
			gt.and(yNr(), vc);
		else if (opCode == ConstraintConnection.OR) 
			gt.or(yNr(), vc);
		else if (opCode == ConstraintConnection.XOR) 
			gt.xor(yNr(), vc);
	
		return varcnt-1;
	}

	/**
	 * Called if a node in the AST is a SimpleNode.
	 */
	public Object visit(SimpleNode node, Object data) {
		return node;
	}

	/**
	 * Called if a node in the AST is AND.
	 */
	public Object visit(And node, Object data) {
		try {
			 Object o = operationVisitor(node, data, ConstraintConnection.AND);
			 return o;
		} catch (ContradictionException e) {
			solver = null;
			return null;
		}
	}

	/**
	 * Called if a node in the AST is a OR.
	 */
	public Object visit(Or node, Object data) {
		try {
			return operationVisitor(node, data, ConstraintConnection.OR);
		} catch (ContradictionException e) {
			solver = null;
			return null;
		}
	}
	
	/**
	 * Called if a node in the AST is a XOR.
	 */
	public Object visit(Xor node, Object data) {
		try {
			return operationVisitor(node, data, ConstraintConnection.XOR);
		} catch (ContradictionException e) {
			solver = null;
			return null;
		}
	}

	/**
	 * Called if a node in the AST is a Not.
	 */
	public Object visit(Not node, Object data) {
		Node operand  = node.jjtGetChild(0);

		if (operand instanceof Not) {
			Object o = operand.jjtGetChild(0).jjtAccept(this, data);
			return o;
		}
		
		Object o = operand.jjtAccept(this, data);
		
		if (o instanceof String) {
			o = "-" + (String)o;
		} else if (o instanceof Integer) {
			GateTranslator gt = new GateTranslator(solver);
			Integer gateNr = yNr();
			try {
				gt.not(gateNr,  (Integer)o);
			} catch (ContradictionException e) {
				solver = null;
				return null;
			}
			o = gateNr;
		} else if (o instanceof Boolean) {
			o = !((Boolean)o);
		}
		return o;
	}

	/**
	 * Called if a node in the AST is a Decision.
	 */
	public Object visit(Lit node, Object data) {
		if (node.parent == null) {
			try {
				String decStr = node.val;
				int decNr = Integer.parseInt(decStr.substring(1));
				gt.gateTrue(decNr);
			} catch (ContradictionException e) {
				solver = null;
				return null;
			}
		}
		return node.val;
	}

	/**
	 * Called if a node in the AST is a Boolean Value.
	 */
	public Object visit(False node, Object data) {
		return new Boolean(false);
	}

	/**
	 * Called if a node in the AST is a Boolean Value.
	 */
	public Object visit(True node, Object data) {
		return new Boolean(true);
	}

	/**
	 * Communication Method
	 * 
	 * @return {@link ISolver} the Solver or if the formula contains a contradiction
	 * i.e. it isn't satisfiable in the first place, then null is returned.
	 * This has to be done, since J4SAT throws a {@link ContradictionException} if a formula 
	 * contains a trivial contradiction.
	 */
	public ISolver getSolver(){
		try {
			for (Decision dec : model.getDecisions()) {
				
				int decNr = dec.getNr();
			
				//enforce that already set variables are not changed
				if (dec.getDecisionValue() != null) {
					
					if (dec.getDecisionValue()) 
						gt.gateTrue(decNr);
	    			else
	    	 			gt.gateFalse(decNr);
				}
			}
			//ensure that the top node of the boolean tree has to be true.
				
			if (varcnt >  getHighestDecisionNumber() + 1)
				gt.gateTrue(varcnt-1);
		} catch (ContradictionException e) {
			return null;
		}return solver;
	}
	
	/**
	 * Used to generate internal variables for the cnf formula
	 * 
	 * @return the index of the next variable.
	 */
	private int yNr(){
		return varcnt++;
	}
	
	/**
	 * @return The highest {@link Decision}number used in the {@link AdoraModel}.
	 */
	private int getHighestDecisionNumber() {
		int maxDecNr = 0;
		
		for (Decision dec : model.getDecisions()) {
			
			int decNr = dec.getNr();
		
			if (decNr > maxDecNr) 
				maxDecNr = decNr; 
		}
		return maxDecNr;
	}
}