package ch.unizh.ifi.adora.productline.ui.views.constraintstable.parser;
import java.util.HashMap;

import org.sat4j.specs.ContradictionException;
import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;


/**
 * Simplifies a logical expression by taking the already set variables into account.
 * 
 * Goal: This visitor returns a cnf formula representing the {@link Constraint}s.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2.12.2008 IV First Version
 */

public class SimplifyVisitor implements ConstraintsVisitor{

	/**
 	 * Stores the simplyfied expression
 	 */
 	private String simplyfiedExpr = "";
	
 	/**
	 * Stores the nr->decision relation.
	 */
	private HashMap<Integer, Decision> decMap;

	/**
	 * Constructs a new InfixVisitor
	 * 
	 * @param constraint 
	 */
	public SimplifyVisitor() {
		decMap = AdoraPlugin.getActiveModel().getDecisionsMap();
	}
	
	/**
	 * Generic method to check the whether the current {@link Constraint}setting
	 * is satisfiable.  
	 * 
	 * @param node The {@link Node}
	 * @param data not used
	 * @param opCode Determines whether the operation is an AND, OR, XOR
	 * @return new variable in the cnf formula
	 * @throws ContradictionException 
	 */
	private Object operationVisitor(Node node, Object data, int opCode)  {
		
		String curExp = "";
		
		Node a = node.jjtGetChild(0);
		Node b = node.jjtGetChild(1);
		
		Object aVal = a.jjtAccept(this, data);
		Object bVal = b.jjtAccept(this, data);
		
		if (aVal instanceof String && ((String)aVal).toUpperCase().matches("D\\d+")) { // check if leaf
			
			//enforce that the truthvalues of the Decisions get considered
			int aInt = Integer.parseInt(((String) aVal).substring(1));
    		if (decMap.get(aInt) != null && decMap.get(aInt).getDecisionValue() != null) {
    			//here we take into account the already set variables.
    			aVal = (boolean)decMap.get(aInt).getDecisionValue();// ^ (a instanceof Not); //invert bBool if  there is a not in front.
    		}
		}
		if (bVal instanceof String && ((String)bVal).toUpperCase().matches("D\\d+")) {
			
			//enforce that the truthvalues of the Decisions get considered
			int bInt = Integer.parseInt(((String) bVal).substring(1));
    		if (decMap.get(bInt) != null && decMap.get(bInt).getDecisionValue() != null) {
    		
    			//here we take into account the already set variables.
    			bVal = (boolean)decMap.get(bInt).getDecisionValue();// ^ (b instanceof Not); //invert bBool if  there is a not in front.
    
    		}
		}
		
		//consider not
		if (aVal instanceof Boolean && a instanceof Not)
			aVal = !(Boolean)aVal;
		if (bVal instanceof Boolean && b instanceof Not)
			bVal = !(Boolean)bVal;
		
		
		
		if (aVal instanceof Boolean && bVal instanceof Boolean) {
			
			if (opCode == ConstraintConnection.AND) 
				return (Boolean)aVal && (Boolean)bVal;
			if (opCode == ConstraintConnection.OR) 
				return (Boolean)aVal || (Boolean)bVal;
			if (opCode == ConstraintConnection.XOR) 
				return ((Boolean)aVal || (Boolean)bVal) && !(((Boolean)aVal && (Boolean)bVal));
		} 
		
		else if (aVal instanceof Boolean) {
			
			//new
			if (b instanceof Not)
				bVal = "not "+bVal;
			
			if (opCode == ConstraintConnection.AND) {
				if (!(Boolean)aVal)
					return false;
				else
					return bVal; //something
			} else if (opCode == ConstraintConnection.OR) {
				if ((Boolean)aVal)
					return true;
				else
					return bVal;
			} else if (opCode == ConstraintConnection.XOR) {
				if ((Boolean)aVal) {
					if (b instanceof Not)
						return ((String)bVal).substring(4); //delete not
					else 
						return "not " + bVal;
				}
				else
					return bVal;
			}
		} 
		
		else if (bVal instanceof Boolean) {
			
			//new
			if (a instanceof Not)
				aVal = "not "+aVal;
			
			if (opCode == ConstraintConnection.AND) {
				if (!(Boolean)bVal)
					return false;
				else
					return aVal; //something
			} else if (opCode == ConstraintConnection.OR) {
				if ((Boolean)bVal)
					return true;
				else
					return aVal;
			} else if (opCode == ConstraintConnection.XOR) {
				if ((Boolean)bVal) {
					if (a instanceof Not)
						return ((String)aVal).substring(4); //delete not
					else 
						return "not " + aVal;
				}
			}
		} 
		
		if (a instanceof Not)
			aVal = "not " + aVal;
		if (b instanceof Not)
			bVal = "not " + bVal;
			
			
		if (opCode == ConstraintConnection.AND) {
			curExp = "(" + aVal + " and " + bVal + ")";
		}
		else if (opCode == ConstraintConnection.OR) { 
			curExp = "(" + aVal + " or " + bVal + ")";
		}
		else if (opCode == ConstraintConnection.XOR) { 
			curExp = "(" + aVal + " xor " + bVal + ")";
		}
		
		return curExp;
	}

	/**
	 * Called if a node in the AST is a SimpleNode.
	 */
	public Object visit(SimpleNode node, Object data) {
		return node;
	}

	/**
	 * Called if a node in the AST is AND.
	 */
	public Object visit(And node, Object data) {
		
		Object o = operationVisitor(node, data, ConstraintConnection.AND);
		
		if (node.jjtGetParent() == null) {
			setSimplyfiedExpr(o);
		}
		
		return o;
		//return operationVisitor(node, data, ConstraintConnection.AND);
	}

	/**
	 * Called if a node in the AST is a OR.
	 */
	public Object visit(Or node, Object data) {

		Object o = operationVisitor(node, data, ConstraintConnection.OR);
		
		if (node.jjtGetParent() == null) {
			setSimplyfiedExpr(o);
		}
		
		return o;
		
		//return operationVisitor(node, data, ConstraintConnection.OR);
	}
	
	/**
	 * Called if a node in the AST is a XOR.
	 */
	public Object visit(Xor node, Object data) {
		

		Object o = operationVisitor(node, data, ConstraintConnection.XOR);
		
		if (node.jjtGetParent() == null) {
			setSimplyfiedExpr(o);
		}
				
		return o;
		
		
		//return operationVisitor(node, data, ConstraintConnection.XOR);
	}

	/**
	 * Called if a node in the AST is a Not.
	 */
	public Object visit(Not node, Object data) {
		Node operand  = node.jjtGetChild(0);

		if (operand instanceof Not)
			return operand.jjtGetChild(0).jjtAccept(this, data);
		else 
			return operand.jjtAccept(this, data);
	}

	/**
	 * Called if a node in the AST is a Decision.
	 */
	public Object visit(Lit node, Object data) {
		
		if (node.jjtGetParent() == null)  {
			
			String exp = node.val;
			
			int aInt = Integer.parseInt((node.val).substring(1));
    		if (decMap.get(aInt) != null && decMap.get(aInt).getDecisionValue() != null) {
    			//here we take into account the already set variables.
    			boolean boolVal = decMap.get(aInt).getDecisionValue();
    			if (boolVal) 
    				exp = "true";
    			else
    				exp = "false";
    		}
			
			setSimplyfiedExpr(exp);
		}
		
		//not d1
		if (node.jjtGetParent() instanceof Not && node.jjtGetParent().jjtGetParent() == null) {
			String exp = "not "+ node.val;
		
    		int aInt = Integer.parseInt((node.val).substring(1));
    		if (decMap.get(aInt) != null && decMap.get(aInt).getDecisionValue() != null) {
    			//here we take into account the already set variables.
    			boolean boolVal = decMap.get(aInt).getDecisionValue();
    			if (boolVal) 
    				exp = "false";
    			else
    				exp = "true";
    		}
		
    		setSimplyfiedExpr(exp);
		}
		
		return node.val;
	}

	/**
	 * Called if a node in the AST is a Boolean Value.
	 */
	public Object visit(False node, Object data) {
		
		if (node.jjtGetParent() == null) {
			setSimplyfiedExpr("false");
		}
		
		return new Boolean(false);
	}

	/**
	 * Called if a node in the AST is a Boolean Value.
	 */
	public Object visit(True node, Object data) {
		
		if (node.jjtGetParent() == null) {
			setSimplyfiedExpr("true");
		}
		
		return new Boolean(true);
	}
	
	/**
	 * Sets the simplyfied expression to exp; 
	 * @param exp The new simplyfied expression.
	 */
	private void setSimplyfiedExpr(Object o) {
		if (o instanceof Boolean) {
			if ((Boolean)o)
				simplyfiedExpr = "true";
			else
				simplyfiedExpr = "false";
		} else {
			simplyfiedExpr = ((String)o);
		}
	}
	
	/**
	 * @return The simplyfied expression.
	 */
	public String getSimplyfiedExp() {
		return simplyfiedExpr;
	}
}