package ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat;

import java.awt.geom.GeneralPath;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;

import org.eclipse.ui.internal.handlers.WizardHandler.New;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.model.util.ModelSerializer;

/**
 * This class generates random {@link AdoraModel}s according to the following complexity parameters:
 * 
 *  mindecCnt			Min Number of {@link Decision}s to be created	
 *  maxDecCnt			Max Number of {@link Decision}s to be created	
 *  decInc				Incrementation of {@link Decision} count
 *  cntSameCmplMod		How many models should be generated of the same complexity
 * 
 *  hierCnt				Number of different {@link AspectContainer} paths.
 *  hierDepth			Average depth, i.e. average number of {@link AspectContainer}s
 *							on each of the commonality paths.
 *
 *  minConstCnt			Min	Number of {@link Constraint}s to be created	
 *  maxConstCnt			Max Number of {@link Constraint}s to be created	
 *  constInc			Incrementation of {@link Constraint} count
 *  constSize			Average number of {@link Decision} per {@link Constraint}.
 *  constComplexity		Average number of conjunctive terms per {@link Constraint}.
 *
 *  minVpCnt			Min	Number of {@link VariationPoint}s to be created
 *  maxVpCnt			Max	Number of {@link VariationPoint}s to be created
 *  vpInc				Incrementation of {@link VariationPoint} count
 * 							
 *  vpSize				Average number of {@link Decision} per {@link VariationPoint}.
 *  variance			The global variance parameter for all randomly generated values.
 *  
 *  
 *  The actual profiling / modelgenration works like this:
 *  
 *  for (int decCnt = minDecCnt; decCnt < maxDecCnt; decCnt += decInc)  
 *	 	for (int constCnt = minConstCnt; constCnt < maxConstCnt; constCnt += constInc) 
 *			for (int vpCnt = minVpCnt; vpCnt < maxVpCnt; vpCnt += minVpCnt) 
 *				profile AdoraModel with given complexity parameters
 *
 *  
 *  {@link AdoraModel} generation: 
 *  We first generate a {@link Component} as the commonality and connect it to (<i>decCnt<\i> / <i>hierDepth<\i>) many disjoint 
 *  commonality paths each having an average of <i>hierDepth<\i> many {@link AspectContainer}s. Each of these {@link AspectContainer}s
 *  contains a {@link Component} which is connected by a {@link Decision} (i.e. a {@link JoinRelation}) to the 
 *  {@link Component} contained in the preceding {@link AspectContainer} of the given commonality path.
 *  
 *  {@link Constraint} generation:
 *  For each {@link AdoraModel} of size <i>decCnt<\i>, we generate <i>constCnt<\i> many {@link Constraint}s.
 *  A {@link Constraint} has the form of a random k-CNF formula implying a random k-CNF formula.
 *  These k-CNF formulas have <i>constComplexity<\i> many clauses and involve <i>constSize<\i> many {@link Decision}s.
 *  
 *  {@link VariationPoint}generation:
 *  For each {@link AdoraModel} of size <i>decCnt<\i>, we generate <i>vpCnt<\i> many {@link VariationPoint}s.
 *  A {@link VariationPoint} has a randomly generated min and max cardinality.
 *  A {@link VariationPoint} involves <i>vpSize<\i> many {@link Decision}s on average.
 *  
 *  All randomly generated values are u.a.r. distributed, each with a specific mean but with the global <i>variance<\i>.
 *  
 * 
 * @author ivovigan
 */
public class ModelGenerator {
	
	private final boolean RUN_ON_SERVER = true;

	/**
	 * Min Number of {@link Decision}s to be created
	 */
	private int minDecCnt;
	
	/**
	 * Max Number of {@link Decision}s to be created
	 */
	private int maxDecCnt;
	
	/**
	 * Number by which number of {@link Decision}s get incremented.
	 */
	private int decInc;
	
	/**
	 * Average depth, i.e. average number of {@link AspectContainer}s
	 * on each of the commonality paths.
	 */
	private int hierDepth; 
	
	/**
	 * Average number of {@link Decision} per {@link VariationPoint}.
	 */
	private int vpSize; 
	
	/**
     *Average number of {@link Decision} per {@link Constraint}.
 	 */
	private int constSize;
	
	/**
	 * Average number of conjunctive terms per {@link Constraint}.
	 */
	private int constComplexity;
	
	/**
	 * The global variance parameter for all randomly generated values.
	 */
	private int variance;
	
	/**
	 * Stores the information of how many {@link AdoraModel}s of the same 
	 * complexity should be generated.
	 */
	private int cntSameComplexityModels;
	
	/**
	 * Determines the ration of the number of {@link Constraint}s in relation to 
	 * the number of {@link VariationPoint}.
	 */
	private double constVpRatio;
	
	/**
	 * Determines the maximal number of attempts used to generate random 
	 * {@link Constraint}s and {@link VariationPoint}s. 
	 * This parameter is needed since we want to generate satisfiable 
	 * {@link Constraint} settings.
	 */
	private final int MAXTRIES = 30;
	
	/**
	 * Determines the location of the commonality.
	 * Since we build our model as a star graph, the commonality 
	 * is the center of the polar coordinate system.
	 */
	private final Point ORIGIN = new Point(300, 300); 
	
	/**
	 * Determines the distance between two consecutive {@link AspectContainer}s
	 * in a commonalty path.
	 */
	private final int CONNECTION_LENGTH = 200; 
	
	/**
	 * Determines, whether or not the generated {@link AdoraModel}s should be saved
	 * to the file System
	 */
	private final boolean SAVE = true; 

	/**
	 * Contains the path of the folder in which the models should be stored.
	 */
	private final String SAVE_PATH_MODEL;
		
	/**
	 * The k in k-CNF
	 */
	private final int CLAUSESIZE = 2;

	private int minConstCnt;

	private int maxConstCnt;

	private int constInc;

	private double crosscutting;
	
	private int minVpCnt;

	private int maxVpCnt;

	private int vpInc;
	
	private double childCnt;
	
	/**
	 * List storing all the sibling Decisions.
	 * This is used to generate {@link VariationPoint}s, since {@link VariationPoint}s 
	 * only make sense between alternatives.
	 */
	private List<List<Decision>> siblingDecisions = new ArrayList<List<Decision>>();
	
	/**
	 * testmethod
	 * 
	 * @param args
	 */
	public static void main(String [] args) {
		ModelGenerator generator = new ModelGenerator(
					
				   20, // -minDecCnt, 
				   50, // -maxDecCnt,
				   2, // -decInc,
				   10, // -cntSameComplexityModels,
				   9, // -hierDepth, 
				   3, // avgChildCnt
				   3, // -minConstCnt, 
				   5, // -maxConstCnt, 
				   1, // -constInc,
				   3, // -constSize,
				   2, // -constComplexity,
				   2, // -minVpCnt,   NOT-USED
				   3, // -maxVpCnt,   NOT-USED
				   1, // -vpInc,      NOT-USED
				   2, // -vpSize,     
				   2,// -variance
				   0.5,  //constTOvpRatio
				   0.2); //crosscutting
		
		
		List<List<AdoraModel>> models = generator.generate();
		
		//temp
		SATprofiler prof = new SATprofiler(
				
				   20, // -minDecCnt, 
				   50, // -maxDecCnt,
				   2, // -decInc,
				   10, // -cntSameComplexityModels,
				   9, // -hierDepth, 
				   3, // avgChildCnt
				   3, // -minConstCnt, 
				   5, // -maxConstCnt, 
				   1, // -constInc,
				   3, // -constSize,
				   2, // -constComplexity,
				   2, // -minVpCnt,   NOT-USED
				   3, // -maxVpCnt,   NOT-USED
				   1, // -vpInc,      NOT-USED
				   2, // -vpSize,     
				   2,// -variance
				   0.5);  //constTOvpRatio
		
		prof.startProfiling();
		System.out.println("exit");
		
	}
	
	/**
	 * 
	 * @return
	 */
	public List<List<AdoraModel>> generate() {
		List<List<AdoraModel>> models = new ArrayList<List<AdoraModel>>();
		
		System.out.print("Generating Models");
		
		for (int decCnt = minDecCnt; decCnt <= maxDecCnt; decCnt += decInc) {
		
			//int percentage = (int)(decCnt/ Math.max((maxDecCnt - minDecCnt), decCnt)*100);
			System.out.print(".");
					
			String decsPath;
			if (SAVE) {
				decsPath = SAVE_PATH_MODEL+"/"+String.valueOf(decCnt)+"Decs/"; 
				new File(decsPath).mkdirs();
			}
			
			List<AdoraModel> complexity_i_Models;
			for (int totalConstCnt = minConstCnt; totalConstCnt <= maxConstCnt; totalConstCnt += constInc) {
				int constCnt = (int)(totalConstCnt * constVpRatio);
				int vpCnt = totalConstCnt - constCnt;
					
				complexity_i_Models = generateCurrentSettingModels(decCnt, constCnt ,vpCnt, cntSameComplexityModels);
				models.add(complexity_i_Models);
				
				if (SAVE) {
					String constPath = decsPath+String.valueOf(totalConstCnt)+"Const/";
					new File(constPath).mkdirs();
					
					int modIndex = 1;
					for (AdoraModel model : complexity_i_Models) {
						saveModel(model, constPath+"model"+modIndex+"_Dec"+decCnt+"_Const"+totalConstCnt+".adora");
						modIndex++;
					}
				}
			}
		}
		System.out.println("");
		System.out.println("Done");
		
		return models;
	}
	
	/**
	 * Generates n random {@link AdoraModel}s.
	 * 
	 * @param n The number of {@link AdoraModel}s to be generated.
	 * @param cntSameComplexityModels 
	 * @param vpCnt 
	 * @param constCnt 
	 * @return
	 */
	private List<AdoraModel> generateCurrentSettingModels(int decCnt, int constCnt, int vpCnt, int cntSameComplexityModels) {
		
		List<AdoraModel> models = new ArrayList<AdoraModel>();
		
		for (int i = 0; i < cntSameComplexityModels; i++) {
			AdoraModel model =  generateRandomModel(decCnt, constCnt, vpCnt, i);
			if (model != null) {
				models.add(model);

			}
		}
		return models;
	}
	
	/**
	 * Randomly generates an {@link AdoraModel} possessing certain complexity
	 * characteristics.
	 * 
	 * @param mindecCnt			Min Number of {@link Decision}s to be created	
	 * @param maxDecCnt			Max Number of {@link Decision}s to be created	
	 * @param decInc			Incrementation of {@link Decision} count
	 * @param cntSameCmplMod	How many models should be generated of the same complexity
	 * 
	 * @param hierCnt			Number of different {@link AspectContainer} paths.
	 * @param hierDepth			Average depth, i.e. average number of {@link AspectContainer}s
	 *							on each of the commonality paths.
	 * @param childCnt			Average outdegree of an {@link AspectContainer}. 
	 *
	 * @param minConstCnt		Min	Number of {@link Constraint}s to be created	
	 * @param maxConstCnt		Max Number of {@link Constraint}s to be created	
	 * @param constInc			Incrementation of {@link Constraint} count
     * @param constSize			Average number of {@link Decision} per {@link Constraint}.
	 * @param constComplexity	Average number of conjunctive terms per {@link Constraint}.
	 *
	 * @param minVpCnt			Min	Number of {@link VariationPoint}s to be created
	 * @param maxVpCnt			Max	Number of {@link VariationPoint}s to be created
	 * @param vpInc				Incrementation of {@link VariationPoint} count
	 * @param vpSize			Average number of {@link Decision} per {@link VariationPoint}.
	 * 
	 * @param variance			The global variance parameter for all randomly generated values.
	 */
	public ModelGenerator( int minDecCnt, 
						   int maxDecCnt,
						   int decInc,
						   int cntSameComplexityModels,
						   int hierDepth, 
						   double childCnt,
						   int minConstCnt, 
						   int maxConstCnt, 
						   int constInc,
						   int constSize,
						   int constComplexity,
						   int minVpCnt,
						   int maxVpCnt,
						   int vpInc,
						   int vpSize,
						   int variance,
						   double constVpRatio, 
						   double crossCutting) {
		
		this.minDecCnt = minDecCnt; 
		this.maxDecCnt = maxDecCnt; 
		this.decInc = decInc;
		this.cntSameComplexityModels = cntSameComplexityModels;

	    this.hierDepth = hierDepth; 
	    this.childCnt = childCnt;

	    this.minConstCnt = minConstCnt; 
	    this.maxConstCnt = maxConstCnt; 
	    this.constInc = constInc; 
	    this.constSize = constSize; 
	    this.constComplexity = constComplexity;
	    
	    this.minVpCnt = minVpCnt; 
	    this.maxVpCnt = maxVpCnt; 
	    this.vpInc = vpInc; 
	    this.vpSize = vpSize; 
	    
	    this.variance = variance;
	    this.constVpRatio = constVpRatio;
	    
	    this.crosscutting = crossCutting;
	    
	    if (RUN_ON_SERVER)
			SAVE_PATH_MODEL = "C:\\SATprofiling\\models"; //windows
		else 
			SAVE_PATH_MODEL = "/Users/ivovigan/Documents/runtime-EclipseApplication/A/Profiling";  //mac
		
	}
	
	/**
	 * 
	 * 
	 * @param index	The index of the current model generated.
	 * 
	 * @return An array containing the time used to perform certain
	 * 		   satsolving tasks.
	 */
	private AdoraModel generateRandomModel(int decCnt, int constCnt, int vpCnt, int index) {
		
		for (int i = 0; i < 30; i++) {	
			AdoraModel model = new AdoraModel();
			
			Component commonality = new Component();
			commonality.setLocation(ORIGIN);
			commonality.setSize(new Dimension(100, 70));
			commonality.setName("Commonality");
			//this is needed for the modelcopy not having NPE
			//commonality.getZoomStructure();
			
			model.addChild(commonality);
			
			generateHierarchies(decCnt, model);
			
			generateConstraints(constCnt, model);
			generateVPs(vpCnt, model);
			
			SatSolver solver = new SatSolver(model);
			boolean SAT = solver.checkSatisfiability();
			if (SAT) {
				//the current constraint / VP setting is satisfiable
				return model;
			}
		}
		return null;
	}
	
	/**
	 * 
	 * @param decCnt 
	 * @param rectangle 
	 * @param decCnt Number of {@link Decision}s to be created	
	 */
	private void generateHierarchies(int decCnt, AdoraModel model) {
		Component commonality = (Component)model.getChildAt(0);
		
		int usedDec = 0;
		int hierID = 0;
		
		List<Integer> hierDepts = new ArrayList<Integer>();
		
		while (usedDec <= decCnt) {
			int decCntForPath = UAR(hierDepth, variance);
			usedDec += decCntForPath;
			
			//if the random variable would generate more than decCnt many decisions in total just
			//generate so many to sum up to decCnt
			if (usedDec > decCnt)
				decCntForPath = decCnt - (usedDec - decCntForPath);
			
			hierDepts.add(decCntForPath);
		}
		
		for (int decCntForPath : hierDepts) {
			AspectContainer predecessor = generateAC(model, commonality, hierDepts.size(), hierID, 0,0, 0);
			int cntAcOutDeg = Math.round(UAR(childCnt, variance));
			if (cntAcOutDeg + cntUsedDec > decCntForPath) {
				cntAcOutDeg = decCntForPath - cntUsedDec;
			}
			cntUsedDec += cntAcOutDeg;
			generateHierarchy(model, hierDepts.size(), hierID, decCntForPath-1, predecessor, cntAcOutDeg);
			cntUsedDec = 0; 
			hierID++;
		}
	}
	
	private int cntUsedDec = 0; 
	private void generateHierarchy(AdoraModel model, int hierCnt,  int hierID, int decCntForPath, ZoomableNodeContainer predecessor, int cntSiblings) {
		
		if (cntUsedDec > decCntForPath) {
			return;
		}
		
		List<Decision> siblings = new ArrayList<Decision>();
		List<Integer> outDegList = new ArrayList<Integer>();
		
		for (int childID = 0; childID < cntSiblings; childID++) {
			
			//determine the out-degree of the given aspectcontainer
			int cntAcOutDeg = Math.round(UAR(childCnt, variance));
			if (cntAcOutDeg + cntUsedDec > decCntForPath) {
				cntAcOutDeg = decCntForPath - cntUsedDec;
			}
			cntUsedDec += cntAcOutDeg;
			outDegList.add(cntAcOutDeg);
		}
		
		outDegList = randomPermutation(outDegList);
			
		for (int childID = 0; childID < cntSiblings; childID++) {
			//The incoming decision of the newly created AC, used to store in siblings.
			AspectContainer nextPredecessor = generateAC(model, predecessor, hierCnt, hierID, cntSiblings, childID, 0);
			
			Decision d = (Decision)((JoinRelation)(((Node)(
									nextPredecessor.getChildAt(0))).
											getSourceConnections().get(0))).getDecision();
			
			siblings.add(d);
			
			
			generateHierarchy(model, hierCnt, hierID, decCntForPath, nextPredecessor, outDegList.get(childID));
		}
		
		//insert siblings in increasing order of size
		int insertionPosition = 0;
		for (List<Decision> curSib : siblingDecisions) {
			if (curSib.size() > siblings.size()) {
				break;
			}
			insertionPosition++;
		}
		
		siblingDecisions.add(insertionPosition, siblings);
	}
	
	/**
	 * Generates an {@link AspectContainer} and connects it with a {@link Decision} to <li>predecessor</li>.
	 * 
	 * @param model The {@link AdoraModel}.
	 * @param predecessor 
	 * @param hierCnt
	 * @param hierID
	 * @param i
	 * 
	 * @return The generated {@link AspectContainer}.
	 */
	private AspectContainer generateAC(AdoraModel model, ZoomableNodeContainer predecessor, int hierCnt, int hierID, int cntSiblings, int childID, int i) {
		AspectContainer ac = new AspectContainer();
		
		Point acLocation = getACLocation(hierCnt, hierID, cntSiblings, childID, predecessor.getBounds());
		Dimension acSize = new Dimension(150, 100);
		//this is needed for the modelcopy not having NPE
		//ac.getZoomStructure();
		
		ac.setLocation(acLocation);
		ac.setSize(acSize);
		model.addChild(ac);
		
		ac.setName("AC "+hierID+"."+i);

		Component c = new Component();
		c.setLocation(30 ,20);
		c.setSize(new Dimension(100, 70));
		//this is needed for the modelcopy not having NPE
		//c.getZoomStructure();
		c.setName("C "+hierID+"."+i);
		ac.addChild(c);
		
		ConnectableElement target; 
		if (predecessor instanceof AspectContainer)
			target = (ConnectableElement)predecessor.getChildren().get(0);
		else 
			target = predecessor;
		
		JoinRelation jr = new JoinRelation(c, target);
		
		
		//crosscutting concerns:
		
		Decision dec;
		if (Math.random() < crosscutting) {
			dec = getNrandomDecs(model, 1).get(0);
			dec.addJoinRelation(jr);
		} else {
			dec = new Decision(jr);
			model.addDecision(dec);
		}
 		
		ac.setVariability(true);
		predecessor = ac;

		return ac;
		
	}
	
	/**
	 * A {@link List} containing the newly created {@link Constraint};
	 * @param constCnt2 
	 * 
	 * @param decCnt Number of {@link Constraint}s to be created	
	 */
	private void generateConstraints(int constCnt, AdoraModel model) {
		
		for (int i = 0; i < constCnt; i++) {
			
			Constraint c = new Constraint(model);
			
			int curConstSize = UAR(constSize, variance);
			List<Decision>  
			decs = getNrandomDecs(model, curConstSize);
			
			String antecedent = generateRandomKsat(model, decs, CLAUSESIZE);
			String consequent = generateRandomKsat(model, decs, CLAUSESIZE);
			
			if (antecedent.trim().equals(""))
				antecedent = "false";
			if (consequent.trim().equals(""))
				antecedent = "true";
				
			c.setAntecedent(antecedent);
			c.setConsequent(consequent);
			
			model.addConstraint(c);
		}
	}
	
	/**
	 * Generates a k-CNF Formula, not necessarily satisfiable.
	 * 
	 * @param decs The {@link Decision} involved in the k-CNF expression.
	 * 
	 * @param k Number of literals per clause
	 *
	 * @return A k-CNF Formula, not necessarily satisfiable.
	 */
	private String generateRandomKsat(AdoraModel model, List<Decision> decs, int k) {
		
		String expr = "";
		
		int m = UAR(constComplexity, variance);
		
		List<Decision> clauseDecs;
		String clauseExpr;
		
		for (int i = 0; i < m; i++) {
			clauseExpr = "";
			clauseDecs = getNrandomDecs(model, k);
			for (int j = 0; j < Math.min(k, clauseDecs.size()); j++) {
				
				if (Math.random() > 0.5)
					clauseExpr += "not ";
				
				clauseExpr += "D"+clauseDecs.get(j).getNr();
				
				if (j < k-1)
					clauseExpr += " or ";	
			}
			expr = expr + "("+clauseExpr+")";
			if (i < m-1)
				expr = expr + " and ";	
		}
		return expr;
	}
	
	
	/**
	 * @param vpCnt 
	 * @return A {@link List} containing the newly created {@link VariationPoint};
	 * 
	 * @param vpCnt Number of {@link VariationPoint}s to be created
	 * @param vpComplexity 
	 */
	private void generateVPs(int vpCnt, AdoraModel model) {
		
		for (int i = 0; i < vpCnt; i++) {
			
			int curVPcomplexity = UAR(vpSize, variance);
			
			List<Decision> decs = new ArrayList<Decision>(); 
			//find the most suitable VP
			for (int ind = 0; ind < siblingDecisions.size(); ind++) {
				List<Decision> siblings = siblingDecisions.get(ind);
				if (siblings.size() >= curVPcomplexity) {
					//maybe previous list element is closer to requested size
					if (ind > 0) {
						int predSize = siblingDecisions.get(ind-1).size();
						
						if (Math.abs(curVPcomplexity - siblings.size()) < Math.abs(curVPcomplexity - predSize)) {
							decs = siblings;
							siblingDecisions.remove(decs);
						} else {
							decs =  siblingDecisions.get(ind-1);
							siblingDecisions.remove(decs);
						}
					} else {
						decs = siblings;
						siblingDecisions.remove(decs);
					}
					break;
				}
			}
			
			if (decs.size() < 2)
				continue;
			
			VariationPoint vp = new VariationPoint(decs);
			
			int minCard =  randomINTinInterval(0, decs.size());
			int maxCard =  randomINTinInterval(minCard, decs.size());
			
			vp.setMinCard(minCard);
			vp.setMaxCard(maxCard);
			
			model.addVariationPoint(vp);
			
		}
	}	
	
	/**
	 * This method is used to randomly select n {@link Decision}s 
	 * from the existing {@link Decision}s in the {@link AdoraModel}.
	 *  
	 *  
	 * @param n the number of {@link Decision}s to be generated.
	 * @return Decisions n randomly selected {@link Decision}s.
	 */
	private List<Decision> getNrandomDecs(AdoraModel model, int n) {
		
		if (n > model.getDecisions().size())
			n = model.getDecisions().size();
		
		List<Decision> randomDecs = new ArrayList<Decision>();
		
		Collection<Decision> modelDecs = model.getDecisions();
		
		//generate n distinct random numbers between 1 and n
		List<Integer> randInds = new ArrayList<Integer>(); 
		for (Decision d : modelDecs) {
			randInds.add(d.getNr());
		}
		
		//now remove 
		for (int i = n+1; i <= modelDecs.size() ; i++) {
			int randomIndex = (int)(Math.random()*randInds.size());
			randInds.remove(randomIndex);
		}
		
		for (int i = 0; i < n; i++) {
			int randomIndex = randInds.get(i);
			randomDecs.add(model.getDecisionsMap().get(randomIndex));
		}
		return randomDecs;
	}
	
	/**
	 * 
	 * 
	 * @param mu
	 * @param variance
	 * @return
	 */
	private int UAR(double mu, double variance) {
		int rand = (int)(Math.random()*variance+mu);
		return rand > 0 ? rand : 1; 
	}
	
	/**
	 * Returns a normally distributed random value with mean <li>mu<\li>
	 * and variance <li>variance<\li>. 
	 * The value is > 0
	 * 
	 * @param mu Mean
	 * @param variance Standard deviation
	 * @return X ~ N(<li>mu<\li>, <li>variance<\li>)
	 */
	private int N(int mu, double variance) {
		double sigma = Math.sqrt(variance);
		int randVal = (int)(sigma*new Random().nextGaussian() + mu);
		return randVal > 0 ? randVal : 0; 
	}
	
	/**
	 * Generates a random integer i :  l <= i <= h u.a.r.  
	 *  
	 * @param l lower bound (inclusive)
	 * @param h upper bound (inclusive)
	 * 
	 * @return random integer i : l <= i <= h u.a.r.
	 */
	private int randomINTinInterval(int l, int h) {
		return (int)(l + Math.random()*(h+1-l));
	}
	
	/**
	 * This method is responsible for placing the {@link AspectContainer} in 
	 * a star shaped manner around the commonality
	 * 
	 * @param hierID The hierarchy number \in {1, ..., cntHier}
	 * @param predecessorBounds 
	 * 				The bounds of the predecessor {@link AspectContainer}
	 * 				in the given hierarchy path.
	 * 
	 * @return  A {@link Point} determining 
	 * 			the coordinate of the current {@link AspectContainer}.
	 */
	private Point getACLocation(int hierCnt, int hierID, int cntSiblings, int childID, Rectangle predecessorBounds) {
		
		double pathAngle = 2*Math.PI/hierCnt*(hierID);
		
		double halfSegAngle = Math.PI/hierCnt;
		
		double predecessorDiag = Math.sqrt( predecessorBounds.width*
											predecessorBounds.width +
											predecessorBounds.height*
											predecessorBounds.height);
		
		double lineSegLength = (predecessorDiag * cntSiblings);
		
		//we calculate the distance to the predecessor
		//it must be s.t. the lineSeg fits into the path sector
		double distanceToLineSeg = Math.max(lineSegLength/(2*Math.tan(halfSegAngle)), CONNECTION_LENGTH); 
		
		//(x,y) is the mid point of the line segment 
		double x = predecessorBounds.x + Math.cos(pathAngle)*(predecessorDiag + distanceToLineSeg);
		double y = predecessorBounds.y + Math.sin(pathAngle)*(predecessorDiag + distanceToLineSeg);
		
		//worst case max expansion size of all siblings
		
		//the general slope of the path so far
		double m1 = Math.tan(pathAngle);
		//now we need to know the orientation of this line-segment;
		//it has the slope -1/m
		double m = -1/m1;
		
		//now we need to find the startPoint of the line-segment, from there we place the i-th child
		//(according to its childID) at a distance of chilID*predecessorDiag following the line with 
		//slope m
		double lineSegAngle = Math.atan(m);
		double deltaX = (Math.sin(lineSegAngle)*lineSegLength/2);
		double deltaY = (Math.cos(lineSegAngle)*lineSegLength/2);
		
		double lineSegStartPointX = (x - deltaX);
		double lineSegStartPointY = (y - deltaY);
			
		//now place the i-th child (according to its childID) at a distance of 
		//chilID*predecessorDiag from lineSegStartPoint in the direction of the line-segment	
		double predDiagXDistanceOnSlopeM = Math.cos(lineSegAngle)*(predecessorDiag);
		double predDiagYDistanceOnSlopeM = Math.sin(lineSegAngle)*(predecessorDiag);
		
		int xRet = (int)(lineSegStartPointX + predDiagXDistanceOnSlopeM*childID);
		int yRet = (int)(lineSegStartPointY + predDiagYDistanceOnSlopeM*childID);
		
		return new Point(xRet, yRet);
	}
	
	/**
	 * Save the generated {@link AdoraModel} into a file.
	 * 
	 * @param path The path of the file where the {@link AdoraModel} should be stored.
	 */
	private void saveModel(AdoraModel modelToSave, String path) {
		try {
			
			File f = new File(path);
			OutputStream fileOut = new FileOutputStream (f); 
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			
			System.out.println("path" + path);
			Runtime runtime = Runtime.getRuntime();
			System.out.println ("Free memory : "  + runtime.freeMemory() );

			ModelSerializer.serializeModelToXMLOutputStream(modelToSave, out, false);

			out.close();
			out.writeTo(fileOut);
			fileOut.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Builds a random permutation
	 * 
	 * @param list The {@link List} of which a random permutation is built.
	 * @return A random permutation of <li>list<\li>.
	 */
	private List<Integer> randomPermutation(List<Integer> list) {
		for (int i=list.size()-1;i>0;i--) {
			int randInd = (int)(Math.random()*i);
			int t = list.get(i);
			list.set(i, list.get(randInd));
			list.set(randInd, t);
		}
		return list;
	}
}