package ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.model.util.ModelSerializer;

/**
 * This class generates random {@link AdoraModel}s according to the following complexity parameters:
 *  mindecCnt			Min Number of {@link Decision}s to be created	
 *  maxDecCnt			Max Number of {@link Decision}s to be created	
 *  decInc				Incrementation of {@link Decision} count
 *  cntSameCmplMod		How many models should be generated of the same complexity
 * 
 *  hierCnt				Number of different {@link AspectContainer} paths.
 *  hierDepth			Average depth, i.e. average number of {@link AspectContainer}s
 *							on each of the commonality paths.
 *
 *  minConstCnt			Min	Number of {@link Constraint}s to be created	
 *  maxConstCnt			Max Number of {@link Constraint}s to be created	
 *  constInc			Incrementation of {@link Constraint} count
 *  constSize			Average number of {@link Decision} per {@link Constraint}.
 *  constComplexity		Average number of conjunctive terms per {@link Constraint}.
 *
 *  minVpCnt			Min	Number of {@link VariationPoint}s to be created
 *  maxVpCnt			Max	Number of {@link VariationPoint}s to be created
 *  vpInc				Incrementation of {@link VariationPoint} count
 * 							
 *  vpSize				Average number of {@link Decision} per {@link VariationPoint}.
 *  variance			The global variance parameter for all randomly generated values.
 *  
 *  
 *  The actual profiling / modelgenration works like this:
 *  
 *  for (int decCnt = minDecCnt; decCnt < maxDecCnt; decCnt += decInc)  
 *	 	for (int constCnt = minConstCnt; constCnt < maxConstCnt; constCnt += constInc) 
 *			for (int vpCnt = minVpCnt; vpCnt < maxVpCnt; vpCnt += minVpCnt) 
 *				profile AdoraModel with given complexity parameters
 *
 *  
 *  {@link AdoraModel} generation: 
 *  We first generate a {@link Component} as the commonality and connect it to (<i>decCnt<\i> / <i>hierDepth<\i>) many disjoint 
 *  commonality paths each having an average of <i>hierDepth<\i> many {@link AspectContainer}s. Each of these {@link AspectContainer}s
 *  contains a {@link Component} which is connected by a {@link Decision} (i.e. a {@link JoinRelation}) to the 
 *  {@link Component} contained in the preceding {@link AspectContainer} of the given commonality path.
 *  
 *  {@link Constraint} generation:
 *  For each {@link AdoraModel} of size <i>decCnt<\i>, we generate <i>constCnt<\i> many {@link Constraint}s.
 *  A {@link Constraint} has the form of a random k-CNF formula implying a random k-CNF formula.
 *  These k-CNF formulas have <i>constComplexity<\i> many clauses and involve <i>constSize<\i> many {@link Decision}s.
 *  
 *  {@link VariationPoint}generation:
 *  For each {@link AdoraModel} of size <i>decCnt<\i>, we generate <i>vpCnt<\i> many {@link VariationPoint}s.
 *  A {@link VariationPoint} has a randomly generated min and max cardinality.
 *  A {@link VariationPoint} involves <i>vpSize<\i> many {@link Decision}s on average.
 *  
 *  All randomly generated values are Gaussian distributed, each with a specific mean but with the global <i>variance<\i>.
 *  
 * 
 * @author ivovigan
 */
public class SATprofiler {

	private final boolean RUN_ON_SERVER = true;

	/**
	 * Min Number of {@link Decision}s to be created
	 */
	private int minDecCnt;
	
	/**
	 * Max Number of {@link Decision}s to be created
	 */
	private int maxDecCnt;
	
	/**
	 * Number by which number of {@link Decision}s get incremented.
	 */
	private int decInc;
	
	/**
	 * Average depth, i.e. average number of {@link AspectContainer}s
	 * on each of the commonality paths.
	 */
	private int hierDepth; 
	
	/**
	 * Average number of {@link Decision} per {@link VariationPoint}.
	 */
	private int vpSize; 
	
	/**
     *Average number of {@link Decision} per {@link Constraint}.
 	 */
	private int constSize;
	
	/**
	 * Average number of conjunctive terms per {@link Constraint}.
	 */
	private int constComplexity;
	
	/**
	 * The global variance parameter for all randomly generated values.
	 */
	private int variance;
	
	/**
	 * Stores the information of how many {@link AdoraModel}s of the same 
	 * complexity should be generated.
	 */
	private int cntSameComplexityModels;
	
	/**
	 * Contains the path of the folder in which the models should be stored.
	 */
	private final String LOAD_PATH_MODEL;
	
	/**
	 * Contains the path used to store the profiler log file.
	 */
	private final String SAVE_PATH_LOG;

	private int minConstCnt;

	private int maxConstCnt;

	private int constInc;

	private int minVpCnt;

	private int maxVpCnt;

	private int vpInc;
	
	private double childCnt;
	

	/**
	 * Determines the ration of the number of {@link Constraint}s in relation to 
	 * the number of {@link VariationPoint}.
	 */
	private double constVpRatio;
	
	
	/**
	 * testmethod
	 * @param args
	 */
	public static void main(String [] args) {
		SATprofiler generator = new SATprofiler(
					
				   200, // -minDecCnt, 
				   200, // -maxDecCnt,
				   2, // -decInc,
				   1, // -cntSameComplexityModels,
				   30, // -hierDepth, 
				   3.5, // avgChildCnt
				   20, // -minConstCnt, 
				   20, // -maxConstCnt, 
				   1, // -constInc,
				   3, // -constSize,
				   2, // -constComplexity,
				   2, // -minVpCnt,   NOT-USED
				   3, // -maxVpCnt,   NOT-USED
				   1, // -vpInc,      NOT-USED
				   2, // -vpSize,     
				   2,// -variance
				   0.5);  //constTOvpRatio
		
		System.out.println("SATprofiler started");
		generator.startProfiling();
	}
	
	public SATprofiler(int minDecCnt, 
			   int maxDecCnt,
			   int decInc,
			   int cntSameComplexityModels,
			   int hierDepth, 
			   double childCnt,
			   int minConstCnt, 
			   int maxConstCnt, 
			   int constInc,
			   int constSize,
			   int constComplexity,
			   int minVpCnt,
			   int maxVpCnt,
			   int vpInc,
			   int vpSize,
			   int variance,
			   double constVpRatio) {

		this.minDecCnt = minDecCnt; 
		this.maxDecCnt = maxDecCnt; 
		this.decInc = decInc;
		this.cntSameComplexityModels = cntSameComplexityModels;
		
		this.hierDepth = hierDepth; 
		this.childCnt = childCnt;
		
		this.minConstCnt = minConstCnt; 
		this.maxConstCnt = maxConstCnt; 
		this.constInc = constInc; 
		this.constSize = constSize; 
		this.constComplexity = constComplexity;
		
		this.minVpCnt = minVpCnt; 
		this.maxVpCnt = maxVpCnt; 
		this.vpInc = vpInc; 
		this.vpSize = vpSize; 
		
		this.variance = variance;
		this.constVpRatio = constVpRatio;
		
		if (RUN_ON_SERVER) {
			LOAD_PATH_MODEL = "C:\\SATprofiling\\models";
			SAVE_PATH_LOG = "C:\\SATprofiling\\logs\\log.txt";
		} else {
			LOAD_PATH_MODEL = "/Users/ivovigan/Documents/runtime-EclipseApplication/A/Profiling";
			SAVE_PATH_LOG = "/Users/ivovigan/Documents/runtime-EclipseApplication/LOGS/log.txt";
			
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public List<List<List<long[]>>> startProfiling() {
		
		
		List<List<List<AdoraModel>>> models = new ArrayList<List<List<AdoraModel>>>();
		models = loadModels();
		
		List<List<List<long[]>>> performance = new ArrayList<List<List<long[]>>>();
		
		writeInitialLOG();
		
		System.out.print("Profiling");
		
		for (List<List<AdoraModel>> decList : models) {
			List<List<long[]>> constTimes = new ArrayList<List<long[]>>();
			for (List<AdoraModel> constList : decList) {
				List<long[]> modelTimes = new ArrayList<long[]>();
				for (AdoraModel model : constList) {
					long[] modelPerformance = profileModel(model);
					modelTimes.add(modelPerformance);
					System.out.print(".");
				}
				constTimes.add(modelTimes);
			}
			performance.add(constTimes);
		}
		
		writeLOG(performance);
		System.out.print("Done");
		
		return performance;
	}

	/**
	 * Perform the actual profiling.
	 * 
	 * @param model
	 * @return
	 */
	private long[] profileModel(AdoraModel model) {
		SatSolver solver = new SatSolver(model);
		solver.solve();
		return solver.getSATperfomance();
	}


	private List<List<List<AdoraModel>>> loadModels() {
		
		System.out.print("Loading Models");
		
		List<List<List<AdoraModel>>> models = new ArrayList<List<List<AdoraModel>>>();
		//int cntDecs;
		
		File baseDir = new File(LOAD_PATH_MODEL);
		if (!baseDir.exists() || !baseDir.isDirectory()) {
			return models;
		}
		
		//cntDecs = Integer.parseInt(baseDir.getName().substring(baseDir.getName().indexOf("Decs")));
		
		//load the decision folders
		File[] decDirs = baseDir.listFiles();
		for (int i = 0; i < decDirs.length; i++) {
			File decDir = decDirs[i];
			if (!decDir.isDirectory())
				continue;
			
			List<List<AdoraModel>> decModels = new ArrayList<List<AdoraModel>>();
			
			//load the constraint folders
			File[] constDirs = decDir.listFiles();
			for (int j = 0; j < constDirs.length; j++) {
				File constDir = constDirs[j];
				if (!constDir.isDirectory())
					continue;
			
				List<AdoraModel> constModels = new ArrayList<AdoraModel>();
				
				//load the adoramodels
				File[] curComplModels = constDir.listFiles();
				for (int k = 0; k < curComplModels.length; k++) {
				
					File curFile = curComplModels[k];
					
					AdoraModel model = loadModel(curFile);
					constModels.add(model);
					System.out.print(".");
				}
				decModels.add(constModels);
			}
			models.add(decModels);
		}
		System.out.println("models loaded");
		return models;
	}
	
	/**
	 * Simulates the user setting k {@link Decision} and 
	 * returns the profiling information of the {@link SatSolver}.
	 * 
	 * @param k 		The number of randomly selected {@link Decision}s 
	 * 					to be manually set to a random truth value.
	 * @param model		The {@link AdoraModel} on which the {@link Decision}s are set.
	 * 
	 * @return			A {@link List} containing the profiling information of the {@link SatSolver}.
	 */
	private List<long[]> setKrandomDecisions(AdoraModel model, int k) {
		List<Decision> manuallySetDecisions = getNrandomDecs(model, k);
		
		List<long[]> satPerformance = new ArrayList<long[]>();
		
		for (Decision d : manuallySetDecisions) {
			
			d.setDecisionValue(Math.random() > 0.5);
			SatSolver solver = new SatSolver(model);
			solver.solve(d, null);
			satPerformance.add(solver.getSATperfomance());
			
		}
		return satPerformance;
	}
	
	/**
	 * This method is used to randomly select n {@link Decision}s 
	 * from the existing {@link Decision}s in the {@link AdoraModel}.
	 *  
	 *  
	 * @param n the number of {@link Decision}s to be generated.
	 * @return Decisions n randomly selected {@link Decision}s.
	 */
	private List<Decision> getNrandomDecs(AdoraModel model, int n) {
		
		if (n > model.getDecisions().size())
			n = model.getDecisions().size();
		
		List<Decision> randomDecs = new ArrayList<Decision>();
		
		Collection<Decision> modelDecs = model.getDecisions();
		
		//generate n distinct random numbers between 1 and n
		List<Integer> randInds = new ArrayList<Integer>(); 
		for (Decision d : modelDecs) {
			randInds.add(d.getNr());
		}
		
		//now remove 
		for (int i = n+1; i <= modelDecs.size() ; i++) {
			int randomIndex = (int)(Math.random()*randInds.size());
			randInds.remove(randomIndex);
		}
		
		for (int i = 0; i < n; i++) {
			int randomIndex = randInds.get(i);
			randomDecs.add(model.getDecisionsMap().get(randomIndex));
		}
		return randomDecs;
	}
	
	private AdoraModel loadModel(File f){
		AdoraModel model = null;
		
		try {
			FileInputStream fis = new FileInputStream(f);
			model = ModelSerializer.deserializeModelFromXMLInputStream(fis, true);
			fis.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return model;
	}
	

	private void writeInitialLOG() {
		try {
			File file = new File(SAVE_PATH_LOG);
			FileWriter fos = new FileWriter(file);
			fos.write("-------------------\n");
			fos.write("mindecCnt \t" + String.valueOf(minDecCnt)+"\n"); 
			fos.write("maxDecCnt \t" + String.valueOf(maxDecCnt)+"\n"); 
			fos.write("decInc \t\t" + String.valueOf(decInc)+"\n");
			fos.write("cntSameCmplMod \t" + String.valueOf(cntSameComplexityModels)+"\n"); 
			fos.write("hierDepth \t" + String.valueOf(hierDepth)+"\n");
			fos.write("minConstCnt \t" + String.valueOf(minConstCnt)+"\n");
			fos.write("maxConstCnt \t" + String.valueOf(maxConstCnt)+"\n"); 
			fos.write("constInc \t" + String.valueOf(constInc)+"\n");
			fos.write("constSize \t" + String.valueOf(constSize)+"\n");
			fos.write("constComplexity " + String.valueOf(constComplexity)+"\n"); 
			fos.write("minVpCnt \t" + String.valueOf(minVpCnt)+"\n");
			fos.write("maxVpCnt \t" + String.valueOf(maxVpCnt)+"\n");
			fos.write("vpInc \t\t" + String.valueOf(vpInc)+"\n");
			fos.write("vpSize \t\t" + String.valueOf(vpSize)+"\n");
			fos.write("variance \t" + String.valueOf(variance)+"\n");
			fos.write("-------------------\n");
			fos.write("\n");
			fos.write("\n");
			
			fos.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Writes the sat profiling information to a log file.
	 * @param vpCnt 
	 * @param constCnt 
	 * @param decCnt 
	 * 
	 * @param The profiling information
	 */
	private void writeLOG(int decCnt, int constCnt, int vpCnt, List<long[]> profile) {
		try {
			File file = new File(SAVE_PATH_LOG);
			FileWriter fos = new FileWriter(file, true);
			fos.write("-------------------\n");
			fos.write("decCnt \t" + String.valueOf(decCnt)+"\n");
			fos.write("constCnt \t" + String.valueOf(constCnt)+"\n");
			fos.write("vpCnt \t" + String.valueOf(vpCnt)+"\n");
			fos.write("-------------------\n");
			
			for (long[] performance : profile) {
				for (int i = 0; i < performance.length; i++) {
					fos.write(String.valueOf(performance[i])+"\n");
				}
				fos.write("-------------------\n");
			}
			fos.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	 * 
	 * @param profile
	 */
	private void writeLOG(List<List<List<long[]>>> profile) {
		try {
			File file = new File(SAVE_PATH_LOG);
			FileWriter fos = new FileWriter(file, true);
			for (List<List<long[]>> decList : profile) {
				for (List<long[]> constList : decList) {
					for (long[] modelTime : constList) {
						for (int i = 0; i < modelTime.length; i++) {
							fos.write(String.valueOf(modelTime[i])+"\n");
						}
						fos.write("-------------------\n");
					}
				}
			}
			fos.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}