package ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat;

import java.io.StringBufferInputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.sat4j.core.VecInt;
import org.sat4j.specs.IProblem;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.TimeoutException;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.parser.ConstraintParser;
import ch.unizh.ifi.adora.productline.ui.views.decisiontable.DecisionTableViewPart;

/**
 * The basic idea is the following: After having specified
 * {@link VariationPoint}s and {@link Constraint}s we have a lock at the
 * {@link DecisionTableViewPart}: There we see the column "Decision" which has
 * all values set to "not decided". We focus on {@link Decision}_i and the
 * "IF-True", "IF-False" columns and wan't to fill them for each
 * {@link Decision} with logical expressions s.t. all {@link VariationPoint}s
 * and and all {@link Constraint}s get evaluated to true if the Decision value
 * of {@link Decision}_i has been set to true, false resp.
 * 
 * After computing the mandatory truthvalues of all {@link Decision}s we update
 * the Decisions column according to the truth values.
 * 
 * {@link VariationPoint} vp n:m => n-1 < |{Decision d \in vp : d = true }| <
 * m+1
 * 
 * @author vigan
 */
public class SatSolver {
	
	/**
	 * The maximum number of variables allowed for satsolving
	 */
	private static int MAXVAR = 31;
	
	/**
	 * The {@link AdoraModel} on which the {@link Decision}-setting is tested.
	 */
	private AdoraModel model;
	
	/**
	 * This array stores the time used to perform certain tasks. 
	 * 
	 * satPerformence[0] = TimeStamp before solving 
	 * satPerformence[1] = TimeStamp after checking satisfiability of current setting
	 * satPerformence[2] = TimeStamp after finding minimal distance solution (optional)
	 * satPerformence[3] = TimeStamp after constraint propagation
	 * satPerformence[4] = TimeStamp after update ifTrue/ifFalse columns
	 */
	private long[] satPerformence = new long[5];
	
	/**
	 * Instantiates a new {@link SatSolver} for a given {@link AdoraModel}. 
	 * 
	 * @param model The {@link AdoraModel} which gets tested.
	 */
	public SatSolver(AdoraModel model) {
		this.model = model;
	}
	
	/**
	 * Checks whether the current {@link Decision} setting
	 * is satisfiable.
	 * 
	 * @param changedDec
	 *            The {@link Decision} which has been edited in the
	 *            DecisionTable.
	 * @param oldDecValue
	 *            The truth-value of the {@link Decision} before the changes
	 *            should be processed with truth-value true or false
	 *            
	 * @return <li>true<\li> iff the current {@link Decision} setting
	 * is satisfiable.
	 */
	public boolean solve(Decision changedDec, Boolean oldDecValue) {

		satPerformence[0] = System.currentTimeMillis();
		
		//reset automatically set decisions which are not fixed
		for (Decision d : model.getDecisions()) {
			if (d.wasDecisionSetAutomatically()) {
				if (d.isFixed())
					//only remove the autoSet-flag
					d.setDecisionValueManually(d.getDecisionValue());
				else
					d.setDecisionValue(null);
			}
		}

		boolean curSettingSAT = true;
		
		if (changedDec == null || changedDec.getDecisionValue() != null) {
			//2nd argument in processConstraints call, false or true, 
			//doesn't matter since decision is null anyway
			curSettingSAT = processConstraints(null, false, true); 
		} else {
			//The user changed a decision d to undecided
			//then the current setting is satisfiable if it is satisfiable 
			//in both cases d=true AND d=false
			boolean dTrue 		 = processConstraints(changedDec, true, true);
			boolean dFalse		 = processConstraints(changedDec, false, true);
			
			//since changedDec is set to undecided there has to be a satisfying assignment for  
			//both cases d=true AND d=false
			curSettingSAT = dTrue && dFalse; 
		}
		
		satPerformence[1] = System.currentTimeMillis();
		
		if (!curSettingSAT) {
			//since current setting was not satisfiable
			//we try to find a solution with minimum difference. 
			
			Map<Decision, Boolean> minDistSol = findMinimalDistanceSolution(changedDec);
			curSettingSAT = minDistSol.size() > 0;
			 
			for (Decision d : minDistSol.keySet()) { 
				if (!d.isFixed()) //probably we dont need this if
					d.setDecisionValueAutomatically(minDistSol.get(d));
			}
			satPerformence[2] = System.currentTimeMillis();
		}
			
		if (!curSettingSAT && changedDec != null) {
			changedDec.setDecisionValue(oldDecValue);
		}
		
		//fix the decisions for which only one truth assignment exists
		constraintPropagation(changedDec);
		
		satPerformence[3] = System.currentTimeMillis();
		
		//update ifTrue/ifFalse columns
		for (Decision dec : model.getDecisions()) {
			//if (dec.getDecisionValue() == null) test for all values
			updateIFtrueIFfalseValues(dec);
		}
		satPerformence[4] = System.currentTimeMillis();;
			
		return curSettingSAT;
	}
	
	/**
	 * Checks whether the current {@link Decision} setting
	 * is satisfiable.
	 * 
	 * @return <li>true<\li> iff the current {@link Decision} setting
	 * is satisfiable.
	 */
	public boolean solve() {
		return solve(null, null);
	}
	
	/**
	 * Method used to check whether the model in its current state is satisfiable.
	 * The difference to the solve methods is that neither constraint-propagation is performed, 
 	 * nor the iftrue/iffalse columns are updated.
	 * 
	 * @return true iff the model is satisfiable.
	 */
	public boolean checkSatisfiability() {
		return processConstraints(null, false, true); 
	}

	/**
	 * This method is used to find a solution for the constraints in 
	 * case the current {@link Decision} setting is unsatisfiable.
	 * 
	 * The policy is to find a satisfiable solution which has 
	 * minimal difference to the current {@link Decision} settings, i.e. 
	 * a setting which alters the minimum amount of {@link Decision}s
	 * in order to find a new satisfiable solution.
	 * 
	 * @param changedDec The {@link Decision} which got altered.
	 * 
	 * @return true if there exists a satisfiable assignment where
	 * the <i>changedDec</i> is fixed, otherwise false.
	 */
	private Map<Decision, Boolean> findMinimalDistanceSolution(Decision changedDec) {
		
		
		Map<Decision, Boolean> minDistSetting = new HashMap<Decision, Boolean>();
		
		Boolean changedDecValue = null;
		if (changedDec != null) {
			changedDecValue = changedDec.getDecisionValue();
		}
		
		Collection<Decision> decisionMap = model.getDecisions();
		
		List<Decision> setDecisions = new ArrayList<Decision>();
		Map<Decision, Boolean> oldDecValues = new HashMap<Decision, Boolean>();
		
		//get all the decisions which are set
		for (Decision d : decisionMap) {
			if (d != changedDec && d.getDecisionValue() != null) {
				setDecisions.add(d);
				oldDecValues.put(d, d.getDecisionValue());
			}
		}
		
		//now test which maximum size subset satisfies the constraints
		for (int n = 1; n <= setDecisions.size(); n++) {
			
			List<List<Decision>> decSubsets = getAllSubsetsOfSize(n, setDecisions);
			
			for (List<Decision> decSubset : decSubsets) {
				
				//TODO: we somehow have to prefer undecided values from true values
				//Here we set the decisions according to the 
				//exhaustively generated decision-value-subsets
				//but we have to forbid to change fixed decisions
				for (Decision dec : decSubset) {
					if (!dec.isFixed())
						dec.testDecisionValue(!dec.getDecisionValue());	
				}
				
				//if the user changed a decision d to undecided, the constraints have to be satisfiable
				//in both cases d = true, d = false
				boolean sat;
				if (changedDec != null && changedDecValue == null) {
					sat = processConstraints(changedDec, true, true);
					sat = sat && processConstraints(changedDec, false, true);
				} else {
					sat = processConstraints(changedDec, changedDecValue, true);
				}
				
				//TODO: IV this can probably be removed because we dont have the setting problem anymore
				//if (changedDec != null)
				//	changedDec.testDecisionValue(changedDecValue);

				if (sat) {
					//set the decision. The difference to test is that a property change message gets fired
					//furthermore the automatically set flag is set 
					for (Decision dec : decSubset) {
						minDistSetting.put(dec, dec.getDecisionValue());
					}
					return minDistSetting;
				}

				//reset the decision values to the value before we tested it
				for (Decision dec : decSubset) {
					dec.testDecisionValue(oldDecValues.get(dec));	
				}
			}
		}
		return minDistSetting;
	}
	
	/**
	 * This method is responsible for the constraint propagation.
	 * This means that it fixes the truth-values of the {@link Decision}s which
	 * appear in the same polarity (i.e. same literal value) in every satisfying assignment.
	 * 
	 * @param changedDec The {@link Decision} which got changed by the user
	 * We need to know this {@link Decision} because we don't want to propagate it.
	 */
	private void constraintPropagation(Decision changedDec) {
		
		for (Decision curD : model.getDecisions()) {
			
			if (curD == changedDec)
				continue;
			
			boolean dFalse		 = processConstraints(curD, false, true);
			boolean dTrue 		 = processConstraints(curD, true, true);
			
			if (curD.wasDecisionSetAutomatically()  && dFalse && dTrue) {
				//if the decision was set automatically, 
				//check if we can reset it to undecided
				if (!curD.isFixed())
					curD.setDecisionValueManually(null);
				else 
					curD.setDecisionValueManually(curD.getDecisionValue());
			} else if (!dFalse && dTrue) {
				curD.setDecisionValueAutomatically(true);
			} else if (dFalse && !dTrue) {
				curD.setDecisionValueAutomatically(false);
			} else {
				//currently nothing to do
			}
		}
	}
	
	/**
	 * This method is used to determine the minimum distance solution
	 * which satisfies all constraints.
	 * 
	 * @param subSize The size of the subset
	 * @param set The set from which the subset-expressions are generated
	 * @return A {@link List} of {@link List} of {@link Decision} which stores 
	 * 		   all the possible subsets of size subSize.
	 */
	private List<List<Decision>> getAllSubsetsOfSize(int subSize, List<Decision> set) {
		
		List<List<Decision>> subsets = new ArrayList<List<Decision>>();
		
		if (subSize == 0)
			return subsets;
	
		int n = set.size();
		
		for (int k = 0; k < Math.exp(Math.log(2) * n); k++) {
			
			String s = Integer.toBinaryString(k);
			int length = s.length();
			int cnt1 = 0;
			boolean candidate = true;
			for (int l = 0; l < length; l++) {
				if (Integer.parseInt(s.substring(l, l + 1)) == 1) {
					cnt1++;
					if (cnt1 > subSize) {
						candidate = false;
						break;
					}
				}
			}
			
			//in the above loop we checked whether cnt1 is bigger than subSize, 
			//here we need also to check whether its the same
			candidate = candidate && cnt1 == subSize;
			if (!candidate)
				continue;
				
			for (int j = 0; j < n - length; j++)
				s = "0".concat(s);
			
			List<Decision> subset = new ArrayList<Decision>(); 
			for (int j = 0; j < n; j++) {
				String dig = s.substring(j, j + 1);
				if (Integer.parseInt(dig) == 1) {
					subset.add(set.get(j));
				}
			}
			subsets.add(subset);
		}
		return subsets;
	}

	/**
	 * Checks if the constraints is satisfiable if {@link Decision}  d is set to
	 * decision.
	 * 
	 * @param d    		The current undecided {@link Decision} to be processed.
	 * @param decision	Determines whether the current undecided {@link Decision}
	 *            		should be processed with turthvalue true or false
	 * 
	 * @return true if constraints are still satisfiable.
	 */
	private boolean processConstraints(Decision d, Boolean decision, boolean incHierConst) {
		
		Boolean oldVal = null;
		
		if (d != null) { 
			//store the old value before testing
			oldVal = d.getDecisionValue();
			
			d.testDecisionValue(decision);
		}
		
		ISolver solver = generateSolver(incHierConst);
		
		//trivial contradiction if solver is null
		boolean satisfiable = solver != null;
		
		int[] tmp = null;
		try {
			//trivial contradiction
			if (solver != null)
				tmp = ((IProblem) solver).findModel();
		} catch (TimeoutException e) {
			satisfiable = false;
		}

		//if tmp is null, no solution was found
		satisfiable = satisfiable && (tmp != null);
		
		//reset the decision to its old value
		if (d != null)
			d.testDecisionValue(oldVal);
		
		return satisfiable;
	}
	
	/**
	 * Method used to encode the {@link Constraint}  <i>c<\i>  as logical expression.
	 * 
	 * @param c The {@link Constraint} which gets encoded as a logical expression
	 * @return  An expression which is true if the {@link Constraint} <i>c<\i> is met, 
	 * or if <i>c<\i> is fully contained in a subgraph without any true/undecided paths to 
	 * a commonality.  
	 */
	private String createConstExpression(Constraint c) {
		
		String constraintExpr = "";
		
		// Encode "=>", "<=>" (Note: (p => q) <=> (not p v q))
		constraintExpr = "( not ( " + c.getAntecedent() + " ) or ( "
				+ c.getConsequent() + " ) ) and ";
		if (c.getOperation().equals(Constraint.EQUIVALENCE)) {
			constraintExpr += " ( not ( " + c.getConsequent() + " ) or ( "
					+ c.getAntecedent() + " ) ) and ";
		}
		//consider relevance due to hierarchical constraints
		String pathsFalseExpr = c.getAllPathsFalseExpression();
		if (pathsFalseExpr.length() > 0) {
			// remove last "and"
			constraintExpr = constraintExpr.substring(0, constraintExpr.length() - 4); 
			return "(" + constraintExpr + " or (" + pathsFalseExpr + ")) and ";
		}
		
		return constraintExpr;
	}
	
	/**
	 * Method used to encode the {@link VariationPoint}  <i>vp<\i>  as logical expression.
	 * 
	 * @param vp The {@link VariationPoint} which gets tested.
	 * 
	 * @return An expression which is true if the {@link VariationPoint} <i>vp<\i> is met, 
	 * or if <i>vp<\i> is fully contained in a subgraph without any true/undecided paths to 
	 * a commonality.  
	 */
	private String createVpExpression(VariationPoint vp) {
		
		String expr = "";
		
		int min = Integer.parseInt(vp.getMinCard());
		int max = Integer.parseInt(vp.getMaxCard());
		
		int decCnt = vp.getDecisions().size();
		
		int[] decNrs = new int[decCnt];
		
		int i = 0;
		for (Decision d : vp.getDecisions()) {
			decNrs[i] = d.getNr();
			i++;
		}
	
		expr = getAllSubsetsOfSize(min, decNrs) ;
		String maxExp;
		if (max < decCnt) {
			maxExp = getAllSubsetsOfSize(max+1, decNrs) ;
			if (!expr.equals(""))
				 expr += " and";
			expr += " not (" + maxExp + ")";
		}
		
		//handle to hierarchical constraints
		String exprFalsePaths = vp.getAllPathsFalseExpression();
		if (!exprFalsePaths.equals("")) {
			if (!exprFalsePaths.equals("") && !expr.equals(""))
				expr =  "(" + expr +  ") or ";
		
			expr += "(" + exprFalsePaths + ")";
		}
		return expr;
	}

	/**
	 * This method encodes the hierarchical constraints as a logical 
	 * expression 
	 * 
	 * @return An expression encoding the hierarchical constraints. 
	 */
	private String createHierarchyExpression() {
		
		String expr = "";

		for (Decision currDec : model.getDecisions()) {
			
			String pathExpr = currDec.getCommonalityPathExpression();
			if (pathExpr.length() == 0) 
				continue; //should never happen
			
			if (currDec.getDecisionValue() == null)  {
				String curDecExpr = "( not D" + currDec.getNr().toString() +" or (" + pathExpr +"))"; 
				if (expr.equals(""))
					expr =  curDecExpr;
				else
					expr += " and " + curDecExpr;
				
			} else if (currDec.getDecisionValue() == true) {
				
				if (expr.equals(""))
					expr = "("+ pathExpr +")";
				else
					expr += " and " + "("+ pathExpr +")";
			}
		}
		return expr;
	}
	
	
	/**
	 * Generates the actual {@link ISolver}.
	 * 
	 * @param includeHieararchyConstriants Determines, whether or not the hierarchical constraints
	 * should be included in the {@link ISolver}.
	 * 
	 * @return The {@link ISolver} corresponding which includes all the {@link Constraint}s, {@link VariationPoint}s
	 * and optionally the hierarchical constraints. Returns null if the system is not satisfiable.
	 */
	@SuppressWarnings("deprecation")
	private ISolver generateSolver(boolean includeHieararchyConstriants) {
		
		//-------------------
		//add constraints
		String expr = "";
		for (Constraint c : model.getConstraints()) {
			expr += createConstExpression(c);
		}
		if (expr.length() > 3)
			expr = expr.substring(0, expr.length() - 4); // remove last "and"
		if (!expr.equals(""))
			expr =  "(" + expr + ")";
		
		//-------------------
		//add hierarchy constraints if necessary
		if (includeHieararchyConstriants) {
			String hierExpr = createHierarchyExpression();
			if (hierExpr.length() > 0) {
				if (expr.length() > 0)
					expr += " and ";
				expr += "(" + hierExpr+")";
			}
		}
		
		//-------------------
		//add variation point constraints
		String exprVp;
		String exprVps = "";
		
		for (VariationPoint vp : model.getVariationPoints()) {
			exprVp = createVpExpression(vp);
			if (exprVps.length() > 0)
				exprVps = "("+exprVps+") and ";
			exprVps +=  "(" + exprVp + ")";
		}
		if (exprVps.length() > 0) {
    		if (expr.length() > 0)
    			expr +=  " and (" + exprVps + ")";
    		else
    			expr = "(" + exprVps + ")";
		}
	
		
		ISolver solver = new ConstraintParser(new StringBufferInputStream(expr
									.concat("\n")), null, false, model).satParser();
		
		return solver;
	}
		


	/**
	 * Method used to count the number of satisfying assignments
	 * 
	 * @return The number of satisfying assignments or -1 if >MAXVAR {@link Decision}s are
	 *         involved in constraints.
	 */
	public int cntSATassignments() {
		
		ISolver solver = generateSolver(true);
		if (solver == null)
			return 0;
		
		int n = model.getDecisions().size();
		if (n > MAXVAR)
			return -1;

		int[] variables = new int[n];
		
		//Get all the decision numbers
		int l = 0;
		for (Decision d : model.getDecisions()) {
			variables[l] = d.getNr();
			l++;
		}
		
		int[] literals = new int[n];

		int cntAssignments = 0;
		for (int k = 0; k < Math.exp(Math.log(2) * n) && n > 0; k++) {
			String s = Integer.toBinaryString(k);
			int length = s.length();
			for (int j = 0; j < n - length; j++)
				s = "0".concat(s);
				
			for (int i = 0; i < n; i++) {
				if (Integer.parseInt(s.substring(i, i + 1)) == 0)
					literals[i] = -variables[i];
				else
					literals[i] = variables[i];
			}
			try {
				if (solver.isSatisfiable(new VecInt(literals)))
					cntAssignments++;
				} 
				catch (TimeoutException e) {}
		}
		return Math.max(cntAssignments, 1);
	}

	/**
	 * Method used to encode the {@link VariationPoint} constraints.
	 * 
	 * @param subSize The size of the subset
	 * @param set The set from which the subset-expressions are generated
	 * @return A String which encodes all the possible subsets of size subSize.
	 */
	private String getAllSubsetsOfSize(int subSize, int set[]) {
		String expr = "";
		
		if (subSize == 0)
			return expr;
	
		int n = set.length;
		
			for (int k = 0; k < Math.exp(Math.log(2) * n); k++) {
			String s = Integer.toBinaryString(k);
			int length = s.length();
			int cnt1 = 0;
			boolean candidate = true;
			for (int l = 0; l < length; l++) {
				if (Integer.parseInt(s.substring(l, l + 1)) == 1) {
					cnt1++;
					if (cnt1 > subSize) {
						candidate = false;
						break;
					}
				}
			}

			//in the above loop we checked whether cnt1 is bigger than subSize, 
			//here we need also to check whether its the same
			candidate = candidate && cnt1 == subSize;
			if (!candidate)
				continue;
			
				
			for (int j = 0; j < n - length; j++)
				s = "0".concat(s);
			
			String clauseExpr = "";
			for (int j = 0; j < n; j++) {
				String dig = s.substring(j, j + 1);
				if (Integer.parseInt(dig) == 1) {
					clauseExpr += "D"+ Integer.toString(set[j]) + " and ";
				}
			}
			if (clauseExpr.length() > 3)
				clauseExpr = clauseExpr.substring(0, clauseExpr.length() - 4); // remove last "and"
			
			if (!clauseExpr.equals(""))
				expr += "(" + clauseExpr+ ") or ";
		}
		
		if (expr.length() > 2) {
			expr = expr.substring(0, expr.length() - 3); // remove last "or"
			expr = "("+expr+")";
		}
		
		return expr;
	}
		
	/**
	 * This method determines the values for ifTrue,
	 * ifFalse column.
	 * 
	 * @param d The {@link Decision} for which the ifTrue, ifFalse values 
	 * are set.
	 */
	private void updateIFtrueIFfalseValues(Decision d) {
				
		d.setReqIfTrue("");
		d.setReqIfFalse("");
		
		// dTmp is the decision which may be added 
		// in the iftrue/iffalse column of the decision d
		for (Decision dTmp : model.getDecisions()) {
		
			if (dTmp == d)
				continue;

			/*
			 * The vals array stores whether the formula is satisfiable if d
			 * is set to x and dTmp is set to y, x,y \in {true, false}.
			 * 
			 * d\dTmp ------------- 
			 * 		  |true | false |
			 * 		  |true | false | 
			 * 		  -------------- 
			 */
			boolean[][] vals = new boolean[2][2];
			
			Boolean oldDVal = d.getDecisionValue();
			Boolean oldDtmpVal = dTmp.getDecisionValue();
			
			//fill in the true column
			if (d.getDecisionValue() == null || !d.getDecisionValue()) {
			
				d.testDecisionValue(true);
				
				vals[0][0] = processConstraints(dTmp, true, true); 
				vals[0][1] = processConstraints(dTmp, false, true);
				
				if ((dTmp.getDecisionValue() == null || !dTmp.getDecisionValue()) && (vals[0][0] && !vals[0][1])) {
					d.setReqIfTrue(d.getReqIfTrue() + " D" + dTmp.getNr());
				} else if ((dTmp.getDecisionValue() == null || dTmp.getDecisionValue()) && (!vals[0][0] && vals[0][1])) {
					d.setReqIfTrue(d.getReqIfTrue() + " D" + dTmp.getNr());
				} else if (!vals[0][0] && !vals[0][1]) {
					// should never happen
					//find min changing dec
					String reqIfTrue = ifTrueifFalseExpressionForSetDecisions(d, dTmp);
					d.setReqIfTrue(reqIfTrue);
				}
			}
			d.testDecisionValue(oldDVal);
			dTmp.testDecisionValue(oldDtmpVal);

			
			//fill in the false column
			if (d.getDecisionValue() == null || d.getDecisionValue()) {
			
				d.testDecisionValue(false);
				vals[1][0] = processConstraints(dTmp, true, true);
				vals[1][1] = processConstraints(dTmp, false, true);
				
				if ((dTmp.getDecisionValue() == null || !dTmp.getDecisionValue()) && (vals[1][0] && !vals[1][1])) {
					d.setReqIfFalse(d.getReqIfFalse() + " D"+ dTmp.getNr());
				} else if ((dTmp.getDecisionValue() == null || dTmp.getDecisionValue()) && (!vals[1][0] && vals[1][1])) {
					d.setReqIfFalse(d.getReqIfFalse() + " D"+ dTmp.getNr());
				} else if (!vals[1][0] && !vals[1][1]) {
					String reqIfFalse = ifTrueifFalseExpressionForSetDecisions(d, dTmp);
					d.setReqIfFalse(reqIfFalse);
				}
			}
			//reset to old value
			d.testDecisionValue(oldDVal);
			dTmp.testDecisionValue(oldDtmpVal);
		}
	}
	
	/**
	 * If a {@link Decision} is set to a certain value, we still want to fill in the ifTrue/ifFalse
	 * column of the opposite polarity. But under certain circumstances, for example if the {@link Decision}
	 * was automatically set, there is no single {@link Decision} which has to be propagated in order 
	 * to satisfy the setting.  
	 * 
	 * @param d
	 * @param dTmp
	 * @return A {@link String} representing
	 */
	private String ifTrueifFalseExpressionForSetDecisions(Decision d, Decision dTmp) {
		
		boolean wasDfixed = d.isFixed();
		d.setFixed(true);
		Map<Decision, Boolean> minDistSol = findMinimalDistanceSolution(dTmp);
		d.setFixed(wasDfixed);
		
		if (minDistSol.keySet().isEmpty())
			return "Unsatisfiable";
			
		String expr = "";
		
		for (Decision dSol : minDistSol.keySet()) {
			Boolean dSolVal = minDistSol.get(dSol);
			if (dSolVal == null) {
				expr += " ?D"+dSol.getNr();
			} else if (dSolVal) {
				expr += " D"+dSol.getNr();
			} else {
				expr += " D"+dSol.getNr();
			}
		}
		return "["+expr+"]";
	}
	
	/**
	 * This method is used to evaluate the performance of the {@link SatSolver}.
	 * It returns an array containing the times used to perform the tasks specified below.  
	 * 
	 * @return 	performance[0] = Time used for checking satisfiability of current setting
	 * 			performance[1] = Time used for finding minimal distance solution (optional)
	 * 			performance[2] = Time used for constraint propagation
	 * 			performance[3] = Time used for updating ifTrue/ifFalse columns
	 */
	public long[] getSATperfomance() {
		
		long[] performance = new long[4];
		
		performance[0] = satPerformence[1] - satPerformence[0]; 
		if (performance[2] != 0) {
			performance[1] = satPerformence[2] - satPerformence[1];
			performance[2] = satPerformence[3] - satPerformence[2];
		} else {
			performance[1] = 0;
			performance[2] = satPerformence[3] - satPerformence[1];
		}
		performance[3] = satPerformence[4] - satPerformence[3];
		
		return performance;
	}
	
}