package ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.qmcalgorithm;

import java.io.StringBufferInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.sat4j.core.VecInt;
import org.sat4j.specs.IProblem;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.TimeoutException;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.parser.ConstraintParser;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * Implementation of the Quine-McClusky algorithm.
 * 
 * @author ivigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Nov 29, 2009 IV First Version
 **/
public class QMCalgorithm {

	/**
	 * The {@link AdoraModel} on which the expressions get simplified.
	 */
	private AdoraModel model;
	
	public QMCalgorithm(AdoraModel model) {
		this.model = model;
	}
	
	
	/**
	 * Main method which simplifies a logical expression using the 
	 * Quine-McClusky algorithm.
	 * 
	 * @param expression The logical expression to be simplified.
	 * 
	 * @return The simplified expression.
	 */
	public String simplify(String expression) {
		
		//nothing to optimize
		if (expression.toLowerCase().equals("true") || 
				expression.toLowerCase().equals("false") || 
				expression.matches("(D)[0-9]+") ||
				expression.matches("not (D)[0-9]+"))
			
			return expression;
		
		List<Integer> variables = getInovedDecisions(expression);

		List<Term> terms = generateMineTerms(expression, variables);
		                 
	    Formula f = new Formula(terms);
	    f.reduceToPrimeImplicants();
	    f.reducePrimeImplicantsToSubset();
	    
	    String reducedExp = formulaToString(f, variables);
	    
	    if (cntOps(reducedExp) < cntOps(expression))
    	    return reducedExp;
	    
	    return expression;
	}
	
	/**
	 * Returns the number of operands (and/or/xor) in this expression.
	 * 
	 * @param expression The expression to be analyzed.
	 * @return The number of operands (and/or/xor) in this expression.
	 */
	public static int cntOps(String expression) {
		
		String[] pat = {"and", "or"}; //note that xor gets counted this way too
		
        int count = 0;
        int index = 0;
        
        for (int i = 0; i < pat.length; i++) {
            while ((index = expression.indexOf(pat[i], index)) != -1) {
                 ++index;
                 ++count;
            }
        }
        return count;
   }


	/**
	 * Returns a {@link List} of Mine-{@link Term}s which encode the logical 
	 * expression.
	 * 
	 * @param expression The logical expression.
	 * @return A {@link List} of Mine-{@link Term}s which encode the logical 
	 * expression.
	 */
	private List<Term> generateMineTerms(String expression, List<Integer> variables) {
		
		List<Term> terms = new ArrayList<Term>();
		
		ISolver solver = new ConstraintParser(
								new StringBufferInputStream(expression
												.concat("\n")), null, false, model).satParser();
		
	
		int n = variables.size();
		
		int[] literals = new int[n];
		
		for (int k = 0; k < Math.exp(Math.log(2) * n); k++) {
			String s = Integer.toBinaryString(k);
			int length = s.length();
			for (int j = 0; j < n - length; j++)
				s = "0".concat(s);
					
			for (int i = 0; i < n; i++) {
				if (Integer.parseInt(s.substring(i, i + 1)) == 0)
					literals[i] = -variables.get(i);
				else
					literals[i] =  variables.get(i);
			}
			try {
				if (((IProblem) solver).isSatisfiable(new VecInt(literals))) {
					char[] binChar = s.toCharArray();
					byte[] zeroOne = new byte[n];
					for (int cCnt = 0; cCnt < n; cCnt++) {
						if (binChar[cCnt] == '1')
							zeroOne[cCnt] = 1;
						else 
							zeroOne[cCnt] = 0;
					}
					terms.add(new Term(zeroOne));
				}
			} 
			catch (TimeoutException e) {}
		}
		return terms;
	}
	
	/**
	 * Returns the {@link Decision} numbers contained in a the logical expression. 
	 * 
	 * @param expression A logical expression.
	 * @return The {@link Decision} numbers contained in a the logical expression.
	 */
	private List<Integer> getInovedDecisions(String expression) {
		List<Integer> decisions = new ArrayList<Integer>();
		
        Pattern p = Pattern.compile("(D)[0-9]+");
        
        java.util.regex.Matcher m = p.matcher(expression);
        
        while (m.find()) {
            String[] splitted=m.group().split("D");
            if (!decisions.contains(Integer.parseInt(splitted[1])))
            		decisions.add(Integer.parseInt(splitted[1]));
        }
        return decisions;
	}
	
	/**
	 * Converts a {@link Formula} to a {@link String}
	 * 
	 * @param f {@link Formula}
	 * @return The corresponding {@link String}.
	 */
	private String formulaToString(Formula f, List<Integer> variables) {
		String expression = "";
		
		List<Term> terms =  f.getTerms();
		
		for (Term t : terms) {
			String termStr = t.toString();
			String termExpr = "";
			termStr = termStr.substring(1, termStr.length()-1);
			termStr = termStr.replaceAll("\\s", "");
			
			for (int i = 0; i < termStr.length(); i++) {
				if (termStr.charAt(i) == '0') {
					termExpr = termExpr + " and not D" + variables.get(i).toString();
				} else if (termStr.charAt(i) == '1') {
					termExpr = termExpr + " and D" + variables.get(i).toString();
				}
			}
			
			termExpr = termExpr.substring(5);
			
			if (termExpr.contains("and") || termExpr.contains("or"))
				expression += " or " + "("+termExpr+")";
			else
				expression += " or " + termExpr;
		}
		expression = expression.substring(4);
		return expression;
	}
}