package ch.unizh.ifi.adora.productline.ui.views.decisiontable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.LayoutOrderingCommand;
import ch.unizh.ifi.adora.model.commands.SetDecision;
import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.commands.SetVariationPointListDecisions;
import ch.unizh.ifi.adora.model.commands.VariationPointConnectionCreate;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.UpdateConstraintConnections;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.SatSolver;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * This class is the {@link ICellModifier} for the {@link DecisionTableViewPart}
 * . It returns, if a column can be modified, what the current value is and it
 * modifies the entry.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-03-05 PS First Version
 * @version $Date: 2010-02-01 22:22:55 -0500 (Mon, 01 Feb 2010) $, $Author: anil
 *          $, $Revision: 5105 $
 */
public class DecisionTableCellModifier extends Observable implements
		ICellModifier {
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
	 * java.lang.String)
	 */
	public boolean canModify(Object element, String property) {
		if (property.equals(DecisionTableViewPart.COL_DECISION)) {
			return ((Decision) element).canModifyDecision();
		}

		if (property.equals(DecisionTableViewPart.COL_DESCRIPTION)
				|| property.equals(DecisionTableViewPart.COL_RECOMMENDATION)
				|| property.equals(DecisionTableViewPart.COL_FIXED)) {
			return true;
		}

		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
	 * java.lang.String)
	 */
	public Object getValue(Object element, String property) {
		Decision decision = (Decision) element;
		if (property.equals(DecisionTableViewPart.COL_DESCRIPTION)) {
			return decision.getDescription();
		}

		if (property.equals(DecisionTableViewPart.COL_CARDINALITY)) {
			return decision.getCardinality();
		}

		if (property.equals(DecisionTableViewPart.COL_IFTRUE)) {
			return decision.getReqIfTrue();
		}

		if (property.equals(DecisionTableViewPart.COL_IFFALSE)) {
			return decision.getReqIfFalse();
		}

		if (property.equals(DecisionTableViewPart.COL_RECOMMENDATION)) {
			return decision.getRecommendation();
		}
		
		if (property.equals(DecisionTableViewPart.COL_FEATURE)) {
			return decision.getFeatureNames();
		}

		if (property.equals(DecisionTableViewPart.COL_DECISION)) {
			if (decision.getDecisionValue() == null) {
				return 2;
			}
			return decision.getDecisionValue() ? 0 : 1;
		}
		if (property.equals(DecisionTableViewPart.COL_FIXED)) {
			return decision.isFixed();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
	 * java.lang.String, java.lang.Object)
	 */
	public void modify(Object element, String property, Object value) {
		if (element == null && property == null && value == null) {
			this.setChanged();
			super.notifyObservers(null);
			return;
		}

		if (element == null) {
			return;
		}

		Command command = null;
		Decision decision = null;
		if(element instanceof TableItem){
			decision = (Decision) ((TableItem) element).getData();
		}
		else{
			decision = (Decision) element;
		}

		if (property.equals(DecisionTableViewPart.COL_DECISION)) {
			
			//only allowed to change decision values if the product derivation
			//has started, i.e. the "start product derivation" button has been presed.
			
			if (AdoraPlugin.getActiveModel() == null || 
					!AdoraPlugin.getActiveModel().hasProductDerivationStarted())
				return;
			
			//TODO: IV currently not allowed to modify automatically set decisions
			//if (decision.wasDecisionSetAutomatically())
			//	return;
			
			modifyDecisionValue(value, decision);

			LayoutOrderingCommand layoutOrderingCommand = new LayoutOrderingCommand();
			layoutOrderingCommand.execute();
			this.refreshModel();
			return;
		}

		if (property.equals(DecisionTableViewPart.COL_DESCRIPTION)) {
			try {
				command = new SetProperty(decision, value, decision.getClass()
						.getMethod("setDescription",
								new Class[] { String.class }), decision
						.getClass().getMethod("getDescription", new Class[] {}));
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}

		if (property.equals(DecisionTableViewPart.COL_CARDINALITY)) {
			try {
				command = new SetProperty(decision, value, decision.getClass()
						.getMethod("setCardinality",
								new Class[] { String.class }), decision
						.getClass().getMethod("getCardinality", new Class[] {}));
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}

		if (property.equals(DecisionTableViewPart.COL_RECOMMENDATION)) {
			try {
				command = new SetProperty(decision, value, decision.getClass()
						.getMethod("setRecommendation",
								new Class[] { String.class }), decision
						.getClass().getMethod("getRecommendation",
								new Class[] {}));
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}
		if (property.equals(DecisionTableViewPart.COL_FIXED)) {
			decision.setFixed(!decision.isFixed());
		}

		if (command != null) {
			ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getCommandStack().execute(command);
		}

		this.refreshModel();
	}

	/**
	 * Repaints the model. After the weaving, the model is sometimes drawn
	 * wrongly, therefore it needs to be refreshed.
	 */
	private void refreshModel() {
		GraphicalViewer viewer = (GraphicalViewer) AdoraPlugin
				.getActiveEditor().getAdapter(GraphicalViewer.class);
		viewer.flush();

		//refresh model
		ModelObserverMediator.modelRefresher();
	}

	/**
	 * Main function for updating the value of a {@link Decision}.
	 * 
	 * @param value The new {@link Decision} value.
	 * @param decision The {@link Decision} to be updated.
	 */
	public void modifyDecisionValue(Object value, Decision decision) {
		Command command;

		//Store old decisions values in order to be able to tell the
		//weaver which decisions need to be weaved
		Map<Decision, Boolean> oldDecVals = new HashMap<Decision, Boolean>();
		for (Decision d : AdoraPlugin.getActiveModel().getDecisions()) {
			oldDecVals.put(d, d.getDecisionValue());
		}
		
		
		Boolean newDecisionValue = null;
		if ((Integer) value == 0) {
			newDecisionValue = true;
		} else if ((Integer) value == 1) {
			newDecisionValue = false;
		}
		Boolean oldDecisionValue = decision.getDecisionValue();
		
		
		//nothing to change
		if (oldDecisionValue == null && newDecisionValue == null)
			return;
		
		//nothing to change
		if (oldDecisionValue != null && newDecisionValue != null && 
				oldDecisionValue == newDecisionValue)
			return;
			
		
		command = new SetDecision(decision, newDecisionValue);
		
		ModelObserverMediator.getInstance().getAdoraModelEditor()
				.getCommandStack().execute(command);

		AdoraModel model = ModelObserverMediator.getInstance()
		.getAdoraModelEditor().getModel();

		SatSolver solver =  new SatSolver(model); 

		//TODO: IV maybe we can get rid of this

		if (!solver.solve(decision, oldDecisionValue)) {
			
			
			showMessage();

			
			
			command = new SetDecision(decision, oldDecisionValue);

			ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getCommandStack().execute(command);
			
			return;
		}
		
		ArrayList<Decision> weavableDecision = new ArrayList<Decision>();
		//new
		for (Decision d : AdoraPlugin.getActiveModel().getDecisions()) {
			if (oldDecVals.get(d) == null && d.getDecisionValue() == null)
				continue;
			if (oldDecVals.get(d) != null && d.getDecisionValue() != null && 
					oldDecVals.get(d) == d.getDecisionValue())
				continue;
			
			weavableDecision.add(d);
			
		}
		//end new
		
/*		weavableDecision.add(decision);
		for (Decision d : solver.getTrueSetDecision()) {
			if (!weavableDecision.contains(d))
				weavableDecision.add(d);
		}*/
		notifyObserver(weavableDecision);
		
/*
		//TODO: IV: everything from here on needs to be refactored
 	
		if (((Integer) value).intValue() == 2) {
			this.checkVPAndConstraintsToReset(decision);
		}
		
		//TODO: here the values get set wrong
		//TODO: IV change from !=2 to == 1 
		//if (((Integer) value).intValue() != 2) {
		if (((Integer) value).intValue() == 0) {
			//new SatSolver().solve(); //TODO IV commented out
			this.checkPropagation(decision);
		}

		// fetch variationpoint and show them
		createAndShowVPandCP();
		//new SatSolver().solve(); //TODO IV commented out
		//TODO: IV 4.1 commented out
		//if (((Integer) value).intValue() != 2) {
		if (((Integer) value).intValue() == 0) {
			this.checkPropagation(decision);
		}

		
		//new SatSolver().solve();

		createAndShowVPandCP();*/
	}

	private void createAndShowVPandCP() {
		Command command = null;
		AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
		List<VariationPoint> selectedVPs = AdoraPlugin.getActiveModel()
				.getVariationPoints();
		List<VariationPoint>shortenedVPs=new ArrayList<VariationPoint>();
		VariationPoint shortenVariationPoint;
		List<Decision> shortenedVariationPointDecisions;
		for (VariationPoint variationPoint : selectedVPs) {
			String decs = ",";
				for (Decision dec : variationPoint.getDecisions()) {
					decs = decs.concat("D".concat(dec.getNr().toString())
							.concat(","));
				}
			command = new SetVariationPointListDecisions(decs,
					variationPoint);
			command.execute();
			
			shortenedVariationPointDecisions=new ArrayList();
			for (Decision dec : variationPoint.getDecisions()) {
				if (dec.getDecisionValue() == null) {
					shortenedVariationPointDecisions.add(dec);
				}
			}
			
			shortenVariationPoint = new VariationPoint(
						shortenedVariationPointDecisions,
						variationPoint.getNr(),
						Integer.parseInt(variationPoint.getMinCard()),
						Integer.parseInt(variationPoint.getMaxCard()),
						variationPoint.getDescription(),
						variationPoint.isVisible(),
						variationPoint.isRelevant());
			
			shortenedVPs.add(shortenVariationPoint);
		
		}
		VariationPointConnectionCreate showCommand = new VariationPointConnectionCreate(
				AdoraPlugin.getActiveModel(), selectedVPs);
		ame.getCommandStack().execute(showCommand);
		// fetch constraint and show them
		List<Constraint> selectedConst = new ArrayList<Constraint>();
		for (Constraint con : AdoraPlugin.getActiveModel().getConstraints()) {
			selectedConst.add(con);
		}

		UpdateConstraintConnections constCreate = new UpdateConstraintConnections(
				AdoraPlugin.getActiveModel(), selectedConst);
		AdoraPlugin.getActiveEditor().getCommandStack().execute(constCreate);
	}

	private void checkVPAndConstraintsToReset(Decision decision) {
		AdoraModel model = AdoraPlugin.getActiveModel();

		List<Integer> decisionsToSet = new ArrayList<Integer>();
		List<Integer> constraintsChecked = new ArrayList<Integer>();
		List<Integer> variationPointsChecked = new ArrayList<Integer>();

		List<Integer> vpsToCheck = decision.getVariationPointsInvolved();
		List<Integer> constraintsToCheck = decision.getConstraintsInvolved();

		// add the directly involved VPs and constraints
		constraintsChecked.addAll(constraintsToCheck);
		variationPointsChecked.addAll(vpsToCheck);

		List<Integer> newConstraintCandidates = new ArrayList<Integer>();
		List<Integer> newVPCandidates = new ArrayList<Integer>();

		boolean somethingAdded = false;

		do {
			somethingAdded = false;

			// find a way from each vp to other vps or constraints
			// get the vp instances from the model
			for (VariationPoint modelVP : model.getVariationPoints()) {
				if (vpsToCheck.contains(modelVP.getNr())) {
					for (Decision toOtherItems : modelVP.getDecisions()) {
						newConstraintCandidates = toOtherItems
								.getConstraintsInvolved();
						newVPCandidates = toOtherItems
								.getVariationPointsInvolved();
					}
				}
			}

			// find a way from each constraint to new vps or constraints
			// get the constraint instance from the model
			for (Constraint modelConstraint : model.getConstraints()) {
				if (constraintsToCheck.contains(modelConstraint.getNr())) {
					for (Integer toIntOtherItems : modelConstraint
							.getDecisionsInvolved()) {
						// get the decision instances from the model
						for (Decision modelDec : model.getDecisions()) {
							if (modelDec.getNr().equals(toIntOtherItems)) {
								newVPCandidates.addAll(modelDec
										.getVariationPointsInvolved());
								newConstraintCandidates.addAll(modelDec
										.getConstraintsInvolved());
							}
						}
					}
				}
			}

			for (Integer newVP : newVPCandidates) {
				if (!variationPointsChecked.contains(newVP)) {
					variationPointsChecked.add(newVP);
					somethingAdded = true;
				}
			}

			for (Integer newC : newConstraintCandidates) {
				if (!constraintsChecked.contains(newC)) {
					constraintsChecked.add(newC);
					somethingAdded = true;
				}
			}

			vpsToCheck = newVPCandidates;
			constraintsToCheck = newConstraintCandidates;

		} while (somethingAdded);

		for (VariationPoint vp : model.getVariationPoints()) {
			if (variationPointsChecked.contains(vp.getNr())) {
				for (Decision d : vp.getDecisions()) {
					decisionsToSet.add(d.getNr());
				}
			}
		}

		for (Constraint c : model.getConstraints()) {
			if (constraintsChecked.contains(c.getNr())) {
				decisionsToSet.addAll(c.getDecisionsInvolved());
			}
		}

		ArrayList<Decision> decisionInstances = new ArrayList<Decision>();

		// now get the decision instances from the model
		for (Decision d : model.getDecisions()) {
			if (decisionsToSet.contains(d.getNr())) {
				decisionInstances.add(d);
				d.setDecisionValueManually(null);
			}
		}

		this.notifyObserver(decisionInstances);
	}

	private void notifyObserver(ArrayList<Decision> decFound) {
		if (decFound != null && decFound.size() > 0) {
			this.setChanged();
			super.notifyObservers(decFound);
		}
	}

	/**
	 * After some dynamic weaving has been performed, this method is executed.
	 * It reads the all the decision's values to enable correct constraint
	 * propagated weaving.
	 */
	private void checkPropagation(Decision decidedDecision) {
		AdoraModel model = AdoraPlugin.getActiveModel();

		Collection<Decision> decisionsToCheck = model.getDecisions();
		ArrayList<Decision> weavableDecision = new ArrayList<Decision>();
		// iterate over all the decisions of the model
		for (Decision decisionToCheck : decisionsToCheck) {
			if (decisionToCheck.getDecisionValue() != null
					&& decidedDecision.getNr() != decisionToCheck.getNr()) {
				weavableDecision.add(decisionToCheck);
			}
		}
		if (weavableDecision.size() != 0) {
			this.notifyObserver(weavableDecision);
		}
	}
	
	/**
	 * Shows a {@link MessageBox} stating that the current {@link Decision}-
	 * setting is not satisfiable.
	 * 
	 * @return MessageBox return-code
	 */
	private int showMessage() {
		
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getShell(), SWT.ICON_INFORMATION
				| SWT.OK);
		mb.setText("Unsatisfiable");
		mb.setMessage("The Decision Setting is Unsatisfiable");
		return mb.open();
	}
}