package ch.unizh.ifi.adora.productline.ui.views.variationpointtable;

import java.beans.PropertyChangeEvent;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.VariationPointConnectionCreate;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;
import ch.unizh.ifi.adora.productline.ui.views.decisiontable.DecisionTableViewPart;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointContentProvider;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointViewPart;

/**
 * This class is the {@link IStructuredContentProvider} of the
 * {@link VariationPointViewPart}. It implements also the observer pattern, so
 * that it can listen to changes in the actual model and refresh the view.
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-04-23 RS First Version
 * @version 
 */
public class VariationPointContentProvider implements IStructuredContentProvider, IPartListener, Observer, ISelectionChangedListener {
	/**
     * The private class for handling the selection.
     * 
     * @author Reinhard Stoiber
     * @copyright Requirements Engineering Research Group<br>
     *            Department of Informatics<br>
     *            University of Zurich<br>
     * @history 2008-04-23 RS First Version (preliminary)
     * @version 
     */
	private final class SelectionListener implements ISelectionListener {
		
		
		/*
         * (non-Javadoc)
         * 
         * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
         *      org.eclipse.jface.viewers.ISelection)
         */
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			
			//TODO: Implementation to visually select (highlight) the corresponding join relations in the graphical model...
			
		}

	}

	/**
     * The {@link TableViewer} of the {@link DecisionTableViewPart}.
     */
	private TableViewer tableViewer;

	/**
     * The constructor, which adds a listener to the page and sets the
     * {@link VariationPointContentProvider#tableViewer}.
     * 
     * @param viewer
     *            The {@link TableViewer} used by the view.
     * @param page
     *            The {@link IWorkbenchPage} used.
     */
	public VariationPointContentProvider(TableViewer viewer, IWorkbenchPage page) {
		
		ModelObserverMediator.getInstance().addObserver(this);
		page.addPartListener(this);
		this.tableViewer = viewer;
		viewer.addSelectionChangedListener(this);

		if (page.getActivePart() instanceof AdoraModelEditor) {
			ModelObserverMediator.getInstance().setAdoraModelEditor((AdoraModelEditor) page.getActivePart());
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
	public void dispose() {
		ModelObserverMediator.getInstance().deleteObserver(this);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
	public Object[] getElements(Object parent) {
		try {
			return ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getModel().getVariationPoints().toArray();
		} catch (NullPointerException e) {
			return new Object[0];
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
	public void inputChanged(Viewer v, Object oldInput, Object newInput) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partActivated(org.eclipse.ui.IWorkbenchPart)
     */
	public void partActivated(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}

		processPart(part);

		ModelObserverMediator observer = ModelObserverMediator.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);
	}

	/**
     * Saves the current page.
     */
	private IWorkbenchPage currentPage;

	/**
     * Saves a listener, which is used for the selections.
     */
	private SelectionListener listener = new SelectionListener();

	/**
     * Removes the selectionlistener from the one part and adds it to the passed
     * one.
     * 
     * @param part
     */
	private void processPart(IWorkbenchPart part) {
		if (currentPage != null) {
			currentPage.removeSelectionListener(listener);
		}

		currentPage = part.getSite().getPage();
		currentPage.addSelectionListener(listener);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart)
     */
	public void partBroughtToTop(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}

		processPart(part);

		ModelObserverMediator observer = ModelObserverMediator.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);
		
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partClosed(org.eclipse.ui.IWorkbenchPart)
     */
	public void partClosed(IWorkbenchPart part) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partDeactivated(org.eclipse.ui.IWorkbenchPart)
     */
	public void partDeactivated(IWorkbenchPart part) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partOpened(org.eclipse.ui.IWorkbenchPart)
     */
	public void partOpened(IWorkbenchPart part) {
	}


	/*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
	public void update(Observable o, Object arg) {
		
		if (arg instanceof String
				&& ModelObserverMediator.MODEL_CHANGED.equals(arg)) {
			tableViewer.refresh();
			return;
		}

		if (!(arg instanceof PropertyChangeEvent)) {
			return;
		}

		PropertyChangeEvent event = (PropertyChangeEvent) arg;

		//TODO: update the variation points table when there were decisions deleted (respectively aspects deleted, aspects changed not to be variability, ...)

		// Variation Points
		if (AdoraModel.VARIATION_POINT_ADDED.equals(event.getPropertyName())) {
			tableViewer.refresh();
		}

		if (AdoraModel.VARIATION_POINT_REMOVED.equals(event.getPropertyName())) {
			tableViewer.refresh();
		}

		if (AdoraModel.VARIATION_POINT_CHANGED.equals(event.getPropertyName())) {
			tableViewer.refresh();
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		
		AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
		List<VariationPoint> selectedVPs = (((StructuredSelection)event.getSelection()).toList());
		
		VariationPointConnectionCreate showCommand = new VariationPointConnectionCreate(AdoraPlugin.getActiveModel(), selectedVPs);
		ame.getCommandStack().execute(showCommand);

	}
}
