package ch.unizh.ifi.adora.requirements.ui.views.DesignDecisionTable;

import java.beans.PropertyChangeEvent;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.CommentableElement;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.requirements.DesignDecision;
import ch.unizh.ifi.adora.model.representation.requirements.Requirement;
import ch.unizh.ifi.adora.parts.AdoraModelEditPart;
import ch.unizh.ifi.adora.requirements.ui.views.ModelObserver;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class maps between the model object used as the input to the table viewer and the internal
 * list structure needed by the table viewer itself.
 */
public class DesignDecisionContentProvider implements IStructuredContentProvider, IPartListener, Observer  {

	/**
	 * Holds the selected model elements from the adora model editor.
	 */
	private Vector<ConnectableElement> selectedModelElements = new Vector<ConnectableElement>();
	
	/**
	 * Generates connectableElements out of editParts.
	 * 
	 * @param List<AbstractGraphicalEditPart> selectedEditParts
	 */
	private void processEditParts(List<AbstractGraphicalEditPart> selectedEditParts) {
		selectedModelElements = new Vector<ConnectableElement>();
		for (AbstractGraphicalEditPart editPart : selectedEditParts) {
			ConnectableElement modelElement = (ConnectableElement) editPart.getModel();
			selectedModelElements.add(modelElement);
		}
	}
	
	/**
	 * Returns the selectedModelElements vector.
	 * 
	 * @return Vector<ConnectableElement> selectedModelElements
	 */
	public Vector<ConnectableElement> getSelectedModelElements() {
		return selectedModelElements;
	}
	
	/**
	 * This class represents the selection listener and handels the selection in the
	 * designDecisionTableViewer.
	 */
	private final class SelectionListener implements ISelectionListener {
		
		//set model element selection from editor and table
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			if (part instanceof AdoraModelEditor) {
				Vector<Object> toSelect = new Vector<Object>();
				if (!(((StructuredSelection) selection).toList().get(0) instanceof AdoraModelEditPart)) {
					processEditParts(((StructuredSelection) selection).toList());
					for (DesignDecision designDecision : getAdoraModel().getDesignDecisions()) {
						if (selectedModelElements.containsAll(designDecision.getModelElements()) && designDecision.getModelElements().containsAll(selectedModelElements) && !toSelect.contains(designDecision) && designDecision != null) {
							toSelect.add(designDecision);
						} 
						else if (designDecision.getModelElements().size() == 1 && !toSelect.contains(designDecision)) {
							for (ConnectableElement element : selectedModelElements) {
								if (designDecision.getModelElements().contains(element)) {
									toSelect.add(designDecision);
								}
							}
						}
					}
					
//					if (toSelect.size() == 0) {
//						for (DesignDecision designDecision : getAdoraModel().getDesignDecisions()) {
//							if (designDecision.getModelElements().size() == 1 && !toSelect.contains(designDecision)) {
//								for (ConnectableElement element : selectedModelElements) {
//									if (designDecision.getModelElements().contains(element)) {
//										toSelect.add(designDecision);
//									}
//								}
//							}
//						}
//					}
					
				} else {
					selectedModelElements.removeAllElements();
				}
				
				//selected elements from the table
				List actualList = ((IStructuredSelection) tableViewer.getSelection()).toList();
				
				//check, if elements already selected in the table
				if (alreadySelected(actualList, toSelect)) {
					return;
				}
				
				//mark selected elements in the table
				tableViewer.setSelection(new StructuredSelection(toSelect));
			}
		}
		
		//check, if selected model elements are already selected in the table
		private boolean alreadySelected(List actualList, List toSelect) {
			if (actualList.size() != toSelect.size()) {
				return false;
			}

			for (Object object : actualList) {
				if (!toSelect.contains(object)) {
					return false;
				}
			}

			return true;
		}
	}
	
	/**
	 * The table viewer.
	 */
	private TableViewer tableViewer;
	
	/**
     * Saves the current page.
     */
	private IWorkbenchPage currentPage;
	

	/**
     * Saves a listener, which is used for the selections.
     */
	SelectionListener listener = new SelectionListener();
	
	/**
	 * The Constructor.
	 * 
	 * @param TableViewer viewer
	 * @param IWorkbenchPage page
	 */
	public DesignDecisionContentProvider(TableViewer viewer, IWorkbenchPage page) {
		ModelObserver.getInstance().addObserver(this);
		page.addPartListener(this);
		this.tableViewer = viewer;

		if (page.getActivePart() instanceof AdoraModelEditor) {
			try {
				ModelObserver.getInstance().setAdoraModelEditor(
						(AdoraModelEditor) page.getActivePart());
			} catch (ClassCastException e) {
				e.printStackTrace();
			}
		}
	}
	
	public Object[] getElements(Object inputElement) {		
		try {
			return ModelObserver.getInstance().getAdoraModelEditor()
					.getModel().getDesignDecisions().toArray();
		} catch (NullPointerException e) {
			return new Object[0];
		}
	}
	
	/**
	 * Returns the current adoraModel.
	 * 
	 * @return AdoraModel adoraModel
	 */
	private AdoraModel getAdoraModel() {
		return ModelObserver.getInstance().getAdoraModelEditor().getModel();
	}
	
	/**
	 * Returns the design decision associated with the model element
	 * (via adora model).
	 * 
	 * @param VosualizableModelElement modelElement
	 * @return DesignDecision designDecision
	 */
	private DesignDecision getDesignDecision(ConnectableElement modelElement) {
		return ModelObserver.getInstance().getAdoraModelEditor().getModel()
				.getDesignDecision(modelElement);
	}

	public void dispose() {		
	}


	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {		
	}

	public void partActivated(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}

		processPart(part);

		ModelObserver observer = ModelObserver.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);	
	}
	
	/**
	 * Adds a the selection listener to the currentPage.
	 * 
	 * @param IWorkbenchPart part
	 */
	private void processPart(IWorkbenchPart part) {
		if (currentPage != null) {
			currentPage.removeSelectionListener(listener);
		}

		currentPage = part.getSite().getPage();
		currentPage.addSelectionListener(listener);
	}

	@SuppressWarnings("unchecked")
	public void update(Observable o, Object arg) {
		
		if (arg instanceof String
				&& ModelObserver.MODEL_CHANGED.equals(arg)) {
			tableViewer.refresh();
			return;
		}

		if (!(arg instanceof PropertyChangeEvent)) {
			return;
		}
		
		PropertyChangeEvent event = (PropertyChangeEvent) arg;

		AdoraModel adoraModel = ModelObserver.getInstance()
				.getAdoraModelEditor().getModel();
		DesignDecision designDecision;
		Requirement requirement;
		ConnectableElement modelElement;
		ConnectableElement parent;
		Vector<ConnectableElement> modelElements = new Vector<ConnectableElement>();
		Vector<ConnectableElement> nestingElements;
		
		// ADDING - EDITING ELEMENTS
		// new model element in adora model container
		if (AdoraModel.CHILD_ADDED_PROP.equals(event.getPropertyName())
				&& event.getSource() instanceof AdoraModel) {
			modelElement = (ConnectableElement) event.getNewValue();
			designDecision = getDesignDecision(modelElement);
			if (modelElement.getParent() instanceof AdoraModel) {
				if (designDecision == null) { // if design decision doesn't
												// exist
					modelElements.add(modelElement);
					adoraModel.addDesignDecision(new DesignDecision(
							modelElements, adoraModel));
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		// adding new container && nesting: new model element in model element
		// (container)
		if (ZoomableNodeContainer.CHILD_ADDED_PROP.equals(event
				.getPropertyName())
				&& event.getSource() instanceof ConnectableElement) {
			modelElement = (ConnectableElement) event.getNewValue();
			designDecision = getDesignDecision(modelElement);
			parent = (ConnectableElement) modelElement.getParent();
			DesignDecision nesting;
			if (modelElement.getParent() instanceof ConnectableElement) {
				if (designDecision == null) { // if design decision not
					// generated yet
					// new Design Decision
					modelElements.add(modelElement);
					adoraModel.addDesignDecision(new DesignDecision(
							modelElements, adoraModel));
					// nesting
					nestingElements = new Vector<ConnectableElement>();
					nestingElements.add(modelElement);
					nestingElements.add(parent);
					nesting = new DesignDecision(
							nestingElements, adoraModel);
					nesting.setType("Structure-Relationship");
					adoraModel.addDesignDecision(nesting);

				} else if (designDecision != null) { // moving one element
					// from adora model into
					// another element
					modelElements.add(modelElement);
					modelElements.add(parent);
					nesting = new DesignDecision(modelElements,
							adoraModel);
					// nesting.setDescription("Nesting: " +
					// modelElement.getName() + " element of " +
					// ((ConnectableElement)
					// modelElement.getParent()).getName());
					processScope(modelElements, nesting);
					nesting.setDescription("Nesting: " + parent.getName()
							+ " consists of " + modelElement.getName());
					nesting.setType("Structure-Relationship");
					adoraModel.addDesignDecision(nesting);
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		// changing name
		if (VisualizableModelElement.NAME_PROP.equals(event.getPropertyName())) {
			modelElement = (ConnectableElement) event.getSource();
			modelElements.add(modelElement);

			for (DesignDecision decision : adoraModel.getDesignDecisions()) {
				if (decision.getModelElements().contains(modelElement)) {
					if (modelElement.getParent() instanceof ConnectableElement) {
						parent = (ConnectableElement) modelElement.getParent();
						if (decision.getModelElements().contains(parent)) {
							modelElements.add(parent);
							decision.setDescription("Nesting: "
									+ parent.getName() + " consists of "
									+ modelElement.getName());
							processScope(modelElements, decision);
							// } else {
							// String scope = "";
							// for (ConnectableElement element :
							// decision.getModelElements()) {
							// scope += element.getName() + ";";
							// }
							// decision.setScope(scope.substring(0,
							// scope.length() - 1));
							// }
						} else {
							decision.setScope(modelElement.getName());
						}
					} else {
						decision.setScope(modelElement.getName());
					}
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		//new connection
		if (ConnectableElement.SOURCECONNECTION_NEW_PROP.equals(event
				.getPropertyName())
				&& event.getNewValue() instanceof Connection) {
			modelElement = (Connection) event.getNewValue();
			designDecision = getDesignDecision(modelElement);
			if (designDecision == null) { // wenn decision nicht vorhanden
				modelElements.add((ConnectableElement) modelElement);
				DesignDecision connection = new DesignDecision(modelElements,
						adoraModel);
				connection.setScope(modelElement.getName());
				adoraModel.addDesignDecision(connection);
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		// new comment
		if (CommentableElement.COMMENT_PROP.equals(event.getPropertyName())) {
			modelElement = (ConnectableElement) event.getSource();
			modelElements.add((ConnectableElement) modelElement);
			DesignDecision comment = new DesignDecision(modelElements,
					adoraModel);
			comment.setDescription(modelElement.getName() + ": "
					+ modelElement.getComment());
			comment.setType("?");
			adoraModel.addDesignDecision(comment);
			comment.setScope(modelElement.getName());
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		//update multiple design decisions, after requirement has been added
		if (Requirement.DESIGN_DECISION_ASSOCIATED.equals(event.getPropertyName())) {
			requirement = (Requirement) event.getOldValue();
			Vector<DesignDecision> associatedDesignDecisions = (Vector<DesignDecision>) event.getNewValue();
			for(DesignDecision decision : associatedDesignDecisions) {
				if (!decision.getRequirements().contains(requirement)) {
					decision.getRequirements().add(requirement);
					processRequirementIDs(decision.getRequirements(), decision, Requirement.EVENT_SOURCE);
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		// if all design decisions have been deleted from a requirement
		if (Requirement.DESIGN_DECISION_DISSOCIATED.equals(event.getPropertyName())) {
			requirement = (Requirement) event.getOldValue();
			for (DesignDecision decision : adoraModel.getDesignDecisions()) {
				if (decision.getRequirements().contains(requirement)) {
					decision.getRequirements().remove(requirement);
					processRequirementIDs(decision.getRequirements(), decision, Requirement.EVENT_SOURCE);
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		// DELETING ELEMENTS
		if (VisualizableModelElement.WAS_DELETED_PROP.equals(event
				.getPropertyName())
				&& event.getSource() instanceof ConnectableElement) {
			if (adoraModel.getDesignDecisions().size() != 0) {
				modelElement = (ConnectableElement) event.getSource();
				int i = adoraModel.getDesignDecisions().size();
				while (i != 0) {
					designDecision = getDesignDecision(modelElement);
					if (designDecision != null) {
						if (designDecision.getModelElements().size() == 1
								|| (designDecision.getModelElements().size() == 2 
								&& designDecision.getDescription().contains("Nesting"))) {
							adoraModel.removeDesignDecision(designDecision);
						} else {
							designDecision.getModelElements().remove(
									modelElement);
							processScope(designDecision.getModelElements(),
									designDecision);
						}
						i--;
					} else {
						i = 0;
					}
				}

				
				// Iterator iterator =
				// adoraModel.getDesignDecisions().iterator();
				// while (iterator.hasNext()) {
				// DesignDecision decision = (DesignDecision) iterator.next();
				// if (decision.getModelElements().contains(modelElement)) {
				// adoraModel.removeDesignDecision(decision);
				// iterator = adoraModel.getDesignDecisions().iterator();
				// }
				// }
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		// deleting a connection
		if (ConnectableElement.DELETE_ELEMENT_PROP.equals(event
				.getPropertyName())
				&& event.getNewValue() instanceof Connection) {
			Connection connection = (Connection) event.getNewValue();
			designDecision = getDesignDecision(connection);
			adoraModel.removeDesignDecision(designDecision);
			tableViewer.refresh();
		}

		// deleting a requirement
		if (AdoraModel.REQUIREMENT_REMOVED.equals(event.getPropertyName())) {
			requirement = (Requirement) event.getOldValue();
			for (DesignDecision decision : adoraModel.getDesignDecisions()) {
				if (decision.getRequirements() != null
						&& decision.getRequirements().contains(requirement)) {
					decision.getRequirements().remove(requirement);
					processRequirementIDs(decision.getRequirements(), decision, Requirement.EVENT_SOURCE);
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		//deleting old nesting decisions
		//TODO not working properly
//		if (AdoraModel.CHILD_REMOVED_PROP.equals(event.getPropertyName())
//				&& event.getSource() instanceof ConnectableElement) {
//			ConnectableElement parentElement = (ConnectableElement) event.getSource();
//			ConnectableElement childElement = (ConnectableElement) event.getNewValue();
//			for (DesignDecision decision : adoraModel.getDesignDecisions()) {
//				if ((decision.getModelElements().contains(parentElement)
//						&& decision.getModelElements().contains(childElement))
//						&& decision.getDescription().contains("Nesting")){
//					adoraModel.removeDesignDecision(decision);
//				}
//			}
//			if (!tableViewer.getTable().isDisposed()) {
//				tableViewer.refresh();
//			}
//		}
	}
	
	/**
	 * Generates the scope of the design decision.
	 * 
	 * @param Vector<ConnectableElement> modelElements
	 * @param DesignDecision designDecision
	 */
	void processScope(Vector<ConnectableElement> modelElements,
			DesignDecision designDecision) {
		String scope = "";
		if (modelElements.size() == 0) {
			designDecision.setScope(scope);
		} else {
			for (ConnectableElement element : designDecision.getModelElements()) {
				if (designDecision.getModelElements().contains(element)) {
					scope += element.getName() + ";";
				}
			}
			designDecision.setScope(scope.substring(0, scope.length() - 1));
		}
	}
	
	/**
	 * Generates the requirementIDs of the design decision.
	 * 
	 * @param Vector<Requirement> requirements
	 * @param DesignDecision designDecision
	 */
	public void processRequirementIDs(Vector<Requirement> requirements,
			DesignDecision designDecision, String eventSource) {
		String requirementIDs = "";
		if (requirements.size() == 0) {
			designDecision.setRequirementIDs("?", eventSource);
		} else {
			for (Requirement requirement : designDecision.getRequirements()) {
				requirementIDs += requirement.getId() + ";";
			}
			designDecision.setRequirementIDs(requirementIDs.substring(0,
					requirementIDs.length() - 1), eventSource);
		}
	}

	public void partBroughtToTop(IWorkbenchPart part) {
	}

	public void partClosed(IWorkbenchPart part) {
	}

	public void partDeactivated(IWorkbenchPart part) {		
	}

	public void partOpened(IWorkbenchPart part) {
	}	
}