package ch.unizh.ifi.adora.requirements.ui.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.Observable;

import org.eclipse.ui.IPropertyListener;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.model.representation.requirements.DesignDecision;
import ch.unizh.ifi.adora.model.representation.requirements.Requirement;
import ch.unizh.ifi.adora.properties.ZoomableNodeContainerProperties;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;


/**
 * This is a mediator between the {@link DecisionTableViewPart} and the editor /
 * model. A class can register itself on this mediator as an observer and gets
 * every propertyChange from the model.
 * 
 * @author		Jaro Habr
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	2008-07-23 JH First Version
 * @version 	$Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */
public class ModelObserver extends Observable implements
		PropertyChangeListener, IPropertyListener {
	/**
     * This string indicated, that the model has changed in the editor.
     */
	public static String MODEL_CHANGED = "model.changed";

	/**
     * The instance (singleton).
     */
	private static ModelObserver instance;

	/**
     * Returns the instance of this class.
     * 
     * @return The single instance of this class.
     */
	public static ModelObserver getInstance() {
		if (instance == null) {
			instance = new ModelObserver();
		}

		return instance;
	}

	/**
     * The current {@link AdoraModelEditor}.
     */
	private AdoraModelEditor observedAdoraModelEditor;

	/**
     * The constructor (private because of the singleton).
     */
	private ModelObserver() {
	}

	/**
     * Adds all listeners to the {@link AdoraModel}.
     * 
     * @param AdoraModel model.
     */
	private void addListeners(AdoraModel model) { // -> add listener to adora model
		model.addPropertyChangeListener(this);

		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();
		while (iterator.hasNext()) {
			addListener(iterator.next());
		}
		
		Iterator<Requirement> reqIterator = model.getRequirements().iterator();
		while(reqIterator.hasNext()) {
			reqIterator.next().addPropertyChangeListener(this);
		}
		
		Iterator<DesignDecision> decisionIterator = model.getDesignDecisions().iterator();
		while(decisionIterator.hasNext()) {
			decisionIterator.next().addPropertyChangeListener(this);
		}
		
	}
	
	private void addListener(Requirement requirement) {
		requirement.addPropertyChangeListener(this);
	}
	
	private void addListener(DesignDecision designDecision) {
		designDecision.addPropertyChangeListener(this);
	}

	/**
     * Adds all listeners on a {@link ModelElement} and its outgoing
     * connections.
     * 
     * @param modelElement
     *            The {@link ModelElement} to observe.
     */
	private void addListener(ModelElement modelElement) { //add listener to all model elements in the model
		if (!(modelElement instanceof ObservableModelElement)) {
			return;
		}

		ObservableModelElement observable = (ObservableModelElement) modelElement;
		observable.addPropertyChangeListener(this);

		if (observable instanceof ConnectableElement) {
			ConnectableElement connectable = (ConnectableElement) observable;
			for (Connection connection : connectable.getConnections()) { //alle arten von verbindungen zu connectable element
				connection.addPropertyChangeListener(this);
			}
		}
	}

	/**
     * Removes all listeners from the {@link AdoraModel}.
     * 
     * @param model
     */
	private void removeListeners(AdoraModel model) { //remove listener from adora model
		model.removePropertyChangeListener(this);

		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();
		while (iterator.hasNext()) {
			removeListener(iterator.next());
		}
	}

	/**
     * Removes a listener from a specific {@link ModelElement}.
     * 
     * @param modelElement
     *            The {@link ModelElement} where the listeners should be
     *            deactivated.
     */
	private void removeListener(ModelElement modelElement) {
		if (!(modelElement instanceof ObservableModelElement)) {
			return;
		}

		ObservableModelElement observable = (ObservableModelElement) modelElement;
		observable.removePropertyChangeListener(this);

		if (observable instanceof ConnectableElement) {
			ConnectableElement connectable = (ConnectableElement) observable;
			for (Connection connection : connectable.getConnections()) {//fr alle connectionstypen
				connection.removePropertyChangeListener(this);
			}
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
	public void propertyChange(PropertyChangeEvent event) {
		setChanged();

		if (ZoomableNodeContainerProperties.DELETE_ELEMENT_PROP.equals(event
				.getPropertyName())) {
			removeListener((ModelElement) event.getNewValue());
		}

		if (INodeContainer.CHILD_ADDED_PROP.equals(event.getPropertyName())) {
			addListener((ModelElement) event.getNewValue());
		}
		
		if (ConnectableElementPropNames.SOURCECONNECTION_NEW_PROP.equals(event.getPropertyName())) {
			addListener((ModelElement) event.getNewValue());
		}
		
		if (AdoraModel.REQUIREMENT_ADDED.equals(event.getPropertyName())) {
			addListener((Requirement) event.getNewValue());
		}
		
		if (AdoraModel.DESIGN_DECISION_ADDED.equals(event.getPropertyName())) {
			addListener((DesignDecision) event.getNewValue());
		}

		notifyObservers(event);
	}

	/**
     * Returns the {@link AdoraModelEditor}.
     * 
     * @return The current {@link AdoraModelEditor}.
     */
	public AdoraModelEditor getAdoraModelEditor() {
		return observedAdoraModelEditor;
	}

	/**
     * Sets the {@link AdoraModelEditor}. There is a check, if the editor has
     * changed. if yes, it removes all listeners from the old editor / model and
     * adds listeners to the new editor and its model.
     * 
     * @param adoraModelEditor
     *            The current {@link AdoraModelEditor}.
     */
	public void setAdoraModelEditor(AdoraModelEditor adoraModelEditor) {
		if (this.observedAdoraModelEditor == adoraModelEditor) {
			return;
		}

		setChanged();

		if (this.observedAdoraModelEditor != null) {
			this.observedAdoraModelEditor.removePropertyListener(this);
			removeListeners(this.observedAdoraModelEditor.getModel());
		}

		this.observedAdoraModelEditor = adoraModelEditor;
		addListeners(this.observedAdoraModelEditor.getModel());
		adoraModelEditor.addPropertyListener(this);

		notifyObservers(MODEL_CHANGED);
	}

	/**
     * Implemented to listen, if the editor reloads the model (for example for
     * the weaving).
     * 
     * @param source
     * @param propId
     */
	public void propertyChanged(Object source, int propId) {
		setChanged();

		if (propId == AdoraModelEditor.PROP_MODEL_CHANGE) {
			removeListeners(observedAdoraModelEditor.getModel());
			addListeners(observedAdoraModelEditor.getModel());

			notifyObservers(MODEL_CHANGED);
		}
	}
}
