package ch.unizh.ifi.adora.requirements.ui.views.RequirementsTable;

import java.beans.PropertyChangeEvent;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.requirements.DesignDecision;
import ch.unizh.ifi.adora.model.representation.requirements.Requirement;
import ch.unizh.ifi.adora.parts.AdoraModelEditPart;
import ch.unizh.ifi.adora.requirements.ui.views.ModelObserver;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;


/**
 * This class maps between the model object used as the input to the table viewer and the internal
 * list structure needed by the table viewer itself.
 */
public class RequirementsContentProvider implements IStructuredContentProvider, IPartListener, Observer  {
		
	/**
	 * Holds the selected model elements from the adora model editor.
	 */
	private Vector<ConnectableElement> selectedModelElements = new Vector<ConnectableElement>();
	
	/**
	 * Generates connectableElements out of editParts.
	 * 
	 * @param List<AbstractGraphicalEditPart> selectedEditParts
	 */
	private void processEditParts(List<AbstractGraphicalEditPart> selectedEditParts) {
		selectedModelElements = new Vector<ConnectableElement>();
		for (AbstractGraphicalEditPart editPart : selectedEditParts) {
			ConnectableElement modelElement = (ConnectableElement) editPart.getModel();
			selectedModelElements.add(modelElement);	
		}
	}
	
	/**
	 * Returns the selectedModelElements vector.
	 * 
	 * @return Vector<ConnectableElement> selectedModelElements
	 */
	public Vector<ConnectableElement> getSelectedModelElements() {
		return selectedModelElements;
	}

	/**
	 * This class represents the selection listener and handels the selection in the
	 * designDecisionTableViewer.
	 * 
	 * @author 		Jaro Habr
	 *
	 */
	private final class SelectionListener implements ISelectionListener {
		//set model element selection from editor and table
		@SuppressWarnings("unchecked")
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			if (part instanceof AdoraModelEditor) {
				Vector<Object> toSelect = new Vector<Object>();
				if (!(((StructuredSelection) selection).toList().get(0) instanceof AdoraModelEditPart)) {
					processEditParts(((StructuredSelection) selection).toList());
					for (Requirement requirement : getAdoraModel().getRequirements()) {
//						if (requirement.getModelElements().size() == 1 && !toSelect.contains(requirement)) {
							for (ConnectableElement element : selectedModelElements) {
								if (requirement.getModelElements().contains(element) && !toSelect.contains(requirement)) {
									toSelect.add(requirement);
								}
							}
//						}
						if (selectedModelElements.containsAll(requirement.getModelElements()) && requirement.getModelElements().containsAll(selectedModelElements) && !toSelect.contains(requirement) && requirement != null) {
							toSelect.add(requirement);
						}
					}
				} else {
					selectedModelElements.removeAllElements();
				}
				
				//selektierten elemente in der Tabelle
				List actualList = ((IStructuredSelection) tableViewer.getSelection()).toList();
				//vergleich zwischen selektierten Modelelementen im editor und in der Tabelle
				if (alreadySelected(actualList, toSelect)) {
					return;
				}
				
				//selektierten elemente in der Tabelle markieren
				viewPart.setSelectedFromEditor(true);
				tableViewer.setSelection(new StructuredSelection(toSelect));
			}
		}

		//vergleicht beide listen (1. modelelemente aus editor; 2. elemente aus tabelle)
		@SuppressWarnings("unchecked")
		private boolean alreadySelected(List actualList, List toSelect) {
			if (actualList.size() != toSelect.size()) {
				return false;
			}

			for (Object object : actualList) {
				if (!toSelect.contains(object)) {
					return false;
				}
			}

			return true;
		}
	}

	/**
	 * The table viewer.
	 */
	private TableViewer tableViewer;
	
	/**
	 * The requirementViewPart.
	 */
	private RequirementsViewPart viewPart;
	
	/**
	 * Saves the current page.
	 */
	private IWorkbenchPage currentPage;
	
	
	/**
	 * Saves a listener, which is used for the selections.
	 */
	SelectionListener listener = new SelectionListener();
	
	/**
	 * The Constructor.
	 * 
	 * @param TableViewer viewer
	 * @param IWorkbenchPage page
	 */
	public RequirementsContentProvider(TableViewer viewer, IWorkbenchPage page, RequirementsViewPart requirementsViewPart) {
		ModelObserver.getInstance().addObserver(this);
		page.addPartListener(this);
		this.tableViewer = viewer;
		this.viewPart = requirementsViewPart;
		
		if (page.getActivePart() instanceof AdoraModelEditor) {
			try {
				ModelObserver.getInstance().setAdoraModelEditor(
						(AdoraModelEditor) page.getActivePart());
			} catch (ClassCastException e) {
				e.printStackTrace();
			}
		}
	}

	public Object[] getElements(Object inputElement) {
		try {
			return ModelObserver.getInstance().getAdoraModelEditor()
					.getModel().getRequirements().toArray();
		} catch (NullPointerException e) {
			return new Object[0];
		}
	}	
	
	/**
	 * Returns the current adora Model.
	 * 
	 * @return AdoraModel adoraModel
	 */
	private AdoraModel getAdoraModel() {
		return ModelObserver.getInstance().getAdoraModelEditor().getModel();
	}
	
	public void dispose() {	
	}


	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {		
	}

	public void partActivated(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}

		processPart(part);

		ModelObserver observer = ModelObserver.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);
	}
	
	private void processPart(IWorkbenchPart part) {
		if (currentPage != null) {
			currentPage.removeSelectionListener(listener);
		}

		currentPage = part.getSite().getPage();
		currentPage.addSelectionListener(listener);
	}

	@SuppressWarnings("unchecked")
	public void update(Observable o, Object arg) {

		if (!(arg instanceof PropertyChangeEvent)) {
			return;
		}
		
		PropertyChangeEvent event = (PropertyChangeEvent) arg;
		AdoraModel adoraModel = ModelObserver.getInstance().getAdoraModelEditor().getModel();
		DesignDecision designDecision;
		
		if (DesignDecision.REQUIREMENT_ASSOCIATED.equals(event.getPropertyName())) {
			designDecision = (DesignDecision) event.getOldValue();
			Vector<Requirement> associatedRequirements = (Vector<Requirement>) event.getNewValue();
			for(Requirement requirement : associatedRequirements) {
				if (!requirement.getDesignDecisions().contains(designDecision)) {
					requirement.getDesignDecisions().add(designDecision);
					processDesignDecisionIDs(requirement.getDesignDecisions(), requirement, Requirement.EVENT_SOURCE);
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		if (DesignDecision.REQUIREMENT_DISSOCIATED.equals(event.getPropertyName())) {
			designDecision = (DesignDecision) event.getOldValue();
			for (Requirement requirement : adoraModel.getRequirements()) {
				if (requirement.getDesignDecisions().contains(designDecision)) {
					requirement.getDesignDecisions().remove(designDecision);
					processDesignDecisionIDs(requirement.getDesignDecisions(), requirement, DesignDecision.EVENT_SOURCE);
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
		if (AdoraModel.DESIGN_DECISION_REMOVED.equals(event.getPropertyName())) {
			designDecision = (DesignDecision) event.getOldValue();
			for (Requirement requirement : adoraModel.getRequirements()) {
				if (requirement.getDesignDecisions() != null && requirement.getDesignDecisions().contains(designDecision)) {
					requirement.getDesignDecisions().remove(designDecision);
					processDesignDecisionIDs(requirement.getDesignDecisions(), requirement, DesignDecision.EVENT_SOURCE);
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
	}

	/**
	 * Generates the designDecisionIDs of the requirement.
	 * 
	 * @param Vector<DesignDecision> designDecisions
	 * @param Requirement requirement
	 */
	private void processDesignDecisionIDs(Vector<DesignDecision> designDecisions, Requirement requirement, String eventSource) {
		String designDecisionIDs = "";
		if (designDecisions.size() == 0) {
			requirement.setDesignDecisionID("?", eventSource);
		} else {
			for (DesignDecision decision : requirement.getDesignDecisions()) {
				designDecisionIDs += decision.getId() + ";";
			}
			requirement.setDesignDecisionID(designDecisionIDs.substring(0, designDecisionIDs.length() - 1), eventSource);
		}
	}

	public void partBroughtToTop(IWorkbenchPart part) {		
	}

	public void partClosed(IWorkbenchPart part) {		
	}

	public void partDeactivated(IWorkbenchPart part) {		
	}

	public void partOpened(IWorkbenchPart part) {
	}
}