package ch.unizh.ifi.adora.requirements.ui.views.RequirementsTable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.gef.EditPart;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.requirements.DesignDecision;
import ch.unizh.ifi.adora.model.representation.requirements.ImplementationProposal;
import ch.unizh.ifi.adora.model.representation.requirements.Requirement;
import ch.unizh.ifi.adora.requirements.RequirementsPlugIn;
import ch.unizh.ifi.adora.requirements.commands.RequirementAdd;
import ch.unizh.ifi.adora.requirements.commands.RequirementRemove;
import ch.unizh.ifi.adora.requirements.ui.views.DesignDecisionTable.DesignDecisionViewPart;
import ch.unizh.ifi.adora.requirements.ui.views.ImplPropTable.ImplPropViewPart;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class is the view, which can be used for creating so called "requirements
 * table".
 * 
 * @author 		Jaro Habr
 * @copyright 	Requirements Engineering Research Group
 *            	University of Zurich
 *            	Department of Informatics
 * @history 	2008-06-19 JH First Version
 * @version 	$Date: 2008-10-16 17:31:47 +0200 (Thu, 16 Oct 2008) $, $Author: jehle $
 */
public class RequirementsViewPart extends ViewPart {
	
	/**
     * The ID of this view.
     */
	public static final String REQUIREMENT_VIEW_ID = "ch.unizh.ifi.adora.requirements.RequirementsTable";
	
	/**
	 * The Description of the COL_ID column.
	 */
	private static final String COL_ID = "ID";
	
	/**
	 * The Description of the COL_TITLE column.
	 */
	private static final String COL_TITLE = "Title";
	
	/**
	 * The Description of the COL_DESCRIPTION column.
	 */
	private static final String COL_DESCRIPTION = "Description";
	
	/**
	 * The Description of the COL_PRIORITY column.
	 */
	private static final String COL_PRIORITY = "Priority";
	
	/**
	 * The Description of the COL_ABSTRACTION_LEVEL column.
	 */
	private static final String COL_ABSTRACTION_LEVEL = "Abstraction Level";
	
	/**
	 * The Description of the COL_LINKT_TO column.
	 */
	private static final String COL_LINKT_TO = "Link to ID";
	
	/**
	 * The Description of the COL_DESIGN_DECISION column.
	 */
	private static final String COL_DESIGN_DECISION = "Design Decision ID";
	
	/**
	 * Holds the references to the column descriptions.
	 */
	private String[] columnNames = new String[] {
			COL_ID, 
			COL_TITLE,
			COL_DESCRIPTION,
			COL_PRIORITY,
			COL_ABSTRACTION_LEVEL,
			COL_LINKT_TO,
			COL_DESIGN_DECISION
			};
	/**
	 * The table.
	 */
	private Table table;
	
	/**
	 * The table viewer.
	 */
	private TableViewer viewer;
	
	/**
	 * The toolbar actions.
	 */
	private Action importRequirementsAction, exportRequirementsAction,
			addRequirementAction, deleteRequirementAction, searchAction,
			filterAction;
	
	/**
	 * Holds the active adora model editor.
	 */
	private AdoraModelEditor activeAdoraEditor;
	
	/**
	 * Holds the active adora model.
	 */
	private AdoraModel adoraModel;
	
	/**
	 * Holds the requirement vector saved in the adora model.
	 */
	private Vector<Requirement> requirements = new Vector<Requirement>();
	
	/**
	 * Holds the selected requirements in the table.
	 */
	private Vector<Requirement> selection = new Vector<Requirement>();
	
	/**
	 * Holds the current requirement IDs list.
	 */
	private Vector<String> requirementIDs = new Vector<String>();
	
	/**
	 * Holds the current design decision IDs list.
	 */
	private Vector<String> designDecisionIDs = new Vector<String>();
	
	/**
	 * Contains the choices for the priority attribute.
	 */
	private String[] priorityChoices = {"?", "1", "2", "3"};
	
	/**
	 * Contains the choices for the abstraction level choices.
	 */
	private String[] abstractionLevelChoices = {"?", "1-Strategy", "2-Feature", "3-Function", "4-Implementation"};
	
	/**
	 * The content provider.
	 */
	private RequirementsContentProvider contentProvider;
	
	/**
	 * If requirements are selected from the requirements table viewer,
	 * selectedFromRequirement is set to true.
	 */
	private boolean selectedFromRequirement = false;
	
	/**
	 * If requirements are selected from the adora editor,
	 * selectedFromEditor is set to true.
	 */
	private boolean selectedFromEditor = false;
	
	/**
	 * IPartListener to listen to active adora model and to update the data from
	 * the adora model.
	 */
	private IPartListener listener = new IPartListener() {

		public void partActivated(IWorkbenchPart part) {
			if (part instanceof AdoraModelEditor) {
				activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();		
				adoraModel = activeAdoraEditor.getModel();
				requirements = adoraModel.getRequirements();
				requirementIDs = adoraModel.getRequirementsIDs();
				designDecisionIDs = adoraModel.getDesignDecisionIDs();
				if (!viewer.getTable().isDisposed()) {
					viewer.refresh();
				}			
			}

//			if (activeAdoraEditor != null) {
//				selectedModelElements = activeAdoraEditor.getViewer().getSelectedEditParts().get;
////				List<EditPart> selectedElements = activeAdoraEditor.getViewer().getSelectedEditParts();
//				String editPartName = "";
//				for (int i = 0; i < selectedModelElements.size(); i++) {
//					try {
//						NodeEditPart editPart = (NodeEditPart) selectedModelElements.get(i);
//						editPartName = ((VisualizableModelElement) editPart.getModel()).getName();
//					} catch (ClassCastException e) {
////						ConnectionEditPart editPart = (ConnectionEditPart) selectedModelElements.get(i);
////						editPartName = ((VisualizableModelElement) editPart.getModel()).getName();
//					}
//					
//					if (!selectedModelElementNames.contains(editPartName)) {
//						selectedModelElementNames.add(editPartName);
//					}
////					else if (modelElement != null && !modelElements.contains(modelElement)) {
////						modelElements.add(modelElement);
////						System.out.println("modelElement: " + modelElement.getName());
////					}
//				}
//				System.out.println("DesignViewPart: elementNames: " + selectedModelElementNames);
//		}
		}

		public void partBroughtToTop(IWorkbenchPart part) {
		}
		
		//set table invisible, if last editor has been closed
		public void partClosed(IWorkbenchPart part) {
//			if (part instanceof AdoraModelEditor) {
//				if (getSite().getPage().getEditors().length == 0) {
//					if (!table.isDisposed()) {
//						try {
//							table.clearAll();
//							table.setVisible(false);
//							disableActions();
//						} catch (Exception e) {
//							e.printStackTrace();
//						}
//					}
//				}
//			}
		}

		public void partDeactivated(IWorkbenchPart part) {
			
		}

		public void partOpened(IWorkbenchPart part) {
//			if (part instanceof AdoraModelEditor) {
//				table.setVisible(true);
//				enableActions();
//				activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
////				activeAdoraEditor = (AdoraModelEditor) PlatformUI.getWorkbench()
////											.getActiveWorkbenchWindow().getActivePage().getActiveEditor();
//				adoraModel = activeAdoraEditor.getModel();
//				requirements = adoraModel.getRequirements();
//				System.out.println("hello from: " + part.getTitle());
//				System.out.println("Anzahl requirements: " + requirements.size());
			if (!viewer.getTable().isDisposed()) {
				viewer.refresh();
			}
//			}
		}
	};
	
	/**
	 * Filter for the filter action in the toolbar.
	 */
	private final ViewerFilter filter = new ViewerFilter() {
	      public boolean select (
	        Viewer viewer,
	        Object parentElement,
	        Object element) {
	        if(((Requirement)element).isSelected()) {
	        	return true;
	        }
	        return false;
	      }
	};

	/**
     * The RequirementsViewPart constructor
     * The {@link TableViewer} used.
     */
	public RequirementsViewPart() {
	}
 
	/**
	 * Creates the part control.
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		this.addChildControls(parent);
//		if (getSite().getPage().getEditors().length == 0) {
//			table.setVisible(false);
//			disableActions();
//		} else { 
		if (getSite().getPage().getActivePart() instanceof AdoraModelEditor) {
			table.setVisible(true);
			activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
			adoraModel = activeAdoraEditor.getModel();
			requirements = adoraModel.getRequirements();
			requirementIDs = adoraModel.getRequirementsIDs();
			designDecisionIDs = adoraModel.getDesignDecisionIDs();
			viewer.refresh();
		}
//		}
		// add listener
		getSite().getPage().addPartListener(listener);				
	}
	
	/**
	 * Creates the controls for the parent composite.
	 * 
	 * @param Composite parent
	 */
	private void addChildControls(Composite parent) {
		GridLayout layout = new GridLayout(2, false);
		parent.setLayout(layout);
		
		createTable(parent);
		createToolbarActions(parent);
		createToolBar();
		createTableViewer();
		initTableViewer();
	}
	
	/**
	 * Creates the view table and adds the table columns.
	 * 
	 * @param Composite parent
	 */
	private void createTable(Composite parent) {
		int style = SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION
				| SWT.CHECK | SWT.MULTI;
		
		table = new Table(parent, style);
		
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 3;
		table.setLayoutData(gridData);
		
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		//table listener
		table.addListener(SWT.Selection, new Listener() {
		      public void handleEvent(Event event) {
		    	  if (event.detail == SWT.CHECK) {
		    		  Requirement requirement = ((Requirement)event.item.getData());
		    		  boolean checked = ((TableItem)event.item).getChecked();
		    		  requirement.setSelected(checked);    		  
		    		  
		    		  if (checked){
		    			  selection.add(requirement);  
		    		  } else {
		    			  selection.remove(requirement);
		    		  }
			      }
		      } 
		});
		
//		//selection listener
//		table.addSelectionListener(new SelectionAdapter() {
//			public void widgetSelected(SelectionEvent e) {
//				if (selection.size() == 0 && designDecisions != null) {
//					DesignDecision designDecision = (DesignDecision) ((IStructuredSelection) 
//							viewer.getSelection()).getFirstElement();
//					selection.add(designDecision);
//				}				
//			}
//		});

		
		// ID column
		TableColumn column = new TableColumn(table, SWT.LEFT, 0);		
		column.setText(COL_ID);
		column.setWidth(60);
		column.setResizable(true);
		
		// Title column
		column = new TableColumn(table, SWT.LEFT, 1);
		column.setText(COL_TITLE);
		column.setWidth(150);
		column.setResizable(true);
		
		// Description column
		column = new TableColumn(table, SWT.LEFT, 2);
		column.setText(COL_DESCRIPTION);
		column.setWidth(200);
		column.setResizable(true);
		
		// AbstractionLevel column
		column = new TableColumn(table, SWT.LEFT, 3);
		column.setText(COL_ABSTRACTION_LEVEL);
		column.setWidth(130);
		column.setResizable(true);
		
		// Priority column
		column = new TableColumn(table, SWT.LEFT, 4);
		column.setText(COL_PRIORITY);
		column.setWidth(50);
		column.setResizable(true);
		
		// LinkTo column
		column = new TableColumn(table, SWT.LEFT, 5);
		column.setText(COL_LINKT_TO);
		column.setWidth(80);
		column.setResizable(true);
		
//		// Scope column
//		column = new TableColumn(table, SWT.LEFT, 6);
//		column.setText(COL_MODEL_ELEMENETS);
//		column.setWidth(200);
//		column.setResizable(true);
		
		// Design Decision column
		column = new TableColumn(table, SWT.LEFT, 6);
		column.setText(COL_DESIGN_DECISION);
		column.setWidth(120);
		column.setResizable(true);
	}
	
	/**
	 * Creates the table viewer and cell editors.
	 */
	private void createTableViewer(){
		viewer = new CheckboxTableViewer(table);
				
		viewer.setUseHashlookup(true);
		viewer.setColumnProperties(columnNames);
//		viewer.setSorter(new ViewerSorter());
		
		// create sell editor
		CellEditor[] editors = new CellEditor[columnNames.length];
		
		// id column (free text)
//		TextCellEditor textEditor = new TextCellEditor(table);
//		((Text) textEditor.getControl()).setTextLimit(50);
//		editors[0] = textEditor;
		
		//title column (free text)
		TextCellEditor textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(300);
		editors[1] = textEditor;
		
		//Description column (free text)
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(1000);
		editors[2] = textEditor;
		
		// abstractionLevel column (combobox)
		editors[3] = new ComboBoxCellEditor(table, abstractionLevelChoices, SWT.READ_ONLY);
		
		// priority column (combobox)
		editors[4] = new ComboBoxCellEditor(table, priorityChoices, SWT.READ_ONLY);
		
		// link to column (free text)
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(100);
		editors[5] = textEditor;
		
		//Model Elements -> no editor
		
		// design decision column (free text)
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(100);
		editors[6] = textEditor;
		
		// Assign the cell editors to the viewer 
		viewer.setCellEditors(editors);
		viewer.setCellModifier(new RequirementCellModifier(this));
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			@SuppressWarnings("unchecked")
			public void selectionChanged(SelectionChangedEvent event) {
				
				if (event.getSelection().isEmpty()) {
					return;
				}

				if (!(event.getSelection() instanceof IStructuredSelection)) {
					return;
				}

				ArrayList selectedModelElements = new ArrayList();
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				Iterator selectionIterator = selection.iterator();
				DesignDecisionViewPart designDecisionViewPart = (DesignDecisionViewPart) PlatformUI
						.getWorkbench().getActiveWorkbenchWindow()
						.getActivePage().findView(
								DesignDecisionViewPart.DESIGN_DECISION_VIEW_ID);
				ImplPropViewPart implPropViewPart = (ImplPropViewPart) PlatformUI
				.getWorkbench().getActiveWorkbenchWindow()
				.getActivePage().findView(
						ImplPropViewPart.IMPL_PROP_VIEW_ID);
				Requirement requirement = null;
				Vector<DesignDecision> designDecisionsToSelect = new Vector<DesignDecision>();
				Vector<ImplementationProposal> implPropsToSelect = new Vector<ImplementationProposal>();
				
				//selected from DD Table
				if (designDecisionViewPart != null && designDecisionViewPart.getSelectedFromDesignDecision() == true) {
					designDecisionViewPart.setSelectedFromDesignDecision(false);
				} else if(implPropViewPart != null && implPropViewPart.getSelectedFromImplProp() == true) {
					implPropViewPart.setSelectedFromImplProp(false);
				} else if (selectedFromEditor == true) {
					setSelectedFromEditor(false);
				} else {
					//select model elements from selected requirement
					while (selectionIterator.hasNext()) {
						requirement = (Requirement) selectionIterator.next();
						if (requirement.getModelElements().size() != 0) {
							for (int a = 0; a < requirement.getModelElements().size(); a++) { //iterate through all modelElements of one designDecisions
								if (!selectedModelElements.contains(requirement.getModelElements().get(a))) {
									selectedModelElements.add(requirement.getModelElements().get(a)); //add one ConnectableElemente to selected Elements
								}
							}
						}
						
						if (implPropViewPart != null) {
							for (ImplementationProposal implProp : adoraModel.getImplProps()) {
								if (implProp.getRequirements().contains(requirement)) {
									if (!implPropsToSelect.contains(implProp)) {
										implPropsToSelect.add(implProp);
									}
								}
							}
						}
						
						//select design decision from Req Table
						try {
							if (requirement.getDesignDecisions().size() != 0 && requirement.getDesignDecisions() != null) {
								for (DesignDecision designDecision : requirement.getDesignDecisions()) {
									designDecisionsToSelect.add(designDecision);
								}
							}
						} catch (NullPointerException e) {
							e.printStackTrace();
						}
					}
					
					//mark selected modelElements in the adoraModel 
					AdoraModelEditor adoraModelEditor = AdoraPlugin.getActiveEditor();
					if (adoraModelEditor != null) {
						if (alreadySelected(adoraModelEditor, selectedModelElements)) {
							return;
						}
						adoraModelEditor.selectElements(selectedModelElements);
					}
					
					// select designDecisions in the DD Table with the associated requirements selected from Req Table
					if (designDecisionViewPart != null) {
						setSelectedFromRequirement(true);
						designDecisionViewPart.getViewer().setSelection(new StructuredSelection(designDecisionsToSelect));
					} else {
						System.out.println("Please open the requirements table first!");
					}
					
					if (implPropViewPart != null) {
						setSelectedFromRequirement(true);
						implPropViewPart.getViewer().setSelection(new StructuredSelection(implPropsToSelect));
					} else {
						System.out.println("Please open the implPropTable first!");
					}
				}		
			}

			/**
             * Checks, if the list of selectedModelElements is already selected
             * in the editor.
             * 
             * @param AdoraModelEditor adoraModelEditor
             * @param List selectedModelElements
             * 
             * @return true if already selected.
             */
			@SuppressWarnings("unchecked")
			private boolean alreadySelected(AdoraModelEditor adoraModelEditor,
					List selectedModelElements) {
				if (adoraModelEditor.getViewer().getSelectedEditParts().size() != selectedModelElements
						.size()) {
					return false;
				}

				boolean equals = true;
				for (Object object : adoraModelEditor.getViewer().getSelectedEditParts()) {
					EditPart part = (EditPart) object;
					if (!selectedModelElements.contains(part.getModel())) {
						equals = false;
					}
				}
				
				return equals;
			}
		});
	}
	
	/**
	 * Initializes the table viewer.
	 */
	private void initTableViewer() {
		contentProvider = new RequirementsContentProvider(viewer, getSite().getPage(), this);
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(new RequirementsLabelProvider());
		//associate viewer with domain model input (objects)
		viewer.setInput(requirements);
	}
	
	/**
	 * Creates the view toolbar and adds actions to it.
	 */
	private void createToolBar(){
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();
		mgr.add(addRequirementAction);
		mgr.add(deleteRequirementAction);
		mgr.add(new Separator());
		mgr.add(filterAction);
		mgr.add(searchAction);
		mgr.add(new Separator());
		mgr.add(importRequirementsAction);
		mgr.add(exportRequirementsAction);
	}
	
	/**
	 * Creates the toolbar actions and sets the action image.
	 */
	public void createToolbarActions (final Composite parent){		
		// add action
		addRequirementAction = new Action("Add Requirement") {
			public void run() {
				RequirementAdd addCommand = new RequirementAdd(adoraModel);
				activeAdoraEditor.getCommandStack().execute(addCommand);
				viewer.refresh();
			}
		};
		addRequirementAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/add.gif"));
		
		// delete action with message Box
		deleteRequirementAction = new Action("Delete Requirement") {
			public void run() {		        
				MessageBox messageBox;
		        String message = new String();
		        		        
		        if (selection.size() == 0){
		        	messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
		        	messageBox.setText("Notification");
		        	message = "Please select a Requirement first!";
		        } else {
		        	messageBox = new MessageBox(parent.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			        messageBox.setText("Confirmation");
			        message = "Are you sure to remove the ";
		        	
			        if (selection.size() == 1) {
			        	message += "selected Requirement (" + selection.get(0).getId() + ")?";
			        } else {
			        	message += selection.size() + " selected Requirements?";
			        }
		        }
			    
		        messageBox.setMessage(message);
		        
		        if (messageBox.open() == SWT.YES) {
		          for(int i = 0; i < selection.size(); i++){
		        	  RequirementRemove removeCommand = new RequirementRemove(adoraModel, selection.elementAt(i));
		        	  activeAdoraEditor.getCommandStack().execute(removeCommand);
//		        	  adoraModel.removeRequirement(selection.elementAt(i));
//		        	  getAdoraModel().getRequirements().remove(selection.elementAt(i));
//		        	  ((Vector<Requirement>) getAdoraModel().getRequirements()).remove(selection.elementAt(i)); //-> doesn not work
//		        	  requirements.remove(selection.elementAt(i));
		        	  viewer.refresh();
//		        	  viewer.remove(selection.elementAt(i));
					}
		          //clear the selection vector -> no elemets are selected
		          selection.removeAllElements();
		          }
		        }
		};
		ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
		deleteRequirementAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		
		//filter action
		filterAction = new Action("Filter Requirements") {
//			final ViewerFilter filter = new ViewerFilter() {
//			      public boolean select (
//			        Viewer viewer,
//			        Object parentElement,
//			        Object element) {
//			        if(((Requirement)element).isSelected()) {
//			        	return true;
//			        }
//			        return false;
//			      }
//			    };
			
			public void run(){
				//isChecked() == false
				if(isChecked()) { //isChecked() == true after klicking on action					
					viewer.addFilter(filter);
				} else {
					viewer.removeFilter(filter);
					}	
				}
		};
		filterAction.setChecked(false);
		filterAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/filter.gif"));
		
		//search action
		searchAction = new Action("Search..."){
			public void run(){
				//TODO
			}
		};
		searchAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/search.gif"));
		
		//import action
		importRequirementsAction = new Action("Import Requirements from Excel") {
			public void run() {
				//TODO
			}
		};
		importRequirementsAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/import.gif"));
		
		//export action
		exportRequirementsAction = new Action("Export Requirements to Excel") {
			public void run() {
				//TODO
			}
		};
		exportRequirementsAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/export.gif"));
	}
	
	/**
	 * Disables all actions in the toolbar.
	 */
	public void disableActions() {
		importRequirementsAction.setEnabled(false);
		addRequirementAction.setEnabled(false);
		deleteRequirementAction.setEnabled(false);
		filterAction.setEnabled(false);
		searchAction.setEnabled(false);
//		addReqIDAction.setEnabled(false);
//		removeReqIDAction.setEnabled(false);
	}
	
	/**
	 * Enables all actions in the toolbar if disabled.
	 */
	private void enableActions() {
		importRequirementsAction.setEnabled(true);
		addRequirementAction.setEnabled(true);
		deleteRequirementAction.setEnabled(true);
		filterAction.setEnabled(true);
		searchAction.setEnabled(true);
//		addReqIDAction.setEnabled(true);
//		removeReqIDAction.setEnabled(true);
	}
	
	/**
	 * Returns column names as an array.
	 * 
	 * @return List<String> columnNames
	 */
	public List<String> getColumnNames() {
		return Arrays.asList(columnNames);
	}
	
	
	/**
	 * Sets the keyboard focus.
	 * 
	 * @override
	 */
	public void setFocus() {
		table.getParent().setFocus();
	}
	
	/**
	 * Returns the table viewer.
	 * 
	 * @return TableViewer viewer
	 */
	public TableViewer getViewer(){
		return viewer;
	}
	
	/**
	 * Returns the priority choices.
	 * 
	 * @return String[] priorityChoices
	 */
	public String[] getPriorityChoices(){
		return priorityChoices;
	}
	
	/**
	 * Returns the abstraction level choices.
	 * 
	 * @return String[] abstractionLevelChoices
	 */
	public String[] getAbstractionLevelChoices() {
		return abstractionLevelChoices;
	}
	
	/**
	 * Returns the selection boolean.
	 * 
	 * @return boolean selectedFromDesignDecision
	 */
	public boolean getSelectedFromRequirement() {
		return selectedFromRequirement;
	}
	
	/**
	 * Sets the selection boolean. Used to avoid endless loop.
	 * 
	 * @param boolean b
	 */
	public void setSelectedFromRequirement(boolean b) {
		selectedFromRequirement = b;
	}
	
	/**
	 * Sets the selection boolean. Used to avoid endless loop.
	 * 
	 * @param boolean b
	 */
	public void setSelectedFromEditor(boolean b) {
		selectedFromEditor = b;
	}
	
//	public String[] toStringArray(List<String> requirementIDsList) {
//		String[] requirementIDs = new String[requirementIDsList.size() + 1];
//    	requirementIDs[0] = "?";
//    	
//    	for(int i = 0; i < requirementIDsList.size(); i++) {
//    		requirementIDs[i + 1] = requirementIDsList.get(i);
//    	}    	
//    	return requirementIDs;
//	}
	
	/**
	 * Returns the requirement vector.
	 * 
	 * @return Vector<Requirement> requirements
	 */
	public Vector<Requirement> getRequirements() {
		return requirements;
	}
	
	/**
	 * Returns the requirementIDs vector.
	 * 
	 * @return Vector<String> requirementIDs
	 */
	public Vector<String> getRequirementIDs() {
		return requirementIDs;
	}
	
	/**
	 * Returns the design decision IDs vector.
	 * 
	 * @return Vector<String> designDecisionIDs
	 */
	public Vector<String> getDesignDecisionIDs() {
		return designDecisionIDs;
	}
	
	/**
	 * Returns the checked requirements.
	 * 
	 * @return Vector<Requirement> selection
	 */
	public Vector<Requirement> getRequirementSelection() {
		return selection;
	}

//	public void propertyChange(PropertyChangeEvent event) {
//		if (event.getPropertyName().equals("ImplPropStarted")) {
//			Vector<Requirement> selection = (Vector<Requirement>) event.getNewValue();
//			for (Requirement requirement : requirements) {
//				requirement.setSelected(true);
//			}
//			filterAction.run();
//		}
//		
//	}
	
//	public ImplPropViewPart getImplPropViewPart() {
//		//select requirement from DD Table
//		ImplPropViewPart implPropViewPart;
//		Object[] views = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getViews();
//		for (int a = 0; a < views.length; a++) {
//			if (views[a] instanceof ImplPropViewPart) {
//				implPropViewPart = (ImplPropViewPart) views[a];
//				return implPropViewPart;
//			}
//		}
//		return null;
//	}
	
	public Action getFilterAction() {
		return filterAction;
	}
	
	public AdoraModelEditor getActiveEditor() {
		return activeAdoraEditor;
	}
	
//	/**
//	 * Generates model elements out of edit parts and sets the list to a requirement
//	 * @param List<AbstractGraphicalEditPart> selectedEditParts
//	 * @param Requirement requirement
//	 * @return Vector<ConnectableElement> selectedModelElements
//	 */
//	public void processEditParts(List<AbstractGraphicalEditPart> selectedEditParts, Requirement requirement) {
//		selectedModelElements = new Vector<ConnectableElement>();
//		for (AbstractGraphicalEditPart editPart : selectedEditParts) {
//			try {
//				ConnectableElement modelElement = (ConnectableElement) editPart.getModel();
//				selectedModelElements.add(modelElement);
//			} catch (Exception e) {
//				
//			}
//			
//		}
//		requirement.setModelElements(selectedModelElements);
//	}
	
//	public String[] toStringArray(List<String> requirementIDsList) {
//	String[] requirementIDs = new String[requirementIDsList.size() + 1];
//	requirementIDs[0] = "?";
//	
//	for(int i = 0; i < requirementIDsList.size(); i++) {
//		requirementIDs[i + 1] = requirementIDsList.get(i);
//	}    	
//	return requirementIDs;
//}
}