package ch.unizh.ifi.adora.unweaver.blackboard.board;

import java.util.ArrayList;
import java.util.List;
import java.util.Observer;

/**
 * This class acts as a blackboard storing jobs for the unweaving
 * algorithm.
 * This pattern helps to minimize dependence between editor, blackboard and unweaver.
 * 
 * @author Michael Jehle
 */
public class Blackboard
{
    /**
     * Stores the last job that was started at the editor.
     */
    private List<Object> lastJob;
    
    private int[] extractionCoordinates = new int[2];

    private List<Observer> observers = new ArrayList<Observer>();
    
    private static final int SELECTION = 0;
    private static final int COORDINATES = 1;

    /**
     * Allows to add an observer to the blackboard which will
     * be notified every time, the blackboard gets a new input.
     * 
     * @param observer Observer
     */
    public void addObserver(Observer observer)
    {
        this.observers.add(observer);
    }
    
    /**
     * Remove an observer from the blackboard. This is used when
     * the active editor changes.
     * 
     * @param observer Observer
     */
    public void removeObserver(Observer observer)
    {
        this.observers.remove(observer);
    }
    
    /**
     * Use this method to add a new unweaving job to the 
     * blackboard.
     * 
     * @param job List<ModelElement>
     */
    public void addJob(List<Object> job)
    {
        this.lastJob = job;
        this.notifyObservers(Blackboard.SELECTION);
    }
    
    /**
     * Returns the last job that has been put to the blackboard.
     * @return
     */
    public List<Object> getLastJob()
    {
        return this.lastJob;
    }
    
    /**
     * Notifies the observers. The unweaver will probably be the only observer.
     * 
     * @param mode int, whether to pass a selection of elements or the coordinate
     *                  where to extract.
     */
    private void notifyObservers(int mode)
    {
        for (Observer o : this.observers)
        {
            if (mode == Blackboard.SELECTION)
            {
                o.update(null, this.lastJob);
            }
            else if (mode == Blackboard.COORDINATES)
            {
                o.update(null, this.extractionCoordinates);
            }
        }
    }
    
    /**
     * This method is called by the unweaving action as soon as the user
     * defines where to extract the new aspect.
     * 
     * @param x int
     * @param y int
     */
    public void addExtractionCoordinates(Integer x, Integer y)
    {
        this.extractionCoordinates[0] = x.intValue();
        this.extractionCoordinates[1] = y.intValue();
        
        this.notifyObservers(Blackboard.COORDINATES);
    }
}