package ch.unizh.ifi.adora.unweaver.modelModification;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.model.commands.InsertNodeVisually;
import ch.unizh.ifi.adora.model.commands.NodeCompact;
import ch.unizh.ifi.adora.model.commands.NodeCreate;
import ch.unizh.ifi.adora.model.commands.ScenarioConnectionReconnect;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.commands.TransitionReconnect;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.TransitionDescription;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.unweaver.selection.StateChain;
import ch.unizh.ifi.adora.unweaver.selection.UserSelection;


public class ModelModifier
{
    private UserSelection selection;
    
    private Point coordinates;
    
    /**
     * This hash map relates every sub aspect to the model elements
     * that belong to it.
     * E.g. the container, where the extracted model elements come from.
     * or the component an actor is associated with.
     */
    private HashMap<IContainer, AspectContainer> subAspects;
    
    /**
     * The variants main aspect.
     */
    private AspectContainer extractionAspect;
    
    /**
     * If no subaspects are used, this is a reference to
     * the container where the extracted elements come from.
     */
    private IContainer extractionSource;
    
    /**
     * This list stores references to all the model elements that already have been
     * unweaved. This is required to trac the progrss of the unweaving and finally to control
     * that all the selected model elements have been extracted.
     */
    private List<VisualizableModelElement> extractedElements = new ArrayList<VisualizableModelElement>();
    
    /**
     * Constructor.
     * 
     * @param selection UserSelection
     * @param coords Point
     */
    public ModelModifier(UserSelection selection, Point coords)
    {
        this.selection = selection;
        this.coordinates = coords;
    }
    
    /**
     * This is the method that finally triggers the unweaving
     * in the model.
     * It is a template method to organize how exactly the components
     * are extracted.
     * Changes to this procedure should be made here (e.g. changing the order
     * of the steps or adding new actions).
     */
    public void unweave()
    {
        LayoutHandler.getInstance().reset();
        this.createContainerAspect();
        this.createSubAspects();
        this.moveElements();
        this.doFinalCompact();
        //this.extractionAspect.setVariability(true); //--this command should be here, and recursively applied to all new aspects
    }
    
    /**
     * This method creates the aspect container, where the extracted
     * model elements will be moved to.
     */
    private void createContainerAspect()
    {
        this.extractionAspect = new AspectContainer();
        this.extractionAspect.setLocation(this.coordinates.x, this.coordinates.y);
        //this.extractionAspect.setVariability(true); //threw an exception
        
        Point topLeft = ModelModifierHelper.getInstance().findTopLeftPoint();
        
        // include the scrolled model into the coordinates 
        ScrollingGraphicalViewer v = (ScrollingGraphicalViewer) AdoraPlugin.getActiveEditor().getViewer();
        FigureCanvas fc = (FigureCanvas) v.getControl();
        
        int xScrolled = fc.getHorizontalBar().getSelection();
        int yScrolled = fc.getVerticalBar().getSelection();
        
        this.coordinates.x = this.coordinates.x + xScrolled;
        this.coordinates.y = this.coordinates.y + yScrolled;
        
        if (topLeft.x < 0)
        {
            this.coordinates.x = this.coordinates.x + topLeft.x;
        }
        
        if (topLeft.y < 0)
        {
            this.coordinates.y = this.coordinates.y + topLeft.y;
        }
                
        NodeCreate create = new NodeCreate(this.extractionAspect, AdoraPlugin.getActiveModel(), new Rectangle(this.coordinates.x, this.coordinates.y, 0, 0));
        create.execute();
    }
    
    /**
     * Creates the sub aspects and stores them (if necessary).
     */
    private void createSubAspects()
    {
        if (this.selection.getParents().size() > 1)
        {
            this.subAspects = new HashMap<IContainer, AspectContainer>();
            
            for (IContainer parent : this.selection.getParents())
            {
                AspectContainer sub = this.createSubAspect();
                
                this.subAspects.put(parent, sub);
            }
        }
        
        // if actors need to be unweaved, sub aspects can be required as well
        if (this.selection.getActorsThatRequireADecision().size() > 0)
        {
            for (Actor a : this.selection.getActorsThatRequireADecision())
            {
                List<Component> componentsAssociated = ModelModifierHelper.getInstance().getAssociatedComponent(a);
                
                if (componentsAssociated.size() == 1)
                {
                    if (this.subAspects != null &&
                        !this.subAspects.keySet().contains(componentsAssociated.get(0)))
                    {
                        AspectContainer aSub = this.createSubAspect();
                        this.subAspects.put(componentsAssociated.get(0), aSub);
                    }
                }
                else if (componentsAssociated.size() > 1)
                {
                    if (this.subAspects == null)
                    {
                        this.subAspects = new HashMap<IContainer, AspectContainer>();
                    }
                    
                    //if the actor is associated to more than one component
                    //a sub aspect is created (and multiple join relations will be distributed).
                    AspectContainer aSub = this.createSubAspect();
                    this.subAspects.put(componentsAssociated.get(0), aSub);
                }
            }
        }
    }
    
    /**
     * 
     */
    private void moveElements()
    {
        if (this.selection.getScenariosThatRequireADecision() != null &&
            this.selection.getScenariosThatRequireADecision().size() > 0)
        {
            this.extractScenarios();
        }
        
        if (this.selection.getActorsThatRequireADecision() != null &&
            this.selection.getActorsThatRequireADecision().size() > 0)
        {
            this.extractActors();
        }
        
        if (this.selection.getStatesThatRequireADecision() != null &&
            this.selection.getStatesThatRequireADecision().size() > 0)
        {
            this.extractStates();
        }

        this.moveRemainingElements();
        
        //just a check to make sure, that all selected elements have been extracted
        for (VisualizableModelElement e : this.selection.getSelectedElements())
        {
            if (!this.extractedElements.contains(e))
            {
                System.out.println("Element not extracted: " + e);
            }
        }
        
    }
    
    /**
     * This method does the extraction of states.
     */
    private void extractStates()
    {
        for (State s : this.selection.getStatesThatRequireADecision())
        {
            // check that because the state could have been extracted 
            // with another state
            if (!this.extractedElements.contains(s))
            {
                StateChain chain = new StateChain(s, this.selection.getStatesThatRequireADecision());
                
                if (chain.isUnweaveableChain())
                {
                    int weavingType = JoinRelation.BEFORE;
                    
                    //decide whether to use the default unweaving type or the after type
                    for (Connection c : chain.getLastState().getConnections())
                    {
                        if (c.getSource().equals(chain.getLastState()))
                        {
                            if (!this.selection.getSelectedElements().contains(c.getTarget()))
                            {
                                if (this.selection.containsElement(c))
                                {
                                    weavingType = JoinRelation.AFTER;
                                    this.selection.removeElementFromSelectoin(c);
                                }
                            }
                        }
                    }
                    
                    //now look where to put the extracted states
                    AspectContainer targetAspect = this.extractionAspect;
                    
                    if (this.subAspects != null)
                    {
                        targetAspect = this.subAspects.get(s.getParent());
                    }
                    
                    //first set the incoming transition of the first state to 
                    //the state that comes after the last state of the chain
                    Transition incoming = null;
                    Transition outgoing = null;
                    ConnectableElement firstAfterChain = null;
                    
                    for (Connection c : chain.getFirstState().getConnections())
                    {
                        if (c instanceof Transition &&
                            c.getTarget().equals(chain.getFirstState()))
                        {
                            if (!this.selection.getSelectedElements().contains(c.getSource()))
                            {
                                incoming = (Transition) c;
                            }
                        }
                    }
                    
                    for (Connection c : chain.getLastState().getConnections())
                    {
                        if (c instanceof Transition &&
                            c.getSource().equals(chain.getLastState()))
                        {
                            if (!this.selection.getSelectedElements().contains(c.getTarget()))
                            {
                                outgoing = (Transition) c;
                                firstAfterChain = c.getTarget();
                            }
                        }
                    }
                    
                    if (weavingType == JoinRelation.AFTER)
                    {
                        //create end point
                        ExitPoint exitPoint = new ExitPoint();
                        exitPoint.setName("Exit Point");
                        
                        NodeCreate nc = new NodeCreate(exitPoint, targetAspect, new Rectangle(5, 5, 100, 70));
                        nc.execute();
                        
                        InsertNodeVisually inv = new InsertNodeVisually(exitPoint);
                        inv.execute();
                        
                        TransitionReconnect tr = new TransitionReconnect(incoming);
                        tr.setNewTarget(firstAfterChain);
                        tr.setTargetClickPoint(new Point(0, 0));
                        tr.execute();
                                                
                        //connect the outgoing connection of the last state of the coin
                        //to the created exit point
                        tr = new TransitionReconnect(outgoing);
                        tr.setNewTarget(exitPoint);
                        tr.setTargetClickPoint(new Point(0, 0));
                        tr.execute();
                        
                        //now move the states of the chain into the aspect
                        List<State> movers = new ArrayList<State>();
                        movers.add(chain.getFirstState());
                        
                        while (movers.size() > 0)
                        {
                            List<State> newMovers = new ArrayList<State>();
                            
                            for (State mover : movers)
                            {
                                ModelModifierHelper.getInstance().moveModelElement(mover, targetAspect);
                                this.extractedElements.add(mover);
                                
                                for (State newMover : chain.getNextStates(mover))
                                {
                                    if (!this.extractedElements.contains(newMover))
                                    {
                                        newMovers.add(newMover);
                                    }
                                }
                            }
                            
                            movers = newMovers;
                        }
                        
                        //now create the join relation
                        this.createJoinRelation(s, incoming, weavingType);
                    }
                    else //weaving type before
                    {
                        List<State> movers = new ArrayList<State>();
                        movers.add(chain.getFirstState());
                        
                        while (movers.size() > 0)
                        {
                            List<State> newMovers = new ArrayList<State>();
                            
                            for (State mover : movers)
                            {
                                ModelModifierHelper.getInstance().moveModelElement(mover, targetAspect);
                                this.extractedElements.add(mover);
                                
                                for (State newMover : chain.getNextStates(mover))
                                {
                                    if (!this.extractedElements.contains(newMover))
                                    {
                                        newMovers.add(newMover);
                                    }
                                }
                            }
                            
                            movers = newMovers;
                        }
                        
                        //create the dummy states
                        State beforeDummy = new State();
                        beforeDummy.setName("before");
                        
                        NodeCreate nc = new NodeCreate(beforeDummy, targetAspect, new Rectangle(5, 5, 100, 70));
                        nc.execute();
                        
                        InsertNodeVisually inv = new InsertNodeVisually(beforeDummy);
                        inv.execute();
                        
                        ExitPoint afterDummy = new ExitPoint();
                        afterDummy.setName("after");
                        
                        nc = new NodeCreate(afterDummy, targetAspect, new Rectangle(5, 5, 100, 70));
                        nc.execute();
                        
                        inv = new InsertNodeVisually(afterDummy);
                        inv.execute();
                        
                        //create transitions for dummy states
                        TransitionCreate createBeforeTransition = new TransitionCreate(beforeDummy);
                        createBeforeTransition.setTarget(chain.getFirstState());
                        createBeforeTransition.setTargetClickPoint(new Point(0, 0));
                        createBeforeTransition.setSourceClickPoint(new Point(0, 0));
                        createBeforeTransition.execute();
                        Transition beforeTransition = (Transition) createBeforeTransition.getConnection();
                        beforeTransition.getLabel().setName("|" + incoming.getAction());
                        beforeTransition.getLabel().setSize(new Dimension(100, 25));

                        TransitionCreate createAfterTransition = new TransitionCreate(chain.getLastState());
                        createAfterTransition.setTarget(afterDummy);
                        createAfterTransition.setTargetClickPoint(new Point(0, 0));
                        createAfterTransition.setSourceClickPoint(new Point(0, 0));
                        createAfterTransition.execute();
                        Transition afterTransition = (Transition) createAfterTransition.getConnection();
                        afterTransition.getLabel().setName(outgoing.getEvent() + "|");
                        afterTransition.getLabel().setSize(new Dimension(100, 25));
                        
                        //reconnect and relabel the incoming connection
                        TransitionDescription throughGoingDescription = new TransitionDescription();
                        throughGoingDescription.setName(incoming.getEvent() + "|"  + outgoing.getAction());
                        incoming.setLabel(throughGoingDescription);
                        
                        TransitionReconnect throughReconnect = new TransitionReconnect(incoming);
                        throughReconnect.setNewTarget(outgoing.getTarget());
                        throughReconnect.setTargetClickPoint(new Point(0, 0));
                        throughReconnect.execute();
                        
                        //delete transition going out at the last state of the chain
                        ConnectionDelete cd = new ConnectionDelete(outgoing);
                        cd.execute();
                        
                        this.createJoinRelation(beforeDummy, incoming, JoinRelation.BEFORE);
                    }
                }
            }
        }
    }

    /**
     * This method does the extraction of scenarios.
     */
    private void extractScenarios()
    {
        for (Scenario s : this.selection.getScenariosThatRequireADecision())
        {
            List<Connection> connections = s.getConnections();
            
            boolean instead = false;
            
            for (Connection c : connections)
            {
                if (c.getTarget().equals(s) && c.getSource().equals(s.getScenarioParent()))
                {
                    if (!this.selection.getSelectedElements().contains(c))
                    {
                        instead = true;
                    }
                    else
                    {
                        this.selection.removeElementFromSelectoin(c);
                    }
                }
            }
            
            if (instead)
            {
                this.extractInsteadScenario(s);
            }
            else
            {
                if (s.getScenarioType() == Scenario.SEQUENCE)
                {
                    this.extractSequenceScenario(s);
                }
                else
                {
                    this.extractDefaultScenario(s);
                }
            }
        }
    }

    /**
     * This method handles the extraction of scenarios of the types
     * root, alternative and concurrency.
     * 
     * @param s Scenario
     */
    private void extractDefaultScenario(Scenario s)
    {
        Scenario target = null;
     
        //look for another alternative of the same scenario tree, which
        //is in the same container, to use it a s target for the join relation.
        Scenario parentScenario = s.getScenarioParent();
        
        List<Scenario> children = parentScenario.getScenarioChildren();
        
        children.remove(s);
        
        for (Scenario child : children)
        {
            if (!this.selection.getSelectedElements().contains(child) &&
                child.getScenarioType() == s.getScenarioType() &&
                child.getParent().equals(s.getParent()))
            {
                target = child;
                break;
            }
        }
        
        IContainer movedTo;
        
        if (this.subAspects == null)
        {
            movedTo = this.extractionAspect;
            ModelModifierHelper.getInstance().moveModelElement(s, this.extractionAspect);
        }
        else
        {
            movedTo = this.subAspects.get(s.getParent());
            ModelModifierHelper.getInstance().moveModelElement(s, this.subAspects.get(s.getParent()));
        }
        
        this.createJoinRelation(s, target, JoinRelation.noType);
        ModelModifierHelper.getInstance().removeScenarioConnectionAfterExtraction(s);
        this.moveChildrenScenarios(s, movedTo);
        this.extractedElements.add(s);
    }
    
    /**
     * This method handles the extraction of scenarios of the type
     * sequence.
     * 
     * @param s Scenario
     */
    private void extractSequenceScenario(Scenario s)
    {
        // the target of the join relation
        Scenario target = null;

        int weavingType = -1;

        //first look, where to attach the join relationship
        //for this scenario
        Scenario parent = s.getScenarioParent();

        List<Scenario> children = parent.getScenarioChildren();

        for (Scenario child : children)
        {
            //if there is another scenario of this scenario chunk
            //that is in the same container as the scenario that
            //is extracted, use that scenario as target for the join relationship
            if (child != s &&
                child.getParent().equals(s.getParent()) &&
                !this.selection.containsElement(child) &&
                child.getScenarioType() == Scenario.SEQUENCE)
            {
                if (Math.abs(child.getSeqNo() - s.getSeqNo()) == 1)
                {
                    target = child;
                    
                    if (child.getSeqNo() < s.getSeqNo())
                    {
                        weavingType = JoinRelation.AFTER;
                    }
                    else
                    {
                        weavingType = JoinRelation.BEFORE;
                    }
                }
            }
        }
        
        IContainer movedTo;
        
        if (this.subAspects == null)
        {
            movedTo = this.extractionAspect;
            ModelModifierHelper.getInstance().moveModelElement(s, this.extractionAspect);
        }
        else
        {
            movedTo = this.subAspects.get(s.getParent());
            ModelModifierHelper.getInstance().moveModelElement(s, this.subAspects.get(s.getParent()));
        }
        
        this.createJoinRelation(s, target, weavingType);

        //after moving the scenario, adapt the sequence numbers of the remaining
        //scenarios
        List<Scenario> childrenToAdapt = s.getScenarioParent().getScenarioChildren();
        childrenToAdapt.remove(s);

        ModelModifierHelper.getInstance().adaptSequenceNumbers(childrenToAdapt);
        
        //do this to remove the sequence number from the extracted scenario
        //it becomes a sequence scenario again, as soon as it is weaved again
        s.setScenarioType(Scenario.ROOT);
        
        ModelModifierHelper.getInstance().removeScenarioConnectionAfterExtraction(s);
        this.moveChildrenScenarios(s, movedTo);
        this.extractedElements.add(s);
    }
    
    /**
     * This method handles the extraction of scenarios with 
     * the weaving type instaed.
     * 
     * @param s Scenario
     */
    private void extractInsteadScenario(Scenario s)
    {
        IContainer targetAspect = this.extractionAspect;
        
        if (this.subAspects != null)
        {
            targetAspect = this.subAspects.get(s.getParent());
        }
        
        //remember position of the extracted scenario
        Point oldPosition = s.getLocation();
        IContainer oldParent = s.getParent();
        int oldScenarioType = s.getScenarioType();
        int oldSequenceNumber = -1;
        
        if (s.getScenarioType() == Scenario.SEQUENCE)
        {
            oldSequenceNumber = s.getSeqNo();
        }
        
        //move the scenario
        ModelModifierHelper.getInstance().moveModelElement(s, targetAspect);
        
        //move its children
        this.moveChildrenScenarios(s, targetAspect);
        
        //create the placeholder
        Scenario placeholder = new Scenario();
        placeholder.setName("PLACEHOLDER");
        placeholder.setLocation(oldPosition);
        
        NodeCreate nc = new NodeCreate(placeholder, (INodeContainer) oldParent, new Rectangle(oldPosition.x, oldPosition.y, 100, 70));
        nc.execute();
        
        InsertNodeVisually inv = new InsertNodeVisually(placeholder);
        inv.execute();
        
        //find and reconnect the scenario connection to the extracted scenario
        ScenarioConnection toTheExtractedScenario = null;
        
        for (Connection c : s.getConnections())
        {
            if (c instanceof ScenarioConnection &&
                c.getTarget().equals(s))
            {
                toTheExtractedScenario = (ScenarioConnection) c;
            }
        }
            
        
        ScenarioConnectionReconnect scr = new ScenarioConnectionReconnect(toTheExtractedScenario);
        scr.setNewTarget(placeholder);
        scr.setTargetClickPoint(new Point(0, 0));
        scr.setSourceClickPoint(new Point(0, 0));
        scr.execute();
        
        placeholder.setScenarioType(oldScenarioType);
        
        if (s.getScenarioType() == Scenario.SEQUENCE)
        {
            placeholder.setSeqNr(oldSequenceNumber);
            s.setScenarioType(Scenario.ROOT);
        }
        
        //create the join relation
        this.createJoinRelation(s, placeholder, JoinRelation.INSTEAD);
        
        this.extractedElements.add(s);
    }
       
    /**
     * Moves the children scenarios of an extracted scenario to the same 
     * aspect container.
     * 
     * @param s Scenario, the extrated scenario
     * @param moveTo AspectContainer
     */
    private void moveChildrenScenarios(Scenario s, IContainer moveTo)
    {
        //now move the children scenarios too
        //but only if they are not going to be unweaved anyway with a selected
        //abstract object
        for (Scenario childrenScenario : s.getAllScenarioChildren())
        {
            boolean moveIt = true;
            
            //iterate over the parents of the scenario to find out, if it 
            //is inside a container that is selected (ant therefore will be unweaved
            //anyway
            IContainer childScenarioParent = childrenScenario.getParent();
            
            while (!(childScenarioParent instanceof AdoraModel))
            {
                if (selection.getSelectedElements().contains(childScenarioParent))
                {
                    moveIt = false;
                    break;
                }
                
                childScenarioParent = childScenarioParent.getParent();
            }
            
            if (moveIt)
            {
                ModelModifierHelper.getInstance().moveModelElement(childrenScenario, moveTo);
            }
        }
    }

    /**
     * Handles the extraction of actors. 
     */
    private void extractActors()
    {
        for (Actor a : this.selection.getActorsThatRequireADecision())
        {
            if (this.subAspects == null)
            {
                ModelModifierHelper.getInstance().moveModelElement(a, this.extractionAspect);
                this.extractionSource = ModelModifierHelper.getInstance().getAssociatedComponent(a).get(0);
            }
            else
            {
                List<Component> componentsAssociated = ModelModifierHelper.getInstance().getAssociatedComponent(a);
                
                if (componentsAssociated.size() == 1)
                {
                    ModelModifierHelper.getInstance().moveModelElement(a, this.subAspects.get(componentsAssociated.get(0)));
                    //note: in this case there is no need to create a join relation, this will be done later on
                }
                else if (componentsAssociated.size() > 1)
                {
                    ModelModifierHelper.getInstance().moveModelElement(a, this.subAspects.get(componentsAssociated.get(0)));
                    
                    //in this case, a join relation has to be created for each component
                    //the actor is associated with
                    for (Component comp : componentsAssociated)
                    {
                        this.createJoinRelation(this.subAspects.get(componentsAssociated.get(0)), comp, JoinRelation.noType);
                    }
                }
            }
            
            this.extractedElements.add(a);
        }
    }

    /**
     * Moves the remaining elements (components or complete chunks) that
     * have not been moved yet.
     */
    private void moveRemainingElements()
    {
        //for all the selected elements
        for (VisualizableModelElement e : this.selection.getSelectedElements())
        {
            // that have not yet been extracted
            if (!this.extractedElements.contains(e))
            {
                if (this.subAspects != null)
                {
                    ModelModifierHelper.getInstance().moveModelElement(e, this.subAspects.get(e.getParent()));
                }
                else
                {
                    // remember where the elements come from to be able
                    // to create a decision (if no one exists yet)
                    if (this.extractionSource == null)
                    {
                        this.extractionSource = e.getParent();
                    }
                    
                    ModelModifierHelper.getInstance().moveModelElement(e, this.extractionAspect);
                }
                
                this.extractedElements.add(e);
            }
        }
        
        //create join relations, where none have been created yet
        this.createRemainingJoinRelationships();
    }
    
    /**
     * Adds a join relationship to the aspects that don't have one yet.
     */
    private void createRemainingJoinRelationships()
    {
        if (this.subAspects == null)
        {
            if (!ModelModifierHelper.getInstance().hasJoinRelationship(this.extractionAspect))
            {
                this.createJoinRelation(this.extractionAspect, (ConnectableElement) this.extractionSource, JoinRelation.noType);
            }
        }
        else
        {
            for (AspectContainer a : this.subAspects.values())
            {
                if (!ModelModifierHelper.getInstance().hasJoinRelationship(a))
                {
                    //find source container
                    for (IContainer ic : this.subAspects.keySet())
                    {
                        if (this.subAspects.get(ic).equals(a))
                        {
                            this.createJoinRelation(a, (ConnectableElement) ic, JoinRelation.noType);        
                        }
                    }
                }
            }
        }
    }    
    
    /**
     * Creates a join relatoin.
     * 
     * @param from Node
     * @param to ConnectableElement
     * @param weavingType int (use the fields of the class JoinRelation)
     */
    private void createJoinRelation(Node from, ConnectableElement to, int weavingType)
    {
        JoinRelation jr = new JoinRelation(from, to);
        
        jr.setWeavingType(weavingType);
        
        if (this.selection.getUnweaveDecision() == null)
        {
            Decision d = new Decision(jr);
            this.selection.setUnweaveDecision(d);
        }
        else
        {
            this.selection.getUnweaveDecision().addJoinRelation(jr);
            jr.setDecision(this.selection.getUnweaveDecision());
        }
    }

    /**
     * Creates a single sub aspect inside the main extraction aspect.
     * This is needed if not all of the extracted elements have the
     * same parent container.
     * 
     * @return AspectContainer
     */
    private AspectContainer createSubAspect() 
    {
        AspectContainer subAspect = new AspectContainer();
        //subAspect.setVariability(true); //threw an exception
        
        NodeCreate nc = new NodeCreate(subAspect, this.extractionAspect, new Rectangle(5, 5, 5, 5));
        nc.execute();
        
        InsertNodeVisually ins = new InsertNodeVisually(subAspect);
        ins.execute();
        
        return subAspect;
    }
    
    /**
     * In the end, all the used aspects are compacted again to
     * get an acceptable layout.
     */
    private void doFinalCompact()
    {
        if (this.subAspects != null)
        {
            for (IContainer c : this.subAspects.keySet())
            {
                NodeCompact nc = new NodeCompact();
                nc.setTarget(this.subAspects.get(c));
                nc.execute();
            }
        }
        
        NodeCompact nc = new NodeCompact();
        nc.setTarget(this.extractionAspect);
        nc.execute();
    }
}