package ch.unizh.ifi.adora.unweaver.modelModification;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.model.commands.InsertNodeVisually;
import ch.unizh.ifi.adora.model.commands.NodeCompact;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.commands.VisualizableModelElementSetConstraint;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;

/**
 * This class offers several functions that are used by the unweaver.
 * These functions are of a general manner and are used by nearly any kind of
 * unweaving, this is why they are they are placed in this helper class.
 * 
 * @author Michael Jehle
 *
 */
public class ModelModifierHelper
{
    private static ModelModifierHelper INSTANCE;

    public static ModelModifierHelper getInstance()
    {
        if (ModelModifierHelper.INSTANCE == null)
        {
            ModelModifierHelper.INSTANCE = new ModelModifierHelper();
        }
        
        return ModelModifierHelper.INSTANCE;
    }
    
    private ModelModifierHelper()
    {
    }
    
    /**
     * Moves a model element into another container.
     * 
     * @param mover VisualizableModelElement, the element that should be moved.
     * @param newParent IContainer, the target container
     */
    public void moveModelElement(VisualizableModelElement mover, IContainer newParent)
    {
        Dimension oldSize = mover.getSize();
        IContainer oldParent = mover.getParent();
        
        RemoveNodeVisually rnv = new RemoveNodeVisually((Node) mover);
        rnv.execute();
        
        mover.getParent().removeChild(mover);

        NodeCompact nc;
        
        if (!(mover instanceof Actor))
        {
            nc = new NodeCompact();
            nc.setTarget((ZoomableNodeContainer) oldParent);
            nc.execute();
        }

        newParent.addChild(mover);
        
        mover.setSize(oldSize);
        
        InsertNodeVisually inv = new InsertNodeVisually((Node) mover, true);
        inv.execute();
        
        Point moverCoords = LayoutHandler.getInstance().getInsertionPointFor(mover);
        
        //enlarge container first, if necessary
        AspectContainer newParentA = (AspectContainer) newParent;
        
        if (newParentA.getSize().width < (moverCoords.x + 120) ||
            newParentA.getSize().height < (moverCoords.y + 90))
        {
            Dimension newParentDimension = new Dimension(moverCoords.x + 120, moverCoords.y + 90);
            VisualizableModelElementSetConstraint enlargeContainer = new VisualizableModelElementSetConstraint((VisualizableModelElement) newParent, new Rectangle(newParentA.getLocation(), newParentDimension));
            enlargeContainer.execute();
        }

//        System.out.print(mover + ": " + moverCoords);

        VisualizableModelElementSetConstraint placeMover = new VisualizableModelElementSetConstraint(mover, new Rectangle(moverCoords, mover.getSize()));
        placeMover.execute();
        
//        System.out.println(" -> " + mover.getLocation());
    }

    /**
     * Adapts the sequence numbers of a sequence of scenarios
     * after one of them has been extracted.
     * 
     * As a result of the adaption, the scenario sequence will be 
     * continuous (1 - n) without gaps.
     * 
     * @param scenarios List<Scenario>
     */
    public void adaptSequenceNumbers(List<Scenario> scenarios)
    {
        int numberOfScenarios = scenarios.size();
        
        for (int lookForNr = 1; lookForNr < numberOfScenarios + 1; lookForNr++)
        {
            boolean foundThatOne  = false;
            
            for (Scenario s : scenarios)
            {
                if (s.getSeqNo() == lookForNr)
                {
                    foundThatOne = true;
                    break;
                }
            }
            
            if (!foundThatOne)
            {
                int lookForNext = lookForNr + 1;
                boolean foundNext = false;
                
                while (!foundNext)
                {
                    for (Scenario s : scenarios)
                    {
                        if (s.getSeqNo() == lookForNext)
                        {
                            s.setSeqNr(lookForNr);
                            foundNext = true;
                            break;
                        }
                    }
                    
                    lookForNext++;
                }
            }
        }
    }
    
    /**
     * Checks if the given aspect container or one of its direct children
     * has an outgoing join relation.
     * 
     * @param a AspectContainer
     * @return boolean
     */
    public boolean hasJoinRelationship(AspectContainer a)
    {
        if (this.hasJoinRelationshipForConnectableElement(a))
        {
            return true;
        }
        
        for (ModelElement e : a.getChildren())
        {
            if (e instanceof ConnectableElement)
            {
                if (this.hasJoinRelationshipForConnectableElement((ConnectableElement) e))
                {
                    return true;
                }
            }
        }
        
        return false;
    }
    
    /**
     * Checks if a connectable element has an outgoing join relation.
     * 
     * @param e ConnectableElement
     * @return boolean
     */
    private boolean hasJoinRelationshipForConnectableElement(ConnectableElement e)
    {
        boolean hasJoinRelation = false;
        
        for (Connection c : e.getConnections())
        {
            if (c instanceof JoinRelation && c.getSource().equals(e))
            {
                hasJoinRelation = true;
            }
        }
        
        return hasJoinRelation;
    }
    
    /**
     * Removes the scenario connection of an extracted scenario.
     * 
     * @param s Scenario
     */
    public void removeScenarioConnectionAfterExtraction(Scenario s)
    {
        for (Connection c : s.getConnections())
        {
            if (c.getSource().equals(s.getScenarioParent()))
            {
                ConnectionDelete cd = new ConnectionDelete(c);
                cd.execute();
            }
        }
    }
    
    /**
     * Returns the components associated to the actor (empty list if none found).
     * 
     * @param a Actor
     * @return List<Component>
     */
    public List<Component> getAssociatedComponent(Actor a)
    {
        List<Component> result = new ArrayList<Component>();
        
        for (Connection c : a.getConnections())
        {
            if (c.getSource() instanceof Component)
            {
                if (!result.contains(c.getSource()))
                {
                    result.add((Component) c.getSource());
                }
            }
            else if (c.getTarget() instanceof Component)
            {
                if (!result.contains(c.getTarget()))
                {
                    result.add((Component) c.getTarget());
                }
            }
        }
        
        return result;
    }
    
    /**
     * Helper method to get all the previous states (or components) of a state.
     * Only elements that are directly before state s, are returned.
     * 
     * @param s State
     * @return List<ConnectableElement>
     */
    public List<State> getPreviousStates(State s)
    {
        List<State> previous = new ArrayList<State>();
        
        for (Connection c : s.getConnections())
        {
            if (c instanceof Transition)
            {
                if (c.getTarget().equals(s))
                {
                    if (c.getSource() instanceof State)
                    {
                        previous.add((State) c.getSource());
                    }
                }
            }
        }
        
        return previous;
    }
    
    /**
     * Helper method. Returns the states that follow directly after state s.
     * 
     * @param s State
     * @return List<ConnectableElement>
     */
    public List<State> getFollowingStates(State s)
    {
        List<State> followers = new ArrayList<State>();
        
        for (Connection c: s.getConnections())
        {
            if (c instanceof Transition)
            {
                if (c.getSource().equals(s))
                {
                    if (c.getTarget() instanceof State)
                    {
                        followers.add((State) c.getTarget());
                    }
                }
            }
        }
        
        return followers;
    }
    
    /**
     * Checks if the state s has exactly one incoming and one outgoing transition.
     * Transitions are only counted, if they are connected to a state that is not
     * part of the selection.
     * Returns true if yes, false otherwise.
     * 
     * @param s State
     * @return boolean.
     */
    public boolean canUnweaveAsSingleState(State s, List<State> selectedStates)
    {
        int outgoing = 0;
        int incoming = 0;
        
        for (Connection c : s.getConnections())
        {
            if (c instanceof Transition)
            {
                if (c.getTarget().equals(s))
                {
                    if (!selectedStates.contains(c.getSource()))
                    {
                        incoming++;
                    }
                }
                if (c.getSource().equals(s))
                {
                    if (!selectedStates.contains(c.getTarget()))
                    {
                        outgoing++;
                    }
                }
            }
        }
        
        if (outgoing == 1 && incoming == 1)
        {
            return true;
        }
        
        return false;
    }
    
    public Point findTopLeftPoint()
    {
        int leftMost = Integer.MAX_VALUE;
        int topMost = Integer.MAX_VALUE;
        
        Iterator<ModelElement> it = AdoraPlugin.getActiveModel().getChildrenIterator();
        
        while (it.hasNext())
        {
            ModelElement e = it.next();
            
            if (e instanceof VisualizableModelElement)
            {
                VisualizableModelElement vme = (VisualizableModelElement) e;
                
                Point loc = vme.getLocation();
                
                if (loc.x < leftMost)
                {
                    leftMost = loc.x;
                }
                
                if (loc.y < topMost)
                {
                    topMost = loc.y;
                }
            }
        }
        
        return new Point(leftMost, topMost);
    }
}