package ch.unizh.ifi.adora.unweaver.selection;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;

/**
 * This class analyzes the selection for situations that are not supported 
 * by the unweaver.
 * 
 * @author Michael Jehle
 */
public class SelectionAnalyzer
{
    private UserSelection userSelection;
    private List<String> errorMessages = new ArrayList<String>();
    
    /**
     * Constructor.
     * 
     * @param selection UserSelection
     */
    public SelectionAnalyzer(UserSelection selection)
    {
        this.userSelection = selection;
        this.analyzeSelection();
    }
    
    /**
     * Template method that directs the analyze of the selection.
     */
    public void analyzeSelection()
    {
        this.checkNoTopLevelComponents();
        this.checkScenarios();
        this.checkStates();
    }
    
    /**
     * Returns the list of error messages found or an empty list
     * if no error messages are detected.
     * 
     * @return List<String>
     */
    public List<String> getErrorMessages()
    {
        return this.errorMessages;
    }
    
    /**
     * Checks, if the selection contains top level components (directly 
     * on the adora model. These can not be unweaved.
     */
    private void checkNoTopLevelComponents()
    {
        for (VisualizableModelElement e : this.userSelection.getSelectedElements())
        {
            if (e instanceof Component)
            {
                if (((Component) e).getParent() instanceof AdoraModel)
                {
                    this.errorMessages.add(e.getName() + ": Extraction of top level components not possible");
                }
            }
        }
    }
    
    /**
     * Check the selected scenarios for:
     * 
     * - Root scenarios (the root scenario of the scenario tree can not be unweaved)
     * - Children of a selected scenario that are in another component
     * - A scenario in the same container, where the join relation can be attached
     */
    private void checkScenarios()
    {
        for (Scenario s : this.userSelection.getScenariosThatRequireADecision())
        {
            //check for root scenario
            if (s.getScenarioParent() == null)
            {
                this.errorMessages.add(s.getName() + ": Extraction of root scenario not possible");
                return;
            }
            
            //check children
            for (Scenario child : s.getAllScenarioChildren())
            {
                if (!child.getParent().equals(s.getParent()))
                {
                    this.errorMessages.add(s.getName() + ": Extraction not possible, child scenario " + child.getName() + " is in a different component");
                }
            }
            
            //check for a scenario to append the join relation
            Scenario sParent = s.getScenarioParent();
            List<Scenario> sameLevelSameContainer = new ArrayList<Scenario>();
            
            if (sParent != null)
            {
                for (Scenario onSameLevel : sParent.getScenarioChildren())
                {
                    if (!onSameLevel.equals(s))
                    {
                       if (onSameLevel.getParent().equals(s.getParent()))
                       {
                           sameLevelSameContainer.add(onSameLevel);
                       }
                    }
                }
            }
            
            boolean foundOne = false;
            
            if (s.getScenarioType() == Scenario.SEQUENCE)
            {
                for (Scenario slsc : sameLevelSameContainer)
                {
                    if (Math.abs(slsc.getSeqNo() - s.getSeqNo()) == 1)
                    {
                        foundOne = true;
                    }
                }
            }
            else
            {
                if (sameLevelSameContainer.size() > 0)
                {
                    foundOne = true;
                }
            }
            
            if (!foundOne)
            {
                this.errorMessages.add(s.getName() + ": there is no scenario, where the join relationship could be attached.");
            }
        }
    }
    
    private void checkStates()
    {
        List<State> statesChecked = new ArrayList<State>();
        
        for (State s : this.userSelection.getStatesThatRequireADecision())
        {
            if (!statesChecked.contains(s))
            {
                //skip single states that don't have any transitions
                if (s.getConnections().size() == 0)
                {
                    statesChecked.add(s);
                    continue;
                }
                
                StateChain chain = new StateChain(s, this.userSelection.getStatesThatRequireADecision());
                
                List<State> chainStates = chain.getStates();
                
                for (State inChain : chainStates)
                {
                    statesChecked.add(inChain);
                }
                
                if (!chain.isUnweaveableChain())
                {
                    StringBuffer message = new StringBuffer();
                    
                    message.append("States ");
                    
                    for (State problemState : chainStates)
                    {
                        message.append(problemState.getName() + ", ");
                    }
                    
                    message.append("can not be unweaved. First and last state of selection must have exactly one incoming and one outgoing transition.");
                    
                    this.errorMessages.add(message.toString());
                }
            }
        }
    }
}