package ch.unizh.ifi.adora.unweaver.selection;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.unweaver.modelModification.ModelModifierHelper;

/**
 * This class represents a chain of states that follow each other directly.
 * The chain's first and last state must be states that have only one incoming and
 * one outgoing state transition (this is a precondition for the unweaving of states).
 *  
 * @author MichaelJehle
 */
public class StateChain
{
    private List<StateChainElement> chain = new ArrayList<StateChainElement>();
    private List<State> selectedStates;
    
    /**
     * List of state chain element that fulfill the criteria to be the last element
     * of the chain. Tha chain chan only be unweaved, if this list contains exactly one element.
     */
    private List<StateChainElement> last = new ArrayList<StateChainElement>();
    
    /**
     * List of state chain element that fulfill the criteria to be the first element
     * of the chain. Tha chain chan only be unweaved, if this list contains exactly one element.
     */
    private List<StateChainElement> first = new ArrayList<StateChainElement>();

    /**
     * Constructor that establishes the whole chain for state s.
     * 
     * @param s State
     * @param selectedStates List<State>
     */
    public StateChain(State s, List<State> selectedStates)
    {
        this.selectedStates = selectedStates;
        
        StateChainElement e = this.createStateChainElement(s, selectedStates);
        
        this.chain.add(e);

        List<State> previous = e.before;
        List<State> following = e.after;

        while (previous.size() > 0)
        {
            List<State> newPrevious = new ArrayList<State>();
            
            for (State p : previous)
            {
                if (!this.containsState(p))
                {
                    StateChainElement sceP = this.createStateChainElement(p, selectedStates);
                    this.chain.add(sceP);
                
                    for (State np : sceP.before)
                    {
                        newPrevious.add(np);
                    }
                }
            }
            
            previous = newPrevious;
        }
        
        while (following.size() > 0)
        {
            List<State> newFollowing = new ArrayList<State>();
            
            for (State f : following)
            {
                if (!this.containsState(f))
                {
                    StateChainElement sceF = this.createStateChainElement(f, selectedStates);
                    this.chain.add(sceF);
                
                    for (State nf : sceF.after)
                    {
                        newFollowing.add(nf);
                    }
                }
            }
            
            following = newFollowing;
        }
        
        this.setFirstChainElements();
        this.setLastChainElements();
    }
    
    /**
     * Finds all the elements that fulfill the criteria of being
     * the chain's first element.
     */
    private void setFirstChainElements()
    {
        for (StateChainElement e : this.chain)
        {
            int before = 0;
            
            State s = e.itself;
            
            List<State> previous = ModelModifierHelper.getInstance().getPreviousStates(s);
            
            for (State b : previous)
            {
                if (!this.selectedStates.contains(b))
                {
                    before++;
                }
            }
            
            if (before == 1)
            {
                this.first.add(e);
            }
        }
    }
    
    /**
     * Finds all the elements that fulfill the criteria of being the chain's 
     * last element.
     */
    private void setLastChainElements()
    {
        for (StateChainElement e : this.chain)
        {
            int after = 0;
            
            State s = e.itself;
            
            List<State> following = ModelModifierHelper.getInstance().getFollowingStates(s);
            
            for (State a : following)
            {
                if (!this.selectedStates.contains(a))
                {
                    after++;
                }
            }
            
            if (after == 1)
            {
                this.last.add(e);
            }
        }
    }
    
    private StateChainElement getLastChainElement()
    {
        if (this.last.size() == 1)
        {
            return this.last.get(0);
        }

        return null;
    }
    
    private StateChainElement getFirstChainElement()
    {
        if (this.first.size() == 1)
        {
            return this.first.get(0);
        }
        
        return null;
    }
    
    public State getLastState()
    {
        return this.getLastChainElement().itself;
    }
    
    public State getFirstState()
    {
        return this.getFirstChainElement().itself;
    }
    
    public List<State> getNextStates(State s)
    {
        for (StateChainElement sce : this.chain)
        {
            if (sce.itself.equals(s))
            {
                return sce.after;
            }
        }
        
        return null;
    }
    
    private boolean containsState(State s)
    {
        for (StateChainElement sce : this.chain)
        {
            if (sce.itself.equals(s))
            {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * Creates a state chain element.
     * 
     * @param s State
     * @param selectedStates List<State>
     * @return StateChainElement
     */
    private StateChainElement createStateChainElement(State s, List<State> selectedStates)
    {
        List<State> prev = ModelModifierHelper.getInstance().getPreviousStates(s);
        List<State> foll = ModelModifierHelper.getInstance().getFollowingStates(s);
        
        //adapt prev and foll according to the user selection
        List<State> prevNew = new ArrayList<State>();
        
        for (State p : prev)
        {
            if (selectedStates.contains(p))
            {
                prevNew.add(p);
            }
        }
        
        prev = prevNew;
        
        List<State> follNew = new ArrayList<State>();
        
        for (State f : foll)
        {
            if (selectedStates.contains(f))
            {
                follNew.add(f);
            }
        }
        
        foll = follNew;
        
        return new StateChainElement(s, prev, foll);
    }
    
    /**
     * Indicates whether this chain of states can be unweaved or not.
     * It can be unweaved if the chain's first and last state have exactly one incoming
     * and one outgoing transition.
     * 
     * @return boolean
     */
    public boolean isUnweaveableChain()
    {
        if (this.last.size() == 1 && this.first.size() == 1)
        {
            return true;
        }
        
        return false;
    }
    
    /**
     * Returns a list containing all the states which are part of the chain.
     * 
     * @return List<State>
     */
    public List<State> getStates()
    {
        List<State> result = new ArrayList<State>();
        
        for (StateChainElement e : this.chain)
        {
            result.add(e.itself);
        }
        
        return result;
    }
    
    /**
     * Inner class. State chain elements act similar to a linked list.
     * Every state chain element knows its previous and following states.
     * 
     * @author Michael Jehle
     */
    private class StateChainElement
    {
        List<State> before;
        List<State> after;
        State itself;

        StateChainElement(State theState, List<State> before, List<State> after)
        {
            this.after = after;
            this.before = before;
            this.itself = theState;
        }
    }
}