package ch.unizh.ifi.adora.unweaver.selection;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.unweaver.modelModification.ModelModifierHelper;


/**
 * This class represents the selection of model elements, the user made
 * to let them be unweaved.
 * Besides the selection itself, this class stores additional information
 * to define how the elements will be unweaved.
 * 
 * @author Michael Jehle
 */
public class UserSelection
{
    /**
     * The elements selected by the user. 
     */
    private List<VisualizableModelElement> selectedElements;
    
    /**
     * References to the scenarios in the selection that definitely need an own join relationship.
     */
    private List<Scenario> scenariosRequireDecision = new ArrayList<Scenario>();
    
    /**
     * References to the states int the selection that need a join relation to be
     * unweaved. This is the case for all the states except state charts that are
     * completely selected.
     */
    private List<State> statesRequireDecision = new ArrayList<State>();
    
    /**
     * References to the selected actors, that definitely need an own join relationship
     * to be unweaved.
     */
    private List<Actor> actorsRequireDecision = new ArrayList<Actor>();
    
    /**
     * A list of all the parent containers that occur in the selection.
     * This is important to find out, if and how many sub aspects are needed.
     */
    private List<IContainer> parents = new ArrayList<IContainer>();
    
    /**
     * This is the decision which is used to unweave this 
     * selection.
     */
    private Decision unweaveWithDecision = null;
    
    /**
     * Constructor.
     * 
     * @param selection List<VisualizableModelElement>
     */
    public UserSelection(List<VisualizableModelElement> selection)
    {
        this.selectedElements = selection;
        this.analyzeParents();
    }
    
    /**
     * Set the decision this selection is unweaved with.
     * @param d Decision
     */
    public void setUnweaveDecision(Decision d)
    {
        this.unweaveWithDecision = d;
    }
    
    /**
     * Returns the decision this selection is unweaved with.
     * Null if it is not set yet.
     * 
     * @return Decision
     */
    public Decision getUnweaveDecision()
    {
        return this.unweaveWithDecision;
    }
    
    /**
     * Returns whether an element is part of the selection or not.
     * 
     * @param e VisualizableModelElement.
     * @return boolean
     */
    public boolean containsElement(VisualizableModelElement e)
    {
        return this.selectedElements.contains(e);
    }
    
    /**
     * Returns a list of all the selected model elements.
     * 
     * @return List<VisualizableModelElements
     */
    public List<VisualizableModelElement> getSelectedElements()
    {
        return this.selectedElements;
    }
    
    /**
     * This method detects model elements that definitely need a decision
     * to unweave them (such as (incomplete) scenarios or states.
     */
    public void detectRequiredDecisions()
    {
        for (VisualizableModelElement element : this.selectedElements)
        {
            if (element instanceof Scenario)
            {
                //if it is only a part of a scenario (not a complete scenario)
                if (!this.isCompleteScenarioSelected((Scenario) element))
                {
                    this.scenariosRequireDecision.add((Scenario) element);
                }
            }
            
            if (element instanceof State)
            {
                this.statesRequireDecision.add((State) element);
            }
            
            if (element instanceof Actor)
            {
                List<Component> componentsAssociated = ModelModifierHelper.getInstance().getAssociatedComponent((Actor) element);
                
                for (Component comp : componentsAssociated)
                {
                    if (comp.getParent() instanceof AdoraModel)
                    {
                        if (!this.actorsRequireDecision.contains(element))
                        {
                            this.actorsRequireDecision.add((Actor) element);
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Checks, if the scenario tree, where s belongs to, is selected
     * completely.
     * 
     * @param s Scenario
     * @return boolean
     */
    private boolean isCompleteScenarioSelected(Scenario s)
    {
        //find the root
        Scenario parent = s;
        
        boolean goHigher = true;
        
        while (goHigher)
        {
            if (parent.getScenarioParent() != null)
            {
                parent = parent.getScenarioParent();
            }
            else
            {
                goHigher = false;
            }
        }
        
        //now check the scenario tree
        List<Scenario> check = new ArrayList<Scenario>();
        check.add(parent);
        
        while (check.size() > 0)
        {
            List<Scenario> checkNext = new ArrayList<Scenario>();
            
            for (Scenario checkIt : check)
            {
                if (!this.selectedElements.contains(checkIt))
                {
                    return false;
                }
                
                for (Scenario ch : checkIt.getScenarioChildren())
                {
                    checkNext.add(ch);
                }
            }
            
            check = checkNext;
        }
        
        return true;
    }
    
    /**
     * Returns the list of scenarios that definitely need 
     * an own join relationship.
     * 
     * @return List<Scenario>
     */
    public List<Scenario> getScenariosThatRequireADecision()
    {
        return this.scenariosRequireDecision;
    }
    
    /**
     * Returns the list of states that need an own decision.
     *  
     * @return List<State>
     */
    public List<State> getStatesThatRequireADecision()
    {
        return this.statesRequireDecision;
    }
    
    /**
     * Returns a list of all the actors that definitely need an own join
     * relation to unweave them.
     * 
     * @return List<Actor>
     */
    public List<Actor> getActorsThatRequireADecision()
    {
        return this.actorsRequireDecision;
    }
    
    /**
     * Removes an element from the selection.
     * 
     * @param vme VisualizableModelElement
     */
    public void removeElementFromSelectoin(VisualizableModelElement vme)
    {
        this.selectedElements.remove(vme);
    }
 
    /**
     * Create the list of all distinct parents of the selected
     * elements.
     */
    private void analyzeParents()
    {
        for (VisualizableModelElement e : this.selectedElements)
        {
            if (!this.parents.contains(e.getParent()))
            {
                if (e.getParent() != null)
                {
                    this.parents.add(e.getParent());   
                }                
            }
        }
    }
    
    public List<IContainer> getParents()
    {
        return this.parents;
    }
}