package ch.unizh.ifi.adora.weaver.engine;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.weaver.engine.analyzer.CrosscuttingNodeGroupList;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingHistory;

/**
 * The weaving main class.<br>
 * Implemented as a singleton.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 18.06.2007 SM First Version<br>
 *          10.12.2007 PS Comments improved.
 * @version $Date: 2009-07-01 12:52:32 +0200 (Wed, 01 Jul 2009) $, $Author: jehle $, $Revision: 4414 $
 */
public class Weaver {
	/**
	 * Weaver Helper contains all Helper Method of this class.
	 */
	private WeaverHelper weaverHelper;
	
	private PluginActivatorHelper pluginActivatorHelper;

	/**
	 * The instance of the weaver.
	 */
	private static Weaver INSTANCE = null;
	/**
	 * Returns the weaver.
	 * 
	 * @return The instance of the weaver.
	 */
	public static Weaver getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new Weaver();
		}

		return INSTANCE;
	}
	/**
	 * Returns the weaver.
	 * 
	 * @param disableLayoutCorrections
	 *            Parameter which you can set to true if you dont want to have
	 *            layout corrections.
	 * 
	 * @return The instance of the weaver.
	 */
	/**
	 * This aspectContainers consists all AspectContainer, which are useless
	 * after the weaving process and need deletion
	 */
	public static Weaver getInstance(boolean disableLayoutCorrections) {
		if (INSTANCE == null) {
			INSTANCE = new Weaver(disableLayoutCorrections);
		}

		return INSTANCE;
	}
	/**
	 * Singleton.
	 * 
	 * @see Weaver.getInstance()
	 */
	private Weaver() {
		this(false);
	}
	/**
	 * Singleton.
	 * 
	 * @param disableLayoutCorrections
	 *            Parameter which you can set to true if you dont want to have
	 *            layout corrections.
	 * 
	 * @see Weaver.getInstance()
	 */
	private Weaver(boolean disableLayoutCorrections) {
		this.weaverHelper = new WeaverHelper();
		this.pluginActivatorHelper=new PluginActivatorHelper();
		weaverHelper.setDisableLayoutCorrections(disableLayoutCorrections);
	}
	/**
	 * Transforms the AdoraModel so that the aspects are weaved into the model.
	 * 
	 * @param AdoraModelEditor
	 *            The AdoraModelEditor with the current model.
	 */
	public boolean transformModel(AdoraModelEditor editor) {
		AdoraModel model;
		try {
			if (pluginActivatorHelper.activateConstraintPluginValidation()) {
				return false;
			}
			if (pluginActivatorHelper.activateSyntaxPluginValidation(editor)) {
				return false;
			}
			weaverHelper.removeFilters(editor);
			/*
			 * Clone the model.
			 */
			model = editor.getModel().modelCopy();
			editor.setModel(model);
//			weaverHelper.removeJoinRelations(model);
			weaverHelper.analyzesScenarioInsteads(model, false);
			// Reset the singletons
			WeavingHistory.reset();
			CrosscuttingNodeGroupList.reset();
			weaverHelper.analyzeModel(model, false);
			weaverHelper.removeActors(model);
			weaverHelper.checkAndConnectServerComponent(model);

			/*
			 * After the weaving process, all the aspect containers are obsolete
			 * and can be deleted.
			 */
			weaverHelper.removeAspectContainers(model, false);
			weaverHelper.duplicateNameCheck(model);
			if (!weaverHelper.isDisableLayoutCorrections()) {
				FilterHelper.getInstance().applyFilters(model);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return true;
	}
}
