package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;

/**
 * This class stores any Actor with outgoing join relations. This actors can be
 * passed to a command generator for the weaving process.<br>
 * Implemented as singleton.
 * 
 * @author vontobel
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 23.10.2007 MV First Version<br>
 *          10.12.2007 PS Comments improved.
 * @version $Date: 2007-12-10 11:19:21 +0100 (Mon, 10 Dec 2007) $, $Author: schoeni $, $Revision: 3373 $
 */
public class ActorAssimilationAnalyzer {
	/**
	 * All the actors, which will be checked for assimilating them.
	 */
	private ArrayList<Actor> actors = new ArrayList<Actor>();

	/**
	 * All the actors, which are already assimilated.
	 */
	private ArrayList<Actor> assimilatedActors = new ArrayList<Actor>();

	/**
	 * Checks if an Actor has an outgoing join relation and than adds this to
	 * the list of actor with outgoing join relations.
	 * 
	 * @param actor
	 *            The actor which will be added if there are outgoing
	 *            {@link JoinRelation}.
	 */
	public void addActor(Actor actor) {
		if (!actors.contains(actor)) {
			for (Connection connection : actor.getSourceConnections()) {
				if (connection instanceof JoinRelation) {
					this.actors.add(actor);
					break;
				}
			}
		}
	}

	/**
	 * Returns a list of actors which have to be assimilated next.
	 * 
	 * @return List of Actors.
	 */
	public List<Actor> getActorsToAssimilate() {
		ArrayList<Actor> actorsToAssimilate = new ArrayList<Actor>();

		ArrayList<Actor> actorsToCheck = new ArrayList<Actor>(actors);
		actorsToCheck.removeAll(assimilatedActors);

		for (Actor actor : actorsToCheck) {
			boolean incoming = false;
			List<Connection> connections = actor.getTargetConnections();
			for (Connection connection : connections) {
				if (connection instanceof JoinRelation) {
					incoming = true;
					break;
				}
			}

			if (!incoming) {
				actorsToAssimilate.add(actor);
			}
		}

		this.assimilatedActors.addAll(actorsToAssimilate);

		return actorsToAssimilate;
	}

	/**
	 * Checks, if there are still actors which are not woven.
	 * 
	 * @return True, if there are actors to assimilate.
	 */
	public boolean hasMoreActorsToAssimilate() {
		return actors.size() - assimilatedActors.size() > 0;
	}

	/**
	 * Analyzes the model for actors with outgoing joinrelations.
	 * 
	 * @param AdoraModel
	 *            The {@link AdoraModel} which will be analyzed.
	 */
	private void analyze(AdoraModel model) {
		actors.clear();

		for (ModelElement element : model.getChildren()) {
			if (element instanceof Actor) {
				addActor((Actor) element);
			}
		}
	}

	/**
	 * Starts the weaving.
	 * 
	 * @param AdoraModel
	 *            The {@link AdoraModel} which will be analyzed.
	 */
	public ActorAssimilationAnalyzer(AdoraModel model) {
		analyze(model);
	}

	/**
	 * @return the actors
	 */
	public ArrayList<Actor> getActors() {
		return actors;
	}
}
