package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;

/**
 * This class is a helper class which keeps the information related to the
 * associations between scenario nodes and actors. After weaving the nodes into
 * their new locations, this information would get lost because of the cloning.
 * This class saves a list of {@link ActorScenarioAssociationContainer}'s so
 * that the actor can be successfully reconnected at the end of the weaving
 * process.
 * 
 * @author Marc Vontobel
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 10.10.2007 MV Basic functionality implemented (beta)<br>
 *          23.10.2007 MV Replacement of cloned nodes<br>
 *          30.10.2007 MV Actor assimilation tested
 * @version $Date: 2008-01-21 13:11:17 +0100 (Mon, 21 Jan 2008) $, $Author: schoeni $, $Revision: 3456 $
 * 
 */
public class ActorScenarioAssociationList {
	/**
	 * The list of the {@link ActorScenarioAssociationContainer}'s.
	 */
	private ArrayList<ActorScenarioAssociationContainer> containers = new ArrayList<ActorScenarioAssociationContainer>();

	/**
	 * The constructor, which creates an
	 * {@link ActorScenarioAssociationContainer}.
	 * 
	 * @param groupCloner
	 */
	public ActorScenarioAssociationList(GroupCloner groupCloner) {
		for (int i = 0; i < groupCloner.getNodeGroup().getNodes().size(); i++) {
			Node node = groupCloner.getNodeGroup().getNodes().get(i);
			Node clonedNode = groupCloner.getClonedNodes().get(i);

			processNode(node, clonedNode);
		}
	}

	/**
	 * This method processes a node. For each Node (which is a scenario), all
	 * connections are processed (by
	 * {@see #processConnection(Node, Node, Connection)}.
	 * 
	 * @param node
	 *            The original node.
	 * @param clonedNode
	 *            The cloned node.
	 */
	private void processNode(Node node, Node clonedNode) {
		List<Connection> connections = node.getConnections();

		for (Connection connection : connections) {
			processConnection(node, clonedNode, connection);
		}
	}

	/**
	 * The connection will be analyzed if the target equals the node and the
	 * source is instanceof Actor or vice-versa. Then there is a new entry in
	 * the containers arraylist.
	 * 
	 * @param node
	 *            The original node.
	 * @param clonedNode
	 *            The cloned node.
	 * @param connection
	 *            The connection.
	 */
	private void processConnection(Node node, Node clonedNode, Connection connection) {
		if (connection.getTarget().equals(node) && connection.getSource() instanceof Actor) {
			containers.add(new ActorScenarioAssociationContainer((Scenario) clonedNode,
					(Association) connection, (Actor) connection.getSource()));
		} else if (connection.getSource().equals(node) && connection.getTarget() instanceof Actor) {
			containers.add(new ActorScenarioAssociationContainer((Scenario) clonedNode,
					(Association) connection, (Actor) connection.getTarget()));
		}
	}

	/**
	 * Checks, if there are {@link ActorScenarioAssociationContainer} to
	 * process.
	 * 
	 * @return boolean There are elements to process.
	 */
	public boolean isEmpty() {
		return this.containers.isEmpty();
	}

	/**
	 * Returns the list of {@link ActorScenarioAssociationContainer}'s.
	 * 
	 * @return A list of the containers.
	 */
	public ArrayList<ActorScenarioAssociationContainer> getContainers() {
		return containers;
	}

}
