package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.util.NodeGroup;
import ch.unizh.ifi.adora.model.util.NodeGroups;

/**
 * Analyzes an aspect and looks for all the contained groups. Out of these
 * groups, it will build group analyzers to handle each group individually.
 * 
 * @author Marc Vontobel
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 29.06.2007 MV First Version<br>
 *          02.07.2007 SM Comments reformatted and completed.<br>
 *          10.12.2007 PS Comments improved.
 * @version $Date: 2009-04-27 16:26:40 +0200 (Mon, 27 Apr 2009) $, $Author: anil $, $Revision: 4118 $
 * 
 */
public class AspectAnalyzer {
	/**
	 * The aspect container which contains all the {@link Scenario}s and
	 * {@link State}'s with its corresponding relations (as it is not an
	 * analyzer).
	 */
	private AspectContainer aspectContainer;

	/**
	 * List of {@link WeavingTargetAnalyzer}.
	 */
	private ArrayList<WeavingTargetAnalyzer> weavingTargetAnalyzers = new ArrayList<WeavingTargetAnalyzer>();

	/**
	 * Collections of all the nodes in an aspect group (used to create
	 * {@link WeavingTargetAnalyzer}'s).
	 */
	private NodeGroups nodeGroups;

	/**
	 * Starts the analyzation of an aspect.
	 * 
	 * @param aspectContainer
	 *            The {@link AspectContainer} which will be analyzed.
	 */
	public AspectAnalyzer(AspectContainer aspectContainer) {
		this.nodeGroups = aspectContainer.getNodeGroups();
		this.aspectContainer = aspectContainer;

		this.generateWeavingTargetAnalyzers();
	}

	/**
	 * Traverses all the groups of the aspect container and creates
	 * {@link WeavingTargetAnalyzer}'s for it.
	 */
	private void generateWeavingTargetAnalyzers() {
		if (nodeGroups.getChunkGroups().size() == nodeGroups.getFinalChunkGroups().size()) {
			// Create a group analyzer for each found group of nodes.
			for (NodeGroup nodeGroup : nodeGroups.getFinalChunkGroups()) {
				weavingTargetAnalyzers.add(new WeavingTargetAnalyzer(this.aspectContainer,
						nodeGroup));
			}

			// Generate the CrosscuttingNodeGroupList for this container.
			CrosscuttingNodeGroupList list = CrosscuttingNodeGroupList.getInstance();
			for (NodeGroup nodeGroup : nodeGroups.getFinalCrosscuttingGroups()) {
				list.addCrosscuttingNodeGroup(new CrosscuttingNodeGroup(nodeGroup,
						this.aspectContainer));
			}
		}
	}

	/**
	 * Returns the analyzed aspect container.
	 * 
	 * @return AspectContainer The analyzed {@link AspectContainer}.
	 */
	public AspectContainer getAspectContainer() {
		return aspectContainer;
	}

	/**
	 * Returns all {@link WeavingTargetAnalyzer}.
	 * 
	 * @return ArrayList<WeavingTargetAnalyzer> All processed
	 *         {@link WeavingTargetAnalyzer}.
	 */
	public ArrayList<WeavingTargetAnalyzer> getWeavingTargetAnalyzers() {
		return weavingTargetAnalyzers;
	}

	/**
	 * Sets the aspect container to analyze.
	 * 
	 * @param aspectContainer
	 *            The aspect container to analyze.
	 */
	public void setAspectContainer(AspectContainer aspectContainer) {
		this.aspectContainer = aspectContainer;
	}
}
