package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;

/**
 * The first analyzer in a chain of analyzers (ModelAnalyzer -> AspectAnalyzer
 * -> {@link WeavingTargetAnalyzer}) which analyzes an AdoraModel and picks out
 * each Aspect contained in it. These aspect containers are all sent to an
 * AspectAnalyzer for further processing.
 * 
 * @author Marc Vontobel
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 29.06.2007 IV First Version.<br>
 *          18.09.2007 SM Comments reformatted and completed.<br>
 *          27.11.2007 MV Comments added.<br>
 *          17.12.2007 PS Review changes.
 * @version $Date: 2010-01-28 04:43:50 +0100 (Thu, 28 Jan 2010) $, $Author: vigan $, $Revision: 5099 $
 * 
 */
public class ModelAnalyzer {
	/**
	 * List of AspectAnalyzers.
	 */
	private ArrayList<AspectAnalyzer> aspectAnalyzers = new ArrayList<AspectAnalyzer>();

	/**
	 * Creates a ModelAnalyzer.
	 * 
	 * @param model
	 *            The AdoraModel which has to get weaved.
	 */
	public ModelAnalyzer(AdoraModel model, boolean onlyCrosscuttingConcerns)
	{
		// Reset the list of WeavingTargets.
		WeavingTargetList.reset();

		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();

		while (iterator.hasNext())
		{
			ModelElement modelElement = iterator.next();
			
			if (modelElement instanceof AspectContainer)
			{
				if (!onlyCrosscuttingConcerns)
				{
					this.aspectAnalyzers.add(new AspectAnalyzer((AspectContainer) modelElement));
				}
				else
				{
					if (!((AspectContainer) modelElement).isVariability())
					{
						this.aspectAnalyzers.add(new AspectAnalyzer((AspectContainer) modelElement));
					}
				}
			}
		}
	}
	
	/**
	 * Creates a ModelAnalyzer for a list of aspectcontainers aspect Container. This is used for
	 * dynamic weaving.
	 * 
	 * @param model
	 *            The AdoraModel which has to get weaved.
	 */
	public ModelAnalyzer(AdoraModel model, List<AspectContainer> aspectList) {
		// Reset the list of WeavingTargets.
		WeavingTargetList.reset();

		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();

		
		//remove duplicates
		//this is a bugfix for the case when a decision has more than one joinrelation
		
		for (int i = 0; i < aspectList.size()-1; i++) 
			for (int j = i+1; j < aspectList.size(); j++) {
				
				if (aspectList.get(i) == aspectList.get(j))
					aspectList.remove(j);
				
			}
		
		
		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();
			if (modelElement instanceof AspectContainer) {
				
				AspectContainer aspInSearch = (AspectContainer) modelElement;
				for(AspectContainer asp:aspectList){
					if(asp.equals(aspInSearch)){
						this.aspectAnalyzers.add(new AspectAnalyzer(
								(AspectContainer) modelElement));
					}
				}
			}
		}
	}


	/**
	 * Returns all the {@link AspectAnalyzer}.
	 * 
	 * @return A list of {@link AspectAnalyzer}'s.
	 */
	public ArrayList<AspectAnalyzer> getAspectAnalyzers() {
		return aspectAnalyzers;
	}
}
