package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.NodeGroup;

/**
 * This analyzer searchs for scenario root nodes, which have multiple incoming
 * instead {@link JoinRelation}'s.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 28.02.2008 PS First Version.<br>
 * @version $Date: 2009-07-01 12:52:32 +0200 (Wed, 01 Jul 2009) $, $Author: jehle $, $Revision: 4414 $
 * 
 */
public class ScenarioInsteadAnalyzer {
	/**
	 * List of root scenarios, which have multiple incoming instead
	 * {@link JoinRelation}'s.
	 */
	private List<Scenario> scenariosToHandle = new ArrayList<Scenario>();

	/**
	 * Creates this analyzer.
	 * 
	 * @param model
	 */
	public ScenarioInsteadAnalyzer(AdoraModel model, boolean onlyCrosscuttingConcerns)
	{
		analyzeForRootScenarioInsteads(model, onlyCrosscuttingConcerns);
	}

	/**
	 * Searchs for {@link Component}'s and {@link AspectContainer}'s which
	 * could have {@link Scenario}'s inside. These are dispatched to
	 * analyzeForRootScenarioInsteads(ZoomableNodeContainer).
	 * 
	 * @param model
	 */
	private void analyzeForRootScenarioInsteads(AdoraModel model, boolean onlyCrosscuttingConcerns)
	{
		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();
		
		while (iterator.hasNext())
		{
			ModelElement modelElement = iterator.next();

			if (modelElement instanceof Component
					|| modelElement instanceof AspectContainer)
			{
				analyzeForRootScenarioInsteads((ZoomableNodeContainer) modelElement, onlyCrosscuttingConcerns);
			}
		}
	}

	/**
	 * Analyzes a {@link ZoomableNodeContainer} for root {@link Scenario} nodes.
	 * If there are any which have more than one incoming {@link JoinRelation}'s,
	 * there are added to the list of scenarios.
	 * 
	 * @param modelElement
	 */
	private void analyzeForRootScenarioInsteads(ZoomableNodeContainer modelElement, boolean onlyCrosscuttingConcerns)
	{
		for (NodeGroup nodeGroup : modelElement.getNodeGroups().getScenarioGroups())
		{
			if (!nodeGroup.isScenarioGroup())
			{
				continue;
			}

			List<JoinRelation> joinRelations = getInsteadJoinRelations(getRootScenario(nodeGroup), onlyCrosscuttingConcerns);

			if (joinRelations.size() > 1) {
				// handle this!
				scenariosToHandle.add(getRootScenario(nodeGroup));
			}
		}
	}

	/**
	 * Returns a list of {@link Scenario}'s which have to be adapted.
	 * 
	 * @return a list of {@link Scenario}'s.
	 */
	public List<Scenario> getScenariosToHandle() {
		return scenariosToHandle;
	}

	/**
	 * Returns the root {@link Scenario} of a {@link NodeGroup}.
	 * 
	 * @param nodeGroup
	 *            The {@link NodeGroup} to parse.
	 * @return The root {@link Scenario} or null, if there is not any.
	 */
	private Scenario getRootScenario(NodeGroup nodeGroup)
	{
		for (Node node : nodeGroup.getNodes())
		{
			if (isRootScenario((Scenario) node))
			{
				return (Scenario) node;
			}
		}
		
		return null;
	}

	/**
	 * Returns a list of {@link JoinRelations}'s which have the weaving type
	 * instead.
	 * 
	 * @param scenario
	 *            The {@link Scenario} from which the {@link JoinRelation}'s
	 *            are extracted.
	 * @return A list of {@link JoinRelation}'s.
	 */
	private List<JoinRelation> getInsteadJoinRelations(Scenario scenario, boolean onlyCrosscuttingConcerns)
	{
		List<JoinRelation> joinRelations = new ArrayList<JoinRelation>();

		if (scenario != null)
		{
			for (Connection con : scenario.getTargetConnections())
			{
				if (con instanceof JoinRelation && ((JoinRelation) con).getWeavingType() == JoinRelation.INSTEAD)
				{
					if (!onlyCrosscuttingConcerns)
					{
						joinRelations.add((JoinRelation) con);		
					}
					else
					{
						if (((JoinRelation) con).getSource() instanceof AspectContainer)
						{
							if (((AspectContainer) ((JoinRelation) con).getSource()).isVariability())
							{
								joinRelations.add((JoinRelation) con);
							}
						}
					}
				}
			}
		}
		
		return joinRelations;
	}

	/**
	 * Checks, if a {@link Scenario} is a root scenario node.
	 * 
	 * @param scenario
	 *            The {@link Scenario} to check.
	 * @return true if the {@link Scenario} is the root node.
	 */
	private boolean isRootScenario(Scenario scenario)
	{
		for (Connection con : scenario.getSourceConnections())
		{
			if (con instanceof ScenarioConnection)
			{
				return false;
			}
		}

		return true;
	}
}
