package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;
import java.util.Iterator;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;

/**
 * Analyzer which analyzes the model for server components.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.11.2007 PS Comments added 06.11.2007 PS First Version
 * @version $Date: 2008-01-21 10:43:52 +0100 (Mon, 21 Jan 2008) $, $Author: schoeni $, $Revision: 3454 $
 */
public class ServerComponentAnalyzer {
	/**
	 * The {@link AdoraModel} which is analyzed.
	 */
	private AdoraModel model;

	/**
	 * The server components found.
	 */
	private ArrayList<Component> serverComponents = new ArrayList<Component>();

	/**
	 * Creates a new server component analyzer and analyzes the model.
	 * 
	 * @param model
	 *            The {@link AdoraModel} to analyze.
	 */
	public ServerComponentAnalyzer(AdoraModel model) {
		this.model = model;

		traverseModel();
	}

	/**
	 * Searches the model for server components.
	 */
	private void traverseModel() {
		for (Component component : getComponents(model)) {
			for (Connection connection : component.getConnections()) {
				if (connection instanceof Association
						&& isServerComponentAssociation((Association) connection)) {
					serverComponents.add(component);
					// make sure, that a component is only once in the list
					break;
				}
			}
		}
	}

	/**
	 * Returns all the server components found (in the model).
	 * 
	 * @return A list of all components.
	 */
	public ArrayList<Component> getServerComponents() {
		return serverComponents;
	}

	/**
	 * Checks if this connection is a server component association. A server
	 * connection is an association which has an {@link AspectContainer} as
	 * source and a component as target (or vice-versa).
	 * 
	 * @param association
	 *            The association to check.
	 * @return True, if it is a server component association.
	 */
	public static boolean isServerComponentAssociation(Association association) {
		boolean isServerConnection = false;

		if ((association.getSource() instanceof Component && association.getTarget() instanceof AspectContainer)
				|| (association.getSource() instanceof AspectContainer && association.getTarget() instanceof Component)) {
			isServerConnection = true;
		}

		return isServerConnection;
	}

	/**
	 * Returns all the children of an {@link INodeContainer}, which are of
	 * class {@link Component}.
	 * 
	 * @param iNodeContainer
	 *            The {@link INodeContainer} which will be analyzed.
	 * @return The childrens of the {@link INodeContainer}.
	 */
	private ArrayList<Component> getComponents(INodeContainer iNodeContainer) {
		ArrayList<Component> components = new ArrayList<Component>();

		Iterator<ModelElement> iterator = iNodeContainer.getRecursiveChildrenIterator();
		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();

			if (modelElement instanceof Component) {
				components.add((Component) modelElement);
			}
		}

		return components;
	}
}
