package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;
import java.util.HashMap;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;

/**
 * This class is needed to store the path of join relationships traversed during
 * a transitional weaving. For example, suppose that two aspects A1 and A2 are
 * given, each containing a behavior chunk. Suppose that the behavior chunk in
 * A1 crosscuts the behavior chunk in A2 which crosscuts a transition in a
 * component C1. Thus, a transitive weaving is given.<br>
 * This class is implemented as a singleton. You have to reset() the instance if
 * you start a new weaving process.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 28.11.2007 PS First Version.<br>
 *          10.12.2007 PS Comments improved.
 * @version $Date: 2009-05-24 17:34:46 +0200 (Sun, 24 May 2009) $, $Author: anil $, $Revision: 4284 $
 */
public class WeavingHistory {
	/**
	 * The only instance of this class.
	 */
	private static WeavingHistory INSTANCE = null;

	/**
	 * Returns the singleton instance of this class.
	 * 
	 * @return WeavingHistory The active instance.
	 */
	public static WeavingHistory getInstance() {
		if (INSTANCE == null) {
			reset();
		}

		return INSTANCE;
	}

	/**
	 * Resets the active instance.
	 */
	public static void reset() {
		INSTANCE = new WeavingHistory();
	}

	/**
	 * This hashmap saves each target module crosscut by an aspect. For example
	 * if aspect A1 crosscuts C1 and C2, then the hashmap contains an entry for
	 * A1 containing an ArrayList instance with C1 and C2.
	 */
	private HashMap<ModelElement, ArrayList<IContainer>> fromTo = new HashMap<ModelElement, ArrayList<IContainer>>();

	/**
	 * Adds an entry for a weaving process.
	 * 
	 * @param from
	 *            ModelElement The source model element, from where the chunk is
	 *            cloned.
	 * @param to
	 *            IContainer The target model element, where the chunk is woven
	 *            into.
	 */
	public void addWeavedTo(ModelElement from, IContainer to) {
		if (!fromTo.containsKey(from)) {
			fromTo.put(from, new ArrayList<IContainer>());
		}

		if (!fromTo.get(from).contains(to)) {
			fromTo.get(from).add(to);
		}
	}

	/**
	 * Returns the list of all direct targets for a given
	 * {@link AspectContainer}.
	 * 
	 * @param from
	 *            ModelElement the source of the weaving.
	 * @return ArrayList<IContainer> list of all the targets.
	 */
	public ArrayList<IContainer> getWeavingTargets(ModelElement from) {
		return fromTo.get(from);
	}

	/**
	 * Returns the whole list of all weaving entries.
	 * 
	 * @return List of all entries.
	 */
	public HashMap<ModelElement, ArrayList<IContainer>> getWeavingTargets() {
		return fromTo;
	}

	/**
	 * This method returns a list containing all targets, ie. all directly and
	 * indirectly (transitively) crosscut modules of a given
	 * {@link AspectContainer}.
	 * 
	 * @param ModelElement
	 *            the source model element, from where the chunk is cloned.
	 * @return List of all entries.
	 */
	public ArrayList<Component> getTransitiveWeavingTargets(ModelElement from) {
		// Stores all crosscut modules.
		ArrayList<Component> targets = new ArrayList<Component>();
		
		// If there are no targets, just return the empty list
		if (getWeavingTargets(from) == null) {
			return targets;
		}

		// All the already visited nodes are stored in this array.
		ArrayList<IContainer> visited = new ArrayList<IContainer>();

		// Queue with the next nodes to visit (set up with the direct targets).
		ArrayList<IContainer> queue = new ArrayList<IContainer>(getWeavingTargets(from));

		while (!queue.isEmpty()) {
			IContainer container = queue.remove(0);

			if (!visited.contains(container)) {
				visited.add(container);

				if (container instanceof Component) {
					// Only add the {@link Component}'s to the target, not the
					// {@link AspectContainer}'s.
					targets.add((Component) container);
				}

				// Add the direct targets of this node to the list of nodes
				// which will be visited.
				ArrayList<IContainer> nextContainers = getWeavingTargets((ModelElement) container);
				if (nextContainers != null) {
					queue.addAll(nextContainers);
				}
			}
		}

		return targets;
	}
	public ArrayList<AspectContainer> getTransitiveWeavingTargetsAspect(ModelElement from) {
		// Stores all crosscut modules.
		ArrayList<AspectContainer> targets = new ArrayList<AspectContainer>();
		
		// If there are no targets, just return the empty list
		if (getWeavingTargets(from) == null) {
			return targets;
		}

		// All the already visited nodes are stored in this array.
		ArrayList<IContainer> visited = new ArrayList<IContainer>();

		// Queue with the next nodes to visit (set up with the direct targets).
		ArrayList<IContainer> queue = new ArrayList<IContainer>(getWeavingTargets(from));

		while (!queue.isEmpty()) {
			IContainer container = queue.remove(0);

			if (!visited.contains(container)) {
				visited.add(container);

				if (container instanceof AspectContainer) {
					// Only add the {@link Component}'s to the target, not the
					// {@link AspectContainer}'s.
					targets.add((AspectContainer) container);
				}

				// Add the direct targets of this node to the list of nodes
				// which will be visited.
				ArrayList<IContainer> nextContainers = getWeavingTargets((ModelElement) container);
				if (nextContainers != null) {
					queue.addAll(nextContainers);
				}
			}
		}

		return targets;
	}
}
