package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.Comparator;
import java.util.TreeSet;

import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.weaver.engine.generator.AbstractCommandGenerator;

/**
 * This class describes a weaving target of a join relationship. A weaving
 * target is a crosscut either by one or more behaviour chunks or by one ore
 * more scenario chunks. The chunks are categorized according to their weaving
 * order, i.e. before, after or instead the targets action.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 20.11.2007 MV Renamed from WeaveComponent to WeavingTarget<br>
 *          27.11.2007 MV Comments added<br>
 *          27.11.2007 PS GroupCloneComparator
 * @version $Date: 2008-01-22 10:18:05 +0100 (Tue, 22 Jan 2008) $, $Author: schoeni $, $Revision: 3465 $
 */
public abstract class WeavingTarget {
	/**
	 * {@link ChunkCloner}'s which have to be placed BEFORE the target node
	 * (default).
	 */
	private TreeSet<ChunkCloner> before = new TreeSet<ChunkCloner>(getComparator());

	/**
	 * {@link ChunkCloner}'s which have to relace the target node.
	 */
	private TreeSet<ChunkCloner> instead = new TreeSet<ChunkCloner>(getComparator());

	/**
	 * {@link ChunkCloner}'s which have to be placed AFTER the target node.
	 */
	private TreeSet<ChunkCloner> after = new TreeSet<ChunkCloner>(getComparator());

	abstract protected Comparator<ChunkCloner> getComparator();
	
	/**
	 * The parent node of the target destination.
	 */
	private INodeContainer parent;

	/**
	 * Returns a TreeSet of {@link ChunkCloner}'s which share the same weaving
	 * order: AFTER.
	 * 
	 * @return The {@link ChunkCloner}'s which have to be placed AFTER the
	 *         target node.
	 */
	public TreeSet<ChunkCloner> getAfter() {
		return after;
	}

	/**
	 * Returns a TreeSet of {@link ChunkCloner}'s which share the same weaving
	 * order: BEFORE.
	 * 
	 * @return The {@link ChunkCloner}'s which have to be placed BEFORE the
	 *         target node.
	 */
	public TreeSet<ChunkCloner> getBefore() {
		return before;
	}

	/**
	 * Returns a TreeSet of {@link ChunkCloner}'s which share the same weaving
	 * order: INSTEAD.
	 * 
	 * @return The {@link ChunkCloner}'s which have REPLACE the target node.
	 */
	public TreeSet<ChunkCloner> getInstead() {
		return instead;
	}

	/**
	 * Adds a groupclone to the "before" list.
	 * 
	 * @param chunkCloner
	 *            A {@link ChunkCloner} which has to be placed before it's target
	 *            node.
	 * @return boolean True if successfully added, false if not.
	 */
	public boolean addBefore(ChunkCloner chunkCloner) {
		return this.before.add(chunkCloner);
	}

	/**
	 * Adds a groupclone to the "instead" list.
	 * 
	 * @param chunkCloner
	 *            A {@link ChunkCloner} which has to be placed instead it's
	 *            target node.
	 * @return boolean True if successfully added, false if not.
	 */
	public boolean addInstead(ChunkCloner chunkCloner) {
		return this.instead.add(chunkCloner);
	}

	/**
	 * Adds a groupclone to the "after" list.
	 * 
	 * @param chunkCloner
	 *            A {@link ChunkCloner} which has to be placed after it's target
	 *            node.
	 * @return boolean True if successfully added, false if not.
	 */
	public boolean addAfter(ChunkCloner chunkCloner) {
		return this.after.add(chunkCloner);
	}

	/**
	 * Returns the INodeContainer which contains the target node of the join
	 * relation. i.e. There is a scenario node s1 in an aspect container a1
	 * which is connected by a join relation to a scenario s2 which is inside of
	 * a container c1. This method returns c1.
	 * 
	 * @return The parent Container of the target node.
	 */
	public INodeContainer getParent() {
		return parent;
	}

	/**
	 * Sets the INodeContainer which contains the target node of the join
	 * relation. i.e. There is a scenario node s1 in an aspect container a1
	 * which is connected by a join relation to a scenario s2 which is inside of
	 * a container c1. This method defines c1.
	 * 
	 * @param The
	 *            parent Container of the target node.
	 */
	public void setParent(INodeContainer parent) {
		this.parent = parent;
	}
	
	public abstract AbstractCommandGenerator getCommandGenerator();
}
