package ch.unizh.ifi.adora.weaver.engine.analyzer;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.util.NodeGroup;

/**
 * The class prepares the targets of the weaving process with the clones of the
 * behavior chunks and scenario chunks.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 29.06.2007 IV First Version<br>
 *          19.09.2007 SM Comments reformatted and completed.<br>
 *          17.12.2007 PS Review changes (GroupAnalyzer =>
 *          WeavingTargetAnalyzer)
 * @version $Date: 2010-01-28 04:43:50 +0100 (Thu, 28 Jan 2010) $, $Author: vigan $, $Revision: 5099 $
 * 
 */
public class WeavingTargetAnalyzer {
	/**
	 * The parent analyzer of this analyzer.
	 */
	private AspectContainer aspectContainer;

	/**
	 * The group of nodes within the aspect.
	 */
	private NodeGroup nodeGroup;

	/**
	 * Creates a group analyzer.
	 * 
	 * @param aspectContainer
	 *            The {@link AspectContainer} used.
	 * @param nodeGroup
	 *            The {@link NodeGroup} used.
	 */
	public WeavingTargetAnalyzer(AspectContainer aspectContainer,
			NodeGroup nodeGroup) {
		this.aspectContainer = aspectContainer;
		this.nodeGroup = nodeGroup;

		this.computeWeavingTargets();
	}

	/**
	 * Computes (generates) all the weaving target objects.
	 */
	private void computeWeavingTargets() {
		if (nodeGroup.isStateGroup()) {
			/**
			 * Check all nodes for outgoing {@link JoinRelation}'s.
			 * */
			for (Node node : nodeGroup.getNodes()) {
				this.handleState(node);
			}
		} else if (nodeGroup.isScenarioGroup()) {
			/**
			 * Check root node for outgoing {@link JoinRelation}'s.
			 * 
			 */
			for (Node node : nodeGroup.getNodes()) {
				this.handleScenario((Scenario) node);
			}
		} else if (nodeGroup.isComponentGroup()) {
			for (Node node : nodeGroup.getNodes()) {
				this.handleComponent((Component) node);
			}
		}
	}

	/**
	 * Returns the corresponding {@link NodeGroup}.
	 * 
	 * @return The corresponding {@link NodeGroup}.
	 */
	public NodeGroup getNodeGroup() {
		return nodeGroup;
	}

	/**
	 * Handles a scenario: if there is an outgoing {@link JoinRelation} to
	 * another scenario this method calls the {@see
	 * #prepareScenarioWeavingTarget(JoinRelation)} method.
	 */
	private void handleScenario(Scenario scenario) {
		for (Connection connection : scenario.getSourceConnections()) {
			// Test, if this connection goes to another scenario
			
			if ((connection instanceof JoinRelation) &&
				(((JoinRelation)connection).getDecision() != null) &&
				(((JoinRelation)connection).getDecision()).getDecisionValue() != null &&
				(((JoinRelation)connection).getDecision()).getDecisionValue()) {

					this.prepareScenarioWeavingTarget((JoinRelation) connection);
	
			}			
		}
	}

	/**
	 * Handles a state: if there is an outgoing {@link JoinRelation} from a
	 * state to a transition this method calls the {@see
	 * #prepareStateWeavingTarget(JoinRelation)} method.
	 */
	private void handleState(ConnectableElement state) {
		for (Connection connection : state.getSourceConnections()) {
			// Test, if this connection goes to a transition
			if (connection instanceof JoinRelation
					&& (connection.getTarget() instanceof Transition)) {
				// state, which is not on the same level: connect
				this.prepareStateWeavingTarget((JoinRelation) connection);
			}
		}
	}

	private void handleComponent(Component component) {
		for (Connection connection : component.getSourceConnections()) {
			// Test, if this connection goes to a transition
			if (connection instanceof JoinRelation
					&& ((connection.getTarget() instanceof Component))) {
				// state, which is not on the same level: connect
				this.prepareComponentWeavingTarget((JoinRelation) connection);
			}
			if(connection instanceof JoinRelation
					&&((connection.getTarget() instanceof AspectContainer))){
				this.prepareAspectWeavingTarget((JoinRelation) connection);
			}
		}
	}

	private void prepareAspectWeavingTarget(JoinRelation joinRelation) {
		AspectContainer target = (AspectContainer) joinRelation.getTarget();
		AspectContainerWeavingTarget weavingTarget = (AspectContainerWeavingTarget) WeavingTargetList
				.getInstance().getWeavingTarget(target);
		
		if (weavingTarget == null) {
			weavingTarget = new AspectContainerWeavingTarget(target);
			WeavingTargetList.getInstance().addWeavingTarget(target,
					weavingTarget);
		}

		/**
		 * Save the information, that this chunk is weaved from the
		 * {@link AspectContainer} to the parent of the weaving target. Then
		 * create the {@link ChunkCloner} object and add this group to the
		 * corresponding treeset (before / instead / after).
		 */

		WeavingHistory.getInstance().addWeavedTo(aspectContainer,
				weavingTarget.getTargetNode());

		ChunkCloner chunkCloner = new ChunkCloner(nodeGroup,
				(JoinRelation) joinRelation);

		if (((JoinRelation) joinRelation).getWeavingType() == JoinRelation.AFTER) {
			weavingTarget.addAfter(chunkCloner);
		} else if (((JoinRelation) joinRelation).getWeavingType() == JoinRelation.INSTEAD) {
			weavingTarget.addInstead(chunkCloner);
		} else {
			weavingTarget.addBefore(chunkCloner);
		}
		
	}

	private void prepareComponentWeavingTarget(JoinRelation joinRelation) {
		Component target = (Component) joinRelation.getTarget();
		ComponentWeavingTarget weavingTarget = (ComponentWeavingTarget) WeavingTargetList
				.getInstance().getWeavingTarget(target);
		
		if (weavingTarget == null) {
			weavingTarget = new ComponentWeavingTarget(target);
			WeavingTargetList.getInstance().addWeavingTarget(target,
					weavingTarget);
		}

		/**
		 * Save the information, that this chunk is weaved from the
		 * {@link AspectContainer} to the parent of the weaving target. Then
		 * create the {@link ChunkCloner} object and add this group to the
		 * corresponding treeset (before / instead / after).
		 */

		WeavingHistory.getInstance().addWeavedTo(aspectContainer,
				weavingTarget.getParent());

		ChunkCloner chunkCloner = new ChunkCloner(nodeGroup,
				(JoinRelation) joinRelation);

		if (((JoinRelation) joinRelation).getWeavingType() == JoinRelation.AFTER) {
			weavingTarget.addAfter(chunkCloner);
		} else if (((JoinRelation) joinRelation).getWeavingType() == JoinRelation.INSTEAD) {
			weavingTarget.addInstead(chunkCloner);
		} else {
			weavingTarget.addBefore(chunkCloner);
		}

	}

	/**
	 * Create a scenario weaving target, which can be processed by the command
	 * generator.
	 * 
	 * @param joinRelation
	 *            The corresponding {@link JoinRelation}.
	 */
	private void prepareScenarioWeavingTarget(JoinRelation joinRelation) {
		/**
		 * Define the target {@link Scenario} and search the corresponding
		 * {@link ScenarioWeavingTarget}. If the weaving target is not found, it
		 * creates a new one for this target {@link Scenario}.
		 */
		Scenario target = (Scenario) joinRelation.getTarget();

		ScenarioWeavingTarget weavingTarget = (ScenarioWeavingTarget) WeavingTargetList
				.getInstance().getWeavingTarget(target);

		//check if we allready weaved this element
		Object o = WeavingTargetList.getInstance().getWeavingTarget(joinRelation.getSource());
		if (o instanceof Scenario && (Scenario)o == target)
			return;
		
		if (weavingTarget == null) {
			weavingTarget = new ScenarioWeavingTarget(target);
			WeavingTargetList.getInstance().addWeavingTarget(target,
					weavingTarget);
		}

		/**
		 * Save the information, that this chunk is weaved from the
		 * {@link AspectContainer} to the parent of the weaving target. Then
		 * create the {@link ChunkCloner} object and add this group to the
		 * corresponding treeset (before / instead / after).
		 */
		WeavingHistory.getInstance().addWeavedTo(aspectContainer,
				weavingTarget.getParent());

		ChunkCloner chunkCloner = new ChunkCloner(nodeGroup,
				(JoinRelation) joinRelation);

		if (((JoinRelation) joinRelation).getWeavingType() == JoinRelation.AFTER) {
			weavingTarget.addAfter(chunkCloner);
		} else if (((JoinRelation) joinRelation).getWeavingType() == JoinRelation.INSTEAD) {
			weavingTarget.addInstead(chunkCloner);
		} else {
			weavingTarget.addBefore(chunkCloner);
		}
	}

	/**
	 * Create a state weaving target, which can be processed by the command
	 * generator.
	 * 
	 * @param joinRelation
	 *            The corresponding {@link JoinRelation}.
	 */
	private void prepareStateWeavingTarget(JoinRelation joinRelation) {
		/**
		 * Define the target {@link Transition} and search the corresponding
		 * {@link StateWeavingTarget}. If the {@link StateWeavingTarget} is not
		 * found, it creates a new one for this {@link Transition}.
		 */
		Transition target = (Transition) joinRelation.getTarget();

		StateWeavingTarget weavingTarget = (StateWeavingTarget) WeavingTargetList
				.getInstance().getWeavingTarget(target);

		if (weavingTarget == null) {
			weavingTarget = new StateWeavingTarget(target);
			WeavingTargetList.getInstance().addWeavingTarget(target,
					weavingTarget);
		}

		/**
		 * Save the information, that this chunk is weaved from the
		 * {@link AspectContainer} to the parent of the weaving target. Then
		 * create the {@link ChunkCloner} object and add this group to the
		 * corresponding treeset (before / instead / after).
		 */
		WeavingHistory.getInstance().addWeavedTo(aspectContainer,
				weavingTarget.getParent());

		ChunkCloner chunkCloner = new ChunkCloner(nodeGroup,
				(JoinRelation) joinRelation);

		if (((JoinRelation) joinRelation).getWeavingType() == JoinRelation.AFTER) {
			weavingTarget.addAfter(chunkCloner);
		} else if (((JoinRelation) joinRelation).getWeavingType() == JoinRelation.INSTEAD) {
			weavingTarget.addInstead(chunkCloner);
		} else {
			weavingTarget.addBefore(chunkCloner);
		}
	}
}
