package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.model.representation.Connection;

/**
 * A command generator processes commands which modify a given model model, so
 * that the weaving operation is performed. This class provides the abstract
 * superclass for all concrete command generators of the weaving process.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.11.2007 PS First Version 10.12.2007 MV Commands improved
 * @version $Date: 2009-04-16 10:09:32 +0200 (Thu, 16 Apr 2009) $, $Author: anil $, $Revision: 4102 $
 */
public abstract class AbstractCommandGenerator {
	/**
	 * A Container for all the commands generated during the process of weaving.
	 * The list is splitted into two seperate containers (weavingCommands and
	 * deleteCommands) which only differ in the order of their execution.
	 */
	private CommandList commandList = new CommandList();

	/**
	 * Creates a command to delete any kind of connection. This method is first
	 * and foremost used to delete the join relation at the end of any weaving
	 * process.
	 * 
	 * @param groupClone
	 */
	protected void deleteConnection(Connection connection) {
		ConnectionDelete cd = new ConnectionDelete(connection, true);
		addDeleteCommand(cd);
	}

	/**
	 * Adds a delete command to the stack of delete commands. Any command which
	 * is added by this method will be executed after any other command added by
	 * the addWeaveCommand method.
	 * 
	 * @param command
	 *            Any command used for the weaving process.
	 */
	protected void addDeleteCommand(Command command) {
		commandList.addDeleteCommand(command);
	}

	/**
	 * Adds a weaving command to the stack of weave commands. Any command which
	 * is added by this method will be executed before any other command added
	 * by the addDeleteCommand method.
	 * 
	 * @param command
	 *            Any command used for the weaving process.
	 */
	protected void addWeaveCommand(Command command) {
		commandList.addWeaveCommand(command);
	}

	/**
	 * Adds a whole list of weaving commands to the stack of weave commands.
	 * 
	 * @param commands
	 *            The list of weaving commands.
	 */
	protected void addWeaveCommand(List<Command> commands) {
		for (Command command : commands) {
			addWeaveCommand(command);
		}
	}

	/**
	 * Returns a list of all commands which are used for the weaving. The order
	 * of the commands is roughly explained as all weavingCommands followed by
	 * all deleteCommands.
	 * 
	 * @return list of commands for the weaving, first the "weaving" commands,
	 *         then the delete commands.
	 */
	public List<Command> getWeaveCommands() {
		return commandList.getWeaveCommands();
	}
}
