package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.List;

import ch.unizh.ifi.adora.model.commands.AssociationReconnect;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * Generates the commands for the assimilation of actors. i.e., the weaving
 * of a crosscutting enviroment object with a crosscut enviroment object. 
 * THis generator removes join relationships between actors and reconnets the
 * associations of crosscutting actors to the crosscut actor.
 * 
 * @author 		Pascal Schoeni
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	27.11.2007 PS First Version
 * 				10.12.2007 MV Changes from last meeting implemented
 * @version 	$Date: 2007-12-17 11:26:15 +0100 (Mon, 17 Dec 2007) $, $Author: schoeni $, $Revision: 3417 $
 */
public class ActorAssimilationCommandGenerator extends AbstractCommandGenerator {

	/**
	 * Looking forward to get a full list of actors, the constructor of this
	 * class forwards all the actors recieved to it's own analyzeAndGenerate method 
	 * where they lie until the adora weaving god takes them for futher destination.
	 * 
	 * @param actors A list of actors of an adora model
	 */
	public ActorAssimilationCommandGenerator(List<Actor> actors) {
		for (Actor actor : actors) {
			findJoinRelation(actor);
		}
	}

	/**
	 * Analyzes the connections of an actor. If it finds any outgoing
	 * join relation, this connection will be analyzed in a further
	 * method called findAndReconnectAssociation.
	 */
	private void findJoinRelation(Actor actor) {
		for (Connection connection : actor.getSourceConnections()) {
			if (connection instanceof JoinRelation) {
				findAndReconnectAssociation((JoinRelation) connection);
			}
		}

	}

	/**
	 * Finds and than reconnects any association from/to the actors
	 * which are getting weaved. i.e. if an actor a1 is weaved into
	 * another actor a2 as a result of a join relation, any association
	 * to a1 will get reconnected to a2.
	 * 
	 * @param joinRelation A JoinRelation outgoing of an actor.
	 */
	private void findAndReconnectAssociation(JoinRelation joinRelation) {
		Actor source = (Actor) joinRelation.getSource();

		for (Connection connection : source.getConnections()) {
			/**
			 * Checks for associations connected to the a scenario  in a component c1.
			 */
			if (connection instanceof Association) {
				handleAssociation(joinRelation, (Association) connection);
			}
		}

		deleteConnection(joinRelation);
	}

	/**
	 * Handles an association. Assume, that there is an actor A1 which is
	 * connected to a scenario in a component C1. Furthermore, A1 crosscuts A2. 
	 * Actor A1 has an outgoing joinrelation to crosscuts (indicated by a join
	 * relation between A1 and A2). Now this method checks, if A2 also has a 
	 * connection to C1. if this is the case, then the association between A1 
	 * and C1 can be reconnected to A2 / C1.
	 * 
	 * @param joinRelation JoinRelation between A1 and A2
	 * @param association Association between A1 and C1
	 */
	private void handleAssociation(JoinRelation joinRelation, Association association) {
		INodeContainer parent = getParentContainerOfWeavingTarget(association, joinRelation.getSource());

		/**
		 * Check, if there is an association which has the same parent. then
		 * reconnect this association to the target of the joinrelation
		 */
		boolean reconnect = false;
		for (Connection connection : joinRelation.getTarget().getConnections()) {
			if (connection instanceof Association) {
				/**
				 * Check if the parent of the weaving target is the same as the target of the join relation
				 */
				if (parent.equals(getParentContainerOfWeavingTarget((Association) connection, joinRelation.getTarget()))) {
					reconnect = true;
					break;
				}
			}
		}

		if (reconnect) {
			AssociationReconnect ar = new AssociationReconnect(association);
			/**
			 * Because associations can be connected from each side (source and target do
			 * not matter), we have to distinquish the two possible cases.
			 */
			if (joinRelation.getSource().equals(association.getSource())) {
				ar.setNewSource(joinRelation.getTarget());
				GeneratorHelper.setDefaultSourceClickPoint(ar);
			} else {
				ar.setNewTarget(joinRelation.getTarget());
				GeneratorHelper.setDefaultTargetClickPoint(ar);
			}

			addWeaveCommand(ar);
		}
	}

	/**
	 * Returns the target of an association. As it is not known, if the
	 * target container is the source or the target of the association, this
	 * method returns the right one.
	 * 
	 * @param association The association between the source and the target
	 * @param source The source of the join relation
	 * @return INodeContainer The parent container of the weaving target
	 */
	private INodeContainer getParentContainerOfWeavingTarget(Association association, ConnectableElement source) {
		Node retVal = association.getTarget();

		if (retVal.equals(source)) {
			retVal = association.getSource();
		}

		return retVal.getParent();
	}

}
