package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.commands.InsertNodesVisually;
import ch.unizh.ifi.adora.model.commands.NodeCompact;
import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.weaver.engine.analyzer.AspectAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingTargetAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ModelAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingTarget;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingTargetList;

/**
 * Dispatches the weavables to the right command generator and collects all the
 * generated commands.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 	29.06.2007 PS First Version 
 * 				18.09.2007 PS Comments reformatted and
 *          completed.
 * @version $Date: 2010-01-28 04:43:50 +0100 (Thu, 28 Jan 2010) $, $Author: vigan $, $Revision: 5099 $
 * 
 */
public class CommandController {
	
	/**
	 * The main analyzer of the weaving process which stores all the weavables.
	 */
	private ModelAnalyzer modelAnalyzer;

	/**
	 * This controller needs the preaviously computed model analyzer
	 * to dispatch the weaving target to the right command generators.
	 * @param modelAnalyzer
	 */
	public CommandController(ModelAnalyzer modelAnalyzer) {
		this.modelAnalyzer = modelAnalyzer;
	}

	/**
	 * Get all commands for the weaving process
	 * 
	 * @return A list of commands in the right order to be executed (delete commands at the end).
	 */
	public List<Command> getCommands() {
		ArrayList<Command> commands = new ArrayList<Command>();

		/**
		 * Getting all the WeavingTargets.
		 */
		Enumeration<WeavingTarget> enumeration = WeavingTargetList.getInstance().getWeavables();

		/**
		 * Go through every WeavingTarget.
		 */
		while (enumeration.hasMoreElements()) {
			WeavingTarget weavingTarget = enumeration.nextElement();

			/**
			 * Create an individual CommandGenerator for each WeavingTarget.
			 */
			AbstractCommandGenerator cg = weavingTarget.getCommandGenerator();
			commands.addAll(cg.getWeaveCommands());
		}

		UnrelatedWeaveComponentCommandGenerator uwccg = new UnrelatedWeaveComponentCommandGenerator();
		commands.addAll(uwccg.getWeaveCommands());

		/**
		 * Delete all the involved group nodes.
		 */

		boolean canDelete = true;
		for (AspectAnalyzer aspectAnalyzer : this.modelAnalyzer.getAspectAnalyzers()) {
			for (WeavingTargetAnalyzer weavingTargetAnalyzer : aspectAnalyzer.getWeavingTargetAnalyzers()) {
				
				//only add delete command if nodegroup has no outgoing joinrelations with an undecided decision value
				for (Node node : weavingTargetAnalyzer.getNodeGroup().getNodes()) {
					
					if (hasNodeRecusivlyUndecidedDecisions(node)){
						canDelete = false;
						break;
					}
				}		
				
				if (canDelete) {
					for (Node node : weavingTargetAnalyzer.getNodeGroup().getNodes()) {
						commands.add(new NodeDelete(node.getParent(), node));
					}
				}
			}
		}
		
		//Add compact node command for all ZoomableNodeContainers which get new children inserted
		List<ZoomableNodeContainer> parents = new ArrayList<ZoomableNodeContainer>(); 
		for (Command c : commands) {
			if (c instanceof InsertNodesVisually) {
				ZoomableNodeContainer parent = ((InsertNodesVisually)c).getParent();
				if (!parents.contains(parent)) {
					parents.add(parent);
				}
			}
		}
		for (ZoomableNodeContainer parent : parents) {
			NodeCompact compactCmd = new NodeCompact();
			compactCmd.setTarget(parent);
			commands.add(compactCmd);
		}
		return commands;
	}
	
	/**
	 * This helper method is used to check whether or not a {@link ConnectableElement} can be deleted from the {@link AdoraModel}.
	 * A {@link ConnectableElement} can be deleted, if it, its parents and its children don't have  outgoing {@link Decision} which are undecided an
	 * 
	 * @param ele The {@link ConnectableElement} which gets tested. 
	 * @return true, if the {@link ConnectableElement} has an undecided outgoing {@link Decision}.
	 */
	private boolean hasNodeRecusivlyUndecidedDecisions(ConnectableElement ele){
		
		//check element
		for (Connection con : ele.getSourceConnections())
			if (con instanceof JoinRelation  && 
					((JoinRelation)con).getDecision() != null &&
					((JoinRelation)con).getDecision().getDecisionValue() == null) 
					
					return true;
		
		//check children
		if (ele instanceof ZoomableNodeContainer) {
			for (JoinRelation jr : ((ZoomableNodeContainer)ele).getAllSourceJRs()) {
				if (jr.getDecision() != null && 
						jr.getDecision().getDecisionValue() == null)
					
						return true;
			}
		}
		
		// if the current element is a connection then test if the source (or target)
		// is inside a container with outgoing undecided decisions.
		if (ele instanceof Connection)
			ele = ((Connection)ele).getSource();
			
		//check parents
		IContainer parent = ele.getParent();
		while (! (parent instanceof AdoraModel)) {
		
			for (Connection c : ((ZoomableNodeContainer)parent).getSourceConnections()) {
				if (c instanceof JoinRelation && 
						((JoinRelation)c).getDecision() != null &&
						((JoinRelation)c).getDecision().getDecisionValue() == null) {
							
						return true;
				}
			}			
			parent = parent.getParent();
		}

		return false;
	}
}