package ch.unizh.ifi.adora.weaver.engine.generator;

import ch.unizh.ifi.adora.model.commands.ConnectionCreate;
import ch.unizh.ifi.adora.model.commands.ConnectionReconnect;
import ch.unizh.ifi.adora.model.commands.TransitionReconnect;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;

/**
 * Helper class for the clickpoint settings.
 * 
 * @author 		Pascal Schoeni
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	29.06.2007 PS First Version.
 * 				18.09.2007 PS Comments reformatted and completed.
 * 				11.12.2007 MV Comments added.
 * @version $Date: 2007-12-11 14:04:17 +0100 (Tue, 11 Dec 2007) $, $Author: vontobel $, $Revision: 3398 $
 * 
 */
public class GeneratorHelper {
	/**
	 * Sets the default click points for uninserted connection.
	 * The midpoint of the source / target ConnectableElement is chosen.
	 * 
	 * @param 	createTransition A ConnectionCreate command.
	 * @pre		Command does have both source and target nodes.
	 */
	public static void setDefaultClickPoints(ConnectionCreate createConnection) {
		createConnection.setSourceClickPoint(calcDefaultClickPoint(createConnection.getSource()));
		createConnection.setTargetClickPoint(calcDefaultClickPoint(createConnection.getTarget()));
	}

	/**
	 * Sets the default click points for an implemented transition.
	 * The midpoint of the source / target ConnectableElement is chosen.
	 * 
	 * @param reconnectCommand The command for a transition reconnection.
	 */
	public static void setDefaultTargetClickPoint(TransitionReconnect reconnectCommand) {
		reconnectCommand.setTargetClickPoint(calcDefaultClickPoint(reconnectCommand.getNewTarget()));
	}

	/**
	 * Sets the default source click point for unimplemented connection.
	 * The midpoint of the source / target ConnectableElement is chosen.
	 * 
	 * @param reconnectCommand The command for a connection reconnection.
	 */
	public static void setDefaultSourceClickPoint(ConnectionReconnect reconnectCommand) {
		reconnectCommand
				.setSourceClickPoint(calcDefaultClickPoint(reconnectCommand.getNewSource()));
	}

	/**
	 * Sets the default target click point for unimplemented connection.
	 * The midpoint of the source / target ConnectableElement is chosen.
	 * 
	 * @param reconnectCommand The command for a connection reconnection.
	 */
	public static void setDefaultTargetClickPoint(ConnectionReconnect reconnectCommand) {
		reconnectCommand
				.setTargetClickPoint(calcDefaultClickPoint(reconnectCommand.getNewTarget()));
	}

	/**
	 * Resets the default source click point of a connection to the
	 * middlepoint of the sourcing ConnectableElement.
	 * 
	 * @param connection The connection which gets a new source click point.
	 */
	public static void setDefaultSourceClickPoint(Connection connection) {
		connection.setSourceClickPoint(calcDefaultClickPoint(connection.getSource()));
	}

	/**
	 * Resets the default target click point of a connection to the
	 * midpoint of the targeting ConnectableElement.
	 * 
	 * @param connection The connection which gets a new target click point.
	 */
	public static void setDefaultTargetClickPoint(Connection connection) {
		connection.setTargetClickPoint(calcDefaultClickPoint(connection.getTarget()));
	}

	/**
	 * Calculates the default clickpoint for a given ConnectableElement.
	 * 
	 * @param connectableElement The element.
	 * @return Point The clickpoint in the middle of the Element.
	 */
	private static Point calcDefaultClickPoint(ConnectableElement connectableElement) {
		Point p = new Point();

		p.x = (int) connectableElement.getBounds().getWidth() / 2;
		p.y = (int) connectableElement.getBounds().getHeight() / 2;

		return p;
	}
}
