package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.List;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * This class provides some main helping functionality to the weaving
 * process of node groups. It is mainly used to define a new area of the
 * target parent container.
 * 
 * @author 		Pascal Schoeni
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	29.06.2007 PS First Version.
 * 				18.09.2007 PS Comments reformatted and completed.
 * 				11.12.2007 MV Comments added.
 * @version 	$Date: 2008-01-21 10:46:25 +0100 (Mon, 21 Jan 2008) $, $Author: vontobel $, $Revision: 3455 $
 * 
 */
public class LayoutHelper {
	/**
	 * This is the minimum of the X value. Initially set to MAX_VALUE
	 * so that as soon as the first comparison takes place, it will be
	 * overwritten by the new value.
	 */
	int minX = Integer.MAX_VALUE;

	/**
	 * This is the minimum of the Y value. Initially set to MAX_VALUE
	 * so that as soon as the first comparison takes place, it will be
	 * overwritten by the new value.
	 */
	int minY = Integer.MAX_VALUE;

	/**
	 * The maximum value of X. Initially set to 0 so that it will be
	 * replaced by the new value as soon as the first comparis has taken place.
	 */
	int maxX = 0;

	/**
	 * The maximum value of Y. Initially set to 0 so that it will be
	 * replaced by the new value as soon as the first comparis has taken place.
	 */
	int maxY = 0;

	/**
	 * The constructur need a group of nodes for which it computes the
	 * coordinates.
	 * @param nodes List of Nodes
	 */
	public LayoutHelper(List<Node> nodes) {
		calculateSpaceUsed(nodes);
	}

	/**
	 * Calculates min and max x/y coords for a group of nodes. This
	 * means that it calculates the space used for the whole group.
	 * 
	 * @param nodes
	 */
	private void calculateSpaceUsed(List<Node> nodes) {
		for (Node node : nodes) {
			if (minX > node.getBounds().x) {
				minX = node.getBounds().x;
			}
			if (maxX < node.getBounds().x + node.getBounds().width) {
				maxX = node.getBounds().x + node.getBounds().width;
			}
			if (minY > node.getBounds().y) {
				minY = node.getBounds().y;
			}
			if (maxY < node.getBounds().y + node.getBounds().height) {
				maxY = node.getBounds().y + node.getBounds().height;
			}
		}
	}

	/**
	 * Returns the dimension of the bounding box.
	 * 
	 * @return Dimension Bounding Box
	 */
	public Dimension getDimension() {
		return new Dimension(maxX - minX, maxY - minY);
	}

	/**
	 * Adapts the location from the "point" to the new position, starting
	 * with "sourcePoint".
	 * 
	 * @param sourcePoint The source point
	 * @param point The point
	 * @return A new Point
	 */
	public Point adaptLocation(Point sourcePoint, Point point) {
		return new Point((int) sourcePoint.getX() + (int) point.getX() - minX, (int) sourcePoint
				.getY()
				+ (int) point.getY() - minY);
	}
}
