package ch.unizh.ifi.adora.weaver.engine.generator;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;

/**
 * This class implements a rename algorithm to ensure that scenarios, states
 * and components do not have the same names. After using the rename method
 * of this class, every of the mentioned visualizable model elements has it's unique name.
 * 
 * @author 		Marc Vontobel
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	29.06.2007 PS First Version.
 * 				18.09.2007 PS Comments reformatted and completed.
 * 				11.12.2007 MV Comments added.
 * @version 	$Date: 2008-03-20 15:20:20 +0100 (Thu, 20 Mar 2008) $, $Author: schoeni $, $Revision: 3578 $
 * 
 */
public class ModelElementRenamer {
	private Hashtable<String, ArrayList<VisualizableModelElement>> nameMap;
	
	/**
	 * This method creates a list of commands containing all the
	 * SetProperty Commands for renaming Scernarios/States/Components
	 * to unique names.
	 * 
	 * @pre       model != null
	 * @post      true
	 * @param model The model which has to be checked.
	 * @return List<Command> for execution.
	 */
	public List<Command> rename(AdoraModel model) {
		nameMap = new Hashtable<String, ArrayList<VisualizableModelElement>>();
		List<ModelElement> modelChildren = model.getChildren();
		for (ModelElement modelChild : modelChildren) {
			if (modelChild instanceof Component) {
				this.findRecursiveElements(modelChild);
			}
		}
		Enumeration<String> names = nameMap.keys();
		List<String> newNames = new ArrayList<String>();
		List<Command> commands = new ArrayList<Command>();
		while (names.hasMoreElements()) {
			String name = names.nextElement();
			if (nameMap.get(name).size() > 1) {
				//System.out.println("True");
				List<VisualizableModelElement> modelElements = nameMap.get(name);
				for (int i = 0, j = 1; i < modelElements.size(); i++, j++) {
					VisualizableModelElement modelElement = modelElements.get(i);

					String uniqueName = modelElement.getName() + "" + j;
					int k = j+1;
					while (nameMap.containsKey(uniqueName) || newNames.contains(uniqueName)) {
						uniqueName = modelElement.getName() + k++;
					}
					
					newNames.add(uniqueName);
					
					Method setMethod;
					Method getMethod;
					try {
						setMethod = modelElement.getClass().getMethod("setName", String.class);
						getMethod = modelElement.getClass().getMethod("getName", new Class [] {});
						SetProperty command = new SetProperty(modelElement, uniqueName, setMethod, getMethod);
						commands.add(command);
					} catch (Exception e) {
						e.printStackTrace();
					}
					
				}
			}
		}
		return commands;
	}
	
	/**
	 * Finds any child of a model element which is eiter a scenario,
	 * a state or a component. If it is a component, there will be a recursive
	 * search within its children.
	 * @pre       true
	 * @post      true
	 * @obligation 
	 * @param element
	 */
	private void findRecursiveElements(ModelElement element) {
		if (element instanceof Component) {
			this.addElement(((Component)element).getName(), element);
			List<ModelElement> children = ((Component)element).getChildren();
			for(ModelElement child : children) {
				this.findRecursiveElements(child);
			}
		} else if (element instanceof Scenario) {
			this.addElement(((Scenario)element).getName(), element);
		} else if (element instanceof State) {
			this.addElement(((State)element).getName(), element);
		}
	}
	
	/**
	 * Adds a ModelElement to the nameMap.
	 * @pre       true
	 * @post      true
	 * @obligation 
	 * @param name
	 * @param element
	 */
	private void addElement(String name, ModelElement element) {
		if (nameMap.containsKey(name)) {
			nameMap.get(name).add((VisualizableModelElement)element);
		} else {
			ArrayList<VisualizableModelElement> elementList = new ArrayList<VisualizableModelElement>();
			elementList.add((VisualizableModelElement)element);
			nameMap.put(name, elementList);
		}
	}
}
