package ch.unizh.ifi.adora.weaver.engine.generator;

import java.lang.reflect.Method;
import java.util.List;
import java.util.TreeSet;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.commands.AssociationReconnect;
import ch.unizh.ifi.adora.model.commands.InsertNodesVisuallyByScenario;
import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.commands.ScenarioConnectionCreate;
import ch.unizh.ifi.adora.model.commands.ScenarioSeqNrIncrement;
import ch.unizh.ifi.adora.model.commands.ScenarioSeqNrSet;
import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.ISpecificationContainer;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.ParseTool;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ActorScenarioAssociationList;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ChunkCloner;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ScenarioWeavingTarget;

/**
 * Handles a weaveable scenario component with all it's INSTEAD, BEFORE and
 * AFTER join relations. This class creates all the commands for the copy-,
 * connect- and delete actions of the scenario weaving process.
 * 
 * @author Marc Vontobel
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 29.06.2007 MV First Version 03.07.2007 MV Comments reformatted and
 *          completed. 11.12.2007 MV Comments updated and renamed
 * @version $Date: 2009-06-22 19:20:19 +0200 (Mon, 22 Jun 2009) $, $Author:
 *          vontobel $, $Revision: 4390 $
 * 
 */
public class ScenarioChunkWeavingCommandGenerator extends
		AbstractCommandGenerator {

	/**
	 * The WeavingTarget containing all the information provided by the analyzer
	 * to create here the right commands for weaving.
	 */
	private ScenarioWeavingTarget weavingTarget;

	/**
	 * The target of the join relation.
	 */
	private Scenario scenarioParentNode;

	/**
	 * The parent INodeContainer which holds the target of the join relation.
	 */
	private INodeContainer parentContainer;

	/**
	 * 
	 * @param weavingTarget
	 *            The weaving target to make commands out of it.
	 */
	public ScenarioChunkWeavingCommandGenerator(
			ScenarioWeavingTarget weavingTarget) {
		this.weavingTarget = weavingTarget;

		// define the global parent node
		this.parentContainer = weavingTarget.getParent();
		this.scenarioParentNode = getParentScenario(weavingTarget
				.getTargetNode());

		this.handleInsteads();
		this.handleBefores();
		this.handleAfters();
	}

	/**
	 * Iterates over all instead join relations. The first one replaces the
	 * target node. Every other instead-node will be inserted AFTER the first
	 * one.
	 * 
	 */
	private void handleInsteads() {
		TreeSet<ChunkCloner> chunkCloners = this.weavingTarget.getInstead();

		int i = 0;
		for (ChunkCloner chunkCloner : chunkCloners) {

			copyGroupCloneIntoTargetContainer(chunkCloner);

			Scenario clonedEntryNode = (Scenario) chunkCloner.getEntryPoint();

			/**
			 * In the following block, the first INSTEAD will be handled
			 * differently than every other instead to the same target node. The
			 * first one will connect itself to the target parent node, adopts
			 * its type to the scenariotype of the target node (and if required
			 * takes the sequence number of it). After that, the target node is
			 * deleted.
			 * 
			 * If there are more than one INSTEAD, the other INSTEADs will be
			 * added after the first one and will not substitute any other node.
			 */
			if (i == 0) {

				/**
				 * Check if the parentNode is defined. If not, the parent node
				 * will get replaced.
				 */
				if (this.scenarioParentNode == null) {
					this.scenarioParentNode = weavingTarget.getTargetNode();

					/**
					 * If there is an actor connected to the node which gets
					 * replaced, we have to reconnect the actor to the new node.
					 * This is only the case if the parent node is connected to
					 * that actor. All other cases are not allowed.
					 */
					List<Connection> connections = this.scenarioParentNode
							.getConnections();
					for (Connection connection : connections) {
						if (connection instanceof Association) {
							AssociationReconnect cmd = new AssociationReconnect(
									(Association) connection);
							if (connection.getTarget().equals(
									this.scenarioParentNode)) {
								cmd.setNewTarget(clonedEntryNode);
								GeneratorHelper.setDefaultTargetClickPoint(cmd);
							} else {
								cmd.setNewSource(clonedEntryNode);
								GeneratorHelper.setDefaultSourceClickPoint(cmd);
							}
							addWeaveCommand(cmd);
						}
					}

					/**
					 * Removing the existing tree because the root node is
					 * getting replaced.
					 */
					this.removeExistingTree(this.scenarioParentNode);
					this.scenarioParentNode = clonedEntryNode;
				} else { // Reconnect the instead
					ScenarioConnectionCreate cmd = new ScenarioConnectionCreate(
							this.scenarioParentNode);
					cmd.setTarget(chunkCloner.getEntryPoint());
					GeneratorHelper.setDefaultClickPoints(cmd);
					addWeaveCommand(cmd);
					/**
					 * If the node is a sequence, we have to add the sequence
					 * number now.
					 */
					if (this.weavingTarget.getTargetScenarioType() == Scenario.SEQUENCE) {
						clonedEntryNode.setScenarioType(Scenario.SEQUENCE);
						clonedEntryNode.setSeqNr(weavingTarget.getTargetNode()
								.getSeqNo());
					} else {
						clonedEntryNode.setScenarioType(this.weavingTarget
								.getTargetScenarioType());
					}

					/**
					 * If there is an actor connected to the node which gets
					 * replaced is connected to an actor, we have to reconnect
					 * the actor to the new node
					 */
					List<Connection> connections = this.weavingTarget
							.getTargetNode().getConnections();
					for (Connection connection : connections) {
						if (connection instanceof Association) {
							AssociationReconnect arcmd = new AssociationReconnect(
									(Association) connection);
							if (connection.getTarget().equals(
									this.weavingTarget.getTargetNode())) {
								arcmd.setNewTarget(clonedEntryNode);
								GeneratorHelper
										.setDefaultTargetClickPoint(arcmd);
							} else {
								arcmd.setNewSource(clonedEntryNode);
								GeneratorHelper
										.setDefaultSourceClickPoint(arcmd);
							}

							addWeaveCommand(arcmd);
						}
					}

					this.removeExistingTree(this.weavingTarget.getTargetNode());
					weavingTarget.setTargetNode(clonedEntryNode);
				}
			} else { // More than one instead and it is not the first one
				this.insertAfter(clonedEntryNode, this.weavingTarget
						.getTargetNode());
			}

			i++;

			deleteConnection(chunkCloner.getJoinRelation());
		}
	}

	/**
	 * If there is a join relation pointing to a root node, and the weaving type
	 * is to replace it (instead), the whole tree of the target area must get
	 * removed and replaced by the new tree. So this method is just a helper to
	 * remove any branches of a node.
	 * 
	 * @param parentNode
	 *            The top node of the tree which has to get deleted (including
	 *            this node)
	 */
	private void removeExistingTree(Scenario parentNode) {
		List<Connection> connections = parentNode.getSourceConnections();
		for (Connection connection : connections) {
			if (connection instanceof ScenarioConnection) {
				this.removeExistingTree((Scenario) connection.getTarget());
			}
		}
		Command removeVisually = new RemoveNodeVisually(parentNode);
		addWeaveCommand(removeVisually);
		Command deleteNode = new NodeDelete(this.weavingTarget.getParent(),
				parentNode);
		addWeaveCommand(deleteNode);
	}

	/**
	 * Iterates over all before relations and inserts the cloned nodes into the
	 * target container.
	 */
	private void handleBefores() {
		TreeSet<ChunkCloner> chunkCloners = this.weavingTarget.getBefore();

		for (ChunkCloner chunkCloner : chunkCloners) {
			copyGroupCloneIntoTargetContainer(chunkCloner);

			Scenario clonedEntryNode = (Scenario) chunkCloner.getEntryPoint();

			this.insertBefore(clonedEntryNode, this.weavingTarget
					.getTargetNode());

			deleteConnection(chunkCloner.getJoinRelation());
		}
	}

	/**
	 * Inserts a scenario node before another scenraio node and connects it to
	 * it's parent.
	 * 
	 * @param clonedEntryNode
	 *            New node to add before the reference node.
	 * @param targetNode
	 *            Targeted node.
	 */
	private void insertBefore(Scenario clonedEntryNode, Scenario targetNode) {
		this.connectToParent(clonedEntryNode, targetNode);

		/**
		 * The following switch block handles the different scenario types. It
		 * handles the sequence number for sequences and converts the cloned
		 * node into the type of the target node in every other case.
		 */
		switch (this.weavingTarget.getTargetScenarioType()) {
		case Scenario.SEQUENCE:
			clonedEntryNode.setScenarioType(Scenario.SEQUENCE);
			addWeaveCommand(new ScenarioSeqNrSet(clonedEntryNode, targetNode,
					ScenarioSeqNrSet.EQUAL));
			addWeaveCommand(new ScenarioSeqNrIncrement(this.scenarioParentNode,
					clonedEntryNode));
			break;
		default:
			//deletes scenario type, which we want to omit
			
		/*	clonedEntryNode.setScenarioType(this.weavingTarget
					.getTargetScenarioType());*/
			break;
		}
	}

	/**
	 * Iterates over all after relations and inserts the cloned nodes into the
	 * target container
	 * 
	 */
	private void handleAfters() {
		TreeSet<ChunkCloner> chunkCloners = this.weavingTarget.getAfter();

		for (ChunkCloner chunkCloner : chunkCloners) {
			copyGroupCloneIntoTargetContainer(chunkCloner);

			Scenario cloneStartNode = (Scenario) chunkCloner.getEntryPoint();

			this
					.insertAfter(cloneStartNode, this.weavingTarget
							.getTargetNode());

			deleteConnection(chunkCloner.getJoinRelation());
		}
	}

	/**
	 * Insert a node after another node and connects it to it's parent
	 * 
	 * @param clonedEntryNode
	 *            new node to add after the reference node
	 * @param targetNode
	 *            reference node
	 */
	private void insertAfter(Scenario clonedEntryNode, Scenario targetNode) {
		this.connectToParent(clonedEntryNode, targetNode);
		/**
		 * The following switch block handles the different scenario types. It
		 * handles the sequence number for sequences and converts the cloned
		 * node into the type of the target node in every other case.
		 */
		switch (this.weavingTarget.getTargetScenarioType()) {
		case Scenario.SEQUENCE:
			clonedEntryNode.setScenarioType(Scenario.SEQUENCE);
			addWeaveCommand(new ScenarioSeqNrSet(clonedEntryNode, targetNode,
					ScenarioSeqNrSet.AFTER));
			addWeaveCommand(new ScenarioSeqNrIncrement(this.scenarioParentNode,
					clonedEntryNode));
			this.weavingTarget.setTargetNode(clonedEntryNode);
			break;
		default:
			//deletes scenario type, which we want to omit
		/*	clonedEntryNode.setScenarioType(this.weavingTarget
					.getTargetScenarioType());*/
			break;
		}
	}

	/**
	 * Inserts all scenariochunks connected by an after join relationship after
	 * the target scenario node.
	 * 
	 * @param node
	 */
	private void connectToParent(Scenario node, Scenario parent) {
		ScenarioConnectionCreate cmd;
		// after and before weaving , when there is only one root
		if (this.scenarioParentNode == null && parent != null) {
			cmd = new ScenarioConnectionCreate(parent);
		} else {
			cmd = new ScenarioConnectionCreate(this.scenarioParentNode);
		}
		cmd.setTarget(node);
		GeneratorHelper.setDefaultClickPoints(cmd);
		addWeaveCommand(cmd);
	}

	/**
	 * Finds the parent node of a target container.
	 * 
	 * @param child
	 *            The child node.
	 * @return parent The next higher node.
	 */
	private Scenario getParentScenario(Scenario child) {
		Scenario found = null;
		List<Connection> connections = child.getTargetConnections();
		for (Connection connection : connections) {
			if (connection instanceof ScenarioConnection) {
				found = ((Scenario) connection.getSource());
				break;
			}
		}
		return found;
	}

	/**
	 * Creates a command to weave the two functional specifications into each
	 * other.
	 * 
	 * @pre chunkClone != null && parentContainer != null. Functional
	 *      Specification of parentContainer = n.
	 * @post Functional Specification of parentContainer >= n.
	 * @param chunkClone
	 *            The clone of which the functional specification comes from.
	 */
	private void handleFunctionalSpecification(ChunkCloner clone) {

		String fs1Unparsed = ((ISpecificationContainer) this.parentContainer)
				.getSpecification();
		String fs2Unparsed = clone.getAspectContainer().getSpecification();

		String combinedParsedFS = ParseTool.getCombinedFunctionalSpecification(
				fs1Unparsed, fs2Unparsed, clone.getAspectContainer().getName());

		Method setMethod;
		Method getMethod;

		try {
			setMethod = ISpecificationContainer.class.getMethod(
					"setSpecification", String.class);
			getMethod = ISpecificationContainer.class
					.getMethod("getSpecification");
		} catch (Exception e) {
			return;
		}

		SetProperty setProperty = new SetProperty(
				(VisualizableModelElement) this.parentContainer,
				combinedParsedFS, setMethod, getMethod);
		addWeaveCommand(setProperty);

	}

	/**
	 * Copies the whole group into the target container
	 * 
	 * @param clone
	 */
	private void copyGroupCloneIntoTargetContainer(ChunkCloner clone) {

		this.handleFunctionalSpecification(clone);
		ActorScenarioAssociationList narc = new ActorScenarioAssociationList(
				clone);

		addWeaveCommand(new InsertNodesVisuallyByScenario(
				(ZoomableNodeContainer) parentContainer,
				clone.getClonedNodes(), (Scenario) clone.getEntryPoint()));

		/**
		 * Checking for any association connected to one of the scenario nodes.
		 */
		if (!narc.isEmpty()) {
			ActorScenarioAssociationCommandGenerator asacg = new ActorScenarioAssociationCommandGenerator(
					narc);
			addWeaveCommand(asacg.getWeaveCommands());
		}
	}
}
