package ch.unizh.ifi.adora.weaver.engine.generator;

import java.lang.reflect.Method;
import java.util.ArrayList;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Role;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.TransitionDescription;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ServerComponentAnalyzer;

/**
 * The server component command generator. It generates commands for the
 * replacement of the roles of associations between an {@link AspectContainer}
 * and a server {@link Component}.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.01.2008 PS Review changes.
 * @version $Date: 2009-05-07 10:34:09 +0200 (Thu, 07 May 2009) $, $Author: anil $, $Revision: 4199 $
 */
public class ServerComponentCommandGenerator extends AbstractCommandGenerator {
	/**
	 * Holds the {@link ServerComponentAnalyzer}.
	 */
	private ServerComponentAnalyzer serverComponentAnalyzer;

	/**
	 * Creates a new instance of this class and generates the commands needed.
	 * 
	 * @param serverComponentAnalyzer
	 */
	public ServerComponentCommandGenerator(ServerComponentAnalyzer serverComponentAnalyzer) {
		this.serverComponentAnalyzer = serverComponentAnalyzer;

		for (Component component : this.serverComponentAnalyzer.getServerComponents()) {
			generateCommands(component);
		}
	}

	/**
	 * Get all command which are necessary to weave this server component.
	 * 
	 * @param component
	 *            the server {@link Component}.
	 */
	private void generateCommands(Component component) {
		for (AssociationGroup associationGroup : getAssociationGroups(component)) {
			if (associationGroup.getAssociations().size() < 2) {
				// there is no change needed
				continue;
			}

			String serverRoleName = associationGroup.getRoleFromServerComponent().getName();
			String aspectRoleName = associationGroup.getRoleFromAspect().getName();

			for (int i = 0; i < associationGroup.getAssociations().size(); i++) {
				Association association = associationGroup.getAssociations().get(i);

				String roleFrom = association.getRoleFrom().getName();
				String roleFromNew = roleFrom + (i + 1);
				String roleTo = association.getRoleTo().getName();
				String roleToNew = roleTo + (i + 1);

				ZoomableNodeContainer target;
				if (association.getSource().equals(component)) {
					target = (ZoomableNodeContainer) association.getTarget();
				} else {
					target = (ZoomableNodeContainer) association.getSource();
				}
				addWeaveCommand(changeTransitions(target, "over " + roleFrom, "over " + roleFromNew));
				addWeaveCommand(changeTransitions(target, "over " + roleTo, "over " + roleToNew));

				addWeaveCommand(getSetNameProperty(Role.class, association.getRoleFrom(),
						roleFromNew));
				addWeaveCommand(getSetNameProperty(Role.class, association.getRoleTo(), roleToNew));
			}

			addWeaveCommand(changeTransitionsInServerComponent(component, associationGroup,
					aspectRoleName));
			addWeaveCommand(changeTransitionsInServerComponent(component, associationGroup,
					serverRoleName));
		}
	}

	/**
	 * special method for the transitions in the server component. there it is
	 * not only a replacement from "over a" to "over a1", in the server
	 * component "over a" will be replaced with "over a1, a2", etc.
	 * 
	 * @param component
	 * @param associationGroup
	 * @param roleName
	 * @return
	 */
	private ArrayList<Command> changeTransitionsInServerComponent(Component component,
			AssociationGroup associationGroup, String roleName) {
		String searchFor = "over " + roleName;
		String replaceWith = "over " + roleName + "1";

		for (int i = 2; i <= associationGroup.getAssociations().size(); i++) {
			replaceWith += ", " + roleName + i;
		}

		return changeTransitions(component, searchFor, replaceWith);
	}

	/**
	 * Changes all transitions in a container. searches for a string and
	 * replaces it in the transition description of the transitions.
	 * 
	 * @param component
	 * @param searchFor
	 * @param replaceWith
	 * @return
	 */
	private ArrayList<Command> changeTransitions(ZoomableNodeContainer component, String searchFor,
			String replaceWith) {
		ArrayList<Command> commands = new ArrayList<Command>();
		ArrayList<Transition> transitions = new ArrayList<Transition>();

		// Getting all transitions between the (child-)states in the container.
		for (ModelElement modelElement : component.getChildren()) {
			if (!(modelElement instanceof State)) {
				continue;
			}

			State state = (State) modelElement;

			for (Connection connection : state.getConnections()) {
				if (!(connection instanceof Transition)) {
					continue;
				}

				Transition transition = (Transition) connection;
				if (!transitions.contains(transition)) {
					transitions.add(transition);
				}
			}
		}

		// replace the transition descriptions
		for (Transition transition : transitions) {
			TransitionDescription td = transition.getLabel();

			if (td == null || td.getName() == null) {
				continue;
			}

			String regex = searchFor;
			String newName = td.getName().replaceAll(regex, replaceWith);

			if (!newName.equals(td.getName())) {
				SetProperty setProperty = getSetNameProperty(TransitionDescription.class, td,
						newName);
				commands.add(setProperty);
			}
		}

		return commands;
	}

	/**
	 * Creates a SetProperty for an element of class clazz.
	 * 
	 * @param clazz
	 * @param modelElement
	 * @param replacement
	 * @return
	 */
	private SetProperty getSetNameProperty(Class<?> clazz, VisualizableModelElement modelElement,
			String replacement) {
		Method setMethod;
		Method getMethod;
		try {
			setMethod = clazz.getMethod("setName", String.class);
			getMethod = clazz.getMethod("getName");
		} catch (Exception e) {
			return null;
		}

		return new SetProperty(modelElement, replacement, setMethod, getMethod);

	}

	/**
	 * group the associations of the server component into groups
	 * 
	 * @param component
	 *            assumes a server component as input.
	 * @return
	 */
	private ArrayList<AssociationGroup> getAssociationGroups(Component component) {
		ArrayList<AssociationGroup> associationGroups = new ArrayList<AssociationGroup>();

		for (Connection connection : component.getConnections()) {
			if (!(connection instanceof Association)
					|| !(connection.getTarget() instanceof Component && connection.getSource() instanceof Component)) {
				continue;
			}

			Association association = (Association) connection;

			boolean groupFound = false;
			for (AssociationGroup associationGroup : associationGroups) {
				if (associationGroup.matches(association)) {
					associationGroup.add(association);
					groupFound = true;
					break;
				}
			}

			if (!groupFound) {
				associationGroups.add(new AssociationGroup(component, association));
			}
		}

		return associationGroups;
	}
}
