package ch.unizh.ifi.adora.weaver.engine.generator;

import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.factories.ModelObjectMapper;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Role;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ServerComponentAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingHistory;

/**
 * This server component command generator generates commands for creating the
 * new associations between the server components and the components they serve
 * after the weaving.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 28.11.2007 PS First Version<br>
 *          16.12.2007 PS Review Changes
 * @version $Date: 2007-12-17 13:44:50 +0100 (Mon, 17 Dec 2007) $, $Author: schoeni $, $Revision: 3419 $
 */
public class ServerComponentReconnectionCommandGenerator extends AbstractCommandGenerator {
	public ServerComponentReconnectionCommandGenerator(
			ServerComponentAnalyzer serverComponentAnalyzer) {

		// getting all server components
		for (Component component : serverComponentAnalyzer.getServerComponents()) {
			handleComponent(component);
		}

	}

	/**
	 * if there are associations which connect this component to another
	 * component (serving), then this association will be investigated by
	 * handleAssociation().
	 * 
	 * @param component
	 */
	private void handleComponent(Component component) {
		for (Connection connection : component.getConnections()) {
			if (connection instanceof Association
					&& ServerComponentAnalyzer
							.isServerComponentAssociation((Association) connection)) {
				handleAssociation((Association) connection);
			}
		}
	}

	/**
	 * Handles an association. The method creates a connection for each
	 * component where the groups of the origin aspect were cloned into.
	 * 
	 * @param association
	 */
	private void handleAssociation(Association association) {
		// find the target aspect
		AspectContainer target = null;

		if (association.getSource() instanceof AspectContainer) {
			target = (AspectContainer) association.getSource();
		} else {
			target = (AspectContainer) association.getTarget();
		}

		ModelObjectMapper.getInstance().reinitialize();

		// find all weaving targets
		for (Component component : WeavingHistory.getInstance().getTransitiveWeavingTargets(target)) {
			AssociationCreate create;

			if (association.getSource() instanceof AspectContainer) {
				create = new AssociationCreate(component);
				create.setTarget(association.getTarget());
			} else {
				create = new AssociationCreate(association.getSource());
				create.setTarget(component);
			}

			try {
				if (!"".equals(association.getRoleFrom().getName())) {
					create.setRoleFrom((Role) association.getRoleFrom().deepClone());
				}
				if (!"".equals(association.getRoleTo().getName())) {
					create.setRoleTo((Role) association.getRoleTo().deepClone());
				}
			} catch (CloneNotSupportedException e) {
				e.printStackTrace();
			}

			GeneratorHelper.setDefaultClickPoints(create);
			addWeaveCommand(create);
		}

		// delete this connection
		deleteConnection(association);
	}

}
