package ch.unizh.ifi.adora.weaver.engine.generator;

import java.lang.reflect.Method;
import java.util.Hashtable;

import ch.unizh.ifi.adora.model.commands.InsertNodeVisually;
import ch.unizh.ifi.adora.model.commands.InsertNodesVisuallyByTransition;
import ch.unizh.ifi.adora.model.commands.NodeCreate;
import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.commands.TransitionReconnect;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.ISpecificationContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.TransitionDescription;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.structure.ContextMap;
import ch.unizh.ifi.adora.model.structure.SimpleTransition;
import ch.unizh.ifi.adora.model.util.ParseTool;
import ch.unizh.ifi.adora.model.util.TransitionAdapterVisitor;
import ch.unizh.ifi.adora.model.util.TransitionVisitor;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ChunkCloner;
import ch.unizh.ifi.adora.weaver.engine.analyzer.GroupCloner;
import ch.unizh.ifi.adora.weaver.engine.analyzer.StateWeavingTarget;

/**
 * Generates the commands for the state command
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 29.06.2007 PS First Version 18.09.2007 PS Comments reformatted and
 *          completed.
 * @version $Date: 2008-01-30 10:25:51 +0100 (Wed, 30 Jan 2008) $, $Author: vontobel $, $Revision: 3511 $
 * 
 */
public class StateCommandGenerator extends AbstractCommandGenerator {
	/**
	 * The WeavingTarget.
	 */
	private StateWeavingTarget stateWeavingTarget;

	/**
	 * Holds the current transition description
	 */
	private String transitionDescription;

	/**
	 * Holds the current target node.
	 */
	private Node targetNode;

	/**
	 * Constructs a state command generator. need a weaveable to process.
	 * 
	 * @param weaveable
	 */
	public StateCommandGenerator(StateWeavingTarget weaveable) {
		this.stateWeavingTarget = weaveable;

		this.transitionDescription = weaveable.getTransition().getLabel().getName();
		this.targetNode = weaveable.getTransition().getTarget();

		handleAfters();
		handleInsteads();
		handleBefores();

		setStartNode();
		setTransitionDescription();
	}

	/**
	 * reconnects the origin transition to the new start node
	 */
	private void setStartNode() {
		TransitionReconnect tr = new TransitionReconnect(stateWeavingTarget.getTransition());
		tr.setNewTarget(this.targetNode);
		GeneratorHelper.setDefaultTargetClickPoint(tr);
		addWeaveCommand(tr);
	}

	/**
	 * replaces the transition description of the origin transition with the
	 * actual transition description
	 */
	private void setTransitionDescription() {
		Method setMethod;
		Method getMethod;

		try {
			setMethod = TransitionDescription.class.getMethod("setName", String.class);
			getMethod = TransitionDescription.class.getMethod("getName");
		} catch (Exception e) {
			return;
		}

		SetProperty setProperty = new SetProperty(stateWeavingTarget.getTransition().getLabel(),
				transitionDescription, setMethod, getMethod);
		addWeaveCommand(setProperty);
	}

	/**
	 * handles all after join relations
	 */
	private void handleAfters() {
		for (ChunkCloner chunkCloner : stateWeavingTarget.getAfter()) {
			startGroupWeaving(chunkCloner);

			handleAfter(chunkCloner);

			finishGroupWeaving(chunkCloner);
		}
	}

	/**
	 * starts the weaving of a group. clones the whole group into the model
	 * 
	 * @param chunkCloner
	 */
	private void startGroupWeaving(ChunkCloner chunkCloner) {

		this.handleFunctionalSpecification(chunkCloner);

		addWeaveCommand(new InsertNodesVisuallyByTransition(
				(ZoomableNodeContainer) stateWeavingTarget.getParent(), chunkCloner
						.getClonedNodes(), stateWeavingTarget.getTransition()));

		Hashtable<String, String> entries = analyzeJoinRelation(chunkCloner.getJoinRelation());

		if (entries != null) {
			for (Node node : chunkCloner.getClonedNodes()) {
				for (Connection connection : node.getSourceConnections()) {
					if (!(connection instanceof Transition)) {
						continue;
					}

					Transition transition = (Transition) connection;
					String simpleTransition = transition.getLabel().getName();

					SimpleTransition me = ParseTool.getSimpleTransition(((Transition) connection)
							.getLabel().getName());

					if (me == null) {
						continue;
					}

					TransitionAdapterVisitor visitor = new TransitionAdapterVisitor(entries);
					// SysoutVisitor visitor = new SysoutVisitor();
					me.jjtAccept(visitor);

					if (simpleTransition.equals(visitor.getOutput())) {
						continue;
					}

					Method setMethod;
					Method getMethod;
					try {
						setMethod = TransitionDescription.class.getMethod("setName", String.class);
						getMethod = TransitionDescription.class.getMethod("getName");
					} catch (Exception e) {
						return;
					}

					SetProperty setProperty = new SetProperty(transition.getLabel(), visitor
							.getOutput(), setMethod, getMethod);
					addWeaveCommand(setProperty);
				}
			}
		}
	}

	/**
	 * Creates a command to weave the two functional specifications into each
	 * other.
	 * 
	 * @pre chunkClone != null && parentContainer != null. Functional
	 *      Specification of parentContainer = n.
	 * @post Functional Specification of parentContainer >= n.
	 * @param chunkCloner
	 *            The clone of which the functional specification comes from.
	 */
	private void handleFunctionalSpecification(ChunkCloner chunkCloner) {

		String fs1Unparsed = ((ISpecificationContainer) this.stateWeavingTarget.getParent())
				.getSpecification();
		String fs2Unparsed = chunkCloner.getAspectContainer().getSpecification();


		String combinedParsedFS = ParseTool.getCombinedFunctionalSpecification(fs1Unparsed,
				fs2Unparsed, chunkCloner.getAspectContainer().getName());

		Method setMethodFS;
		Method getMethodFS;

		try {
			setMethodFS = ISpecificationContainer.class.getMethod("setSpecification", String.class);
			getMethodFS = ISpecificationContainer.class.getMethod("getSpecification");
		} catch (Exception e) {
			return;
		}

		SetProperty setPropertyFS = new SetProperty(
				(VisualizableModelElement) this.stateWeavingTarget.getParent(), combinedParsedFS,
				setMethodFS, getMethodFS);
		addWeaveCommand(setPropertyFS);
	}

	/**
	 * Analyzes the context map on the joinrelation and retrieves the entries
	 * (identifier : typename).
	 * 
	 * @param joinRelation
	 * @return
	 */
	private Hashtable<String, String> analyzeJoinRelation(JoinRelation joinRelation) {
		String contextMap = joinRelation.getContextMap();

		if (contextMap == null || "".equals(contextMap)) {
			return null;
		}

		ContextMap me = (ContextMap) ParseTool.parseString("ContextMap", contextMap);

		if (me == null) {
			return null;
		}

		SymbolTableVisitor visitor = new SymbolTableVisitor();
		me.jjtAccept(visitor);

		return visitor.getHashtable();
	}

	/**
	 * Removes the joinrelation and the exitpoint of a groupclone and sets the
	 * target node to the start node of this group
	 * 
	 * @param chunkCloner
	 */
	private void finishGroupWeaving(ChunkCloner chunkCloner) {
		targetNode = chunkCloner.getEntryPoint();

		deleteConnection(chunkCloner.getJoinRelation());
		deleteExitPoint(chunkCloner);
	}

	/**
	 * handles an after join relation
	 * 
	 * @param groupCloner
	 */
	private void handleAfter(GroupCloner groupCloner) {
		reconnectIncomingTransitions(getExitPoint(groupCloner), targetNode);
	}

	/**
	 * handles all before joinrelations
	 */
	private void handleBefores() {
		for (ChunkCloner chunkCloner : stateWeavingTarget.getBefore()) {
			startGroupWeaving(chunkCloner);

			handleBefore(chunkCloner);

			finishGroupWeaving(chunkCloner);
		}
	}

	/**
	 * handles a before
	 * 
	 * @param groupCloner
	 */
	private void handleBefore(GroupCloner groupCloner) {
		// edit the transition description
		String actionPart = getActionPart(transitionDescription);
		transitionDescription = getConditionPart(transitionDescription) + " |";

		// Adding a new, empty state with the name of the exitpoint
		ExitPoint exitPoint = getExitPoint(groupCloner);

		State state = new State();
		state.setName(exitPoint.getName());
		Rectangle bounds = new Rectangle(exitPoint.getLocation(), exitPoint.getSize());
		NodeCreate nodeCreate = new NodeCreate(state, stateWeavingTarget.getParent(), bounds);
		addWeaveCommand(nodeCreate);

		InsertNodeVisually moveCommand = new InsertNodeVisually(state, false);
		addWeaveCommand(moveCommand);

		// connect the transitions from the exit point to the new node
		reconnectIncomingTransitions(getExitPoint(groupCloner), state);

		// creating a transition from the new state to the old start state
		TransitionCreate createTransition = new TransitionCreate(state);
		createTransition.setTransitionDescription(actionPart);
		createTransition.setTarget(targetNode);
		GeneratorHelper.setDefaultClickPoints(createTransition);
		addWeaveCommand(createTransition);
	}

	/**
	 * Handles all insets
	 */
	private void handleInsteads() {
		for (ChunkCloner chunkCloner : stateWeavingTarget.getInstead()) {
			startGroupWeaving(chunkCloner);

			handleInstead(chunkCloner);

			finishGroupWeaving(chunkCloner);
		}
	}

	/**
	 * handles an inset
	 * 
	 * @param groupCloner
	 */
	private void handleInstead(GroupCloner groupCloner) {
		transitionDescription = getConditionPart(transitionDescription) + " |";

		reconnectIncomingTransitions(getExitPoint(groupCloner), this.targetNode);
	}

	/**
	 * reconnects the incoming connections of from to to
	 * 
	 * @param from
	 * @param target
	 */
	private Transition reconnectIncomingTransitions(ConnectableElement from,
			ConnectableElement target) {
		Transition endTransition = null;

		for (Connection connection : from.getTargetConnections()) {
			TransitionReconnect tr = new TransitionReconnect((Transition) connection);
			tr.setNewTarget(target);

			addWeaveCommand(tr);

			GeneratorHelper.setDefaultTargetClickPoint(tr);

			endTransition = (Transition) connection;
		}

		return endTransition;
	}

	/**
	 * deletes the exitpoint
	 */
	private void deleteExitPoint(GroupCloner groupCloner) {
		Node exitPoint = getExitPoint(groupCloner);

		RemoveNodeVisually radialRemove = new RemoveNodeVisually(exitPoint);
		addDeleteCommand(radialRemove);

		NodeDelete deleteCommand = new NodeDelete(stateWeavingTarget.getParent(), exitPoint);
		addDeleteCommand(deleteCommand);
	}

	/**
	 * Getting the exitpoint of a groupclone
	 * 
	 * @return ExitPoint
	 */
	private ExitPoint getExitPoint(GroupCloner groupCloner) {
		for (Node node : groupCloner.getClonedNodes()) {
			if (!(node instanceof ExitPoint)) {
				continue;
			}

			return (ExitPoint) node;
		}

		return null;
	}

	/**
	 * Condition part of the transition
	 * 
	 * @param simpleTransition
	 * @return
	 */
	private String getConditionPart(String simpleTransition) {
		SimpleTransition me = ParseTool.getSimpleTransition(simpleTransition);

		if (me == null) {
			return "";
		}

		TransitionVisitor visitor = new TransitionVisitor();

		me.getConditionPart().jjtAccept(visitor);

		return visitor.getOutput();
	}

	/**
	 * Action part of the transition
	 * 
	 * @param simpleTransition
	 * @return
	 */
	private String getActionPart(String simpleTransition) {
		SimpleTransition me = ParseTool.getSimpleTransition(simpleTransition);

		if (me == null) {
			return "";
		}

		TransitionVisitor visitor = new TransitionVisitor();

		me.getAdoraActionPart().jjtAccept(visitor);

		return visitor.getOutput();
	}

}
