package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import ch.unizh.ifi.adora.model.commands.InsertNodesVisuallyByOrientation;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.NodeGroup;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ActorScenarioAssociationList;
import ch.unizh.ifi.adora.weaver.engine.analyzer.CrosscuttingNodeGroup;
import ch.unizh.ifi.adora.weaver.engine.analyzer.CrosscuttingNodeGroupList;
import ch.unizh.ifi.adora.weaver.engine.analyzer.GroupCloner;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingHistory;

/**
 * A Command Generator for unrelated weaving components (aspect component with
 * no outgoing or incoming join relation (i.e. an aspect component or a
 * beheaviour chunk)
 * 
 * @author Marc Vontobel
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 08.10.2007 MV First Version 08.10.2007 MV Comments reformatted and
 *          completed.
 * @version $Date: 2009-06-18 15:18:56 +0200 (Thu, 18 Jun 2009) $, $Author: anil
 *          $, $Revision: 4372 $
 * 
 */
public class UnrelatedWeaveComponentCommandGenerator extends
		AbstractCommandGenerator {
	public UnrelatedWeaveComponentCommandGenerator() {
		List<CrosscuttingNodeGroup> components = CrosscuttingNodeGroupList
				.getInstance().getCrosscuttingNodeGroups();

		for (CrosscuttingNodeGroup component : components) {
			// handles components
			for (Component target : WeavingHistory
					.getInstance()
					.getTransitiveWeavingTargets(component.getSourceContainer())) {
				this.handleCrosscuttingWeaveComponent(component, target);
			}
			// handles aspects
			for (AspectContainer target : WeavingHistory.getInstance()
					.getTransitiveWeavingTargetsAspect(
							component.getSourceContainer())) {
				this.handleCrosscuttingWeaveComponent(component, target);
			}
		}
	}

	private void handleCrosscuttingWeaveComponent(
			CrosscuttingNodeGroup component, ZoomableNodeContainer target) {
		GroupCloner groupCloner = new GroupCloner(component);
		List<Node> nodes = groupCloner.getClonedNodes();
		copyJoinRelations(component, nodes);
		// Use the layout helper to calculate min x / y and max x / y
		LayoutHelper layoutHelper = new LayoutHelper(nodes);

		// calculate the gaps
		AspectContainer aspectContainer = (AspectContainer) component
				.getNodes().get(0).getParent();

		// /store the connections for rerouting
		this.copyConnections(component.getNodes(), nodes);
		// /

		int gapLeft = (int) layoutHelper.minX;
		int gapRight = (int) aspectContainer.getSize().getWidth()
				- layoutHelper.maxX;
		int gapTop = (int) layoutHelper.minY;
		int gapBottom = (int) aspectContainer.getSize().getHeight()
				- layoutHelper.maxY;

		int orientation;

		if (gapLeft < gapRight) {
			// WEST
			if (gapTop < gapBottom) {
				orientation = InsertNodesVisuallyByOrientation.NORTH_WEST;
			} else {
				orientation = InsertNodesVisuallyByOrientation.SOUTH_WEST;
			}
		} else {
			// EAST
			if (gapTop < gapBottom) {
				orientation = InsertNodesVisuallyByOrientation.NORTH_EAST;
			} else {
				orientation = InsertNodesVisuallyByOrientation.SOUTH_EAST;
			}
		}

		addWeaveCommand(new InsertNodesVisuallyByOrientation(target, nodes,
				orientation));

		ActorScenarioAssociationList narc = new ActorScenarioAssociationList(
				groupCloner);

		if (!narc.isEmpty()) {
			ActorScenarioAssociationCommandGenerator asacg = new ActorScenarioAssociationCommandGenerator(
					narc);
			this.addWeaveCommand(asacg.getWeaveCommands());
		}
	}

	/**
	 * Copy connections to the from the state transitions to the cloned state
	 * transitions. This is only applicable for not weaved state charts with
	 * join relationsips, which are undecided.
	 * 
	 * @param component
	 * @param nodes
	 */
	private void copyJoinRelations(CrosscuttingNodeGroup component,
			List<Node> nodes) {
		HashMap<String, JoinRelation> joinRelationReRoutingTable =	new HashMap<String, JoinRelation>();
		for (int i=0;i<component.getNodes().size();i++) {
			joinRelationReRoutingTable=getStates(component.getNodes().get(i));
			reconnect(nodes.get(i),joinRelationReRoutingTable);
		}
		
		
	}

	private void reconnect(Node n,
			HashMap<String, JoinRelation> joinRelationReRoutingTable) {
		if (n instanceof Component) {
			for (NodeGroup g : ((Component) n).getNodeGroups().getStateGroups()) {
				for (Node node : g.getNodes()) {
					if (node instanceof State) {
						for (Connection con : node.getTargetConnections()) {
							if (con instanceof Transition) {
								Transition transition = (Transition) con;
										JoinRelation jr=joinRelationReRoutingTable.get((transition.getName()));
										if(jr!=null){
										Node source=jr.getSource();
										jr.disconnect();
										jr.reconnect(source, transition);
										}
								
								
							}
						}
					}
					if (node instanceof Component) {
						reconnect(node, joinRelationReRoutingTable);
					}
				}
			}

		}
		
	}

	private HashMap<String,JoinRelation> getStates(Node n) {
		HashMap<String, JoinRelation> joinRelationReRoutingTable = new HashMap<String, JoinRelation>();
		if (n instanceof Component) {
			for (NodeGroup g : ((Component) n).getNodeGroups().getStateGroups()) {
				for (Node node : g.getNodes()) {
					if (node instanceof State) {
						for (Connection con : node.getTargetConnections()) {
							if (con instanceof Transition) {
								Transition transition = (Transition) con;
								for (Connection ctransition : transition
										.getTargetConnections()) {
									if (ctransition instanceof JoinRelation) {
										JoinRelation jr = (JoinRelation) ctransition;
										if (jr.getDecision() == null) {
											joinRelationReRoutingTable.put(transition
													.getName(), jr);
										}

									}
								}
							}
						}
					}
					if (node instanceof Component) {
						joinRelationReRoutingTable.putAll(getStates(node));
					}
				}
			}

		}
		return joinRelationReRoutingTable;
	}

	/**
	 * Copy of all connections of a component.
	 * 
	 * @param nodes
	 * @param nodes2
	 */
	private void copyConnections(ArrayList<Node> nodes, List<Node> nodes2) {
		for (int i = 0; i < nodes.size(); i++) {
			int connectionSize = nodes.get(i).getConnectionsCount();
			for (int j = 0; j < connectionSize; j++) {
				Connection connection = nodes.get(i).getConnections().get(0);
				ConnectableElement connectableElementSource;
				ConnectableElement connectableElementTarget;
				if (connection.getSource() == nodes.get(i)) {
					connectableElementTarget = nodes.get(i).getConnections()
							.get(0).getTarget();
					connectableElementSource = nodes2.get(i);

				} else {
					connectableElementSource = nodes.get(i).getConnections()
							.get(0).getSource();
					connectableElementTarget = nodes2.get(i);
				}
				connection.disconnect();
				connection.reconnect(connectableElementSource,
						connectableElementTarget);
				connection.setVisible(true);
				nodes2.get(i).addConnection(connection);
			}
		}

	}

}
