package ch.unizh.ifi.adora.zoom.adora;

import java.awt.Insets;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.zoom.IBorderedZoomNode;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.model.representation.zoom.Interval;
import ch.unizh.ifi.adora.model.representation.zoom.IntervalStructure;
import ch.unizh.ifi.adora.model.representation.zoom.ZoomHole;
import ch.unizh.ifi.adora.zoom.berner.algorithm.LayoutAlgorithm;
import ch.unizh.ifi.adora.zoom.continuous.algorithm.ContinuousZoom;

/**
 * Adapter for the continuous zoom that handles ADORA/GEF specifics like negative
 * coordinates or node borders before the zoom algorithm is called.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-08-09 TR First Version
 * @version				$Date: 2009-07-20 15:44:11 +0200 (Mon, 20 Jul 2009) $, $Author: treinhard $, $Revision: 4528 $
 *
 */
public class AdoraZoomAdapter extends LayoutAlgorithm {

	/**
	 * Serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Singleton instance variable
	 */
	private static AdoraZoomAdapter INSTANCE;
	
	/**
	 * The adapted class.
	 */
	private ContinuousZoom zoom;
	
	/**
	 * Set to true if gap intervals should be scaled during filter operations
	 */
	private boolean scaleGaps = false;
	
	
	/**
	 * Singleton method to get the instance
	 * 
	 * @return the zoom instance
	 */
    public static AdoraZoomAdapter getInstance() {
    	
    	if(INSTANCE == null) {
    		INSTANCE = new AdoraZoomAdapter();
    	}
    	
        return INSTANCE;
    }
    
    
    /**
     * Singleton: Hide constructor
     *
     */
    private AdoraZoomAdapter() {
    	
    	zoom = ContinuousZoom.getInstance();
    	
    }
    
  
    /**
	 * Zoom the node given as argument <i>container</i>. The node is zoomed-in
	 * if <i>zoomIn</i> is true and zoomed-out if </i>zoomIn</i> is false.
	 * 
	 * @param container the node that should be zoomed
	 * @param zoomIn true if the node should be zoomed-in, false otherwise
	 */
    public void zoom(ZoomableNodeContainer container, boolean zoomIn) {
    	if (container.isZoomedIn() != zoomIn) {
	    	firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
	    	firePropertyChange(START_ZOOM, null, null);
	    	
	//    	Rectangle originalBounds = getAbsoluteBounds(container);
	    	
	        container.setZoomedIn(!container.isZoomedIn());
	        zoom.zoom((IZoomNode)container, container.isZoomedIn());
	        
	//        int dx = (int)(originalBounds.getCenterX() - getAbsoluteBounds(container).getCenterX());
	//        int dy = (int)(originalBounds.getCenterY() - getAbsoluteBounds(container).getCenterY());
	//        adjustTopLevel(container, dx, dy);
	        
	        informListeners(container);
	        firePropertyChange(END_ZOOM, null, container);
    	}
    }

	
    /**
	 * Adjusts the model to the changed bounds of the node
	 * given as argument.
	 * 
	 * @param node the node whose bounds should be changed
	 * @param newBounds the new bounds of the node
	 */
	public void changeNodeBounds(IZoomNode node, Rectangle newBounds) {
		
		firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
		
		int x = newBounds.x;
		int y = newBounds.y;
		
		zoom.changeNodeBounds(node, newBounds);
		
		int dx = x - node.getZoomBounds().x;
		int dy = y - node.getZoomBounds().y;

		adjustTopLevel(node, dx, dy);
		
		informListeners(node);

	}


	/**
	 * Method to handle the hiding (i.e. filtering out) of a node.
	 * 
	 * @param node the node that should be hidden or shown
	 */
	public void filterNode(IZoomNode node) {
		
		firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
		
		zoom.filterNode(node, scaleGaps);
		
		informListeners(node);

	}


	/**
	 * Adjusts the model (i.e. positions of the siblings) if the node given
	 * as argument is added.
	 * 
	 * @param node the node that should be added
	 */
	public void insertNode(IZoomNode node) {
		
		firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
		
		Rectangle originalBounds = getAbsoluteBounds(node);
		
		zoom.insertNode(node);
		
        int dx = (int)(originalBounds.x - getAbsoluteBounds(node).x);
        int dy = (int)(originalBounds.y - getAbsoluteBounds(node).y);
        adjustTopLevel(node.getZoomParent(), dx, dy);
		
		informListeners(node);

	}

	/**
	 * TODO: Old ADORA method remove
	 */
	protected void moveNodesInSameLevel(Node n,
			ch.unizh.ifi.adora.model.geometry.Rectangle orgShape,
			ch.unizh.ifi.adora.model.geometry.Rectangle newShape) {

	}


	/**
	 * Adjusts the model (i.e. positions of the siblings) if the node given
	 * as argument is removed.
	 * 
	 * @param node the node that should be removed
	 */
	public void removeNode(IZoomNode node) {
		
		firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
		
		Rectangle originalParentBounds = getAbsoluteBounds(node.getZoomParent());
		
		zoom.removeNode(node);
		
        int dx = (int)(originalParentBounds.getCenterX() - getAbsoluteBounds(node.getZoomParent()).getCenterX());
        int dy = (int)(originalParentBounds.getCenterY() - getAbsoluteBounds(node.getZoomParent()).getCenterY());
        adjustTopLevel(node.getZoomParent(), dx, dy);
		
		informListeners(node);

	}
	
	
	/**
	 * Reduce the size of the node given as argument by reducing the size of
	 * the border around its children.
	 * 
	 * @param node the node whose size should be reduced.
	 */
	public void compactNode(IZoomNode node) {
		
		if(node.isZoomedIn()) {
		
			firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
			
			zoom.compactNode(node);
		
			informListeners(node);
		}
		
	}
	
	
	/**
	 * Adjusts the children nodes to provide enough space for the new border
	 * of the node given as argument.
	 * 
	 * @param node the node of which the border has changed
	 */
	public void adjustBorder(IBorderedZoomNode node) {
		
		firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
		
		Insets insets = node.getZoomInsets();
		IntervalStructure structure = node.getZoomStructure();
		
		Interval topInterval = structure.getVerticalIntervalAt(0);
		if(topInterval.getLength() < insets.top) {
			topInterval.moveEndTo(insets.top);
		}
		Interval bottomInterval = structure.getVerticalIntervals().get(structure.getVerticalIntervals().size() - 1);
		if(bottomInterval.getLength() < insets.bottom) {
			bottomInterval.moveEndTo(bottomInterval.getStart() + insets.bottom);
		}
		
		Interval leftInterval = structure.getHorizontalIntervalAt(0);
		if(leftInterval.getLength() < insets.left) {
			leftInterval.moveEndTo(insets.left);
		}
		Interval rightInterval = structure.getHorizontalIntervals().get(structure.getHorizontalIntervals().size() - 1);
		if(rightInterval.getLength() < insets.right) {
			rightInterval.moveEndTo(rightInterval.getStart() + insets.right);
		}
		
		structure.adjustNodePositions();
		zoom.zoom(node, node.isZoomedIn());
		
		informListeners(node);
		
	}
	
	
	/**
	 * This method moves all the nodes in the model by the vector given as 
	 * arguments dx and dy.
	 * 
	 * @param node an arbitrary node in the hierarchy
	 * @param dx the x part of the translation vector
	 * @param dy the y part of the translation vector
	 */
	private void adjustTopLevel(IZoomNode node, int dx, int dy) {
		
		firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
		
		IZoomNode parent = node;
		while(parent.getZoomParent() != null) {
			parent = parent.getZoomParent();
		}
		
		IntervalStructure structure = zoom.getIntervalStructureFor(parent);
		
		Interval hInterval = structure.getHorizontalIntervalAt(structure.getBounds().x);
		hInterval.moveTo(hInterval.getStart() + dx);
		
		Interval vInterval = structure.getVerticalIntervalAt(structure.getBounds().y);
		vInterval.moveTo(vInterval.getStart() + dy);
		
		structure.adjustNodePositions();
		
	}
	
	
	/**
	 * Returns a list of rectangles representing the horizontal intervals of
	 * the interval structure for the node given as argument. The coordinates
	 * of the rectangles are relative to the node given as argument.
	 * 
	 * @param container the node to get the horizontal intervals for
	 * @return list of rectangles representing the horizontal intervals
	 */
	public List<Rectangle> getHorizontalIntervals(IZoomNode container) {
		
		List<Rectangle> rects = new ArrayList<Rectangle>();
		IntervalStructure structure = zoom.getIntervalStructureFor(container);
		if(structure != null) {
			Iterator<Interval> iter = structure.getHorizontalIntervals().iterator();
			while(iter.hasNext()) {
				Interval interval = iter.next();

				Rectangle rect = new Rectangle(interval.getStart(), structure.getBounds().y, interval.getLength(), structure.getSize().height);

				rects.add(rect);
			}
		}
		
		return rects;
		
	}
	
	
	/**
	 * Returns a list of rectangles representing the vertical intervals of
	 * the interval structure for the node given as argument. The coordinates
	 * of the rectangles are relative to the node given as argument.
	 * 
	 * @param container the node to get the vertical intervals for
	 * @return list of rectangles representing the vertical intervals
	 */
	public List<Rectangle> getVerticalIntervals(IZoomNode container) {
		
		List<Rectangle> rects = new ArrayList<Rectangle>();
		IntervalStructure structure = zoom.getIntervalStructureFor(container);
		if(structure != null) {
			Iterator<Interval> iter = structure.getVerticalIntervals().iterator();
			while(iter.hasNext()) {
				Interval interval = iter.next();

				Rectangle rect = new Rectangle(structure.getBounds().x, interval.getStart(), structure.getSize().width, interval.getLength());

				rects.add(rect);
			}
		}
		
		return rects;
		
	}
	
	
	/**
	 * Returns a list of rectangles representing the zoom holes of
	 * the interval structure for the node given as argument. The coordinates
	 * of the rectangles are relative to the node given as argument.
	 * 
	 * @param container the node to get the zoom holes for
	 * @return list of rectangles representing the zoom holes
	 */
	public List<Rectangle> getZoomHoles(IZoomNode container) {
		
		List<Rectangle> rects = new ArrayList<Rectangle>();
		IntervalStructure structure = zoom.getIntervalStructureFor(container);
		if(structure != null) {
			Iterator<ZoomHole> iter = structure.getZoomHoles();
			while(iter.hasNext()) {
				ZoomHole hole = iter.next();
				Rectangle rect = new Rectangle(hole.getBounds().x, hole.getBounds().y, hole.getBounds().width, hole.getBounds().height);

				rects.add(rect);
			}
		}
		
		return rects;
		
	}
	
	
	/**
	 * Inform all the listeners that the interval structures of the node given as
	 * argument and its zoom holes changed.
	 * 
	 * @param changedNode the node for which the structure changed.
	 */
	private void informListeners(IZoomNode changedNode) {
		
		IZoomNode node = changedNode.getZoomParent();
		
		// Inform listeners about the auxiliary figures (intervals and zoom holes)
		// We have to do that after the recursion returned because the coordinates
		// of the figures are relative to the parent node and the parent node's
		// location may change during the recursive zoom.
		while(node != null) {
		
		
			firePropertyChange(INTERVALS_HORIZONTAL, null, node);
			firePropertyChange(INTERVALS_VERTICAL, null, node);
			firePropertyChange(ZOOM_HOLES, null, node);
			
			node = node.getZoomParent();
			
		}
	
	}
	
	
	/**
	 * Returns a rectangle that represents the absolute bounds of the node
	 * given as argument.
	 * 
	 * TODO: Include in Interface or in utility
	 * 
	 * @param node the node to get the absolute bounds of
	 * @return the absolute bounds of the node given as argument
	 */
	private Rectangle getAbsoluteBounds(IZoomNode node) {
		
		Rectangle bounds = node.getZoomBounds();
		while(node.getZoomParent() != null) {
			node = node.getZoomParent();
			bounds.translate(node.getZoomBounds().x, node.getZoomBounds().y);
		}
		
		return bounds;
	}
	
	
	/**
	 * Allocates the white space indicated by the argument <i>spaceBounds</i>
	 * inside the node given as argument <i>parent</i>. The coordinates of the
	 * <i>spaceBounds</i> have to be relative to the <i>parent</i> node.
	 * 
	 * @param parent the node to allocate the white space in
	 * @param spaceBounds the bounds of the requested white space
	 */
	public void allocateWhiteSpace(IZoomNode parent, Rectangle spaceBounds) {
		
		DummyNode dummy = new DummyNode(parent, spaceBounds);
		
		insertNode(dummy);
		
		// Remove the dummy node from the interval structure
		IntervalStructure structure = parent.getZoomStructure();
		structure.removeNode(dummy);

	}
	
	
	public void setScaleGapIntervals(boolean scale) {
		scaleGaps = scale;
	}

}
