package ch.unizh.ifi.adora.zoom.adora;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Iterator;

import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.model.representation.zoom.IntervalStructure;

/**
 * A simple IZoomNode implementation that is used for layout operations for
 * which the zoom algorithm requires a node but which don't have a node itself.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-10-22 TR First Version
 * @version				$Date: 2007-10-23 11:24:05 +0200 (Tue, 23 Oct 2007) $, $Author: treinhard $, $Revision: 3157 $
 *
 */
public class DummyNode implements IZoomNode {

	/**
	 * Serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The parent IZoomNode (i.e. the node this node is contained in)
	 */
	private IZoomNode parent;
	
	/**
	 * The bounds of this node
	 */
	private Rectangle bounds;
	
	/**
	 * Stores the interval structure of this node
	 */
	private IntervalStructure structure;
	
	
	/**
	 * Constructor
	 * 
	 * @param parent the parent IZoomNode
	 * @param bounds the bounds of this node
	 */
	public DummyNode(IZoomNode parent, Rectangle bounds) {
		
		this.parent = parent;
		this.bounds = bounds;
		
	}
	
	
	/**
	 * Returns the bounds of this node. The coordinates of the bounds have to
	 * be relative to the parent of this node.
	 * 
	 * @return the relative bounds of the node
	 */
	public Rectangle getZoomBounds() {
		
		return new Rectangle(bounds);
		
	}

	
	/**
	 * Returns an iterator over an empty collection because a DummyNode doesn't 
	 * have any children.
	 * 
	 * @return iterator over an empty collection
	 */
	public Iterator<IZoomNode> getZoomChildren() {
		
		return new ArrayList<IZoomNode>(0).iterator();
		
	}

	
	/**
	 * Returns null because this feature is not used for the DummyNode.
	 * 
	 * @return null
	 */
	public Dimension getZoomMinimalSize() {
		
		return null;
		
	}

	
	/**
	 * Returns the parent of this node.
	 * 
	 * @return the parent node
	 */
	public IZoomNode getZoomParent() {

		return parent;
		
	}

	
	/**
	 * Returns the interval structure this nodes children are represented in.
	 * The interval structure can be serialized.
	 * 
	 * @return the interval structure.
	 */
	public IntervalStructure getZoomStructure() {

		if(structure == null) {
			structure = new IntervalStructure(this);
		}
		
		return structure;
		
	}

	
	/**
	 * Returns always true because the DummyNode can't be zoomed in or out.
	 * 
	 * @return true
	 */
	public boolean isZoomedIn() {

		return true;
		
	}

	
	/**
	 * Called by the zoom algorithm to adjust the position of this node
	 * after a zoom-in or zoom-out. The position is relative to the parent
	 * of this node.
	 * 
	 * @param p the new relative location
	 */
	public void setZoomLocation(Point p) {
		
		bounds.setLocation(p);

	}

	
	/**
	 * Doesn't do anything.
	 * 
	 * @param minSize ignored
	 */
	public void setZoomMinimalSize(Dimension minSize) {
		

	}

	
	/**
	 * Called by the zoom algorithm to adjust the size of a zoomed-in or
	 * zoomed-out node.
	 * 
	 * @param d the new size
	 */
	public void setZoomSize(Dimension d) {
		
		bounds.setSize(d);

	}

}
