package ch.unizh.ifi.adora.zoom.berner.algorithm;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.zoom.berner.geometry.BufferZoneRectangle;
import ch.unizh.ifi.adora.zoom.berner.geometry.ConstructionLine;
import ch.unizh.ifi.adora.zoom.berner.geometry.ConstructionRectangle;
import ch.unizh.ifi.adora.zoom.berner.geometry.LayoutUtils;
import ch.unizh.ifi.adora.zoom.ZoomEventConstants;

/**
 * The class LayoutAlgorithm includes mainly the functionality of the zooming
 * algorithm but also related functions which modify the shape of objects. The basic
 * target of LayoutAlgorithm is to keep the model in an occlusion-free state.<br>
 * Usually, one of the following functionalities is used:
 * <ul>
 * 	<li>
 * 		Adjusting of nodes. ZoomableNodeContainers are reshaped so that all 
 * 		visible children fit into it.
 * 	</li>
 * 	<li>
 * 		Reshaping. This allows to change the size of nodes without causing any
 * 		occlusions. Afterwards, parent containers are adjusted. <br>
 * 		Reshaping is also used by the commands to move the siblings of a node when 
 * 		changing the visibility of a node or inserting/removing a node into/from a 
 * 		parent.
 * 	</li>
 * 	<li>
 * 		Zooming. Sets size of a node properly (making it bigger or smaller depending
 * 		on if it's a zoom in or a zoom out) by reshaping it.
 * 	<li>
 * </ul>
 * Note that the reshape and adjust methods will call each other until the parent of
 * a reshaped node is not a ZoomableNodeContainer.
 * <p>
 * The method moveNodesInSameLevel is an abstract method that has to be
 * implemented by a specific layout algorithm. This means that the way elements
 * are beeing moved after calling a layout function is defined in another
 * class.
 * <p>
 * The LayoutAlgorithm class fires events, such as OCCLUSION, which are then
 * used by the LayoutVisualizationLayer to visualize the process of layouting.
 * <p>
 * Currently, an occlusion free state also means that invisible objects are not
 * occluded either. This has been introduced in order to avoid problems with the
 * {@link ch.unizh.ifi.adora.representation.management.ViewManager}, which appeared
 * when several objects changed their visibility state at once. However, this is 
 * not desirable, since chains of invisible nodes can influence nodes that are far
 * away from relayouted nodes.
 * 
 * @author Christian Seybold
 * @history ... 26.08.02 MM don't solve occlusions with invisible objects
 *          29.08.02 MM if an element is unvisible, the method adjustShape
 *          should change its stored size instead of reshaping the element.
 *          adjustShape will not resize the top element anymore, instead,
 *          normalize is called 01.09.02 MM changes to solve aspect
 *          visualisation problems 20.11.02 CS Introduction of HierarchyObject
 *          22.11.02 CS method-access more restricted moved to layout.algorithm
 *          package 02.12.02 CS Bug 101: layout algorithm restructered 10.04.04
 *          CS syntactical reformating 25.04.03 CS introduced reshape with
 *          spaceShape parameter for free border 30.10.05 SM Adapted version for
 *          ADORA Eclipse. 02.12.05 SM Comments, additional methods.
 *          10.05.06 JS made LayoutAlgorithm an ObservableModelElement, fires 
 *                      property change events for visualization now
 *          24.05.06 JS adapted zoom method to have at least the size required by a 
 *                      node's children
 *          31.05.06 JS split some methods into a public X() and a protected doX() 
 *                      method. The class itself calls doX(), while outside callers
 *                      use X() (helps visualization)
 *          11.08.06 JS replaced literals with node.getDefaultSize()
 *          14.08.06 JS Bug 553: When zooming, the size of the container is
 *                      calculated dynamically instead of using the stored size.
 *          15.08.06 JS Added a border of MIN_DISTANCE around the children of 
 *                      zoomed containers
 *          25.08.06 JS Reimplemented the above enhancement, required a border 
 *                      around nodes in the solveOcclusions(Node)
 *          14.09.06 JS Updated documentation, renamed moveDirectChilds(...) to 
 *          			moveDirectChildren(...)
 *                      
 * @version $Date: 2007-11-28 11:20:34 +0100 (Wed, 28 Nov 2007) $, $Author: treinhard $, $Revision: 3333 $
 * 
 */
public abstract class LayoutAlgorithm extends ObservableModelElement implements ZoomEventConstants {
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(LayoutAlgorithm.class
			.getName());
	
	/**
	 * The minimal distance between two elements after solving an occlusion.
	 * This constant is used by several layout algorithms
	 */
	protected final static int MIN_DISTANCE = 10;

	/**
	 * Applies the fisheye zoom layout algorithm. If the container is zoomed in
	 * it is zoomed out and vice-versa.
	 * 
	 * @see #doZoom(ZoomableNodeContainer, boolean)
	 * 
	 * @param container
	 *            The container to zoom.
	 * @pre container != null
	 */
	public void zoom(ZoomableNodeContainer container) {
        container.setZoomedIn(!container.isZoomedIn());
        zoom(container, container.isZoomedIn());
    }
    
	/**
	 * Applies the fisheye zoom layout algorithm.
	 * 
	 * @see #doZoom(ZoomableNodeContainer, boolean)
	 * 
	 * @pre container != null
	 * @param zoomIn
	 *            true means zoom in.
	 * @param container
	 *            The container to zoom.
	 * @throws IllegalArgumentException
	 *             if the container should be zoomed out but it has no children.
	 */
	public void zoom(ZoomableNodeContainer container, boolean zoomIn) {
		if (!zoomIn && !container.hasChildren())
			throw new IllegalArgumentException(
					"zoom out only possible when children exist");
        
        firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
		doZoom(container, zoomIn);
	}
	
	/**
	 * Implementation for {@link #zoom(ZoomableNodeContainer, boolean)}.
	 * 
	 * The new size is computed so that all children fit into the container. The 
	 * method makes sure that zooming in doesn't shrink the container, and zooming 
	 * out can't make it larger. 
	 * 
	 * @pre container != null
	 * @param container
	 *            The container to zoom.
	 * @param zoomIn
	 *            true means zoom in.
	 */
	protected void doZoom(ZoomableNodeContainer container, boolean zoomIn) {
		// apply stored size and store current size
		Dimension repSize = container.getSize();
		Dimension newSize = container.getDefaultSize();// container.getStoredSize() is used
													   // here for non-dynamical zooming
		Rectangle minRect = calcChildrenBounds(container);
		container.setStoredSize(repSize);

		if (logger.isLoggable(Level.FINE)) {
			logger
					.fine("Zooming container="
							+ container + ", repSize=" + 
							repSize + ",storedSize=" + newSize + 
							",minRect=" + minRect + ",location=" + 
							container.getLocation());
		}
		
		// smart resizing
		// if no size was stored, calculate new size
		if (newSize.width == 0 || // not initialized
				(!zoomIn && newSize.width >= repSize.width)) // or too
			// large
			newSize.width = repSize.width;;
		if (newSize.height == 0 || // not initialized
				(!zoomIn && newSize.height >= repSize.height)) // or too
			// large
			newSize.height = repSize.height;;
		if (zoomIn) { // not smaller than unzoomed state
			if (newSize.width < repSize.width)
				newSize.width = repSize.width;
			if (newSize.height < repSize.height)
				newSize.height = repSize.height;
		}
		if (newSize.width < minRect.width) {
			newSize.width = minRect.width;
		}
		if (newSize.height < minRect.height) {
			newSize.height = minRect.height;
		}
		
		// Size is ok now, but move children if they're not inside the 
		// provided space
		Point minMove = new Point(0,0);
		if(newSize.width<minRect.x+minRect.width) {
			minMove.x = newSize.width - (minRect.x + minRect.width);
		}
		else if(minRect.x<0) {
			minMove.x = -minRect.x;
		}
		if(newSize.height<minRect.y+minRect.height) {
			minMove.y = newSize.height - (minRect.y + minRect.height);
		}
		else if(minRect.y<0) {
			minMove.y = -minRect.y;
		}
		if(!minMove.equals(new Point(0,0))) {
			moveDirectChildren(container, minMove);
		}
		
		Point newLoc = LayoutUtils.calcCenteredLocation(
				container.getLocation(), repSize, newSize);

/*        if(zoomIn){
           adjustZoomedNode(container);
        }
        else{
  */  
            //apply new shape (before moving brothers to avoid overlapping)
            doReshape(container, new Rectangle(newLoc, newSize), zoomIn);
    //    }
        //Resize parent node if zoomed in.
        //Can't be implemented statically, since the children bounds can change
        //due to the view management.
        
		if (logger.isLoggable(Level.FINE)) {
			logger
					.fine("Zoomed container="
							+ container + ", Size=" + 
							container.getSize() + ",calculatedNewLoc=" + newLoc + 
							",actualLocation=" + 
							container.getLocation());
		}
	}


    
    /**
     * Returns the space occupied by the children of the passed Container.
     * This space is calculated dynamically, considering the border of the 
     * Container. If there are visible children, free space of MIN_DISTANCE is
     * added around them.
     * 
     * @pre       container != null
     * @post      true
     * @obligation 
     * @param container Node the childsize of which has to be calculated   
     * @return Bounds of the children area.
     */
   public Rectangle calcChildrenBounds(INodeContainer container) {
        final List c = container.getChildren();
        Rectangle bounds = null;

        for (int i = 0; i < c.size(); i++) {
            Node nc = (Node) c.get(i);
            if (nc.isVisible()) {
                if (bounds == null) {
                    bounds = nc.getBounds();
                } else {
                    bounds = this.union(bounds, nc.getBounds());
                }
            }
        }
        if (bounds == null) {
            bounds = new Rectangle();
        } else {
            bounds.width += container.getBorderDimension().width;
            bounds.height +=container.getBorderDimension().height;  
            bounds.expand(MIN_DISTANCE, MIN_DISTANCE);
        }
        return bounds;
    }
   
	/**
	 * Returns a Rectangle that contains both Rectangle a and Rectangle b.
	 * 
	 * @pre a != null && b != null
	 * @param a The first Rectangle contained in the returned Rectangle
	 * @param b The second Rectangle contained in the returned Rectangle
	 * @return A Rectangle large enough to contain a and b
	 */
	public Rectangle union(Rectangle a, Rectangle b) {
		int x1 = Math.min(a.x, b.x);
		int x2 = Math.max(a.x + a.width, b.x + b.width);
		int y1 = Math.min(a.y, b.y);
		int y2 = Math.max(a.y + a.height, b.y + b.height);

		return new Rectangle(x1, y1, x2 - x1, y2 - y1);
	}

	/**
	 * Adjusts the shape (location and size) of this element to the required
	 * space of the included childs when necessary. <br/>
     * The new shapes are calculated recursively for all parents up to the top 
     * level node. <br/>
	 * Reshaping is only done when necessary, that means either a new location
	 * or a new size (or both) become necessary for an object. A new location
	 * becomes necessary when an inner object overlaps the left or top border
	 * (incl. border). Then, the location is adjusted, the size enlarged about
	 * this amount and the inner objects are moved in the opposite direction, so
	 * that it seems as they would stay at the same place. The size changes
	 * either when the location changes or inner objects are overlapping the
	 * right or bottom border.
	 * 
	 * @pre node != null
	 * @param node ZoomableNodeContainer the size of which must be adapted
	 */
	public void adjustNode(ZoomableNodeContainer node) {
		firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
		doAdjustNode(node);
	}
	
	/**
	 * Implementation for {@link #adjustNode(ZoomableNodeContainer)}.
	 * 
	 * @pre node != null
	 * @param node ZoomableNodeContainer the size of which must be adapted
	 */
	protected void doAdjustNode(ZoomableNodeContainer node) {
		Rectangle reqShape = calcChildrenBounds(node);

		Rectangle cBounds = node.getStoredChildrenBounds();
		if (!reqShape.getLocation().equals(cBounds.getLocation())) {
			Point vector = new Point(cBounds.x - reqShape.getLocation().x,
					cBounds.y - reqShape.getLocation().y);
			moveDirectChildren(node, vector);
		}
        
		if (!reqShape.getSize().equals(cBounds.getSize())) {
			int diffWidth = node.getSize().width - cBounds.width;
			int diffHeight = node.getSize().height - cBounds.height;
			Dimension newSize = new Dimension(reqShape.width + diffWidth,
					reqShape.height + diffHeight);
         
			if(newSize.width < node.getDefaultSize().width){
                newSize.width = node.getDefaultSize().width;
            }
            if(newSize.height< node.getDefaultSize().height){
                newSize.height= node.getDefaultSize().height;
            }
            
  			Point newLoc = LayoutUtils.calcCenteredLocation(node.getLocation(),
					node.getSize(), newSize);
			doReshape(node, new Rectangle(newLoc, newSize), false);
		}
	}
    
    /**
	 * Solves any occlusion on the same level as the specified element with the
	 * help of the half plane shift method. This method is looking for
	 * occlusions by checking one element after the other. If an occlusion is
	 * found, it will directly be solved. An occlusion is solved by dividing
	 * the plane along an axis that splits the occlusion in two equal parts
	 * along the smaller side of the occlusion rectangle. All elements that are
	 * not in the half plane of the specified element will be shifted. The shift
	 * vector is perpendicular to the splitting axis and has a length equal to
	 * the smaller dimension of the occlusion rectangle.<b>
	 * This method can be used to make sure that a subclassing algorithm that 
	 * doesn't guarantee to result in an occlusion free state still ends in a
	 * valid model. This method doesn't adjust the parent of node, though.
	 * 
	 * @pre node != null
	 * @param node the Node the siblings of which must be checked for occlusions
	 */
	protected void solveOcclusions(Node node) {
		ConstructionRectangle referenceShape = new ConstructionRectangle(node
				.getBounds());

		List siblings = node.getSiblings();

		for (int i = 0; i < siblings.size(); i++) {
			for (int j = 0; j < siblings.size(); j++) {
				Node nodei = (Node) siblings.get(i);
				Node nodej = (Node) siblings.get(j);
				if ((nodei != nodej) /*&& nodei.isVisible() && nodej.isVisible()*/) {
					ConstructionRectangle shape = new ConstructionRectangle(
							nodei.getBounds());
					ConstructionRectangle currShape = new BufferZoneRectangle(
							nodej.getBounds(), MIN_DISTANCE);
					ConstructionRectangle occlusion = shape
							.getOcclusion(currShape);
					if (occlusion != null) {
                        // tell visualisation that an occlusion has been found
						firePropertyChange(OCCLUSION, null, getAbsoluteRectangle(occlusion, node.getParent()));

						Point p1 = occlusion.getCenter();
						Point p2 = occlusion.getCenter();
						ConstructionLine division;
						ConstructionRectangle extendedOcclusion = shape
								.getExtendedOcclusion(currShape);
                        
						// vertical plane division
						if (extendedOcclusion.width < extendedOcclusion.height) {
							p1.translate(0, -(occlusion.height));
							p2.translate(0, occlusion.height);
							division = new ConstructionLine(p1, p2);
							int referencePos = division.outcode(referenceShape
									.getCenter());
							int thisPos = division.outcode(shape.getCenter());
							int otherPos = division.outcode(currShape
									.getCenter());
							if (referencePos == ConstructionLine.RIGHT) {
								Point vector = new Point(
										-extendedOcclusion.width - MIN_DISTANCE,
										0);
								moveHalfPlaneBy(siblings, division,
										ConstructionLine.LEFT, vector);
								if ((thisPos == 0)
										&& (otherPos != ConstructionLine.LEFT)) {
									nodei.moveBy(vector);
								} else if ((thisPos == ConstructionLine.RIGHT)
										&& (otherPos == 0)) {
									nodej.moveBy(vector);
								}
							} else {
								Point vector = new Point(
										extendedOcclusion.width + MIN_DISTANCE,
										0);
								moveHalfPlaneBy(siblings, division,
										ConstructionLine.RIGHT, vector);
								if ((thisPos == 0)
										&& (otherPos != ConstructionLine.RIGHT)) {
									nodei.moveBy(vector);
								} else if ((thisPos == ConstructionLine.LEFT)
										&& (otherPos == 0)) {
									nodej.moveBy(vector);
								}
							}
						}
						// horizontal plane division
						else {
							p1.translate(-(occlusion.width), 0);
							p2.translate(occlusion.width, 0);
							division = new ConstructionLine(p1, p2);
							int referencePos = division.outcode(referenceShape
									.getCenter());
							int thisPos = division.outcode(shape.getCenter());
							int otherPos = division.outcode(currShape
									.getCenter());
							if (referencePos == ConstructionLine.BOTTOM) {
								Point vector = new Point(0,
										-extendedOcclusion.height
												- MIN_DISTANCE);
								moveHalfPlaneBy(siblings, division,
										ConstructionLine.TOP, vector);
								if ((thisPos == 0)
										&& (otherPos != ConstructionLine.TOP)) {
									nodei.moveBy(vector);
								} else if ((thisPos == ConstructionLine.BOTTOM)
										&& (otherPos == 0)) {
									nodej.moveBy(vector);
								}
							} else {
								Point vector = new Point(0,
										extendedOcclusion.height + MIN_DISTANCE);
								moveHalfPlaneBy(siblings, division,
										ConstructionLine.BOTTOM, vector);
								if ((thisPos == 0)
										&& (otherPos != ConstructionLine.BOTTOM)) {
									nodei.moveBy(vector);
								} else if ((thisPos == ConstructionLine.TOP)
										&& (otherPos == 0)) {
									nodej.moveBy(vector);
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Moves all given elements that are located on the specified half plane by
	 * a given vector. The half plane is given by a ConstructionLine that
	 * divides the plane, and the concerned half plane is the one given by an
	 * integer bit mask of the ConstructionLine. The shift vector is specified
	 * by the Point's coordinates.
	 * 
	 * @param nodes
	 *            The concerned elements
	 * @param division
	 *            The ConstructionLine that parts the plane
	 * @param pos
	 *            The ConstructionLine's bit mask defining which is the
	 *            concerned half plane
	 * @param vector
	 *            The vector to shift the elements by
	 */
	private void moveHalfPlaneBy(List nodes, ConstructionLine division,
			int pos, Point vector) {
		Node node;
		for (int i = 0; i < nodes.size(); i++) {
			node = (Node) nodes.get(i);
			ConstructionRectangle shape = new ConstructionRectangle(node
					.getBounds());
			if (division.outcode(shape.getCenter()) == pos) {
				node.moveBy(vector);
			}
		}
	}

	/**
	 * Changes the shape from the current rectangle to the given new Rectangle
	 * without any further checks of the necessity. 
	 * The subclass that implements LayoutAlgorithm decides how to move the sibling 
	 * nodes of <code>node</code>. This is done either before or after the new 
	 * shape is applied, depending on whether the new shape is larger or smaller 
	 * than the current one (specified in moveFirst) to avoid temporary overlapping 
	 * of nodes. <br/> 
	 * Just in case the implementing subclass doesn't guarantee an occlusion free 
	 * model, {@link #solveOcclusions(Node)} is called. After this, the parent node 
	 * is adjusted, which can lead to further reshapes.<br/> 
	 * This method is used for zooming but also when resizing becomes
	 * necessary because of new nodes. <br/> 
	 * SpaceShape specifies the area around newShape that is kept free. So
	 * spaceShape is at least as large as newShape. If they have the same size,
	 * then no free border is available around the object.
	 * 
	 * @pre node != null && newShape != null && spaceShape != null
	 * @pre spaceShape.width >= newShape.width && spaceShape.height >=
	 * 	     newShape.height
	 * @param node the node to resize/relocate
	 * @param newShape the new shape of node
	 * @param spaceShape a rectangle that specifies the free space around node
	 * @param moveFirst specifies if the siblings of node should be moved before or
	 *                  after resizing
	 */
	public void reshape(Node node, Rectangle newShape, Rectangle spaceShape,
			boolean moveFirst) {
        
        firePropertyChange(NEW_LAYOUT_OPERATION, null, null);
        
        doReshape(node, newShape, spaceShape, moveFirst);
	}
	
	/**
	 * Implementation for {@link #reshape(Node, Rectangle, Rectangle, boolean)}
	 */
	protected void doReshape(Node node, Rectangle newShape, Rectangle spaceShape,
			boolean moveFirst) {
        
		// store the present children bounds before reshaping
		// Don't try to store the bounds if the parent node doesn't exist
		// anymore (i.e. multiple selected nodes have been deleted)
		if(node.getParent() != null) {
			Rectangle rect = this.calcChildrenBounds(node.getParent());
			node.getParent().setStoredChildrenBounds(rect);
		}

		Rectangle oldShape = node.getBounds();

		// move brother nodes before ...
		if (moveFirst)
			moveNodesInSameLevel(node, oldShape, spaceShape);

		node.setLocation(newShape.getLocation());
        Dimension size = newShape.getSize();
        node.setSize(size);

		// ... or move after changes habe been applied
		if (!moveFirst)
			moveNodesInSameLevel(node, oldShape, spaceShape);
		solveOcclusions(node);

		final INodeContainer nodeParent = node.getParent();
		if (nodeParent instanceof ZoomableNodeContainer) {
			doAdjustNode((ZoomableNodeContainer) nodeParent);
		}
	}

	/**
	 * For this reshape, it is assumed that newShape and spaceShape have the
	 * same size.
	 */
	public void reshape(Node node, Rectangle newShape, boolean moveFirst) {
		reshape(node, newShape, new Rectangle(newShape), moveFirst);
	}
	
	/**
	 * Implementation for {@link #reshape(Node, Rectangle, boolean)}
	 */
	protected void doReshape(Node node, Rectangle newShape, boolean moveFirst) {
		doReshape(node, newShape, new Rectangle(newShape), moveFirst);
	}

	/**
	 * This method will calculate, for all neighboured elements (means same
	 * parent and on the same level), how to shift and actually shift this
	 * elements if deemed necessary. The original and the new shape of this 
	 * element are needed for the calculation(org and newShape). This method is 
	 * used for reshaping after a zoom was applied for example.
	 * 
	 * @pre n != null && orgShape != null && zoomed != null
	 * @param n the Node that changes its bounds
	 * @param orgShape the original shape of n
	 * @param newShape the new shape of n
	 */
	protected abstract void moveNodesInSameLevel(Node n, Rectangle orgShape,
			Rectangle newShape);

	/**
	 * Moves all child nodes excluding their children by the given amount in
	 * the vector. This is needed for resizing in the upper left corner.
	 * 
	 * @pre node != null && vector != null
	 */
	protected void moveDirectChildren(ZoomableNodeContainer node, Point vector) {
		List c = node.getChildren();
		Node n;
		for (int i = 0; i < c.size(); i++) {
			n = (Node) c.get(i);
            Rectangle originalPos = getAbsoluteRectangle(n.getBounds(),node);
			n.moveBy(vector);
            firePropertyChange(CHILD_POSITION_CHANGE, originalPos, getAbsoluteRectangle(n.getBounds(), node));
		}
	}
    
    /**
     * Calculates the bounds of a Container's children.
     * Used before ScenarioZoom, TreeView and View Management operations.
     * 
     * @pre       container != null
     * @post      true
     * @param container the ZoomableNodeContainer the children bounds of which
     *                  should be calculated
     */
    public void calculateStoredChildrenBounds(ZoomableNodeContainer container){
        Rectangle r = calcChildrenBounds(container);
        
        //Concern lower right corner.
//        r.width  = container.getSize().width  - (r.x + r.width);
        //r.height = container.getSize().height - (r.y + r.height);
        
        container.setStoredChildrenBounds(r);
    }

    /**
     * Convenience method to transform a Rectangle positioned relative to its
     * parent to its absolute position.
     * 
     * @param rect a Rectangle with a position relative to <code>parent</code>
     * @param parent the parent to which the position of <code>rect</code> is relative
     * @return a Rectangle of the same size, but with the absolute position
     */
    protected Rectangle getAbsoluteRectangle(Rectangle rect, INodeContainer parent)
    {
        Rectangle absoluteRectangle = new Rectangle(rect);
        if(parent instanceof ZoomableNodeContainer)
        {
            absoluteRectangle.translate(((ZoomableNodeContainer) parent).getAbsoluteLocation());
        }
        return absoluteRectangle;
    }
 
	/**
	 * Shifts all children by the same vector so that none of them have negative
	 * coordinates. No shift is performed if no children have negative 
	 * coordinates.
	 * 
	 * @pre       nc != null
     * @post      true
     * @param nc ZoomableNodeContainer the children of which should be moved
	 */
	public void normalize(ZoomableNodeContainer nc) {
		Rectangle reqShape = calcChildrenBounds(nc);
		reqShape.translate(nc.getLocation().x, nc.getLocation().y);
		Rectangle newShape = union(reqShape, nc.getBounds());
		Point newLoc = newShape.getLocation();
		// test whether changing of the location necessary
		if (!newLoc.equals(nc.getLocation())) {
			Point vector = new Point(nc.getLocation().x - newLoc.x, nc
					.getLocation().y
					- newLoc.y);

			// change the coordinates of direct children
			moveDirectChildren(nc, vector);
		}
	}
	
	
	/**
	 * Method to handle the insertion of a node.
	 * 
	 * @param node the node that has been added
	 */
	public abstract void insertNode(IZoomNode node);
	
	
	/**
	 * Method to handle the removal of a node.
	 * 
	 * @param node the node that has been removed.
	 */
	public abstract void removeNode(IZoomNode node);
	
	
	/**
	 * Method to handle the movement/resizing of a node;
	 * 
	 * @param node the node that is moved or resized
	 * @param newBounds the new bounds of the node
	 */
	public abstract void changeNodeBounds(IZoomNode node, java.awt.Rectangle newBounds);
	
	
	/**
	 * Method to handle the hidding (i.e. filtering out) of a node.
	 * 
	 * @param node the node that should be hidden
	 */
	public abstract void filterNode(IZoomNode node);
	
	
	/**
	 * Allocates the white space indicated by the argument <i>spaceBounds</i>
	 * inside the node given as argument <i>parent</i>. The coordinates of the
	 * <i>spaceBounds</i> have to be relative to the <i>parent</i> node.
	 * 
	 * @param parent the node to allocate the white space in
	 * @param spaceBounds the bounds of the requested white space
	 */
	public abstract void allocateWhiteSpace(IZoomNode parent, java.awt.Rectangle spaceBounds);
	
}
