package ch.unizh.ifi.adora.zoom.berner.algorithm;

import java.util.List;
import java.util.Vector;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.zoom.berner.geometry.BufferZoneRectangle;
import ch.unizh.ifi.adora.zoom.berner.geometry.ConstructionLine;
import ch.unizh.ifi.adora.zoom.berner.geometry.ConstructionRectangle;
import ch.unizh.ifi.adora.zoom.berner.geometry.ConstructionVector;
import ch.unizh.ifi.adora.zoom.berner.geometry.LayoutUtils;

/**
 * This class is an occlusion free implementation of the LayoutAlgorithm class (or
 * nearly, see note at {@link #moveNodesInSameLevel(Node, Rectangle, Rectangle)}).<br>
 * The algorithm moves the surrounding elements radially. In first place, only
 * the surrounding elements that collide with the zoomed element will be moved.
 * If the moved element collides with a further surrounding element, the same
 * shift vector will be applied to it. A cascade of occlusion checks and moves
 * will be triggered, until no new occlusion is found. This algorithm has the
 * particularity to move only the concerned elements by the smallest possible
 * distance, making use of the space between the elements. The space between the
 * elements will not be preserved. This LayoutAlgorithm is suitable for insert
 * layout operations. This class uses ConstructionObjects for geometrical
 * calculation.<br>
 * 
 * @author  CS
 * 
 * @history 15.08.02 MM Class created 
 *          25.08.02 MM Use methods getOcclusion and getExtendedOcclusion 
 *                      from ConstructionRectangle Overwrite method
 *                      adjustShape from LayoutAlgorithm Solved problem 
 *                      with endless loops by existing overlaps 
 *          02.12.02 CS Bug 101: layout algorithm restructered
 *          30.10.05 SM Adapted version for ADORA Eclipse.
 *          02.12.05 SM Default instance introduced. Removed some bugs.
 *          17.05.06 JS added firing of property change events for 
 *                      visualization
 *          31.05.06 JS split some methods into a public X() and a protected doX() 
 *                      method. The class itself calls doX(), while outside callers
 *                      use X() (helps visualization)
 *          07.06.06 JS introduced break condition for solveOcclusion method
 *          15.08.06 JS Bug 601: solveOcclusion uses a buffer of MIN_DISTANCE
 *                      around nodes tested for occlusions with their siblings
 *          17.08.06 JS Not requiring a MIN_DISTANCE between nodes anymore if one
 *                      of them is invisible.
 *                      Bug 717: Got rid of stack overflow in solveOcclusion(...)
 *                      caused by cyclic calls: The same node won't be moved twice by
 *                      the same recursion anymore.
 *          14.09.06 JS Changed moveNodesInSameLevel(...) so that occlusions are 
 *                      resolved in order of distance to the occluding node, but
 *                      left the sorting command commented out
 *          15.09.06 JS Added an intersection test to moveNodesInSameLevel(...) in
 *                      order to fix Bug 770, added/updated documentation
 * 
 * 
 * @version $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 * 
 * @see LayoutAlgorithm
 * @see util.geometry.ConstructionObject
 * 
 */
public class MinRadial extends LayoutAlgorithm {

	/**
	 * Stores the default instance of this layout algorithm.
	 */
	private static MinRadial INSTANCE = new MinRadial();
	
	/**
	 * Factor used to calculate the additional space held free around a node.
	 */
	private final static float SPACE_FACTOR = 0.1f;
	
	/**
	 * Returns the default instance of the min radial layout algorithm.
	 * @pre true
	 * @post return != null
	 * @return The instance of the min radial algorithm.
	 */
	public static MinRadial getInstance() {
		return INSTANCE;
	}
	
    /**
     * Private constructor to prevent instantiations (Singleton)
     */
    private MinRadial() {
        
    }
    
	/**
	 * This implementation of LayoutAlgorithm shifts the siblings of the reshaped 
	 * Node only if an occlusion is caused by the new bounds. In this case, the 
	 * sibling is moved just outside of nc, then the new position is tested for new 
	 * occlusions. {@link #solveOcclusion(Node, ConstructionVector, Vector)} is 
	 * used to perform this test and to solve any new occlusions.<br>
	 * Note: If the new center of nc lies exactly on the center of a sibling, the
	 * occlusion is not solved. Occlusions that existed before the call to this
	 * method may or may not be solved, since the method assumes a valid state of
	 * the model.
	 * @see ch.unizh.ifi.adora.representation.layout.algorithm.LayoutAlgorithm#moveNodesInSameLevel(Node, Rectangle, Rectangle)
	 */
	protected void moveNodesInSameLevel(final Node nc, Rectangle org,
			Rectangle zoomed) {
        
		// the elements on the same level as the moved one
		List siblings = nc.getSiblings();

		// the zoomed element's old shape
		ConstructionRectangle oldRect = new ConstructionRectangle(org);
		// the zoomed element's new shape
		ConstructionRectangle newRect = new ConstructionRectangle(zoomed);
		// the center of the zoomed element
		Point center = oldRect.getCenter();
		
		Vector<Node> occludedNodes = new Vector<Node>();
		// check for occlusions with the zoomed element
		Node sibling;
		for (int i = 0; i < siblings.size(); i++) {
			sibling = (Node) siblings.get(i);
			if (nc != sibling) {
				// the shape of the element to check
				ConstructionRectangle brotherShape = new ConstructionRectangle(
						sibling.getBounds());
				// put a bufferzone around the element's shape to preserve a min
				// distance of MIN_DISTANCE
				BufferZoneRectangle bBrotherShape = new BufferZoneRectangle(
						brotherShape, MIN_DISTANCE);
				// if the zoomed element collides with the element to check (or
				// its bufferzone)
				if (bBrotherShape.intersects(newRect)) {
                    
					occludedNodes.add(sibling);
				}
			}
		}


		// Sort the occluded nodes by distance to nc, so if two of them are moved on
		// top of each other, the one that was farther away from the center of nc will
		// be moved first. Then, when the second node is moved, it will push the first node
		// away even more, so that the order of the nodes should be preserved. 
		Node[] occludedArray = new Node[occludedNodes.size()]; 
		occludedNodes.toArray(occludedArray);
		
		/* The following piece of code has been deactivated due to the leaping 
		 * problem (see bug 764)
		// a comparator which sorts nodes so that the center of the last element is 
		// nearest to the center of nc
		Comparator<Node> distanceComparator = new Comparator<Node>() {
			public int compare(Node n1, Node n2) {
				return n2.getBounds().getCenter().getDistance2(nc.getBounds().getCenter()) - n1.getBounds().getCenter().getDistance2(nc.getBounds().getCenter());
			}
		};
		Arrays.sort(occludedArray, distanceComparator);
		*/
		
		for(int i=0; i < occludedArray.length; ++i) {
			sibling = occludedArray[i];
			// the shape of the element to check
			ConstructionRectangle brotherShape = new ConstructionRectangle(
					sibling.getBounds());
			// put a bufferzone around the element's shape to preserve a min
			// distance of MIN_DISTANCE
			BufferZoneRectangle bBrotherShape = new BufferZoneRectangle(
					brotherShape, MIN_DISTANCE);
			
			// We have to test for intersection again, because solving one of the
			// previous occlusions may have solved the occlusion in the meantime
			if (bBrotherShape.intersects(newRect)) {
	            firePropertyChange(OCCLUSION, null, getAbsoluteRectangle(bBrotherShape.getOcclusion(newRect), nc.getParent())); // the rectangle drawn has now a buffer around it, but brotherShape.getOcclusion(newRect) might return null 
	            
				// the extended occlusion is needed to find out the shortest
				// way to solve the occlusion
				ConstructionRectangle extendedOcclusion = bBrotherShape
						.getExtendedOcclusion(newRect);
				// the center of the element to check
				Point brotherCenter = brotherShape.getCenter();
				// the connection line between the zoomed element's center
				// and the center of the element to check
				ConstructionLine connection = new ConstructionLine(center, brotherCenter);
				// the normed vector of the connecting line has the
				// direction of
				// the shift vector
				ConstructionVector v = connection.getNormedVector();
				// find the shortest way out of the occlusion
				v.scaleToRelHeight(extendedOcclusion.height);
				double hLength = v.getLength();
				v.scaleToRelWidth(extendedOcclusion.width);
				double wLength = v.getLength();
				if (hLength < wLength) {
					v.scaleToRelHeight(extendedOcclusion.height);
				}
				// set the vector's anchor to the element to check's center
				v.setAnchor(brotherShape.getCenter());
	
				// move the checked element by the found vector
	            Rectangle originalPos = getAbsoluteRectangle(sibling.getBounds(), nc.getParent());
				sibling.moveBy(new Point((int) v.getWidth(), (int) v
						.getHeight()));
	            firePropertyChange(POSITION_CHANGE, originalPos, getAbsoluteRectangle(sibling.getBounds(), nc.getParent()));                   
	
				// check if the moved element causes new occlusions
	            Vector<Node> callers = new Vector<Node>();
	            callers.add(nc);
				solveOcclusion(sibling, v, callers);
			}
		}
	}

	/**
	 * Checks if the specified Node overlaps with any other element on the same
	 * level. If an occlusion is found, the overlapping element will be moved by
	 * the specified vector. If this is not possible because a zero vector is
	 * provided, nothing is done. If a moved node causes a further occlusion,
	 * the other node will be moved as well (recursively), but occlusions 
	 * between moved siblings that existed before will not be solved. 
	 * 
	 * @pre node != null && vector != null && callers != null
	 * @param node
	 *            The Node to check for occlusions
	 * @param vector
	 *            The vector to shift potential overlapping elements by
	 * @param callers
	 *            Vector containing the overlapping sibling node to avoid endless 
	 *            loop with existing overlaps (while recursing, callers will 
	 *            contain all nodes that have been moved so far)
	 */
	private void solveOcclusion(Node node, ConstructionVector vector,
			Vector<Node> callers) {
        // If vector is (0,0), no movement is performed anyway, so we can stop
		// here. This will prevent (all?) StackOverflowErrors (endless recursion) 
		// that can result from the unability of the algorithm to solve occlusions
		// when vector is (0,0)
		if(vector.getLength()==0) {
			return;
		}
		// the elements in the same level as the specified one
		List siblings = node.getSiblings();

		// the shape of the specified element
		ConstructionRectangle nodeShape = new ConstructionRectangle(node.getBounds());
		// search for overlapping brother elements
		Node sibling;
		for (int i = 0; i < siblings.size(); i++) {
			sibling = (Node) siblings.get(i);
			if (sibling != node) {
				// the shape of the brother element
				ConstructionRectangle brotherShape;
				// if both nodes are visible, a minimum distance is required, otherwise
				// preventing a normal occlusion is sufficient
				//if(node.isVisible() && sibling.isVisible()) {
					brotherShape = new BufferZoneRectangle(sibling.getBounds(), MIN_DISTANCE);
				/*}
				else {
					brotherShape = new ConstructionRectangle(sibling.getBounds());
				}*/
				// check for occlusion
				if ((brotherShape.intersects(nodeShape)) && !callers.contains(sibling)) {
                    
                    firePropertyChange(OCCLUSION, null, getAbsoluteRectangle(brotherShape.getOcclusion(nodeShape), callers.elementAt(0).getParent())); 
                    
					// make a copy of the specified vector
					ConstructionVector v = new ConstructionVector(vector
							.getWidth(), vector.getHeight());
					// set the vector's anchor point to the brother element's
					// center
					v.setAnchor(brotherShape.getCenter());
					// move the brother element as defined by the vector
                    Rectangle originalPos = getAbsoluteRectangle(sibling.getBounds(), node.getParent());
					sibling.moveBy(new Point((int) vector.getWidth(),
							(int) vector.getHeight()));
                    firePropertyChange(POSITION_CHANGE, originalPos, getAbsoluteRectangle(sibling.getBounds(), node.getParent()));
					// recursive check for further caused occlusions
                    callers.add(node);
					solveOcclusion(sibling, vector, callers);
				}
			}
		}
	}

	/**
	 * Overwrites {@link ch.unizh.ifi.adora.representation.layout.algorithm.LayoutAlgorithm#adjustNode(ZoomableNodeContainer)}
	 * so no event is fired.
	 */
	public void adjustNode(ZoomableNodeContainer node) {
		doAdjustNode(node);
	}
	
	/**
	 * Implementation for {@link #adjustNode(ZoomableNodeContainer)}
	 */
	protected void doAdjustNode(ZoomableNodeContainer node) {
		
	    Rectangle reqShape = calcChildrenBounds(node);
		reqShape.translate(node.getLocation().x, node.getLocation().y);
		Rectangle newShape = reqShape.union(node.getBounds());
		if (!newShape.getLocation().equals(node.getLocation())) {
			Point vector = new Point(node.getLocation().x
					- newShape.getLocation().x, node.getLocation().y
					- newShape.getLocation().y);
			moveDirectChildren(node, vector);
		}
		if (!newShape.getSize().equals(node.getSize())) {
			newShape.setLocation(LayoutUtils.calcCenteredLocation(node
					.getLocation(), node.getSize(), newShape.getSize()));
			doReshape(node, newShape, true);
		}
	}

	/**
	 * Shifts the sourrounding siblings radialy away if a new node is inserted.
	 * 
	 * @param node the node that was inserted
	 */
	public void insertNode(IZoomNode node) {
		
		Node n = (Node)node;
		
	    // compute the newly needed space
	    Rectangle space = new Rectangle(n.getBounds());
	    space.expand((int)(space.width*SPACE_FACTOR), (int)(space.height*SPACE_FACTOR));
	    // make a radial shift of all surrounding siblings and their 
	    // ancestors.

		reshape(n, n.getBounds(), space, false);
		
	}

	/**
	 * Doesn't do anything because this algorithm isn't used as node removal
	 * layout algorithm at the moment.
	 * 
	 * @param node the node that was removed
	 */
	public void removeNode(IZoomNode node) {
		
		
		
	}
	
	
	/**
	 * Method to handle the movement/resizing of a node;
	 * 
	 * @param node the node that is moved or resized
	 * @param newBounds the new bounds of the node
	 */
	public void changeNodeBounds(IZoomNode node, java.awt.Rectangle newBounds) {
		
		java.awt.Rectangle oldBounds = node.getZoomBounds();

		if (!(oldBounds.getSize().equals(newBounds.getSize()))){
			node.setZoomSize(newBounds.getSize());
		}
		if (!(oldBounds.getLocation().equals(newBounds.getLocation()))){
			node.setZoomLocation(newBounds.getLocation());
		}

		Rectangle space = new Rectangle(newBounds.x, newBounds.y, newBounds.width, newBounds.height);
		space.expand((int)(space.width*SPACE_FACTOR), (int)(space.height*SPACE_FACTOR));

		Rectangle rect = new Rectangle(newBounds.x, newBounds.y, newBounds.width, newBounds.height);

		reshape((Node)node, rect, space, false);
		
	}
	
	
	/**
	 * Method to handle the hidding (i.e. filtering out) of a node.
	 * 
	 * @param node the node that should be hidden
	 */
	public void filterNode(IZoomNode node) {
		
		Node nodeToInsert = (Node)node;
		
		double SPACE_FACTOR=0.1;
        Point newLoc = LayoutUtils.calcCenteredLocation(nodeToInsert.getLocation(), nodeToInsert.getSize(), nodeToInsert.getStoredSize());

	    // compute the newly needed space
        Rectangle insertionBounds = new Rectangle(newLoc, nodeToInsert.getStoredSize());
	    Rectangle space = new Rectangle(insertionBounds);
	    
	    space.expand((int)(space.width*SPACE_FACTOR), (int)(space.height*SPACE_FACTOR));
	    // make a radial shift of all surrounding siblings and their 
	    // ancestors.
	    
	    reshape(nodeToInsert, insertionBounds, space, true);
		
	}

	/**
	 * Not implemented.
	 */
	public void allocateWhiteSpace(IZoomNode parent, java.awt.Rectangle spaceBounds) {
		
	}
}
