package ch.unizh.ifi.adora.zoom.berner.algorithm;

import java.util.HashMap;
import java.util.List;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.zoom.berner.geometry.BufferZoneRectangle;
import ch.unizh.ifi.adora.zoom.berner.geometry.ConstructionLine;
import ch.unizh.ifi.adora.zoom.berner.geometry.ConstructionRectangle;
import ch.unizh.ifi.adora.zoom.berner.geometry.ConstructionVector;
import ch.unizh.ifi.adora.zoom.berner.geometry.LayoutUtils;

/**
 * This class provides an occlusion free implementation of the LayoutAlgorithm 
 * class if the size of a node is increased, but will cause occlusions when 
 * shrinking nodes, as it will try to contract all surrounding nodes.<br>
 * The algorithm is the one defined in Berners Diss and enhanced in Michael Marty's
 * diploma thesis. This class uses ConstructionObjects for geometrical calculation.
 * 
 * @see LayoutAlgorithm
 * @see model.NodeRepresentation
 * @see util.geometry.ConstructionObject
 * 
 * @author Christian Seybold
 * @history 13.08.02 MM Class created 
 *          27.08.02 MM Make use of
 *                      ConstructionRectangle's getExtendedOcclusion method to handle
 *                      occlusions easier and correctly. Only check for occlusions on
 *                      visible objects. Occlusion with the zoomed object is an allowed
 *                      state as invisible objects can build occlusions 
 *          02.12.02 CS Bug 101: layout algorithm restructered
 *          30.10.05 SM Adapted version for ADORA Eclipse
 *          02.12.05 SM Default instance introduced.
 *          17.05.06 JS added firing of property change events for 
 *                      visualization
 *          17.08.06 JS A MIN_DISTANCE is required between nodes (this still doesn't 
 *                      work sometimes, though)
 *          24.08.06 JS Bug 732: solveOcclusion shouldn't ask the ber2dev HashMap for
 *                      values it doesn't contain
 *          25.08.06 JS Fixed yet another cause for null pointers
 *          15.09.06 JS Updated documentation 
 * 
 * @version $Date: 2007-10-23 11:24:05 +0200 (Tue, 23 Oct 2007) $, $Autho$, $Revision: 3157 $
 */
public class NewBerner extends LayoutAlgorithm {

	/**
	 * Direction constant for east that can be OR-ed
	 */
	private final static int IN_EAST = 1;
	/**
	 * Direction constant for north that can be OR-ed
	 */
	private final static int IN_NORTH = 2;
	/**
	 * Direction constant for west that can be OR-ed
	 */
	private final static int IN_WEST = 4;
	/**
	 * Direction constant for south that can be OR-ed
	 */
	private final static int IN_SOUTH = 8;
	/**
	 * Direction constant for west (used as an array index)
	 */
	private final static int WEST = 0;
	/**
	 * Direction constant for north (used as an array index)
	 */
	private final static int NORTH = 1;
	/**
	 * Direction constant for east (used as an array index)
	 */
	private final static int EAST = 2;
	/**
	 * Direction constant for south (used as an array index)
	 */
	private final static int SOUTH = 3;

	/**
	 * Maps each brother Node to its corresponding zone before it was moved.
	 */
	private final HashMap<Node, Integer> brotherZones = new HashMap<Node, Integer>();

	/**
	 * Maps each brother Node to their lateral component of the movement.
	 */
	private final HashMap<Node, Integer> ber2dev = new HashMap<Node, Integer>();

	/**
	 * The four delta values, which are the differencies between each side of
	 * the zoomed element and the corresponding side of the original shape.
	 */
	private int[] resizeDelta = new int[4];

	/**
	 * The focal point of the element's moves.
	 */
	private Point center;

    
    /**
     * Stores the default instance of this layout algorithm.
     */
    private static NewBerner INSTANCE = new NewBerner();
    
	/**
	 * Returns the default instance of the NewBerner layout algorithm.
	 * @pre true
	 * @post return != null
	 * @return The instance of this algorithm.
	 */
    public static NewBerner getInstance() {
        return INSTANCE;
    }
        
    /**
     * Protected constructor to prevent instantiations (Singleton)
     */
    protected NewBerner(){
        
    }

    
	/**
	 * Implements LayoutAlgorithm. All sibling nodes are moved depending on
	 * the zone they're in (east, west, north, south or a combination of two zones).
	 * Occlusions are prevented if the area of the node has increased, but not if
	 * the size has shrunk. <br>
	 * See Berner's dissertation or Michael Marty's diploma thesis for details.
	 * 
	 * @see ch.unizh.ifi.adora.representation.layout.algorithm.LayoutAlgorithm#moveNodesInSameLevel(Node, Rectangle, Rectangle)
	 */
	public void moveNodesInSameLevel(Node node, Rectangle orgShape,
			Rectangle zoomed) {
		// the old shape
		ConstructionRectangle oldRect = new ConstructionRectangle(orgShape);
		// the new shape
		ConstructionRectangle newRect = new ConstructionRectangle(zoomed);
		// the focus is the center of the old shape
		center = oldRect.getCenter();
		// the corners of the old shape
		Point[] cornersOld = oldRect.getCorners();
		// the corners of the new shape
		Point[] cornersNew = newRect.getCorners();
		// calculates the four delta values
		for (int i = 0; i < resizeDelta.length; i++) {
			resizeDelta[i] = cornersNew[i].x - cornersOld[i].x;
			resizeDelta[++i] = cornersNew[i].y - cornersOld[i].y;
		}
		// the four straight lines that divide the space in four half planes
		ConstructionLine[] line = new ConstructionLine[4];
		// init the four lines
		for (int i = 0; i < line.length; i++) {
			line[i] = new ConstructionLine(cornersOld[(i + 3) % 4], cornersOld[i]);
		}

		// find all nodes in the same level (the brothers).
		final List siblings = node.getSiblings();

		// move the nodes in the same level
		for (int i = 0; i < siblings.size(); i++) {
			final Node sibling = (Node) siblings.get(i);
			if (sibling != node) {
				// calc the position of any brother node
				Point siblingCenter = getRelCenterWithin(node.getParent(),sibling);
				int position = getPosition(oldRect, siblingCenter);
				
				// map the brother node to his position
				brotherZones.put(sibling, new Integer(position));
				// calc the shift vector
				ConstructionVector vector = calcShiftVector(sibling/*
																	 * ,
																	 * parentCons
																	 */);
				// move the brother object
				int xTranslation = (int) vector.getWidth();
				int yTranslation = (int) vector.getHeight();
				Point translationValues = new Point(xTranslation, yTranslation);
				Rectangle originalPos = getAbsoluteRectangle(sibling.getBounds(), node.getParent());
                sibling.moveBy(translationValues);
                firePropertyChange(POSITION_CHANGE, originalPos, getAbsoluteRectangle(sibling.getBounds(), node.getParent()));
			}
		}
		// solve the occlusions if it was a zoom in
		if ((oldRect.width * oldRect.height) < (newRect.width * newRect.height)) {
			// if the biggest growth is vertical
			if (resizeDelta[SOUTH] - resizeDelta[NORTH] > resizeDelta[EAST] - resizeDelta[WEST]) {
				// solve the occlusions by moving the objects that where in
				// the north and south
				solveOcclusion(IN_NORTH, siblings/* , parentCons */);
				solveOcclusion(IN_SOUTH, siblings/* , parentCons */);
			}
			// the biggest growth is horizontal
			else {
				// solve the occlusions by moving the objects that where in
				// the east and west
				solveOcclusion(IN_EAST, siblings/* , parentCons */);
				solveOcclusion(IN_WEST, siblings/* , parentCons */);
			}
		}
	}

	/**
	 * Calculates the vector by which surronding elements are shifted in case of
	 * zooming for example. The elements will be moved according to their
	 * calculated position (ber2zone) and the corresponding delta values. The shift 
	 * vector will be calculated for the given element brother. No actual
	 * reshaping or resizing is done here. The calculation is done as
	 * described in Berner's Diss.
	 * 
	 * @pre brother != null
	 * @param brother
	 *            The element to move.
	 * @return The calculated shift vector.
	 */
	private ConstructionVector calcShiftVector(Node brother/*,
			Construction parentCons*/) {
		// the old shape of the rectangle to move
		final ConstructionRectangle brotherShape = new ConstructionRectangle(
				brother.getBounds());
		// the center of the element to move
		final Point brotherCenter = brotherShape.getCenter();
		// the straight line that connects the two centers
		final ConstructionLine centersConnectingLine = new ConstructionLine(center, brotherCenter);
		// which planes contain the center of the element to move
		final int position = brotherZones.get(brother).intValue();
		// the direction of the shift vector is given through the direction
		// vector
		// of the connecting line
		ConstructionVector v = centersConnectingLine.getNormedVector();

		switch (position) {
		case IN_EAST:
			v.scaleToWidth(resizeDelta[EAST]);
			ber2dev.put(brother, new Integer((int) v.getHeight()));
			break;
		case IN_NORTH:
			v.scaleToHeight(resizeDelta[NORTH]);
			ber2dev.put(brother, new Integer((int) v.getWidth()));
			break;
		case IN_WEST:
			v.scaleToWidth(resizeDelta[WEST]);
			ber2dev.put(brother, new Integer((int) v.getHeight()));
			break;
		case IN_SOUTH:
			v.scaleToHeight(resizeDelta[SOUTH]);
			ber2dev.put(brother, new Integer((int) v.getWidth()));
			break;
		case IN_EAST + IN_NORTH:
			v = new ConstructionVector(resizeDelta[EAST], resizeDelta[NORTH]);
			break;
		case IN_NORTH + IN_WEST:
			v = new ConstructionVector(resizeDelta[WEST], resizeDelta[NORTH]);
			break;
		case IN_WEST + IN_SOUTH:
			v = new ConstructionVector(resizeDelta[WEST], resizeDelta[SOUTH]);
			break;
		case IN_SOUTH + IN_EAST:
			v = new ConstructionVector(resizeDelta[EAST], resizeDelta[SOUTH]);
			break;
		default:
			;// the element to move lies on the element
		// (possible state when element to move is invisible).
		}
		// set the vector's anchor to the center of the element to move
		v.setAnchor(brotherCenter);
		return v;
	}

	/**
	 * Determines where the specified Point lies with respect to the specified
	 * Rectangle. This method computes a binary OR of the appropriate mask
	 * values indicating which is the relative position of the specified point
	 * to the rectangle.
	 * 
	 * @pre rect != null && p != null
	 * @param rect
	 *            The specified rectangle
	 * @param p
	 *            The specified point.
	 * @return The logical or of the code.
	 */
	private int getPosition(Rectangle rect, Point p) {
		int position = 0;
		if (p.x < rect.x) {
			position += IN_WEST;
		}
		if (p.y < rect.y) {
			position += IN_NORTH;
		}
		if (p.x > rect.x + rect.width) {
			position += IN_EAST;
		}
		if (p.y > rect.y + rect.height) {
			position += IN_SOUTH;
		}
		return position;
	}

	/**
	 * This method solves some specific occlusions between elements of the
	 * specified zone and elements of any neighbour zone, by moving the elements
	 * in the specified zone. The algorithm searches the biggest occlusion (in
	 * width or height depending of the zone) and then proportionally moves the
	 * elements of this zone together. Moving the elements this way will not
	 * allow new occlusions to be created.<br>
	 * See Michael Marty's diploma thesis for details.
	 * 
	 * @pre siblings != null
	 * @param zone
	 *            The logical or code of the zone
	 * @param siblings
	 *            A list of the siblings to check for occlusions
	 */
	private void solveOcclusion(int zone, List siblings /*,
			Construction parentCons*/) {
		// the maximal quotient for calculating the distance of the element's
		// move
		float maxQuotient = 0;
		Node nodei;
		Node nodej;
		for (int i = 0; i < siblings.size(); i++) {
			nodei = (Node)siblings.get(i);
			// consider only the elements in the specified zone (the zone they
			// where originally in)
			if (brotherZones.get(nodei) == null || (brotherZones.
					get(nodei).
					intValue() != zone))
				continue;
			ConstructionRectangle shape = new BufferZoneRectangle(nodei.getBounds(), MIN_DISTANCE);
			//Point[] corners = shape.getCorners();
			for (int j = 0; j < siblings.size(); j++) {
				nodej = (Node)siblings.get(j);
				// avoids considering occlusions of the element with himself and
				// occlusions with invisile objects
				if ((nodej == nodei) /*|| !nodej.isVisible()
						|| !nodei.isVisible()*/)
					continue;
				ConstructionRectangle currShape = new ConstructionRectangle(
						nodej.getBounds());
				// test for occlusion
				if (currShape.intersects(shape)) {
                    
                    firePropertyChange(OCCLUSION, null, getAbsoluteRectangle(currShape.getOcclusion(shape), nodei.getParent())); 

					// this quotient will be used for calculating the
					// proportionality of the element's moves
					float quotient = 0;
					// add an OcclusionRectangle for this occlusion
					ConstructionRectangle occlusion = currShape
							.getExtendedOcclusion(shape);
					if ((zone == IN_NORTH) || (zone == IN_SOUTH)) {
						quotient = (float) (occlusion.width)
								/ (float) ber2dev.get(nodei)
										.intValue();
					} else if ((zone == IN_EAST) || (zone == IN_WEST)) {
						// calc the quotient
						quotient = (float) (occlusion.height)
								/ (float) ber2dev.get(nodei)
										.intValue();
					}
					// always keep the biggest absolute value of the quotient
					if (Math.abs(quotient) > maxQuotient)
						maxQuotient = Math.abs(quotient);
				}
			}
		}
		maxQuotient = -maxQuotient; // quotient has always to be negative so
									// that the objects move together
		if (maxQuotient < 0) { // maxQuotient != 0
			for (int i = 0; i < siblings.size(); i++) {
				nodei = (Node)siblings.get(i);
				// move all elements in the specified zone
				if (brotherZones.get(nodei) == null || brotherZones.get(nodei).intValue() != zone)
					continue;
				ConstructionVector v = null;
				if ((zone == IN_NORTH) || (zone == IN_SOUTH)) {
					// the x component of the vector is the sideway movement of
					// the element * the quotient
					v = new ConstructionVector(
							(maxQuotient * ber2dev.get(nodei)
									.intValue()), 0);
				} else if ((zone == IN_EAST) || (zone == IN_WEST)) {
					// the y component of the vector is the sideway movement of
					// the element * the quotient
					v = new ConstructionVector(0, maxQuotient
							* ber2dev.get(nodei).intValue());
				}
				v.setAnchor(getRelCenterWithin(
						nodei.getParent(),nodei));
                Rectangle originalPos = getAbsoluteRectangle(nodei.getBounds(), nodei.getParent());
				nodei.moveBy(
						new Point((int) v.getWidth(), (int) v.getHeight()));
                firePropertyChange(POSITION_CHANGE, originalPos, getAbsoluteRectangle(nodei.getBounds(), nodei.getParent()));
			}
		}
	}
  
    /**
     * Calculates the center of representation element in the coordinate system of the
     * given parent element (needs not to be a direct parent). If parent is
     * null, the used coordinates are absolute (corresponding to OMV). If parent
     * is element itself, then the center of the component itself is given.
     * 
     * @post return != null
     */
    public Point getRelCenterWithin(INodeContainer parent, Node n) {
        Point p = new Point(0, 0);
        if (parent != this) // TODO: check this if statement - parent can't be this!
            p = new Point(getRelLocationWithin(parent,n));
        p.translate(n.getSize().width / 2, n.getSize().height / 2);
        return p;
    }

    /**
     * Calculates the relative position of nodeRep element within the specified
     * element parent node, which must be a parent of nodeRep. If parent is
     * null, the relative position within OMV is calculated, what is equivalent
     * to the absolute position.
     * 
     * @post return != null
     */
    public Point getRelLocationWithin(INodeContainer node, Node sibling) {
        Point abs = null;

        if (sibling.getParent() == node) {
            abs = sibling.getLocation();

        } 
        // stop condition: specified parent or the top element is reached
        // (what happens when parent is null or
        // the specified parent is not a parent of nodeRep)
        else if (this == node || // TODO: check this - node can't be this!
                sibling.getParent() == null || 
                !(sibling.getParent() instanceof ZoomableNodeContainer)) {
            abs = new Point(0, 0);

        } else {
            
            abs = new Point(getRelLocationWithin(sibling.getParent(),(Node)node));
            // add the local position to the accumulated parent positions
            abs.translate(sibling.getLocation().x, sibling.getLocation().y);
        }
        return abs;
    }

	/**
	 * Doesn't do anything because this algorithm isn't used as node insertion
	 * layout algorithm at the moment.
	 * 
	 * @param node the node that was inserted
	 */
	public void insertNode(IZoomNode node) {
		
		
	}

	/**
	 * Moves the sibblings and resizes the parent if the node given as
	 * argument is removed.
	 * 
	 * @param node the node that was removed.
	 */
	public void removeNode(IZoomNode node) {
		
		Node n = (Node)node;
		
		// If node is the only child, resize the parent
        if (n.getParent().getChildCount()<2 && n.getParent() instanceof ZoomableNodeContainer) {
        	ZoomableNodeContainer parent = (ZoomableNodeContainer) n.getParent();
        	
            Dimension shrunkSize = LayoutUtils.calcShrunkSize(parent.getSize(), n.getDefaultSize());
    		Point newLoc = LayoutUtils.calcCenteredLocation(parent.getLocation(), parent.getSize(), shrunkSize);
    		Rectangle newShape = new Rectangle(newLoc, shrunkSize);
    		
            reshape(parent, newShape, false);
        }
        else {
            Dimension shrunkSize = new Dimension(1,1);
    		Point newLoc = LayoutUtils.calcCenteredLocation(n.getLocation(), n.getSize(), shrunkSize);
    		Rectangle newShape = new Rectangle(newLoc, shrunkSize);
	
            reshape(n, newShape, false);
        }
		
	}
	
	
	/**
	 * Method to handle the movement/resizing of a node;
	 * 
	 * @param node the node that is moved or resized
	 * @param newBounds the new bounds of the node
	 */
	public void changeNodeBounds(IZoomNode node, java.awt.Rectangle newBounds) {
		
	}
	
	
	/**
	 * Method to handle the hidding (i.e. filtering out) of a node.
	 * 
	 * @param node the node that should be hidden
	 */
	public void filterNode(IZoomNode node) {
		
		// The expansion of the node is handled by the minimal radial zoom
		if(node.getZoomBounds().width == 0 && node.getZoomBounds().height == 0) {
			MinRadial.getInstance().filterNode(node);
			return;
		}
		
		Node n = (Node)node;
		
		Dimension shrunkSize = new Dimension(0,0);//LayoutUtils.calcShrunkSize(node.getSize());
        Point newLoc = LayoutUtils.calcCenteredLocation(n.getLocation(), n.getSize(), shrunkSize);
        Rectangle newShape = new Rectangle(newLoc, shrunkSize);
            
        
        //Prevent saving a shrinked size as a stored size
        //Could happen if tree&view filter applied in squence.
        if(n.getSize().width!=shrunkSize.width&&n.getSize().height!=shrunkSize.height){
            n.setStoredSize(n.getSize());
        }
                
        reshape((Node)node, newShape, false);
		
	}
	
	/**
	 * Not implemented.
	 */
	public void allocateWhiteSpace(IZoomNode parent, java.awt.Rectangle spaceBounds) {
		
	}
}
