package ch.unizh.ifi.adora.zoom.berner.geometry;

import java.awt.geom.Point2D;

import ch.unizh.ifi.adora.model.geometry.Point;



/**
 * This ConstructionObject represents a staight line. The Line is given trough
 * the coordinates of two points. The class automatically calculates some
 * representations of a straight line that contains the given points.
 * <p>
 * Beside the representation throug two points, there is a possible
 * representation through a linear function (y=ax+b). This representation is not
 * possible for vertical lines, as there is no way to represent them as a
 * function.
 * <p>
 * A straight line can also be given through a vector and one point. This class
 * generates such a vector of length 1 and having the direction of point one to
 * point two.
 * <p>
 * This class also allows the painting of the line in any component. As the
 * class internaly works with double precision numbers to avoid loosing
 * precision, it transforms the coordinates of the points to integer values for
 * painting.
 * 
 * <pre>
 *  History:
 *  09.07.02  MM  Class created
 *  23.07.02  MM  Added translation method
 *  25.07.02  MM  Added visibility
 *  14.08.02  MM  No intersection with lines reduced to points
 *  28.08.02  MM  Changes for setColor method
 * 
 * </pre>
 * 
 * @author $Author: treinhard $
 * @version $Date: 2007-03-14 14:11:05 +0100 (Wed, 14 Mar 2007) $ $Revision: 2527 $
 * 
 */

public class ConstructionLine {

	/**
	 * The bitmask that indicates that a point lies below this line.
	 */
	public final static int BOTTOM = 8;

	/**
	 * The bitmask that indicates that a point lies to the left of this line.
	 */
	public final static int LEFT = 1;

	/**
	 * The bitmask that indicates that a point lies to the right of this line.
	 */
	public final static int RIGHT = 4;

	/**
	 * The bitmask that indicates that a point lies above this line.
	 */
	public final static int TOP = 2;

	/**
	 * A point containing two double precision nubers. The x coordinate
	 * represents gradient (a) and the y coordinate the shift (b) of a linear
	 * function (y=a*x + b). If the straight line can't be represented through a
	 * linear function, it's value is null.
	 */
	private Point2D function = null;

	/**
	 * The double precision vector of lenghth 1 describing the direction of the
	 * straight line.
	 */
	private ConstructionVector normedVector = null;

	/**
	 * The two points that defines the line.
	 */
	private Point p1, p2 = null;

	/**
	 * The constructor method.
	 * 
	 * @param p1
	 *            The first point that defines the line.
	 * @param p2
	 *            The second point that defines the line.
	 */
	public ConstructionLine(Point p1, Point p2) {
		setLine(p1, p2);
	}

	/**
	 * Checks whether or not a specified Point2D with double precision is on
	 * this line.
	 * 
	 * @param point
	 *            The specified point.
	 * @return true if the Point2D is on the line, false otherwise
	 */
	public boolean contains(Point2D point) {
		if (p1.x == p2.x && point.getX() == p1.x) {
			return true;
		} else if (getPointWithX(point.getX()).getY() == point.getY()) {
			return true;
		}
		return false;
	}

	/**
	 * Returns a copy of the normed vector that describes the line.
	 * 
	 * @return A copy of the normed vector.
	 */
	public ConstructionVector getNormedVector() {
		ConstructionVector v = new ConstructionVector(normedVector.getWidth(),
				normedVector.getHeight());
		return v;
	}

	/**
	 * Returns a copy of the first point that defines the line.
	 * 
	 * @return A copy of the first point.
	 */
	public Point getP1() {
		return new Point(p1);
	}

	/**
	 * Returns a copy of the second point that defines the line.
	 * 
	 * @return A copy of the second point.
	 */
	public Point getP2() {
		return new Point(p2);
	}

	/**
	 * Calculates the intersection of this line with a specified one.
	 * 
	 * @param line
	 *            The specified line.
	 * @return A double precision Point2D containing the coordinates of the
	 *         intersection or null if the lines don't intersect.
	 */
	public Point2D intersection(ConstructionLine line) {
		/**
		 * Lsung der linearen Gleichung mit zwei unbekannten. Die Gleichung der
		 * Geraden gegeben durch einen Punkt p und einem Vektor v lautet: p1 +
		 * a1*v1 und p2 + b2*v2 => der Schnittpunkt: p1 + alpha*v1 = p2 +
		 * beta*v2 Dabei sind p1=(x1,y1), p2=(x2,y2), v1=(vx1, vy1), v2=(vx2,
		 * vy2) Lsung durch Gaussches Verfahren: alpha beta 1  alpha beta 1
		 * ---------------------  ----------------- vx2 -vx1 x1-x2  a11 a12 b1
		 * vy2 -vy1 y1-y2  a21 a22 b2
		 * 
		 * Lsung: beta = ((a11/a21)*b2 - b1) / ((a11/a21)*a22 - a12) alpha =
		 * (beta*vx2 - x1 +x2) / vx1
		 */
		Point2D intersection = null;
		if (intersects(line)) {
			double beta = 0;
			double a11 = normedVector.getWidth();
			double a12 = -line.normedVector.getWidth();
			double a21 = normedVector.getHeight();
			double a22 = -line.normedVector.getHeight();
			double b1 = line.getP1().x - getP1().x;
			double b2 = line.getP1().y - getP1().y;
			if (a11 == 0) {
				beta = b1 / a12;
			} else if (a21 == 0) {
				beta = b2 / a22;
			} else {
				double quot = a11 / a21;
				beta = (quot * b2 - b1) / (quot * a22 - a12);
			}
			double x = line.getP1().x + beta * line.normedVector.getWidth();
			double y = line.getP1().y + beta * line.normedVector.getHeight();
			intersection = new Point2D.Double(x, y);
			return intersection;
		}
		return null;
	}

	/**
	 * Checks whether or not this line intersects with a specified one.
	 * 
	 * @param line
	 *            The specified line.
	 * @return true if the lines intersect, false otherwise.
	 */
	public boolean intersects(ConstructionLine line) {
		if (normedVector.getWidth() == line.normedVector.getWidth()
				&& normedVector.getHeight() == line.normedVector.getHeight()
				|| normedVector.getWidth() == -line.normedVector.getWidth()
				&& normedVector.getHeight() == -line.normedVector.getHeight()
				|| normedVector.getWidth() == 0
				&& normedVector.getHeight() == 0
				|| line.normedVector.getWidth() == 0
				&& line.normedVector.getHeight() == 0)
			return false;
		else
			return true;
	}

	/**
	 * Determines where the specified Point lies with respect to this line. This
	 * method computes a binary OR of the appropriate mask values indicating,
	 * which is the relative position of the specified point to the line.
	 * 
	 * @param point
	 *            the specified point.
	 * @return the logical or of the out code, 0 if the line is degenerated or
	 *         if the point is on the line.
	 */
	public int outcode(Point point) {
		int code = 0;
		Point2D pWithSameX = getPointWithX(point.x);
		Point2D pWithSameY = getPointWithY(point.y);
		if (pWithSameY != null) {
			if (point.x < pWithSameY.getX()) {
				code += 1; // left
			}
			if (point.x > pWithSameY.getX()) {
				code += 4; // right
			}
		}
		if (pWithSameX != null) {
			if (point.y < pWithSameX.getY()) {
				code += 2; // top
			}
			if (point.y > pWithSameX.getY()) {
				code += 8; // bottom
			}
		}
		return code;
	}

	/**
	 * Sets the points that defines the line. Both the linear representation of
	 * the line and the normed vector will be calculated from new.
	 * 
	 * @param p1
	 *            The first point that defines the line.
	 * @param p2
	 *            The second point that defines the line.
	 */
	public void setLine(Point p1, Point p2) {
		this.p1 = new Point(p1);
		this.p2 = new Point(p2);
		function = calcFunction();
		normedVector = calcNormVector();
	}

	/**
	 * Translate the ConstructionLine by the values specified in the Point.
	 * Implements ConstructionObject.
	 * 
	 * @param vector
	 *            The specified Point.
	 */
	public void translateObject(Point vector) {
		Point newP1 = getP1();
		Point newP2 = getP2();
		newP1.translate(vector.x, vector.y);
		newP2.translate(vector.x, vector.y);
		setLine(newP1, newP2);
	}

	/**
	 * Calculates the gradient (a) and the the shift (b) of the characteristic
	 * linear function (y=a*x + b) of this line.
	 * 
	 * @return A Point2D containing the double values of a and b or null, if the
	 *         straight line can't be represented through a linear function.
	 */
	private Point2D calcFunction() {
		if (isLinear()) {
			double a = 0, b = 0;
			double dx = p2.x - p1.x;
			double dy = p2.y - p1.y;
			a = dy / dx;
			b = p1.y - a * p1.x;
			Point2D values = new Point2D.Double(a, b);
			return values;
		} else
			return null;
	}

	/**
	 * Calculates the double precision vector of lenghth 1 describing the
	 * direction of this straight line. The direction of the vector goes from
	 * point one to point two.
	 * 
	 * @return The normed vector.
	 */
	private ConstructionVector calcNormVector() {
		double vx = p2.x - p1.x;
		double vy = p2.y - p1.y;
		ConstructionVector v = new ConstructionVector(vx, vy);
		v.norm();
		return v;
	}

	/**
	 * Returns the point of the line with a specified x value if it exists.
	 * 
	 * @param x
	 *            The specified x value.
	 * @return The searched point of the line in double precision or null if the
	 *         line is vertical.
	 */
	private Point2D getPointWithX(double x) {
		Point2D p = null;
		if (function != null) {
			double searchedY = function.getX() * x + function.getY();
			p = new Point2D.Double(x, searchedY);
		}
		return p;
	}

	/**
	 * Returns the point of the line with a specified y value if it exists.
	 * 
	 * @param y
	 *            The specified y value.
	 * @return The searched point of the line in double precision or null if the
	 *         line is horizontal.
	 */
	private Point2D getPointWithY(double y) {
		Point2D p = null;
		if (function != null && function.getX() != 0) {
			double searchedX = (y - function.getY()) / function.getX();
			p = new Point2D.Double(searchedX, y);
		} else if (p2.x == p1.x) {
			p = new Point2D.Double(p2.x, y);
		}
		return p;
	}

	/**
	 * Tests whether the line can be represented by a linear function (a*x + b)
	 * i.e. the line is not vertical.
	 * 
	 * @return true if the line can be represented by a linear function, false
	 *         otherwise.
	 */
	private boolean isLinear() {
		if (p1 == null || p2 == null || (p2.x == p1.x)) {
			return false;
		}
		return true;
	}

}
