package ch.unizh.ifi.adora.zoom.berner.geometry;

import java.awt.geom.Point2D;

import ch.unizh.ifi.adora.model.geometry.Point;


/**
 * This ConstructionObject represents a vector. The vector is defined by his
 * height and width. For drawing purposes a anchor point can be defined. This
 * anchor point hasn't a geometrical meaning but is used to set the position
 * where the vector will be graphically represented.
 * <p>
 *
 * <pre>
 * History:
 * 09.07.02  MM  Class created
 * 23.07.02  MM  Added translation method
 * 25.07.02  MM  Added methods for visibility handling
 * 09.08.02  MM  Added method for relative scaling of the width/height
 * 14.08.02  MM  Added some precondition checks
 *
 * </pre>
 *
 * @author  $Author: treinhard $
 * @version $Date: 2007-03-14 14:11:05 +0100 (Wed, 14 Mar 2007) $ $Revision: 2527 $
 */
public class ConstructionVector {
  

  /**
   * The width (x coordinate) of this vector.
   */
  private double width = 0;
  /**
   * The height (y coordinate) of this vector.
   */
  private double height = 0;
  /**
   * An anchor point for the graphical representation of the vector.
   */
  private Point anchor = new Point();
  /**
   * Creates a new vector with integer values.
   * @param width The width of the vector.
   * @param height The height of the vector.
   */
  public ConstructionVector(int width, int height) {
    this.width = width;
    this.height = height;
  }
  /**
   * Creates a new vector with double precision values.
   * @param width The width of the vector.
   * @param height The height of the vector.
   */
  public ConstructionVector(double width, double height) {
    this.width = width;
    this.height = height;
  }
  /**
   * Creates a new vector with the coordinates of a Point2D
   * @param coord The Point2D containing the width and height values of the vector.
   */
  public ConstructionVector(Point2D.Double coord) {
    width = coord.getX();
    height = coord.getY();
  }
  /**
   * Creates a new vector with the coordinates of a Point
   * @param coord The Point containing the width and height values of the vector.
   */
  public ConstructionVector(Point coord) {
    width = coord.x;
    height = coord.y;
  }
  /**
   * Returns a normed vector of any specified vector. The returned vector has the
   * same direction as the specified one but a length of one.
   * @param v The specified vector.
   * @return A vector of same direction as the specified one, but with a length of
   * one
   */
  public static ConstructionVector norm(ConstructionVector v) {
    double length = v.getLength();
    double newHeight = 0;
    double newWidth = 0;
    if (length != 0) {
      newHeight = v.height/length;
      newWidth = v.width/length;
    }
    return new ConstructionVector(newWidth, newHeight);
  }
  /**
   * Sets the anchor point of this vector. The anchor point has no geometrical
   * meaning and is only used to represent the vector graphically.
   * @param anchor The Point, where the vector has to be drawn.
   */
  public void setAnchor(Point anchor) {
    this.anchor = anchor;
  }
  /**
   * Returns this vector's width.
   * @return The width of this vector in double precision.
   */
  public double getWidth() {
    return width;
  }
  /**
   * Returns this vector's height.
   * @return The height of this vector in double precision.
   */
  public double getHeight() {
    return height;
  }
  /**
   * Returns the length of this vector.
   * @return The length of this vector in double precision.
   */
  public double getLength() {
    return Math.sqrt(height*height + width*width);
  }
  /**
   * Returns the result of the scalar product of this vector with a specified one.
   * @param v The specified vector.
   * @return The result of the scalar product in double precision.
   */
  public double scalarProd(ConstructionVector v) {
    return width*v.width + height*v.height;
  }
  /**
   * Returns the angle of this vector to a specified one.
   * @param v The specified vector.
   * @return The angle in radians.
   */
  public double angleTo(ConstructionVector v) {
    double phi = 0;
    if ((getLength() != 0) && (v.getLength() != 0 )) {
      phi = Math.acos(scalarProd(v)/(getLength()*v.getLength()));
    }
    return phi;
  }
  /**
   * Changes the values of this vector to the sum of this vector and a specified one.
   * @param v The vector too add.
   */
  public void add(ConstructionVector v) {
    width += v.width;
    height += v.height;
  }
  /**
   * Scales this vector by a specified factor.
   * @param factor The scaling factor.
   */
  public void scale(double factor) {
    width *= factor;
    height *= factor;
  }
  /**
   * Scales this vector, so that it gets a specified width while keeping his direction.
   * @param newWidth The specified new width of the vector.
   */
  public void scaleToWidth(double newWidth) {
    if (width != 0) {
      scale(newWidth/width);
    }
  }
  /**
   * Scales this vector, so that it gets a specified height while keeping his direction.
   * @param newHeight The specified new height.
   */
  public void scaleToHeight(double newHeight) {
    if (height != 0) {
      scale(newHeight/height);
    }
  }
    /**
   * Scales this vector, so that it gets a specified width while keeping his direction.
   * This method differs from scaleToWidth in a way that the head of the vector
   * will still show in the same direction afterwards if the new width is positive.
   * @param newWidth The specified new width of the vector.
   */
  public void scaleToRelWidth(double newWidth) {
    if (width != 0) {
      scale(newWidth/Math.abs(width));
    }
  }
  /**
   * Scales this vector, so that it gets a specified height while keeping his direction.
   * This method differs from scaleToHeight in a way that the head of the vector
   * will still show in the same direction afterwards if the new height is positive.
   * @param newHeight The specified new height.
   */
  public void scaleToRelHeight(double newHeight) {
    if (height != 0) {
      scale(newHeight/Math.abs(height));
    }
  }
  /**
   * Norms this vector. Scales the vector to a length of one.
   */
  public void norm() {
    double length = getLength();
    if (length != 0) {
      height /= length;
      width /= length;
    }
  }
  /**
   * Translate the anchor point of this ConstructionVector by the values specified
   * in the Point.
   * Implements ConstructionObject.
   * @param vector The specified Point.
   */
  public void translateObject(Point vector) {
    anchor.translate(vector.x, vector.y);
  }
}
