package ch.unizh.ifi.adora.zoom.continuous.algorithm;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.util.List;

import ch.unizh.ifi.adora.model.representation.zoom.ContinuousZoomConstants;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.model.representation.zoom.Interval;
import ch.unizh.ifi.adora.model.representation.zoom.IntervalStructure;
import ch.unizh.ifi.adora.model.representation.zoom.ZoomHole;
import ch.unizh.ifi.adora.zoom.adora.DummyNode;

/**
 * Zoom algorithm that is based on the idea of scaling horizontal and vertical
 * intervals independently (Based on the continuous zoom by Lyn Bartram).
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-03-12 TR First Version
 * @version				$Date: 2009-07-20 15:44:11 +0200 (Mon, 20 Jul 2009) $, $Author: treinhard $, $Revision: 4528 $
 *
 */
public class ContinuousZoom {

	/**
	 * Singleton instance variable
	 */
	private static ContinuousZoom INSTANCE = new ContinuousZoom();
	
	
	/**
	 * Singleton method to get the instance
	 * 
	 * @return the continuous zoom instance
	 */
    public static ContinuousZoom getInstance() {
        return INSTANCE;
    }
        

    /**
     * Singleton: Hide constructor
     *
     */
    protected ContinuousZoom(){
        
    }
        
    
    /**
     * Zooms the node <i>container</i> in if <i>zoomIn</i> is true, zooms the node
     * out if <i>zoomIn</i> is false.
     * 
     * @param container the node that should be zoomed
     * @param zoomIn true if this is a zoom-in, false if it is a zoom-out
     */
	public void zoom(IZoomNode container, boolean zoomIn) {
		
		Dimension newDimension = calcSize(container);
		adjustNodes(container, newDimension, false);
		
		
	}
	
	
	/**
	 * Sets the size of the <i>container</i> to the size <i>newDimension</i>,
	 * adjusts the positions of all the siblings and recursively adjusts the
	 * size and positions of all the direct and indirect parents and its
	 * siblings.
	 * 
	 * @param container the node for which the size changes
	 * @param newDimension the new size
	 * @param scaleGaps true if neighboring gap intervals should be scaled as well
	 */
	private void adjustNodes(IZoomNode container, Dimension newDimension, boolean scaleGaps) {
		
		// Get the interval structure for the parent of the container that has
		// to be adjusted.
		IZoomNode parent = container.getZoomParent();
		IntervalStructure structure = getIntervalStructureFor(parent);
		
		// Don't do anything if the old and the new size are the same
		// and the size of the parent's structure is still the same
		if(container.getZoomBounds().getSize().equals(newDimension)
				&& parent.getZoomBounds().width == structure.getBounds().width
				&& parent.getZoomBounds().height == structure.getBounds().height) {
			return;
		}
		
		// Maintain a minimal size of the node
		if(!container.isZoomedIn() && container.getZoomMinimalSize() != null) {
			if(newDimension.width > 1 && newDimension.width < container.getZoomMinimalSize().width) {
				newDimension.width = container.getZoomMinimalSize().width;
			}
			if(newDimension.height > 1 && newDimension.height < container.getZoomMinimalSize().height) {
				newDimension.height = container.getZoomMinimalSize().height;
			}
		}
		
		// Adjust the interval structure to the new size by scaling the intervals
		ZoomHole zoomHole = structure.getZoomHoleFor(container);
		
		if (zoomHole != null) {
			zoomHole.adjustIntervals(newDimension);
			
			if(scaleGaps == true) {
				if(newDimension.equals(new Dimension(1, 1))) {

					adjustNeighboredGapIntervals(container, 0.5);

				}

				else if(container.getZoomBounds().getSize().equals(new Dimension(1, 1))) {

					adjustNeighboredGapIntervals(container, 2.0);

				}
			}
			
		}
		
		// Set the location and size of the root zoom node
		if(container.getZoomParent().getZoomParent() == null) {
			
			container.getZoomParent().setZoomLocation(container.getZoomParent().getZoomBounds().getLocation());
			container.getZoomParent().setZoomSize(structure.getBounds().getSize());
			
		}
		
		// Adjust the size of the container
		container.setZoomSize(newDimension);
		
		// Adjust the positions of the siblings to the adjusted interval structure
		structure.adjustNodePositions();
				
		// Adjust all direct and indirect parents recursively
		Dimension parentSize = calcSize(container.getZoomParent());
		if(container.getZoomParent().getZoomParent() != null 
				&& !container.getZoomParent().getZoomBounds().getSize().equals(new Dimension(1, 1))) {
			adjustNodes(container.getZoomParent(), parentSize, scaleGaps);
		}
		
	}
	
	
	/**
	 * Scales all neighboring space intervals of this zoom hole by the factor 
	 * given as argument.
	 * 
	 * @param factor the scale factor
	 */
	private void adjustNeighboredGapIntervals(IZoomNode node, double factor) {

		ZoomHole hole = node.getZoomParent().getZoomStructure().getZoomHoleFor(node);
		
		// Left neighbor
		Interval first = hole.getHorizontalIntervals().get(0);
		// Scale only if there is a left neighbor which is a space interval and
		// the leftmost interval of this zoom hole has no other projections to it.
		if(first.getPrevious() != null && first.getPrevious().getHoles().size() == 0
				&& first.getHoles().size() == 1) {

			Interval leftNeighbor = first.getPrevious();
			double newLength = leftNeighbor.getLength() * factor;

			leftNeighbor.moveEndTo(leftNeighbor.getStart() + (int)Math.round(newLength));

		}

		// Right neighbor
		Interval last = hole.getHorizontalIntervals().get(hole.getHorizontalIntervals().size() - 1);
		if(last.getNext() != null && last.getNext().getHoles().size() == 0
				&& last.getHoles().size() == 1) {

			Interval rightNeighbor = last.getNext();
			double newLength = rightNeighbor.getLength() * factor;

			rightNeighbor.moveEndTo(rightNeighbor.getStart() + (int)Math.round(newLength));

		}
		
		// Top neighbor
		first = hole.getVerticalIntervals().get(0);
		if(first.getPrevious() != null && first.getPrevious().getHoles().size() == 0
				&& first.getHoles().size() == 1) {

			Interval topNeighbor = first.getPrevious();
			double newLength = topNeighbor.getLength() * factor;

			topNeighbor.moveEndTo(topNeighbor.getStart() + (int)Math.round(newLength));

		}
		
		// bottom neighbor
		last = hole.getVerticalIntervals().get(hole.getVerticalIntervals().size() - 1);
		if(last.getNext() != null && last.getNext().getHoles().size() == 0
				&& last.getHoles().size() == 1) {

			Interval bottomNeighbor = last.getNext();
			double newLength = bottomNeighbor.getLength() * factor;

			bottomNeighbor.moveEndTo(bottomNeighbor.getStart() + (int)Math.round(newLength));

		}

		
	}
	
	
	/**
	 * Interface to the continuous zoom without any ADORA or GEF dependencies.
	 * Used for the automatic functional tests. 
	 * 
	 * @param parent the container that contains the zoomed node
	 * @param zoomed the node for which the size changes
	 * @param newSize the new size of the zoomed node
	 */
	public void zoom(IZoomNode parent, IZoomNode zoomed, Dimension newSize) {
		
		IntervalStructure structure = getIntervalStructureFor(parent);
		
		structure.getZoomHoleFor(zoomed).adjustIntervals(newSize);
		
		zoomed.setZoomSize(newSize);
		
		structure.adjustNodePositions();
		
		parent.setZoomSize(structure.getSize());
		
	}
	
	
	/**
	 * Calculates the new size for the node given as argument <i>container</i>.
	 * 
	 * @param container the node to calculate the new size for
	 * @return the new size of the node <i>container</i>
	 */
	private Dimension calcSize(IZoomNode container) {
		
		Dimension newSize = container.getZoomMinimalSize();
		IntervalStructure structure = getIntervalStructureFor(container);
		
		if(container.isZoomedIn()) {
			
			if(newSize.width < structure.getBounds().width) {
				newSize.width = structure.getBounds().width;
			}
			
			if(newSize.height < structure.getBounds().height) {
				newSize.height = structure.getBounds().height;
			}
			
		}
			
		return newSize;
		
	}
	
	
	/**
	 * Adds the node given as argument to the interval structure (if the 
	 * structure exists already).
	 * 
	 * @param node the node that should be added
	 */
	public void insertNode(IZoomNode node) {
		
		Rectangle bounds = new Rectangle(node.getZoomBounds());
		
		IZoomNode parent = node.getZoomParent();
		IntervalStructure structure = getIntervalStructureFor(parent);
		
		// Check whether the node lies inside an expanded zoom hole
		List<ZoomHole> existingHoles = structure.getExpandedZoomHoles(bounds);
		for(ZoomHole existingHole: existingHoles) {
			
			// Remove and reinsert the node. This resets the size of the zoom
			// hole to the size of the node.
			IZoomNode existingNode = existingHole.getNode();
			structure.removeNode(existingNode);
			structure.insertNode(existingNode);
			
		}
		
		// Expand the bounds of the node by the minimal gap between two nodes
		// or some additional space (if there's an overlaping)
		Rectangle expandedBounds = getGapExpandedBounds(node);
		
		// Calculate the space the new node can maximally occupy without moving
		// any sibling nodes away.
		Rectangle freeSpace = structure.getFreeSpace(expandedBounds);
		
		// Move the node to the left or to the bottom if it overlaps with a
		// left or top neighbor.
		int x = expandedBounds.x;
		int y = expandedBounds.y;
		if(freeSpace.x > expandedBounds.x) {
			x = freeSpace.x;
		}
		if(freeSpace.y > expandedBounds.y) {
			y = freeSpace.y;
		}
		node.setZoomLocation(new java.awt.Point(x, y));
		node.setZoomSize(freeSpace.getSize());
		
		// Insert the node (with a possibly reduced size)
		structure.insertNode(node);
		adjustNodes(node, expandedBounds.getSize(), false);
		
		// Set the location and the size of the node to the intended values
		// (without the minimal gap added)
		node.setZoomSize(bounds.getSize());
		int deltaX = bounds.x - expandedBounds.x;
		int deltaY = bounds.y - expandedBounds.y;

		if(deltaX < 0) {
			deltaX = ContinuousZoomConstants.MINIMAL_GAP;
		}
		if(deltaY < 0) {
			deltaY = ContinuousZoomConstants.MINIMAL_GAP;
		}
	
		node.setZoomLocation(new java.awt.Point(x + deltaX, y + deltaY));
		
		// Adjusts the zoom hole to the size of the node
		structure.removeNode(node);
		structure.insertNode(node);
		
		// Initialize the zoom structure of the inserted node
		node.getZoomStructure();
		
	}
	
	
	/**
	 * Removes the node given as argument from the interval structure.
	 * 
	 * @param node the node that should be removed
	 */
	public void removeNode(IZoomNode node) {
		
		// Set the size to (1, 1) to move the siblings and adjust the size
		// of the parents
		Dimension newDimension = new Dimension(1, 1);
		adjustNodes(node, newDimension, false);
		
		// Remove the intervals and zoom holes
		IZoomNode parent = node.getZoomParent();
		IntervalStructure structure = getIntervalStructureFor(parent);
		structure.removeNode(node);
		
	}
	
	
	/**
	 * Adjusts the interval structures to the changed bounds of the node
	 * given as argument.
	 * 
	 * @param node the node whose bounds changed
	 */
	public void changeNodeBounds(IZoomNode node, Rectangle newBounds) {
		
		Dimension originalSize = node.getZoomBounds().getSize();
		IZoomNode parent = node.getZoomParent();
		IntervalStructure structure = getIntervalStructureFor(parent);
		
		// Remove the node from the structure
		structure.removeNode(node);
		node.setZoomLocation(newBounds.getLocation());
		node.setZoomSize(newBounds.getSize());
		// Insert the node at the new position with the new size
		insertNode(node);
		
		// Adjust the interval structure inside the node
		structure = getIntervalStructureFor(node);
		if(structure != null) {
			// Only adjust if the size changed
			if(!originalSize.equals(newBounds.getSize())) {
				if(node.isZoomedIn()) {
					if(originalSize.width != newBounds.width) {
						structure.getHorizontalIntervalAt(structure.getBounds().width).
							moveEndTo(newBounds.getSize().width);
					}
					if(originalSize.height != newBounds.height) {
						structure.getVerticalIntervalAt(structure.getBounds().height).
							moveEndTo(newBounds.getSize().height);
					}
				}
				else {
					node.setZoomMinimalSize(newBounds.getSize());
				}
			}
		}
	}
	
	
	/**
	 * Compatibility with old interface
	 */
	public void filterNode(IZoomNode node) {
		
		filterNode(node, false);
		
	}
	
	
	/**
	 * Method to handle the hiding (i.e. filtering out) of a node.
	 * 
	 * @param node the node that should be hidden
	 */
	public void filterNode(IZoomNode node, boolean scaleGaps) {
		
		IntervalStructure structure = getIntervalStructureFor(node);
		// Set the size to (1, 1) to move the siblings and adjust the size
		// of the parents
		Dimension newDimension = new Dimension(1, 1);
		if(node.getZoomBounds().getSize().equals(newDimension)) {
			newDimension = structure.getSize();
			if(structure.getNumberOfZoomHoles() > 0) {
				newDimension = calcSize(node);
			}
		}
		
		adjustNodes(node, newDimension, scaleGaps);
		
	}
	
	
	/**
	 * Reduce the size of the node given as argument by reducing the size of
	 * the border around the children.
	 * 
	 * @param node the node whose size should be reduced.
	 */
	public void compactNode(IZoomNode node) {
		
		IntervalStructure structure = node.getZoomStructure();
		
		structure.compact();
		adjustNodes(node, structure.getBounds().getSize(), false);
		
	}
	
	
	/**
	 * Returns the interval structure that has the node given as argument as
	 * parent. The method may return null if the structure hasn't been
	 * constructed yet.
	 * 
	 * @param node the parent node of the structure
	 * @return the interval structure
	 */
	public IntervalStructure getIntervalStructureFor(IZoomNode node) {
		
		return node.getZoomStructure();
		
	}
	
	
	/**
	 * Returns the rectangle that results if the bounds of the node given as 
	 * argument are expanded by the minimal gap (if the node doesn't overlap 
	 * with one of its siblings) or the some additional space (if there is an 
	 * overlapping).
	 * 
	 * @param node the node for which the expanded rectangle should be calculated
	 * @return the expanded rectangle
	 */
	public Rectangle getGapExpandedBounds(IZoomNode node) {
		
		if(node instanceof DummyNode) {
			return new Rectangle(node.getZoomBounds());
		}
		
		IntervalStructure structure = getIntervalStructureFor(node.getZoomParent());
		
		// Increase the size of the requested size so that there exists
		// an ADDITIONAL_SPACE border around the node
		Rectangle expandedBounds = new Rectangle(node.getZoomBounds());
		Rectangle freeSpace = structure.getFreeSpace(expandedBounds);
		// Left
		if(freeSpace.x > expandedBounds.x) {
			expandedBounds.x -= ContinuousZoomConstants.ADDITIONAL_SPACE;
			expandedBounds.width += ContinuousZoomConstants.ADDITIONAL_SPACE;
		} else {
			expandedBounds.x -= ContinuousZoomConstants.MINIMAL_GAP;
			expandedBounds.width += ContinuousZoomConstants.MINIMAL_GAP;
		}
		
		// Right
		// We have to calculate the free space newly because the expansion
		// may influence the selection of the maximal available space.
		freeSpace = structure.getFreeSpace(expandedBounds);
		if(freeSpace.getMaxX() < expandedBounds.getMaxX()) {
			expandedBounds.width += ContinuousZoomConstants.ADDITIONAL_SPACE;
		} else {
			expandedBounds.width += ContinuousZoomConstants.MINIMAL_GAP;
		}
		
		// Top
		freeSpace = structure.getFreeSpace(expandedBounds);
		if(freeSpace.y > expandedBounds.y) {
			expandedBounds.y -= ContinuousZoomConstants.ADDITIONAL_SPACE;
			expandedBounds.height += ContinuousZoomConstants.ADDITIONAL_SPACE;
		} else {
			expandedBounds.y -= ContinuousZoomConstants.MINIMAL_GAP;
			expandedBounds.height += ContinuousZoomConstants.MINIMAL_GAP;
		}
		
		// Bottom
		freeSpace = structure.getFreeSpace(expandedBounds);
		if(freeSpace.getMaxY() < expandedBounds.getMaxY()) {
			expandedBounds.height += ContinuousZoomConstants.ADDITIONAL_SPACE;
		} else {
			expandedBounds.height += ContinuousZoomConstants.MINIMAL_GAP;
		}
		
		return expandedBounds;
		
	}
	
}
